/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.action;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate2;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PlatformUI;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.AnzeigeTypC2EditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.AnzeigeTypD3EditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.AnzeigeTypDEditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.DoMitVerbLinieEditPart;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeInhaltAuswahlDialog;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeInhaltFreiTextDialog;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.DynamischerWechselText.EnumDynamischerWechseltextTextAusrichtung;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DataHelper;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Action zur Anzeige der Auswahl fr Anzeigen der Parameter-Dialoge. 
 * Es sollen nur Daten im Kontext Parameter verndert werden.
 * @author Pittner
 *
 */
public class ActionKontextParameterAnzeigeABC implements IWorkbenchWindowActionDelegate, IActionDelegate2
{
  ClientDavInterface _connection = DaVVerbindung.getInstanz().getDaVVerbindung();

  SystemObject[] fonts = null;

  private void proceed(Data data)
  {
    System.out.println("ActionKontextParameterAnzeigeABC.proceed()");
    
    GraphicalViewer viewer = getViewer();
    if (viewer != null)
    {
      ISelection selection = viewer.getSelection();

      StructuredSelection ss = (StructuredSelection)selection;

      List<DoMitVerbLinieEditPart> list1 = new ArrayList<DoMitVerbLinieEditPart>();

      Iterator it = ss.iterator();
      while (it.hasNext())
      {
        Object obj = it.next();

        DoMitVerbLinieEditPart domep = null;
        AnzeigeModelTypAbstract doModel = null;
        AnzeigeElementFigure figure = null;

        if (obj instanceof DoMitVerbLinieEditPart)
        {
          domep = (DoMitVerbLinieEditPart)obj;
          DoModelMitVerbLinie model = domep.getModel();
          if (model instanceof AnzeigeModelTypAbstract)
          {
            doModel = (AnzeigeModelTypAbstract)model;
            fonts = DaVHelper.holeFontsUeberAnzeige(_connection, doModel.getSystemObject());
            list1.add(domep);
          }
        }

        boolean unterschiedlich = false;

        List<DoMitVerbLinieEditPart> copyList = new ArrayList<DoMitVerbLinieEditPart>(list1);

        for (DoMitVerbLinieEditPart epart : list1)
        {
          for (DoMitVerbLinieEditPart copy : copyList)
          {
            if (!epart.getClass().equals(copy.getClass()))
            {
              unterschiedlich = true;
              break;
            }
          }
          if (unterschiedlich)
            break;
        }

        if (unterschiedlich)
        {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_ERROR | SWT.OK);
          mb.setText("Fehler Selektion");
          mb.setMessage("Es knnnen nur Inhalte eines Anzeigetyps gesetzt werden!");
          mb.open();
          return;
        }

      }

      if (list1.size() > 0)
      {
//        for (DoMitVerbLinieEditPart domep : list1)
        {
          DoMitVerbLinieEditPart domep = list1.get(0);
          AnzeigeModelTypAbstract doModel = (AnzeigeModelTypAbstract)domep.getModel();
          AnzeigeElementFigure figure = (AnzeigeElementFigure)domep.getFigure();
          
//        System.out.println(domep.getClass().getSimpleName() + " : " + doModel.getClass().getSimpleName());
          
          
          
          if (!(domep instanceof AnzeigeTypDEditPart || domep instanceof AnzeigeTypD3EditPart))
          {
            AnzeigeInhaltAuswahlDialog anzeigeInhaltAuswahlDialog = new AnzeigeInhaltAuswahlDialog(PlatformUI
                .getWorkbench().getDisplay().getActiveShell(), true);
            AnzeigeInhaltFreiTextDialog freiTextAuswahlDialog = new AnzeigeInhaltFreiTextDialog(PlatformUI.getWorkbench()
                .getDisplay().getActiveShell());
            
            if (doModel != null && figure != null)
            {
              EList<String> inhalte1 = (EList<String>)doModel.getInhalte1();
              if (inhalte1 != null)
              {
                inhalte1.add(AnzeigeInhaltTyp.BELIEBIG_BI.getType());
//              inhalte1.add(AnzeigeInhaltTyp._BI.getType());
                
                if (domep instanceof AnzeigeTypC2EditPart)
                  inhalte1.add(AnzeigeInhaltTyp.FREITEXT_BI.getType());
                
                AnzeigeInhaltTyp inhalt = anzeigeInhaltAuswahlDialog.oeffne(figure.getTyp(), inhalte1);
                
                if (inhalt != null)
                {
                  
                  boolean geoeffnet = false;
                  
                  for (DoMitVerbLinieEditPart domep2 : list1)
                  {
                    AnzeigeModelTypAbstract doModel2 = (AnzeigeModelTypAbstract)domep2.getModel();
//                    AnzeigeModelTypAbstract doModel2 = (AnzeigeModelTypAbstract)domep2.getModel();
                    Object value2 = doModel2.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
                    if (value2 instanceof AnzeigeInhaltTyp)
                    {
                      AnzeigeInhaltTyp inhaltAlt = (AnzeigeInhaltTyp)value2;
                      if (!inhaltAlt.equals(inhalt)){
                        doModel2.setAbsolutNotwendig(0);
                        doModel2.setNotwendig(0);
                        doModel2.setBlinken(0);
                        doModel2.setBlinktakt("Undefiniert");
                        doModel2.setPrioritaet("undefiniert");
//                        domep2.abmeldenTaktgeber();
                      }
                    }
                    AnzeigeElementFigure figure2 = (AnzeigeElementFigure)domep2.getFigure();
                    // System.out.println("Inhalt: " + inhalt.getType() + " -> " + doModel2);
                    SystemObject so = doModel2.getSystemObject();
                    
                    if (!inhalt.equals(AnzeigeInhaltTyp.FREITEXT_BI))
                    {
                      
//                    System.out.println("---> dataName: " + data.getName() + " : " + data.getAttributeType());
                      
                      Array array = data.getItem("AnzeigenDatenstze").asArray();
//                    Array array = data.getItem("AnzeigenDatenstze").asArray();
                      SystemObject wvzInhalt = null;
                      
                      // Index nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                      int loc = 0;
                      List<Data> list = new ArrayList<Data>();
                      for (int i = 0; i < array.getLength(); i++)
                      {
                        Data item = array.getItem(i);
                        SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                        if (anzeige != null && so != null && anzeige.equals(so))
                        {
                          if (_connection != null)
                          {
                            if (!inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) && !inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                            {
                              // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                              wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                              item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue()
                              .setSystemObject(wvzInhalt);
                              
                              item.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("TextAusrichtung")
                              .asTextValue().setText("Linksbndig");
                              item.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("Font")
                              .asReferenceValue().setSystemObject(null);
                              item.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("Textdefinition")
                              .asArray().setLength(0);
                            }
                            else
                            {
                              loc = i;
                            }
                            // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                            // break;
                          }
                        }
                        // Liste wird vorbelegt
                        list.add(item);
                      }
                      
                      if (wvzInhalt == null)
                      {
                        if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                        {
                          // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                          if (!list.isEmpty() && list.size() - 1 >= loc)
                          {
                            list.remove(loc);
                          }
                          array.setLength(list.size());
                          for (int i = 0; i < array.getLength(); i++)
                          {
                            Data ziel = array.getItem(i);
                            Data quelle = list.get(i);
                            // ziel = quelle;
                            DataHelper.kopiereRekursiv(quelle, ziel);
                          }
                        }
                        else
                        {
                          // Array um ein Feldelement erweitern
                          array.setLength(array.getLength() + 1);
                          // Index dieses neuen Feldelements ist um eins kleiner
                          Data item = array.getItem(array.getLength() - 1);
                          item.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue().setText("Ja");
                          item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue().setText("Nein");
                          item.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue().setText("undefiniert");
                          item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asTextValue().setText("0");
                          item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asTextValue().setText("0");
                          
                          item.getItem("WVZInhalt").getItem("Blinken").asTextValue().setText("Nein");
                          item.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText("Undefiniert");
                          item.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("TextAusrichtung")
                          .asTextValue().setText("Linksbndig");
                          
                          // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                          wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                          
                          item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                          item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue()
                          .setSystemObject(wvzInhalt);
                          
                        }
                      }
                      
                      EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                      
                      for (DoMitVerbLinieEditPart epart : list1)
                      {
                        DoModelMitVerbLinie model = epart.getModel();
                        final SetCommand cmd = new SetCommand(model, feature, inhalt);
                        getCommandStack().execute(cmd);
                      }
                      
                    }
                    else
                    {
                      
                      // System.out.println("--------> inhaltTyp: " + inhalt.getType() + " -> " + doModel2);
                      
                      if (doModel2 != null && figure2 != null)
                      {
                        
                        Object value = doModel2
                        .eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
                        
                        VisuAnzeigeKapsel kapsel = null;
                        if (value instanceof VisuAnzeigeKapsel)
                        {
                          kapsel = (VisuAnzeigeKapsel)value;
                          kapsel.setSchriftart(fonts[0].getPid());
                        }
//                      else if (value instanceof ResultData)
//                      {
//                        ResultData result = (ResultData)value;
//                        kapsel = DaVHelper.holeDynamischerWechselTextAusDaVData(result);
//                        
//                      }
                        else
                        {
                          kapsel = new VisuAnzeigeKapsel();
                          kapsel.setAbstand(0);
                          kapsel.setAusrichtung(EnumDynamischerWechseltextTextAusrichtung.Zentriert.toString());
                          kapsel.setSchriftart(fonts[0].getPid());
                          kapsel.setPidsZeichen(new String[0]);
                        }
                        
                        if (!geoeffnet)
                        {
                          kapsel = freiTextAuswahlDialog.oeffne(kapsel, doModel.getSystemObject());
                        }
                        
                        if (kapsel != null)
                        {
                          geoeffnet = true;
                          // System.out.println("kapsel: " + kapsel);
                          SystemObject so2 = doModel2.getSystemObject();
                          Array array = data.getItem("AnzeigenDatenstze").asArray();
                          Data dynamischerWechseltext = null;
                          // Index nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                          int loc = 0;
                          List<Data> list = new ArrayList<Data>();
                          for (int i = 0; i < array.getLength(); i++)
                          {
                            Data item = array.getItem(i);
                            SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                            
                            if (anzeige != null && so != null && anzeige.equals(so))
                            {
                              if (_connection != null)
                              {
                                if (!kapsel.isInaktiv())
                                {
                                  dynamischerWechseltext = item.getItem("WVZInhalt").getItem("DynamischerWechseltext");
                                  uebernehmeDaten(kapsel, dynamischerWechseltext);
                                  item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);
                                }
                                else
                                {
                                  loc = i;
                                }
                                // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                                // break;
                              }
                            }
                            // Liste wird vorbelegt
                            list.add(item);
                          }
                          
                          if (dynamischerWechseltext == null)
                          {
                            if (kapsel.isInaktiv())
                            {
                              // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                              if (!list.isEmpty() && list.size() - 1 >= loc)
                              {
                                list.remove(loc);
                              }
                              array.setLength(list.size());
                              for (int i = 0; i < array.getLength(); i++)
                              {
                                Data ziel = array.getItem(i);
                                Data quelle = list.get(i);
                                // ziel = quelle;
                                DataHelper.kopiereRekursiv(quelle, ziel);
                              }
                            }
                            else
                            {
                              // Array um ein Feldelement erweitern
                              array.setLength(array.getLength() + 1);
                              // Index dieses neuen Feldelements ist um eins kleiner
                              Data item = array.getItem(array.getLength() - 1);
                              item.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue()
                              .setText(doModel2.getPrioritaet());
                              item.getItem("WVZInhalt").getItem("Blinken").asTextValue()
                              .setText(doModel2.getBlinken() > 0 ? "Ja" : "Nein");
                              item.getItem("WVZInhalt").getItem("Blinktakt").asTextValue()
                              .setText(doModel2.getBlinktakt());
                              item.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue()
                              .setText(doModel2.getNotwendig() > 0 ? "Ja" : "Nein");
                              item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue()
                              .setText(doModel2.getAbsolutNotwendig() > 0 ? "Ja" : "Nein");
                              item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asUnscaledValue()
                              .set(doModel2.getVerschiebungVorgaenger());
                              item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asUnscaledValue()
                              .set(doModel2.getVerschiebungNachfolger());
                              
                              // item.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue().setText("Ja");
                              // item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue().setText("Nein");
                              // item.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue().setText("undefiniert");
                              // item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asTextValue().setText("0");
                              // item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asTextValue().setText("0");
                              // item.getItem("WVZInhalt").getItem("Blinken").asTextValue().setText("Nein");
                              // item.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText("Undefiniert");
                              item.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("TextAusrichtung")
                              .asTextValue().setText("Linksbndig");
                              
                              // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                              // // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                              // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                              dynamischerWechseltext = item.getItem("WVZInhalt").getItem("DynamischerWechseltext");
                              uebernehmeDaten(kapsel, dynamischerWechseltext);
                              
                              item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                              item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);
                            }
                          }
                          
                          EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                          
                          for (DoMitVerbLinieEditPart epart : list1)
                          {
                            DoModelMitVerbLinie model = epart.getModel();
                            SetCommand cmd = new SetCommand(model, feature, kapsel);
                            if (kapsel.isInaktiv())
                            {
                              cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                            }
                            getCommandStack().execute(cmd);
                          }
                          
                          // Array array = data.getItem("AnzeigenDatenstze").asArray();
                          // SystemObject wvzInhalt = null;
                          //
                          // // Index wird nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                          // int loc = 0;
                          // List<Data> list = new ArrayList<Data>();
                          // for (int i = 0; i < array.getLength(); i++)
                          // {
                          // Data item = array.getItem(i);
                          // SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                          // if (anzeige != null && so != null && anzeige.equals(so))
                          // {
                          // if (_connection != null)
                          // {
                          // if (!inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) &&
                          // !inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                          // {
                          // // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                          // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                          // item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(wvzInhalt);
                          // }
                          // else
                          // {
                          // loc = i;
                          // }
                          // // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                          // // break;
                          // }
                          // }
                          // // Liste wird vorbelegt
                          // list.add(item);
                          // }
                          //
                          // if (wvzInhalt == null)
                          // {
                          // if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                          // {
                          // // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                          // if (!list.isEmpty() && list.size() - 1 >= loc)
                          // {
                          // list.remove(loc);
                          // }
                          // array.setLength(list.size());
                          // for (int i = 0; i < array.getLength(); i++)
                          // {
                          // Data ziel = array.getItem(i);
                          // Data quelle = list.get(i);
                          // // ziel = quelle;
                          // DataHelper.kopiereRekursiv(quelle, ziel);
                          // }
                          // }
                          // else
                          // {
                          // // Array um ein Feldelement erweitern
                          // array.setLength(array.getLength() + 1);
                          // // Index dieses neuen Feldelements ist um eins kleiner
                          // Data item = array.getItem(array.getLength() - 1);
                          //
                          // item.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue().setText(doModel2.getPrioritaet());
                          // item.getItem("WVZInhalt").getItem("Blinken").asTextValue().setText(doModel2.getBlinken() > 0
                          // ? "Ja" : "Nein");
                          // item.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText(doModel2.getBlinktakt());
                          // item.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue().setText(doModel2.getNotwendig()
                          // > 0 ? "Ja" : "Nein");
                          // item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue().setText(doModel2.getAbsolutNotwendig()
                          // > 0 ? "Ja" : "Nein");
                          // item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asUnscaledValue().set(doModel2.getVerschiebungVorgaenger());
                          // item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asUnscaledValue().set(doModel2.getVerschiebungNachfolger());
                          //
                          // item.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("TextAusrichtung").asTextValue()
                          // .setText("Linksbndig");
                          //
                          // // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                          // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                          //
                          // item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                          // item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(wvzInhalt);
                          //
                          // }
                          // }
                          //
                          // EStructuralFeature feature =
                          // AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                          //
                          // for (DoMitVerbLinieEditPart epart : list1)
                          // {
                          // DoModelMitVerbLinie model = epart.getModel();
                          // SetCommand cmd = new SetCommand(model, feature, kapsel);
                          // if (kapsel.isInaktiv()){
                          // cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                          // }
                          // getCommandStack().execute(cmd);
                          // }
                        }
                        else
                        {
                          System.out.println("kapsel ist null!!!");
                        }
                      }
                      else
                      {
                        System.out.println(doModel2 + " " + figure2);
                      }
                    }
                  }
                  
                }
              }
            }
          }
          else
          {
            AnzeigeInhaltFreiTextDialog anzeigeInhaltAuswahlDialog = new AnzeigeInhaltFreiTextDialog(PlatformUI
                .getWorkbench().getDisplay().getActiveShell());
            
            if (doModel != null && figure != null)
            {
              SystemObject so = doModel.getSystemObject();
              
              System.out.println("ActionParameterAnzeigeABC.proceed(): " + so);
              
              Object value = doModel.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
              VisuAnzeigeKapsel kapsel = null;
              if (value instanceof VisuAnzeigeKapsel)
              {
                kapsel = (VisuAnzeigeKapsel)value;
              }
//            else if (value instanceof ResultData)
//            {
//              ResultData result = (ResultData)value;
//              kapsel = DaVHelper.holeDynamischerWechselTextAusDaVData(result);
//              
//            }
              else
              {
                kapsel = new VisuAnzeigeKapsel(new String[] {}, fonts[0].getPid(),
                    EnumDynamischerWechseltextTextAusrichtung.Linksbndig.toString(), 4);
              }
              VisuAnzeigeKapsel kapselNeu = anzeigeInhaltAuswahlDialog.oeffne(kapsel, doModel.getSystemObject());
              if (kapselNeu != null)
              {
                for (DoMitVerbLinieEditPart domep2 : list1)
                {
                  AnzeigeModelTypAbstract doModel2 = (AnzeigeModelTypAbstract)domep2.getModel();
                  AnzeigeElementFigure figure2 = (AnzeigeElementFigure)domep2.getFigure();
                  System.out.println("kapsel: " + kapselNeu);
                  SystemObject so2 = doModel2.getSystemObject();
                  Array array = data.getItem("AnzeigenDatenstze").asArray();
                  Data dynamischerWechseltext = null;
                  // Index nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                  int loc = 0;
                  List<Data> list = new ArrayList<Data>();
                  for (int i = 0; i < array.getLength(); i++)
                  {
                    Data item = array.getItem(i);
                    SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                    
                    if (anzeige != null && so != null && anzeige.equals(so))
                    {
                      if (_connection != null)
                      {
                        if (!kapselNeu.isInaktiv())
                        {
                          dynamischerWechseltext = item.getItem("WVZInhalt").getItem("DynamischerWechseltext");
                          uebernehmeDaten(kapselNeu, dynamischerWechseltext);
                          item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);
                        }
                        else
                        {
                          loc = i;
                        }
                        // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                        // break;
                      }
                    }
                    // Liste wird vorbelegt
                    list.add(item);
                  }
                  
                  if (dynamischerWechseltext == null)
                  {
                    if (kapselNeu.isInaktiv())
                    {
                      // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                      if (!list.isEmpty() && list.size() - 1 >= loc)
                      {
                        list.remove(loc);
                      }
                      array.setLength(list.size());
                      for (int i = 0; i < array.getLength(); i++)
                      {
                        Data ziel = array.getItem(i);
                        Data quelle = list.get(i);
                        // ziel = quelle;
                        DataHelper.kopiereRekursiv(quelle, ziel);
                      }
                    }
                    else
                    {
                      // Array um ein Feldelement erweitern
                      array.setLength(array.getLength() + 1);
                      // Index dieses neuen Feldelements ist um eins kleiner
                      Data item = array.getItem(array.getLength() - 1);
                      item.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue()
                      .setText(doModel2.getPrioritaet());
                      item.getItem("WVZInhalt").getItem("Blinken").asTextValue()
                      .setText(doModel2.getBlinken() > 0 ? "Ja" : "Nein");
                      item.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText(doModel2.getBlinktakt());
                      item.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue()
                      .setText(doModel2.getNotwendig() > 0 ? "Ja" : "Nein");
                      item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue()
                      .setText(doModel2.getAbsolutNotwendig() > 0 ? "Ja" : "Nein");
                      item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asUnscaledValue()
                      .set(doModel2.getVerschiebungVorgaenger());
                      item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asUnscaledValue()
                      .set(doModel2.getVerschiebungNachfolger());
                      
                      // item.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue().setText("Ja");
                      // item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue().setText("Nein");
                      // item.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue().setText("undefiniert");
                      // item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asTextValue().setText("0");
                      // item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asTextValue().setText("0");
                      // item.getItem("WVZInhalt").getItem("Blinken").asTextValue().setText("Nein");
                      // item.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText("Undefiniert");
                      item.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("TextAusrichtung")
                      .asTextValue().setText("Linksbndig");
                      
                      // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                      // // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                      // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                      dynamischerWechseltext = item.getItem("WVZInhalt").getItem("DynamischerWechseltext");
                      uebernehmeDaten(kapselNeu, dynamischerWechseltext);
                      
                      item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                      item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);
                      
                    }
                  }
                  
                  EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                  
                  for (DoMitVerbLinieEditPart epart : list1)
                  {
                    DoModelMitVerbLinie model = epart.getModel();
                    SetCommand cmd = new SetCommand(model, feature, kapselNeu);
                    if (kapsel.isInaktiv())
                    {
                      cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                    }
                    getCommandStack().execute(cmd);
                  }
                }
              }
            }
          }
        }
          
        }
        
        
    }
    else
    {
      System.out.println("Viewer ist null");
    }
  }
  
  private void proceed1(Data data)
  {
    System.out.println("ActionKontextParameterAnzeigeABC.proceed1()");
    
    GraphicalViewer viewer = getViewer();
    if (viewer != null)
    {
      ISelection selection = viewer.getSelection();

      StructuredSelection ss = (StructuredSelection)selection;

      List<DoMitVerbLinieEditPart> list1 = new ArrayList<DoMitVerbLinieEditPart>();

      Iterator it = ss.iterator();
      while (it.hasNext())
      {
        Object obj = it.next();

        DoMitVerbLinieEditPart domep = null;
        AnzeigeModelTypAbstract doModel = null;
        AnzeigeElementFigure figure = null;

        if (obj instanceof DoMitVerbLinieEditPart)
        {
          domep = (DoMitVerbLinieEditPart)obj;
          DoModelMitVerbLinie model = domep.getModel();
          if (model instanceof AnzeigeModelTypAbstract)
          {
            doModel = (AnzeigeModelTypAbstract)model;
            fonts = DaVHelper.holeFontsUeberAnzeige(_connection, doModel.getSystemObject());
            list1.add(domep);
          }
        }

        boolean unterschiedlich = false;

        List<DoMitVerbLinieEditPart> copyList = new ArrayList<DoMitVerbLinieEditPart>(list1);

        for (DoMitVerbLinieEditPart epart : list1)
        {
          for (DoMitVerbLinieEditPart copy : copyList)
          {
            if (!epart.getClass().equals(copy.getClass()))
            {
              unterschiedlich = true;
              break;
            }
          }
          if (unterschiedlich)
            break;
        }

        if (unterschiedlich)
        {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_ERROR | SWT.OK);
          mb.setText("Fehler Selektion");
          mb.setMessage("Es knnnen nur Inhalte eines Anzeigetyps gesetzt werden!");
          mb.open();
          return;
        }
      }

      if (list1.size() > 0)
      {
        
//        for (DoMitVerbLinieEditPart domep : list1)
        {
          DoMitVerbLinieEditPart domep = list1.get(0);
          AnzeigeModelTypAbstract doModel = (AnzeigeModelTypAbstract)domep.getModel();
          AnzeigeElementFigure figure = (AnzeigeElementFigure)domep.getFigure();
          
//        System.out.println(domep.getClass().getSimpleName() + " : " + doModel.getClass().getSimpleName());
          
          if (!(domep instanceof AnzeigeTypDEditPart || domep instanceof AnzeigeTypD3EditPart))
          {
            AnzeigeInhaltAuswahlDialog anzeigeInhaltAuswahlDialog = new AnzeigeInhaltAuswahlDialog(PlatformUI
                .getWorkbench().getDisplay().getActiveShell(), true);
            AnzeigeInhaltFreiTextDialog freiTextAuswahlDialog = new AnzeigeInhaltFreiTextDialog(PlatformUI.getWorkbench()
                .getDisplay().getActiveShell(), true);
            
            if (doModel != null && figure != null)
            {
              EList<String> inhalte1 = (EList<String>)doModel.getInhalte1();
              if (inhalte1 != null)
              {
                inhalte1.add(AnzeigeInhaltTyp.BELIEBIG_BI.getType());
//              inhalte1.add(AnzeigeInhaltTyp._BI.getType());
                inhalte1.add(AnzeigeInhaltTyp.BEIBEHALTEN_BI.getType());
                
                if (domep instanceof AnzeigeTypC2EditPart)
                  inhalte1.add(AnzeigeInhaltTyp.FREITEXT_BI.getType());
                
                AnzeigeInhaltTyp inhalt = anzeigeInhaltAuswahlDialog.oeffne(figure.getTyp(), inhalte1);
                
                if (inhalt != null)
                {
                  boolean geoeffnet = false;
                  
                  for (DoMitVerbLinieEditPart domep2 : list1)
                  {
                    AnzeigeModelTypAbstract doModel2 = (AnzeigeModelTypAbstract)domep2.getModel();
//                    AnzeigeModelTypAbstract doModel2 = (AnzeigeModelTypAbstract)domep2.getModel();
                    Object value2 = doModel2.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
                    if (value2 instanceof AnzeigeInhaltTyp)
                    {
                      AnzeigeInhaltTyp inhaltAlt = (AnzeigeInhaltTyp)value2;
                      if (!inhaltAlt.equals(inhalt)){
                        doModel2.setAbsolutNotwendig(0);
                        doModel2.setNotwendig(0);
                        doModel2.setBlinken(0);
                        doModel2.setBlinktakt("Undefiniert");
                        doModel2.setPrioritaet("undefiniert");
//                        domep2.abmeldenTaktgeber();
                      }
                    }
                    AnzeigeElementFigure figure2 = (AnzeigeElementFigure)domep2.getFigure();
                    // System.out.println("Inhalt: " + inhalt.getType() + " -> " + doModel2);
                    SystemObject so = doModel2.getSystemObject();
                    
                    if (!inhalt.equals(AnzeigeInhaltTyp.FREITEXT_BI))
                    {
                      
                      System.out.println("---> dataName: " + data.getName() + " : " + data.getAttributeType());
                      
                      Array array = data.getItem("Anzeige").asArray();
                      SystemObject wvzInhalt = null;
                      
                      // Index nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                      int loc = 0;
                      List<Data> list = new ArrayList<Data>();
                      for (int i = 0; i < array.getLength(); i++)
                      {
                        Data item = array.getItem(i);
                        SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                        if (anzeige != null && so != null && anzeige.equals(so))
                        {
                          if (_connection != null)
                          {
                            if (!inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) && !inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                            {
                              // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                              wvzInhalt = inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI) ?
                                  DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType()) :
                                    DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                                  
                                  item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue()
                                  .setSystemObject(wvzInhalt);
                                  
                                  item.getItem("Schaltbild").getItem("DynamischerWechseltext").getItem("TextAusrichtung")
                                  .asTextValue().setText("Linksbndig");
                                  item.getItem("Schaltbild").getItem("DynamischerWechseltext").getItem("Font")
                                  .asReferenceValue().setSystemObject(null);
                                  item.getItem("Schaltbild").getItem("DynamischerWechseltext").getItem("Textdefinition")
                                  .asArray().setLength(0);
                            }
                            else
                            {
                              loc = i;
                            }
                            // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                            // break;
                          }
                        }
                        // Liste wird vorbelegt
                        list.add(item);
                      }
                      
                      if (wvzInhalt == null)
                      {
                        if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                        {
                          // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                          if (!list.isEmpty() && list.size() - 1 >= loc)
                          {
                            list.remove(loc);
                          }
                          array.setLength(list.size());
                          for (int i = 0; i < array.getLength(); i++)
                          {
                            Data ziel = array.getItem(i);
                            Data quelle = list.get(i);
                            // ziel = quelle;
                            DataHelper.kopiereRekursiv(quelle, ziel);
                          }
                        }
                        else
                        {
                          // Array um ein Feldelement erweitern
                          array.setLength(array.getLength() + 1);
                          // Index dieses neuen Feldelements ist um eins kleiner
                          Data item = array.getItem(array.getLength() - 1);
                          
                          item.getItem("Schaltbild").getItem("Blinken").asTextValue().setText("Nein");
                          item.getItem("Schaltbild").getItem("Blinktakt").asTextValue().setText("Undefiniert");
                          item.getItem("Schaltbild").getItem("DynamischerWechseltext").getItem("TextAusrichtung")
                          .asTextValue().setText("Linksbndig");
                          
                          // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                          wvzInhalt = inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI) ?
                              DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType()) :
                                DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                              
                              item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                              item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue()
                              .setSystemObject(wvzInhalt);
                              
                        }
                      }
                      
                      EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                      
                      for (DoMitVerbLinieEditPart epart : list1)
                      {
                        DoModelMitVerbLinie model = epart.getModel();
                        final SetCommand cmd = new SetCommand(model, feature, inhalt);
                        getCommandStack().execute(cmd);
                      }
                      
                    }
                    else
                    {
                      
                      // System.out.println("--------> inhaltTyp: " + inhalt.getType() + " -> " + doModel2);
                      
                      if (doModel2 != null && figure2 != null)
                      {
                        
                        Object value = doModel2
                        .eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
                        
                        VisuAnzeigeKapsel kapsel = null;
                        if (value instanceof VisuAnzeigeKapsel)
                        {
                          kapsel = (VisuAnzeigeKapsel)value;
                          kapsel.setSchriftart(fonts[0].getPid());
                        }
//                      else if (value instanceof ResultData)
//                      {
//                        ResultData result = (ResultData)value;
//                        kapsel = DaVHelper.holeDynamischerWechselTextAusDaVData(result);
//                        
//                      }
                        else
                        {
                          kapsel = new VisuAnzeigeKapsel();
                          kapsel.setAbstand(0);
                          kapsel.setAusrichtung(EnumDynamischerWechseltextTextAusrichtung.Zentriert.toString());
                          kapsel.setSchriftart(fonts[0].getPid());
                          kapsel.setPidsZeichen(new String[0]);
                        }
                        
                        if (!geoeffnet)
                        {
                          kapsel = freiTextAuswahlDialog.oeffne(kapsel, doModel.getSystemObject());
                        }
                        
                        if (kapsel != null)
                        {
                          geoeffnet = true;
                          // System.out.println("kapsel: " + kapsel);
                          SystemObject so2 = doModel2.getSystemObject();
                          Array array = data.getItem("Anzeige").asArray();
                          Data dynamischerWechseltext = null;
                          // Index nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                          int loc = 0;
                          List<Data> list = new ArrayList<Data>();
                          for (int i = 0; i < array.getLength(); i++)
                          {
                            Data item = array.getItem(i);
                            SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                            
                            if (anzeige != null && so != null && anzeige.equals(so))
                            {
                              if (_connection != null)
                              {
                                if (!kapsel.isInaktiv())
                                {
                                  dynamischerWechseltext = item.getItem("Schaltbild").getItem("DynamischerWechseltext");
                                  if (!kapsel.isBeibehalten()) {
                                    uebernehmeDaten(kapsel, dynamischerWechseltext);
                                    item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);  
                                  }
                                  else {
                                    SystemObject wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, 
                                        AnzeigeInhaltTyp.BEIBEHALTEN_BI.getType());
                                    item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(wvzInhalt);
                                    
                                    dynamischerWechseltext.getItem("TextAusrichtung").asTextValue().setText("Linksbndig");
                                    dynamischerWechseltext.getItem("Font").asReferenceValue().setSystemObject(null);
                                    dynamischerWechseltext.getItem("Textdefinition").asArray().setLength(0);
                                  }
                                }
                                else
                                {
                                  loc = i;
                                }
                                // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                                // break;
                              }
                            }
                            // Liste wird vorbelegt
                            list.add(item);
                          }
                          
                          if (dynamischerWechseltext == null)
                          {
                            if (kapsel.isInaktiv())
                            {
                              // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                              if (!list.isEmpty() && list.size() - 1 >= loc)
                              {
                                list.remove(loc);
                              }
                              array.setLength(list.size());
                              for (int i = 0; i < array.getLength(); i++)
                              {
                                Data ziel = array.getItem(i);
                                Data quelle = list.get(i);
                                // ziel = quelle;
                                DataHelper.kopiereRekursiv(quelle, ziel);
                              }
                            }
                            else
                            {
                              // Array um ein Feldelement erweitern
                              array.setLength(array.getLength() + 1);
                              // Index dieses neuen Feldelements ist um eins kleiner
                              Data item = array.getItem(array.getLength() - 1);
                              item.getItem("Schaltbild").getItem("Blinken").asTextValue()
                              .setText(doModel2.getBlinken() > 0 ? "Ja" : "Nein");
                              item.getItem("Schaltbild").getItem("Blinktakt").asTextValue()
                              .setText(doModel2.getBlinktakt());
                              dynamischerWechseltext = item.getItem("Schaltbild").getItem("DynamischerWechseltext");
                              dynamischerWechseltext.getItem("TextAusrichtung").asTextValue().setText("Linksbndig");
                              
                              // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                              // // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                              // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                              
                              if (!kapsel.isBeibehalten()) {
                                uebernehmeDaten(kapsel, dynamischerWechseltext);
                                item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);  
                              }
                              else {
                                SystemObject wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, 
                                    AnzeigeInhaltTyp.BEIBEHALTEN_BI.getType());
                                item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(wvzInhalt);
                                
                                dynamischerWechseltext.getItem("TextAusrichtung").asTextValue().setText("Linksbndig");
                                dynamischerWechseltext.getItem("Font").asReferenceValue().setSystemObject(null);
                                dynamischerWechseltext.getItem("Textdefinition").asArray().setLength(0);
                              }
                              
                              item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                              
                            }
                          }
                          
                          EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                          
                          for (DoMitVerbLinieEditPart epart : list1)
                          {
                            DoModelMitVerbLinie model = epart.getModel();
                            SetCommand cmd;
                            if (kapsel.isBeibehalten()) {
                              cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BEIBEHALTEN_BI);
                            }
                            else if (kapsel.isInaktiv())
                            {
                              cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                            }
                            else {
                              cmd = new SetCommand(model, feature, kapsel);  
                            }
                            
                            getCommandStack().execute(cmd);
                          }
                          
                          // Array array = data.getItem("AnzeigenDatenstze").asArray();
                          // SystemObject wvzInhalt = null;
                          //
                          // // Index wird nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                          // int loc = 0;
                          // List<Data> list = new ArrayList<Data>();
                          // for (int i = 0; i < array.getLength(); i++)
                          // {
                          // Data item = array.getItem(i);
                          // SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                          // if (anzeige != null && so != null && anzeige.equals(so))
                          // {
                          // if (_connection != null)
                          // {
                          // if (!inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) &&
                          // !inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                          // {
                          // // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                          // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                          // item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(wvzInhalt);
                          // }
                          // else
                          // {
                          // loc = i;
                          // }
                          // // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                          // // break;
                          // }
                          // }
                          // // Liste wird vorbelegt
                          // list.add(item);
                          // }
                          //
                          // if (wvzInhalt == null)
                          // {
                          // if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                          // {
                          // // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                          // if (!list.isEmpty() && list.size() - 1 >= loc)
                          // {
                          // list.remove(loc);
                          // }
                          // array.setLength(list.size());
                          // for (int i = 0; i < array.getLength(); i++)
                          // {
                          // Data ziel = array.getItem(i);
                          // Data quelle = list.get(i);
                          // // ziel = quelle;
                          // DataHelper.kopiereRekursiv(quelle, ziel);
                          // }
                          // }
                          // else
                          // {
                          // // Array um ein Feldelement erweitern
                          // array.setLength(array.getLength() + 1);
                          // // Index dieses neuen Feldelements ist um eins kleiner
                          // Data item = array.getItem(array.getLength() - 1);
                          //
                          // item.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue().setText(doModel2.getPrioritaet());
                          // item.getItem("WVZInhalt").getItem("Blinken").asTextValue().setText(doModel2.getBlinken() > 0
                          // ? "Ja" : "Nein");
                          // item.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText(doModel2.getBlinktakt());
                          // item.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue().setText(doModel2.getNotwendig()
                          // > 0 ? "Ja" : "Nein");
                          // item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue().setText(doModel2.getAbsolutNotwendig()
                          // > 0 ? "Ja" : "Nein");
                          // item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asUnscaledValue().set(doModel2.getVerschiebungVorgaenger());
                          // item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asUnscaledValue().set(doModel2.getVerschiebungNachfolger());
                          //
                          // item.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("TextAusrichtung").asTextValue()
                          // .setText("Linksbndig");
                          //
                          // // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                          // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                          //
                          // item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                          // item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(wvzInhalt);
                          //
                          // }
                          // }
                          //
                          // EStructuralFeature feature =
                          // AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                          //
                          // for (DoMitVerbLinieEditPart epart : list1)
                          // {
                          // DoModelMitVerbLinie model = epart.getModel();
                          // SetCommand cmd = new SetCommand(model, feature, kapsel);
                          // if (kapsel.isInaktiv()){
                          // cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                          // }
                          // getCommandStack().execute(cmd);
                          // }
                        }
                        else
                        {
                          System.out.println("kapsel ist null!!!");
                        }
                      }
                      else
                      {
                        System.out.println(doModel2 + " " + figure2);
                      }
                    }
                  }
                }
              }
            }
          }
          else
          {
            AnzeigeInhaltFreiTextDialog anzeigeInhaltAuswahlDialog = new AnzeigeInhaltFreiTextDialog(PlatformUI
                .getWorkbench().getDisplay().getActiveShell(), true);
            
            if (doModel != null && figure != null)
            {
              SystemObject so = doModel.getSystemObject();
              
              System.out.println("ActionParameterAnzeigeABC.proceed(): " + so);
              
              Object value = doModel.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
              VisuAnzeigeKapsel kapsel = null;
              if (value instanceof VisuAnzeigeKapsel)
              {
                kapsel = (VisuAnzeigeKapsel)value;
              }
//            else if (value instanceof ResultData)
//            {
//              ResultData result = (ResultData)value;
//              kapsel = DaVHelper.holeDynamischerWechselTextAusDaVData(result);
//              
//            }
              else
              {
                kapsel = new VisuAnzeigeKapsel(new String[] {}, fonts[0].getPid(),
                    EnumDynamischerWechseltextTextAusrichtung.Linksbndig.toString(), 4);
              }
              VisuAnzeigeKapsel kapselNeu = anzeigeInhaltAuswahlDialog.oeffne(kapsel, doModel.getSystemObject());
              if (kapselNeu != null)
              {
                for (DoMitVerbLinieEditPart domep2 : list1)
                {
                  AnzeigeModelTypAbstract doModel2 = (AnzeigeModelTypAbstract)domep2.getModel();
                  AnzeigeElementFigure figure2 = (AnzeigeElementFigure)domep2.getFigure();
                  System.out.println("kapsel: " + kapselNeu);
                  SystemObject so2 = doModel2.getSystemObject();
                  Array array = data.getItem("Anzeige").asArray();
                  Data dynamischerWechseltext = null;
                  // Index nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                  int loc = 0;
                  List<Data> list = new ArrayList<Data>();
                  for (int i = 0; i < array.getLength(); i++)
                  {
                    Data item = array.getItem(i);
                    SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                    
                    if (anzeige != null && so != null && anzeige.equals(so))
                    {
                      if (_connection != null)
                      {
                        if (!kapselNeu.isInaktiv())
                        {
                          dynamischerWechseltext = item.getItem("Schaltbild").getItem("DynamischerWechseltext");
                          if (!kapselNeu.isBeibehalten()) {
                            uebernehmeDaten(kapselNeu, dynamischerWechseltext);
                            item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);                          
                          }
                          else {
                            SystemObject wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, 
                                AnzeigeInhaltTyp.BEIBEHALTEN_BI.getType());
                            item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(wvzInhalt);
                            
                            dynamischerWechseltext.getItem("TextAusrichtung").asTextValue().setText("Linksbndig");
                            dynamischerWechseltext.getItem("Font").asReferenceValue().setSystemObject(null);
                            dynamischerWechseltext.getItem("Textdefinition").asArray().setLength(0);
                          }
                        }
                        else
                        {
                          loc = i;
                        }
                        // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                        // break;
                      }
                    }
                    // Liste wird vorbelegt
                    list.add(item);
                  }
                  
                  if (dynamischerWechseltext == null)
                  {
                    if (kapselNeu.isInaktiv())
                    {
                      // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                      if (!list.isEmpty() && list.size() - 1 >= loc)
                      {
                        list.remove(loc);
                      }
                      array.setLength(list.size());
                      for (int i = 0; i < array.getLength(); i++)
                      {
                        Data ziel = array.getItem(i);
                        Data quelle = list.get(i);
                        // ziel = quelle;
                        DataHelper.kopiereRekursiv(quelle, ziel);
                      }
                    }
                    else
                    {
                      // Array um ein Feldelement erweitern
                      array.setLength(array.getLength() + 1);
                      // Index dieses neuen Feldelements ist um eins kleiner
                      Data item = array.getItem(array.getLength() - 1);
                      item.getItem("Schaltbild").getItem("Blinken").asTextValue()
                      .setText(doModel2.getBlinken() > 0 ? "Ja" : "Nein");
                      item.getItem("Schaltbild").getItem("Blinktakt").asTextValue().setText(doModel2.getBlinktakt());
                      
                      dynamischerWechseltext = item.getItem("Schaltbild").getItem("DynamischerWechseltext");
                      dynamischerWechseltext.getItem("TextAusrichtung").asTextValue().setText("Linksbndig");
                      
                      // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                      // // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                      // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                      
                      if (!kapselNeu.isBeibehalten()) {
                        uebernehmeDaten(kapselNeu, dynamischerWechseltext);
                        item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);
                      }
                      else {
                        SystemObject wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, 
                            AnzeigeInhaltTyp.BEIBEHALTEN_BI.getType());
                        item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(wvzInhalt);
                        
                        dynamischerWechseltext.getItem("TextAusrichtung").asTextValue().setText("Linksbndig");
                        dynamischerWechseltext.getItem("Font").asReferenceValue().setSystemObject(null);
                        dynamischerWechseltext.getItem("Textdefinition").asArray().setLength(0);
                      }
                      item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                    }
                  }
                  
                  EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                  
                  for (DoMitVerbLinieEditPart epart : list1)
                  {
                    DoModelMitVerbLinie model = epart.getModel();
                    SetCommand cmd;
                    if (kapsel.isInaktiv())
                    {
                      cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                    }
                    else if (kapselNeu.isBeibehalten()) {
                      cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BEIBEHALTEN_BI);
                    }
                    else {
                      cmd = new SetCommand(model, feature, kapselNeu);
                    }
                    
                    getCommandStack().execute(cmd);
                  }
                }
              }
            }
          }
        }
          
        }
        
    }
    else
    {
      System.out.println("Viewer ist null");
    }
  }
  
  private void proceed2(Data data)
  {
    System.out.println("ActionKontextParameterAnzeigeABC.proceed2()");
    
    GraphicalViewer viewer = getViewer();
    if (viewer != null)
    {
      ISelection selection = viewer.getSelection();
      
      StructuredSelection ss = (StructuredSelection)selection;
      
      List<DoMitVerbLinieEditPart> list1 = new ArrayList<DoMitVerbLinieEditPart>();
      
      Iterator it = ss.iterator();
      while (it.hasNext())
      {
        Object obj = it.next();
        
        DoMitVerbLinieEditPart domep = null;
        AnzeigeModelTypAbstract doModel = null;
        AnzeigeElementFigure figure = null;
        
        if (obj instanceof DoMitVerbLinieEditPart)
        {
          domep = (DoMitVerbLinieEditPart)obj;
          DoModelMitVerbLinie model = domep.getModel();
          if (model instanceof AnzeigeModelTypAbstract)
          {
            doModel = (AnzeigeModelTypAbstract)model;
            list1.add(domep);
          }
        }
        
        boolean unterschiedlich = false;
        
        List<DoMitVerbLinieEditPart> copyList = new ArrayList<DoMitVerbLinieEditPart>(list1);
        
        for (DoMitVerbLinieEditPart epart : list1)
        {
          for (DoMitVerbLinieEditPart copy : copyList)
          {
            if (!epart.getClass().equals(copy.getClass()))
            {
              unterschiedlich = true;
              break;
            }
          }
          if (unterschiedlich)
            break;
        }
        
        if (unterschiedlich)
        {
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_ERROR | SWT.OK);
          mb.setText("Fehler Selektion");
          mb.setMessage("Es knnnen nur Inhalte eines Anzeigetyps gesetzt werden!");
          mb.open();
          return;
        }
        
      }
      
      if (list1.size() > 0)
      {
        DoMitVerbLinieEditPart domep = list1.get(0);
        AnzeigeModelTypAbstract doModel = (AnzeigeModelTypAbstract)domep.getModel();
        AnzeigeElementFigure figure = (AnzeigeElementFigure)domep.getFigure();
        
//        System.out.println(domep.getClass().getSimpleName() + " : " + doModel.getClass().getSimpleName());
        
        if (!(domep instanceof AnzeigeTypDEditPart || domep instanceof AnzeigeTypD3EditPart))
        {
          AnzeigeInhaltAuswahlDialog anzeigeInhaltAuswahlDialog = new AnzeigeInhaltAuswahlDialog(PlatformUI
              .getWorkbench().getDisplay().getActiveShell(), true);
          AnzeigeInhaltFreiTextDialog freiTextAuswahlDialog = new AnzeigeInhaltFreiTextDialog(PlatformUI.getWorkbench()
              .getDisplay().getActiveShell());
          
          if (doModel != null && figure != null)
          {
            EList<String> inhalte1 = (EList<String>)doModel.getInhalte1();
            if (inhalte1 != null)
            {
              inhalte1.add(AnzeigeInhaltTyp.BELIEBIG_BI.getType());
              
              if (domep instanceof AnzeigeTypC2EditPart)
                inhalte1.add(AnzeigeInhaltTyp.FREITEXT_BI.getType());
              
              AnzeigeInhaltTyp inhalt = anzeigeInhaltAuswahlDialog.oeffne(figure.getTyp(), inhalte1);
              
              
              
              if (inhalt != null)
              {
               
                boolean geoeffnet = false;
                
                for (DoMitVerbLinieEditPart domep2 : list1)
                {
                  
                  AnzeigeModelTypAbstract doModel2 = (AnzeigeModelTypAbstract)domep2.getModel();
                  Object value2 = doModel2.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
                  if (value2 instanceof AnzeigeInhaltTyp)
                  {
                    AnzeigeInhaltTyp inhaltAlt = (AnzeigeInhaltTyp)value2;
                    if (!inhaltAlt.equals(inhalt)){
                      doModel2.setAbsolutNotwendig(0);
                      doModel2.setNotwendig(0);
                      doModel2.setBlinken(0);
                      doModel2.setBlinktakt("Undefiniert");
                      doModel2.setPrioritaet("undefiniert");
//                      domep2.abmeldenTaktgeber();
                    }
                  }
                  AnzeigeElementFigure figure2 = (AnzeigeElementFigure)domep2.getFigure();
                  // System.out.println("Inhalt: " + inhalt.getType() + " -> " + doModel2);
                  SystemObject so = doModel2.getSystemObject();
                  
                  if (!inhalt.equals(AnzeigeInhaltTyp.FREITEXT_BI))
                  {
                    
                    System.out.println("---> dataName: " + data.getName() + " : " + data.getAttributeType());
                    
                    Array array = data.asArray();
//                    Array array = data.getItem("AnzeigenDatenstze").asArray();
                    SystemObject wvzInhalt = null;
                    
                    // Index nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                    int loc = 0;
                    List<Data> list = new ArrayList<Data>();
                    for (int i = 0; i < array.getLength(); i++)
                    {
                      Data item = array.getItem(i);
                      SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                      if (anzeige != null && so != null && anzeige.equals(so))
                      {
                        if (_connection != null)
                        {
                          if (!inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) && !inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                          {
                            // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                            wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                            item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue()
                            .setSystemObject(wvzInhalt);
                            
                            item.getItem("Schaltbild").getItem("DynamischerWechseltext").getItem("TextAusrichtung")
                            .asTextValue().setText("Linksbndig");
                            item.getItem("Schaltbild").getItem("DynamischerWechseltext").getItem("Font")
                            .asReferenceValue().setSystemObject(null);
                            item.getItem("Schaltbild").getItem("DynamischerWechseltext").getItem("Textdefinition")
                            .asArray().setLength(0);
                          }
                          else
                          {
                            loc = i;
                          }
                          // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                          // break;
                        }
                      }
                      // Liste wird vorbelegt
                      list.add(item);
                    }
                    
                    
                    if (wvzInhalt == null)
                    {
                      if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                      {
                        // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                        if (!list.isEmpty() && list.size() - 1 >= loc)
                        {
                          list.remove(loc);
                        }
                        array.setLength(list.size());
                        for (int i = 0; i < array.getLength(); i++)
                        {
                          Data ziel = array.getItem(i);
                          Data quelle = list.get(i);
                          // ziel = quelle;
                          DataHelper.kopiereRekursiv(quelle, ziel);
                        }
                      }
                      else
                      {
                        // Array um ein Feldelement erweitern
                        array.setLength(array.getLength() + 1);
                        // Index dieses neuen Feldelements ist um eins kleiner
                        Data item = array.getItem(array.getLength() - 1);
                        item.getItem("notwendig").asTextValue().setText("Ja");
//                        item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue().setText("Nein");
                        item.getItem("Prioritt").asTextValue().setText("undefiniert");
//                        item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asTextValue().setText("0");
//                        item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asTextValue().setText("0");
                        
                        item.getItem("Schaltbild").getItem("Blinken").asTextValue().setText("Nein");
                        item.getItem("Schaltbild").getItem("Blinktakt").asTextValue().setText("Undefiniert");
                        item.getItem("Schaltbild").getItem("DynamischerWechseltext").getItem("TextAusrichtung")
                        .asTextValue().setText("Linksbndig");
                        
                        // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                        wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                        
                        item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                        item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue()
                        .setSystemObject(wvzInhalt);
                        
                      }
                    }
                    
                    EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                    
                    for (DoMitVerbLinieEditPart epart : list1)
                    {
                      DoModelMitVerbLinie model = epart.getModel();
                      final SetCommand cmd = new SetCommand(model, feature, inhalt);
                      getCommandStack().execute(cmd);
                    }
                    
                  }
                  else
                  {
                    
                    // System.out.println("--------> inhaltTyp: " + inhalt.getType() + " -> " + doModel2);
                    
                    if (doModel2 != null && figure2 != null)
                    {
                      
                      Object value = doModel2
                      .eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
                      
                      VisuAnzeigeKapsel kapsel = null;
                      if (value instanceof VisuAnzeigeKapsel)
                      {
                        kapsel = (VisuAnzeigeKapsel)value;
                        kapsel.setSchriftart(fonts[0].getPid());
                      }
//                      else if (value instanceof ResultData)
//                      {
//                        ResultData result = (ResultData)value;
//                        kapsel = DaVHelper.holeDynamischerWechselTextAusDaVData(result);
//                        
//                      }
                      else
                      {
                        kapsel = new VisuAnzeigeKapsel();
                        kapsel.setAbstand(0);
                        kapsel.setAusrichtung(EnumDynamischerWechseltextTextAusrichtung.Zentriert.toString());
                        kapsel.setSchriftart(fonts[0].getPid());
                        kapsel.setPidsZeichen(new String[0]);
                      }
                      
                      if (!geoeffnet)
                      {
                        kapsel = freiTextAuswahlDialog.oeffne(kapsel, doModel.getSystemObject());
                      }
                      
                      if (kapsel != null)
                      {
                        geoeffnet = true;
                        // System.out.println("kapsel: " + kapsel);
                        SystemObject so2 = doModel2.getSystemObject();
//                        Array array = data.getItem("AnzeigenDatenstze").asArray();
                        Array array = data.asArray();
                        Data dynamischerWechseltext = null;
                        // Index nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                        int loc = 0;
                        List<Data> list = new ArrayList<Data>();
                        for (int i = 0; i < array.getLength(); i++)
                        {
                          Data item = array.getItem(i);
                          SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                          
                          if (anzeige != null && so != null && anzeige.equals(so))
                          {
                            if (_connection != null)
                            {
                              if (!kapsel.isInaktiv())
                              {
                                dynamischerWechseltext = item.getItem("Schaltbild").getItem("DynamischerWechseltext");
                                uebernehmeDaten(kapsel, dynamischerWechseltext);
                                item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);
                              }
                              else
                              {
                                loc = i;
                              }
                              // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                              // break;
                            }
                          }
                          // Liste wird vorbelegt
                          list.add(item);
                        }
                        
                        if (dynamischerWechseltext == null)
                        {
                          if (kapsel.isInaktiv())
                          {
                            // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                            if (!list.isEmpty() && list.size() - 1 >= loc)
                            {
                              list.remove(loc);
                            }
                            array.setLength(list.size());
                            for (int i = 0; i < array.getLength(); i++)
                            {
                              Data ziel = array.getItem(i);
                              Data quelle = list.get(i);
                              // ziel = quelle;
                              DataHelper.kopiereRekursiv(quelle, ziel);
                            }
                          }
                          else
                          {
                            // Array um ein Feldelement erweitern
                            array.setLength(array.getLength() + 1);
                            // Index dieses neuen Feldelements ist um eins kleiner
                            Data item = array.getItem(array.getLength() - 1);
                            item.getItem("Prioritt").asTextValue()
                            .setText(doModel2.getPrioritaet());
                            item.getItem("Schaltbild").getItem("Blinken").asTextValue()
                            .setText(doModel2.getBlinken() > 0 ? "Ja" : "Nein");
                            item.getItem("Schaltbild").getItem("Blinktakt").asTextValue()
                            .setText(doModel2.getBlinktakt());
                            item.getItem("notwendig").asTextValue()
                            .setText(doModel2.getNotwendig() > 0 ? "Ja" : "Nein");
//                            item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue()
//                            .setText(doModel2.getAbsolutNotwendig() > 0 ? "Ja" : "Nein");
//                            item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asUnscaledValue()
//                            .set(doModel2.getVerschiebungVorgaenger());
//                            item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asUnscaledValue()
//                            .set(doModel2.getVerschiebungNachfolger());
                            
                            // item.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue().setText("Ja");
                            // item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue().setText("Nein");
                            // item.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue().setText("undefiniert");
                            // item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asTextValue().setText("0");
                            // item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asTextValue().setText("0");
                            // item.getItem("WVZInhalt").getItem("Blinken").asTextValue().setText("Nein");
                            // item.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText("Undefiniert");
                            item.getItem("Schaltbild").getItem("DynamischerWechseltext").getItem("TextAusrichtung")
                            .asTextValue().setText("Linksbndig");
                            
                            // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                            // // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                            // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                            dynamischerWechseltext = item.getItem("Schaltbild").getItem("DynamischerWechseltext");
                            uebernehmeDaten(kapsel, dynamischerWechseltext);
                            
                            item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                            item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);
                          }
                        }
                        
                        EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                        
                        for (DoMitVerbLinieEditPart epart : list1)
                        {
                          DoModelMitVerbLinie model = epart.getModel();
                          SetCommand cmd = new SetCommand(model, feature, kapsel);
                          if (kapsel.isInaktiv())
                          {
                            cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                          }
                          getCommandStack().execute(cmd);
                        }
                        
                        // Array array = data.getItem("AnzeigenDatenstze").asArray();
                        // SystemObject wvzInhalt = null;
                        //
                        // // Index wird nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                        // int loc = 0;
                        // List<Data> list = new ArrayList<Data>();
                        // for (int i = 0; i < array.getLength(); i++)
                        // {
                        // Data item = array.getItem(i);
                        // SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                        // if (anzeige != null && so != null && anzeige.equals(so))
                        // {
                        // if (_connection != null)
                        // {
                        // if (!inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) &&
                        // !inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                        // {
                        // // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                        // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                        // item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(wvzInhalt);
                        // }
                        // else
                        // {
                        // loc = i;
                        // }
                        // // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                        // // break;
                        // }
                        // }
                        // // Liste wird vorbelegt
                        // list.add(item);
                        // }
                        //
                        // if (wvzInhalt == null)
                        // {
                        // if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
                        // {
                        // // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                        // if (!list.isEmpty() && list.size() - 1 >= loc)
                        // {
                        // list.remove(loc);
                        // }
                        // array.setLength(list.size());
                        // for (int i = 0; i < array.getLength(); i++)
                        // {
                        // Data ziel = array.getItem(i);
                        // Data quelle = list.get(i);
                        // // ziel = quelle;
                        // DataHelper.kopiereRekursiv(quelle, ziel);
                        // }
                        // }
                        // else
                        // {
                        // // Array um ein Feldelement erweitern
                        // array.setLength(array.getLength() + 1);
                        // // Index dieses neuen Feldelements ist um eins kleiner
                        // Data item = array.getItem(array.getLength() - 1);
                        //
                        // item.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue().setText(doModel2.getPrioritaet());
                        // item.getItem("WVZInhalt").getItem("Blinken").asTextValue().setText(doModel2.getBlinken() > 0
                        // ? "Ja" : "Nein");
                        // item.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText(doModel2.getBlinktakt());
                        // item.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue().setText(doModel2.getNotwendig()
                        // > 0 ? "Ja" : "Nein");
                        // item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue().setText(doModel2.getAbsolutNotwendig()
                        // > 0 ? "Ja" : "Nein");
                        // item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asUnscaledValue().set(doModel2.getVerschiebungVorgaenger());
                        // item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asUnscaledValue().set(doModel2.getVerschiebungNachfolger());
                        //
                        // item.getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("TextAusrichtung").asTextValue()
                        // .setText("Linksbndig");
                        //
                        // // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                        // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                        //
                        // item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                        // item.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(wvzInhalt);
                        //
                        // }
                        // }
                        //
                        // EStructuralFeature feature =
                        // AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                        //
                        // for (DoMitVerbLinieEditPart epart : list1)
                        // {
                        // DoModelMitVerbLinie model = epart.getModel();
                        // SetCommand cmd = new SetCommand(model, feature, kapsel);
                        // if (kapsel.isInaktiv()){
                        // cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                        // }
                        // getCommandStack().execute(cmd);
                        // }
                      }
                      else
                      {
                        System.out.println("kapsel ist null!!!");
                      }
                    }
                    else
                    {
                      System.out.println(doModel2 + " " + figure2);
                    }
                  }
                }
                
              }
            }
          }
        }
        else
        {
          AnzeigeInhaltFreiTextDialog anzeigeInhaltAuswahlDialog = new AnzeigeInhaltFreiTextDialog(PlatformUI
              .getWorkbench().getDisplay().getActiveShell());
          
          if (doModel != null && figure != null)
          {
            SystemObject so = doModel.getSystemObject();
            
            System.out.println("ActionParameterAnzeigeABC.proceed(): " + so);
            
            Object value = doModel.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);
            VisuAnzeigeKapsel kapsel = null;
            if (value instanceof VisuAnzeigeKapsel)
            {
              kapsel = (VisuAnzeigeKapsel)value;
            }
//            else if (value instanceof ResultData)
//            {
//              ResultData result = (ResultData)value;
//              kapsel = DaVHelper.holeDynamischerWechselTextAusDaVData(result);
//              System.out.println("Kapsel: " + kapsel.toString());
//            }
            else
            {
              kapsel = new VisuAnzeigeKapsel(new String[] {}, fonts[0].getPid(),
                  EnumDynamischerWechseltextTextAusrichtung.Linksbndig.toString(), 4);
            }
            VisuAnzeigeKapsel kapselNeu = anzeigeInhaltAuswahlDialog.oeffne(kapsel, doModel.getSystemObject());
            if (kapselNeu != null)
            {
              for (DoMitVerbLinieEditPart domep2 : list1)
              {
                AnzeigeModelTypAbstract doModel2 = (AnzeigeModelTypAbstract)domep2.getModel();
                AnzeigeElementFigure figure2 = (AnzeigeElementFigure)domep2.getFigure();
                System.out.println("kapsel: " + kapselNeu);
                SystemObject so2 = doModel2.getSystemObject();
//                Array array = data.getItem("AnzeigenDatenstze").asArray();
                Array array = data.asArray();
                Data dynamischerWechseltext = null;
                // Index nur zum Loeschen gebraucht, also bei AnzeigeInhaltTyp.BELIEBIG_BI
                int loc = 0;
                List<Data> list = new ArrayList<Data>();
                for (int i = 0; i < array.getLength(); i++)
                {
                  Data item = array.getItem(i);
                  SystemObject anzeige = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                  
                  if (anzeige != null && so != null && anzeige.equals(so))
                  {
                    if (_connection != null)
                    {
                      if (!kapselNeu.isInaktiv())
                      {
                        dynamischerWechseltext = item.getItem("Schaltbild").getItem("DynamischerWechseltext");
                        uebernehmeDaten(kapselNeu, dynamischerWechseltext);
                        item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);
                      }
                      else
                      {
                        loc = i;
                      }
                      // Das ganze Feld muss durchlaufen werden, um Liste vorzubelegen
                      // break;
                    }
                  }
                  // Liste wird vorbelegt
                  list.add(item);
                }
                
                if (dynamischerWechseltext == null)
                {
                  if (kapselNeu.isInaktiv())
                  {
                    // Loeschen des Feldelementes aus der Datenstruktur, welches auf beliebig gesetzt wird
                    if (!list.isEmpty() && list.size() - 1 >= loc)
                    {
                      list.remove(loc);
                    }
                    array.setLength(list.size());
                    for (int i = 0; i < array.getLength(); i++)
                    {
                      Data ziel = array.getItem(i);
                      Data quelle = list.get(i);
                      // ziel = quelle;
                      DataHelper.kopiereRekursiv(quelle, ziel);
                    }
                  }
                  else
                  {
                    // Array um ein Feldelement erweitern
                    array.setLength(array.getLength() + 1);
                    // Index dieses neuen Feldelements ist um eins kleiner
                    Data item = array.getItem(array.getLength() - 1);
                    item.getItem("Prioritt").asTextValue()
                    .setText(doModel2.getPrioritaet());
                    item.getItem("Schaltbild").getItem("Blinken").asTextValue()
                    .setText(doModel2.getBlinken() > 0 ? "Ja" : "Nein");
                    item.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText(doModel2.getBlinktakt());
                    item.getItem("notwendig").asTextValue()
                    .setText(doModel2.getNotwendig() > 0 ? "Ja" : "Nein");
//                    item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue()
//                    .setText(doModel2.getAbsolutNotwendig() > 0 ? "Ja" : "Nein");
//                    item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asUnscaledValue()
//                    .set(doModel2.getVerschiebungVorgaenger());
//                    item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asUnscaledValue()
//                    .set(doModel2.getVerschiebungNachfolger());
                    
                    // item.getItem("AnzeigenDatensatz").getItem("notwendig").asTextValue().setText("Ja");
                    // item.getItem("AnzeigenDatensatz").getItem("absolutNotwendig").asTextValue().setText("Nein");
                    // item.getItem("AnzeigenDatensatz").getItem("Prioritt").asTextValue().setText("undefiniert");
                    // item.getItem("AnzeigenDatensatz").getItem("verschiebbarVorgnger").asTextValue().setText("0");
                    // item.getItem("AnzeigenDatensatz").getItem("verschiebbarNachfolger").asTextValue().setText("0");
                    // item.getItem("WVZInhalt").getItem("Blinken").asTextValue().setText("Nein");
                    // item.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText("Undefiniert");
                    item.getItem("Schaltbild").getItem("DynamischerWechseltext").getItem("TextAusrichtung")
                    .asTextValue().setText("Linksbndig");
                    
                    // SystemObjekt WVZInhalt aus Bildinhalt bestimmen
                    // // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
                    // wvzInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, so, inhalt.getType());
                    dynamischerWechseltext = item.getItem("Schaltbild").getItem("DynamischerWechseltext");
                    uebernehmeDaten(kapselNeu, dynamischerWechseltext);
                    
                    item.getItem("Anzeige").asReferenceValue().setSystemObject(doModel2.getSystemObject());
                    item.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(null);
                    
                  }
                }
                
                EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                
                for (DoMitVerbLinieEditPart epart : list1)
                {
                  DoModelMitVerbLinie model = epart.getModel();
                  SetCommand cmd = new SetCommand(model, feature, kapselNeu);
                  if (kapsel.isInaktiv())
                  {
                    cmd = new SetCommand(model, feature, AnzeigeInhaltTyp.BELIEBIG_BI);
                  }
                  getCommandStack().execute(cmd);
                }
              }
            }
          }
        }
      }
    }
    else
    {
      System.out.println("Viewer ist null");
    }
  }

  private void uebernehmeDaten(VisuAnzeigeKapsel kapselNeu, Data dynamischerWechseltext)
  {
    // System.out.println("ActionParameterAnzeigeABC.uebernehmeDaten()");

    if (dynamischerWechseltext != null)
    {

      // System.out.println("kapsel: " + kapselNeu);

      String s1 = kapselNeu.getSchriftart();
      if (s1 != null)
      {
        SystemObject font = _connection.getDataModel().getObject(s1);
        if (font != null)
        {
          dynamischerWechseltext.getItem("Font").asReferenceValue().setSystemObject(font);
        }
      }
      String s2 = kapselNeu.getAusrichtung();
      if (s2 != null)
      {
        dynamischerWechseltext.getItem("TextAusrichtung").asTextValue().setText(s2);
      }

      List<SystemObject> listZeichen = new ArrayList<SystemObject>();
      String[] pidsZeichen = kapselNeu.getPidsZeichen();
      if (pidsZeichen != null)
      {
        for (String pid : pidsZeichen)
        {
          SystemObject zeichen = _connection.getDataModel().getObject(pid);
          // System.out.print(" " + zeichen);
          if (zeichen != null)
            listZeichen.add(zeichen);
        }

        ReferenceArray refs = dynamischerWechseltext.getItem("Textdefinition").asReferenceArray();
        refs.setLength(listZeichen.size());
        for (int k = 0; k < refs.getLength(); k++)
        {
          ReferenceValue ref = refs.getReferenceValue(k);
          ref.setSystemObject(listZeichen.get(k));
        }
      }
    }
  }

  @Override
  public void selectionChanged(IAction action, ISelection selection)
  {
  }

  /**
   * Gibt den {@link GraphicalViewer} des Workbench Parts zurck.
   * 
   * @return der Viewer oder <code>null</code>, wenn es keinen gibt.
   */
  protected GraphicalViewer getViewer()
  {
    IWorkbench wb = PlatformUI.getWorkbench();
    IWorkbenchWindow window = wb.getActiveWorkbenchWindow();
    IWorkbenchPage page = window.getActivePage();
    IWorkbenchPart part = page.getActivePart();
    return (GraphicalViewer)part.getAdapter(GraphicalViewer.class);
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {
    CommandStack stack = getViewer().getEditDomain().getCommandStack();
    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  @Override
  public void run(IAction action)
  {
    // TODO Auto-generated method stub
  }

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
  }

  @Override
  public void runWithEvent(IAction action, Event event)
  {
    Object object = event.data;
    if (object instanceof Data)
    {      
      Data data = (Data)object;
       AttributeType at = data.getAttributeType();
      if (at != null && at.getPid().equals("atl.sswSbaUeSchaltbild"))
        proceed2(data);
      else if ("atg.sswSbaAutarkprogramm".equals(data.getName()))
        proceed1(data);
      else
        proceed(data);
    }
  }

  @Override
  public void init(IWorkbenchWindow window)
  {
    // TODO Auto-generated method stub

  }

  @Override
  public void init(IAction action)
  {
    // TODO Auto-generated method stub
  }
}
