/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.action;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts.DoMitVerbLinieEditPart;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Action zur Anzeige des Tooltip fr Ufd-Messstellen
 * 
 * @author Pittner
 * 
 */
public class ActionUmfeldDatenToolTip implements IWorkbenchWindowActionDelegate
{
  private IWorkbenchPart _workbenchPart;

  private final static SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  @Override
  public void dispose()
  {
  }

  @Override
  public void init(IWorkbenchWindow window)
  {
    _workbenchPart = window.getActivePage().getActivePart();
  }

  @Override
  public void run(IAction action)
  {    
    GraphicalViewer viewer = getViewer();
    if (viewer != null)
    {
      Point pt = Display.getCurrent().getCursorLocation();
      Point pt2 = viewer.getControl().toControl(pt);
      org.eclipse.draw2d.geometry.Point point = new org.eclipse.draw2d.geometry.Point(pt2);

      Object obj = viewer.findObjectAt(point);

      if (obj instanceof DoMitVerbLinieEditPart)
      {
        DoMitVerbLinieEditPart domep = (DoMitVerbLinieEditPart)obj;
        DoModel doModel = domep.getModel();

        SystemObject so = doModel.getSystemObject();

        ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
        if (connection != null && so != null)
        {
          AttributeGroup atg = null;

          String item = null;         
          Aspect asp = so.getDataModel().getAspect("asp.messWertErsetzung");

          if (so.isOfType("typ.ufdsFahrBahnOberFlchenZustand"))
          {
            atg = so.getDataModel().getAttributeGroup("atg.ufdsFahrBahnOberFlchenZustand");

            item = "FahrBahnOberFlchenZustand";
          }
          else if (so.isOfType("typ.ufdsHelligkeit"))
          {
            atg = so.getDataModel().getAttributeGroup("atg.ufdsHelligkeit");

            item = "Helligkeit";
          }
          else if (so.isOfType("typ.ufdsSichtWeite"))
          {
            atg = so.getDataModel().getAttributeGroup("atg.ufdsSichtWeite");
            item = "SichtWeite";
          }
          else if (so.isOfType("typ.sswSbaUmfeldDatenMessStelle"))
          {
            atg = so.getDataModel().getAttributeGroup("atg.sswSbaUfdmsNsseStufe");
            asp = so.getDataModel().getAspect("asp.klassifizierung");
            item = "NsseStufe";
          }
          else if (so.isOfType("typ.umfeldDatenMessStelle"))
          {
            EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE;
            final SetCommand cmd = new SetCommand(doModel, feature, "");
            getCommandStack().execute(cmd);
            return;
          }
          else
          {
            
          }

          if (atg != null && asp != null)
          {
            DataDescription dd = new DataDescription(atg, asp);
            ResultData rd = connection.getData(so, dd, 1000);

            Date d = new Date();
            d.setTime(rd.getDataTime());
            Data data = rd.getData();
            String wert = "keine Daten";
            if (rd.getDataState().equals(DataState.NO_SOURCE))
            {
              wert = "keineQuelle";
            }
            if (item != null)
            {
              String s = "";
              if (item != "NsseStufe")
              {

                if (data != null)
                  wert = data.getItem(item).getItem("Wert").asTextValue().getText();

                s = "UmfelddatenSensor: " + so.getNameOrPidOrId();
                s += "\n" + item + " " + wert;
              }
              else
              {

                if (data != null)
                  wert = data.getItem(item).asTextValue().getText();

                s = "UmfelddatenMessStelle " + so.getNameOrPidOrId();
                s += "\n" + item + " " + data.getItem(item).asTextValue().getText();
              }
              s += " (" + _sdf.format(d) + " )";
              EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE;
              final SetCommand cmd = new SetCommand(doModel, feature, s);
              getCommandStack().execute(cmd);
            }
          }
        }
      }
    }
  }

  @Override
  public void selectionChanged(IAction action, ISelection selection)
  {
  }

  /**
   * @return
   */
  protected IWorkbenchPart getWorkbenchPart()
  {
    return _workbenchPart;
  }

  /**
   * Gibt den {@link GraphicalViewer} des Workbench Parts zurck.
   * 
   * @return der Viewer oder <code>null</code>, wenn es keinen gibt.
   */
  protected GraphicalViewer getViewer()
  {
    return (GraphicalViewer)getWorkbenchPart().getAdapter(GraphicalViewer.class);
  }

  /**
   * 
   * @param workbenchPart
   */
  protected void setWorkbenchPart(IWorkbenchPart workbenchPart)
  {
    _workbenchPart = workbenchPart;
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {
    CommandStack stack = getViewer().getEditDomain().getCommandStack();
    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

}
