/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeFonts;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link AnzeigeModelTypAbstract} verwendet.
 * 
 * @author Pittner
 * 
 */
public class AnzeigeTypC2EditPart extends AnzeigeTypAEditPart
{

  private final Dimension DIM = new Dimension(68, 17);

//  private VisuAnzeigeKapsel _kapsel;

  @Override
  protected AnzeigeElementFigure createFigure()
  {
    ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
    DataModel dataModel = dav.getDataModel();

    AnzeigeModelTypAbstract model = getCastedModel();

    setzteInhalteModell(model, dataModel);

    boolean isFrNord = getCastedModel().getFahrtrichtung() != 1 ? true : false;
    AnzeigeElementFigure aef = new AnzeigeElementFigure("AnzeigeC2", AnzeigeElementTyp.WZG_C2,
        AnzeigeInhaltTyp.DUNKEL_BI, true, isFrNord);
    aef.setOpaque(true);
    aef.setSize(DIM.width, DIM.height);
    aef.setBorder(new LineBorder(ColorConstants.orange, 2));

    addToolTipListener(aef);

    return aef;
  }

  @Override
  public void notifyChanged(Notification notification)
  {

    super.notifyChanged(notification);

    if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE.equals(notification.getFeature()))
    {

      Object newValue = notification.getNewValue();
      Object oldValue = notification.getOldValue();
      
      
      if (oldValue instanceof AnzeigeInhaltTyp)
      {
        AnzeigeInhaltTyp inhaltAlt = (AnzeigeInhaltTyp)oldValue;
        if (AnzeigeElementFigure.pruefeAufStatischenInhalt(inhaltAlt))
          ((AnzeigeElementFigure)getFigure()).setImageDynamisch(false);
      }
      

      int kontext = 0;
      if (getParent() != null)
      {
        Object parentModel = getParent().getModel();
        {
          if (parentModel instanceof AnzeigeQuerschnittModelTypAbstract)
          {
            AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parentModel;
            kontext = aq.getKontext();
          }
        }
      }

      if (newValue instanceof VisuAnzeigeKapsel)
      {
        VisuAnzeigeKapsel _kapsel = (VisuAnzeigeKapsel)newValue;        
        kapselUpdateAnzeigeModelAbstract(_kapsel);
      }
      else if (newValue instanceof AnzeigeInhaltTyp)
      {
//        AnzeigeInhaltTyp inhaltNeu = (AnzeigeInhaltTyp)newValue;        
//        inhaltUpdateAnzeigeModelTypAbstract(AnzeigeElementTyp.WZG_C2, inhaltNeu);
        updateAnzeigeInhalt(kontext, newValue, oldValue, AnzeigeElementTyp.WZG_C2);        
      }
      else if (newValue instanceof ResultData)
      {
        ResultData result = (ResultData)newValue;

        if (result != null && result.hasData())
        {
          Data data = result.getData();
          SystemObject so = data.getItem("Eigenschaft").getItem("AnzeigeInhalt").asReferenceValue().getSystemObject();
          ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
          AttributeGroup atg2 = dav.getDataModel().getAttributeGroup("atg.wvzInhalt");

          // AnzeigeInhalt hat eine hoehere Prio, d.h. wenn ein AnzeigeInhalt vorhanden ist,
          // wird er auch angezeigt
          if (so != null)
          {
            Data data2 = so.getConfigurationData(atg2);
            final String inhalt = data2.getItem("Bildinhalt").asTextValue().getValueText();
            AnzeigeInhaltTyp inhaltNeu = AnzeigeInhaltTyp.setBildInhalt(inhalt, result.getDataTime());            
            inhaltUpdateAnzeigeModelTypAbstract(AnzeigeElementTyp.WZG_C2, inhaltNeu);
          }
          else
          {
            VisuAnzeigeKapsel _kapsel = DaVHelper.holeDynamischerWechselTextAusDaVData((ResultData)newValue);
            kapselUpdateAnzeigeModelAbstract(_kapsel);
          }
        }
        else
        {
          inhaltUpdateAnzeigeModelTypAbstract(AnzeigeElementTyp.WZG_C2, AnzeigeInhaltTyp.GESTOERT_BI);
        }
      }
    }
  }

  private void kapselUpdateAnzeigeModelAbstract(final VisuAnzeigeKapsel kapsel)
  {
    Image image = null;
    final AnzeigeElementFigure aef = (AnzeigeElementFigure)getFigure();
    
    boolean dynamisch = false;
    if (kapsel != null)
    {
      VisuAnzeigeFonts fonts = VisuAnzeigeFonts.getInstanz();
      String[] pidsZeichen = kapsel.getPidsZeichen();
      String ausrichtung = kapsel.getAusrichtung();
      String schriftart = kapsel.getSchriftart();
      int abstand = kapsel.getAbstand();
      
      image = fonts.erstelleBitmapSWT(pidsZeichen, ausrichtung, schriftart, DIM.width - 4, DIM.height - 3, abstand);
      dynamisch = true;
      
      if (kapsel.isInaktiv())
      {
//        System.out.println("----> Kapsel ist inaktiv!!!" + getModel().getSystemObject().getName());
        
        if (image != null && !image.isDisposed()){
          
        }else{
          // Rckfallebene wg. Meldung von inaktiven C-Typen 
          DoModelMitVerbLinie doModel = getModel();        
          Object value = doModel.eGet(AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE);                
          if (value instanceof AnzeigeInhaltTyp)
          {
            AnzeigeInhaltTyp inhaltAlt = (AnzeigeInhaltTyp)value;
            inhaltUpdateAnzeigeModelTypAbstract(AnzeigeElementTyp.WZG_C2, inhaltAlt);
//          image = ImageFactory.getImage("icons/c2_inaktiv.png");
          }
        }        
      }
    }
    else
    {      
      image = ImageFactory.getImage("icons/c2_stoer.png");
    }

    if (image != null && !image.isDisposed())
    {
      
      final Image fimage = image;
      if (aef.isImageDynamisch()){
        Image imageOld = aef.getImage();
        if (imageOld != null && kapsel != null && !kapsel.isInaktiv())
        {
          imageOld.dispose();
          imageOld = null;
        }        
      }
      
      final boolean fdynamisch = dynamisch;
      
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
        @Override
        public void run()
        {
          if (fimage != null && !fimage.isDisposed())
          {
            aef.setImage(fimage);
            aef.setImageDynamisch(fdynamisch);            
          }else{
            System.out.println("Image ist schon disposed31" + fimage);
          }
        }
      });
    }else{
      System.out.println("Image ist schon disposed3" + image);
    }
  }
  
//  @Override
//  public Image getIcon()
//  {
//    // TODO Auto-generated method stub
//      return ImageFactory.getImage("icons/legende/anzeige_typ_c2.png");
//  }
}
