/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.properties;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;

import com.bitctrl.lib.eclipse.emf.gef.commands.RemoveCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.doeditor.model.AttributeTextDecorator;
import de.bsvrz.buv.plugin.doeditor.model.DoeditorPackage;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;


/**
 * Darstellung der AttributTextDekorierer eines Elements eines DOT. DIe Darstellung erfolgt tabellarisch als Sektion in
 * einer TabbedPropertySheetPage.
 * 
 * @author Pittner
 * @param <T>
 */
public class ListBildInhalt1Section extends AbstractDobjSection<AnzeigeModelTypAbstract>
{

  /**
   * Aktion zum Anlegen eines neuen Dekorierers. Die Aktion ffnet den Dialog zum Eingeben der gewnschten Parameter des
   * Dekorierers und legt einen neuen Dekorierer in der Liste der Dekorierer des bearbeitenden Elements an.
   */
  private class AddDekoriererAction extends Action
  {

    /** Konstruktor. */
    public AddDekoriererAction()
    {
      super("Dekorierer hinzufgen");
    }

    @Override
    public void run()
    {
      ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
      DataModel dataModel = dav.getDataModel();

      AttributeGroup atg = dataModel.getAttributeGroup("atg.wvzInhalt");

      SystemObjectType type = dataModel.getType("typ.wvzInhalt");
      List<SystemObject> elements = type.getElements();

      ArrayList<String> list = new ArrayList<String>();

      for (SystemObject so : elements)
      {
        Data data = so.getConfigurationData(atg);
        String s = data.getItem("Bildinhalt").asTextValue().getValueText();
        list.add(s);
      }

      decorators.setInput(list.toArray());
      EList<String> elist = new BasicEList<String>(list);
      getElement().setInhalte1(elist);      
    }
  }

  /**
   * Aktion zum Entfernen von Dekorierern eines Elementes eines Darstellungsobjekttyps.
   */
  private class RemoveDekoriererAction extends Action
  {

    /** Konstruktor. */
    public RemoveDekoriererAction()
    {
      super("Dekorierer entfernen");
    }

    @Override
    public boolean isEnabled()
    {
      return super.isEnabled() && (!decorators.getSelection().isEmpty());
    }

    @Override
    public void run()
    {
      final Object[] zuLoeschen = ((IStructuredSelection)decorators.getSelection()).toArray();
      if (zuLoeschen.length > 0)
      {
        if (MessageDialog.openConfirm(getPart().getSite().getShell(), "Dekorierer lschen",
            "Sollen die ausgewhlten Dekorierer wirklich gelscht werden?"))
        {
          for (final Object candidate : zuLoeschen)
          {
            if (candidate instanceof AttributeTextDecorator)
            {
              final RemoveCommand removeCmd = new RemoveCommand(getElement(),
                  DoeditorPackage.Literals.VISIBLE_FORM__DECORATORS, candidate);
              getCommandStack().execute(removeCmd);
            }
          }
        }
      }
    }
  }

  /**
   * Provider fr die Textdarstellung der Dekorierer in der Dekorierertabelle.
   */
  private class AttributeTextDecoratorLabelProvider extends LabelProvider implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(final Object element, final int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(final Object element, final int columnIndex)
    {
      String result = null;
      if (element instanceof String)
      {
        final String s = (String)element;
        switch (columnIndex)
        {
        // Attributgruppe
          case 0:
            if (s != null)
            {
              result = s;
            }
            break;          
          default:
            result = "???";
            break;
        }

      }
      if (result == null)
      {
        result = "";
      }
      return result;
    }
  }

  /**
   * die Tabelle zur Darstellung der mit dem aktuellen Element verbundenen Dekorierer.
   */
  private TableViewer decorators;

  /** Schaltflche zum Anlegen eines neuen Dekorierers. */
  private Button addButton;


  /** Schaltflche zum Entfernen der aktuell ausgewhlten Dekorierer. */
  private Button removeButton;

  @Override
  protected void aktualisiereDaten()
  {
    AnzeigeModelTypAbstract model = getElement();

    SystemObject so = model.getSystemObject();

    if (so != null && so.isOfType("typ.anzeige"))
    {

      ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
      DataModel dataModel = dav.getDataModel();

      AttributeGroup atg = dataModel.getAttributeGroup("atg.anzeige");
      AttributeGroup atg2 = dataModel.getAttributeGroup("atg.wvzInhalt");
      Data cd = so.getConfigurationData(atg);

      ConfigurationObject co = (ConfigurationObject)cd.getItem("AnzeigeTyp").asReferenceValue().getSystemObject();
      NonMutableSet set = co.getNonMutableSet("WvzInhalt");
      List<SystemObject> inhalte = set.getElements();

      List<String> list = new ArrayList<String>();

      for (SystemObject inhalt : inhalte)
      {
        Data cd2 = inhalt.getConfigurationData(atg2);
        String bi = cd2.getItem("Bildinhalt").asTextValue().getValueText();
        list.add(bi);
      }

      decorators.setInput(list.toArray());
      // EList<String> elist = new BasicEList<String>(list);
      // model.setInhalte1(elist);
    }
  }

  @Override
  public void createUI(final Composite panel)
  {
    final Table table = getWidgetFactory().createTable(panel, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI);
    FormData data = new FormData();
    data.left = new FormAttachment(0);
    data.right = new FormAttachment(90);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    data.bottom = new FormAttachment(100, -ITabbedPropertyConstants.VSPACE);
    data.height = 150;
    table.setLayoutData(data);
    table.setHeaderVisible(true);
    table.setLinesVisible(true);

    final TableLayout tabLayout = new TableLayout();
    table.setLayout(tabLayout);

    TableColumn col = new TableColumn(table, SWT.NONE);
    col.setText("Bildinhalt");
    tabLayout.addColumnData(new ColumnWeightData(2));

    decorators = new TableViewer(table);
    decorators.setContentProvider(new ArrayContentProvider());
    decorators.setLabelProvider(new AttributeTextDecoratorLabelProvider());

    createDecoratorsContextMenue();
    updateButtons();
  }

  /**
   * aktualisiert die Darstellung der Befehlsschaltflchen entsprechend der aktuellen Auswahl innerhalb der
   * Dekoriererliste.
   */
  private void updateButtons()
  {
    // addButton.setEnabled(((Action)addButton.getData()).isEnabled());
    // removeButton.setEnabled(((Action)removeButton.getData()).isEnabled());
    // editButton.setEnabled(((Action)editButton.getData()).isEnabled());
  }

  /** erzeugt das Kontextmen der Dekoriererliste. */
  private void createDecoratorsContextMenue()
  {
    final MenuManager menuMgr = new MenuManager();
    menuMgr.add(new AddDekoriererAction());
    menuMgr.add(new RemoveDekoriererAction());
    decorators.getControl().setMenu(menuMgr.createContextMenu(decorators.getControl()));
  }

  @Override
  protected void installListeners()
  { 
    decorators.addSelectionChangedListener(new ISelectionChangedListener() {
      @Override
      public void selectionChanged(final SelectionChangedEvent event)
      {
        updateButtons();
      }
    });
  }
}
