/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.editor;

import org.eclipse.gef.EditDomain;
import org.eclipse.ui.IViewPart;

/**
 * A default implementation of {@link EditDomain}. An {@link IViewPart} is required in the constructor, but it can be
 * <code>null</code>.
 * <P>
 * A {@link org.eclipse.gef.tools.SelectionTool} will be the active Tool until:
 * <UL>
 * <LI>A {@link org.eclipse.gef.palette.PaletteRoot} is provided which contains a default entry which is a
 * {@link org.eclipse.gef.palette.ToolEntry}. In which case that entry's tool is made the active Tool.
 * </UL>
 * <P>
 * DefaultEditDomain can be configured with a {@link org.eclipse.gef.ui.palette.PaletteViewer}. When provided, the
 * DefaultEditDomain will listen for PaletteEvents, and will switch the active Tool automatically in response.
 */
public class DefaultEditDomainViewPart extends EditDomain
{

  private IViewPart viewPart;

  /**
   * Constructs a DefaultEditDomain with the specified IEditorPart
   * 
   * @param viewPart
   *          <code>null</code> or an IEditorPart
   */
  public DefaultEditDomainViewPart(IViewPart viewPart)
  {
    setViewPart(viewPart);
  }

  /**
   * @return the IEditorPart for this EditDomain
   */
  public IViewPart getEditorPart()
  {
    return viewPart;
  }

  /**
   * Sets the IEditorPart for this EditDomain.
   * 
   * @param editorPart
   *          the editor
   */
  protected void setViewPart(IViewPart viewPart)
  {
    this.viewPart = viewPart;
  }

}
