/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.ImageFigure;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.omg.CORBA._PolicyStub;

import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeFonts;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.DynamischerWechselText.EnumDynamischerWechseltextTextAusrichtung;
import de.bsvrz.buv.rw.rw.interfaces.diagramme.IDiagramm.BeschriftungsPosition;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Figure, welche eine Anzeige eines beliebigen Typs reprsentiert.
 * 
 * @author Pittner
 * 
 */
public class AnzeigeElementFigure extends ImageFigure implements ScalableFigure
{
  private String _pid;

  private AnzeigeInhaltTyp _inhalt;

  private AnzeigeElementTyp _typ;

  // private boolean _aktiv = true;

  private double scale = 1.0;

  private boolean _isFrNord;

  private static List<AnzeigeInhaltTyp> listStatischeInhalte = new ArrayList<AnzeigeInhaltTyp>() {
    {
      add(AnzeigeInhaltTyp.DUNKEL);
      add(AnzeigeInhaltTyp.DUNKEL_BI);
      add(AnzeigeInhaltTyp.DUNKEL_HP_BI);
      add(AnzeigeInhaltTyp.DUNKEL_TEXT);
      add(AnzeigeInhaltTyp.DUNKEL_BI_TEXT);
      add(AnzeigeInhaltTyp.DUNKEL_HP_BI_TEXT);
      add(AnzeigeInhaltTyp.DONT_CARE_BI);
      add(AnzeigeInhaltTyp.BEIBEHALTEN_BI);
      add(AnzeigeInhaltTyp.GESTOERT);
      add(AnzeigeInhaltTyp.GESTOERT_TEXT);
      add(AnzeigeInhaltTyp.GESTOERT_BI);
      add(AnzeigeInhaltTyp.GESTOERT_BI_TEXT);
      add(AnzeigeInhaltTyp.UNDEFINIERT);
      add(AnzeigeInhaltTyp.UNDEFINIERT_BI);
      add(AnzeigeInhaltTyp.UNDEFINIERT_TEXT);
      add(AnzeigeInhaltTyp.UNDEFINIERT_BI_TEXT);
      add(AnzeigeInhaltTyp.TLS_NEBEL);
      add(AnzeigeInhaltTyp.TLS_NEBEL_BI);
      add(AnzeigeInhaltTyp.TLS_NAESSE);
      add(AnzeigeInhaltTyp.TLS_NAESSE_BI);
      add(AnzeigeInhaltTyp.TLS_UNFALL);
      add(AnzeigeInhaltTyp.TLS_UNFALL_BI);
      add(AnzeigeInhaltTyp.TLS_STAU);
      add(AnzeigeInhaltTyp.TLS_STAU_BI);
      add(AnzeigeInhaltTyp.TLS_SMOG);
      add(AnzeigeInhaltTyp.TLS_SMOG_BI);
      add(AnzeigeInhaltTyp.TLS_STAU_GEFAHR);
      add(AnzeigeInhaltTyp.TLS_STAU_GEFAHR_BI);
      add(AnzeigeInhaltTyp.TLS_OELSPUR);
      add(AnzeigeInhaltTyp.TLS_OELSPUR_BI);
      add(AnzeigeInhaltTyp.TLS_SICHT);
      add(AnzeigeInhaltTyp.TLS_SICHT_BI);
      add(AnzeigeInhaltTyp.TLS_KONTROLLE);
      add(AnzeigeInhaltTyp.TLS_KONTROLLE_BI);
      add(AnzeigeInhaltTyp.TLS_TIERE);
      add(AnzeigeInhaltTyp.TLS_TIERE_BI);
      add(AnzeigeInhaltTyp.TLS_AS_GLAETTE);
      add(AnzeigeInhaltTyp.TLS_AS_GLAETTE_BI);
      add(AnzeigeInhaltTyp.TLS_AS_OELSPUR);
      add(AnzeigeInhaltTyp.TLS_AS_OELSPUR_BI);
      add(AnzeigeInhaltTyp.TLS_7KOMMA5TONNEN);
      add(AnzeigeInhaltTyp.TLS_7KOMMA5TONNEN_BI);
      add(AnzeigeInhaltTyp.TLS_500M);
      add(AnzeigeInhaltTyp.TLS_500M_BI);
      add(AnzeigeInhaltTyp.TLS_1000M);
      add(AnzeigeInhaltTyp.TLS_1000M_BI);
      add(AnzeigeInhaltTyp.TLS_1500M);
      add(AnzeigeInhaltTyp.TLS_1500M_BI);
      add(AnzeigeInhaltTyp.TLS_2000M);
      add(AnzeigeInhaltTyp.TLS_2000M_BI);
      add(AnzeigeInhaltTyp.TLS_2500M);
      add(AnzeigeInhaltTyp.TLS_2500M_BI);
      add(AnzeigeInhaltTyp.TLS_3000M);
      add(AnzeigeInhaltTyp.TLS_3000M_BI);
      add(AnzeigeInhaltTyp.TLS_1_KILOMETER);
      add(AnzeigeInhaltTyp.TLS_1_KILOMETER_BI);
      add(AnzeigeInhaltTyp.TLS_1_KOMMA_5_KILOMETER);
      add(AnzeigeInhaltTyp.TLS_1_KOMMA_5_KILOMETER_BI);
      add(AnzeigeInhaltTyp.TLS_2_KILOMETER);
      add(AnzeigeInhaltTyp.TLS_2_KILOMETER_BI);
      add(AnzeigeInhaltTyp.TLS_2_KOMMA_5_KILOMETER);
      add(AnzeigeInhaltTyp.TLS_2_KOMMA_5_KILOMETER_BI);
      add(AnzeigeInhaltTyp.TLS_3_KILOMETER);
      add(AnzeigeInhaltTyp.TLS_3_KILOMETER_BI);
      add(AnzeigeInhaltTyp.INAKTIV);
      add(AnzeigeInhaltTyp.INAKTIV_BI);
      add(AnzeigeInhaltTyp.BELIEBIG);
      add(AnzeigeInhaltTyp.BELIEBIG_BI);
      add(AnzeigeInhaltTyp.FREITEXT);
      add(AnzeigeInhaltTyp.FREITEXT_BI);
    }
  };

  /**
   * Flagge, die anzeigt ob das Image dynamisch erstellt wurde. Nur diese werden beim erneuten erstellen disposed.
   */
  private boolean _isImageDynamisch = false;

  private static Dimension DIM = new Dimension(29, 29);

  public static Color columnColor = new Color(null, 255, 255, 255);

  public AnzeigeElementFigure(String pid, AnzeigeElementTyp typ, AnzeigeInhaltTyp inhalt, boolean aktiv,
      boolean isFrNord)
  {
    _pid = pid;
    _inhalt = inhalt;
    _typ = typ;
    // _aktiv = aktiv;
    _isFrNord = isFrNord;

    createImage(typ, inhalt, isFrNord);

    if (!typ.equals(AnzeigeElementTyp.PROGRAMM) && !typ.equals(AnzeigeElementTyp.BLINKER_PROGRAMM)
        && !typ.equals(AnzeigeElementTyp.STRECKEN_STATION))
      setBorder(new LineBorder(ColorConstants.gray, 2));
    setOpaque(true);

  }

  private void createImage(AnzeigeElementTyp typ, AnzeigeInhaltTyp inhalt, boolean isFrNord)
  {
    Image image = null;

    if (!typ.equals(AnzeigeElementTyp.STRECKEN_STATION))
      setBorder(new LineBorder(ColorConstants.orange, 2));

    if (!typ.equals(AnzeigeElementTyp.FREITEXT) && !typ.equals(AnzeigeElementTyp.FREITEXT2))
    {
      String imagepath = null;

      // if (_aktiv)
      // imagepath = createImagePath(typ, inhalt, isFrNord);
      // else
      // {
      // imagepath = createImagePath(typ, AnzeigeInhaltTyp.INAKTIV, isFrNord);
      // }

      imagepath = createImagePath(typ, inhalt, isFrNord);
      if (imagepath != null)
      {
        image = ImageFactory.getImage(imagepath);
        if (image != null && !image.isDisposed())
        {
          setImageDynamisch(false);
          setImage(image);
        }
        else
        {
          System.out.println("imageIsDisposed1: " + imagepath);
        }
      }
    }
    else
    {
      // String imagepath = createImagePath(typ, inhalt, isFrNord);
      // if (imagepath != null)
      // {
      // image = ImageFactory.getImage(imagepath);
      // if (image != null && !image.isDisposed()){
      // setImageDynamisch(false);
      // setImage(image);
      // }else{
      // System.out.println("imageIsDisposed1: " + imagepath);
      // }
      // }

      Dimension DIM = new Dimension(220, 24);
      if (typ.equals(AnzeigeElementTyp.FREITEXT2))
        DIM = new Dimension(144, 23);

      String[] pidsZeichen = new String[] {};
      
      SystemObject font = getFontUeberSchriftBeschreibung("Mittelschrift");
      String pidFont = "";
      if (font != null)
        pidFont = font.getPid();
      
      VisuAnzeigeKapsel kapsel = new VisuAnzeigeKapsel(pidsZeichen, pidFont,
          EnumDynamischerWechseltextTextAusrichtung.Linksbndig.toString(), 4);

      image = VisuAnzeigeFonts.getInstanz().erstelleBitmapSWT(pidsZeichen, kapsel.getAusrichtung(),
          kapsel.getSchriftart(), DIM.width, DIM.height, kapsel.getAbstand());

      if (image != null && !image.isDisposed())
      {
        setImageDynamisch(true);
        setImage(image);
      }
      else
      {
        System.out.println("imageIsDisposed2: " + kapsel.getPidsZeichen().length);
      }
    }
  }
  
  public SystemObject getFontUeberSchriftBeschreibung(String beschreibung)
  {
    Collection<BuvWechselTextKombi> values = VisuAnzeigeFonts.getInstanz().getMapBuVWechselTextKombi().values();
    for (BuvWechselTextKombi buvWechselTextKombi : values)
    {
       String text = buvWechselTextKombi.getBeschreibung();
       if (text.equals(beschreibung))
         return buvWechselTextKombi.getFont();       
    }       
    return null;
  }

  public AnzeigeElementFigure(String pid, AnzeigeElementTyp typ, AnzeigeInhaltTyp inhalt, boolean aktiv, Point pos,
      boolean isFrNord)
  {
    _pid = pid;
    _inhalt = inhalt;
    _typ = typ;
    // _aktiv = aktiv;
    _isFrNord = isFrNord;

    createImage(typ, inhalt, isFrNord);

    if (!typ.equals(AnzeigeElementTyp.PROGRAMM) && !typ.equals(AnzeigeElementTyp.BLINKER_PROGRAMM))
      setBorder(new LineBorder(ColorConstants.gray, 2));

    updateFigure();

  }

  public void setSelected(boolean isSelected)
  {
    LineBorder lineBorder = (LineBorder)getBorder();
    if (isSelected)
    {
      lineBorder.setWidth(1);
      lineBorder.setColor(ColorConstants.black);
    }
    else
    {
      lineBorder.setWidth(2);
      lineBorder.setColor(ColorConstants.orange);
    }
  }

  public static String createImagePath(AnzeigeElementTyp typ, AnzeigeInhaltTyp inhalt)
  {
    return createImagePath(typ, inhalt, true);
  }

  /**
   * Prueft od er Inhalt wiederverwendet wird.
   * 
   * @param inhaltAlt
   * @return
   */
  public static boolean pruefeAufStatischenInhalt(AnzeigeInhaltTyp inhaltAlt)
  {
    if (listStatischeInhalte.contains(inhaltAlt))
    {
      return true;
    }
    return false;
  }

  /**
   * Liefert den Image-Path anhand des bergebenen Anzeige- bzw. Inhalt-Typs zurck. Bercksichtigt zudem auch die
   * Fahrtrichtung.
   * 
   * @param typ
   * @param inhalt
   * @param isFrNord
   * @return
   */
  public static String createImagePath(AnzeigeElementTyp typ, AnzeigeInhaltTyp inhalt, boolean isFrNord)
  {
    if (typ == null || inhalt == null)
      return null;

    String imagepath = null;

    if (typ.equals(AnzeigeElementTyp.WZG_A))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/a1_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/a1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/a1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/a1_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_GESPERRT)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_GESPERRT_BI))
      {
        imagepath = "icons/a1_111.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS_BI))
      {
        if (isFrNord)
          imagepath = "icons/a1_113.png";
        else
          imagepath = "icons/a1_114.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS_BI))
      {
        if (isFrNord)
          imagepath = "icons/a1_114.png";
        else
          imagepath = "icons/a1_113.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAU) || inhalt.equals(AnzeigeInhaltTyp.TLS_STAU_BI))
      {
        imagepath = "icons/e1_9.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_40)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_40_BI))
      {
        imagepath = "icons/e1_22.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_50)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_50_BI))
      {
        imagepath = "icons/e1_23.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_60)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_60_BI))
      {
        imagepath = "icons/e1_24.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_80)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_80_BI))
      {
        imagepath = "icons/e1_26.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_100)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_100_BI))
      {
        imagepath = "icons/e1_27.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_120)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_120_BI))
      {
        imagepath = "icons/e1_28.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_130)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_130_BI))
      {
        imagepath = "icons/e1_33.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_40)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_40_BI))
      {
        imagepath = "icons/e1_42.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_50)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_50_BI))
      {
        imagepath = "icons/e1_43.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_60)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_60_BI))
      {
        imagepath = "icons/e1_44.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_80)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_80_BI))
      {
        imagepath = "icons/e1_46.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_100)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_100_BI))
      {
        imagepath = "icons/e1_48.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_120)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_120_BI))
      {
        imagepath = "icons/e1_50.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_130)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_130_BI))
      {
        imagepath = "icons/e1_54.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE_BI))
      {
        imagepath = "icons/e1_99.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/a1_inaktiv.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/a1_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/a1_unbekannt.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.WZG_B))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/b1_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/b1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/b1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/b1_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GEFAHRENSTELLE)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_GEFAHRENSTELLE_BI))
      {
        imagepath = "icons/b1_1.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAU) || inhalt.equals(AnzeigeInhaltTyp.TLS_STAU_BI))
      {
        imagepath = "icons/b1_9.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GLAETTEGEFAHR)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_GLAETTEGEFAHR_BI))
      {
        imagepath = "icons/b1_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_SCHLEUDERGEFAHR)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_SCHLEUDERGEFAHR_BI))
      {
        imagepath = "icons/b1_4.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_ALLGEMEIN)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_ALLGEMEIN_BI))
      {
        imagepath = "icons/b1_31.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_LKW)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_LKW_BI))
      {
        imagepath = "icons/b1_32.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_BAUSTELLE) || inhalt.equals(AnzeigeInhaltTyp.TLS_BAUSTELLE_BI))
      {
        imagepath = "icons/b1_8.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE_BI))
      {
        imagepath = "icons/b1_53.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_ALLGEMEIN)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_ALLGEMEIN_BI))
      {
        imagepath = "icons/b1_51.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_LKW)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_LKW_BI))
      {
        imagepath = "icons/b1_52.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_NUR_LKW_UND_KRAFT_OMNIBUS) || inhalt.equals(AnzeigeInhaltTyp.TLS_NUR_LKW_UND_KRAFT_OMNIBUS_BI))
      {
        imagepath = "icons/b1_222.png";
      }              
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/b1_inaktiv.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/b1_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/b1_unbekannt.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.WZG_C))
    {

      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/c1_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/c1_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_NEBEL) || inhalt.equals(AnzeigeInhaltTyp.TLS_NEBEL_BI))
      {
        imagepath = "icons/c1_64.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_NAESSE) || inhalt.equals(AnzeigeInhaltTyp.TLS_NAESSE_BI))
      {
        imagepath = "icons/c1_65.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_UNFALL) || inhalt.equals(AnzeigeInhaltTyp.TLS_UNFALL_BI))
      {
        imagepath = "icons/c1_66.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAU) || inhalt.equals(AnzeigeInhaltTyp.TLS_STAU_BI))
      {
        imagepath = "icons/c1_62.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_BAUSTELLE) || inhalt.equals(AnzeigeInhaltTyp.TLS_BAUSTELLE_BI))
      {
        imagepath = "icons/c-zeichen/c1_TLS.Baustelle.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_SMOG) || inhalt.equals(AnzeigeInhaltTyp.TLS_SMOG_BI))
      {
        imagepath = "icons/c1_69.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAU) || inhalt.equals(AnzeigeInhaltTyp.TLS_STAU_BI))
      {
        imagepath = "icons/c1_62.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAU_GEFAHR) || inhalt.equals(AnzeigeInhaltTyp.TLS_STAU_GEFAHR_BI))
      {
        imagepath = "icons/c1_63.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_OELSPUR) || inhalt.equals(AnzeigeInhaltTyp.TLS_OELSPUR_BI))
      {
        imagepath = "icons/c1_204.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_SICHT) || inhalt.equals(AnzeigeInhaltTyp.TLS_SICHT_BI))
      {
        imagepath = "icons/c1_70.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_KONTROLLE) || inhalt.equals(AnzeigeInhaltTyp.TLS_KONTROLLE_BI))
      {
        imagepath = "icons/c1_71.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_TIERE) || inhalt.equals(AnzeigeInhaltTyp.TLS_TIERE_BI))
      {
        imagepath = "icons/c1_72.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AS_GLAETTE) || inhalt.equals(AnzeigeInhaltTyp.TLS_AS_GLAETTE_BI))
      {
        imagepath = "icons/c1_203.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AS_OELSPUR) || inhalt.equals(AnzeigeInhaltTyp.TLS_AS_OELSPUR_BI))
      {
        imagepath = "icons/c1_204.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_7KOMMA5TONNEN)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_7KOMMA5TONNEN_BI))
      {
        imagepath = "icons/c1_102.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_500M_BI))
      {
        imagepath = "icons/c1_81.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_1000M_BI))
      {
        imagepath = "icons/c1_67.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_1500M_BI))
      {
        imagepath = "icons/c1_83.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_2000M_BI))
      {
        imagepath = "icons/c1_68.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_2500M_BI))
      {
        imagepath = "icons/c1_85.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_3000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_3000M_BI))
      {
        imagepath = "icons/c1_97.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_500M_BI))
      {
        imagepath = "icons/c1_91.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_1000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_1000M_BI))
      {
        imagepath = "icons/c1_92.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_1500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_1500M_BI))
      {
        imagepath = "icons/c1_93.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_2000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_2000M_BI))
      {
        imagepath = "icons/c1_94.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_2500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_2500M_BI))
      {
        imagepath = "icons/c1_95.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_3000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_3000M_BI))
      {
        imagepath = "icons/c1_99.png";
      }

      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1_KILOMETER) || inhalt.equals(AnzeigeInhaltTyp.TLS_1_KILOMETER_BI))
      {
        imagepath = "icons/c1_74.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1_KOMMA_5_KILOMETER)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_1_KOMMA_5_KILOMETER_BI))
      {
        imagepath = "icons/c1_76.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2_KILOMETER) || inhalt.equals(AnzeigeInhaltTyp.TLS_2_KILOMETER_BI))
      {
        imagepath = "icons/c1_73.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2_KOMMA_5_KILOMETER)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_2_KOMMA_5_KILOMETER_BI))
      {
        imagepath = "icons/c1_77.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_3_KILOMETER) || inhalt.equals(AnzeigeInhaltTyp.TLS_3_KILOMETER_BI))
      {
        imagepath = "icons/c1_75.png";
      }

      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/c1_inaktiv.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/c1_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/c1_unbekannt.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.WZG_C2))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/c2_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/c2_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/c2_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/c2_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI_TEXT))
      {
        imagepath = "icons/c2_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_NEBEL) || inhalt.equals(AnzeigeInhaltTyp.TLS_NEBEL_BI))
      {
        imagepath = "icons/c2_64.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_NAESSE) || inhalt.equals(AnzeigeInhaltTyp.TLS_NAESSE_BI))
      {
        imagepath = "icons/c2_65.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_UNFALL) || inhalt.equals(AnzeigeInhaltTyp.TLS_UNFALL_BI))
      {
        imagepath = "icons/c2_66.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAU) || inhalt.equals(AnzeigeInhaltTyp.TLS_STAU_BI))
      {
        imagepath = "icons/c2_62.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_SMOG) || inhalt.equals(AnzeigeInhaltTyp.TLS_SMOG_BI))
      {
        imagepath = "icons/c2_69.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAU) || inhalt.equals(AnzeigeInhaltTyp.TLS_STAU_BI))
      {
        imagepath = "icons/c2_62.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_STAU_GEFAHR) || inhalt.equals(AnzeigeInhaltTyp.TLS_STAU_GEFAHR_BI))
      {
        imagepath = "icons/c2_63.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_OELSPUR) || inhalt.equals(AnzeigeInhaltTyp.TLS_OELSPUR_BI))
      {
        imagepath = "icons/c2_204.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_SICHT) || inhalt.equals(AnzeigeInhaltTyp.TLS_SICHT_BI))
      {
        imagepath = "icons/c2_70.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_KONTROLLE) || inhalt.equals(AnzeigeInhaltTyp.TLS_KONTROLLE_BI))
      {
        imagepath = "icons/c2_71.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_TIERE) || inhalt.equals(AnzeigeInhaltTyp.TLS_TIERE_BI))
      {
        imagepath = "icons/c2_72.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AS_GLAETTE) || inhalt.equals(AnzeigeInhaltTyp.TLS_AS_GLAETTE_BI))
      {
        imagepath = "icons/c2_203.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AS_OELSPUR) || inhalt.equals(AnzeigeInhaltTyp.TLS_AS_OELSPUR_BI))
      {
        imagepath = "icons/c2_204.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_7KOMMA5TONNEN)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_7KOMMA5TONNEN_BI))
      {
        imagepath = "icons/c2_102.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_500M_BI))
      {
        imagepath = "icons/c2_81.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_1000M_BI))
      {
        imagepath = "icons/c2_67.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_1500M_BI))
      {
        imagepath = "icons/c2_83.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_2000M_BI))
      {
        imagepath = "icons/c2_68.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_2500M_BI))
      {
        imagepath = "icons/c2_85.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_3000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_3000M_BI))
      {
        imagepath = "icons/c2_97.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_500M_BI))
      {
        imagepath = "icons/c2_91.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_1000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_1000M_BI))
      {
        imagepath = "icons/c2_92.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_1500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_1500M_BI))
      {
        imagepath = "icons/c2_93.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_2000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_2000M_BI))
      {
        imagepath = "icons/c2_94.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_2500M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_2500M_BI))
      {
        imagepath = "icons/c2_95.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_3000M) || inhalt.equals(AnzeigeInhaltTyp.TLS_AUF_3000M_BI))
      {
        imagepath = "icons/c2_99.png";
      }

      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1_KILOMETER) || inhalt.equals(AnzeigeInhaltTyp.TLS_1_KILOMETER_BI))
      {
        imagepath = "icons/c2_74.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_1_KOMMA_5_KILOMETER)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_1_KOMMA_5_KILOMETER_BI))
      {
        imagepath = "icons/c2_76.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2_KILOMETER) || inhalt.equals(AnzeigeInhaltTyp.TLS_2_KILOMETER_BI))
      {
        imagepath = "icons/c2_73.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_2_KOMMA_5_KILOMETER)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_2_KOMMA_5_KILOMETER_BI))
      {
        imagepath = "icons/c2_77.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_3_KILOMETER) || inhalt.equals(AnzeigeInhaltTyp.TLS_3_KILOMETER_BI))
      {
        imagepath = "icons/c2_75.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/c2_inaktiv.png";
        // imagepath = "icons/c2_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/c2_bel_anzeige.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.FREITEXT) || inhalt.equals(AnzeigeInhaltTyp.FREITEXT_BI))
      {
        imagepath = "icons/c2_frei_text1.png";
      }
      else
      {
        imagepath = "icons/c2_unbekannt.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.WZG_D2))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/d2_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/d2_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/d2_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/d2_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_HALTEN)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_HALTEN_BI))
      {
        imagepath = "icons/d2_2.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_GESPERRT)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_GESPERRT_BI))
      {
        imagepath = "icons/d2_2.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/d2_inaktiv.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/d2_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/d2_1.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.BLINKER))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI))
      {
        imagepath = "icons/bl1_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/bl1_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_BI))
      {
        imagepath = "icons/bl1_61.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_SYNCHRON)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_SYNCHRON_BI))
      {
        imagepath = "icons/bl1_61.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_ASYNCHRON)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_ASYNCHRON_BI))
      {
        imagepath = "icons/bl1_61a.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/bl1_inaktiv.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/d2_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/bl1_leer.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.BLINKER_B))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/b2_1.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/b2_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT)
          || inhalt.equals(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT_BI))
      {
        imagepath = "icons/b2_2.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/b2_1.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/b2_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/b2_1.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.PR_T))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/t1_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/t1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/t1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/t1_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_SEITENSTREIFEN_BEFAHREN))
      {
        imagepath = "icons/t1_1.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_SEITENSTREIFEN_NICHT_BEFAHREN))
      {
        imagepath = "icons/t1_2.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_SEITENSTREIFEN_RAEUMEN))
      {
        imagepath = "icons/t1_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_UNBEKLEBT))
      {
        imagepath = "icons/t1_unbeklebt.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_NEUTRAL))
      {
        imagepath = "icons/t1_inaktiv.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/t1_inaktiv.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/t1_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/t1_leer.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.PR_T2))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/t2_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/t2_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/t2_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/t2_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_BLOCKABFERTIGUNG_LKW))
      {
        imagepath = "icons/t2_1.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_BLOCKABFERTIGUNG_LKW_STOP))
      {
        imagepath = "icons/t2_2.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_UNBEKLEBT))
      {
        imagepath = "icons/t2_unbeklebt.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_NEUTRAL))
      {
        imagepath = "icons/t2_inaktiv.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/t2_inaktiv.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/t2_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/t2_leer.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.WZG_BAG))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/bag_leer";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/bag_dc";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/bag_stoer";
      }      
      else if (inhalt.equals(AnzeigeInhaltTyp.TLS_NUR_LKW_UND_KRAFT_OMNIBUS) || inhalt.equals(AnzeigeInhaltTyp.TLS_NUR_LKW_UND_KRAFT_OMNIBUS_BI))
      {
        imagepath = "icons/bag_1";
      }      
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/bag_inaktiv";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/bag_bel_anzeige";
      }
      else
      {
        imagepath = "icons/bag_leer";
      }
      
      
      if (!isFrNord)
        imagepath += "_gedreht";
      
      imagepath += ".png";
      
//      System.out.println(typ.getType() + " - " + inhalt.getType() + " - " + isFrNord + " -> " + imagepath);
    }
    else if (typ.equals(AnzeigeElementTyp.PR_W))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/w1_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/w1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/w1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/w1_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_1))
      {
        imagepath = "icons/w1_1.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_2))
      {
        imagepath = "icons/w1_21.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_3))
      {
        imagepath = "icons/w1_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_4))
      {
        imagepath = "icons/w1_unbeklebt.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_UNBEKLEBT))
      {
        imagepath = "icons/w1_unbeklebt.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_NEUTRAL))
      {
        imagepath = "icons/w1_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/w1_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/w1_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/w1_leer.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.PR_W1))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/w1_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/w1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/w1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/w1_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_1))
      {
        imagepath = "icons/w1_1.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_2))
      {
        imagepath = "icons/w1_23.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_3))
      {
        imagepath = "icons/w1_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_4))
      {
        imagepath = "icons/w1_unbeklebt.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_UNBEKLEBT))
      {
        imagepath = "icons/w1_unbeklebt.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_NEUTRAL))
      {
        imagepath = "icons/w1_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/w1_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/w1_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/w1_leer.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.PR_W11))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/w1_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/w1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/w1_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/w1_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_1))
      {
        imagepath = "icons/w1_1.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_2))
      {
        imagepath = "icons/w1_22.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_3))
      {
        imagepath = "icons/w1_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_4))
      {
        imagepath = "icons/w1_unbeklebt.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_UNBEKLEBT))
      {
        imagepath = "icons/w1_unbeklebt.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_NEUTRAL))
      {
        imagepath = "icons/w1_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/w1_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/w1_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/w1_leer.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.PR_W2))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI))
      {
        imagepath = "icons/w2_leer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/w2_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/w2_dc.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/w2_stoer.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_1))
      {
        imagepath = "icons/w2_2.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_2))
      {
        imagepath = "icons/w2_1.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_3))
      {
        imagepath = "icons/w2_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_4))
      {
        imagepath = "icons/w2_unbeklebt.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_UNBEKLEBT))
      {
        imagepath = "icons/w2_unbeklebt.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PR_TLS_NEUTRAL))
      {
        imagepath = "icons/w2_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/w2_3.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/w2_bel_anzeige.png";
      }
      else
      {
        imagepath = "icons/w2_leer.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.FREITEXT))
    {

      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL_TEXT) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI_TEXT)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI_TEXT))
      {
        imagepath = "icons/zeile_dunkel.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.DONT_CARE_BI))
      {
        imagepath = "icons/zeile_dc.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/zeile_dc.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT_TEXT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI_TEXT))
      {
        imagepath = "icons/zeile_gestoert.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.UNDEFINIERT_TEXT) || inhalt.equals(AnzeigeInhaltTyp.UNDEFINIERT_BI_TEXT))
      {
        imagepath = "icons/zeile_undefiniert.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV_TEXT) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI_TEXT)
          || inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/zeile_inaktiv.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_TEXT) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI_TEXT)
          || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/zeile_bel_anzeige.PNG";
      }
      else
      {
        System.out.println("AnzeigeElementFigure.createImagePath(else1): " + inhalt.getType());
        // imagepath = "icons/zeile_dunkel.PNG";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.FREITEXT2))
    {

      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL_TEXT) || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI_TEXT)
          || inhalt.equals(AnzeigeInhaltTyp.DUNKEL_HP_BI_TEXT))
      {
        imagepath = "icons/zeile_dunkel2.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI))
      {
        imagepath = "icons/zeile_gestoert2.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT_TEXT) || inhalt.equals(AnzeigeInhaltTyp.GESTOERT_BI_TEXT))
      {
        imagepath = "icons/zeile_gestoert2.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.UNDEFINIERT_TEXT) || inhalt.equals(AnzeigeInhaltTyp.UNDEFINIERT_BI_TEXT))
      {
        imagepath = "icons/zeile_undefiniert2.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV_TEXT) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI_TEXT)
          || inhalt.equals(AnzeigeInhaltTyp.INAKTIV) || inhalt.equals(AnzeigeInhaltTyp.INAKTIV_BI))
      {
        imagepath = "icons/zeile_inaktiv2.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_TEXT) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI_TEXT)
          || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG) || inhalt.equals(AnzeigeInhaltTyp.BELIEBIG_BI))
      {
        imagepath = "icons/zeile_bel_anzeige2.PNG";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.BEIBEHALTEN_BI))
      {
        imagepath = "icons/zeile_dc.PNG";
      }
      else
      {
        System.out.println("AnzeigeElementFigure.createImagePath(else2): " + inhalt.getType());
        // imagepath = "icons/zeile_dunkel2.PNG";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.PROGRAMM))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.PROGRAMM_AUTOMATIK))
      {
        imagepath = "icons/prg1_a_15x15.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PROGRAMM_AUTOMATIK_BLIND))
      {
        imagepath = "icons/prg2_a_15x15.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PROGRAMM_DEAKTIVIERT))
      {
        imagepath = "icons/prg1_h_15x15.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PROGRAMM_DEAKTIVIERT_BLIND))
      {
        imagepath = "icons/prg2_h_15x15.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PROGRAMM_ZEITSTEUERUNG))
      {
        imagepath = "icons/prg1_h_15x15.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PROGRAMM_HAND_BETRIEB))
      {
        imagepath = "icons/prg1_h_15x15.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PROGRAMM_VORORT_BETRIEB))
      {
        imagepath = "icons/prg1_v_15x15.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PROGRAMM_AUTARK_BETRIEB))
      {
        imagepath = "icons/prg1_x_15x15.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PROGRAMM_NOT_BETRIEB))
      {
        imagepath = "icons/prg1_n_15x15.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PROGRAMM_TEST_BETRIEB))
      {
        imagepath = "icons/prg1_y_15x15.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.PROGRAMM_UNDEFINIERT))
      {
        imagepath = "icons/prg1_leer_15x15.png";
      }
      else
      {
        imagepath = "icons/prg1_leer_15x15.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.STRECKEN_STATION))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL))
      {
        imagepath = "icons/sst_grau2.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT))
      {
        imagepath = "icons/sst_rot.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.OK))
      {
        imagepath = "icons/sst_gruen.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TEILSTOERUNG))
      {
        imagepath = "icons/sst_gelb.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV))
      {
        imagepath = "icons/sst_grau2.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.UNDEFINIERT))
      {
        imagepath = "icons/sst_grau2.png";
      }
      else
      {
        imagepath = "icons/sst_grau2.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.STRECKEN_STATION_A14))
    {
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL))
      {
        imagepath = "icons/sst_grau2.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.GESTOERT))
      {
        imagepath = "icons/sst_rot.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.OK))
      {
        imagepath = "icons/sst_gruen.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.TEILSTOERUNG))
      {
        imagepath = "icons/sst_gelb_a14.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.INAKTIV))
      {
        imagepath = "icons/sst_grau2.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.UNDEFINIERT))
      {
        imagepath = "icons/sst_grau2.png";
      }
      else
      {
        imagepath = "icons/sst_grau2.png";
      }
    }
    else if (typ.equals(AnzeigeElementTyp.LSA_A81))
    {
      /*
       * LSA
       */
      if (inhalt.equals(AnzeigeInhaltTyp.DUNKEL_BI))
      {
        imagepath = "icons/lsa/lsa_dunkel_16x29.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.LSA_A81_GELB))
      {
        imagepath = "icons/lsa/lsa_gelb_16x29.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.LSA_A81_ROT))
      {
        imagepath = "icons/lsa/lsa_rot_16x29.png";
      }
      else if (inhalt.equals(AnzeigeInhaltTyp.LSA_A81_UNBEKANNT))
      {
        imagepath = "icons/lsa/lsa_unbekannt_16x29.png";
      }
      else
      {
        imagepath = "icons/lsa/lsa_unbekannt_16x29.png";
      }

      if (!isFrNord)
      {
        /*
         * Sdausrichtung ermglichen.
         */
        imagepath = imagepath + ";180";
      }
    }
    else
    {

    }

    return imagepath;
  }

  /**
   * Liefert die Inhalte eines Anzeige-Typs zurck.
   * 
   * @param typ
   * @return
   */
  public static List<AnzeigeInhaltTyp> createAnzeigeInhalte(AnzeigeElementTyp typ)
  {

    List<AnzeigeInhaltTyp> list = new ArrayList<AnzeigeInhaltTyp>();

    if (typ.equals(AnzeigeElementTyp.WZG_A))
    {
      list.add(AnzeigeInhaltTyp.DUNKEL);
      list.add(AnzeigeInhaltTyp.GESTOERT);
      list.add(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_GESPERRT);
      list.add(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS);
      list.add(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS);
      list.add(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_40);
      list.add(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_60);
      list.add(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_80);
      list.add(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_100);
      list.add(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_120);
      // list.add("icons/a1_unbekannt.png");

    }
    else if (typ.equals(AnzeigeElementTyp.WZG_A2))
    {
      list.add(AnzeigeInhaltTyp.DUNKEL);
      list.add(AnzeigeInhaltTyp.GESTOERT);
      list.add(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_GESPERRT);
      list.add(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS);
      list.add(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS);
      list.add(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_40);
      list.add(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_60);
      list.add(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_80);
      list.add(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_100);
      list.add(AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_120);
      list.add(AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_HALTEN);
      // list.add("icons/a1_unbekannt.png");
    }
    else if (typ.equals(AnzeigeElementTyp.WZG_B))
    {
      list.add(AnzeigeInhaltTyp.DUNKEL);
      list.add(AnzeigeInhaltTyp.GESTOERT);
      list.add(AnzeigeInhaltTyp.TLS_GEFAHRENSTELLE);
      list.add(AnzeigeInhaltTyp.TLS_STAU);
      list.add(AnzeigeInhaltTyp.TLS_GLAETTEGEFAHR);
      list.add(AnzeigeInhaltTyp.TLS_SCHLEUDERGEFAHR);
      list.add(AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_ALLGEMEIN);
      list.add(AnzeigeInhaltTyp.TLS_UEBERHOLVERBOT_LKW);
      list.add(AnzeigeInhaltTyp.TLS_BAUSTELLE);
      list.add(AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE);
      list.add(AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_ALLGEMEIN);
      list.add(AnzeigeInhaltTyp.TLS_ENDE_UEBERHOLVERBOT_LKW);
      // list.add("icons/b1_unbekannt.png");
    }
    else if (typ.equals(AnzeigeElementTyp.WZG_C))
    {
      list.add(AnzeigeInhaltTyp.DUNKEL);
      list.add(AnzeigeInhaltTyp.GESTOERT);
      list.add(AnzeigeInhaltTyp.TLS_NEBEL);
      list.add(AnzeigeInhaltTyp.TLS_UNFALL);
      list.add(AnzeigeInhaltTyp.TLS_7KOMMA5TONNEN);
      // list.add("icons/c1_unbekannt.png");
    }
    else if (typ.equals(AnzeigeElementTyp.WZG_C2))
    {
      list.add(AnzeigeInhaltTyp.DUNKEL);
      list.add(AnzeigeInhaltTyp.GESTOERT);
      list.add(AnzeigeInhaltTyp.TLS_1000M);
      list.add(AnzeigeInhaltTyp.TLS_2000M);
      list.add(AnzeigeInhaltTyp.TLS_NEBEL);
      list.add(AnzeigeInhaltTyp.TLS_UNFALL);
      list.add(AnzeigeInhaltTyp.TLS_7KOMMA5TONNEN);
      // list.add("icons/c1_unbekannt.png");
    }
    else if (typ.equals(AnzeigeElementTyp.FREITEXT))
    {
      list.add(AnzeigeInhaltTyp.DUNKEL_TEXT);
      list.add(AnzeigeInhaltTyp.GESTOERT_TEXT);
      list.add(AnzeigeInhaltTyp.INAKTIV_TEXT);
      list.add(AnzeigeInhaltTyp.UNDEFINIERT_TEXT);
    }
    else if (typ.equals(AnzeigeElementTyp.FREITEXT2))
    {
      list.add(AnzeigeInhaltTyp.DUNKEL_TEXT);
      list.add(AnzeigeInhaltTyp.GESTOERT_TEXT);
      list.add(AnzeigeInhaltTyp.INAKTIV_TEXT);
      list.add(AnzeigeInhaltTyp.UNDEFINIERT_TEXT);
    }
    else if (typ.equals(AnzeigeElementTyp.BLINKER) || typ.equals(AnzeigeElementTyp.BLINKER_B))
    {
      list.add(AnzeigeInhaltTyp.DUNKEL);
      list.add(AnzeigeInhaltTyp.TLS_GELBES_BLINKLICHT);
    }
    else
    {

    }

    return list;
  }

  public String getPid()
  {
    return _pid;
  }

  public void setPid(String pid)
  {
    _pid = pid;
  }

  public AnzeigeInhaltTyp getInhalt()
  {
    return _inhalt;
  }

  public void setInhalt(AnzeigeInhaltTyp inhalt)
  {
    _inhalt = inhalt;
  }

  public AnzeigeElementTyp getTyp()
  {
    return _typ;
  }

  public void setTyp(AnzeigeElementTyp typ)
  {
    _typ = typ;
  }

  // public boolean isAktiv()
  // {
  // return _aktiv;
  // }
  //
  // public void setAktiv(boolean aktiv)
  // {
  // _aktiv = aktiv;
  //
  // }

  @Override
  public double getScale()
  {
    // TODO Auto-generated method stub
    return scale;
  }

  /**
   * Sets the zoom level
   * 
   * @param newZoom
   *          The new zoom level
   */
  public void setScale(double newZoom)
  {
    if (scale == newZoom)
      return;
    scale = newZoom;
    // sorgt dafr, dass die Gre der Figur unabhngig von der Zoomstufe
    // gleich bleibt.
    // setSize((int) (DIM.width / scale), (int) (DIM.height / scale));
    // TODO: Die Skalierung des Textes anpassen.
    updateFigure();
    layout();
    repaint();
  }

  /**
   * Bestimmt die Bounds der Figur neu.
   */
  private void updateFigure()
  {
    Dimension newDimension = new Dimension((int)(DIM.width / scale), (int)(DIM.height / scale));
    setBounds(new Rectangle(getLocation(), newDimension));
  }

  @Override
  protected void paintFigure(Graphics graphics)
  {
    // TODO Wird zu Testzwecken implementiert, wird wieder entfernt
    // Zur Erkennung ob bei den Freitextanzeigen disposte Images existieren
    // super.paintFigure(graphics);
    if (getImage() != null && !getImage().isDisposed())
      super.paintFigure(graphics);
    else
    {
      // System.err.println("AnzeigeElementFigure.paintFigure(else): " + getImage() + " -> " + _typ.getType() + " : " +
      // _inhalt.getType() + " > " + createImagePath(_typ, _inhalt, _isFrNord) + " -> " + _isImageDynamisch);
    }
  }

  /**
   * @return
   */
  public boolean isImageDynamisch()
  {
    return _isImageDynamisch;
  }

  /**
   * @param isImageDynamisch
   */
  public void setImageDynamisch(boolean isImageDynamisch)
  {
    _isImageDynamisch = isImageDynamisch;
  }

}
