/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;

/**
 * Figure, welche eine Ufd-Messstelle mit Sichtweiten-Sensor reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class UmfeldDatenSensorSichtweitenFigure extends Figure
{

  private PointList _plist = new PointList(24);

  private String _pid = null;

  private int[] _pts;

  private ColoredPolygon _poly;

  public UmfeldDatenSensorSichtweitenFigure(String pid, int posx, int posy)
  {
    _pid = pid;

    setOpaque(true);    

    setBounds(new Rectangle(posx, posy + 12, 31, 19));

    _pts = new int[] { 0, 0, 12, 0, 6, 6, 24, 6, 18, 0, 30, 0, 30, 9, 21, 12, 21, 18, 9, 18, 9, 12, 0, 9 };

    Rectangle r = bounds;
    for (int i = 0; i < _pts.length - 1; i += 2)
    {
      int x = _pts[i];
      int y = _pts[i + 1];
      _plist.addPoint(r.x + x, r.y + y);
    }

    _poly = new ColoredPolygon(ColorConstants.gray);
    _poly.setOpaque(true);
    _poly.setForegroundColor(ColorConstants.black);
    _poly.setPoints(_plist);

    add(_poly);
    setConstraint(_poly, new Rectangle(posx, posy + 12, 31, 19));
  }

  public UmfeldDatenSensorSichtweitenFigure(String pid)
  {
    _pid = pid;

    setOpaque(true);
    setBorder(new LineBorder(ColorConstants.black, 1));
    Rectangle r = bounds;

    setBounds(new Rectangle(0, 0, -1, -1));

    _pts = new int[] { 0, 0, 12, 0, 6, 6, 24, 6, 18, 0, 30, 0, 30, 9, 21, 12, 21, 18, 9, 18, 9, 12, 0, 9 };

    for (int i = 0; i < _pts.length - 1; i += 2)
    {
      int x = _pts[i];
      int y = _pts[i + 1];
      _plist.addPoint(r.x + x, r.y + y);
      // _plist.addPoint(x, y);
    }

    _poly = new ColoredPolygon(ColorConstants.gray);
    _poly.setOpaque(true);
    _poly.setForegroundColor(ColorConstants.black);
    _poly.setPoints(_plist);

    add(_poly);
    
    updateFigure();
  }

  public String getPid()
  {
    return _pid;
  }

  public void setPid(String pid)
  {
    _pid = pid;
  }

  public void setFarbe(Color color)
  {
    _poly.setFillColor(color);
  }

  public void getFarbe(Color color)
  {
    _poly.getFillColor();
  }

  /**
   * Bestimmt die Bounds der Figur neu.
   */
  private void updateFigure()
  {

    setBounds(new Rectangle(getLocation(), getSize()));
  }

}
