/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Typklasse fr Inhalt von Anzeigen
 * 
 * @author Pittner
 * 
 */
public class AnzeigeInhaltTyp implements Cloneable
{
  private static List typen = new ArrayList();

  // Inhalte ber werden ber die Pid des Systemobjektes des WVZ- Inhaltes bestimmt
  // Allgemein, gilt fr alle WVZ-Typen
  public static AnzeigeInhaltTyp DUNKEL = new AnzeigeInhaltTyp("wvzInhalt.uz.A8.sba_stg.Dunkel");

  public static AnzeigeInhaltTyp BELIEBIG = new AnzeigeInhaltTyp("wvzInhalt.uz.B27.Beliebig");

  public static AnzeigeInhaltTyp GESTOERT = new AnzeigeInhaltTyp("wvzInhalt.uz.B27.Gestrt");

  public static AnzeigeInhaltTyp FREITEXT = new AnzeigeInhaltTyp("wvzInhalt.uz.B27.Freitext");

  public static AnzeigeInhaltTyp INAKTIV = new AnzeigeInhaltTyp("wvzInhalt.uz.B27.Inaktiv");

  public static AnzeigeInhaltTyp UNDEFINIERT = new AnzeigeInhaltTyp("wvzInhalt.uz.B27.Undefiniert");

  public static AnzeigeInhaltTyp DUNKEL_TEXT = new AnzeigeInhaltTyp("nw.sba.wvzInhalt.Dunkel.Text");

  public static AnzeigeInhaltTyp GESTOERT_TEXT = new AnzeigeInhaltTyp("nw.sba.wvzInhalt.Gestrt.Text");

  public static AnzeigeInhaltTyp INAKTIV_TEXT = new AnzeigeInhaltTyp("nw.sba.wvzInhalt.Inaktiv.Text");

  public static AnzeigeInhaltTyp UNDEFINIERT_TEXT = new AnzeigeInhaltTyp("nw.sba.wvzInhalt.Undefiniert.Text");

  public static AnzeigeInhaltTyp BELIEBIG_TEXT = new AnzeigeInhaltTyp("nw.sba.wvzInhalt.Beliebig.Text");

  // WZG-A
  public static AnzeigeInhaltTyp TLS_FAHRSTREIFEN_GESPERRT = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsFahrstreifenGesperrt");

  public static AnzeigeInhaltTyp TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsFahrstreifenWechselnNachLinks");

  public static AnzeigeInhaltTyp TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsFahrstreifenWechselnNachRechts");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_40 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsZulaessigeHoechst40");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_50 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsZulaessigeHoechst50");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_60 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsZulaessigeHoechst60");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_80 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsZulaessigeHoechst80");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_100 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsZulaessigeHoechst100");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_120 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsZulaessigeHoechst120");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_130 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsZulaessigeHoechst130");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_40 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsEndeZulaessigeHoechst40");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_50 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsEndeZulaessigeHoechst50");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_60 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsEndeZulaessigeHoechst60");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_80 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsEndeZulaessigeHoechst80");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_100 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsEndeZulaessigeHoechst100");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_120 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsEndeZulaessigeHoechst120");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_130 = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsEndeZulaessigeHoechst130");

  public static AnzeigeInhaltTyp TLS_FAHRSTREIFEN_HALTEN = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsFahrstreifenHalten");

  // WZG-B
  public static AnzeigeInhaltTyp TLS_GEFAHRENSTELLE = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsGefahrenstelle");

  public static AnzeigeInhaltTyp TLS_STAU = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsStau");

  public static AnzeigeInhaltTyp TLS_GLAETTEGEFAHR = new AnzeigeInhaltTyp("wvzInhalt.uz.a14.tlsGlaettegefahr");

  public static AnzeigeInhaltTyp TLS_SCHLEUDERGEFAHR = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsSchleudergefahr");

  public static AnzeigeInhaltTyp TLS_UEBERHOLVERBOT_ALLGEMEIN = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsUeberholverbotAllgemein");

  public static AnzeigeInhaltTyp TLS_UEBERHOLVERBOT_LKW = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsUeberholverbotLkw");

  public static AnzeigeInhaltTyp TLS_BAUSTELLE = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsBaustelle");

  public static AnzeigeInhaltTyp TLS_ENDE_ALLER_STRECKENVERBOTE = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsEndeAllerStreckenverbote");

  public static AnzeigeInhaltTyp TLS_ENDE_UEBERHOLVERBOT_ALLGEMEIN = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsEndeUeberholverbotAllgemein");

  public static AnzeigeInhaltTyp TLS_ENDE_UEBERHOLVERBOT_LKW = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsEndeUeberholverbotLkw");

  // WZG-C
  public static AnzeigeInhaltTyp TLS_500M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls500Meter");

  public static AnzeigeInhaltTyp TLS_1000M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls1000Meter");

  public static AnzeigeInhaltTyp TLS_1500M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls1500Meter");

  public static AnzeigeInhaltTyp TLS_2000M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls2000Meter");

  public static AnzeigeInhaltTyp TLS_2500M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls2500Meter");

  public static AnzeigeInhaltTyp TLS_3000M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls3000Meter");

  public static AnzeigeInhaltTyp TLS_1_KILOMETER = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls1Kilometer");

  public static AnzeigeInhaltTyp TLS_1_KOMMA_5_KILOMETER = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls1Komma5Kilometer");

  public static AnzeigeInhaltTyp TLS_2_KILOMETER = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls2Kilometer");

  public static AnzeigeInhaltTyp TLS_2_KOMMA_5_KILOMETER = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls2Komma5Kilometer");

  public static AnzeigeInhaltTyp TLS_3_KILOMETER = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls3Kilometer");

  public static AnzeigeInhaltTyp TLS_AUF_500M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsAuf500Meter");

  public static AnzeigeInhaltTyp TLS_AUF_1000M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsAuf1000Meter");

  public static AnzeigeInhaltTyp TLS_AUF_1500M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsAuf1500Meter");

  public static AnzeigeInhaltTyp TLS_AUF_2000M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsAuf2000Meter");

  public static AnzeigeInhaltTyp TLS_AUF_2500M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsAuf2500Meter");

  public static AnzeigeInhaltTyp TLS_AUF_3000M = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsAuf3000Meter");

  public static AnzeigeInhaltTyp TLS_NEBEL = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsNebel");

  public static AnzeigeInhaltTyp TLS_NAESSE = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsNaesse");

  public static AnzeigeInhaltTyp TLS_UNFALL = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsUnfall");

  public static AnzeigeInhaltTyp TLS_OELSPUR = new AnzeigeInhaltTyp("wvzInhalt.uz.a14.tlsOelspur");

  public static AnzeigeInhaltTyp TLS_SMOG = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsSmog");

  public static AnzeigeInhaltTyp TLS_KONTROLLE = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsKontrolle");

  public static AnzeigeInhaltTyp TLS_TIERE = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsTiere");

  public static AnzeigeInhaltTyp TLS_AS_GLAETTE = new AnzeigeInhaltTyp("wvzInhalt.uz.a14.tls-asGlaette");

  public static AnzeigeInhaltTyp TLS_AS_OELSPUR = new AnzeigeInhaltTyp("wvzInhalt.uz.a14.tls-asOelspur");

  public static AnzeigeInhaltTyp TLS_7KOMMA5TONNEN = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tls7Komma5Tonnen");

  public static AnzeigeInhaltTyp TLS_STAU_GEFAHR = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsStaugefahr");

  public static AnzeigeInhaltTyp TLS_SICHT = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsSicht");

  // Blinker
  public static AnzeigeInhaltTyp TLS_GELBES_BLINKLICHT = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.tlsGelbesBlinklicht");

  public static AnzeigeInhaltTyp TLS_GELBES_BLINKLICHT_ASYNCHRON = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsGelbesBlinklichtAsynchron");

  public static AnzeigeInhaltTyp TLS_GELBES_BLINKLICHT_SYNCHRON = new AnzeigeInhaltTyp(
      "wvzInhalt.uz.A14.tlsGelbesBlinklichtSynchron");
  
  public static AnzeigeInhaltTyp TLS_NUR_LKW_UND_KRAFT_OMNIBUS = new AnzeigeInhaltTyp(
  "wvzInhalt.uz.A14.tlsNurLkwUndKraftOmnibus");

  // Prisma
  // public static AnzeigeInhaltTyp PR_SEITE_1 = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.prSeite1");
  // public static AnzeigeInhaltTyp PR_SEITE_2 = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.prSeite2");
  // public static AnzeigeInhaltTyp PR_SEITE_3 = new AnzeigeInhaltTyp("wvzInhalt.uz.A14.prSeite3");

  // Inhalte ber werden ber die eindeutigen String "Bildinhalt" des WVZ-Inhaltes bestimmt
  // Allgemein, gilt fr alle WVZ-Typen
  public static AnzeigeInhaltTyp DUNKEL_BI = new AnzeigeInhaltTyp("Dunkel");

  public static AnzeigeInhaltTyp BELIEBIG_BI = new AnzeigeInhaltTyp("Beliebig");

  public static AnzeigeInhaltTyp GESTOERT_BI = new AnzeigeInhaltTyp("Gestrt");

  public static AnzeigeInhaltTyp FREITEXT_BI = new AnzeigeInhaltTyp("Freitext");

  public static AnzeigeInhaltTyp INAKTIV_BI = new AnzeigeInhaltTyp("Inaktiv");

  public static AnzeigeInhaltTyp UNDEFINIERT_BI = new AnzeigeInhaltTyp("Undefiniert");

  public static AnzeigeInhaltTyp DUNKEL_BI_TEXT = new AnzeigeInhaltTyp("Dunkel.Text");

  public static AnzeigeInhaltTyp GESTOERT_BI_TEXT = new AnzeigeInhaltTyp("Gestrt.Text");

  public static AnzeigeInhaltTyp INAKTIV_BI_TEXT = new AnzeigeInhaltTyp("Inaktiv.Text");

  public static AnzeigeInhaltTyp UNDEFINIERT_BI_TEXT = new AnzeigeInhaltTyp("Undefiniert.Text");

  public static AnzeigeInhaltTyp BELIEBIG_BI_TEXT = new AnzeigeInhaltTyp("Beliebig.Text");

  public static AnzeigeInhaltTyp DONT_CARE_BI = new AnzeigeInhaltTyp("DontCare");

  public static AnzeigeInhaltTyp BEIBEHALTEN_BI = new AnzeigeInhaltTyp("Beibehalten");
  
  // WZG-A
  public static AnzeigeInhaltTyp TLS_FAHRSTREIFEN_GESPERRT_BI = new AnzeigeInhaltTyp("TLS.FahrstreifenGesperrt");

  public static AnzeigeInhaltTyp TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS_BI = new AnzeigeInhaltTyp(
      "TLS.FahrstreifenWechselnNachLinks");

  public static AnzeigeInhaltTyp TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS_BI = new AnzeigeInhaltTyp(
      "TLS.FahrstreifenWechselnNachRechts");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_40_BI = new AnzeigeInhaltTyp("TLS.ZulaessigeHoechst40");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_50_BI = new AnzeigeInhaltTyp("TLS.ZulaessigeHoechst50");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_60_BI = new AnzeigeInhaltTyp("TLS.ZulaessigeHoechst60");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_80_BI = new AnzeigeInhaltTyp("TLS.ZulaessigeHoechst80");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_100_BI = new AnzeigeInhaltTyp("TLS.ZulaessigeHoechst100");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_120_BI = new AnzeigeInhaltTyp("TLS.ZulaessigeHoechst120");

  public static AnzeigeInhaltTyp TLS_ZULAESSIGE_HOECHST_130_BI = new AnzeigeInhaltTyp("TLS.ZulaessigeHoechst130");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_40_BI = new AnzeigeInhaltTyp("TLS.EndeZulaessigeHoechst40");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_50_BI = new AnzeigeInhaltTyp("TLS.EndeZulaessigeHoechst50");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_60_BI = new AnzeigeInhaltTyp("TLS.EndeZulaessigeHoechst60");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_80_BI = new AnzeigeInhaltTyp("TLS.EndeZulaessigeHoechst80");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_100_BI = new AnzeigeInhaltTyp(
      "TLS.EndeZulaessigeHoechst100");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_120_BI = new AnzeigeInhaltTyp(
      "TLS.EndeZulaessigeHoechst120");

  public static AnzeigeInhaltTyp TLS_ENDE_ZULAESSIGE_HOECHST_130_BI = new AnzeigeInhaltTyp(
      "TLS.EndeZulaessigeHoechst130");

  public static AnzeigeInhaltTyp TLS_FAHRSTREIFEN_HALTEN_BI = new AnzeigeInhaltTyp("TLS.FahrstreifenHalten");

  // WZG-B
  public static AnzeigeInhaltTyp TLS_GEFAHRENSTELLE_BI = new AnzeigeInhaltTyp("TLS.Gefahrenstelle");

  public static AnzeigeInhaltTyp TLS_STAU_BI = new AnzeigeInhaltTyp("TLS.Stau");

  public static AnzeigeInhaltTyp TLS_GLAETTEGEFAHR_BI = new AnzeigeInhaltTyp("TLS.Glaettegefahr");

  public static AnzeigeInhaltTyp TLS_SCHLEUDERGEFAHR_BI = new AnzeigeInhaltTyp("TLS.Schleudergefahr");

  public static AnzeigeInhaltTyp TLS_UEBERHOLVERBOT_ALLGEMEIN_BI = new AnzeigeInhaltTyp("TLS.UeberholverbotAllgemein");

  public static AnzeigeInhaltTyp TLS_UEBERHOLVERBOT_LKW_BI = new AnzeigeInhaltTyp("TLS.UeberholverbotLkw");

  public static AnzeigeInhaltTyp TLS_BAUSTELLE_BI = new AnzeigeInhaltTyp("TLS.Baustelle");

  public static AnzeigeInhaltTyp TLS_ENDE_ALLER_STRECKENVERBOTE_BI = new AnzeigeInhaltTyp(
      "TLS.EndeAllerStreckenverbote");

  public static AnzeigeInhaltTyp TLS_ENDE_UEBERHOLVERBOT_ALLGEMEIN_BI = new AnzeigeInhaltTyp(
      "TLS.EndeUeberholverbotAllgemein");

  public static AnzeigeInhaltTyp TLS_ENDE_UEBERHOLVERBOT_LKW_BI = new AnzeigeInhaltTyp("TLS.EndeUeberholverbotLkw");

  // WZG-C
  public static AnzeigeInhaltTyp TLS_500M_BI = new AnzeigeInhaltTyp("TLS.500Meter");

  public static AnzeigeInhaltTyp TLS_1000M_BI = new AnzeigeInhaltTyp("TLS.1000Meter");

  public static AnzeigeInhaltTyp TLS_1500M_BI = new AnzeigeInhaltTyp("TLS.1500Meter");

  public static AnzeigeInhaltTyp TLS_2000M_BI = new AnzeigeInhaltTyp("TLS.2000Meter");

  public static AnzeigeInhaltTyp TLS_2500M_BI = new AnzeigeInhaltTyp("TLS.2500Meter");

  public static AnzeigeInhaltTyp TLS_3000M_BI = new AnzeigeInhaltTyp("TLS.3000Meter");

  public static AnzeigeInhaltTyp TLS_1_KILOMETER_BI = new AnzeigeInhaltTyp("TLS.1Kilometer");

  public static AnzeigeInhaltTyp TLS_1_KOMMA_5_KILOMETER_BI = new AnzeigeInhaltTyp("TLS.1Komma5Kilometer");

  public static AnzeigeInhaltTyp TLS_2_KILOMETER_BI = new AnzeigeInhaltTyp("TLS.2Kilometer");

  public static AnzeigeInhaltTyp TLS_2_KOMMA_5_KILOMETER_BI = new AnzeigeInhaltTyp("TLS.2Komma5Kilometer");

  public static AnzeigeInhaltTyp TLS_3_KILOMETER_BI = new AnzeigeInhaltTyp("TLS.3Kilometer");

  public static AnzeigeInhaltTyp TLS_AUF_500M_BI = new AnzeigeInhaltTyp("TLS.Auf500Meter");

  public static AnzeigeInhaltTyp TLS_AUF_1000M_BI = new AnzeigeInhaltTyp("TLS.Auf1000Meter");

  public static AnzeigeInhaltTyp TLS_AUF_1500M_BI = new AnzeigeInhaltTyp("TLS.Auf1500Meter");

  public static AnzeigeInhaltTyp TLS_AUF_2000M_BI = new AnzeigeInhaltTyp("TLS.Auf2000Meter");

  public static AnzeigeInhaltTyp TLS_AUF_2500M_BI = new AnzeigeInhaltTyp("TLS.Auf2500Meter");

  public static AnzeigeInhaltTyp TLS_AUF_3000M_BI = new AnzeigeInhaltTyp("TLS.Auf3000Meter");

  public static AnzeigeInhaltTyp TLS_NEBEL_BI = new AnzeigeInhaltTyp("TLS.Nebel");

  public static AnzeigeInhaltTyp TLS_NAESSE_BI = new AnzeigeInhaltTyp("TLS.Naesse");

  public static AnzeigeInhaltTyp TLS_UNFALL_BI = new AnzeigeInhaltTyp("TLS.Unfall");

  public static AnzeigeInhaltTyp TLS_SMOG_BI = new AnzeigeInhaltTyp("TLS.Smog");

  public static AnzeigeInhaltTyp TLS_AS_GLAETTE_BI = new AnzeigeInhaltTyp("TLS-AS.Gltte");

  public static AnzeigeInhaltTyp TLS_AS_OELSPUR_BI = new AnzeigeInhaltTyp("TLS-AS.lspur");

  public static AnzeigeInhaltTyp TLS_7KOMMA5TONNEN_BI = new AnzeigeInhaltTyp("TLS.7Komma5Tonnen");

  public static AnzeigeInhaltTyp TLS_STAU_GEFAHR_BI = new AnzeigeInhaltTyp("TLS.Staugefahr");

  public static AnzeigeInhaltTyp TLS_SICHT_BI = new AnzeigeInhaltTyp("TLS.Sicht");

  public static AnzeigeInhaltTyp TLS_KONTROLLE_BI = new AnzeigeInhaltTyp("TLS.Kontrolle");

  public static AnzeigeInhaltTyp TLS_TIERE_BI = new AnzeigeInhaltTyp("TLS.Tiere");

  public static AnzeigeInhaltTyp TLS_OELSPUR_BI = new AnzeigeInhaltTyp("TLS.Oelspur");

  // Blinker
  public static AnzeigeInhaltTyp TLS_GELBES_BLINKLICHT_BI = new AnzeigeInhaltTyp("TLS.GelbesBlinklicht");

  public static AnzeigeInhaltTyp TLS_GELBES_BLINKLICHT_ASYNCHRON_BI = new AnzeigeInhaltTyp(
      "TLS.GelbesBlinklichtAsynchron");

  public static AnzeigeInhaltTyp TLS_GELBES_BLINKLICHT_SYNCHRON_BI = new AnzeigeInhaltTyp(
      "TLS.GelbesBlinklichtSynchron");
  
  public static AnzeigeInhaltTyp TLS_NUR_LKW_UND_KRAFT_OMNIBUS_BI = new AnzeigeInhaltTyp(
  "TLS.NurLkwUndKraftomnibus");

  public static AnzeigeInhaltTyp DUNKEL_HP_BI = new AnzeigeInhaltTyp("DunkelHP");

  public static AnzeigeInhaltTyp DUNKEL_HP_BI_TEXT = new AnzeigeInhaltTyp("DunkelHP.Text");

  public static AnzeigeInhaltTyp PR_TLS_UNBEKLEBT = new AnzeigeInhaltTyp("TLS.Unbeklebt");

  // Prisma
  // public static AnzeigeInhaltTyp PR_SEITE_1_BI = new AnzeigeInhaltTyp("PR.Seite1");
  // public static AnzeigeInhaltTyp PR_SEITE_2_BI = new AnzeigeInhaltTyp("PR.Seite2");
  // public static AnzeigeInhaltTyp PR_SEITE_3_BI = new AnzeigeInhaltTyp("PR.Seite3");
  // public static AnzeigeInhaltTyp PR_SEITE_3_BI = new AnzeigeInhaltTyp("");
  public static AnzeigeInhaltTyp PR_TLS_BLOCKABFERTIGUNG_LKW = new AnzeigeInhaltTyp("TLS.BlockabfertigungLkw");

  public static AnzeigeInhaltTyp PR_TLS_BLOCKABFERTIGUNG_LKW_STOP = new AnzeigeInhaltTyp("TLS.BlockabfertigungLkwStop");

  public static AnzeigeInhaltTyp PR_TLS_SEITENSTREIFEN_BEFAHREN = new AnzeigeInhaltTyp("TLS.SeitenstreifenBefahren");

  public static AnzeigeInhaltTyp PR_TLS_SEITENSTREIFEN_NICHT_BEFAHREN = new AnzeigeInhaltTyp(
      "TLS.SeitenstreifenNichtBefahren");

  public static AnzeigeInhaltTyp PR_TLS_SEITENSTREIFEN_RAEUMEN = new AnzeigeInhaltTyp("TLS.SeitenstreifenRaeumen");

  public static AnzeigeInhaltTyp PR_TLS_PRISMA_SEITE_1 = new AnzeigeInhaltTyp("TLS.PrismaSeite1");

  public static AnzeigeInhaltTyp PR_TLS_PRISMA_SEITE_2 = new AnzeigeInhaltTyp("TLS.PrismaSeite2");

  public static AnzeigeInhaltTyp PR_TLS_PRISMA_SEITE_3 = new AnzeigeInhaltTyp("TLS.PrismaSeite3");

  public static AnzeigeInhaltTyp PR_TLS_PRISMA_SEITE_4 = new AnzeigeInhaltTyp("TLS.PrismaSeite4");

  public static AnzeigeInhaltTyp PR_TLS_NEUTRAL = new AnzeigeInhaltTyp("TLS.Neutral");

  // ZufahrtsRegelung
  public static AnzeigeInhaltTyp PROGRAMM_AUTOMATIK = new AnzeigeInhaltTyp("Automatik");

  public static AnzeigeInhaltTyp PROGRAMM_DEAKTIVIERT = new AnzeigeInhaltTyp("deaktiviert");

  public static AnzeigeInhaltTyp PROGRAMM_ZEITSTEUERUNG = new AnzeigeInhaltTyp("Zeitsteuerung");

  public static AnzeigeInhaltTyp PROGRAMM_UNDEFINIERT = new AnzeigeInhaltTyp("Undefiniert");

  public static AnzeigeInhaltTyp PROGRAMM_AUS = new AnzeigeInhaltTyp("aus");

  public static AnzeigeInhaltTyp PROGRAMM_DAUERROT = new AnzeigeInhaltTyp("Dauerrot");

  public static AnzeigeInhaltTyp PROGRAMM_ROTSCHALTZEIT = new AnzeigeInhaltTyp("Rotschaltzeit");

  public static AnzeigeInhaltTyp PROGRAMM_DEAKTIVIERT_JA = new AnzeigeInhaltTyp("Ja");

  public static AnzeigeInhaltTyp PROGRAMM_DEAKTIVIERT_NEIN = new AnzeigeInhaltTyp("Nein");

  public static AnzeigeInhaltTyp PROGRAMM_NORMAL_BETRIEB = new AnzeigeInhaltTyp("Normalbetrieb");

  public static AnzeigeInhaltTyp PROGRAMM_BLIND_BETRIEB = new AnzeigeInhaltTyp("Blindbetrieb");

  public static AnzeigeInhaltTyp PROGRAMM_HAND_BETRIEB = new AnzeigeInhaltTyp("Handbetrieb");

  public static AnzeigeInhaltTyp PROGRAMM_VORORT_BETRIEB = new AnzeigeInhaltTyp("Vor Ort");

  public static AnzeigeInhaltTyp PROGRAMM_AUTARK_BETRIEB = new AnzeigeInhaltTyp("Autarker Betrieb");

  public static AnzeigeInhaltTyp PROGRAMM_TEST_BETRIEB = new AnzeigeInhaltTyp("Testbetrieb");

  public static AnzeigeInhaltTyp PROGRAMM_NOT_BETRIEB = new AnzeigeInhaltTyp("Notbetrieb");

  public static AnzeigeInhaltTyp PROGRAMM_AUTOMATIK_BLIND = new AnzeigeInhaltTyp("Automatik_Blind");

  public static AnzeigeInhaltTyp PROGRAMM_DEAKTIVIERT_BLIND = new AnzeigeInhaltTyp("deaktiviert_Blind");

  public static AnzeigeInhaltTyp OK = new AnzeigeInhaltTyp("ok");

  public static AnzeigeInhaltTyp TEILSTOERUNG = new AnzeigeInhaltTyp("teil");

  /*
   * Fr die LSA
   */
  public static AnzeigeInhaltTyp LSA_A81_GELB = new AnzeigeInhaltTyp("gelb");

  public static AnzeigeInhaltTyp LSA_A81_ROT = new AnzeigeInhaltTyp("rot");

  public static AnzeigeInhaltTyp LSA_A81_UNBEKANNT = new AnzeigeInhaltTyp("LSA unbekannt");

  private String type;

  private Long _zeit;

  private AnzeigeInhaltTyp(String type)
  {
    this.type = type;
    String typeToAdd = this.getType();
    typen.add(typeToAdd);
    // typen.add(this);
  }

  /**
   * @return Returns the type.
   */
  public String getType()
  {
    return type;
  }

  /**
   * @return Returns the type.
   */
  public long getZeit()
  {
    return _zeit;
  }

  public static boolean hasType(String type)
  {
    return typen.contains(type);
  }

  public static String getTypes()
  {
    StringBuffer typeBuffer = new StringBuffer();
    for (Iterator iter = typen.iterator(); iter.hasNext();)
    {
      String element = (String)iter.next();
      // AnzeigeInhaltTyp typ = (AnzeigeInhaltTyp) iter.next();
      // String element = typ.getType();
      typeBuffer.append(element).append(", ");
    }

    if (typen.size() >= 1)
    {
      typeBuffer.delete(typeBuffer.length() - 2, typeBuffer.length());
    }
    return typeBuffer.toString();
  }

  public static List getTypesAsList()
  {
    return typen;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.bsvrz.buv.plugin.anlagendarstellung.status.Status#setStatus(java.lang.String)
   */
  public static AnzeigeInhaltTyp setTyp(String s)
  {
    // TODO Auto-generated method stub
    AnzeigeInhaltTyp ait = null;

    if (s.equals(DUNKEL.getType()))
      ait = DUNKEL;
    else if (s.equals(INAKTIV.getType()))
      ait = INAKTIV;
    else if (s.equals(DUNKEL_TEXT.getType()))
      ait = DUNKEL_TEXT;
    else if (s.equals(GESTOERT_TEXT.getType()))
      ait = GESTOERT_TEXT;
    else if (s.equals(INAKTIV_TEXT.getType()))
      ait = INAKTIV_TEXT;
    else if (s.equals(UNDEFINIERT_TEXT.getType()))
      ait = UNDEFINIERT_TEXT;
    else if (s.equals(TLS_GELBES_BLINKLICHT.getType()))
      ait = TLS_GELBES_BLINKLICHT;
    else if (s.equals(TLS_500M.getType()))
      ait = TLS_500M;
    else if (s.equals(TLS_1000M.getType()))
      ait = TLS_1000M;
    else if (s.equals(TLS_1500M.getType()))
      ait = TLS_1500M;
    else if (s.equals(TLS_2000M.getType()))
      ait = TLS_2000M;
    else if (s.equals(TLS_2500M.getType()))
      ait = TLS_2500M;
    else if (s.equals(TLS_3000M.getType()))
      ait = TLS_3000M;
    else if (s.equals(TLS_1_KILOMETER.getType()))
      ait = TLS_1_KILOMETER;
    else if (s.equals(TLS_1_KOMMA_5_KILOMETER.getType()))
      ait = TLS_1_KOMMA_5_KILOMETER;
    else if (s.equals(TLS_2_KILOMETER.getType()))
      ait = TLS_2_KILOMETER;
    else if (s.equals(TLS_2_KOMMA_5_KILOMETER.getType()))
      ait = TLS_2_KOMMA_5_KILOMETER;
    else if (s.equals(TLS_3_KILOMETER.getType()))
      ait = TLS_3_KILOMETER;
    else if (s.equals(TLS_7KOMMA5TONNEN.getType()))
      ait = TLS_7KOMMA5TONNEN;
    else if (s.equals(TLS_BAUSTELLE.getType()))
      ait = TLS_BAUSTELLE;
    else if (s.equals(TLS_ENDE_ALLER_STRECKENVERBOTE.getType()))
      ait = TLS_ENDE_ALLER_STRECKENVERBOTE;
    else if (s.equals(TLS_ENDE_UEBERHOLVERBOT_ALLGEMEIN.getType()))
      ait = TLS_ENDE_UEBERHOLVERBOT_ALLGEMEIN;
    else if (s.equals(TLS_ENDE_UEBERHOLVERBOT_LKW.getType()))
      ait = TLS_ENDE_UEBERHOLVERBOT_LKW;
    else if (s.equals(TLS_FAHRSTREIFEN_GESPERRT.getType()))
      ait = TLS_FAHRSTREIFEN_GESPERRT;
    else if (s.equals(TLS_FAHRSTREIFEN_HALTEN.getType()))
      ait = TLS_FAHRSTREIFEN_HALTEN;
    else if (s.equals(TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS.getType()))
      ait = TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS;
    else if (s.equals(TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS.getType()))
      ait = TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS;
    else if (s.equals(TLS_GEFAHRENSTELLE.getType()))
      ait = TLS_GEFAHRENSTELLE;
    else if (s.equals(TLS_GLAETTEGEFAHR.getType()))
      ait = TLS_GLAETTEGEFAHR;
    else if (s.equals(TLS_NEBEL.getType()))
      ait = TLS_NEBEL;
    else if (s.equals(TLS_STAU.getType()))
      ait = TLS_STAU;
    else if (s.equals(TLS_STAU_GEFAHR.getType()))
      ait = TLS_STAU_GEFAHR;
    else if (s.equals(TLS_SICHT.getType()))
      ait = TLS_SICHT;
    else if (s.equals(TLS_NAESSE.getType()))
      ait = TLS_NAESSE;
    else if (s.equals(TLS_KONTROLLE.getType()))
      ait = TLS_KONTROLLE;
    else if (s.equals(TLS_TIERE.getType()))
      ait = TLS_TIERE;
    else if (s.equals(TLS_AS_GLAETTE.getType()))
      ait = TLS_AS_GLAETTE;
    else if (s.equals(TLS_AS_OELSPUR.getType()))
      ait = TLS_AS_OELSPUR;
    else if (s.equals(TLS_SCHLEUDERGEFAHR.getType()))
      ait = TLS_SCHLEUDERGEFAHR;
    else if (s.equals(TLS_UEBERHOLVERBOT_ALLGEMEIN.getType()))
      ait = TLS_UEBERHOLVERBOT_ALLGEMEIN;
    else if (s.equals(TLS_UEBERHOLVERBOT_LKW.getType()))
      ait = TLS_UEBERHOLVERBOT_LKW;
    else if (s.equals(TLS_UNFALL.getType()))
      ait = TLS_UNFALL;
    else if (s.equals(TLS_ZULAESSIGE_HOECHST_100.getType()))
      ait = TLS_ZULAESSIGE_HOECHST_100;
    else if (s.equals(TLS_ZULAESSIGE_HOECHST_120.getType()))
      ait = TLS_ZULAESSIGE_HOECHST_120;
    else if (s.equals(TLS_ZULAESSIGE_HOECHST_130.getType()))
      ait = TLS_ZULAESSIGE_HOECHST_130;
    else if (s.equals(TLS_ZULAESSIGE_HOECHST_40.getType()))
      ait = TLS_ZULAESSIGE_HOECHST_40;
    else if (s.equals(TLS_ZULAESSIGE_HOECHST_50.getType()))
      ait = TLS_ZULAESSIGE_HOECHST_50;
    else if (s.equals(TLS_ZULAESSIGE_HOECHST_60.getType()))
      ait = TLS_ZULAESSIGE_HOECHST_60;
    else if (s.equals(TLS_ZULAESSIGE_HOECHST_80.getType()))
      ait = TLS_ZULAESSIGE_HOECHST_80;
    else if (s.equals(TLS_ENDE_ZULAESSIGE_HOECHST_100.getType()))
      ait = TLS_ENDE_ZULAESSIGE_HOECHST_100;
    else if (s.equals(TLS_ENDE_ZULAESSIGE_HOECHST_120.getType()))
      ait = TLS_ENDE_ZULAESSIGE_HOECHST_120;
    else if (s.equals(TLS_ENDE_ZULAESSIGE_HOECHST_130.getType()))
      ait = TLS_ENDE_ZULAESSIGE_HOECHST_130;
    else if (s.equals(TLS_ENDE_ZULAESSIGE_HOECHST_40.getType()))
      ait = TLS_ENDE_ZULAESSIGE_HOECHST_40;
    else if (s.equals(TLS_ENDE_ZULAESSIGE_HOECHST_50.getType()))
      ait = TLS_ENDE_ZULAESSIGE_HOECHST_50;
    else if (s.equals(TLS_ENDE_ZULAESSIGE_HOECHST_60.getType()))
      ait = TLS_ENDE_ZULAESSIGE_HOECHST_60;
    else if (s.equals(TLS_ENDE_ZULAESSIGE_HOECHST_80.getType()))
      ait = TLS_ENDE_ZULAESSIGE_HOECHST_80;
    else if (s.equals(PROGRAMM_AUTOMATIK.getType()))
      ait = PROGRAMM_AUTOMATIK;
    else if (s.equals(PROGRAMM_DEAKTIVIERT.getType()))
      ait = PROGRAMM_DEAKTIVIERT;
    else if (s.equals(PROGRAMM_ZEITSTEUERUNG.getType()))
      ait = PROGRAMM_ZEITSTEUERUNG;
    else if (s.equals(PROGRAMM_UNDEFINIERT.getType()))
      ait = PROGRAMM_UNDEFINIERT;
    else if (s.equals(PROGRAMM_DEAKTIVIERT_JA.getType()))
      ait = PROGRAMM_DEAKTIVIERT_JA;
    else if (s.equals(PROGRAMM_DEAKTIVIERT_NEIN.getType()))
      ait = PROGRAMM_DEAKTIVIERT_NEIN;
    else if (s.equals(PROGRAMM_NORMAL_BETRIEB.getType()))
      ait = PROGRAMM_NORMAL_BETRIEB;
    else if (s.equals(PROGRAMM_BLIND_BETRIEB.getType()))
      ait = PROGRAMM_BLIND_BETRIEB;
    else if (s.equals(PROGRAMM_HAND_BETRIEB.getType()))
      ait = PROGRAMM_HAND_BETRIEB;
    else if (s.equals(PROGRAMM_VORORT_BETRIEB.getType()))
      ait = PROGRAMM_VORORT_BETRIEB;
    else if (s.equals(PROGRAMM_AUTARK_BETRIEB.getType()))
      ait = PROGRAMM_AUTARK_BETRIEB;
    else if (s.equals(PROGRAMM_TEST_BETRIEB.getType()))
      ait = PROGRAMM_TEST_BETRIEB;
    else if (s.equals(PROGRAMM_NOT_BETRIEB.getType()))
      ait = PROGRAMM_NOT_BETRIEB;
    else if (s.equals(PROGRAMM_AUTOMATIK_BLIND.getType()))
      ait = PROGRAMM_AUTOMATIK_BLIND;
    else if (s.equals(PROGRAMM_DEAKTIVIERT_BLIND.getType()))
      ait = PROGRAMM_DEAKTIVIERT_BLIND;
    else if (s.equals(OK.getType()))
      ait = OK;
    else if (s.equals(TEILSTOERUNG.getType()))
      ait = TEILSTOERUNG;
    else
    {
      ait = INAKTIV_BI;
    }

    return ait;
  }

  public static AnzeigeInhaltTyp setBildInhalt(String s, Long time)
  {
    AnzeigeInhaltTyp ait = null;

    if (hasType(s))
    {
      ait = new AnzeigeInhaltTyp(s);

      if (time != null)
        ait.setZeit(time);

    }
    else
    {
      ait = new AnzeigeInhaltTyp(AnzeigeInhaltTyp.INAKTIV_BI.getType());
      System.out.println("AnzeigeInhaltTyp.setBildInhalt()-> else: " + s);
    }
    return ait;
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.bsvrz.buv.plugin.anlagendarstellung.status.Status#setStatus(java.lang.String)
   */
  public static AnzeigeInhaltTyp createNewInstance(AnzeigeInhaltTyp s)
  {
    // TODO Auto-generated method stub
    AnzeigeInhaltTyp ait = null;

    if (hasType(s.getType()))
    {
      ait = new AnzeigeInhaltTyp(s.getType());
    }

    // System.out.println("AnzeigeInhaltTyp.setBildInhalt()-> else: " + s);

    return ait;
  }

  private void setZeit(long time)
  {
    // TODO Auto-generated method stub
    _zeit = time;
  }

  public boolean hasTime()
  {
    return _zeit != null;
  }

  @Override
  public boolean equals(Object obj)
  {
    // TODO Auto-generated method stub

    if (obj instanceof AnzeigeInhaltTyp)
    {
      AnzeigeInhaltTyp that = (AnzeigeInhaltTyp)obj;
      return this.getType().equals(that.getType());
    }

    return false;
  }

}
