/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.model;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Model Object of a Picker, wird aus Kompatibilittsgrunden bentigt
 * 
 * @deprecated  
 * @author Pittner
 * 
 */
public class Picker extends PropertyAwareObject
{

  private Rectangle _bounds = null;

  private List _sourceConnections = new ArrayList();

  private List _targetConnections = new ArrayList();

  private FahrstreifenTyp _fsTyp;

  private PickerTyp _pickerTyp;

  private int _x = 0;

  private int _y = 0;

  private int _width = 0;

  private int _height = 0;

  private String _viewId;

  public Picker()
  {
    super();
  }

  public Picker(FahrstreifenTyp fsTyp, PickerTyp pTyp, int x, int y, int width, int heigt, String viewId)
  {

    this._fsTyp = fsTyp;
    this._pickerTyp = pTyp;
    this._x = x;
    this._y = y;
    this._width = width;
    this._height = heigt;
    this._viewId = viewId;

  }

  /**
   * Returns the persistent position and size of the node. The edit part handling this node has the task to translate it
   * into a constraint that enables the draw2D system to place the figure of the node. Therefore, the coordinate system
   * of the rectangle is known only to the edit part (but in fact, in this example it is always the local coordinate
   * system of the figure of the node).
   */
  public Rectangle getConstraint()
  {

    Rectangle rect = (Rectangle)getProperty("constraint");
    return rect;
  }

  /**
   * Sets the persistent position and size of the node.
   * 
   * @param constraint
   *          The data that enables the edit part handling this node to create a constraint that enables the draw2D
   *          system to place the figure of the node.
   */
  public void setConstraint(Rectangle constraint)
  {
    if (constraint != null)
    {
      _y = constraint.y;      
    }

    setProperty("constraint", constraint);

  }

  /**
   * Sets bounds without firing off any event notifications
   * 
   * @param bounds
   *          The bounds to set.
   */
  public void setBounds(Rectangle bounds)
  {
    this._bounds = bounds;
  }

  /**
   * Adds relationship where the current object is the foreign key table in a relationship
   * 
   * @param table
   *          the primary key relationship
   */
  public void addTargetConnections(PickerLinkElement table)
  {
    _targetConnections.add(table);
    // firePropertyChange(OUTPUT, null, table);
  }

  /**
   * Adds relationship where the current object is the primary key table in a relationship
   * 
   * @param table
   *          the foreign key relationship
   */
  public void addSourceConnections(PickerLinkElement table)
  {
    _sourceConnections.add(table);
    // firePropertyChange(INPUT, null, table);
  }

  /**
   * Removes relationship where the current object is the foreign key table in a relationship
   * 
   * @param table
   *          the primary key relationship
   */
  public void removeSourceConnections(PickerLinkElement table)
  {
    _sourceConnections.remove(table);
    // firePropertyChange(OUTPUT, table, null);
  }

  /**
   * Removes relationship where the current object is the primary key table in a relationship
   * 
   * @param table
   *          the foreign key relationship
   */
  public void removeTargetConnections(PickerLinkElement table)
  {
    _targetConnections.remove(table);
    // firePropertyChange(INPUT, table, null);
  }

  /**
   * If modified, sets bounds and fires off event notification
   * 
   * @param bounds
   *          The bounds to set.
   */
  public void modifyBounds(Rectangle bounds)
  {
    Rectangle oldBounds = this._bounds;
    if (!bounds.equals(oldBounds))
    {
      this._bounds = bounds;
      // firePropertyChange(BOUNDS, null, bounds);
    }
  }

  /**
   * @return Returns the foreignKeyRelationships.
   */
  public List getTargetConnections()
  {
    return _targetConnections;
  }

  /**
   * @return Returns the primaryKeyRelationships.
   */
  public List getSourceConnections()
  {
    return _sourceConnections;
  }

  /**
   * @return Returns the bounds.
   */
  public Rectangle getBounds()
  {
    return _bounds;
  }

  public int getX()
  {
    return _x;
  }

  public int getY()
  {
    return _y;
  }

  public FahrstreifenTyp getFsTyp()
  {
    return _fsTyp;
  }

  public void setFsTyp(FahrstreifenTyp fsTyp)
  {
    _fsTyp = fsTyp;
  }

  public void setY(int y)
  {
    // TODO Auto-generated method stub
    _y = y;
  }

  public PickerTyp getPickerTyp()
  {
    return _pickerTyp;
  }

  public void setPickerTyp(PickerTyp pickerTyp)
  {
    _pickerTyp = pickerTyp;
  }

  public int getWidth()
  {
    return _width;
  }

  public int getHeight()
  {
    return _height;
  }

  public String getViewId()
  {
    return _viewId;
  }

}
