/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.preferences;

import java.util.List;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaEinstellungen;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Fhrt die Initialisierung der Default-Parameter der Prferenzen dieses Plug-Ins durch.
 * 
 * @author Pittner
 * 
 */
public class PreferenceInitializer extends AbstractPreferenceInitializer
{
  @Override
  public void initializeDefaultPreferences()
  {
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();

    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    String s = "";
    if (connection != null)
    {
      SystemObjectType type = connection.getDataModel().getType("typ.sswSbaStreckenBeeinflussungsAnlage");
      if (type != null)
      {
        List<SystemObject> elements = type.getElements();
        if (elements != null && elements.size() > 0)
        {
          // Die Pid der ersten SBA verwenden
          s = elements.get(0).getPid();
        }
      }
    }

    
    store.setDefault(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE, s);

    Ansicht ansicht = SbaEinstellungen.INSTANCE.getAnsicht(0);

    String s2 = "";
    if (ansicht != null)
    {
      s2 = ansicht.getName();
    }
    
    store.setDefault(DefsPreference.P_DARSTELLUNG_ZUSTAND_IST, s2);
  
    store.setDefault(DefsPreference.P_MENUE_ZUSTAND_IST, "");

    store.setDefault(DefsPreference.P_DARSTELLUNG_ZUSTAND_SOLL, s2);

    store.setDefault(DefsPreference.P_MENUE_ZUSTAND_SOLL, "");

    store.setDefault(DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_NO, s2);

    store.setDefault(DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_NO, 18.0);

    store.setDefault(DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_NO, 0.0);

    store.setDefault(DefsPreference.P_MENUE_SONDER_PROGRAMM_NO, "");

    store.setDefault(DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_SW, s2);

    store.setDefault(DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_SW, 0.0);

    store.setDefault(DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_SW, 18.0);

    store.setDefault(DefsPreference.P_MENUE_SONDER_PROGRAMM_SW, "");

    store.setDefault(DefsPreference.P_DARSTELLUNG_PARAMETER, s2);

    store.setDefault(DefsPreference.P_USE_ZUSATZ_PRUEFUNG_SONDER_PRG, true);
    
    store.setDefault(DefsPreference.P_FR_DARSTELLUNG_PARAMETER_VON_OBEN_NACH_UNTEN, "SW");
    
    store.setDefault(DefsPreference.P_FR_DARSTELLUNG_PARAMETER_VON_UNTEN_NACH_OBEN, "NO");
    
    store.setDefault(DefsPreference.P_USE_SEMI_AUTOMATIC_URSACHEN_EINHEIT, true);
  }
}
