/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.preferences;

import java.util.ArrayList;
import java.util.Formatter;
import java.util.List;

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ListDialog;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaEinstellungen;
import de.bsvrz.buv.plugin.anlagendarstellung.semi.DialogSemiGuiManager;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungSonderProgramm.ListLabelProvider;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * {@link PreferencePage} des Plug-Ins Anlagendarstellung. Es kann:
 * <ul>
 * <li>einstellen, unter welcher StreckenBeeinflussungsAnlage die Zustandsdarstellung arbeitet.</li>
 * <li>einstellen, welche Darstellung fr den IST-Zustand verwendet werden soll.</li>
 * <li>einstellen, welche Bezeichnung fr den Menpunkt der IST-Darstellung verwendet werden soll.</li>
 * <li>einstellen, welche Darstellung fr den SOLL-Zustand verwendet werden soll.</li>
 * <li>einstellen, welche Bezeichnung fr den Menpunkt der SOLL-Darstellung verwendet werden soll.</li>
 * <li>einstellen, welche Darstellung fr die Sonderprogramm-Darstellung in FR No verwendet werden soll.</li>
 * <li>einstellen, welche Bezeichnung fr den Menpunkt der Sonderprogramm-Darstellung in FR No verwendet werden soll.</li>
 * <li>einstellen, welche Darstellung fr die Sonderprogramm-Darstellung in FR Sw verwendet werden soll.</li>
 * <li>einstellen, welche Bezeichnung fr den Menpunkt der Sonderprogramm-Darstellung in FR Sw verwendet werden soll.</li>
 * <li>einstellen, welche Darstellung fr die Parameter verwendet werden soll.</li>
 * </ul>
 * 
 * @author Pittner
 * 
 */
public class PreferencePageAnlagenDarstellung extends PreferencePage implements IWorkbenchPreferencePage
{
  /**
   * ID dieser PreferencePage
   */
  public static final String ID = "de.bsvrz.buv.plugin.anlagendarstellung.pageAnlagenDarstellung";

  
  private Group _groupStreckenBeeinflussungsAnlage;
  private Text _textStreckenBeeinflussungsAnlage;
  private Button _buttonStreckenBeeinflussungsAnlage;
  
  private Group _groupIstZustand;
  private Text _textDarstellungIstZustand;
  private Button _buttonIstZustand;
  private Text _textMenuIstZustand;

  private Group _groupSollZustand;
  private Text _textDarstellungSollZustand;
  private Button _buttonSollZustand;
  private Text _textMenuSollZustand;

  private Group _groupSonderProgrammNo;
  private Text _textDarstellungSonderProgrammNo;
  private Button _buttonSonderProgrammNo;
  private Text _textKmSonderProgrammNoVon;
  private Text _textKmSonderProgrammNoBis;
  private Text _textMenuSonderProgrammNo;
  
  private Group _groupSonderProgrammSw;
  private Text _textDarstellungSonderProgrammSw;
  private Button _buttonSonderProgrammSw;
  private Text _textKmSonderProgrammSwVon;
  private Text _textKmSonderProgrammSwBis;
  private Text _textMenuSonderProgrammSw;

  private Group _groupParameter;
  private Text _textDarstellungParameter;
  private Button _buttonParameter;
  
  private Group _groupZusatzPruefung;
  private Button _checkZusatzPruefung;
  private Group _groupSemiAutomatic;
  private Button _checkSemiAutomatic;
   
  private IDaVVerbindungZuhoerer _verbindungZuhoerer;

  private Combo _comboParameter;
  private Combo _comboParameter2;


  /**
   * Konstruktor. Setzt {@link PreferenceStore} und Titel dieser {@link PreferencePage}.
   */
  public PreferencePageAnlagenDarstellung()
  {
    super();
    setPreferenceStore(Activator.getDefault().getPreferenceStore());
    setDescription("Prferenzen des Anlagenzustandes");
  }

  @Override
  protected Control createContents(Composite parent)
  {
    Composite top = new Composite(parent, SWT.NONE);
    
	PlatformUI.getWorkbench().getHelpSystem().setHelp(top,
			Activator.PLUGIN_ID + ".zustandsdarstellungEinstellungen");

    createGroupStreckenBeeinflussungsAnlage(top);
    createGroupIstZustand(top);
    createGroupSollZustand(top);
    createGroupSonderProgrammNo(top);
    createGroupSonderprogrammSw(top);
    createGroupParameter(top);
    createGroupZusatzPruefung(top);
    createGroupSemiAutomatic(top);

    actionLoad();

    /*
     * TabList
     */
    Control[] tabList = new Control[] { _groupStreckenBeeinflussungsAnlage, _groupZusatzPruefung, _groupSemiAutomatic };
    top.setTabList(tabList);

    /*
     * Layout
     */
    GridLayout glTop = new GridLayout(1, false);
    top.setLayout(glTop);

    GridData gdTop = new GridData();
    gdTop.grabExcessHorizontalSpace = true;
    gdTop.horizontalAlignment = GridData.FILL;
    gdTop.grabExcessVerticalSpace = true;
    gdTop.verticalAlignment = GridData.FILL;
    top.setLayoutData(gdTop);

    GridData gdConfArea = new GridData();
    gdConfArea.grabExcessHorizontalSpace = true;
    gdConfArea.horizontalAlignment = GridData.FILL;
    _groupStreckenBeeinflussungsAnlage.setLayoutData(gdConfArea);

    GridData gdConfArea2 = new GridData();
    gdConfArea2.grabExcessHorizontalSpace = true;
    gdConfArea2.horizontalAlignment = GridData.FILL;
    _groupIstZustand.setLayoutData(gdConfArea2);

    GridData gdConfArea3 = new GridData();
    gdConfArea3.grabExcessHorizontalSpace = true;
    gdConfArea3.horizontalAlignment = GridData.FILL;
    _groupSollZustand.setLayoutData(gdConfArea3);

    GridData gdConfArea4 = new GridData();
    gdConfArea4.grabExcessHorizontalSpace = true;
    gdConfArea4.horizontalAlignment = GridData.FILL;
    _groupSonderProgrammNo.setLayoutData(gdConfArea4);

    GridData gdConfArea5 = new GridData();
    gdConfArea5.grabExcessHorizontalSpace = true;
    gdConfArea5.horizontalAlignment = GridData.FILL;
    _groupSonderProgrammSw.setLayoutData(gdConfArea5);

    GridData gdConfArea6 = new GridData();
    gdConfArea6.grabExcessHorizontalSpace = true;
    gdConfArea6.horizontalAlignment = GridData.FILL;
    _groupParameter.setLayoutData(gdConfArea6);

    GridData gdGroupMisc = new GridData();
    gdGroupMisc.grabExcessHorizontalSpace = true;
    gdGroupMisc.horizontalAlignment = GridData.FILL;
    gdGroupMisc.grabExcessVerticalSpace = true;
    gdGroupMisc.verticalAlignment = GridData.FILL;
    _groupZusatzPruefung.setLayoutData(gdGroupMisc);
    
    GridData gdGroupMisc2 = new GridData();
    gdGroupMisc2.grabExcessHorizontalSpace = true;
    gdGroupMisc2.horizontalAlignment = GridData.FILL;
    gdGroupMisc2.grabExcessVerticalSpace = true;
    gdGroupMisc2.verticalAlignment = GridData.FILL;
    _groupSemiAutomatic.setLayoutData(gdGroupMisc2);

    DaVVerbindung daVVerbindung = DaVVerbindung.getInstanz();
    if (!daVVerbindung.istDaVVerbindungAktiv())
    {
      _buttonStreckenBeeinflussungsAnlage.setEnabled(false);
    }

    _verbindungZuhoerer = new IDaVVerbindungZuhoerer() {

      @Override
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _buttonStreckenBeeinflussungsAnlage.setEnabled(false);

      }

      @Override
      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        _buttonStreckenBeeinflussungsAnlage.setEnabled(true);
      }

    };
    daVVerbindung.hinzufuegenZuhoerer(_verbindungZuhoerer);

    return null;
  }

  
  private Group createGroupStreckenBeeinflussungsAnlage(Composite parent)
  {
    if (_groupStreckenBeeinflussungsAnlage == null)
    {
      _groupStreckenBeeinflussungsAnlage = new Group(parent, SWT.NONE);
      _groupStreckenBeeinflussungsAnlage.setText("StreckenBeeinflussungsAnlage:");

      _textStreckenBeeinflussungsAnlage = new Text(_groupStreckenBeeinflussungsAnlage, SWT.BORDER);
      _buttonStreckenBeeinflussungsAnlage = new Button(_groupStreckenBeeinflussungsAnlage, SWT.PUSH);
      _buttonStreckenBeeinflussungsAnlage.setText("whlen");
      _buttonStreckenBeeinflussungsAnlage.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          actionSelectStreckenBeeinflussungsAnlage();
        }
      });

      /*
       * TabList
       */
      Control[] tabList = new Control[] { _textStreckenBeeinflussungsAnlage };
      _groupStreckenBeeinflussungsAnlage.setTabList(tabList);

      /*
       * Layout
       */
      GridLayout glConfArea = new GridLayout(2, false);
      _groupStreckenBeeinflussungsAnlage.setLayout(glConfArea);

      GridData gdTextConfArea = new GridData();
      gdTextConfArea.grabExcessHorizontalSpace = true;
      gdTextConfArea.horizontalAlignment = GridData.FILL;
      gdTextConfArea.verticalAlignment = GridData.FILL;
      _textStreckenBeeinflussungsAnlage.setLayoutData(gdTextConfArea);

    }

    return _groupStreckenBeeinflussungsAnlage;
  }

  /**
   * Aktion hinter dem Button auswhlen. Es wird ein Dialog zur Auswahl der StreckenBeeinflussungsAnlage dargestellt. 
   */
  private void actionSelectStreckenBeeinflussungsAnlage()
  {

    ListDialog listDialog = new ListDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell()) {
      protected int getTableStyle()
      {
        return SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER;
      }
      
      @Override
      protected void configureShell(Shell shell) {
    	  // Fr die F1 Taste
          PlatformUI.getWorkbench().getHelpSystem().setHelp(shell, Activator.PLUGIN_ID + ".zustandsdarstellungEinstellungen");
    	  
    	  super.configureShell(shell);
      }
    };
    listDialog.setTitle("Auswahl Anlage");
    listDialog.setMessage("Folgende Anlagen stehen zur Auswahl:");
    listDialog.setContentProvider(new ArrayContentProvider());
    listDialog.setLabelProvider(new ListLabelProvider());

    SystemObjectType type = DaVVerbindung.getInstanz().getDaVVerbindung().getDataModel()
        .getType("typ.sswSbaStreckenBeeinflussungsAnlage");

    if (type != null)
    {
      List<SystemObject> anlagen = type.getElements();
      if (anlagen != null)
        listDialog.setInput(anlagen);
    }

    if (listDialog.open() == ListDialog.OK)
    {
      Object[] results = listDialog.getResult();
      if ((results != null) && (results.length > 0))
      {
        _textStreckenBeeinflussungsAnlage.setText(((SystemObject)results[0]).getPid());
      }
    }
  }

  /**
   * Aktion hinter dem Button auswhlen. Es wird ein Dialog zur Auswahl der Konfigurationsbereiche dargestellt. Ist
   * bereits im Textfeld bereits ein Konfigurationsbereich (der im DaV auch existieren muss) angegeben, so wird dieser
   * im Auswahldialog selektiert hervorgehoben.
   */
  private void actionSelectDarstellung(Text text)
  {

    ListDialog listDialog = new ListDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell()) {
      protected int getTableStyle()
      {
        return SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER;
      }
      
      @Override
      protected void configureShell(Shell shell) {
    	  // Fr die F1 Taste
          PlatformUI.getWorkbench().getHelpSystem().setHelp(shell, Activator.PLUGIN_ID + ".zustandsdarstellungEinstellungen");
    	  
    	  super.configureShell(shell);
      }
    };
    listDialog.setTitle("Auswahl Darstellungen");
    listDialog.setMessage("Folgende Darstellungen stehen zur Auswahl:");
    listDialog.setContentProvider(new ArrayContentProvider());
    listDialog.setLabelProvider(new AnsichtLabelProvider());

    List<Ansicht> ansichten = SbaEinstellungen.INSTANCE.getAnsichten();

    List<Ansicht> list = new ArrayList<Ansicht>();
    if (ansichten != null)
    {

      for (Ansicht ansicht : ansichten)
      {
        Darstellung darstellung = ansicht.getDarstellung();
        if (darstellung instanceof StilisierteDarstellung)
        {
          list.add(ansicht);

        }
      }
      listDialog.setInput(list);
    }

    if (listDialog.open() == ListDialog.OK)
    {
      Object[] results = listDialog.getResult();
      if ((results != null) && (results.length > 0))
      {
        text.setText(((Ansicht)results[0]).getName());
      }
    }
  }

  private Group createGroupZusatzPruefung(Composite parent)
  {
    if (_groupZusatzPruefung == null)
    {
      _groupZusatzPruefung = new Group(parent, SWT.NONE);
      _groupZusatzPruefung.setText("Sonderprogramme:");

      _checkZusatzPruefung = new Button(_groupZusatzPruefung, SWT.CHECK);
      _checkZusatzPruefung.setText("Aktiviere Zusatzprfung");

      /*
       * TabList
       */
      Control[] tabList = new Control[] { _checkZusatzPruefung };
      _groupZusatzPruefung.setTabList(tabList);

      /*
       * Layout
       */
      GridLayout glMisc = new GridLayout(1, false);
      _groupZusatzPruefung.setLayout(glMisc);
    }

    return _groupZusatzPruefung;
  }
  
  private Group createGroupSemiAutomatic(Composite parent)
  {
    if (_groupSemiAutomatic == null)
    {
      _groupSemiAutomatic = new Group(parent, SWT.NONE);
      _groupSemiAutomatic.setText("Semi-Automatik:");
      
      _checkSemiAutomatic = new Button(_groupSemiAutomatic, SWT.CHECK);
      _checkSemiAutomatic.setText("Vorschlag Anzeigen");
      
      /*
       * TabList
       */
      Control[] tabList = new Control[] { _checkSemiAutomatic };
      _groupSemiAutomatic.setTabList(tabList);
      
      /*
       * Layout
       */
      GridLayout glMisc = new GridLayout(1, false);
      _groupSemiAutomatic.setLayout(glMisc);
    }
    
    return _groupZusatzPruefung;
  }

  private Group createGroupIstZustand(Composite parent)
  {
    if (_groupIstZustand == null)
    {
      _groupIstZustand = new Group(parent, SWT.NONE);
      _groupIstZustand.setText("Ist-Darstellung:");

      Label labelConfArea2 = new Label(_groupIstZustand, SWT.NONE);
      labelConfArea2.setText("Darstellung");
      _textDarstellungIstZustand = new Text(_groupIstZustand, SWT.BORDER);
      _buttonIstZustand = new Button(_groupIstZustand, SWT.PUSH);
      _buttonIstZustand.setText("whlen");
      _buttonIstZustand.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          actionSelectDarstellung(_textDarstellungIstZustand);
        }
      });

      Composite compConfArea21 = new Composite(_groupIstZustand, SWT.NONE);
      Label labelConfArea21 = new Label(compConfArea21, SWT.NONE);
      labelConfArea21.setText("Meneintrag");

      _textMenuIstZustand = new Text(compConfArea21, SWT.BORDER);

      /*
       * TabList
       */
      Control[] tabList = new Control[] { _textDarstellungIstZustand };
      _groupIstZustand.setTabList(tabList);

      /*
       * TabList2
       */
      Control[] tabList2 = new Control[] { _textMenuIstZustand };
      compConfArea21.setTabList(tabList2);

      /*
       * Layout
       */
      GridLayout glConfArea21 = new GridLayout(2, false);
      compConfArea21.setLayout(glConfArea21);
      GridData gdTextConfArea21 = new GridData();
      gdTextConfArea21.horizontalSpan = 3;
      gdTextConfArea21.grabExcessHorizontalSpace = true;
      gdTextConfArea21.horizontalAlignment = GridData.FILL;
      gdTextConfArea21.verticalAlignment = GridData.FILL;
      compConfArea21.setLayoutData(gdTextConfArea21);

      GridData gdTextConfArea = new GridData();
      gdTextConfArea.grabExcessHorizontalSpace = true;
      gdTextConfArea.horizontalAlignment = GridData.FILL;
      gdTextConfArea.verticalAlignment = GridData.FILL;
      _textDarstellungIstZustand.setLayoutData(gdTextConfArea);
      _textMenuIstZustand.setLayoutData(gdTextConfArea);

      GridLayout glConfArea = new GridLayout(3, false);
      _groupIstZustand.setLayout(glConfArea);
    }

    return _groupIstZustand;
  }

  private Group createGroupSollZustand(Composite parent)
  {
    if (_groupSollZustand == null)
    {
      _groupSollZustand = new Group(parent, SWT.NONE);
      _groupSollZustand.setText("Soll-Darstellung:");

      Label labelConfArea3 = new Label(_groupSollZustand, SWT.NONE);
      labelConfArea3.setText("Darstellung");
      _textDarstellungSollZustand = new Text(_groupSollZustand, SWT.BORDER);
      _buttonSollZustand = new Button(_groupSollZustand, SWT.PUSH);
      _buttonSollZustand.setText("whlen");
      _buttonSollZustand.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          actionSelectDarstellung(_textDarstellungSollZustand);
        }
      });

      Composite compConfArea31 = new Composite(_groupSollZustand, SWT.NONE);
      Label labelConfArea31 = new Label(compConfArea31, SWT.NONE);
      labelConfArea31.setText("Meneintrag");

      _textMenuSollZustand = new Text(compConfArea31, SWT.BORDER);

      /*
       * TabList
       */
      Control[] tabList = new Control[] { _textDarstellungSollZustand };
      _groupSollZustand.setTabList(tabList);

      /*
       * TabList2
       */
      Control[] tabList2 = new Control[] { _textMenuSollZustand };
      compConfArea31.setTabList(tabList2);

      /*
       * Layout
       */
      GridLayout glConfArea31 = new GridLayout(2, false);
      compConfArea31.setLayout(glConfArea31);
      GridData gdTextConfArea31 = new GridData();
      gdTextConfArea31.horizontalSpan = 3;
      gdTextConfArea31.grabExcessHorizontalSpace = true;
      gdTextConfArea31.horizontalAlignment = GridData.FILL;
      gdTextConfArea31.verticalAlignment = GridData.FILL;
      compConfArea31.setLayoutData(gdTextConfArea31);

      GridLayout glConfArea = new GridLayout(3, false);
      _groupSollZustand.setLayout(glConfArea);

      GridData gdTextConfArea = new GridData();
      gdTextConfArea.grabExcessHorizontalSpace = true;
      gdTextConfArea.horizontalAlignment = GridData.FILL;
      gdTextConfArea.verticalAlignment = GridData.FILL;
      _textDarstellungSollZustand.setLayoutData(gdTextConfArea);
      _textMenuSollZustand.setLayoutData(gdTextConfArea);

    }

    return _groupSollZustand;
  }

  private Group createGroupSonderProgrammNo(Composite parent)
  {
    if (_groupSonderProgrammNo == null)
    {
      _groupSonderProgrammNo = new Group(parent, SWT.NONE);
      _groupSonderProgrammNo.setText("Sonderprogramm-Darstellung FR NO:");

      Label labelConfArea4 = new Label(_groupSonderProgrammNo, SWT.NONE);
      labelConfArea4.setText("Darstellung");
      _textDarstellungSonderProgrammNo = new Text(_groupSonderProgrammNo, SWT.BORDER);
      _buttonSonderProgrammNo = new Button(_groupSonderProgrammNo, SWT.PUSH);
      _buttonSonderProgrammNo.setText("whlen");
      _buttonSonderProgrammNo.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          actionSelectDarstellung(_textDarstellungSonderProgrammNo);
        }
      });

      Composite groupStreckenbereich = new Composite(_groupSonderProgrammNo, SWT.NONE);

      Label label21 = new Label(groupStreckenbereich, SWT.NONE);
      label21.setText("startKm");

      String s = new Formatter().format("%1$7.3f", new Double(0)).toString();

      _textKmSonderProgrammNoVon = new Text(groupStreckenbereich, SWT.BORDER | SWT.RIGHT);
      _textKmSonderProgrammNoVon.setTextLimit(7);
      _textKmSonderProgrammNoVon.setText(s);

      Label label22 = new Label(groupStreckenbereich, SWT.NONE);
      label22.setText("endeKm");

      _textKmSonderProgrammNoBis = new Text(groupStreckenbereich, SWT.BORDER | SWT.RIGHT);
      _textKmSonderProgrammNoBis.setTextLimit(7);
      _textKmSonderProgrammNoBis.setText(s);

      Composite compConfArea41 = new Composite(_groupSonderProgrammNo, SWT.NONE);
      Label labelConfArea41 = new Label(compConfArea41, SWT.NONE);
      labelConfArea41.setText("Meneintrag");

      _textMenuSonderProgrammNo = new Text(compConfArea41, SWT.BORDER);

      /*
       * TabList
       */
      Control[] tabList = new Control[] { _textDarstellungSonderProgrammNo };
      _groupSonderProgrammNo.setTabList(tabList);

      /*
       * TabList2
       */
      Control[] tabList2 = new Control[] { _textKmSonderProgrammNoVon, _textKmSonderProgrammNoBis };
      groupStreckenbereich.setTabList(tabList2);

      /*
       * TabList3
       */
      Control[] tabList3 = new Control[] { _textMenuSonderProgrammNo };
      compConfArea41.setTabList(tabList3);

      /*
       * Layout
       */
      GridLayout glConfArea41 = new GridLayout(2, false);
      compConfArea41.setLayout(glConfArea41);
      GridData gdTextConfArea41 = new GridData();
      gdTextConfArea41.horizontalSpan = 3;
      gdTextConfArea41.grabExcessHorizontalSpace = true;
      gdTextConfArea41.horizontalAlignment = GridData.FILL;
      gdTextConfArea41.verticalAlignment = GridData.FILL;
      compConfArea41.setLayoutData(gdTextConfArea41);

      GridData gridData11 = new GridData();
      gridData11.verticalAlignment = GridData.CENTER;
      gridData11.horizontalAlignment = GridData.FILL;
      GridData gridData15 = new GridData();
      gridData15.horizontalAlignment = GridData.FILL;
      GridData gridData18 = new GridData();
      gridData18.horizontalSpan = 3;
      gridData18.horizontalAlignment = GridData.FILL;
      gridData18.grabExcessHorizontalSpace = true;
      gridData18.grabExcessVerticalSpace = false;
      gridData18.verticalAlignment = GridData.CENTER;

      GridLayout gridLayout13 = new GridLayout();
      gridLayout13.numColumns = 4;
      gridLayout13.makeColumnsEqualWidth = false;

      groupStreckenbereich.setLayoutData(gridData18);
      groupStreckenbereich.setLayout(gridLayout13);
      label21.setLayoutData(gridData11);
      _textKmSonderProgrammNoVon.setLayoutData(gridData15);
      label22.setLayoutData(gridData11);
      _textKmSonderProgrammNoBis.setLayoutData(gridData15);

      GridLayout glConfArea = new GridLayout(3, false);
      _groupSonderProgrammNo.setLayout(glConfArea);

      GridData gdTextConfArea = new GridData();
      gdTextConfArea.grabExcessHorizontalSpace = true;
      gdTextConfArea.horizontalAlignment = GridData.FILL;
      gdTextConfArea.verticalAlignment = GridData.FILL;
      _textDarstellungSonderProgrammNo.setLayoutData(gdTextConfArea);
      _textMenuSonderProgrammNo.setLayoutData(gdTextConfArea);

    }

    return _groupSonderProgrammNo;
  }

  private Group createGroupSonderprogrammSw(Composite parent)
  {
    if (_groupSonderProgrammSw == null)
    {
      _groupSonderProgrammSw = new Group(parent, SWT.NONE);
      _groupSonderProgrammSw.setText("Sonderprogramm-Darstellung FR SW:");

      Label labelConfArea5 = new Label(_groupSonderProgrammSw, SWT.NONE);
      labelConfArea5.setText("Darstellung");
      _textDarstellungSonderProgrammSw = new Text(_groupSonderProgrammSw, SWT.BORDER);
      _buttonSonderProgrammSw = new Button(_groupSonderProgrammSw, SWT.PUSH);
      _buttonSonderProgrammSw.setText("whlen");
      _buttonSonderProgrammSw.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          actionSelectDarstellung(_textDarstellungSonderProgrammSw);
        }
      });

      Composite groupStreckenbereich = new Composite(_groupSonderProgrammSw, SWT.NONE);

      Label label21 = new Label(groupStreckenbereich, SWT.NONE);
      label21.setText("startKm");

      String s = new Formatter().format("%1$7.3f", new Double(0)).toString();

      _textKmSonderProgrammSwVon = new Text(groupStreckenbereich, SWT.BORDER | SWT.RIGHT);
      _textKmSonderProgrammSwVon.setTextLimit(7);
      _textKmSonderProgrammSwVon.setText(s);

      Label label22 = new Label(groupStreckenbereich, SWT.NONE);
      label22.setText("endeKm");

      _textKmSonderProgrammSwBis = new Text(groupStreckenbereich, SWT.BORDER | SWT.RIGHT);
      _textKmSonderProgrammSwBis.setTextLimit(7);
      _textKmSonderProgrammSwBis.setText(s);

      Composite compConfArea51 = new Composite(_groupSonderProgrammSw, SWT.NONE);
      Label labelConfArea51 = new Label(compConfArea51, SWT.NONE);
      labelConfArea51.setText("Meneintrag");

      _textMenuSonderProgrammSw = new Text(compConfArea51, SWT.BORDER);

      /*
       * TabList
       */
      Control[] tabList = new Control[] { _textDarstellungSonderProgrammSw };
      _groupSonderProgrammSw.setTabList(tabList);

      /*
       * TabList2
       */
      Control[] tabList2 = new Control[] { _textKmSonderProgrammSwVon, _textKmSonderProgrammSwBis };
      groupStreckenbereich.setTabList(tabList2);

      /*
       * TabList3
       */
      Control[] tabList3 = new Control[] { _textMenuSonderProgrammSw };
      compConfArea51.setTabList(tabList3);

      /*
       * Layout
       */
      GridLayout glConfArea51 = new GridLayout(2, false);
      compConfArea51.setLayout(glConfArea51);
      GridData gdTextConfArea51 = new GridData();
      gdTextConfArea51.horizontalSpan = 3;
      gdTextConfArea51.grabExcessHorizontalSpace = true;
      gdTextConfArea51.horizontalAlignment = GridData.FILL;
      gdTextConfArea51.verticalAlignment = GridData.FILL;
      compConfArea51.setLayoutData(gdTextConfArea51);

      GridData gridData11 = new GridData();
      gridData11.verticalAlignment = GridData.CENTER;
      gridData11.horizontalAlignment = GridData.FILL;
      GridData gridData15 = new GridData();
      gridData15.horizontalAlignment = GridData.FILL;
      GridData gridData18 = new GridData();
      gridData18.horizontalSpan = 3;
      gridData18.horizontalAlignment = GridData.FILL;
      gridData18.grabExcessHorizontalSpace = true;
      gridData18.grabExcessVerticalSpace = false;
      gridData18.verticalAlignment = GridData.CENTER;

      GridLayout gridLayout13 = new GridLayout();
      gridLayout13.numColumns = 4;
      gridLayout13.makeColumnsEqualWidth = false;

      groupStreckenbereich.setLayoutData(gridData18);
      groupStreckenbereich.setLayout(gridLayout13);
      label21.setLayoutData(gridData11);
      _textKmSonderProgrammSwVon.setLayoutData(gridData15);
      label22.setLayoutData(gridData11);
      _textKmSonderProgrammSwBis.setLayoutData(gridData15);

      GridLayout glConfArea = new GridLayout(3, false);
      _groupSonderProgrammSw.setLayout(glConfArea);

      GridData gdTextConfArea = new GridData();
      gdTextConfArea.grabExcessHorizontalSpace = true;
      gdTextConfArea.horizontalAlignment = GridData.FILL;
      gdTextConfArea.verticalAlignment = GridData.FILL;
      _textDarstellungSonderProgrammSw.setLayoutData(gdTextConfArea);
      _textMenuSonderProgrammSw.setLayoutData(gdTextConfArea);

    }

    return _groupSonderProgrammSw;
  }

  private Group createGroupParameter(Composite parent)
  {
    if (_groupParameter == null)
    {
      _groupParameter = new Group(parent, SWT.NONE);
      _groupParameter.setText("Parameter-Darstellung:");

      _textDarstellungParameter = new Text(_groupParameter, SWT.BORDER);
      _buttonParameter = new Button(_groupParameter, SWT.PUSH);
      _buttonParameter.setText("whlen");
      _buttonParameter.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          actionSelectDarstellung(_textDarstellungParameter);
        }
      });
      
      

      Composite compositeGroupParameter = new Composite(_groupParameter, SWT.NONE);
      String[] ss = new String[]{
        "NO", "SW"
        };
          
      
      Label labelDarstellungParameter2 = new Label(compositeGroupParameter, SWT.NONE);
      labelDarstellungParameter2.setText("FR von oben nach unten");
      _comboParameter = new Combo(compositeGroupParameter, SWT.READ_ONLY);
      _comboParameter.setItems(ss);
      _comboParameter.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          
        }
      });
      
      Label labelDarstellungParameter = new Label(compositeGroupParameter, SWT.NONE);
      labelDarstellungParameter.setText("FR von unten nach oben");
      _comboParameter2 = new Combo(compositeGroupParameter, SWT.READ_ONLY);
      _comboParameter2.setItems(ss);
      _comboParameter2.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e)
        {
          
        }
      });

      /*
       * TabList
       */
      Control[] tabList = new Control[] { _textDarstellungParameter };
      _groupParameter.setTabList(tabList);

      /*
       * Layout
       */
      GridLayout glConfArea = new GridLayout(2, false);
      _groupParameter.setLayout(glConfArea);
      GridLayout glConfArea2 = new GridLayout(2, false);
      compositeGroupParameter.setLayout(glConfArea2);

      GridData gdTextConfArea = new GridData();
      gdTextConfArea.grabExcessHorizontalSpace = true;
      gdTextConfArea.horizontalAlignment = GridData.FILL;
      gdTextConfArea.verticalAlignment = GridData.FILL;
      GridData gdTextConfArea0 = new GridData();
      gdTextConfArea0.grabExcessHorizontalSpace = true;
      gdTextConfArea0.horizontalAlignment = GridData.FILL;
      gdTextConfArea0.verticalAlignment = GridData.FILL;
      gdTextConfArea0.horizontalSpan = 2;
      GridData gdTextConfArea2 = new GridData();
      gdTextConfArea2.grabExcessHorizontalSpace = true;
      gdTextConfArea2.horizontalAlignment = GridData.FILL;
      gdTextConfArea2.verticalAlignment = GridData.FILL;
      GridData gdTextConfArea3 = new GridData();
      gdTextConfArea3.grabExcessHorizontalSpace = true;
      gdTextConfArea3.horizontalAlignment = GridData.FILL;
      gdTextConfArea3.verticalAlignment = GridData.FILL;
      compositeGroupParameter.setLayoutData(gdTextConfArea0);
      _textDarstellungParameter.setLayoutData(gdTextConfArea);
      _textDarstellungParameter.setLayoutData(gdTextConfArea);
      _comboParameter.setLayoutData(gdTextConfArea2);
      _comboParameter2.setLayoutData(gdTextConfArea3);
    }

    return _groupParameter;
  }

  /**
   * Ldt die Einstellungen aus dem {@link PreferenceStore}.
   */
  private void actionLoad()
  {
    /*
     * Konfigurationsbereich
     */
    String pidConfArea = getPreferenceStore().getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    _textStreckenBeeinflussungsAnlage.setText(pidConfArea);

    /*
     * Konfigurationsbereich
     */
    String pidConfArea2 = getPreferenceStore().getString(DefsPreference.P_DARSTELLUNG_ZUSTAND_IST);
    _textDarstellungIstZustand.setText(pidConfArea2);

    /*
     * Konfigurationsbereich
     */
    String menuConfArea2 = getPreferenceStore().getString(DefsPreference.P_MENUE_ZUSTAND_IST);
    _textMenuIstZustand.setText(menuConfArea2);

    /*
     * Konfigurationsbereich
     */
    String pidConfArea3 = getPreferenceStore().getString(DefsPreference.P_DARSTELLUNG_ZUSTAND_SOLL);
    _textDarstellungSollZustand.setText(pidConfArea3);

    /*
     * Konfigurationsbereich
     */
    String menuConfArea3 = getPreferenceStore().getString(DefsPreference.P_MENUE_ZUSTAND_SOLL);
    _textMenuSollZustand.setText(menuConfArea3);

    /*
     * Konfigurationsbereich
     */
    String pidConfArea4 = getPreferenceStore().getString(DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_NO);
    _textDarstellungSonderProgrammNo.setText(pidConfArea4);

    /*
     * Konfigurationsbereich
     */
    double d1 = getPreferenceStore().getDouble(DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_NO);
    String pidConfArea41 = new Formatter().format("%1$7.3f", new Double(d1)).toString();
    _textKmSonderProgrammNoVon.setText(pidConfArea41);

    /*
     * Konfigurationsbereich
     */
    double d2 = getPreferenceStore().getDouble(DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_NO);
    String pidConfArea42 = new Formatter().format("%1$7.3f", new Double(d2)).toString();
    _textKmSonderProgrammNoBis.setText(pidConfArea42);

    /*
     * Konfigurationsbereich
     */
    String menuConfArea4 = getPreferenceStore().getString(DefsPreference.P_MENUE_SONDER_PROGRAMM_NO);
    _textMenuSonderProgrammNo.setText(menuConfArea4);

    /*
     * Konfigurationsbereich
     */
    String pidConfArea5 = getPreferenceStore().getString(DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_SW);
    _textDarstellungSonderProgrammSw.setText(pidConfArea5);

    /*
     * Konfigurationsbereich
     */
    double d3 = getPreferenceStore().getDouble(DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_SW);
    String pidConfArea51 = new Formatter().format("%1$7.3f", new Double(d3)).toString();
    _textKmSonderProgrammSwVon.setText(pidConfArea51);

    /*
     * Konfigurationsbereich
     */
    double d4 = getPreferenceStore().getDouble(DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_SW);
    String pidConfArea52 = new Formatter().format("%1$7.3f", new Double(d4)).toString();
    _textKmSonderProgrammSwBis.setText(pidConfArea52);

    /*
     * Konfigurationsbereich
     */
    String menuConfArea5 = getPreferenceStore().getString(DefsPreference.P_MENUE_SONDER_PROGRAMM_SW);
    _textMenuSonderProgrammSw.setText(menuConfArea5);

    /*
     * Konfigurationsbereich
     */
    String pidConfArea6 = getPreferenceStore().getString(DefsPreference.P_DARSTELLUNG_PARAMETER);
    _textDarstellungParameter.setText(pidConfArea6);

    /*
     * Benutzer ohne Berechtigungsklasse
     */
    boolean userWithoutClass = getPreferenceStore().getBoolean(DefsPreference.P_USE_ZUSATZ_PRUEFUNG_SONDER_PRG);
    _checkZusatzPruefung.setSelection(userWithoutClass);
    
    /*
     * Benutzer ohne Berechtigungsklasse
     */
    String pidConfArea7 = getPreferenceStore().getString(DefsPreference.P_FR_DARSTELLUNG_PARAMETER_VON_OBEN_NACH_UNTEN);
    _comboParameter.setText(pidConfArea7);
    
    /*
     * Benutzer ohne Berechtigungsklasse
     */
    String pidConfArea8 = getPreferenceStore().getString(DefsPreference.P_FR_DARSTELLUNG_PARAMETER_VON_UNTEN_NACH_OBEN);
    _comboParameter2.setText(pidConfArea8);
    
    /*
     * Benutzer ohne Berechtigungsklasse
     */
    boolean userWithoutClass2 = getPreferenceStore().getBoolean(DefsPreference.P_USE_SEMI_AUTOMATIC_URSACHEN_EINHEIT);
    _checkSemiAutomatic.setSelection(userWithoutClass2);

  }

  /**
   * Ldt die Default-Einstellungen aus dem {@link PreferenceStore}.
   */
  private void actionRestoreDefaults()
  {
    /*
     * Konfigurationsbereich
     */
    String pidConfArea = getPreferenceStore().getDefaultString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    _textStreckenBeeinflussungsAnlage.setText(pidConfArea);

    /*
     * Konfigurationsbereich
     */
    String pidConfArea2 = getPreferenceStore().getDefaultString(DefsPreference.P_DARSTELLUNG_ZUSTAND_IST);
    _textDarstellungIstZustand.setText(pidConfArea2);

    /*
     * Konfigurationsbereich
     */
    String menuConfArea2 = getPreferenceStore().getDefaultString(DefsPreference.P_MENUE_ZUSTAND_IST);
    _textMenuIstZustand.setText(menuConfArea2);

    /*
     * Konfigurationsbereich
     */
    String pidConfArea3 = getPreferenceStore().getDefaultString(DefsPreference.P_DARSTELLUNG_ZUSTAND_SOLL);
    _textDarstellungSollZustand.setText(pidConfArea3);

    /*
     * Konfigurationsbereich
     */
    String menuConfArea3 = getPreferenceStore().getDefaultString(DefsPreference.P_MENUE_ZUSTAND_SOLL);
    _textMenuSollZustand.setText(menuConfArea3);

    /*
     * Konfigurationsbereich
     */
    String pidConfArea4 = getPreferenceStore().getDefaultString(DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_NO);
    _textDarstellungSonderProgrammNo.setText(pidConfArea4);

    /*
     * Konfigurationsbereich
     */
    double d1 = getPreferenceStore().getDefaultDouble(DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_NO);
    String pidConfArea41 = new Formatter().format("%1$7.3f", new Double(d1)).toString();
    _textKmSonderProgrammNoVon.setText(pidConfArea41);

    /*
     * Konfigurationsbereich
     */
    double d2 = getPreferenceStore().getDefaultDouble(DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_NO);
    String pidConfArea42 = new Formatter().format("%1$7.3f", new Double(d2)).toString();
    _textKmSonderProgrammNoBis.setText(pidConfArea42);

    /*
     * Konfigurationsbereich
     */
    String menuConfArea4 = getPreferenceStore().getDefaultString(DefsPreference.P_MENUE_SONDER_PROGRAMM_NO);
    _textMenuSonderProgrammNo.setText(menuConfArea4);

    /*
     * Konfigurationsbereich
     */
    String pidConfArea5 = getPreferenceStore().getDefaultString(DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_SW);
    _textDarstellungSonderProgrammSw.setText(pidConfArea5);

    /*
     * Konfigurationsbereich
     */
    double d3 = getPreferenceStore().getDefaultDouble(DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_SW);
    String pidConfArea51 = new Formatter().format("%1$7.3f", new Double(d3)).toString();
    _textKmSonderProgrammSwVon.setText(pidConfArea51);

    /*
     * Konfigurationsbereich
     */
    double d4 = getPreferenceStore().getDefaultDouble(DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_SW);
    String pidConfArea52 = new Formatter().format("%1$7.3f", new Double(d4)).toString();
    _textKmSonderProgrammSwBis.setText(pidConfArea52);

    /*
     * Konfigurationsbereich
     */
    String menuConfArea5 = getPreferenceStore().getDefaultString(DefsPreference.P_MENUE_SONDER_PROGRAMM_SW);
    _textMenuSonderProgrammSw.setText(menuConfArea5);

    /*
     * Konfigurationsbereich
     */
    String pidConfArea6 = getPreferenceStore().getDefaultString(DefsPreference.P_DARSTELLUNG_PARAMETER);
    _textDarstellungParameter.setText(pidConfArea6);

    /*
     * Benutzer ohne Berechtigungsklasse (Default-Wert)
     */
    boolean userWithoutClass = getPreferenceStore().getDefaultBoolean(DefsPreference.P_USE_ZUSATZ_PRUEFUNG_SONDER_PRG);
    _checkZusatzPruefung.setSelection(userWithoutClass);
    
    /*
     * Benutzer ohne Berechtigungsklasse
     */
    String pidConfArea7 = getPreferenceStore().getDefaultString(DefsPreference.P_FR_DARSTELLUNG_PARAMETER_VON_OBEN_NACH_UNTEN);
    _comboParameter.setText(pidConfArea7);
    
    /*
     * Benutzer ohne Berechtigungsklasse
     */
    String pidConfArea8 = getPreferenceStore().getDefaultString(DefsPreference.P_FR_DARSTELLUNG_PARAMETER_VON_UNTEN_NACH_OBEN);
    _comboParameter2.setText(pidConfArea8);

    /*
     * Benutzer ohne Berechtigungsklasse (Default-Wert)
     */
    boolean userWithoutClass2 = getPreferenceStore().getDefaultBoolean(DefsPreference.P_USE_ZUSATZ_PRUEFUNG_SONDER_PRG);
    _checkSemiAutomatic.setSelection(userWithoutClass2);
  }

  /**
   * Speichert die Einstellungen im {@link PreferenceStore}.
   */
  private boolean actionSave()
  {
    /*
     * Konfigurationsbereich
     */
    String pidConfArea = _textStreckenBeeinflussungsAnlage.getText();

    /*
     * Konfigurationsbereich
     */
    String pidConfArea2 = _textDarstellungIstZustand.getText();

    /*
     * Konfigurationsbereich
     */
    String menuConfArea2 = _textMenuIstZustand.getText();

    /*
     * Konfigurationsbereich
     */
    String pidConfArea3 = _textDarstellungSollZustand.getText();

    /*
     * Konfigurationsbereich
     */
    String menuConfArea3 = _textMenuSollZustand.getText();

    /*
     * Konfigurationsbereich
     */
    String pidConfArea4 = _textDarstellungSonderProgrammNo.getText();

    /*
     * Konfigurationsbereich
     */
    Double d1 = validateTextInput(_textKmSonderProgrammNoVon.getText());

    /*
     * Konfigurationsbereich
     */
    Double d2 = validateTextInput(_textKmSonderProgrammNoBis.getText());

    /*
     * Konfigurationsbereich
     */
    String menuConfArea4 = _textMenuSonderProgrammNo.getText();

    /*
     * Konfigurationsbereich
     */
    String pidConfArea5 = _textDarstellungSonderProgrammSw.getText();

    /*
     * Konfigurationsbereich
     */
    Double d3 = validateTextInput(_textKmSonderProgrammSwVon.getText());

    /*
     * Konfigurationsbereich
     */
    Double d4 = validateTextInput(_textKmSonderProgrammSwBis.getText());

    /*
     * Konfigurationsbereich
     */
    String menuConfArea5 = _textMenuSonderProgrammSw.getText();

    /*
     * Konfigurationsbereich
     */
    String pidConfArea6 = _textDarstellungParameter.getText();

    /*
     * Benutzer ohne Berechtigungsklasse
     */
    boolean showUserWithoutClass = _checkZusatzPruefung.getSelection();
    
    /*
     * Semi-Automatik UrsachenEinheit
     */
//    boolean oldValue = getPreferenceStore().getBoolean(DefsPreference.P_USE_SEMI_AUTOMATIC_URSACHEN_EINHEIT);
    boolean newValue = _checkSemiAutomatic.getSelection();
        
//    if (oldValue != newValue){
//      if (newValue){
//        DialogSemiGuiManager.getInstance().anmeldenAufDaten();
//      }else{
//        DialogSemiGuiManager.getInstance().abmeldenAufDaten();        
//      }
//    }
    
    
    /*
     * Benutzer ohne Berechtigungsklasse
     */
    
    String pidConfArea7 = _comboParameter.getText();
    /*
     * Benutzer ohne Berechtigungsklasse
     */
    String pidConfArea8 = _comboParameter2.getText();

    boolean gueltig = true;

    if (!((pidConfArea != null) && (pidConfArea.length() > 0)))
    {
      gueltig = false;
    }
    if (!((pidConfArea2 != null) && (pidConfArea2.length() > 0)))
    {
      gueltig = false;
    }
    if (!((menuConfArea2 != null) && (menuConfArea2.length() > 0)))
    {
      gueltig = false;
    }
    if (!((pidConfArea3 != null) && (pidConfArea3.length() > 0)))
    {
      gueltig = false;
    }
    if (!((menuConfArea3 != null) && (menuConfArea3.length() > 0)))
    {
      gueltig = false;
    }
    if (!((pidConfArea4 != null) && (pidConfArea4.length() > 0)))
    {
      gueltig = false;
    }
    if (!((d1 != null) && (d1 >= 0.0)))
    {
      gueltig = false;
    }
    if (!((d2 != null) && (d2 >= 0.0)))
    {
      gueltig = false;
    }
    if (!((menuConfArea4 != null) && (menuConfArea4.length() > 0)))
    {
      gueltig = false;
    }
    if (!((pidConfArea5 != null) && (pidConfArea5.length() > 0)))
    {
      gueltig = false;
    }
    if (!((d3 != null) && (d3 >= 0.0)))
    {
      gueltig = false;
    }
    if (!((d4 != null) && (d4 >= 0.0)))
    {
      gueltig = false;
    }
    if (!((menuConfArea5 != null) && (menuConfArea5.length() > 0)))
    {
      gueltig = false;
    }
    if (!((pidConfArea6 != null) && (pidConfArea6.length() > 0)))
    {
      gueltig = false;
    }
    if (!((pidConfArea7 != null) && (pidConfArea7.length() > 0)))
    {
      gueltig = false;
    }
    if (!((pidConfArea8 != null) && (pidConfArea8.length() > 0)))
    {
      gueltig = false;
    }

    if (gueltig)
    {
      getPreferenceStore().setValue(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE, pidConfArea);
      getPreferenceStore().setValue(DefsPreference.P_DARSTELLUNG_ZUSTAND_IST, pidConfArea2);
      getPreferenceStore().setValue(DefsPreference.P_MENUE_ZUSTAND_IST, menuConfArea2);
      getPreferenceStore().setValue(DefsPreference.P_DARSTELLUNG_ZUSTAND_SOLL, pidConfArea3);
      getPreferenceStore().setValue(DefsPreference.P_MENUE_ZUSTAND_SOLL, menuConfArea3);
      getPreferenceStore().setValue(DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_NO, pidConfArea4);
      getPreferenceStore().setValue(DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_NO, d1);
      getPreferenceStore().setValue(DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_NO, d2);
      getPreferenceStore().setValue(DefsPreference.P_MENUE_SONDER_PROGRAMM_NO, menuConfArea4);
      getPreferenceStore().setValue(DefsPreference.P_DARSTELLUNG_SONDER_PROGRAMM_SW, pidConfArea5);
      getPreferenceStore().setValue(DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_SW, d3);
      getPreferenceStore().setValue(DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_SW, d4);
      getPreferenceStore().setValue(DefsPreference.P_MENUE_SONDER_PROGRAMM_SW, menuConfArea5);
      getPreferenceStore().setValue(DefsPreference.P_DARSTELLUNG_PARAMETER, pidConfArea6);
      getPreferenceStore().setValue(DefsPreference.P_USE_ZUSATZ_PRUEFUNG_SONDER_PRG, showUserWithoutClass);
      getPreferenceStore().setValue(DefsPreference.P_FR_DARSTELLUNG_PARAMETER_VON_OBEN_NACH_UNTEN, pidConfArea7);
      getPreferenceStore().setValue(DefsPreference.P_FR_DARSTELLUNG_PARAMETER_VON_UNTEN_NACH_OBEN, pidConfArea8);
      getPreferenceStore().setValue(DefsPreference.P_USE_SEMI_AUTOMATIC_URSACHEN_EINHEIT, newValue);
    }
    return gueltig;
  }

  @Override
  protected void performApply()
  {
    actionSave();
  }

  @Override
  public boolean performOk()
  {
    if (actionSave())
      return super.performOk();
    return false;
  }

  @Override
  protected void performDefaults()
  {
    actionRestoreDefaults();
    updateApplyButton();
  }

  @Override
  public boolean performCancel()
  {
    return true;
  }

  @Override
  public void performHelp()
  {
    String helpContextId = Activator.PLUGIN_ID + ".zustandsdarstellungEinstellungen";
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
  }

  @Override
  public void init(IWorkbench workbench)
  {
  }

  @Override
  public void dispose()
  {
    super.dispose();

    /*
     * Deregistrieren des Verbindungszuhrers
     */
    if (_verbindungZuhoerer != null)
    {
      DaVVerbindung.getInstanz().entferneZuhoerer(_verbindungZuhoerer);
    }
  }

  public static class AnsichtLabelProvider implements ILabelProvider
  {
    private static AnsichtLabelProvider instance;

    public static AnsichtLabelProvider getInstance()
    {
      synchronized (AnsichtLabelProvider.class)
      {
        if (instance == null)
        {
          instance = new AnsichtLabelProvider();
        }
        return instance;
      }
    }

    @Override
    public Image getImage(Object element)
    {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getText(Object element)
    {
      // TODO Auto-generated method stub
      return ((Ansicht)element).getName();
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
      // TODO Auto-generated method stub

    }

    @Override
    public void dispose()
    {
      // TODO Auto-generated method stub

    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
      // TODO Auto-generated method stub

    }
  }

  private Double validateTextInput(String s)
  {
    Double d = null;
    if (s.contains(","))
      s = s.replace(",", ".");
    try
    {
      d = new Double(s);
    }
    catch (NumberFormatException e)
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Definition km");
      mb.setMessage("Eingabe ungltig: " + s);
      mb.open();
      return null;
    }
    return d;
  }

}
