/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Liefert die Datenstruktur des Schaltbildes. <br>
 * Implementiert das Interface {@link Cloneable}.
 * 
 * @author Pittner
 * 
 */
public class Anzeige implements Cloneable
{
  /**
   * Das SystemObjekt des Anzeige
   */
  private SystemObject _anzeige = null;

  private boolean _notwendig = false;

  private boolean _absolutNotwendig = false;

  private boolean _blinken = false;

  private String _blinktakt = "Undefiniert";

  private int _verschiebbarerVorgaenger = 0;

  private int _verschiebbarerNachfolger = 0;

  public boolean isBlinken()
  {
    return _blinken;
  }

  public void setBlinken(boolean blinken)
  {
    _blinken = blinken;
  }

  public String getBlinktakt()
  {
    return _blinktakt;
  }

  public void setBlinktakt(String blinktakt)
  {
    _blinktakt = blinktakt;
  }

  public boolean isAbsolutNotwendig()
  {
    return _absolutNotwendig;
  }

  public void setAbsolutNotwendig(boolean absolutNotwendig)
  {
    _absolutNotwendig = absolutNotwendig;
  }

  private String _prioritaet = "undefiniert";

  /**
   * Mgliche SonderProgrammZustaende
   * 
   * @author Pittner
   */
  public enum EnumAnzeigeProgrammStatus {
    Automatikprogramm, Sonderprogramm, Handprogramm, _3, _10, undefiniert
  };

  /**
   * Gewhlter SonderProgrammzustand
   */
  private EnumAnzeigeProgrammStatus _gewaehlterAnzeigeProgrammStatus = null;

  /**
   * Das SystemObjekt des Anzeige
   */
  private SystemObject _quelle = null;

  private Bild _bild = null;

  private SystemObject _wechselText = null;

  private DynamischerWechselText _dynamischerWechselText = null;

  public SystemObject getAnzeige()
  {
    return _anzeige;
  }

  public void setAnzeige(SystemObject anzeige)
  {
    _anzeige = anzeige;
  }

  public boolean isNotwendig()
  {
    return _notwendig;
  }

  public void setNotwendig(boolean notwendig)
  {
    _notwendig = notwendig;
  }

  public String getPrioritaet()
  {
    return _prioritaet;
  }

  public void setPrioritaet(String prioritaet)
  {
    _prioritaet = prioritaet;
  }

  public EnumAnzeigeProgrammStatus getGewaehlterAnzeigeProgrammStatus()
  {
    return _gewaehlterAnzeigeProgrammStatus;
  }

  public void setGewaehlterAnzeigeProgrammStatus(EnumAnzeigeProgrammStatus gewaehlterAnzeigeProgrammStatus)
  {
    _gewaehlterAnzeigeProgrammStatus = gewaehlterAnzeigeProgrammStatus;
  }

  public SystemObject getQuelle()
  {
    return _quelle;
  }

  public void setQuelle(SystemObject quelle)
  {
    _quelle = quelle;
  }

  public Bild getBild()
  {
    return _bild;
  }

  public void setBild(Bild bild)
  {
    _bild = bild;
  }

  public SystemObject getWechselText()
  {
    return _wechselText;
  }

  public void setWechselText(SystemObject wechselText)
  {
    _wechselText = wechselText;
  }

  public DynamischerWechselText getDynamischerWechselText()
  {
    return _dynamischerWechselText;
  }

  public void setDynamischerWechselText(DynamischerWechselText dynamischerWechselText)
  {
    _dynamischerWechselText = dynamischerWechselText;
  }

  public int getVerschiebbarerVorgaenger()
  {
    return _verschiebbarerVorgaenger;
  }

  public void setVerschiebbarerVorgaenger(int verschiebbarerVorgaenger)
  {
    _verschiebbarerVorgaenger = verschiebbarerVorgaenger;
  }

  public int getVerschiebbarerNachfolger()
  {
    return _verschiebbarerNachfolger;
  }

  public void setVerschiebbarerNachfolger(int verschiebbarerNachfolger)
  {
    _verschiebbarerNachfolger = verschiebbarerNachfolger;
  }

}
