/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.ActionDescriptor;
import org.eclipse.ui.internal.PluginAction;
import org.eclipse.ui.internal.registry.IWorkbenchRegistryConstants;

import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.VerhaltenDoTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.editor.GraphicalEditorToView;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * View-Klasse, welche die Sicht der Parameter Einzelmassnahmen darstellt
 * 
 * @author Pittner
 * 
 */
public class ProgrammHierarchieViewParameter extends GraphicalEditorToView
{
  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.programmHierarchieViewParameter";

  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  private static final String _attributgruppe = "atg.sswSbaSonderprogrammHierarchie";

  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspektEmpfangen = "asp.parameterSoll";

  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspektVersenden = "asp.parameterVorgabe";

  /**
   * Composites
   */
  private Composite _composite;

  private Composite _composite3;

  private Composite _compositeButton3;

  /**
   * Schaltflaechen
   */
//  private Button _buttonAlleUbernehmen;

  private Button _buttonUbernehmen;

  private Button _buttonOk;

  private Button _buttonAbbrechen;

  private Button _buttonHilfe;

  /**
   * Die Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection;

  /**
   * Das Register fr die einzelnen Reiter
   */
  private CTabFolder _tabFolder;

  /**
   * Die Secondary View-ID
   */
  private String _secondViewId;

  /**
   * Das Speicherobjekt zum Wiederherstellen des Zustandes nach dem Start der View
   */
  private IMemento _memento;

  private MenuManager _kontextMenuMgr;

  @Override
  public void init(IViewSite site, IMemento memento) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init(site, memento);
  }

  /**
   * Stellt den Zustand vor dem Beenden das Sicht wieder her
   */
  private void restoreState()
  {
    if (_memento == null)
    {
      return;
    }
    _secondViewId = _memento.getString("pidMemento");
  }

  @Override
  public void saveState(IMemento memento)
  {
    memento.putString("pidMemento", _secondViewId);
    super.saveState(memento);
  }

  @Override
  public void createPartControl(Composite parent)
  {
    _composite = new Composite(parent, SWT.NONE);
    _composite.setLayout(new GridLayout());
    _composite.setLayoutData(GridDataFactory.fillDefaults());

    // Fr die F1 Taste
//    PlatformUI.getWorkbench().getHelpSystem().setHelp(_composite,
//        de.bsvrz.buv.plugin.ursacheneinheit.Activator.PLUGIN_ID + ".parameterEinzelmassnahmen");

    createCompositeTab();
    createCompositeButton();
    restoreState();

    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    if (_connection != null && _secondViewId != null)
    {
      List<SystemObject> list = holeSystemObjekteAusSecondId();
      if (list.size() > 0)
      {
        erzeugeTabItems(list);
//        if (list.size() > 1)
//          _buttonAlleUbernehmen.setEnabled(true);
      }
    }
  
//    registriereActions();        
//    erzeugeKontextMenue();
  }

  /**
   * Holt die {@code SystemObject} der Ursacheneinheiten aus der SecondaryId.
   * 
   * @return
   */
  private List<SystemObject> holeSystemObjekteAusSecondId()
  {
    List<SystemObject> list = new ArrayList<SystemObject>();
    if (_secondViewId.contains(";"))
    {
      String[] split = _secondViewId.split(";");
      for (String s : split)
      {
        SystemObject so = _connection.getDataModel().getObject(s);
        if (so != null)
          list.add(so);
      }
    }
    else
    {
      SystemObject so = _connection.getDataModel().getObject(_secondViewId);
      if (so != null)
        list.add(so);
    }
    return list;
  }

  /**
   * This method initializes _tabFolder (Bereich ber der ButtonLeiste)
   * 
   */
  private void createCompositeTab()
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder(_composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridData);
    _tabFolder.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();
        CTabItem itemSel = folder.getSelection();
        SystemObject so = (SystemObject)itemSel.getData(itemSel.getText());

        Composite cSel = (Composite)itemSel.getControl();
        Control[] children = cSel.getChildren();

        ProgrammHierarchieViewParameterComposite tabComp = null;

        if (children.length == 0)
        {
          GridData gridData = new GridData();
          gridData.horizontalAlignment = GridData.FILL;
          gridData.grabExcessHorizontalSpace = true;
          gridData.grabExcessVerticalSpace = true;
          gridData.verticalAlignment = GridData.FILL;

          tabComp = new ProgrammHierarchieViewParameterComposite(cSel, SWT.NONE, so);
          tabComp.setLayout(new GridLayout());
          tabComp.setLayoutData(gridData);
        }
        else
        {
          tabComp = (ProgrammHierarchieViewParameterComposite)cSel.getChildren()[0];
        }

        cSel.layout();
        if (tabComp != null)
        {
          tabComp.layout();
          // Wenn Alle setzen bettigt wurde, muss der Baum neu initialisiert werden.          
          tabComp.aktualisiereBaum(so);
        }
      }
    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems(List<SystemObject> list)
  {

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    for (SystemObject so : list)
    {
      CTabItem tabItem = new CTabItem(_tabFolder, SWT.NONE);

      String tabName = Tools.getName(so);
      tabItem.setText(tabName);
      tabItem.setData(tabName, so);

      Composite c1 = new Composite(_tabFolder, SWT.NONE);
      c1.setLayout(new GridLayout());
      c1.setLayoutData(gridData2);

      tabItem.setControl(c1);

    }
    _tabFolder.setSelection(0);

    // da durch setSelection in diesem Fall der SelectionListener nicht aktiv wird, muss er selektierte Tab hier gesetzt
    // werden
    CTabItem itemSel = _tabFolder.getSelection();
    Composite cSel = (Composite)itemSel.getControl();
    Control[] children = cSel.getChildren();

    if (children.length == 0)
    {
      String soName = itemSel.getText();
      SystemObject so = (SystemObject)itemSel.getData(soName);
      ProgrammHierarchieViewParameterComposite tabComp = new ProgrammHierarchieViewParameterComposite(cSel, SWT.NONE, so);
      tabComp.setLayout(new GridLayout());
      tabComp.setLayoutData(gridData2);
    }

  }

  /**
   * This method initializes _composite3
   * 
   */
  private void createCompositeButton()
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    _composite3 = new Composite(_composite, SWT.NONE);
    _composite3.setLayout(new GridLayout());
    _composite3.setLayoutData(gridData);

    createCompositeButton3();
  }

  /**
   * This method initializes _compositeButton3
   * 
   */
  private void createCompositeButton3()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.justify = false;
    rowLayout.wrap = true;
    rowLayout.pack = false;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.END;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    _compositeButton3 = new Composite(_composite3, SWT.NONE);
    _compositeButton3.setLayout(rowLayout);
    _compositeButton3.setLayoutData(gridData);
//    _buttonAlleUbernehmen = new Button(_compositeButton3, SWT.NONE);
//    _buttonAlleUbernehmen.setText("Alle setzen");
//    _buttonAlleUbernehmen.setEnabled(false);
//    _buttonAlleUbernehmen.addMouseListener(new MouseAdapter() {
//      @Override
//      public void mouseDown(MouseEvent e)
//      {
//        super.mouseDown(e);
//        aktionAlleUebernehmen();
//      }
//    });
    _buttonOk = new Button(_compositeButton3, SWT.NONE);
    _buttonOk.setText("Ok");
    _buttonOk.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionSchalten();
      }
    });

    _buttonUbernehmen = new Button(_compositeButton3, SWT.NONE);
    _buttonUbernehmen.setText("bernehmen");
    _buttonUbernehmen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionUebernehmen();
      }
    });

    _buttonAbbrechen = new Button(_compositeButton3, SWT.NONE);
    _buttonAbbrechen.setText("Abbrechen");
    _buttonAbbrechen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionAbbrechen();
      }
    });

    _buttonHilfe = new Button(_compositeButton3, SWT.NONE);
    _buttonHilfe.setText("Hilfe");
    _buttonHilfe.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHilfe();
      }
    });
  }

  /**
   * Daten an DaV senden, danach schliesen
   */
  protected void aktionSchalten()
  {
    if (schaltenUndParametrieren())
    {
      aktionAbbrechen();
    }
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Urlasser");
      mb.setMessage("Objekt konnte nicht parametriert werden: " + "\nEventuell keinen AQ ausgewhlt");
      mb.open();
      return;
    }

  }

  /**
   * Daten an DaV senden
   */
  protected void aktionUebernehmen()
  {
    if (schaltenUndParametrieren())
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Urlasser");
      mb.setMessage("Objekt konnte nicht parametriert werden: " + "\nEventuell keinen AQ ausgewhlt");
      mb.open();
      return;
    }

  }

  /**
   * Fr alle Reiter die Daten des aktiven Reiters an den DaV senden
   */
  protected void aktionAlleUebernehmen()
  {
    if (schaltenUndParametrierenAlle())
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Urlasser");
      mb.setMessage("Objekt konnte nicht parametriert werden: " + "\nEventuell keinen AQ ausgewhlt");
      mb.open();
      return;
    }

  }

  /**
   * Methode bernimmt das Versenden der Daten
   * 
   * @return true wenn erfolgreich
   */
  private boolean schaltenUndParametrieren()
  {
    if (_connection != null)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup(_attributgruppe);
      Aspect asp = _connection.getDataModel().getAspect(_aspektVersenden);
      Data data = _connection.createData(atg);

      kopiereDatenAusDarstellung(data);

      CTabItem item = _tabFolder.getSelection();
      List<ResultData> list = new ArrayList<ResultData>();
      String tabName = item.getText();
      SystemObject so = (SystemObject)item.getData(tabName);
      long l = new Date().getTime();
      ResultData rd = new ResultData(so, new DataDescription(atg, asp), l, data);
      list.add(rd);

      UrlasserDialog urlasserDialog = new UrlasserDialog(Display.getCurrent().getActiveShell());
      Boolean schlieenOk = (urlasserDialog.anzeigen(list)).getEingabeOk();
      return schlieenOk;
    }
    return false;
  }

  /**
   * Methode bernimmt das Versenden der Daten
   * 
   * @return true wenn erfolgreich
   */
  private boolean schaltenUndParametrierenAlle()
  {
    if (_connection != null)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup(_attributgruppe);
      Aspect asp = _connection.getDataModel().getAspect(_aspektVersenden);
      Data data = _connection.createData(atg);

      kopiereDatenAusDarstellung(data);
      
      List<ResultData> list = new ArrayList<ResultData>();
      
      CTabItem[] items = _tabFolder.getItems();
      for (CTabItem item : items)
      {         
        SystemObject so = (SystemObject)item.getData(item.getText());

        Composite cSel = (Composite)item.getControl();
        Control[] children = cSel.getChildren();

        ProgrammHierarchieViewParameterComposite tabComp = null;
        
        if (children.length == 0)
        {         
          GridData gridData = new GridData();
          gridData.horizontalAlignment = GridData.FILL;
          gridData.grabExcessHorizontalSpace = true;
          gridData.grabExcessVerticalSpace = true;
          gridData.verticalAlignment = GridData.FILL;
    
          tabComp = new ProgrammHierarchieViewParameterComposite(cSel, SWT.NONE, so);
          tabComp.setLayout(new GridLayout());
          tabComp.setLayoutData(gridData);                   
        }
        else
        {
          tabComp = (ProgrammHierarchieViewParameterComposite)cSel.getChildren()[0];                    
        }
        long l = new Date().getTime();
        ResultData rd = new ResultData(so, new DataDescription(atg, asp), l, data);
        list.add(rd);         
 
      }
       
      UrlasserDialog urlasserDialog = new UrlasserDialog(Display.getCurrent().getActiveShell());
      Boolean schlieenOk = (urlasserDialog.anzeigen(list)).getEingabeOk();

      return schlieenOk;
    }
    return false;
  }

  /**
   * Kopiert die Daten aus der Darstellung in das uebergebene Daten-Objekt
   * 
   * @param data
   *          das Ziel in welches die Daten kopiert werden
   */
  private void kopiereDatenAusDarstellung(Data data)
  {
    
    CTabItem item = _tabFolder.getSelection();
    Composite itemComp = (Composite)item.getControl();
    ProgrammHierarchieViewParameterComposite comp = (ProgrammHierarchieViewParameterComposite)itemComp.getChildren()[0];
    
            
    List<ProgrammHierarchie> listAll = new ArrayList<ProgrammHierarchie>(comp.getMapHierarchie().values());
    
    Array array = data.getItem("Men").asArray();
    array.setLength(listAll.size());
    
    for (int i = 0; i < listAll.size(); i++)
    {
      Data dataWbZiel = array.getItem(i);
      ProgrammHierarchie ph = listAll.get(i);
      dataWbZiel.getItem("Name").asTextValue().setText(ph.bezeichnung);
      dataWbZiel.getItem("Pid").asTextValue().setText("");
      Array arraySpZiel = dataWbZiel.getItem("Sonderprogramme").asArray();
      Programm[] programme = ph.getProgramme().toArray(new Programm[ph.getProgramme().size()]);
      arraySpZiel.setLength(programme.length);
      for (int j = 0; j < programme.length; j++)
      {
        Programm prg = programme[j];
        Data ziel = arraySpZiel.getItem(j);   
        SystemObject so = prg.getResult().getObject();
        if (so != null)
          ziel.asReferenceValue().setSystemObject(so);
      }
      Array arrayUmZiel = dataWbZiel.getItem("UnterMen").asArray();
      String[] menues = ph.getUntermenues().toArray(new String[ph.getUntermenues().size()]);
      arrayUmZiel.setLength(menues.length);
      for (int j = 0; j < menues.length; j++)
      {
        String menue = menues[j];
        Data ziel = arrayUmZiel.getItem(j);   
        if (menue != null)
          ziel.asTextValue().setText(menue);
      }
    }   
  }
  
  /**
   * Sicht wird geschlossen
   */
  protected void aktionAbbrechen()
  {
    dispose();
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  protected void aktionHilfe()
  {
    String contextId = de.bsvrz.buv.plugin.anlagendarstellung.Activator.PLUGIN_ID + ".parameterEinzelmassnahmen";
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  @Override
  public void dispose()
  {
    IViewSite viewSite = getViewSite();
    if (viewSite != null)
    {
      IWorkbenchPage page = viewSite.getPage();
      if (page != null)
        page.hideView(ProgrammHierarchieViewParameter.this);
    }
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_composite != null)
      _composite.setFocus();
  }

//  /*
//   * (non-Javadoc)
//   * 
//   * @see de.bsvrz.buv.plugin.zustandsdarstellung.editor.GraphicalEditorToView#getGraphicalViewer()
//   */
//  public GraphicalViewer getGraphicalViewer()
//  {
//    // Die Anfrage wird an den Viewer des selektierten Reiters weitergeleitet, dieser liefert dann den Viewer
//    CTabItem itemSel = _tabFolder.getSelection();
//    Composite cSel = (Composite)itemSel.getControl();
//    ProgrammHierarchieViewParameterComposite tabComp = (ProgrammHierarchieViewParameterComposite)cSel.getChildren()[0];
//    return tabComp.getGraphicalViewer();
//  }

//  /*
//   * (non-Javadoc)
//   * 
//   * @see
//   * de.bsvrz.buv.plugin.zustandsdarstellung.editor.GraphicalEditorToView#setGraphicalViewer(org.eclipse.gef.GraphicalViewer
//   * )
//   */
//  public void setGraphicalViewer(GraphicalViewer graphicalViewer)
//  {
//    // Die Anfrage wird an den Viewer des selektierten Reiters weitergeleitet
//    CTabItem itemSel = _tabFolder.getSelection();
//    Composite cSel = (Composite)itemSel.getControl();
//    ProgrammHierarchieViewParameterComposite tabComp = (ProgrammHierarchieViewParameterComposite)cSel.getChildren()[0];
//    tabComp.setGraphicalViewer(graphicalViewer);
//  }

  /**
   * Adaptable implementation for {@code GraphicalEditorToView}
   */
  public Object getAdapter(Class adapter)
  {
    // we need to handle common GEF elements we created
    if (adapter == GraphicalViewer.class || adapter == EditPartViewer.class)
      return getGraphicalViewer();
    else if (adapter == CommandStack.class)
      return getCommandStack();
    else if (adapter == EditDomain.class)
      return getEditDomain();
    else if (adapter == ActionRegistry.class)
      return getActionRegistry();    

    // the super implementation handles the rest
    return super.getAdapter(adapter);
  }

  @Override
  protected void initializeGraphicalViewer()
  {
    // TODO Auto-generated method stub
    /**
     * Hier passiert nichts, da die View keinen GEF-Viewer besitzt.
     */
  }

//  /**
//   * Gibt den Command Stack zurck.
//   * 
//   * @return der Command Stack, niemals <code>null</code>.
//   */
//  protected CommandStack getCommandStack()
//  {
//    // Die Anfrage wird an den Viewer des selektierten Reiters weitergeleitet, dieser liefert dann den Stack
//    CTabItem itemSel = _tabFolder.getSelection();
//    Composite cSel = (Composite)itemSel.getControl();
//    ProgrammHierarchieViewParameterComposite tabComp = (ProgrammHierarchieViewParameterComposite)cSel.getChildren()[0];
//    CommandStack stack = tabComp.getGraphicalViewer().getEditDomain().getCommandStack();
//
//    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
//    return stack;
//  }
  
  
//  /**
//   * Registriert die Actions zum Aufruf der Outline-View
//   */
//  private void registriereActions()
//  {
//    IMenuManager menuManager = this.getViewSite().getActionBars().getMenuManager();    
//     
//      AnsichtEditPart contents = (AnsichtEditPart)getGraphicalViewer().getContents();
//      Ansicht ansicht = contents.getModel();
//      Darstellung darstellung = ansicht.getDarstellung();
//      EList<Double> stufen = darstellung.getZoomStufen();
//      
//      final ScalableRootEditPart rootEditPart = (ScalableRootEditPart)getGraphicalViewer().getRootEditPart();
//      final ZoomManager zoomManager = rootEditPart.getZoomManager();
//      
//      double[] dd = new double[stufen.size()];
//      
//      for (int i = 0; i < dd.length; i++)
//      {
//        dd[i] = stufen.get(i);      
//      }
//      
//      zoomManager.setZoomLevels(dd);
//      
//      for (final Double stufe : stufen)
//      {
//        Action zoomAction = new Action() {
//          public void run()
//          { 
//            CTabItem item = _tabFolder.getSelection();          
//            Composite cSel = (Composite)item.getControl();
//            ProgrammHierarchieViewParameterComposite tabComp = (ProgrammHierarchieViewParameterComposite)cSel.getChildren()[0];            
//            GraphicalViewer graphicalViewer = tabComp.getGraphicalViewer();
//            final ZoomManager zoomManager = ((ScalableRootEditPart)graphicalViewer.getRootEditPart()).getZoomManager();            
//            zoomManager.setZoom(stufe );
//          }
//
//        };
//        zoomAction.setText("Zoomstufe " + stufe);
//        menuManager.add(zoomAction);
//      }      
//    }
  
  
  /**
   * Liefert das KontextMenue zum Navigator
   */
  private void erzeugeKontextMenue()
  {
    _kontextMenuMgr = new MenuManager("#EinzelMassnahmenMenu");     
    _kontextMenuMgr.addMenuListener(new IMenuListener()
    {
      public void menuAboutToShow(IMenuManager manager)
      {
        IStructuredSelection ss = (IStructuredSelection)getGraphicalViewer().getSelection();

        if (!ss.isEmpty())
        {
          Object obj = ss.getFirstElement();

          _kontextMenuMgr.removeAll();

          if (obj instanceof DoModelEditPart)
          {
            DoModelEditPart part = (DoModelEditPart)obj;
            DoModel model = (DoModel)part.getModel();
            DoTyp doTyp = model.getDoTyp();
            if (doTyp instanceof VerhaltenDoTyp)
            {
              VerhaltenDoTyp coDoTyp = (VerhaltenDoTyp)doTyp;

              int kontext = -1;

              if (model instanceof AnzeigeQuerschnittModelTypAbstract)
              {
                AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)model;
                kontext = aq.getKontext();
              }
              else if (model instanceof AnzeigeModelTypAbstract)
              {
                AnzeigeModelTypAbstract a = (AnzeigeModelTypAbstract)model;
                EObject parent = a.eContainer();
                if (parent != null)
                {
                  if (parent instanceof AnzeigeQuerschnittModelTypAbstract)
                  {
                    AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parent;
                    kontext = aq.getKontext();
                  }
                }
              }

              String id = null;

              switch (kontext)
              {
                case 0:
                  id = coDoTyp.getKontextMenueIdIstZustand();
                  break;
                case 1:
                  id = coDoTyp.getKontextMenueIdSollZustand();
                  break;
                case 2:
                  id = coDoTyp.getKontextMenueIdHandSchaltung();
                  break;
                case 3:
                  id = coDoTyp.getKontextMenueIdSimulation();
                  break;
                case 4:
                  id = coDoTyp.getKontextMenueIdTrichter();
                  break;
                case 5:
                  id = coDoTyp.getKontextMenueIdParameter();
                  break;

                default:
                  break;
              }
              if (id != null)
                leseActionSets(id);
            }
          }
        }
      }
    });

    // Men erzeugen - Viewer wird dabei dem Men bekannt gemacht
    Menu menu = _kontextMenuMgr.createContextMenu(getGraphicalViewer().getControl());
    // Das Men dem Baum bekannt machen
    getGraphicalViewer().getControl().setMenu(menu);
    getSite().registerContextMenu(_kontextMenuMgr, getGraphicalViewer());

  }
  
  /**
   * Liest die Aktionen unter Bercksichtigung des aktuellen WorkbenchWindow ein.<br>
   * Die Aktionen werden so erzeugt, dass die Actions am SelectionProvider des aktuellen EclipseWindows hngen. <br>
   * Evtl. muss noch das aktuelle Workbenchwindow beim speichern der Aktionen bercksichtigt werden. Dies ist fr ein
   * Window aber NICHT ntig.
   */
  @SuppressWarnings("restriction")
  public void leseActionSets(String id)
  {
    IExtensionRegistry reg = Platform.getExtensionRegistry();
    IConfigurationElement[] aConfigurationElementsFor = reg.getConfigurationElementsFor("org.eclipse.ui.actionSets");
    IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    for (IConfigurationElement confEle : aConfigurationElementsFor)
    {
      String contributor = confEle.getContributor().getName();            
//      if (Activator.PLUGIN_ID.equals(contributor))
      {     
        IConfigurationElement[] iceAction = confEle.getChildren("action");
        for (IConfigurationElement element : iceAction)
        {          
          String att = element.getAttribute("id");  
          if (att.equals(id)){            
            boolean pullDownStyle = false;
            String style = element.getAttribute(IWorkbenchRegistryConstants.ATT_STYLE);
            if (style != null)
            {
              pullDownStyle = style.equals(ActionDescriptor.STYLE_PULLDOWN);
            } else
            {
              String pulldown = element.getAttribute(ActionDescriptor.STYLE_PULLDOWN);
              pullDownStyle = pulldown != null && pulldown.equals("true");
            }
            
            ActionDescriptor desc = null;
            if (pullDownStyle)
            {
              desc = new ActionDescriptor(element,
                  ActionDescriptor.T_WORKBENCH_PULLDOWN, window);
            } else
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH, window);
            }           
            PluginAction action = desc.getAction();                
            _kontextMenuMgr.add(action);            
          }  
        }        
      }      
    }
  }
}
