/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITableFontProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;
import com.bitctrl.lib.eclipse.emf.viewers.EmfLabelProvider;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaAnsichtProvider;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DoModelHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.views.SbaScalableRootEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.DarstellungEditPartFactory;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Die Klasse realisert die Darstellung und Bearbeitung des Schaltbild-Parameters (Einzelmassnahme) einer
 * Ursacheneinheit. Diese wird in einer Registerkarte der Sicht {@code EinzelmassnahmenViewParameter} verwendet.
 * 
 * @author Pittner
 * 
 */
public class WanderbaustelleViewParameterComposite extends Composite
{
  /**
   * Die Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection;

  /**
   * Die geteilte Ansicht
   */
//  private SashForm _sf1;

  /**
   * Der Baum
   */
  private Tree _tree;

  /**
   * Composites
   */
//  private Composite _composite1;

  private Composite _composite12;

  private Composite _composite2;

  private Composite _composite21;

  private Composite _compositeButton1;

  /**
   * Schaltflaechen
   */
  private Button _buttonHinzufuegen;

  private Button _buttonLoeschen;

  private Button _buttonHoch;

  private Button _buttonRunter;

  private Button _buttonKopieren;

  /**
   * Der GEF-Viewer
   */
  private GraphicalViewer graphicalViewer;

  private MenuManager _kontextMenuMgr;

  private ActionRegistry _actionRegistry;

  private Button _buttonHinzufuegen2;

  private Button _buttonUmbenennen;

  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  private static final String _attributgruppe = "atg.sswSbaWanderbaustelle";

  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspekt = "asp.parameterSoll";

  /**
   * Konstruktor
   * 
   * @param parent
   *          das Elternobjekt
   * @param style
   *          der Stil wie das {@code Composite} dargestellt werden soll
   * @param so
   *          die Ursacheneinheit
   */
  public WanderbaustelleViewParameterComposite(Composite parent, int style, SystemObject so)
  {
    super(parent, style);
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    
    this.setLayout(new FillLayout());
    this.setLayoutData(gridData3);
    
    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
    createInhaltTab(this, so);

  }

  /**
   * Initialisiert den Inhalt des {@code TabItem}
   * 
   * @param tabItemComposite
   *          das Eltern Objekt
   * @param so
   *          die Ursacheneinheit
   */
  private void createInhaltTab(Composite tabItemComposite, SystemObject so)
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    _composite12 = new Composite(tabItemComposite, SWT.NONE);
    _composite12.setLayout(new GridLayout(2, false));
    _composite12.setLayoutData(gridData3);

    createGroupTree();
    createCompositeButton();
    
    
//    _sf1.setWeights(new int[] { 60, 140 });

//    setzeAlleAnzeigen(AnzeigeInhaltTyp.INAKTIV_BI);
    aktualisiereBaum(so);
  }



  /**
   * This method initializes _tree
   */
  private void createGroupTree()
  {
    Group grp = new Group(_composite12, SWT.NONE);
    grp.setText("Wanderbaustellen");

    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessVerticalSpace = true;
    
    grp.setLayout(new GridLayout());
    grp.setLayoutData(gd);

    _tree = new Tree(grp, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
    _tree.setHeaderVisible(false);
    _tree.setLinesVisible(false);
    _tree.setLayoutData(gd);

    _tree.addListener(SWT.Selection, new Listener() {

      @Override
      public void handleEvent(Event event)
      {
        // TODO Auto-generated method stub
        Widget widget = event.item;
        if (widget instanceof TreeItem)
        {
          updateButtons();
//          updateKontextMenu();
          
          TreeItem treeitem = (TreeItem)widget;
          
          String text = treeitem.getText();
          AnzeigeInhaltTyp inhalt = null;
          
          
          
          TreeItem parentItem = treeitem.getParentItem();
          Data data = (Data)treeitem.getData();
          
//          System.out.println(treeitem + " : " + parentItem + " -> " + data);
        }
      }
    });   
  }

  /**
   * This method initializes _compositeButton1
   * 
   */
  private void createCompositeButton()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout.justify = false;
    rowLayout.wrap = true;
    rowLayout.pack = false;
    rowLayout.marginTop = 5;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.END;
    gridData.grabExcessHorizontalSpace = false;
    gridData.verticalAlignment = GridData.BEGINNING;
    _compositeButton1 = new Composite(_composite12, SWT.NONE);
    _compositeButton1.setLayout(rowLayout);
    _compositeButton1.setLayoutData(gridData);
    _buttonHinzufuegen2 = new Button(_compositeButton1, SWT.NONE);
    _buttonHinzufuegen2.setText("Wanderbaustelle Hinzufgen");
    _buttonHinzufuegen2.setEnabled(true);
    _buttonHinzufuegen2.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHinzufuegen2("");
      }
    });
    _buttonHinzufuegen = new Button(_compositeButton1, SWT.NONE);
    _buttonHinzufuegen.setText("Programm Hinzufgen");
    _buttonHinzufuegen.setEnabled(false);
    _buttonHinzufuegen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHinzufuegen();
      }
    });
    _buttonLoeschen = new Button(_compositeButton1, SWT.NONE);
    _buttonLoeschen.setText("Lschen");
    _buttonLoeschen.setEnabled(false);
    _buttonLoeschen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionLoeschen();
      }
    });
    _buttonHoch = new Button(_compositeButton1, SWT.NONE);
    _buttonHoch.setText("Hoch");
    _buttonHoch.setEnabled(false);
    _buttonHoch.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHoch();
        updateButtons();
      }
    });
    _buttonRunter = new Button(_compositeButton1, SWT.NONE);
    _buttonRunter.setText("Runter");
    _buttonRunter.setEnabled(false);
    _buttonRunter.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionRunter();
        updateButtons();
      }
    });
    _buttonKopieren = new Button(_compositeButton1, SWT.NONE);
    _buttonKopieren.setText("Kopieren");
    _buttonKopieren.setEnabled(false);
    _buttonKopieren.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionKopieren();
      }
    });
    
    _buttonUmbenennen = new Button(_compositeButton1, SWT.NONE);
    _buttonUmbenennen.setText("Umbenennen");
    _buttonUmbenennen.setEnabled(false);
    _buttonUmbenennen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);        
        aktionUmbenennen();
      }
    });
  }

  /**
   * This method initializes _composite2
   * 
   */
  private void createComposite2(Composite parent)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _composite2 = new Composite(parent, SWT.NONE);
    _composite2.setLayout(new GridLayout());
    _composite2.setLayoutData(gridData);

    createGraphicalViewer();
  }

  /**
   * GraphicalViewer fr die Darstellung
   */
  private void createGraphicalViewer()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _composite21 = new Composite(_composite2, SWT.BORDER);
    _composite21.setLayout(new FillLayout());
    _composite21.setLayoutData(gridData);

    // Ansicht in einen extra Viewer packen.
    graphicalViewer = new ScrollingGraphicalViewer();
    graphicalViewer.createControl(_composite21);
    // XXX: Wichtig - Damit wei der Viewer welchen EditPart er fr ein
    // Modell Objekt benutzen soll.
    graphicalViewer.setEditDomain(new EditDomain());
    graphicalViewer.setRootEditPart(new SbaScalableRootEditPart());
    graphicalViewer.setEditPartFactory(new DarstellungEditPartFactory());
    
//    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
//    String name = store.getString(DefsPreference.P_DARSTELLUNG_PARAMETER);    
//    if (name != null && name.length() > 0){
//      graphicalViewer.setContents(EcoreUtil.copy(SbaEinstellungen.INSTANCE.getAnsicht(name)));    
//    }
    
    
    Ansicht ansicht = SbaAnsichtProvider.getInstanz().getAnsichtKopie(DefsPreference.P_DARSTELLUNG_PARAMETER);
    if (ansicht != null)
      graphicalViewer.setContents(ansicht);    
    
    getCommandStack().setUndoLimit(1);
    
    final ScalableRootEditPart rootEditPart = (ScalableRootEditPart)graphicalViewer.getRootEditPart();

//    final ZoomManager zoomManager = rootEditPart.getZoomManager();
//    zoomManager.setZoomLevels(new double[] { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0 });
//    graphicalViewer.setProperty(MouseWheelHandler.KeyGenerator.getKey(SWT.NONE), MouseWheelZoomHandler.SINGLETON);

    if (DaVKommunikationDobj.getInstanz().holeDaVVerbindung() != null)
    {

      AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();
//      Ansicht ansicht = contents.getModel();

      StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
      darstellung.setSpaltenBorder(true);

      contents = (AnsichtEditPart)graphicalViewer.getContents();
      ansicht = contents.getModel();
      darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    }
    
    erzeugeKontextMenue();

    graphicalViewer.getControl().addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDoubleClick(MouseEvent e)
      {
        // TODO Auto-generated method stub
        ISelection selection = graphicalViewer.getSelection();

        TreeItem[] tree_selection = _tree.getSelection();
        if (tree_selection != null && tree_selection.length > 0)
        {
          Data data = (Data)tree_selection[0].getData();
          DoModelHelper.holeDoppelKlickAktionAusViewerSelection(selection, data);
        }

      }
    });
  }

  /**
   * 
   */
  /**
   * Vorbelegung aller Anzeigen mit einer Inhalt
   * @param inhalt
   */
  public void setzeAlleAnzeigen(AnzeigeInhaltTyp  inhalt)
  {
    AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {
      EList<DoModel> dos = ebene.getDoObjekte();
      for (DoModel dom : dos)
      {
        if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
        {
          AnzeigeQuerschnittModelTypAbstract model = (AnzeigeQuerschnittModelTypAbstract)dom;
          EList<AnzeigeModelTypAbstract> komponenten = model.getKomponenten();
          for (AnzeigeModelTypAbstract anzeige : komponenten)
          {
            EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
            final SetCommand cmd = new SetCommand(anzeige, feature, inhalt);
            getCommandStack().execute(cmd);
          }
        }
      }
    }
  }
  
  /**
   * Der Label-Provider des Trees
   * 
   * @author Pittner
   * 
   */
  private class ProgrammLabelProvider2 implements ITableLabelProvider
  , ITableFontProvider
//  , IColorProvider
  {
    
    public Image getColumnImage(Object element, int columnIndex)
    {
      Image re = null;
      
//      System.out.println("getColumnImage(): " + columnIndex + " -> " + element);
      
      switch (columnIndex)
      {
        
        case 0:
          if (element instanceof Programm)
          {                       
            
            String simu = ((Programm)element).getSimulation();
            if (simu.contains("Ja")){
              re = ImageFactory.getImage("icons/sp_simu.png");                            
            }
            String stand = ((Programm)element).getStand();
            if (stand.equals("Aktiv")){
              re = ImageFactory.getImage("icons/sp_aktiv.png");              
            }else if (stand.equals("Pause")){
              re = ImageFactory.getImage("icons/sp_pause.png");                            
            }
          }
          break;         
        default:
      }
      
      return re;
    }
    
    public String getColumnText(Object element, int columnIndex)
    {
      
      switch (columnIndex)
      {
        case 1:
          return element.toString();
        case 2:
          if (element instanceof Programm)
            return ((Programm)element).getStand();
        case 3:
          if (element instanceof Programm)
            return ((Programm)element).getSimulation();
          
      }
      return null;
    }
    
    public void addListener(ILabelProviderListener listener)
    {
    }
    
    public void dispose()
    {
    }
    
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }
    
    public void removeListener(ILabelProviderListener listener)
    {
    }
    
    @Override
    public Font getFont(Object element, int columnIndex)
    {
      switch (columnIndex)
      {
        case 1:
          if (element instanceof Wanderbaustelle)
          {
            FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
            String name = fontData.getName();
            return JFaceResources.getFontRegistry().getBold(name);
          }
          if (element instanceof Programm)
          {
            String stand = ((Programm)element).getStand();
            if (stand.equals("Aktiv"))
            {
              FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
              String name = fontData.getName();
              return JFaceResources.getFontRegistry().getItalic(name);
            }
          }
          
      }
      return null;
    }    
  }

  /**
   * Aktualisert die Darstellung im {@code GraphicalViewer} mit dem Daten des uebergebenen {@code SystemObject}
   * 
   * @param die
   *          Ursacheneinheit
   */
  protected void aktualisiereBaum(SystemObject so)
  {
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(_attributgruppe);
    Aspect asp = _connection.getDataModel().getAspect(_aspekt);
    ResultData result = _connection.getData(so, new DataDescription(atg, asp), 1500);
 
    _tree.removeAll();

    if (result.hasData())
    {
      if (result != null && result.hasData())
      {
        Data dataNeu = result.getData().createModifiableCopy();       
        Array array = dataNeu.getItem("Wanderbaustelle").asArray();
        for (int i = 0; i < array.getLength(); i++)
        {
          Data data = array.getItem(i);
          TreeItem item = new TreeItem(_tree, SWT.NONE);
          item.setData(data);
          item.setText(data.getItem("Name").asTextValue().getValueText());
          Array array2 = data.getItem("Sonderprogramme").asArray();
          for (int j = 0; j < array2.getLength(); j++)
          {
            Data subData = array2.getItem(j);
            TreeItem subItem = new TreeItem(item, SWT.NONE);
            subItem.setData(subData);
            
//            System.out.println(subData + " : " + subItem + " > " + j);
            
            subItem.setText(subData.asReferenceValue().getSystemObject().toString());
          }
        }
      }
    }
  }

  /**
   * Aktivieren bzw. Deaktivieren der Buttons anhanf der Selektion im Baum
   */
  private void updateButtons()
  {
    
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      String text = ti.getText();
      // Das Parent ist null, wenn es sich um eine Wanderbaustelle handelt
      TreeItem parentItem = ti.getParentItem();
      if (parentItem == null)
      {
//        _buttonHinzufuegen2.setEnabled(true);
        _buttonHinzufuegen.setEnabled(true);
        _buttonLoeschen.setEnabled(true);
        _buttonHoch.setEnabled(false);
        _buttonRunter.setEnabled(false);
        _buttonKopieren.setEnabled(true);        
        _buttonUmbenennen.setEnabled(true);        
      }
      else
      {
//        _buttonHinzufuegen2.setEnabled(true);
        _buttonHinzufuegen.setEnabled(true);
        _buttonLoeschen.setEnabled(true);
        int cnt = parentItem.getItemCount();
        int cntNeu = cnt;
        if (cnt > 0)
          cntNeu -= 1;
        if (cnt == 1)
        {
          _buttonHoch.setEnabled(false);
          _buttonRunter.setEnabled(false);
        } 
        else
        {          
          _buttonHoch.setEnabled(false);
          _buttonRunter.setEnabled(false);
          
          if (!isFirstItem(ti))
            _buttonHoch.setEnabled(true);
          if (!isLastItem(ti))
            _buttonRunter.setEnabled(true);
        }

        _buttonKopieren.setEnabled(false);
        _buttonUmbenennen.setEnabled(false);        
      }
    }
    else
    {
//      _buttonHinzufuegen2.setEnabled(true);
      _buttonHinzufuegen.setEnabled(false);
      _buttonLoeschen.setEnabled(false);
      _buttonHoch.setEnabled(false);
      _buttonRunter.setEnabled(false);
      _buttonKopieren.setEnabled(false);
      _buttonUmbenennen.setEnabled(false);        
    }
  }
  
  /**
   * Aktivieren bzw. Deaktivieren der Kontextmenues anhand der Selektion im Baum
   */
  private void updateKontextMenu()
  {
    
    IAction actionHinzu = _actionRegistry.getAction("Hinzufuegen");
    IAction actionLoesch = _actionRegistry.getAction("Loeschen");
    IAction actionHoch = _actionRegistry.getAction("Hoch");
    IAction actionRunter = _actionRegistry.getAction("Runter");
    IAction actionKopie = _actionRegistry.getAction("Kopieren");
    
    
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      String text = ti.getText();
      if (text.equals("Aktiv") || text.equals("Inaktiv"))
      {
        
        actionHinzu.setEnabled(true);
        actionLoesch.setEnabled(false);
        actionHoch.setEnabled(false);
        actionRunter.setEnabled(false);
        actionKopie.setEnabled(false);
//        _buttonHinzufuegen.setEnabled(true);
//        _buttonLoeschen.setEnabled(false);
//        _buttonHoch.setEnabled(false);
//        _buttonRunter.setEnabled(false);
//        _buttonKopieren.setEnabled(false);
      }
      else
      {
        actionHinzu.setEnabled(true);
        actionLoesch.setEnabled(true);
//        _buttonHinzufuegen.setEnabled(true);
//        _buttonLoeschen.setEnabled(true);
        
        TreeItem parentItem = ti.getParentItem();
        int cnt = parentItem.getItemCount();
        int cntNeu = cnt;
        if (cnt > 0)
          cntNeu -= 1;
        if (cnt == 1)
        {
          actionHoch.setEnabled(false);
          actionRunter.setEnabled(false);
//          _buttonHoch.setEnabled(false);
//          _buttonRunter.setEnabled(false);
        }
        else if (text.equals("Alternative: 0"))
        {
          actionHoch.setEnabled(false);
          actionRunter.setEnabled(true);
//          _buttonHoch.setEnabled(false);
//          _buttonRunter.setEnabled(true);
        }
        else if (text.equals("Alternative: " + cntNeu))
        {
          actionHoch.setEnabled(true);
          actionRunter.setEnabled(false);
//          _buttonHoch.setEnabled(true);
//          _buttonRunter.setEnabled(false);
        }
        else
        {
          actionHoch.setEnabled(true);
          actionRunter.setEnabled(true);
//          _buttonHoch.setEnabled(true);
//          _buttonRunter.setEnabled(true);
        }
        
        actionKopie.setEnabled(true);
//        _buttonKopieren.setEnabled(true);
      }
    }
    else
    {
      actionHinzu.setEnabled(false);
      actionLoesch.setEnabled(false);
      actionHoch.setEnabled(false);
      actionRunter.setEnabled(false);
      actionKopie.setEnabled(false);
//      _buttonHinzufuegen.setEnabled(false);
//      _buttonLoeschen.setEnabled(false);
    }
  }

  /**
   * Erzeugt eine neue Alternative und fgt diese am Ende der Alternativen ein
   */
  protected void aktionHinzufuegen2(String name)
  {
    TreeItem[] selection = _tree.getSelection();
//    if (selection != null && selection.length > 0)
    {
//      TreeItem selectedItem = selection[0];
//      String text = selectedItem.getText();
//      
//      TreeItem parent = selectedItem.getParentItem();
      
//      if (parent == null)
      {
        
        InputDialog inputDialog = new InputDialog(this.getShell(), "Wanderbaustelle anlegen", "Bitte Namen eingeben", "", null);
        int open = inputDialog.open();
        if (open == IStatus.OK){
          String nameNeu = inputDialog.getValue();
          TreeItem treeItem = new TreeItem(_tree, SWT.NONE);
          treeItem.setText(nameNeu);
          _tree.setSelection(treeItem);
        }
        
      }
    }
  }
  /**
   * Erzeugt eine neue Alternative und fgt diese am Ende der Alternativen ein
   */
  protected void aktionHinzufuegen()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem selectedItem = selection[0];
      String text = selectedItem.getText();
      {
        
        List<Object> contents = new ArrayList<Object>();
        
        final String REMOVE_RELATIV_ZU = "<Referenz lschen>";
        contents.add(0, REMOVE_RELATIV_ZU);

        SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaUeSonderprogramm");
        if (type != null)
          contents.addAll(type.getElements());
        
        final ElementListSelectionDialog dialog = new ElementListSelectionDialog(
            this.getShell(), new EmfLabelProvider());
        dialog.setTitle("SystemObjekt auswhlen");
        dialog.setMessage("Bitte whlen Sie eine anderes SystemObjekt zur Auswahl.");
        dialog.setMultipleSelection(true);
        dialog.setElements(contents.toArray());

        if (text != null)
        {
          dialog.setInitialElementSelections(Collections.singletonList(text));
        }

        if (dialog.open() == Window.OK)
        {
          Object value = null;
          if (REMOVE_RELATIV_ZU.equals(dialog.getFirstResult()))
          {
            value = null;
          }
          else
          {
            AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaWanderbaustelle");
//            Object object = dialog.getFirstResult();
            TreeItem parent = selectedItem.getParentItem();  
            
            Object[] result = dialog.getResult();
            TreeItem treeItem = null;
            for (Object object : result)
            {
              if (parent == null)
              {
//              System.out.println("Parent ist null: " + text);
                treeItem = new TreeItem(selectedItem, SWT.NONE);
                parent = selectedItem;
              }
              else
              {
                treeItem = new TreeItem(parent, SWT.NONE);
              }
              if (_connection != null)
              {
                Data data = _connection.createData(atg).createModifiableCopy();
                Array array = data.getItem("Wanderbaustelle").asArray();
                array.setLength(1);
                Data item0 = array.getItem(0);
                Array array2 = item0.getItem("Sonderprogramme").asArray();
                array2.setLength(parent.getItemCount());
                Data item1 = array2.getItem(parent.getItemCount()-1);
                
                if (object instanceof SystemObject)
                {
                  SystemObject so = (SystemObject)object;
                  item1.asReferenceValue().setSystemObject(so);                
                }
                
                treeItem.setData(item1);  
                treeItem.setText(object.toString());
                
//              System.out.println(object.toString() + " -> set: " + item1);
                
              }
              
            }
            if (treeItem != null)
              _tree.setSelection(treeItem);
            
            
          }
      }
        
        
      }

//      if (text.equals("Aktiv") || text.equals("Inaktiv"))
//      {
//        parent = selectedItem;
//      }
//      else
//      {
//        parent = selectedItem.getParentItem();
//      }
//
//      if (parent != null)
//      {
//        TreeItem subItem = new TreeItem(parent, SWT.NONE);
//        int index = parent.getItemCount() - 1;
//        subItem.setText(new String("Alternative: " + index));
//
//        if (_connection != null)
//        {
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaEinzelMassnahme");
//          Data data = _connection.createData(atg).createModifiableCopy();
//          Array array = data.getItem(parent.getText()).asArray();
//          array.setLength(parent.getItemCount());
//          Data item = array.getItem(index);
//          subItem.setData(item);
//
//        }
//
//        _tree.setSelection(subItem);
//        Event event = new Event();
//        event.item = subItem;
//        _tree.notifyListeners(SWT.Selection, event);
//
//      }
    }
  }

  /**
   * Loescht die selektierte Alternative
   */
  protected void aktionLoeschen()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      TreeItem parent = ti.getParentItem();
      
      TreeItem[] items = null;
      if (parent != null){
        items = parent.getItems();
      }else{
        items = ti.getItems();
      }
      
      if (ti != null && !ti.isDisposed())
        ti.dispose();
      
//      for (int i = 0; i < items.length; i++)
//      {
//        TreeItem item = items[i];
//      }        
    }

  }

  /**
   * Verschiebt die Alternative um eine Position nach oben
   */
  protected void aktionHoch()
  {
    // TODO Auto-generated method stub
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      
      TreeItem[] items = null;
      
      TreeItem parent = ti.getParentItem();      
      if (parent != null){
        items = parent.getItems();
      }
 

      Map<Data, String> map = new HashMap<Data, String>();

      int index = -1;

      List<Data> list = new ArrayList<Data>();
      for (int i = 0; i < items.length; i++)
      {
        TreeItem item = items[i];
        Data dta = (Data)item.getData();
        String text = item.getText();
        map.put(dta, text);

        if (item.equals(ti))
        {
          index = i;
//          System.out.println(index + " : " + text + " : " + dta);
        }
        list.add(dta);
      }
//      List<Data> list = new ArrayList<Data>(map.keySet());
      if (index >= 0)
        Collections.swap(list, index - 1, index);
      
     
      for (int i = 0; i < list.size(); i++)
      {
        Data dta = list.get(i);
        String text = map.get(dta);
//        System.out.println(i + " # " + text + " # " + dta);
        TreeItem item = items[i];        
        item.setData(dta);
        item.setText(text);
      }
      if (index > 0){
        _tree.redraw();
        _tree.select(parent.getItem(index - 1));
      }      
    }
  }
  
  /**
   * Verschiebt die Alternative um eine Position nach unten
   */
  protected void aktionRunter()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      TreeItem parent = ti.getParentItem();
      
      TreeItem[] items = null;
      if (parent != null)
       items = parent.getItems();       

      Map<Data, String> map = new HashMap<Data, String>();

      int index = -1;

      List<Data> list = new ArrayList<Data>();
      for (int i = 0; i < items.length; i++)
      {
        TreeItem item = items[i];
        Data dta = (Data)item.getData();
        String text = item.getText();
        map.put(dta, text);

        if (item.equals(ti))
        {
          index = i;
        }

        list.add(dta);
      }
//      List<Data> list = new ArrayList<Data>(map.keySet());
      if (index >= 0)
        Collections.swap(list, index + 1, index);
      
     
      for (int i = 0; i < list.size(); i++)
      {
        Data dta = list.get(i);
        String text = map.get(dta);
        TreeItem item = items[i];        
        item.setData(dta);
        item.setText(text);
      }
      if (index > 0){
        _tree.redraw();
        _tree.select(parent.getItem(index + 1));
      }
    }

  }

  /**
   * Kopiert die selektierte Alternative mit ihren Daten und fgt diese am Ende der Alternativen ein
   */
  protected void aktionKopieren()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem selectedItem = selection[0];
      String text = selectedItem.getText();

      TreeItem treeItemNeu = new TreeItem(_tree, SWT.NONE);
      treeItemNeu.setText("Kopie von " + text);
      
      
      TreeItem[] items = selectedItem.getItems();
      for (TreeItem subItem : items)
      {
        
        TreeItem subTreeItemNeu = new TreeItem(treeItemNeu, SWT.NONE);
        subTreeItemNeu.setData(subItem.getData());
        subTreeItemNeu.setText(subItem.getText());
      }
      
      _tree.setSelection(treeItemNeu);
      
//      TreeItem parent = null;
//
//      if (text.equals("Aktiv") || text.equals("Inaktiv"))
//      {
//        parent = selectedItem;
//      }
//      else
//      {
//        parent = selectedItem.getParentItem();
//      }
//
//      if (parent != null)
//      {
//        TreeItem subItem = new TreeItem(parent, SWT.NONE);
//        int index = parent.getItemCount() - 1;
//        subItem.setText(new String("Alternative: " + index));
//
//        Data data = (Data)selectedItem.getData();
//
//        if (data != null)
//        {
//          subItem.setData(data);
//        }
//
//        _tree.setSelection(subItem);
        Event event = new Event();
        event.item = treeItemNeu;
        _tree.notifyListeners(SWT.Selection, event);

//      }
    }
  }
  
  
  private boolean isFirstItem(TreeItem item){
    TreeItem parent = item.getParentItem();
    if (parent != null){
      TreeItem[] items = parent.getItems();
      for (int i = 0; i < items.length; i++)
      {
        TreeItem treeItem = items[i];
        if (i == 0 && treeItem.equals(item))
          return true;
      }
    }    
    return false;    
  }
  
  private boolean isLastItem(TreeItem item){    
    TreeItem parent = item.getParentItem();
    if (parent != null){
      TreeItem[] items = parent.getItems();
      for (int i = 0; i < items.length; i++)
      {
        TreeItem treeItem = items[i];
        if (i == (items.length-1) && treeItem.getText().equals(item.getText()))
          return true;
      }
    }    
    return false;    
  }
  

  /**
   * Holt das {@code DoModel} aus dem Parameter atg.sswSbaEinzelMassnahme und benachrichtigt den {@code GraphicalViewer}
   * zur Darstellung der Aenderung
   * 
   * @param die
   *          Daten aus der Atg
   */
  private void aktualisiereDarstellung(Data item)
  {
    if (item != null)
    {

      Array array = item.getItem("AnzeigenDatenstze").asArray();
      for (int i = 0; i < array.getLength(); i++)
      {
        Data item2 = array.getItem(i);
        SystemObject soAnzeige = item2.getItem("Anzeige").asReferenceValue().getSystemObject();
        if (soAnzeige != null)
        {
          AnzeigeModelTypAbstract doModel = DoModelHelper.holeAnzeigeModelTypAbstractVonSystemObject(
              (AnsichtEditPart)graphicalViewer.getContents(), soAnzeige,
              DoModelHelper.ENUM_DARSTELLUNG_KONTEXT.PARAMETER);
          
          SystemObject soAnzeigeInhalt = item2.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue()
              .getSystemObject();
          
          Data item3 = item2.getItem("WVZInhalt").getItem("DynamischerWechseltext");
          
          if (soAnzeigeInhalt != null){
            AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.wvzInhalt");
            Data cd = soAnzeigeInhalt.getConfigurationData(atg);
            String bi = cd.getItem("Bildinhalt").asTextValue().getText();
            AnzeigeInhaltTyp inhalt = AnzeigeInhaltTyp.setBildInhalt(bi, null);            
           
            if (doModel != null)
            {
              EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              final SetCommand cmd = new SetCommand(doModel, feature, inhalt);
              getCommandStack().execute(cmd);
            }            
          }else if (item3 != null){
            VisuAnzeigeKapsel kapsel = DaVHelper.holeDynamischerWechselTextAusData(item3);           
            if (doModel != null)
            {
              EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              final SetCommand cmd = new SetCommand(doModel, feature, kapsel);
              getCommandStack().execute(cmd);
            }           
          }
        }
      }
    }
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {

    CommandStack stack = graphicalViewer.getEditDomain().getCommandStack();

    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  /**
   * Holt den GEF-Viewer
   * 
   * @return viewer
   */
  public GraphicalViewer getGraphicalViewer()
  {
    return graphicalViewer;
  }

  /**
   * Setzt den GEF-Viewer
   * 
   * @param graphicalViewer
   */
  public void setGraphicalViewer(GraphicalViewer graphicalViewer)
  {
    // TODO Auto-generated method stub
    this.graphicalViewer = graphicalViewer;
  }

  /**
   * Holt das Baum-Objekt
   * 
   * @return
   */
  public Tree getTree()
  {
    return _tree;
  }
  
  
  /**
   * Liefert das KontextMenue zum Baum
   */
  private void erzeugeKontextMenue()
  {  
    _kontextMenuMgr = new MenuManager("#EinzelmassnahmenMenu");
    erzeugeAktionen();
    
    addtoKontextMenue((Action)_actionRegistry.getAction("Hinzufuegen"));
    addtoKontextMenue((Action)_actionRegistry.getAction("Loeschen"));
    addtoKontextMenue((Action)_actionRegistry.getAction("Hoch"));
    addtoKontextMenue((Action)_actionRegistry.getAction("Runter"));
    addtoKontextMenue((Action)_actionRegistry.getAction("Kopieren"));
    
    // Men erzeugen - Viewer wird dabei dem Men bekannt gemacht
    Menu menu = _kontextMenuMgr.createContextMenu(_tree);
    // Das Men dem Baum bekannt machen
    _tree.setMenu( menu );        
  }
  
  /**
   * Aktion im Kontextmen setzen
   * 
   * @param action  Aktion die im Kontextmen erscheinen soll
   */
  public void addtoKontextMenue(Action action)
  {
    
    if ( action != null )
      _kontextMenuMgr.add(action);
  }
  
  /**
   * Erzeugt die Aktionen zum Verwalten der Hierarchieobjekte
   */
  private void erzeugeAktionen()
  {
    _actionRegistry = new ActionRegistry();

    Action action1 = new Action() {
      public void run()
      {
        aktionHinzufuegen();
      }
    };
    action1.setId("Hinzufuegen");
    action1.setText("Hinzufgen");
    action1.setToolTipText("Erstellt eine neue Alternative");
    _actionRegistry.registerAction(action1);
    
    Action action2 = new Action() {
      public void run()
      {
        aktionLoeschen();
      }
    };
    action2.setId("Loeschen");
    action2.setText("Lschen");
    action2.setToolTipText("Lscht eine bestehende Alternative");
    _actionRegistry.registerAction(action2);
    
    Action action3 = new Action() {
      public void run()
      {
        aktionHoch();
      }
    };
    action3.setId("Hoch");
    action3.setText("Hoch");
    action3.setToolTipText("Verschiebt die Alternative um eine Position nach oben");
    _actionRegistry.registerAction(action3);
    
    Action action4 = new Action() {
      public void run()
      {
        aktionRunter();
      }
    };
    action4.setId("Runter");
    action4.setText("Runter");
    action4.setToolTipText("Verschiebt die Alternative um eine Position nach unten");
    _actionRegistry.registerAction(action4);
    
    Action action5 = new Action() {
      public void run()
      {
        aktionKopieren();
      }
    };
    action5.setId("Kopieren");
    action5.setText("Kopieren");
    action5.setToolTipText("Kopiert die Alternative und fgt diese am Ende der Alternativen ein");
    _actionRegistry.registerAction(action5);
  }

  private void aktionUmbenennen()
  {
    if (_tree.getSelection().length > 0){
      TreeItem treeItem = _tree.getSelection()[0];
      String text = treeItem.getText();
      InputDialog inputDialog = new InputDialog(this.getShell(), "Wanderbaustelle umbenennen", "Bitte Namen eingeben", text, null);
      int open = inputDialog.open();
      if (open == IStatus.OK){
        String nameNeu = inputDialog.getValue();          
        treeItem.setText(nameNeu);
        _tree.setSelection(treeItem);
      }          
    }
  }

}
