/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Sicht zum Auswhlen von Ebenen einer {@link Ansicht}.
 * 
 * 
 * @author Pittner
 * 
 */
public class AuswahlEbenenView extends ViewPart 
{

  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.auswahlEbenenView";

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private Composite _parent;

  private Composite compositeTop;

  private Group _gruppeUebersicht;

  private Table _tabelleEbenen;

  private TableViewer _tableViewerEbenen;

  private Composite compositeBottom;

  private Button buttonAbbrechen;

  private Button buttonHilfe;

  private EditPart _contents;

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;

    parent.setLayout(new GridLayout());
    createCompositeTop(parent);
    createCompositeBottom(parent);
    List<Ebene> sortList = new ArrayList<Ebene>();

    if (_contents != null)
    {
      AnsichtEditPart contents = (AnsichtEditPart)_contents;

      Ansicht ansicht = contents.getModel();
      StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();

      EList<Ebene> ebenen = darstellung.getEbenen();
      for (Ebene e : ebenen)
      {
        sortList.add(e);
      }
      _tableViewerEbenen.setInput(sortList);
    }
  }

  public TableViewer getTableViewerEbenen()
  {
    return _tableViewerEbenen;
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();
  }

  /**
   * Erzeugt gruppeUebersicht
   * 
   */
  private void createCompositeTop(Composite parent)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    compositeTop = new Composite(parent, SWT.NONE);
    compositeTop.setLayout(new GridLayout());
    compositeTop.setLayoutData(gridData);
    GridData gridDataTabelle = new GridData();
    gridDataTabelle.horizontalAlignment = GridData.FILL;
    gridDataTabelle.grabExcessHorizontalSpace = true;
    gridDataTabelle.grabExcessVerticalSpace = true;
    gridDataTabelle.verticalAlignment = GridData.FILL;
    GridData gridDataGruppeUebersicht = new GridData();
    gridDataGruppeUebersicht.horizontalAlignment = GridData.FILL;
    gridDataGruppeUebersicht.grabExcessHorizontalSpace = true;
    gridDataGruppeUebersicht.grabExcessVerticalSpace = true;
    gridDataGruppeUebersicht.verticalAlignment = GridData.FILL;
    _gruppeUebersicht = new Group(compositeTop, SWT.NONE);
    _gruppeUebersicht.setLayout(new GridLayout());
    _gruppeUebersicht.setLayoutData(gridDataGruppeUebersicht);
    _gruppeUebersicht.setText("Auswahl Ebenen:");
    _tabelleEbenen = new Table(_gruppeUebersicht, SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION | SWT.CHECK);
    _tabelleEbenen.setHeaderVisible(true);
    _tabelleEbenen.setLayoutData(gridDataTabelle);
    _tabelleEbenen.setLinesVisible(true);
    _tableViewerEbenen = new TableViewer(_tabelleEbenen);    

    TableColumn tcSicht = new TableColumn(_tabelleEbenen, SWT.LEFT);
    tcSicht.setText("Sichtbar:");
    tcSicht.setWidth(50);

    TableColumn tcTyp = new TableColumn(_tabelleEbenen, SWT.LEFT);
    tcTyp.setText("Name:");
    tcTyp.setWidth(150);

    TableColumn tcKey = new TableColumn(_tabelleEbenen, SWT.LEFT);
    tcKey.setText("ID:");
    tcKey.setWidth(350);

    _tableViewerEbenen.getTable().addListener(SWT.Selection, new Listener() {
      public void handleEvent(Event event)
      {
        if (event.detail == SWT.CHECK)
        {
          TableItem ti = (TableItem)event.item;
          Ebene e = (Ebene)ti.getData();
          if (e != null && ti != null)
          {
            e.setVisible(ti.getChecked());
          }
        }
      }
    });
  }

  private void createCompositeBottom(Composite parent)
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(parent, SWT.NONE);
    compositeBottom.setLayoutData(gridData3);
    compositeBottom.setLayout(rowLayout);

    buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionAbbrechen();
      }

    });

    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHilfe();
      }
    });
  }

  protected void aktionAbbrechen()
  {
    PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(this);
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  private void aktionHilfe()
  {
    // String contextId = Activator.PLUGIN_ID + ".auswahlParameter";
    // _debug.fine("ContextID: " + contextId);
    // PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  private class ContentProvider implements IStructuredContentProvider
  {
    @Override
    public Object[] getElements(Object inputElement)
    {
      java.util.List list = (java.util.List)inputElement;

      Object[] re = new Object[list.size()];

      for (int i = 0; i < list.size(); i++)
      {
        re[i] = list.get(i);
      }
      return re;
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class LabelProvider implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String returnString = null;

      switch (columnIndex)
      {
        case 0:
          returnString = ((Ebene)element).toString();
          break;
        case 1:
          returnString = ((Ebene)element).getName();
          break;

        case 2:
          returnString = ((Ebene)element).getName();
          break;

        default:
          returnString = "default";
      }

      return returnString;

    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  public void setContent(AnsichtEditPart contents)
  {
    this._contents = contents;

    if (contents != null)
    {

      _tableViewerEbenen.getTable().removeAll();

      Ansicht ansicht = contents.getModel();
      Darstellung darstellung = ansicht.getDarstellung();
      EList<Ebene> ebenen = darstellung.getEbenen();

      for (Ebene e : ebenen)
      {
        TableItem ti = new TableItem(_tableViewerEbenen.getTable(), SWT.NONE);
        ti.setChecked(e.isVisible());
        ti.setText(1, e.getName());
        ti.setText(2, e.toString());
        ti.setData(e);
      }
    }
  }
}
