/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchPage;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Liste von Anzeigequerschnitten, die markiert werden knnen. Mit den markierten AQ knnen Aktionen ausgefhrt werden,
 * z. B. "Betriebsart einstellen". *
 * 
 * @author Sans
 * 
 */
public class EinstellungenAnzeigeQuerschnitte extends ViewPart implements IDaVDataUpdateListener
{

  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.einstellungenAnzeigeQuerschnitte";

  private Composite _parent;

  private Composite _composite;

  private TableViewer _tvAnzeigeQuerschnitte;

  private Map<String, ModelObject> _mapModel = new LinkedHashMap<String, ModelObject>();

  private Action _actionBetriebsartEinstellen;

  private Action _actionHelligkeitEinstellen;

  public enum Betriebsart {
    AQNormalbetrieb("Normalbetrieb"), AQBlindbetrieb("Blindbetrieb"), AQVorortbetrieb("Vorortbetrieb"), AQAutarkbetrieb(
        "Autarkbetrieb"), AQTestbetrieb("Testbetrieb"), AQNotbetrieb("Notbetrieb"), AQExterneSteuerung(
        "Externe Steuerung"), AQTunnelbetrieb("Tunnelbetrieb"), Undefiniert("Undefiniert"), ;

    private String _name;

    private Betriebsart(String name)
    {
      _name = name;
    }

    @Override
    public String toString()
    {
      return _name;
    }

  }

  @Override
  public void init(IViewSite site) throws PartInitException
  {

    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null)
    {
      DataModel dataModel = connection.getDataModel();

      Collection<SystemObject> sorted = new TreeSet<SystemObject>(new Comparator<SystemObject>() {
        public int compare(SystemObject so1, SystemObject so2)
        {
          String name1 = so1.getName();
          String name2 = so2.getName();
          return (name1 != null ? name1 : "").compareTo(name2 != null ? name2 : "");
        }
      });
      sorted.addAll(dataModel.getType("typ.anzeigeQuerschnitt").getElements());

      for (SystemObject so : sorted)
      {
        String pid = so.getPid();
        _mapModel.put(pid, new ModelObject(pid, so.getName()));
      }

      List<String> list = Arrays.asList(_mapModel.keySet().toArray(new String[_mapModel.keySet().size()]));
      DaVKommunikationDobj.getInstanz().anmeldenAufDatenMQDaten(list, "atg.anzeigeQuerschnittEigenschaftIst",
          "asp.aqZustand");
      DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);
    }
    super.init(site);
  }

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;
    
    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".aqAuswahl");
    
    _parent.setLayout(new GridLayout());

    createActions();

    createContent();
    _tvAnzeigeQuerschnitte.setInput(_mapModel.values());

    createContextMenu();
  }

  private void createContent()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _composite = new Composite(_parent, SWT.NONE);
    _composite.setLayout(new GridLayout());
    _composite.setLayoutData(gridData);

    GridData gridDataTabelle = new GridData();
    gridDataTabelle.horizontalAlignment = GridData.FILL;
    gridDataTabelle.grabExcessHorizontalSpace = true;
    gridDataTabelle.grabExcessVerticalSpace = true;
    gridDataTabelle.verticalAlignment = GridData.FILL;

    _tvAnzeigeQuerschnitte = new TableViewer(_composite, SWT.MULTI | SWT.FULL_SELECTION);

    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeigeQuerschnitte, SWT.LEFT);
    column1.getColumn().setWidth(180);
    column1.getColumn().setText("Name");

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeigeQuerschnitte, SWT.LEFT);
    column2.getColumn().setWidth(180);
    column2.getColumn().setText("Pid");

    TableViewerColumn column3 = new TableViewerColumn(_tvAnzeigeQuerschnitte, SWT.LEFT);
    column3.getColumn().setWidth(120);
    column3.getColumn().setText("Betriebsart (Ist)");

    TableViewerColumn column4 = new TableViewerColumn(_tvAnzeigeQuerschnitte, SWT.LEFT);
    column4.getColumn().setWidth(120);
    column4.getColumn().setText("Helligkeit (Ist)");

    _tvAnzeigeQuerschnitte.setContentProvider(new ContentProvider());
    _tvAnzeigeQuerschnitte.setLabelProvider(new LabelProvider());
    _tvAnzeigeQuerschnitte.getTable().setLayoutData(gridDataTabelle);
    _tvAnzeigeQuerschnitte.getTable().setLinesVisible(true);
    _tvAnzeigeQuerschnitte.getTable().setHeaderVisible(true);

    _tvAnzeigeQuerschnitte.addSelectionChangedListener(new ISelectionChangedListener() {

      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        ISelection selection = event.getSelection();
        if (selection instanceof StructuredSelection)
        {
          enableDisableActions((StructuredSelection)selection);
        }
      }
    });
  }

  private void createActions()
  {
    if (_actionBetriebsartEinstellen == null)
    {
      _actionBetriebsartEinstellen = new Action() {
        public void run()
        {
          ISelection selection = _tvAnzeigeQuerschnitte.getSelection();
          if (selection instanceof StructuredSelection)
          {
            String secId = "";
            StructuredSelection ss = (StructuredSelection)selection;
            Iterator it = ss.iterator();
            while (it.hasNext())
            {
              Object obj = it.next();
              if (obj instanceof ModelObject)
              {
                if (!secId.isEmpty())
                {
                  secId += ";";
                }
                secId += ((ModelObject)obj).getPid();
              }
            }

            IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
            IWorkbenchPage page = dw.getActivePage();
            try
            {
              page.showView(BetriebsartEinstellen.ID, secId, IWorkbenchPage.VIEW_CREATE);
              IViewReference ref = page.findViewReference(BetriebsartEinstellen.ID, secId);
              ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(ref);
              IWorkbenchPart wbPart = ref.getPart(false);
              Shell shell = wbPart.getSite().getShell();
              shell.setSize(500, 330);

            }
            catch (PartInitException e)
            {
              e.printStackTrace(System.err);
            }

          }

        }
      };
      _actionBetriebsartEinstellen.setId(ID + ".actionBetriebsartEinstellen");
      _actionBetriebsartEinstellen.setText("Betriebsart einstellen");
      _actionBetriebsartEinstellen
          .setToolTipText("Betriebsart einstellen fr einen, mehrere oder alle Anzeigequerschnitte");
      _actionBetriebsartEinstellen.setEnabled(false);
    }
    if (_actionHelligkeitEinstellen == null)
    {
      _actionHelligkeitEinstellen = new Action() {
        public void run()
        {
          ISelection selection = _tvAnzeigeQuerschnitte.getSelection();
          if (selection instanceof StructuredSelection)
          {
            String secId = "";
            StructuredSelection ss = (StructuredSelection)selection;
            Iterator it = ss.iterator();
            while (it.hasNext())
            {
              Object obj = it.next();
              if (obj instanceof ModelObject)
              {
                if (!secId.isEmpty())
                {
                  secId += ";";
                }
                secId += ((ModelObject)obj).getPid();
              }
            }

            IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
            IWorkbenchPage page = dw.getActivePage();
            try
            {
              page.showView(HelligkeitEinstellen.ID, secId, IWorkbenchPage.VIEW_CREATE);
              IViewReference ref = page.findViewReference(HelligkeitEinstellen.ID, secId);
              ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(ref);
              IWorkbenchPart wbPart = ref.getPart(false);
              Shell shell = wbPart.getSite().getShell();
              shell.setSize(500, 330);

            }
            catch (PartInitException e)
            {
              e.printStackTrace(System.err);
            }

          }

        }
      };
      _actionHelligkeitEinstellen.setId(ID + ".actionHelligkeitEinstellen");
      _actionHelligkeitEinstellen.setText("Helligkeit einstellen");
      _actionHelligkeitEinstellen
          .setToolTipText("Helligkeit einstellen fr einen, mehrere oder alle Anzeigequerschnitte");
      _actionHelligkeitEinstellen.setEnabled(false);
    }

    IMenuManager mm = getViewSite().getActionBars().getMenuManager();
    mm.add(_actionBetriebsartEinstellen);
    mm.add(_actionHelligkeitEinstellen);
  }

  private void enableDisableActions(StructuredSelection ss)
  {
    if (_actionBetriebsartEinstellen != null)
    {
      _actionBetriebsartEinstellen.setEnabled(!ss.isEmpty());
    }
    if (_actionHelligkeitEinstellen != null)
    {
      _actionHelligkeitEinstellen.setEnabled(!ss.isEmpty());
    }
  }

  private void createContextMenu()
  {
    MenuManager menuMgr = new MenuManager("#PopupMenu");
    menuMgr.setRemoveAllWhenShown(true);
    menuMgr.addMenuListener(new IMenuListener() {
      public void menuAboutToShow(IMenuManager manager)
      {
        fillContextMenu(manager);
      }
    });

    Menu menu = menuMgr.createContextMenu(_tvAnzeigeQuerschnitte.getControl());
    _tvAnzeigeQuerschnitte.getControl().setMenu(menu);
    getSite().registerContextMenu(menuMgr, _tvAnzeigeQuerschnitte);
  }

  private void fillContextMenu(IMenuManager manager)
  {
    manager.add(_actionBetriebsartEinstellen);
    manager.add(_actionHelligkeitEinstellen);
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();
  }

  @Override
  public void dispose()
  {
    super.dispose();
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null)
    {
      DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
    }
  }

  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
    ResultData rd = e.getResult();
    DataDescription dd = rd.getDataDescription();
    if (dd.getAttributeGroup().getPid().equals("atg.anzeigeQuerschnittEigenschaftIst")
        && dd.getAspect().getPid().equals("asp.aqZustand"))
    {
      SystemObject so = rd.getObject();
      ModelObject modelObject = _mapModel.get(so.getPid());
      if (modelObject != null)
      {
        modelObject.setBetriebsart(null);
        Data data = rd.getData();
        if (data != null)
        {
          String ba = data.getItem("Eigenschaft").getTextValue("Betriebszustand").getText();
          if (ba != null)
          {
            Betriebsart betriebsart = Betriebsart.valueOf(ba);
            if (betriebsart != null)
            {
              modelObject.setBetriebsart(betriebsart.toString());
            }
          }
          String hk = data.getItem("Eigenschaft").getTextValue("Helligkeit").getText();
          if (hk != null)
          {
            modelObject.setHelligkeit(hk);
          }
        }
        else
        {
          modelObject.setBetriebsart("Keine Daten");
          modelObject.setHelligkeit("Keine Daten");
        }
        _tvAnzeigeQuerschnitte.refresh();
      }
    }
  }

  private static class ModelObject
  {

    private String _pid;

    private String _name;

    private String _betriebsart;

    private String _helligkeit;

    public ModelObject(String pid, String name)
    {
      _pid = pid;
      _name = name;
    }

    public String getBetriebsart()
    {
      return _betriebsart;
    }

    public void setBetriebsart(String betriebsart)
    {
      _betriebsart = betriebsart;
    }

    public String getHelligkeit()
    {
      return _helligkeit;
    }

    public void setHelligkeit(String helligkeit)
    {
      _helligkeit = helligkeit;
    }

    public String getPid()
    {
      return _pid;
    }

    public String getName()
    {
      return _name;
    }

  }

  private static class ContentProvider implements IStructuredContentProvider
  {
    @Override
    public Object[] getElements(Object inputElement)
    {
      return ((Collection)inputElement).toArray();
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }
  }

  private static class LabelProvider implements ITableLabelProvider
  {
    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String result = null;
      ModelObject modelObject = (ModelObject)element;

      switch (columnIndex)
      {
        case 0:
          String name = modelObject.getName();
          result = name != null ? name : "";
          break;
        case 1:
          String pid = modelObject.getPid();
          result = pid != null ? pid : "";
          break;
        case 2:
          String betriebsart = modelObject.getBetriebsart();
          result = betriebsart != null ? betriebsart : "";
          break;
        case 3:
          String helligkeit = modelObject.getHelligkeit();
          result = helligkeit != null ? helligkeit : "";
          break;
        default:
          result = "";
          break;
      }

      return result;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }
  }
}
