/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.util.Fg6Helper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.Fg6Helper.FG6_STATUS;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Sicht zur Anzeige von FG6-Daten.
 * 
 * @author Pittner
 * 
 */ 
public class Fg6View extends ViewPart
{

  public final static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.Fg6View";

  public final static String UNDEF = "undefiniert";

  public final static String NO_DATA = "Keine Daten";

  public final static int COL_WIDTH = 100;

  public final static SimpleDateFormat TIME_FORMAT = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  private String _pidSm;

  private CTabFolder _tabFolder;

  private DataReceiver _dataReceiver = null;

  @Override
  public void init(IViewSite site) throws PartInitException
  {
    String secondaryId = site.getSecondaryId();

    if (secondaryId != null)
    {
      String[] components = secondaryId.split("#");
      if (components != null && components.length > 0)
      {
        _pidSm = components[0];
      }
    }

    String partName = "SSt";
    if (_pidSm != null && !"".equals(_pidSm))
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      if (connection != null)
      {
        SystemObject so = connection.getDataModel().getObject(_pidSm);
        if (so != null)
          partName += ' ' + so.getNameOrPidOrId();
        else
          System.out.println("SystemObjekt fr SM ist null: " + _pidSm);
      }
      else
      {
        partName += ' ' + _pidSm;
      }
    }
    setPartName(partName);

    super.init(site);
  }

  @Override
  public void createPartControl(Composite parent)
  {

    String contDesc = "Knotennummer ??";
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    List<String> davAnmeldung = new ArrayList<String>();

    _tabFolder = new CTabFolder(parent, SWT.TOP);

    if (connection != null)
    {
      DataModel dataModel = connection.getDataModel();
      SystemObject sm = dataModel.getObject(_pidSm);
      if (sm != null && sm.isOfType("typ.steuerModul"))
      {
        AttributeGroup atg = dataModel.getAttributeGroup("atg.gert");
        if (atg != null)
        {
          Data data = sm.getConfigurationData(atg);
          int knotenNummer = data.getItem("KnotenNummer").asUnscaledValue().intValue();
          contDesc = "Knotennummer" + ' ' + knotenNummer;
        }
        else
        {
          System.err.println("Attributgruppe atg.gert ist nicht vorhanden");
        }
      }

      for (SystemObject eak : getFg6Eak(connection, _pidSm))
      {
        if (eak != null && eak.isOfType("typ.eak"))
        {
          String textEak = "EAK " + eak.getPid();
          AttributeGroup atgEak = dataModel.getAttributeGroup("atg.eak");
          if (atgEak != null)
          {
            Data data = eak.getConfigurationData(atgEak);
            if (data != null)
            {
              int osi2Adresse = data.getItem("OSI2Adresse").asUnscaledValue().intValue();
              textEak = "EAK" + osi2Adresse;
            }
          }
          else
          {
            System.err.println("Attributgruppe atg.eak ist nicht vorhanden");
          }
          CTabItem item = new CTabItem(_tabFolder, SWT.NONE);
          item.setText(textEak);
          item.setData(FG6_STATUS.UNDEFINED);
          CTabFolder tabFolder = new CTabFolder(_tabFolder, SWT.TOP);
          tabFolder.setData(item);
          item.setControl(tabFolder);

          for (SystemObject de : getFg6De(connection, _pidSm, eak.getPid()))
          {
            if (de != null && de.isOfType("typ.deVlt"))
            {
              String textDe = "DE " + de.getPid();
              AttributeGroup atgDe = dataModel.getAttributeGroup("atg.de");
              if (atgDe != null)
              {
                Data data = de.getConfigurationData(atgDe);
                if (data != null)
                {
                  String bezeichnung = data.getItem("Bezeichnung").asTextValue().getText();
                  int deKanal = data.getItem("DEKanal").asUnscaledValue().intValue();
                  textDe = "DE" + deKanal + ' ' + bezeichnung;
                }
              }
              else
              {
                System.err.println("Attributgruppe atg.de ist nicht vorhanden");
              }

              item = new CTabItem(tabFolder, SWT.NONE);
              item.setText(textDe);
              TableComposite comp = new TableComposite(tabFolder, SWT.NONE);

              item.setControl(comp);
              item.setData(FG6_STATUS.UNDEFINED);

              tabFolder.setData(de.getPid(), item);

              _tabFolder.setData(de.getPid(), comp);
              davAnmeldung.add(de.getPid());
            }
          }
          if (tabFolder.getItemCount() > 0)
          {
            tabFolder.setSelection(0);
          }
        }
      }
    }

    if (_tabFolder.getItemCount() > 0)
    {
      _tabFolder.setSelection(0);
    }
    contDesc += "  FG6";
    setContentDescription(contDesc);

    davAnmeldung(davAnmeldung);
  }

  private Collection<SystemObject> getFg6Eak(ClientDavInterface connection, String pidSm)
  {

    Map<Integer, SystemObject> result = new TreeMap<Integer, SystemObject>();
    DataModel dataModel = connection.getDataModel();

    for (String pidEak : Fg6Helper.getFg6Eak(connection, pidSm))
    {
      SystemObject eak = dataModel.getObject(pidEak);
      if (eak != null && eak.isOfType("typ.eak"))
      {
        AttributeGroup atg = dataModel.getAttributeGroup("atg.eak");
        if (atg != null)
        {
          Data data = eak.getConfigurationData(atg);
          if (data != null)
          {
            int osi2Adresse = data.getItem("OSI2Adresse").asUnscaledValue().intValue();
            result.put(osi2Adresse, eak);
          }
        }
        else
        {
          System.err.println("Attributgruppe atg.eak ist nicht vorhanden");
        }
      }
    }
    return result.values();
  }

  private Collection<SystemObject> getFg6De(ClientDavInterface connection, String pidSm, String pidEak)
  {

    Map<Integer, SystemObject> result = new TreeMap<Integer, SystemObject>();
    DataModel dataModel = connection.getDataModel();

    for (String pidDe : Fg6Helper.getFg6De(connection, pidSm, pidEak))
    {
      SystemObject de = dataModel.getObject(pidDe);
      if (de != null && de.isOfType("typ.deVlt"))
      {
        AttributeGroup atg = dataModel.getAttributeGroup("atg.de");
        if (atg != null)
        {
          Data data = de.getConfigurationData(atg);
          if (data != null)
          {
            int deKanal = data.getItem("DEKanal").asUnscaledValue().intValue();
            result.put(deKanal, de);
          }
        }
        else
        {
          System.err.println("Attributgruppe atg.de ist nicht vorhanden");
        }
      }
    }
    return result.values();
  }

  @Override
  public void setFocus()
  {
  }

  @Override
  public void dispose()
  {
    davAbmeldung();
    super.dispose();
  }

  private class TableComposite extends Composite
  {
    public final Label TIME_LABEL;

    public final Label ATG_LABEL;

    public final TableViewer TABLE_VIEWER;

    TableComposite(Composite parent, int style)
    {
      super(parent, style);

      GridData gd1 = new GridData();
      gd1.horizontalAlignment = GridData.FILL;
      gd1.grabExcessHorizontalSpace = true;
      gd1.grabExcessVerticalSpace = true;
      gd1.verticalAlignment = GridData.FILL;

      GridData gd2 = new GridData();
      gd2.horizontalAlignment = GridData.FILL;
      gd2.grabExcessHorizontalSpace = true;

      setLayout(new GridLayout());
      setLayoutData(gd1);

      TIME_LABEL = new Label(this, SWT.NONE);
      TIME_LABEL.setLayoutData(gd2);
      TIME_LABEL.setText("--.--.---- --:--:--");

      ATG_LABEL = new Label(this, SWT.NONE);
      ATG_LABEL.setLayoutData(gd2);
      ATG_LABEL.setText(NO_DATA);

      TABLE_VIEWER = new TableViewer(this, SWT.HIDE_SELECTION);

      TableViewerColumn column1 = new TableViewerColumn(TABLE_VIEWER, SWT.NONE);
      column1.getColumn().setWidth(COL_WIDTH);
      column1.getColumn().setText("Attribut");

      TableViewerColumn column2 = new TableViewerColumn(TABLE_VIEWER, SWT.NONE);
      column2.getColumn().setWidth(100);
      column2.getColumn().setText("Wert");

      TABLE_VIEWER.setContentProvider(new ArrayContentProvider());
      TABLE_VIEWER.setLabelProvider(new TableLabelProvider());
      TABLE_VIEWER.getTable().setLayoutData(gd1);
      TABLE_VIEWER.getTable().setLinesVisible(true);
      TABLE_VIEWER.getTable().setHeaderVisible(false);
    }
  }

  private void davAnmeldung(List<String> davAnmeldung)
  {

    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null)
    {
      _dataReceiver = new DataReceiver(davAnmeldung);
    }
  }

  private void davAbmeldung()
  {
    if (_dataReceiver != null)
    {
      _dataReceiver.dispose();
      _dataReceiver = null;
    }
  }

  private class DataReceiver implements IDaVDataUpdateListener
  {
    private Set<String> _setPid = new HashSet<String>();

    public DataReceiver(List<String> davAnmeldung)
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      if (connection != null)
      {
        if (davAnmeldung.size() > 0)
        {
          for (String pidAtg : Fg6Helper.FG6_ATG)
          {
            DaVKommunikationDobj.getInstanz().anmeldenAufDatenSm(davAnmeldung, pidAtg, Fg6Helper.FG6_ASP);
            System.out.println(davAnmeldung.size() + " DE Fg6 angemeldet auf " + pidAtg + "/" + Fg6Helper.FG6_ASP);
          }
          DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);
          _setPid.addAll(davAnmeldung);
        }
      }
    }

    public void dispose()
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      if (connection != null)
      {
        if (!_setPid.isEmpty())
        {
          DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
        }
      }
    }

    @Override
    public void daVDataUpdate(DaVDataUpdateEvent e)
    {

      ResultData result = e.getResult();

      Aspect asp = result.getDataDescription().getAspect();
      if (!Fg6Helper.FG6_ASP.equals(asp.getPid()))
      {
        return;
      }

      AttributeGroup atg = result.getDataDescription().getAttributeGroup();
      if (!Arrays.asList(Fg6Helper.FG6_ATG).contains(atg.getPid()))
      {
        return;
      }

      SystemObject so = result.getObject();
      if (!_setPid.contains(so.getPid()))
      {
        return;
      }

      System.out.println("FG6-Daten empfangen fr SSt " + _pidSm + "\n\t DE " + so.getPid() + "\n\t "
          + result.getData());

      TableComposite comp = (TableComposite)_tabFolder.getData(so.getPid());
      if (comp != null)
      {
        FG6_STATUS statusDe;
        long timeStamp = result.getDataTime();
        Data data = result.getData();
        String atgComp = (String)comp.getData();
        List<Data> model = new ArrayList<Data>();

        if (data != null)
        {
          Iterator it = data.iterator();
          while (it.hasNext())
          {
            Data item = (Data)it.next();
            if (item.isPlain())
            {
              model.add(item);
            }
          }
          if (atgComp != null)
          {
            if (atgComp.equals(atg.getPid()))
            {
              // Es werden Daten derselbenATG empfangen wie vorher
              comp.TIME_LABEL.setText(TIME_FORMAT.format(new Date(timeStamp)));
              comp.TABLE_VIEWER.setInput(model);
              for (TableColumn tc : comp.TABLE_VIEWER.getTable().getColumns())
              {
                tc.pack();
              }
            }
            else
            {
              // Es werden Daten einer ANDEREN ATG empfangen
              comp.setData(atg.getPid());
              comp.TIME_LABEL.setText(TIME_FORMAT.format(new Date(timeStamp)));
              comp.ATG_LABEL.setText("ATG: " + atg.getName());
              comp.TABLE_VIEWER.setInput(model);
              for (TableColumn tc : comp.TABLE_VIEWER.getTable().getColumns())
              {
                tc.pack();
              }
            }
          }
          else
          {
            // Es werden Daten empfangen, vorher war "keine Daten"
            comp.setData(atg.getPid());
            comp.TIME_LABEL.setText(TIME_FORMAT.format(new Date(timeStamp)));
            comp.ATG_LABEL.setText("ATG: " + atg.getName());
            comp.TABLE_VIEWER.setInput(model);
            for (TableColumn tc : comp.TABLE_VIEWER.getTable().getColumns())
            {
              tc.pack();
            }
          }
          statusDe = Fg6Helper.getFg6Status(atg.getPid(), data);
        }
        else
        {
          if (atgComp != null)
          {
            if (atgComp.equals(atg.getPid()))
            {
              // Vorher empfangene ATG geht auf "keine Daten"
              comp.TIME_LABEL.setText(TIME_FORMAT.format(new Date(timeStamp)));
              comp.ATG_LABEL.setText(NO_DATA);
              comp.TABLE_VIEWER.setInput(model);
              for (TableColumn tc : comp.TABLE_VIEWER.getTable().getColumns())
              {
                tc.setWidth(COL_WIDTH);
              }
              comp.setData(null);
              statusDe = FG6_STATUS.FAILURE; // Kommunikationsstrung
            }
            else
              // aktuelle ATG "keine Daten", aber vorher Daten anderer ATG empfangen
              statusDe = FG6_STATUS.UNDEFINED; // tue nichts
          }
          else
            statusDe = FG6_STATUS.FAILURE; // Kommunikationsstrung
        }

        if (statusDe != FG6_STATUS.UNDEFINED)
        {
          CTabFolder tabFolder = (CTabFolder)comp.getParent();
          CTabItem itemEak = (CTabItem)tabFolder.getData();
          CTabItem itemDe = (CTabItem)tabFolder.getData(so.getPid());
          FG6_STATUS statusEak = FG6_STATUS.UNDEFINED;

          itemDe.setData(statusDe);
          Image image = statusDe.getImage();
          System.out.println("ATG: " + atg.getPid() + "\tStatus: " + statusDe + "\tImage: " + image);
          itemDe.setImage(image);

          for (CTabItem item : tabFolder.getItems())
          {           
            FG6_STATUS status = (FG6_STATUS)item.getData();
            switch (status)
            {
              case FAILURE:
                statusEak = FG6_STATUS.FAILURE;
                break;
              case DISTURBED:
                if (statusEak != FG6_STATUS.FAILURE)
                  statusEak = FG6_STATUS.DISTURBED;
                break;
              case OK:
                if (statusEak != FG6_STATUS.FAILURE && statusEak != FG6_STATUS.DISTURBED)
                  statusEak = FG6_STATUS.OK;
                break;
            }
          }
          if (statusEak != FG6_STATUS.UNDEFINED)
          {
            itemEak.setData(statusEak);
            itemEak.setImage(statusEak.getImage());
          }
        }
      }
      else
      {
        new IllegalStateException("Unerwartetes Objekt " + so.getPid()).printStackTrace(System.err);
      }
    }
  }

  public class TableLabelProvider implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String returnString = UNDEF;

      Data data = (Data)element;

      switch (columnIndex)
      {
        case 0:
          returnString = data.getName();
          break;
        case 1:
          returnString = data.asTextValue().getText();
          break;
        default:
      }
      return returnString;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
      // TODO Auto-generated method stub

    }

    @Override
    public void dispose()
    {
      // TODO Auto-generated method stub

    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
      // TODO Auto-generated method stub
    }
  }

}
