/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.editors;

import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.widgets.Text;

/**
 * KeyListener, welcher nur die Eingabe von gltigen Fliekommazahlen erlaubt.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: DoubleKeyListener.java,v 1.2 2014/11/14 07:29:40 Pittner Exp $
 */
public class DoubleKeyListener extends KeyAdapter {

	/** Das Text-Widget, fr welches dieser KeyListener genutzt werden soll. */
	private final Text textWidget;

	/** Die Anzahl der erlaubten Nachkommastellen. */
	private final int nachkommaStellen;

	/**
	 * Der Konstruktor.
	 * 
	 * @param textWidget
	 *            das Text-Widget, fr welches dieser KeyListener genutzt werden
	 *            soll
	 * @param nachkommaStellen
	 *            die Anzahl der erlaubten Nachkommastellen, muss ein Wert >= 0
	 *            sein
	 */
	public DoubleKeyListener(final Text textWidget, final int nachkommaStellen) {
		this.textWidget = textWidget;
		this.nachkommaStellen = nachkommaStellen;
	}

	@Override
	public void keyPressed(final KeyEvent e) {
		String text = textWidget.getText();
		int caretPos = textWidget.getCaretPosition();
		if (textWidget.getSelectionCount() > 0) {
			caretPos = textWidget.getSelection().x;
			text = text.substring(0, caretPos) + text.substring(textWidget.getSelection().y);
		}

		if (Character.isDigit(e.character)) {
			// Es darf eine Zahl nur eingefgt werden, wenn dadurch nicht zu
			// viele Nachkommastellen entstehen.
			final int index = text.indexOf(',');
			if (index > -1 && caretPos > index) {
				e.doit = text.length() - index <= nachkommaStellen;
			}
		} else if (e.character == ',') {
			// Es darf nur ein Komma eingefgt werden, wenn noch keines
			// existiert. Auerdem darf es nicht an erster Stelle oder an einer
			// Stelle eingefgt werden, sodass zuviele Nachkommastelle
			// entstehen.
			e.doit = nachkommaStellen > 0 && !text.contains(",")
					&& caretPos > 0
					&& text.length() - caretPos <= nachkommaStellen;
		} else if (e.keyCode != 127 && e.keyCode != 8 && e.keyCode != 16777219
				&& e.keyCode != 16777220 && e.keyCode != 16777223
				&& e.keyCode != 16777224) {
			// Alle Eingaben die nicht zum Navigieren (Pos1, Ende, Cursor-Tasten
			// Links, Rechts) bzw. zum Lschen dienen, werden geblockt.
			e.doit = false;
		}
	}
}
