/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.views;

/**
 * Bean, welches die Einstellung der Ansicht 'RDS-Meldungen' speichert. Dies
 * beinhaltet die Spaltenbreiten, die Spaltensichtbarkeiten, die Sortierung
 * sowie die CheckStates der Aktionen.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: AbstractBauEditorViewPartEinstellungen.java,v 1.2 2014/11/14 07:29:42 Pittner Exp $
 */
public class AbstractBauEditorViewPartEinstellungen {

	/** Check-State der Aktion 'Verknpfe mit Editor'. */
	private boolean editorActionChecked;

	/**
	 * Check-State der Aktion 'Verknpfe Selektionen' bzw. 'Verknpfe mit
	 * Navigator'.
	 */
	private boolean verknuepfeActionChecked;

	/**
	 * Der Default-Konstruktor.
	 */
	public AbstractBauEditorViewPartEinstellungen() {
		editorActionChecked = true;
		verknuepfeActionChecked = true;
	}

	/**
	 * Liefert <code>true</code> zurck, wenn die Aktion 'Verknpfe mit Editor'
	 * aktiviert ist.
	 * 
	 * @return <code>true</code>, wenn die Aktion 'Verknpfe mit Editor'
	 *         aktiviert ist, ansonsten false
	 */
	public boolean isEditorActionChecked() {
		return editorActionChecked;
	}

	/**
	 * Setzt das Flag, welches angibt, ob die Aktion 'Verknpfe mit Editor'
	 * aktiviert sein soll.
	 * 
	 * @param editorActionChecked
	 *            der Zustand der Aktion
	 */
	public void setEditorActionChecked(final boolean editorActionChecked) {
		this.editorActionChecked = editorActionChecked;
	}

	/**
	 * Liefert <code>true</code> zurck, wenn die Aktion 'Verknpfe Selektionen'
	 * bzw. 'Verknpfe mit Navigator' aktiviert ist.
	 * 
	 * @return <code>true</code>, wenn die Aktion 'Verknpfe Selektionen' bzw.
	 *         'Verknpfe mit Navigator' aktiviert ist, ansonsten false
	 */
	public boolean isVerknuepfeActionChecked() {
		return verknuepfeActionChecked;
	}

	/**
	 * Setzt das Flag, welches angibt, ob die Aktion 'Verknpfe Selektionen'
	 * bzw. 'Verknpfe mit Navigator' aktiviert sein soll.
	 * 
	 * @param verknuepfeActionChecked
	 *            der Zustand der Aktion
	 */
	public void setVerknuepfeActionChecked(final boolean verknuepfeActionChecked) {
		this.verknuepfeActionChecked = verknuepfeActionChecked;
	}
}
