/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.views;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;

import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.BaustellenWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.EngstellenWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.UnfallWrapper;

/**
 * {@link ViewerSorter} fr den TableViewer der RDS-Meldungen-Ansicht.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: EngstellenTableSorter.java,v 1.2 2014/11/14 07:29:42 Pittner Exp $
 */
public class EngstellenTableSorter extends ViewerSorter {

	/**
	 * Maximaler Wert fr den Spaltenindex.
	 */
	private static final int MAX_SPALTEN_INDEX = 8;

	/**
	 * Aufsteigende Sortierreihenfolge.
	 */
	public static final int AUFSTEIGEND = 0;

	/**
	 * Absteigende Sortierreihenfolge.
	 */
	public static final int ABSTEIGEND = 1;

	/**
	 * Der Index der aktuell ausgewhlten Spalte nach welcher sortiert werden
	 * soll.
	 */
	private int aktuellerSpaltenIndex;

	/**
	 * Der aktuelle Sortierrichtung.
	 */
	private int aktuelleRichtung = AUFSTEIGEND;

	/**
	 * Der Default-Konstruktor.
	 * 
	 * @param spaltenIndex
	 *            der aktuelle Spaltenindex
	 * @param richtung
	 *            die aktuelle Richtung
	 */
	public EngstellenTableSorter(final Integer spaltenIndex,
			final Integer richtung) {
		if (spaltenIndex != null) {
			aktuellerSpaltenIndex = spaltenIndex;
		}
		if (aktuellerSpaltenIndex < 0
				|| aktuellerSpaltenIndex > MAX_SPALTEN_INDEX) {
			aktuellerSpaltenIndex = 0;
		}
		if (richtung != null) {
			aktuelleRichtung = richtung;
		}
		if (aktuelleRichtung != AUFSTEIGEND && aktuelleRichtung != ABSTEIGEND) {
			aktuelleRichtung = AUFSTEIGEND;
		}
	}

	/**
	 * Setzt die Spalte nach welcher die Tabelle sortiert werden soll.
	 * 
	 * @param spaltenIndex
	 *            der Index der Spalte
	 */
	public void setSpalte(final int spaltenIndex) {
		if (spaltenIndex == aktuellerSpaltenIndex) {
			// Same column as last sort; toggle the direction
			aktuelleRichtung = 1 - aktuelleRichtung;
		} else {
			// New column; do an ascending sort
			aktuellerSpaltenIndex = spaltenIndex;
			aktuelleRichtung = AUFSTEIGEND;
		}
	}

	/**
	 * Liefert den Indes der Spalte zurck, nach welcher die Tabelle aktuell
	 * sortiert wird.
	 * 
	 * @return den Indes der Spalte, nach welcher die Tabelle aktuell sortiert
	 *         wird
	 */
	public int getSpalte() {
		return aktuellerSpaltenIndex;
	}

	/**
	 * Liefert die aktuelle Richtung zurck, in welche die aktuelle Spalte der
	 * Tabelle sortiert wird.
	 * 
	 * @return die aktuelle Richtung, in welche die aktuelle Spalte der Tabelle
	 *         sortiert wird
	 */
	public int getRichtung() {
		return aktuelleRichtung;
	}

	@Override
	public int compare(final Viewer viewer, final Object o1, final Object o2) {
		if (!(o1 instanceof EngstellenWrapper)
				|| !(o2 instanceof EngstellenWrapper)) {
			return compare(viewer, o1, o2);
		}

		final EngstellenWrapper w1 = (EngstellenWrapper) o1;
		final EngstellenWrapper w2 = (EngstellenWrapper) o2;
		int wert = 0;
		switch (aktuellerSpaltenIndex) {
		case 0: // Typ
			if (w1 instanceof BaustellenWrapper && w2 instanceof UnfallWrapper) {
				wert = 1;
			} else if (w1 instanceof UnfallWrapper
					&& w1 instanceof BaustellenWrapper) {
				wert = -1;
			}
			wert = 0;
			break;
		case 1: // Name
			if (w1.getName() != null) {
				wert = w1.getName().compareTo(w2.getName());
			}
			break;
		case 2: // Richtung
			if (w1.getRichtung() != null) {
				wert = w1.getRichtung().compareTo(w2.getRichtung());
			}
			break;
		case 3: // Strae
			if (w1.getStrasse() != null && w2.getStrasse() != null) {
				wert = w1.getStrasse().getName().compareTo(
						w2.getStrasse().getName());
			}
			break;
		case 4: // von
			if (w1.getStartKnoten() != null && w2.getStartKnoten() != null) {
				wert = w1.getStartKnoten().getName().compareTo(
						w2.getStartKnoten().getName());
			}
			break;
		case 5: // bis
			if (w1.getEndKnoten() != null && w2.getEndKnoten() != null) {
				wert = w1.getEndKnoten().getName().compareTo(
						w2.getEndKnoten().getName());
			}
			break;
		case 6: // Startzeit
			if (w1.getStartZeit() != null) {
				wert = w1.getStartZeit().compareTo(w2.getStartZeit());
			}
			break;
		case 7: // Endzeit
			if (w1.getDauer() != null && w2.getDauer() != null) {
				if (w1.getDauer() > 0 && w2.getDauer() > 0) {
					wert = w1.getEndZeit().compareTo(w2.getEndZeit());
				} else {
					wert = new Boolean(w1.getDauer() > 0)
							.compareTo(new Boolean(w2.getDauer() > 0));
				}
			}
			break;
		case 8: // Anzahl gesperrte Fahrstreifen
			if (w1.getLageGesperrterFahrstreifen() != null
					&& w2.getLageGesperrterFahrstreifen() != null) {
				wert = new Integer(w1.getLageGesperrterFahrstreifen().size())
						.compareTo(new Integer(w2
								.getLageGesperrterFahrstreifen().size()));
			}
			break;
		default:
			wert = 0;
		}

		// Bei absteigender Richtung, dreh den berechneten Wert um
		if (aktuelleRichtung == ABSTEIGEND) {
			wert = -wert;
		}
		return wert;
	}
}
