/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.parameter.tsf.ablauf;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.ui.IViewSite;

//import de.bsvrz.buv.plugin.ursacheneinheit.einzelmassnahmen.EinzelmassnahmenViewParameterComposite;
import de.bsvrz.buv.plugin.parameter.tsf.DaVKommunikation;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Darstellung des Fensters zur Parameternderung der Parameter Ablauf-TSF (Programm)
 * @author Pittner
 */
public class CompositeParameterProgramme extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = true;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe TSF-Ablauf
   */
  private ResultData _resultData;

  /**
   * Hauptcomposite
   */
  private Composite _composite;

  /**
   * Composite mit dem Spinner fr der Programme
   */
  private Composite _compositeSpinnerProgramme = null;

  /**
   * Composite mit den Reitern  
   */
  private CTabFolder _tabFolder = null;

  /**
   * Liste mit den unterschiedlichen Parametergruppen
   */
	private ArrayList<String> _list;

	/**
	 * Composites mit den Stufen
	 */
	private CompositeParameterStufen[] _compositeParameterStufen;


  /**
	 * Anzahl der Programme
	 */
	private int _anzahlProgramme;

  /**
	 * Spinner Anzahl Programme
	 */
	private Spinner _spinnerAnzahlProgramme;

	/**
	 * Group Anzahl Programme
	 */
	private Group groupProgramme;

  private IViewSite _viewSite;
	
	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultData 
	 */
  public CompositeParameterProgramme(IViewSite viewSite, Composite parent, int style, ResultData resultData)
  {
    super(parent, style);
    this._viewSite = viewSite;
    this._resultData = resultData;
    initialize();
    
    aktualisiereDarstellung();
    
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    _list = new ArrayList<String>();

    createSpinnerAnzahlProgramme();
    createCompositeTab();
    
    int anzahl = bestimmeAnzahlProgramme();    
    erzeugeBenoetigteReiter(anzahl);
    
  }

  /**
   * Methode bestimmt die Anzahl der Programme aus dem Ergebnis des DaV
   * @return Anzahl Programme
   */
  private int bestimmeAnzahlProgramme ()
  {
    int anzahl = 0;
    if (_resultData != null && _resultData.hasData())
    {
    	Data data = _resultData.getData();
    	anzahl = data.getArray("Programme").getLength();
    }
    
    return anzahl;
  }
  
  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultData Ergebnis Programme
   */
  public synchronized void update (ResultData resultData)
  {
    this._resultData = resultData;    
    int anzahl = bestimmeAnzahlProgramme();
    erzeugeBenoetigteReiter(anzahl);
    updateData ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultData != null)
    	updataDataProgramme();
  }

  /**
   * Methode legt die bentigte Anzahl der Reiter an
   * @param anzahl Anzahl bentigter Reiter (= Anzahl Programme)
   */
  private void erzeugeBenoetigteReiter (int anzahl)
  {
  	if (anzahl == _anzahlProgramme)
  		return;
  	
  	/*
  	 * Beim ersten Anlegen der Tabelle wird Reiter 0 markiert, bei nderung der Anzahl der
  	 * Reiter ber den Spinner wird der letzten Reiter markiert.
  	 */
  	int indexAnwahl = 0;
  	if (_compositeParameterStufen != null)
  		indexAnwahl = anzahl - 1;

		if (_spinnerAnzahlProgramme != null)
			_spinnerAnzahlProgramme.setSelection(anzahl);
		
		/*
		 * Namen der Reiter anpassen
		 */
		_list.clear();
    for (int i=0; i<anzahl; ++i)
      _list.add ("Programm " + (i+1));
    
    /*
     * Bisherige Composite temp. speichern
     */
		Vector<CompositeParameterStufen> tmp = new Vector<CompositeParameterStufen>();
		if (_compositeParameterStufen != null)
			for (CompositeParameterStufen cpa : _compositeParameterStufen)
				tmp.add(cpa);

		/*
		 * Nicht bentigte Reiter entfernen
		 */
	  CTabItem[] items = _tabFolder.getItems();
	  for (int i=items.length - 1; i >= anzahl; i--)
		  items [i].dispose();

		/*
		 * Feld mit den Composites erweitern bzw. reduzieren
		 */
  	_compositeParameterStufen = new CompositeParameterStufen [_list.size()];
  	for (int i=0; i<tmp.size() && i<anzahl; ++i)
  		_compositeParameterStufen [i] = tmp.get(i);
		
		/*
		 * Reiter hinzufgen
		 */
		if (anzahl > _anzahlProgramme)
		{
		  for (int i=_anzahlProgramme; i<anzahl; ++i)
		  {
		  	erzeugeTabItem(_list.get(i));
		  	updataDataProgramme(i);
		  }
		}
    
	  _anzahlProgramme = anzahl;

	  if (indexAnwahl >= 0)
  	  _tabFolder.setSelection( indexAnwahl );

    _tabFolder.redraw();
  }

  /**
   * Updatefunktion fr die Programme
   */
  private void updataDataProgramme()
  {
//    int indexProgramme = _tabFolder.getSelectionIndex();
    
    for (int index = 0; index < _tabFolder.getItems().length; ++index)
//      if (indexProgramme == index)
        updataDataProgramme(index);   
  }

  /**
   * Updatefunktion fr die Programme
   */
  private void updataDataProgramme (int indexReiter)
  {
    Data data = null;
    if (_resultData != null && _resultData.hasData())
    {
      data = _resultData.getData();

			CTabItem[] items = _tabFolder.getItems();
	    CTabItem item = items [indexReiter];
	    
	    if (item != null )
	    {
        Composite c = (Composite) item.getControl();
        Control[] children = c.getChildren();
        CompositeParameterStufen compPar = null;

        Data aktuell;

     	  Array a = data.getArray("Programme");
				if (a.getLength() > indexReiter)
          aktuell = data.getArray("Programme").getItem(indexReiter);
        else
          aktuell = null;
        
        if ( children.length == 0 )
        {
          compPar = new CompositeParameterStufen(_viewSite, c, SWT.NONE, _resultData, indexReiter);
          _compositeParameterStufen [indexReiter] = compPar;          
          compPar.layout( true );
          c.layout( true );
        }
        else
        {
          compPar = (CompositeParameterStufen) children[0];
          compPar.layout( true );
          c.layout( true );
        }
      }else
        System.out.println("item ist null fr index: " + indexReiter);
    }
  }

  /**
   * Methode erteugt die Darstellung der Auswahl der Programme
   * 
   */
  private void createSpinnerAnzahlProgramme()
  {
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    groupProgramme = new Group(_composite, SWT.NONE);
    groupProgramme.setText("Anzahl Programme");
    groupProgramme.setLayout(new GridLayout(1, false));
    groupProgramme.setLayoutData(gridData2);

    _compositeSpinnerProgramme = new Composite(groupProgramme, SWT.NONE);
    _compositeSpinnerProgramme.setLayout(new GridLayout(1, false));

  	_spinnerAnzahlProgramme = new Spinner(_compositeSpinnerProgramme, SWT.BORDER);
  	_spinnerAnzahlProgramme.setSelection(_anzahlProgramme);
  	
  	_spinnerAnzahlProgramme.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (e.getSource() instanceof Spinner)
				{
					Spinner sp = (Spinner) e.getSource();
					int selection = sp.getSelection();
					erzeugeBenoetigteReiter (selection);
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
  }

  /**
   * Methode zum Anlegen des Composite mit den Reitern
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;
    gridDataTabFolder.heightHint = 300;

    _tabFolder = new CTabFolder(groupProgramme, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktualisiereDarstellung();
      }
    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<String> list )
  {
  	for (int i=0; i<list.size(); ++i)
  		erzeugeTabItem(list.get(i));
  	
    _tabFolder.redraw();
    _tabFolder.layout(true);
  }

  /**
   * Erzeugt die einzelnen Reiter
   * @param name
   */
  private void erzeugeTabItem (String name)
  { 	
    String so = name;
    
    CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

    String tabName = so;
    tabItem.setText( tabName );
    tabItem.setData( tabName, so );

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    Composite c1 = new Composite( _tabFolder, SWT.NONE );
    c1.setLayout( new GridLayout() );
    c1.setLayoutData( gridData2 );

    tabItem.setControl( c1 );
  }
 
  /**
   * Aktualisiert die Darstllung
   */
  public void aktualisiereDarstellung()
  {
//    CTabItem itemSel = _tabFolder.getSelection();
//    int indexPrg = _tabFolder.getSelectionIndex();
    
    CTabItem[] items = _tabFolder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      CTabItem itemSel = items[i];
      Composite cSel = (Composite)itemSel.getControl();      
      if (((Composite)cSel).getChildren().length > 0)
      {
        CompositeParameterStufen compParSel = (CompositeParameterStufen)cSel.getChildren()[0];
//        compParSel.setIndexPrg(i);
        compParSel.aktualisiereViewer();
      }      
    }
    
     

//    if (itemSel != null)
//    {
//      Composite cSel = (Composite)itemSel.getControl();      
//      if (((Composite)cSel).getChildren().length > 0)
//      {
//        CompositeParameterStufen compParSel = (CompositeParameterStufen)cSel.getChildren()[0];
//        compParSel.setIndexPrg(indexPrg);
//        compParSel.aktualisiereViewer();
//      }      
//    }
  }
  
  /**
   * @return Anzahl der Programme
   */
  public int getAnzahlProgramme()
  {
    return _anzahlProgramme;
  }
  
  /**
   * Liefert die Compoistes mit den Stufem
   * @return
   */
  public CompositeParameterStufen[] getCompositeParameterStufen()
  {
    return _compositeParameterStufen;
  }
  
  public ResultData getResultData()
  {
    return _resultData;
  }

  public void setResultData(ResultData resultData)
  {
    _resultData = resultData;
    CTabItem[] items = _tabFolder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      CTabItem item = items[i];
      Composite cSel = (Composite)item.getControl();
      CompositeParameterStufen tabComp = (CompositeParameterStufen)cSel.getChildren()[0];
      tabComp.setResultData(_resultData);          
    }
  }

  public CTabFolder getTabFolder()
  {
    return _tabFolder;
  }

  public void setTabFolder(CTabFolder tabFolder)
  {
    _tabFolder = tabFolder;
  }
  
  @Override
  public void dispose()
  {
    super.dispose();
//    System.err.println("---> CompositeParameterProgramme.dispose()");
    CTabItem[] items = _tabFolder.getItems();
    for (int i = 0; i < items.length; i++)
    {
      CTabItem item = items[i];     
      if (!item.isDisposed()){
        Composite cSel = (Composite)item.getControl();
        CompositeParameterStufen tabComp = (CompositeParameterStufen)cSel.getChildren()[0];
        if (!tabComp.isDisposed())
          tabComp.dispose();
        tabComp = null;        
        item.dispose();
      }
      item = null;
    }
  }  
}
