/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.parameter.tsf.mqParaSteuerung;

import java.math.RoundingMode;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;

/**
 * Darstellung des Fensters zur Parameternderung ohne ButtonLeiste
 * 
 * @author Krause
 *
 */
public class MqParaSteuerungCompositeParameter extends Composite
{  
 /**
   * ParameterElemente des DAV, die als Spinner angezeigt werden
   */
  private static final String[] _parItemSpin = {
    "QBEin", "VKfzEin", "AnzahlMessZyklenEin", "AnzahlMessZyklenAus"
  };
  
  private Composite _composite;

  //Die Anzahl dieser Elemente entspricht der Anzahl der Parametergruppen
  private Composite _composite1;

  private Group _groupFilterDef;

  private TableViewer _tvAnzeige;

  private ResultData _resultData;
  

//  private static AnzeigeInhaltAuswahlDialog _anzeigeInhaltAuswahlDialog;

  
  MqParaSteuerungCompositeParameter(Composite parent, int style, ResultData result)
  {
    super(parent, style);
    _resultData = result;
    initialize();
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
 
    //Initialisierungen der Arrays
    List<TableModel[]> inputList = new ArrayList<TableModel[]>();
  
    createGroupDefinition();
        
    Table table = _tvAnzeige.getTable();
    TableModel[] input = createModelArray(table);
    
    if ( input != null )
    {
      _tvAnzeige.setInput(input); 
      inputList.add( input );
    }    
    updateData(inputList);
  }

  private void updateData( List<TableModel[]> modelList )
  {
	if (_resultData != null) {
		Data item = _resultData.getData();
	    AttributeGroup atg = _resultData.getDataDescription().getAttributeGroup();
	    
	    if (item != null){
	      // alle TableModelle durchlaufen
	      for (int i = 0; i < modelList.size(); i++)
	      {
	        //Parametergruppe zum TabellenModell ermitteln
	        
	        //Elemente des TabellenModells durchlaufen
	        for (TableModel tm : modelList.get( i ))
	        {
	          Label lb = (Label) tm.getAttribut().getChildren()[0];
	                    
	          Control control = tm.getWert().getChildren()[0];
	          
	          if (control instanceof Spinner) {
	            Spinner sp = (Spinner)control;
	
	            setSpinnerMinMax(atg, null, lb.getText(), sp);
	            setSpinnerTooltip( sp, lb );
	           
	            // Wert ermitteln und umwandeln
	            String tWert = item.getItem(lb.getText()).asTextValue().getValueText();
	            
	            int iWert;
	            try
	            {
	              iWert = Integer.parseInt( tWert );
	            }
	            catch (Exception e)
	            {
	              iWert = 0;
	            }
	            
	            sp.setSelection( iWert );
	          }                   
	        }
	      }     
	    }	
	}
  }

  /**
   * Minimum- und Maximum-Werte und Digits eines Spinners mit den Daten aus dem DAV setzen
   * 
   * @param attG
   *          AttributeGroup zum ResultData
   *          (result.getDataDescription().getAttributeGroup())
   * @param group
   *          String der Gruppe, zu dem das Item gehrt, oder null falls keine
   *          Gruppe existiert
   * @param item
   *          String des Items zu dessen Attribut die Werte ermittelt werden
   *          sollen
   * @param spinner
   *          Spinner bei dem die Werte gesetzt werden sollen
   * 
   */
  private void setSpinnerMinMax(AttributeGroup attG, String group, String item, Spinner spinner)
  {
    //Attributliste zu der Gruppe ermitteln
    IntegerAttributeType iat1 = null;
    if ( group != null )
    {
      AttributeListDefinition ald = ( AttributeListDefinition )attG.getAttribute(group).getAttributeType();
      iat1 = (IntegerAttributeType)ald.getAttribute(item).getAttributeType();
    }
    else
      iat1 = (IntegerAttributeType)attG.getAttribute(item).getAttributeType();
    
    IntegerValueRange range1 = iat1.getRange();
    
    
    double rangeFactor = range1.getConversionFactor();
    long rangeMin = range1.getMinimum();
    long rangeMax = range1.getMaximum(); 
    
    //Digits ermitteln (log10(0.0001) = -4; log10(0.01) = -2)
    Double digits = - Math.log10( rangeFactor );
    spinner.setDigits(digits.intValue());  
    spinner.setData( rangeFactor );
    
    //Min- Max-Integer-Werte fr den Spinner setzen
    int iMin = (int) rangeMin;
    int iMax = Integer.MAX_VALUE;
    if ( rangeMax < (long)iMax )
      iMax = (int) rangeMax;
    
    spinner.setMinimum( iMin );
    spinner.setMaximum( iMax );    
    
//    System.out.println(item + " ==> Min: " + range1.getMinimum() + "->"+ iMin 
//        + " Max: " + range1.getMaximum() +"->"+ iMax + " Digits: " + rangeFactor + "->" + spinner.getDigits());
       
  }

  /**
   * Setzen des Tooltips beim Spinner und dem zugehrigen Label
   * 
   * @param spinner Spinner
   * @param label   Label
   */
  private void setSpinnerTooltip(Spinner spinner, Label label)
  {
    int digits = spinner.getDigits();
    Double rangeFactor = (Double)spinner.getData();
    if ( rangeFactor == null )
    {
      rangeFactor = 1/Math.pow( 10, digits );
      spinner.setData(rangeFactor);
    }
    double minSpin = (double)spinner.getMinimum() * rangeFactor;
    double maxSpin = (double)spinner.getMaximum() * rangeFactor;
    
    NumberFormat f1 = NumberFormat.getInstance();
    f1.setRoundingMode(RoundingMode.HALF_UP);
    f1.setMaximumFractionDigits(digits);
    f1.setMinimumFractionDigits(digits);
    f1.setGroupingUsed(true);
    
    String tt = "Wertebereich: " + f1.format(minSpin) + " bis " + f1.format(maxSpin) ;
    spinner.setToolTipText(tt);
    if ( label != null )
      label.setToolTipText(tt);

//    System.out.println("Tooltip: Max: " + spinner.getMaximum() + " Digits: " + spinner.getDigits() + " Faktor: " + spinner.getData());
       
  }
  
  /**
   * This method initializes createGroupDefinition
   * 
   */
  private void createGroupDefinition()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    //Gre der 3.Gruppe fest setzen, damit diese grer wird
    _groupFilterDef = new Group(_composite, SWT.NONE);
    _groupFilterDef.setText("Parameter");
    _groupFilterDef.setLayout(fillLayout1);
    _groupFilterDef.setLayoutData(gridData2);

    erzeugeTabelle();

  }
  
  
  private void erzeugeTabelle()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    _composite1 = new Composite(_groupFilterDef, SWT.None);
    _composite1.setLayout(fillLayout1);
    _tvAnzeige = new TableViewer(_composite1, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAnzeige.setContentProvider(new TableContentProvider());
   
    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column1.getColumn().setWidth(150);
    column1.getColumn().setText("Attribut");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column2.getColumn().setWidth(150);
    column2.getColumn().setText("Wert");
    column2.setLabelProvider(new MyCellLabelProvider());
    
    TableViewerColumn column3 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column3.getColumn().setWidth(70);
    column3.getColumn().setText("Einheit");
    column3.setLabelProvider(new MyCellLabelProvider());

    // resize the row height using a MeasureItem listener
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
            event.height = 30;
            break;

          default:
        }

      }

    };
    _tvAnzeige.getTable().addListener(SWT.MeasureItem, listener);
    _tvAnzeige.getTable().setLinesVisible(true);
    _tvAnzeige.getTable().setHeaderVisible(true);
  }


  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
    private TableViewer tableViewer;

    public Object[] getElements(Object inputElement)
    {
      return (TableModel[])inputElement;      
    }

    public void dispose()
    {

    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.deleteObserver(this);
          }

        }

      }
      
     
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.addObserver(this);
          }
        }

      }

    }

    @Override
    public void update(Observable arg0, Object arg1)
    {

      if (tableViewer != null)
      {
        tableViewer.refresh();

      }
    }
  }

  TableModel createModel(String item, Table table)
  {
	  GridData gd0 = new GridData();
	  gd0.horizontalAlignment = GridData.CENTER;
	  gd0.verticalAlignment = GridData.CENTER;
	  gd0.grabExcessHorizontalSpace = true;
	  gd0.grabExcessVerticalSpace = true;
	  Composite comp0 = new Composite( table, SWT.NONE );
	  comp0.setBackground( table.getBackground() );
	  comp0.setBackgroundMode( SWT.INHERIT_DEFAULT );
	  comp0.setLayoutData( gd0 );
	  comp0.setLayout( new GridLayout() );
	  Label label = new Label( comp0, SWT.NONE );
	  label.setText(item);
	  
	  GridData gd1 = new GridData();
	  gd1.horizontalAlignment = GridData.CENTER;
	  gd1.verticalAlignment = GridData.END;
	  gd1.grabExcessHorizontalSpace = true;
	  gd1.grabExcessVerticalSpace = true;
	  Composite comp1 = new Composite( table, SWT.NONE );
	  comp1.setBackground( table.getBackground() );
	  comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
	  comp1.setLayoutData( gd1 );
	  comp1.setLayout( new GridLayout() );
	  GridData gdTableItem = new GridData();
      gdTableItem.horizontalAlignment = GridData.FILL;
	  gdTableItem.grabExcessHorizontalSpace = true;
	  gdTableItem.verticalAlignment = GridData.CENTER;
	  Spinner spinner = new Spinner( comp1, SWT.BORDER );
	  spinner.setLayoutData( gdTableItem );
	  
	  GridData gd2 = new GridData();
	  gd2.horizontalAlignment = GridData.CENTER;
	  gd2.grabExcessHorizontalSpace = true;
	  gd2.verticalAlignment = GridData.CENTER;
	  gd2.grabExcessVerticalSpace = true;
	  Composite comp2 = new Composite(table, SWT.NONE);
      comp2.setBackground(table.getBackground());
	  comp2.setBackgroundMode(SWT.INHERIT_DEFAULT);
	  comp2.setLayoutData(gd2);
	  comp2.setLayout(new GridLayout());
	  Label label2 = new Label(comp2, SWT.NONE);
	  
	  // Hole Einheiten aus DaV-Metamodell, sofern hier vorhanden
	  if (_resultData != null) {
		  AttributeGroup atg = _resultData.getDataDescription().getAttributeGroup();
		  AttributeType at = atg.getAttribute(item).getAttributeType();
		  if (at instanceof IntegerAttributeType)
		  {
			  String unit = ((IntegerAttributeType)at).getRange().getUnit();
			  label2.setText(unit);
		  }
	  }
	  
	  return new TableModel( comp0, comp1, comp2 );
  }
  
  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * 
   * @param table
   * @return TabellenModell
   */
  private TableModel[] createModelArray(Table table)
  {    
    TableModel[] elements = new TableModel[_parItemSpin.length];

    for (int i = 0; i < elements.length; i += 1) {
    	elements[i] = createModel(_parItemSpin[i], table);
    }
        
    return elements;
  }

  /**
   * Anzahl der vorhandenen Gruppen
   * @return
   */
  public int getAnzahlGruppen()
  {
    return 1;
  }

  /**
   * Anzahl der vorhandenen Items je Gruppe
   * 
   * @param index   Index der Gruppe
   * @return
   */
  public int getAnzahlItems(int index)
  {
	  return _parItemSpin.length;
  }
  
  public TableModel[] getItems(int index)
  {
    return (TableModel[])_tvAnzeige.getInput();
  }
  
//  /**
//   * Anzahl der vorhandenen TabelViewer
//   * @return
//   */
//  public int getAnzahlTvAnzeige()
//  {
//    return _tvAnzeige.length;
//  }
//  
  public TableViewer getTvAnzeige()
  {
    return _tvAnzeige;
  }
  
  public void setTvAnzeige( TableViewer tvSet )
  {
    setTvAnzeige( _tvAnzeige, tvSet );
  }
  
  
  private void setTvAnzeige( TableViewer tvAnzeige, TableViewer tvSet )
  {
    TableModel[] modelSet = (TableModel[]) tvSet.getInput();
    TableModel[] models1 = (TableModel[]) tvAnzeige.getInput();

    if ( models1.length == modelSet.length )
    {
      for (TableModel tm : models1)
      {
        Label lb = (Label) tm.getAttribut().getChildren()[0];

        String wert = "";        
        for (TableModel tmSet : modelSet)
        {
          Label lbSet = (Label) tmSet.getAttribut().getChildren()[0];

          
          if ( lb.getText().equals( lbSet.getText() ) )
          {
            wert = tmSet.getSelectedWert();      
          }
        }
        tm.setSelectedWert( wert);
      }
    }
  }

  private class MyCellLabelProvider extends CellLabelProvider
  {

    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

      if (item.getData("EDITOR") != null)
      {
        TableEditor editor = (TableEditor)item.getData("EDITOR");
        editor.dispose();
      }

      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getAttribut();
          break;
        case 1:
          comp = model.getWert();
          break;
        case 2:
            comp = model.getEinheit();
            break;  
        default:
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);

    }
  }
  
  public ResultData getResultData() {
	return _resultData;
  }
	
  public void setResultData(ResultData resultData) {
	_resultData = resultData;
  }
 
}
