/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.parameter.ufd.nsparam;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ComboBoxViewerCellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.CellEditor.LayoutData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;

import de.bsvrz.buv.plugin.parameter.ufd.common.AbstractDataContainerComposite;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;

/**
 * Darstellung des Fensters fr die Bearbeitung der Parameter "Ursacheneinheit"
 * @author ObertM
 *
 */
public class NSKlassifizierungNaesseStufeComposite extends AbstractDataContainerComposite
{ 
  /**
   * Mgliche Werte fr die ComboBoxen
   */
  private static final String [] _values = new String []
  {
    "trocken",
    "nass1",
    "nass2",
    "nass3",
    "nass4",
    "Platzregen",
    "nicht ermittelbar",
    "darf nicht ermittelt werden"
  };

  private enum NI {
    Stufe0("0", "NI 0"),
    Stufe1("1", "NI 1"),
    Stufe2("2", "NI 2"),
    Stufe3("3", "NI 3"),
    Stufe4("4", "NI 4"),
    Undefiniert("undefiniert", "NI n.v.");
    
    public final String value;
    public final String name;
    
    public static int indexOf(String value) {
      NI[] array = values();
      for (int i = 0; i < array.length; i++) {
        if (array[i].value.equals(value)) {
          return i;
        }
      }
      return -1;
    }
    
    private NI(String value, String name) {
      this.value = value;
      this.name = name;
    }
  };
  
  private enum WFD {
    Stufe0("0", "WFD 0"),
    Stufe1("1", "WFD 1"),
    Stufe2("2", "WFD 2"),
    Stufe3("3", "WFD 3"),
    Undefiniert("undefiniert", "WFD n.v.");
    
    public final String value;
    public final String name;
    
    public static int indexOf(String value) {
      WFD[] array = values();
      for (int i = 0; i < array.length; i++) {
        if (array[i].value.equals(value)) {
          return i;
        }
      }
      return -1;
    }
    
    private WFD(String value, String name) {
      this.value = value;
      this.name = name;
    }
  };
  
  private String[][] _matrix = new String[WFD.values().length][NI.values().length + 1];
  
  private TableViewer _tableViewer;

 
  
  public NSKlassifizierungNaesseStufeComposite(Composite parent, int style) {
    super(parent, style);
  }



  public void setUpdate(Data data) {
    if (!isInitialized())
      return;
    
    // Matrix initialisieren
    for (int row = 0; row < WFD.values().length; row++) {
      for (int col = 0; col < NI.values().length + 1; col++) {
        if (col > 0) {
          _matrix[row][col] = null; 
        }
      }
    }
  
    // Daten aus data in Matrix schreiben
    Array array = data.getArray("NsseStufeZuordnungen");
    for (int i = 0; i < array.getLength(); i++) {
      Data item = array.getItem(i);
      
      String wfdStufe = item.getTextValue("WfdStufe").getValueText();
      int row = WFD.indexOf(wfdStufe);
      if (row >= 0) {
        String niStufe = item.getTextValue("NiStufe").getValueText();
        int col = NI.indexOf(niStufe);
        if (col >= 0) {
          String value = item.getTextValue("NsseStufe").getText();
            _matrix[row][col + 1] = value;  
        }  
      }
    }
    
    _tableViewer.refresh();
  }
  
  
  
  public Data getUpdate() {
    if (!isInitialized())
      return null;
    
    Data result = _data.createModifiableCopy();
      
    WFD[] rows = WFD.values();
    NI[] cols = NI.values();
    
    Array array = result.getArray("NsseStufeZuordnungen");
    array.setLength(0);
    
    for (int row = 0; row < rows.length; row++) {
      for (int col = 0; col < cols.length + 1; col++) {
        if (col > 0) {
          if (_matrix[row][col] != null) {
                     
            int index = array.getLength();
            array.setLength(index + 1);
            
            Data item = array.getItem(index);
            
            String niStufe = cols[col - 1].value;
            item.getTextValue("NiStufe").setText(niStufe);
            
            String wfdStufe = rows[row].value;
            item.getTextValue("WfdStufe").setText(wfdStufe);
            
            item.getTextValue("NsseStufe").setText(_matrix[row][col]);
          }
        }
      }
    }
    
    return result;
  }

 
  /**
   * This method initializes createGroupDefinition
   * 
   */
  @Override
  protected void createControls()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
  
    Group groupParameter = new Group(this, SWT.NONE);
    groupParameter.setLayout(new GridLayout());
    groupParameter.setLayoutData(gridData);
    groupParameter.setText("Klassifizierung Nssestufe");

    createTable(groupParameter);
  }

  private void createTable(Composite parent)
  {
    _tableViewer = new TableViewer(parent, SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tableViewer.setContentProvider(new ArrayContentProvider());
    
    NI[] cols = NI.values();
    
    for (int i = 0; i < cols.length + 1; i++) {
      TableViewerColumn column = new TableViewerColumn(_tableViewer, SWT.NONE);
      
      final int index = i;
      column.setLabelProvider(new ColumnLabelProvider() {

        @Override
        public String getText(Object element)
        {
          if (element instanceof String[]) {
            return ((String[])element)[index];
          }
          return null;
        }
        
      });
      
      if (i > 0) {
        column.getColumn().setWidth(140);
        column.getColumn().setText(cols[i - 1].name);
        
        column.setEditingSupport(new ComboColumnEditingSupport(_tableViewer, i));  
      }
      else {
        column.getColumn().setWidth(60);
      }
      column.getColumn().setMoveable(false);
      column.getColumn().setResizable(true);
    }
    
    _tableViewer.getTable().setLinesVisible(true);
    _tableViewer.getTable().setHeaderVisible(true);
    
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    _tableViewer.getTable().setLayoutData(gridData);
        
    WFD[] rows = WFD.values();
    for (int row = 0; row < rows.length; row++) {
      for (int col = 0; col < cols.length; col++) {
        if (col == 0) {
          _matrix[row][col] = rows[row].name; 
        }
        else {
          _matrix[row][col] = null;
        }
      }
    }
    
    _tableViewer.setInput(_matrix);
  }

  
  private class ComboColumnEditingSupport extends EditingSupport {
    
    private int _index;
    private ComboBoxViewerCellEditor _cellEditor;

    public ComboColumnEditingSupport(TableViewer tableViewer, int index) {
      super(tableViewer);
      
      _index = index;
      _cellEditor = new ComboBoxViewerCellEditor((Composite)_tableViewer.getTable(), SWT.READ_ONLY) {
        @Override
        public LayoutData getLayoutData() {
          LayoutData result = new LayoutData();
          Control control = getControl();
          if (control != null) {
            result.minimumWidth = SWT.DEFAULT;
          }
          return result;
        }
      };;
      _cellEditor.setLabelProvider(new LabelProvider());
      _cellEditor.setContenProvider(new ArrayContentProvider());
      _cellEditor.setInput(_values);
    }

    @Override
    protected CellEditor getCellEditor(Object element) {
            
      return _cellEditor;
    }

    @Override
    protected boolean canEdit(Object element) {
  
      if (!(element instanceof String[]) ||
          (WFD.Undefiniert.name.equals(((String[])element)[0]) && 
           _index - 1 == NI.indexOf(NI.Undefiniert.value) )) {
        
        return false;
      }
      else {
        return true;  
      }
    }

    @Override
    protected Object getValue(Object element) {
      if (element instanceof String[]) {
        return ((String[])element)[_index];
      }
      return null;
    }

    @Override
    protected void setValue(Object element, Object value) {
      if (element instanceof String[]) {
        ((String[])element)[_index] = value != null ? value.toString() : null;
        getViewer().update(element, null);
      }
    }
  } 
 
}
