/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

/**
 * Hilfsmethoden.
 * 
 * @author HofmannJ
 * 
 */
public class ATools
{
	/**
	 * Base 64 Encoder.
	 */
	private static final BASE64Encoder b64enc = new BASE64Encoder();

	/**
	 * Base 64 Decoder.
	 */
	private static final BASE64Decoder b64dec = new BASE64Decoder();

	/**
	 * Rckt einen bergebenen {@link String} zeilenweise ein. Arbeitet schnell
	 * mit Hilfe eines {@link StringBuffer}s.
	 * 
	 * @param str
	 *          Einzurckender {@link String}.
	 * @param depth
	 *          Tiefe der Einrckungen.
	 * @return Eingerckter {@link String}.
	 */
	public static String indent(String str, int depth)
	{
		StringBuffer sb = new StringBuffer();

		if (str != null)
		{
			String[] split = str.split("\n");

			for (String splitted : split)
			{
				for (int i = 0; i < depth; i++)
				{
					sb.append("\t");
				}
				sb.append(splitted);
				sb.append("\n");
			}
		}

		return sb.toString();
	}

	/**
	 * Erzeugt eine tiefe Kopie eines Objektes. Diese Objekt muss serialisierbar
	 * sein.
	 * 
	 * @param o
	 *          Zu kopierendes Objekte.
	 * @return Das kopierte Objekt.
	 * @throws Exception
	 *           Tritt auf, wenn das Objekt nicht kopiert werden konnte.
	 */
	public static Object deepCopy(Object o) throws Exception
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		new ObjectOutputStream(baos).writeObject(o);
		ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
		return new ObjectInputStream(bais).readObject();
	}

	/**
	 * Serialisiert, komprimiert und Base64-Kodierung eines beliebigen,
	 * serialisierbaren Objektes.
	 * 
	 * @param object
	 *          Zu serialisierendes Objekt.
	 * @return Serialisiertes Objekt.
	 */
	public static String serB64(Object object)
	{
		String re = null;

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try
		{
			GZIPOutputStream gzipOs = new GZIPOutputStream(baos);
			ObjectOutputStream oos = new ObjectOutputStream(gzipOs);
			oos.writeObject(object);
			oos.close();
			baos.close();
			byte[] byteArray = baos.toByteArray();

			String encode = b64enc.encode(byteArray);
			if ((encode != null) && (encode.length() > 0))
			{
				re = encode;
			}

		} catch (IOException e)
		{
			e.printStackTrace();
		}

		return re;
	}

	/**
	 * Base64-Dekodierung, dekomprimierung und deserialisierung eines
	 * serialisierten Objektes.
	 * 
	 * @param ser
	 *          Serialisiertes Objekt.
	 * @return Deserialisiertes Objekt.
	 */
	public static Object deSerB64(String ser)
	{
		Object re = null;

		try
		{
			byte[] serArray = b64dec.decodeBuffer(ser);

			if ((serArray != null) && (serArray.length > 0))
			{
				ByteArrayInputStream bais = new ByteArrayInputStream(serArray);
				GZIPInputStream gzipIs = new GZIPInputStream(bais);
				ObjectInputStream bis = new ObjectInputStream(gzipIs);
				try
				{
					Object readObject = bis.readObject();
					re = readObject;
				} catch (ClassNotFoundException e)
				{
					e.printStackTrace();
				} finally
				{
					bis.close();
					bais.close();
				}

			}
		} catch (IOException e)
		{
			e.printStackTrace();
		}

		return re;
	}

	/**
	 * Kodiert einen {@link String}ng nach Base64.
	 * 
	 * @param str
	 *          {@link String}.
	 * @return Nach Base64 kodierter {@link String}.
	 */
	public static String codiereStringB64(String str)
	{
		String re = null;
		if (str != null)
		{
			re = b64enc.encode(str.getBytes());
		}
		return re;
	}

	/**
	 * Dekodiert einen Base64 kodierten {@link String}.
	 * 
	 * @param b64
	 *          Base64 kodierter {@link String}.
	 * @return Dekodierter {@link String}.
	 */
	public static String decodiereStringB64(String b64)
	{
		String re = null;

		if (b64 != null)
		{
			try
			{
				byte[] decodeBuffer = b64dec.decodeBuffer(b64);
				if ((decodeBuffer != null) && (decodeBuffer.length > 0))
				{
					re = new String(decodeBuffer);
				}
			} catch (IOException e)
			{
				e.printStackTrace();
			}
		}

		return re;
	}

	/**
	 * Optimiert die Spaltenbreite der {@link Table} eines {@link TableViewer}s.
	 * 
	 * @param tableViewer
	 *          {@link TableViewer}, dessen Spaltenbreite optimiert werden soll.
	 */
	public static void spaltenBreiteOptimieren(TableViewer tableViewer)
	{
		TableColumn[] columns = tableViewer.getTable().getColumns();
		for (TableColumn tableColumn : columns)
		{
			tableColumn.pack();
		}
	}

	/**
	 * Optimiert die Spaltenbreite einer {@link Table}.
	 * 
	 * @param table
	 *          {@link Table}, deren Spaltenbreiten optimiert werden sollen.
	 */
	public static void spaltenBreiteOptimieren(Table table)
	{
		TableColumn[] columns = table.getColumns();
		for (TableColumn tableColumn : columns)
		{
			tableColumn.pack();
		}
	}

	/**
	 * Konvertiert eine {@link Integer}-Liste nach int-Array.
	 * 
	 * @param lIntegers
	 *          Liste von {@link Integer}s.
	 * @return Array vom Typ int.
	 */
	public static int[] convIntegerArrayListToIntArray(
			ArrayList<Integer> lIntegers)
	{

		int[] re = null;

		if (lIntegers != null)
		{
			re = new int[lIntegers.size()];

			int i = 0;
			for (Integer integer : lIntegers)
			{
				re[i] = integer;
				i++;
			}
		}

		return re;
	}
}
