/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis;

import java.io.File;
import java.util.Collection;

import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.buv.plugin.pua.basis.sba.xml.XMLPrinter;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse ermglicht den Export einer Tabelle in eine CSV-Datei
 * 
 * @author Swarco Traffic Systems GmbH
 */
public class ExportXml
{
  /**
   * Tabellenmodell
   */
  private Table _table;

  /**
   * Konstruktor der Klasse
   * 
   * @param model
   *          der Tabelle zugrundeliegendes Tabellenmodell
   */
  public ExportXml(Table table)
  {
    _table = table;
  }

  /**
   * Methode erzeugt die XML-Struktur der Tabelle und speichert diese in einer Datei
   * 
   * @param selectedFile
   *          ausgewhlte Datei
   * @param info
   *          Protokollinformationen
   */
  public void speichereXml(File selectedFile, ProtokollInfo info, Collection<SystemObject> objekte)
  {
    XMLPrinter xmlp = null;

    xmlp = new XMLPrinter();

    int anzZeilen = _table.getItemCount();
    int anzSpalten = _table.getColumnCount();

    // Allgemeine Informationen
    xmlp.addInfo(info, objekte);

    // berschriften
    String daten[] = new String[anzSpalten];
    for (int s = 0; s < anzSpalten; ++s)
      daten[s] = _table.getColumn(s).getText();
    xmlp.addKopf(daten);

    // Daten
    for (int z = 0; z < anzZeilen; ++z)
    {
      TableItem item = _table.getItem(z);
      for (int s = 0; s < anzSpalten; ++s)
        daten[s] = item.getText(s);

      xmlp.addZeile(daten);
    }

    xmlp.schreibeDateiFormatiert(selectedFile);
  }
}
