/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.expose;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.expose.INachbereitung;
import de.bsvrz.buv.plugin.pua.basis.expose.IProjektInfo;
import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.buv.plugin.pua.basis.views.ProtokollAusgabeSicht;
import de.bsvrz.buv.plugin.pua.basis.views.Tools;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResultData;
import de.bsvrz.pua.prot.data.ProtocolData;

public class NachbereitungZraAktiv implements INachbereitung {
	
	public Collection<SystemObject> getObjekte(Collection<SystemObject> objekte,
			ProtokollInformationen protokollInformationen) {
		List<SystemObject> result = new ArrayList<SystemObject>();
		for (SystemObject so: objekte) {
			if (so != null && so.isOfType("typ.zuflussRegelungsAnlage")) {
				result.add(so);	
			}
		}
		
		return result;
	}
	
	public List<String[]> getZellenInhalte(List<SystemObject> objekte, int[] objektZuordnung, 
			ProtocolResultData prd, IProjektInfo projektInfo, ProtokollInformationen protokollInformationen) {
		
		List<String[]> result = new ArrayList<String[]>();
		ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
		if (connection != null) {
			DataModel dataModel = connection.getDataModel();
			List<ProtocolData> dataList = prd.getDataList(dataModel);
	  	  	String zeitStempel = INachbereitung.SDF.format(new Date(prd.getTimeStamp()));
	  	    int anzahlAttribute = ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL + projektInfo.getAnzahlAttribute();
	  	    	
	  	  	for (int i = 0; i < objekte.size(); i++)
	        {
	          String[] zellenwerte = new String[anzahlAttribute];

	          // Datum + Objektbezeichner
	          zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_DATUMZEIT_INDEX] = zeitStempel;
	          String name = objekte.get(i).getNameOrPidOrId();
	          zellenwerte[ProtokollAusgabeSicht.ZUSATZ_INFO_OBJEKTBEZEICHNER_INDEX] = name;

	          // Attributwerte
	          int position = ProtokollAusgabeSicht.ZUSATZ_INFO_ANZAHL;
	          for (int j = i; j < dataList.size() && position < anzahlAttribute; j += objekte.size())
	          {
	            ProtocolData protocolData = dataList.get(j);
	            String text = Tools.toText(protocolData);
	            if ("Rotschaltzeit".equals(protocolData.getName())) {
	            	if ("0".equals(text)) {
	            		text = "aus";
	            	}
	            	else if ("241".equals(text)) {
	            		text = "Dauerrot";
	            	}	
	            }
	            zellenwerte[position] = text;
	            position++;
	          }
	          result.add(zellenwerte);
	        }

		}	
  	  	return result;
	}

}
