/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.expose;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * ObjektProvider fr AQ-Befehle
 * 
 * @author Sans
 *
 */
public class ObjektProviderAqSoll extends ObjektProviderDefault implements IObjektProvider {
	
	public SystemObject getHauptObjekt(List<SystemObject> objekte) {
		SystemObject so = objekte.isEmpty() ? null : objekte.get(0);
		if (so.isOfType("typ.anzeigeQuerschnitt")) {
			NonMutableSet set = ((ConfigurationObject)so).getNonMutableSet("Anzeigen");
			if (set != null) {
				return set.getElements().get(0);
			}	
		}
		return null;
	}
	
	/**
	 * 
	 * @see de.bsvrz.buv.plugin.pua.basis.expose.IObjektProvider#getPuaObjekte(java.util.Collection) 
	 * @return fr jedes in<code>objekte</code> bergebenen Objekt vom Typ typ.anzeigeQuerschnitt alle 
	 * zugehrigen Objekte vom Typ Anzeige, unter dem Pseudoobjekt-Alias "mq0"
	 */
	public Map<String, String[]> getPseudoObjekte(List<SystemObject> objekte) {
		Map<String, String[]> result = new HashMap<String, String[]>();
		List<String> anzeigePids = new ArrayList<String>();			
		for (SystemObject so: objekte) {
			if (so.isOfType("typ.anzeigeQuerschnitt")) {
				NonMutableSet set = ((ConfigurationObject)so).getNonMutableSet("Anzeigen");
				if (set != null) {
					for (SystemObject element: set.getElements()) {
						if (element != null) {
							anzeigePids.add(element.getPid());
						}
					}
				}	
			}
		}
		result.put("mq0", anzeigePids.toArray(new String[anzeigePids.size()]));
		return result;
	}

}
