/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.protokolle;

import java.util.HashMap;

/**
 * Klasse definiert die gespeicherten Protokolle
 * @author Dambach Werke GmbH
 */
public class VorbereiteteProtokolle
{
  /**
   * Singelton Instanz
   */
  private static VorbereiteteProtokolle _instanz = null;
  
  /**
   * Hashmap mit den Protokolldefinitionen
   */
  private HashMap<String, VorbereiteteProtokolleDefinition> _protokolle = new HashMap<String, VorbereiteteProtokolleDefinition> ();
  
  /**
   * Privater Konstruktor
   */
  private VorbereiteteProtokolle ()
  {
    // Betriebsmeldungen
    
    String aktion = "PUA_BETRIEBSMELDUNGEN";
    String skript = "pua.skript.nw.atgBetriebsMeldung1";
    
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.information",
                                                            "typ.informationsKanal",
                                                            "Betriebsmeldungen"));
    
    // De-Fehler global
    
    aktion = "PUA_DE_FEHLER_GLOBAL";
    skript = "pua.skript.nw.atgTlsGloDeFehler1";
    
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.tlsAntwort",
                                                            "typ.de",
                                                            "De-Fehler global"));

    // De-Fehler ergaenzende Meldung
    
    aktion = "PUA_DE_FEHLER_ERGAENZENDE_MELDUNG";
    skript = "pua.skript.nw.atgTlsLveErgaenzendeDeFehlerMeldung1";
    
    _protokolle.put( aktion, 
        new VorbereiteteProtokolleDefinition (aktion, skript,
            "asp.tlsAntwort",
            "typ.de",
        "De-Fehler ergaenzende Meldung"));

    // De ergaenzende Meldung Version 0-4
    
    aktion = "PUA_DE_ERGAENZENDE_MELDUNG_VERSION_0_BIS_4";
    skript = "pua.skript.nw.atgTlsLveErgebnisMeldungVersion0Bis4";
    
    _protokolle.put( aktion, 
        new VorbereiteteProtokolleDefinition (aktion, skript,
            "asp.tlsAntwort",
            "typ.de",
        "De-Fehler ergaenzende Meldung"));
    
    
    
    // Verkehrsdaten
    
    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_MQ_ANALYSE";
    skript = "pua.skript.nw.atgVerkehrsDatenKurzZeitMqWerte1";
    
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.analyse",
                                                            "typ.messQuerschnitt",
                                                            "Verkehrsdaten: Analysewerte"));

    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_MQ_1_MINUTE";
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.agregation1Minute",
                                                            "typ.messQuerschnitt",
                                                            "Verkehrsdaten: 1 Minuten Werte"));

    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_MQ_5_MINUTEN";
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.agregation5Minuten",
                                                            "typ.messQuerschnitt",
                                                            "Verkehrsdaten: 5 Minuten Werte"));

    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_MQ_15_MINUTEN";
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.agregation15Minuten",
                                                            "typ.messQuerschnitt",
                                                            "Verkehrsdaten: 15 Minuten Werte"));
    
    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_MQ_30_MINUTEN";
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.agregation30Minuten",
                                                            "typ.messQuerschnitt",
                                                            "Verkehrsdaten: 30 Minuten Werte"));
    
    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_MQ_60_MINUTEN";
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.agregation60Minuten",
                                                            "typ.messQuerschnitt",
                                                            "Verkehrsdaten: 60 Minuten Werte"));

    // Prognosedaten (geglttet)
    
    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_GEGLAETTET_MQ_NORMAL";
    skript = "pua.skript.nw.atgVerkehrsDatenKurzZeitGeglaettetMqWerte1";
    
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.prognoseNormal",
                                                            "typ.messQuerschnitt",
                                                            "Prognosedaten (geglttet): normal"));
    
    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_GEGLAETTET_MQ_FLINK";
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.prognoseFlink",
                                                            "typ.messQuerschnitt",
                                                            "Prognosedaten (geglttet): flink"));
    
    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_GEGLAETTET_MQ_TRAEGE";
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.prognoseTrge",
                                                            "typ.messQuerschnitt",
                                                            "Prognosedaten (geglttet): trge"));
    
   // Prognosedaten prognostiziert
    
    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_EXTRA_POLATION_MQ_NORMAL";
    skript = "pua.skript.nw.atgVerkehrsDatenKurzZeitTrendExtraPolationMqWerte1";
    
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.prognoseTrge",
                                                            "typ.messQuerschnitt",
                                                            "Prognosedaten (prognostiziert): normal"));
    
    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_EXTRA_POLATION_MQ_FLINK";
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.prognoseFlink",
                                                            "typ.messQuerschnitt",
                                                            "Prognosedaten (prognostiziert): flink"));
    
    aktion = "PUA_VERKEHRSDATEN_KURZZEIT_EXTRA_POLATION_MQ_TRAEGE";
    _protokolle.put( aktion, 
                      new VorbereiteteProtokolleDefinition (aktion, skript,
                                                            "asp.prognoseTrge",
                                                            "typ.messQuerschnitt",
                                                            "Prognosedaten (prognostiziert): trge"));
  }
  
  /**
   * Liefert die Singleton Instanz der Klasse {@link VorbereiteteProtokolle}.
   * @return Singleton Instanz der Klasse {@link VorbereiteteProtokolle}.
   */
  public static VorbereiteteProtokolle getInstanz ()
  {
    if (_instanz == null)
    {
      synchronized (VorbereiteteProtokolle.class)
      {
        if (_instanz == null)
        {
          _instanz = new VorbereiteteProtokolle ();
        }
      }
    }
    
    return _instanz;
  }
  
  /**
   * Methode liefert die zu einer Aktion gehrenden Protokolldefinition
   * @param aktion Aktion
   * @return Definition im Fehlerfall null
   */
  public VorbereiteteProtokolleDefinition getProtokollDefinition (String aktion)
  {
    if (_protokolle.containsKey( aktion ))
      return _protokolle.get( aktion );
    
    return null;
  }
}
