/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.puaDataUpdateListener;

import java.util.Iterator;
import java.util.LinkedList;

import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResult;
/**
 * Die Klasse kapselt die Verwaltungsmethoden fr die Listener-Funktionalitt 
 * des {@link PuADataUpdateListener}. Diese Klasse implemntiert das Singleton-Pattern
 * und stellt dadurch sicher dass der Listener nur einmal Instanziiert wird. 
 * 
 * 
 * @author Pittner
 *
 */
public class PuADataUpdateListener
{
  /**
   * Liste der {@link IPuADataUpdateListener}, die ber einen neue PuA-Daten informiert werden mchten.
   */
  private LinkedList<IPuADataUpdateListener> _puADataUpdateListener = new LinkedList<IPuADataUpdateListener>();
  
  /**
   * Singleton-Instanz
   */
  private static PuADataUpdateListener _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static PuADataUpdateListener getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new PuADataUpdateListener();
    }
    return _instanz;
  }
    

  /**
   * Fgt einen Listener zur Liste der {@link PuADataUpdateListener} hinzu.
   * 
   * @param listener
   */
  public void addPuADataUpdateListener(IPuADataUpdateListener listener)
  {
    if (!_puADataUpdateListener.contains(listener))
    {
      _puADataUpdateListener.add(listener);
    }
    else
    {
      System.err
          .println("Der Listener ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt");
    }
  }
  
  /**
   * Benachrichtigt alle registrierten {@link IPuADataUpdateListener} darber, dass neue Daten vorhanden sind.
   * 
   */
  public void notifyPuADataUpdateListener(ProtocolResult result, ProtokollInformationen info, int viewId, Long protokollId)
  {

    PuADataUpdateEvent event = new PuADataUpdateEvent(this, result, info, viewId, protokollId);

    Iterator<IPuADataUpdateListener> it = _puADataUpdateListener.iterator();
    while (it.hasNext())
    {
      IPuADataUpdateListener listener = it.next();
      listener.puADataUpdate(event);
    }
  }
  
  /**
   * Entfernt einen {@link IPuADataUpdateListener} von der Liste der Zuhrer.
   * 
   * @param IPuADataUpdateListener
   *          Zu entfernender {@link IPuADataUpdateListener}.
   */
  public void removePuADataUpdateListener(IPuADataUpdateListener listener)
  {
    if (_puADataUpdateListener.contains(listener))
    {
      _puADataUpdateListener.remove(listener);
    }
    else
    {
      System.err.println("Der Listener ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
    }
  }
  
}
