/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.daten;


import org.eclipse.swt.graphics.Color;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.daten.StreckenProfilKonfigurationData.BezeichnungFahrtRichtung;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditor;
 
/**
 * Modelliert die grafischen Eigenschaften einer Streckenprofil-DatenSerie
 * 
 * @author Pittner
 *
 */
public class StreckenProfilGrafischeEigenschaftenData implements IStreckenProfilDaten, Cloneable {
  private static final long serialVersionUID = -6713427173153117154L;
	 
	public enum Typ{		
		LINIE("Linie"), BALKEN("Balken"), GESTAPELTE_BALEN("Gestapelte Balken");
				
		public final String _typ;		
		
		private Typ(String typ){
			_typ = typ;
		}
		
		@Override
		public String toString() {
			// TODO Auto-generated method stub
			return _typ;
		}

		// hor9tr added get here
    public static Typ get(int index) {
      return values()[index];     
    }
	}
	
	public enum LinienArt{		
		LINIEN("Linien"), STRICHE("Striche"), PUNKTE("Punkte"), PUNKT_STRICH("Punkt-Strich");
		
		public final String _linieArt;		
		
		private LinienArt(String linienArt){
			_linieArt = linienArt;
		}
		
		@Override
		public String toString() {
			// TODO Auto-generated method stub
			return _linieArt;
		}
		
	  // hor9tr added get here
    public static LinienArt get(int index) {
      return values()[index];     
    }
	}
	
	public enum LinienDicke{		
		
		DUENN("Duenn"), NORMAL("Normal"), DICK("Dick"), DICKER("Dicker");	
		
		public final String _linieDicke;		
		
		private LinienDicke(String linienDicke){
			_linieDicke = linienDicke;
		}
		
		@Override
		public String toString() {
			// TODO Auto-generated method stub
			return _linieDicke;
		}

    // hor9tr added get here
    public static LinienDicke get(int index) {
      return values()[index];     
    }
	}
	
	public enum KnotenArt{		
		QUADRAT("Quadrat"), KREIS("Kreis"), KREUZ("Kreuz"), HAARKREUZ("Haarkreuz"), OHNE("Ohne");	
		
		public final String _knotenArt;		
		
		private KnotenArt(String knotenArt){
			_knotenArt = knotenArt;
		}
		
		@Override
		public String toString() {
			// TODO Auto-generated method stub
			return _knotenArt;
		}

    // hor9tr added get here
    public static KnotenArt get(int index) {
      return values()[index];     
    }
	}
	
	
	private String bezeichnung;

//  private BezeichnungFahrtRichtung fahrtRichtung;

  private boolean sichtbar;
	
	private Typ typ;
	
	private LinienArt linienArt;
	
	private Color linienFarbe;
	
  private LinienDicke linienDicke;
	
	private KnotenArt knotenArt;
	 	
	private Color knotenFuellFarbe;

  private String editor;


public StreckenProfilGrafischeEigenschaftenData(String bezeichnung, String editor, boolean sichtbar, Typ typ, LinienArt linienArt, Color linienFarbe, LinienDicke linienDicke, KnotenArt knotenArt, Color knotenFuellFarbe) {		 
		this.bezeichnung = bezeichnung;
		this.editor = editor;
//		this.fahrtRichtung = fahrtRichtung;
		this.sichtbar = sichtbar;
		this.typ = typ;
		this.linienArt = linienArt;		
		this.linienFarbe = linienFarbe;		
		this.linienDicke = linienDicke;		
		this.knotenArt = knotenArt;		
		this.knotenFuellFarbe = knotenFuellFarbe;
	}

	/** @see java.lang.Object#equals(java.lang.Object) */
	@Override
    public boolean equals(Object object) {
		if(object == null) {
			return false;
		}
		else if(object instanceof StreckenProfilGrafischeEigenschaftenData) {
			StreckenProfilGrafischeEigenschaftenData other = (StreckenProfilGrafischeEigenschaftenData)object;

			return this.bezeichnung.equals(other.bezeichnung)
//			    && this.fahrtRichtung == other.fahrtRichtung
			    && this.editor == other.editor
					&& this.sichtbar == other.sichtbar
					&& this.typ.equals(other.typ)
					&& this.linienArt.equals(other.linienArt)
					&& this.linienDicke.equals(other.linienDicke)
					&& this.linienFarbe.equals(other.linienFarbe)
					&& this.knotenArt.equals(other.knotenArt)
					&& this.knotenFuellFarbe.equals(other.knotenFuellFarbe);
		}
		else {
			return false;
		}
	}

	/** @see java.lang.Object#toString() */
	@Override
    public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("StreckenprofilGrafischeEigenschaften: " + bezeichnung + "\n");
		sb.append("\nEditor:\t" + editor);
		sb.append("\nSichtbar:\t" + sichtbar);
		sb.append("\nTyp:\t" + typ);
		sb.append("\nLinienArt:\t" + linienArt);
		sb.append("\nLinienFarbe:\t" + linienFarbe);
		sb.append("\nLinienDicke:\t" + linienDicke);
		sb.append("\nKnotenArt:\t" + knotenArt);
		sb.append("\nKnotenFuellFarbe:\t" + knotenFuellFarbe);
		return sb.toString();
	}

	/** @see de.bsvrz.pua.prot.client.dataobject.ProtocolResult#getType() */
	public int getType() {
		return IStreckenProfilDaten.GRAFISCHE_EIGENSCHAFTEN_DATA;
	}
	
  public String getBezeichnung()
  {
    return bezeichnung;
  }
  
  public String getEditor()
  {
    return editor;
  }

//  public BezeichnungFahrtRichtung getFahrtRichtung()
//  {
//    return fahrtRichtung;
//  }
  
  public boolean isSichtbar()
  {
    return sichtbar;
  }

  public Typ getTyp()
  {
    return typ;
  }

  public LinienArt getLinienArt()
  {
    return linienArt;
  }

  public Color getLinienFarbe()
  {
    return linienFarbe;
  }
  
  public void setLinienFarbe(Color color)
  {
    linienFarbe = color;
  }

  public LinienDicke getLinienDicke()
  {
    return linienDicke;
  }

  public KnotenArt getKnotenArt()
  {
    return knotenArt;
  }

  public Color getKnotenFuellFarbe()
  {
    return knotenFuellFarbe;
  }
  
  public void setBezeichnung(String bezeichnung)
  {
    this.bezeichnung = bezeichnung;
  }

//  public void setFahrtRichtung(BezeichnungFahrtRichtung fahrtRichtung)
//  {
//    this.fahrtRichtung = fahrtRichtung;
//  }

  public void setSichtbar(boolean sichtbar)
  {
    this.sichtbar = sichtbar;
  }

  public void setTyp(Typ typ)
  {
    this.typ = typ;
  }

  public void setLinienArt(LinienArt linienArt)
  {
    this.linienArt = linienArt;
  }

  public void setLinienDicke(LinienDicke linienDicke)
  {
    this.linienDicke = linienDicke;
  }

  public void setKnotenArt(KnotenArt knotenArt)
  {
    this.knotenArt = knotenArt;
  }

  public void setKnotenFuellFarbe(Color knotenFuellFarbe)
  {
    this.knotenFuellFarbe = knotenFuellFarbe;
  }
	
  @Override
  public Object clone() {
    return new StreckenProfilGrafischeEigenschaftenData(
        bezeichnung, 
        editor, 
        sichtbar, 
        typ, 
        linienArt, 
        linienFarbe, 
        linienDicke, 
        knotenArt, 
        knotenFuellFarbe
    );        
  }
}
