/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor;

import net.sf.paperclips.PagePrint;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.editor.FormPage;
import org.eclipse.ui.forms.editor.IFormPage;

import com.bitctrl.lib.eclipse.editors.ExtendableFormEditor;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Modell;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.actions.StreckenprofilSaveAction;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.daten.StreckenProfilKonfigurationData.BezeichnungFahrtRichtung;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.pages.StreckenprofilEditorGrafischeEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.pages.StreckenprofilEditorKonfiguration;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.pages.StreckenprofilEditorStreckenProfil2;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.simulation.SimulationBeendenUndVernichtenJob;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.adapter.WegeCalculatorJob;
import de.bsvrz.buv.rw.basislib.legende.ILegendenAbschnitt;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau.IDruckvorschau;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegende;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.InvalidationListener;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AttSimulationsZustand;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.objekte.OfflineSimulation;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.onlinedaten.OdSimulationsSteuerungOffline;

/**
 * Der Streckenprofil - Editor.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilEditor.java,v 1.14 2014/04/17 15:51:08 Pittner Exp $
 */
public class StreckenprofilEditor extends ExtendableFormEditor implements DatensatzUpdateListener,
    InvalidationListener, IDruckvorschau
{

  /**
   * Editor ID.
   */
  public static final String EDITOR_ID = StreckenprofilEditor.class.getName();

  /**
   * Das betrachtet {@link StreckenprofilEditorModel}.
   */
  private StreckenprofilEditorModel streckenprofil;

  /**
   * Die Editor-Page "Konfiguration".
   */
  private StreckenprofilEditorKonfiguration konfiguration;

  /**
   * Die Offline-Simulation.
   */
  private OfflineSimulation aktuelleSimulation;

  /**
   * Die Editor-Page "Erweitertes Streckenprofil".
   */
  private StreckenprofilEditorStreckenProfil2 streckenprofilFormPage2;

  private boolean dirty;

  private boolean simulationAktiv;

//  private int alternativeObjektbezeichnung = -1;

  @Override
  public void doSave(final IProgressMonitor monitor)
  {
    // final IEditorInput input = getEditorInput();
    // if (input instanceof StreckenprofilEditorInput
    // && ((StreckenprofilEditorInput) input).getEinstellungsArt() != null)
    // {
    // if (StreckenprofilEinstellungen
    // .getInstance()
    // .setModellEinstellungen(
    // ((StreckenprofilEditorInput) input)
    // .getEinstellungsArt(),
    // streckenprofil.getStreckenZug().getName(),
    // streckenprofil)) {
    // for (final Object p : pages) {
    // if (p instanceof FormPage) {
    // ((FormPage) p).doSave(monitor);
    // }
    // }
    // }
    // } else {
    final StreckenprofilSaveAction saveAction = new StreckenprofilSaveAction(streckenprofil);

    saveAction.run();
    if (saveAction.isErfolgreichGespeichert())
    {
      for (final Object p : pages)
      {
        if (p instanceof FormPage)
        {
          ((FormPage)p).doSave(monitor);
        }
      }
      setPartName("Streckenprofil " + streckenprofil.getStreckenZug().getName());
      dirty = false;
      firePropertyChange(PROP_DIRTY);
    }
    // }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.ui.forms.editor.FormEditor#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
   */
  @Override
  public void init(final IEditorSite site, final IEditorInput input) throws PartInitException
  {
    super.init(site, input);
    if (input instanceof StreckenprofilEditorInput)
    {
      streckenprofil = ((StreckenprofilEditorInput)input).getModel();
      if (streckenprofil != null && streckenprofil.getStreckenZug() != null)
      {
        setPartName("Streckenprofil " + streckenprofil.getStreckenZug().getName());
      }
      if (((StreckenprofilEditorInput)input).getEinstellungsArt() == null)
      {
        // noch nirgends gespeichert
        dirty = true;
        firePropertyChange(PROP_DIRTY);
      }
    }
  }

  @Override
  public void doSaveAs()
  {
    final StreckenprofilSaveAction saveAction = new StreckenprofilSaveAction(streckenprofil);
    saveAction.run();

    if (saveAction.isErfolgreichGespeichert())
    {
      for (final Object p : pages)
      {
        if (p instanceof FormPage)
        {
          ((FormPage)p).doSaveAs();
        }
      }
      dirty = false;
      firePropertyChange(PROP_DIRTY);
      setPartName("Streckenprofil " + streckenprofil.getStreckenZug().getName());
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.ui.forms.editor.FormEditor#isDirty()
   */
  @Override
  public boolean isDirty()
  {
    boolean result = super.isDirty();
    if (!result)
    {
      result = dirty;
    }
    return result;
  }

  @Override
  public boolean isSaveAsAllowed()
  {
    return true;
  }

  @Override
  protected void addPages()
  {
    try
    {
      Modell modell = new Modell(SWTObservables.getRealm(getSite().getShell().getDisplay()));
      if (streckenprofil != null)
      {
        modell = new Modell(SWTObservables.getRealm(getSite().getShell().getDisplay()), streckenprofil);
      }

      streckenprofil = modell.getEMFModell();

      if (streckenprofil.getWegeCalculator() == null)
      {
        streckenprofil.setWegeCalculator(new WegeCalculatorJob(streckenprofil));
      }

      konfiguration = new StreckenprofilEditorKonfiguration(this, modell);
      addPage(konfiguration);
      addPage(new StreckenprofilEditorGrafischeEigenschaften(this, modell));
      // streckenprofilFormPage2 = new StreckenprofilEditorStreckenProfil2(
      // this, modell);

      AttTmcRichtung richtung = null;
        
      if (modell.getErsteStrecke() != null)  
        richtung = modell.getErsteStrecke().getRichtung();

      BezeichnungFahrtRichtung bezeichnungFahrtRichtung = BezeichnungFahrtRichtung.SW;
      if (richtung != null && richtung.toString().equals("negativ"))
      {
        bezeichnungFahrtRichtung = BezeichnungFahrtRichtung.NO;
      }

      streckenprofilFormPage2 = new StreckenprofilEditorStreckenProfil2(this, bezeichnungFahrtRichtung, modell);

      addPage(streckenprofilFormPage2);
    }
    catch (final PartInitException e)
    {
      e.printStackTrace();
    }
    super.addPages();
  }

  /**
   * Liefert die aktuelle Offlinesimulation oder <code>null</code>, wenn nicht Simuliert wird.
   * 
   * @return the aktuelleSimulation
   */
  public final OfflineSimulation getAktuelleSimulation()
  {
    return aktuelleSimulation;
  }

  /**
   * Setzt die zu verwendende {@link OfflineSimulation}, oder <code>null</code>, wenn nicht simuliert werden soll.
   * 
   * @param aktuelleSimulation
   *          the aktuelleSimulation to set
   */
  public final void setAktuelleSimulation(final OfflineSimulation aktuelleSimulation)
  {
    this.aktuelleSimulation = aktuelleSimulation;
    simulationUeberwachen();
  }

  /**
   * Anmeldung, zur berwachung der momentan verwendeten {@link OfflineSimulation}.
   */
  private void simulationUeberwachen()
  {
    if (aktuelleSimulation != null)
    {
      aktuelleSimulation.getOdSimulationsSteuerungOffline().addUpdateListener(
          OdSimulationsSteuerungOffline.Aspekte.Zustand, this);

      ((DynamicObject)aktuelleSimulation.getSystemObject()).addListenerForInvalidation(this);

    }
  }

  @Override
  public void datensatzAktualisiert(final DatensatzUpdateEvent event)
  {
    if (event.getDatum() instanceof OdSimulationsSteuerungOffline.Daten)
    {
      final OdSimulationsSteuerungOffline.Daten datum = (OdSimulationsSteuerungOffline.Daten)event.getDatum();
      konfiguration.setSimulationsZustand(datum.getSimulationsZustand());
      if (AttSimulationsZustand.ZUSTAND_2_START.equals(datum.getSimulationsZustand())
          || AttSimulationsZustand.ZUSTAND_5_PAUSE.equals(datum.getSimulationsZustand())
          || AttSimulationsZustand.ZUSTAND_6_EINZELSCHRITT.equals(datum.getSimulationsZustand()))
      {
        // streckenprofilFormPage.setSimulation(aktuelleSimulation);
      }
      else
      {
        // streckenprofilFormPage.setSimulation(null);
      }

    }

  }

  @Override
  public void invalidObject(final DynamicObject dynamicObject)
  {
    if (aktuelleSimulation != null && aktuelleSimulation.getSystemObject().equals(dynamicObject))
    {
      aktuelleSimulation.getOdSimulationsSteuerungOffline().removeUpdateListener(
          OdSimulationsSteuerungOffline.Aspekte.Zustand, this);
      dynamicObject.removeListenerForInvalidation(this);
      aktuelleSimulation = null;
      konfiguration.setSimulationsZustand(AttSimulationsZustand.ZUSTAND_4_GELOESCHT);
    }

  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.ui.forms.editor.FormEditor#dispose()
   */
  @Override
  public void dispose()
  {
    if (aktuelleSimulation != null)
    {
      final SimulationBeendenUndVernichtenJob j = new SimulationBeendenUndVernichtenJob(aktuelleSimulation);
      j.setUser(true);
      j.schedule();
    }
    super.dispose();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.ui.part.MultiPageEditorPart#getAdapter(java.lang.Class)
   */
  @SuppressWarnings("unchecked")
  @Override
  public Object getAdapter(final Class adapter)
  {
    if (ILegende.class.equals(adapter))
    {
      return streckenprofilFormPage2.getAdapter(adapter);
    }
    else if (ILegendenAbschnitt.class.equals(adapter))
    {
      return streckenprofilFormPage2.getAdapter(adapter);
    }
    return super.getAdapter(adapter);
  }

  @Override
  public PagePrint getDruckAuftrag()
  {
    PagePrint result = null;
    if (streckenprofilFormPage2 != null && streckenprofilFormPage2.isActive())
    {
      result = streckenprofilFormPage2.getDruckAuftrag();
    }
    return result;
  }

  @Override
  public String getTitel()
  {
    String result = null;
    if (streckenprofilFormPage2 != null && streckenprofilFormPage2.isActive())
    {
      result = streckenprofilFormPage2.getTitel();
    }
    return result;
  }

  public boolean isSimulationAktiv()
  {
    return simulationAktiv;
  }

  public void setSimulationAktiv(boolean simulationAktiv)
  {
    this.simulationAktiv = simulationAktiv;
  }
  
//  public int getAlternativeObjektbezeichnung()
//  {
//    return alternativeObjektbezeichnung;
//  }
//  
//  public void setAlternativeObjektbezeichnung(int index)
//  {    
//    this.alternativeObjektbezeichnung  = index;
//  }

  @Override
  protected void pageChange(int newPageIndex)
  {
    super.pageChange(newPageIndex);
    if (newPageIndex == 2)
    {
      IFormPage page = getActivePageInstance();
      if (page instanceof StreckenprofilEditorStreckenProfil2)
      {
        StreckenprofilEditorStreckenProfil2 formPage = (StreckenprofilEditorStreckenProfil2)page;
        formPage.getChartComposite().initChart();
        if (!isSimulationAktiv())
          formPage.getChartComposite().anmeldung();
      }
    }
  }
}
