/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.adapter.WegeCalculatorJob;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;

/**
 * {@link IEditorInput} fr den {@link StreckenprofilEditor}.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilEditorInput.java,v 1.3 2014/04/17 15:51:08 Pittner Exp $
 */
public class StreckenprofilEditorInput implements IEditorInput {

	/**
	 * Das {@link StreckenprofilEditorModel}.
	 */
	private StreckenprofilEditorModel model;

	/**
	 * Die {@link EinstellungsArt}, unter der das
	 * {@link StreckenprofilEditorModel} schon mal gespeichert wurde, oder
	 * <code>null</code>, wenn es noch nie gespeichert wurde.
	 */
	private EinstellungsArt einstellungsArt;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorInput#exists()
	 */
	@Override
	public boolean exists() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
	 */
	@Override
	public ImageDescriptor getImageDescriptor() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	@Override
	public String getName() {
		return getToolTipText();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorInput#getPersistable()
	 */
	@Override
	public IPersistableElement getPersistable() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	@Override
	public String getToolTipText() {
		if (model != null && model.getStreckenZug().getName() != null) {
			return model.getStreckenZug().getName();
		} else {
			return "Unbenannt";
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Object getAdapter(final Class adapter) {
		return null;
	}

	/**
	 * Liefert das {@link StreckenprofilEditorModel} des
	 * {@link StreckenprofilEditorInput}s.
	 * 
	 * @return the model
	 */
	public final StreckenprofilEditorModel getModel() {
		StreckenprofilEditorModel result = null;
		if (model != null) {
			result = (StreckenprofilEditorModel) EcoreUtil.copy(model);
			result.setWegeCalculator(new WegeCalculatorJob(result));
			result.getWegeCalculator().schedule(50);
		}
		return result;
	}

	/**
	 * Setzt das {@link StreckenprofilEditorModel} des
	 * {@link StreckenprofilEditorInput}s.
	 * 
	 * @param model
	 *            the model to set
	 */
	public final void setModel(final StreckenprofilEditorModel model) {
		this.model = model;
	}

	/**
	 * Liefert, die {@link EinstellungsArt}, unter der das
	 * {@link StreckenprofilEditorModel} bereits einmal gepspeichert wurde, oder
	 * <code>null</code>, wenn das {@link StreckenprofilEditorModel} noch nicht
	 * gespeichert wurde.
	 * 
	 * @return the einstellungsArt
	 */
	public final EinstellungsArt getEinstellungsArt() {
		return einstellungsArt;
	}

	/**
	 * Setzt die {@link EinstellungsArt}, unter der das
	 * {@link StreckenprofilEditorModel} des {@link StreckenprofilEditorInput}
	 * gespeichert wurde.
	 * 
	 * @param einstellungsArt
	 *            the einstellungsArt to set
	 */
	public final void setEinstellungsArt(final EinstellungsArt einstellungsArt) {
		this.einstellungsArt = einstellungsArt;
	}

}
