/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.pages;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.FunktionsBerechtigungsListener;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.StreckenprofilPluginFunktion;

/**
 * Wrapper fr einen {@link Button}, der die Oberfchenberechtigung des
 * Rahmenwerks bercksichtigt.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: ButtonMitFunktionsBerechtigung.java,v 1.3 2014/04/17 15:51:13 Pittner Exp $
 */
public class ButtonMitFunktionsBerechtigung extends Composite implements
		FunktionsBerechtigungsListener {

	/**
	 * Das Enabled Flag des eingschlossenen Buttons.
	 */
	private boolean enabled = true;

	/**
	 * Der umschlossene Button.
	 */
	private final Button b;

	/**
	 * Erzeugt einen neuen Button, der auf die Funktionsberechtigung fr das
	 * Plugin Streckenprofil des Dav hrt.
	 * 
	 * @param parent
	 *            das Parent Composite.
	 * @param style
	 *            der SWT Style.
	 */
	public ButtonMitFunktionsBerechtigung(final Composite parent,
			final int style) {
		super(parent, SWT.NONE);
		setLayout(new FillLayout());
		b = new Button(this, style);
		StreckenprofilPluginFunktion.HISTORISCHES_STRECKENPROFIL
				.addFunktionsBerechtigungsListener(this);
		setEnabled(enabled);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seede.bsvrz.buv.plugin.streckenprofil.FunktionsBerechtigungsListener#
	 * berechtigungsAenderung()
	 */
	@Override
	public void berechtigungsAenderung() {
		if (!isDisposed()) {
			getDisplay().asyncExec(new Runnable() {

				@Override
				public void run() {
					setEnabled(enabled);
				}
			});
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Widget#dispose()
	 */
	@Override
	public void dispose() {
		StreckenprofilPluginFunktion.HISTORISCHES_STRECKENPROFIL
				.removeFunktionsBerechtigungsListener(this);
		super.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Control#setEnabled(boolean)
	 */
	@Override
	public void setEnabled(final boolean enabled) {
		this.enabled = enabled;
		b.setEnabled(enabled
				&& StreckenprofilPluginFunktion.HISTORISCHES_STRECKENPROFIL
						.isFreigegeben());
	}

	/**
	 * Bestimmt eine Image fr den eingeschlossenen {@link Button}.
	 * 
	 * @see Button#setImage(Image)
	 * @param imgStart
	 *            das Image - Icon des {@link Button}s.
	 */
	public void setImage(final Image imgStart) {
		b.setImage(imgStart);
	}

	/**
	 * Fgt einen {@link org.eclipse.swt.events.SelectionListener} zu dem
	 * eingeschlossenen Button hinzu.
	 * 
	 * @see Button#addSelectionListener(org.eclipse.swt.events.SelectionListener)
	 * @param selectionAdapter
	 *            dieses Objekt mchte bei Selektion des Button benachrichtigt
	 *            werden.
	 */
	public void addSelectionListener(final SelectionAdapter selectionAdapter) {
		b.addSelectionListener(selectionAdapter);
	}

}
