/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.export;

import org.eclipse.nebula.widgets.grid.GridCellRenderer;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.graphics.Transform;
import org.eclipse.swt.widgets.Display;

import com.bitctrl.lib.eclipse.paperclips.custom.CustomPrint;
import com.bitctrl.lib.eclipse.paperclips.custom.CustomPrintHandler;

/**
 * Ermglicht das Drucken von Grid-Objekten mit manuell gezeichneten Zellen.
 * 
 * @param <R>
 *            Typ das Objektes, das in der Zelle angezeigt werden soll.
 * 
 * @author BitCtrl Systems GmbH, Enrico Schnepel
 * @version $Id: GridCellRendererPrint.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
 */
public class GridCellRendererPrint<R> extends CustomPrint {

	/**
	 * {@link PrintHandler} fr eine einzelne Zelle des
	 * {@link org.eclipse.nebula.widgets.grid.Grid}.
	 * 
	 * @author BitCtrl Systems GmbH, Schnepel / Hoesel
	 * @version $Id: GridCellRendererPrint.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
	 */
	public static class PrintHandler<R> implements CustomPrintHandler {

		/**
		 * Die minimale Gre einer einzelnen Zelle.
		 */
		private static Rectangle minBounds = new Rectangle(0, 0, 1, 1);

		/**
		 * Die DPI Zahl des Monitors.
		 */
		private static Point screenDPI = Display.getDefault().getDPI();

		/**
		 * Skalierungsfaktor.
		 */
		private static float scale = 2.0f;

		/**
		 * Der Renderer, welcher fr die Zelle verwendet wird.
		 */
		private final GridCellRenderer cellRenderer;

		/**
		 * Der Inhalt der Zelle.
		 */
		private final R r;

		/**
		 * Das {@link Device}, auf das gezeichnet werden soll (also gewhnlich
		 * ein Drucker).
		 */
		private Device device;

		/**
		 * Die aktuelle Gre der Zelle.
		 */
		private Point size = new Point(0, 0);

		/**
		 * Erzeugt einen neuen Handler zum Ausgeben einer Zelle eines
		 * {@link org.eclipse.nebula.widgets.grid.Grid}s auf einem Drucker.
		 * 
		 * @param cellRenderer
		 *            Der Renderer, welcher fr die Zelle verwendet wird.
		 * @param r
		 *            Der Inhalt der Zelle.
		 */
		public PrintHandler(final GridCellRenderer cellRenderer, final R r) {
			this.cellRenderer = cellRenderer;
			this.r = r;
		}

		@Override
		public void setDevice(final Device device) {
			this.device = device;
		}

		@Override
		public Point getSize() {
			cellRenderer.setBounds(minBounds);
			return convertDPI(cellRenderer.getSize(), screenDPI, device
					.getDPI(), 1 / scale);
		}

		@Override
		public void setSize(final int width, final int height) {
			this.size = new Point(width, height);
		}

		@Override
		public void paint(final GC gc, final int x, final int y) {
			final Transform oldTransform = new Transform(gc.getDevice());
			gc.getTransform(oldTransform);
			final Transform transform = new Transform(gc.getDevice());
			gc.getTransform(transform);
			transform.translate(x, y);
			final Point deviceDPI = device.getDPI();
			transform.scale(deviceDPI.x / scale / screenDPI.x, deviceDPI.y
					/ scale / screenDPI.y);
			gc.setTransform(transform);
			final Point printSize = convertDPI(size, deviceDPI, screenDPI,
					scale);
			cellRenderer.setBounds(0, 0, printSize.x, printSize.y);
			cellRenderer.paint(gc, r);
			gc.setTransform(oldTransform);
		}

		/**
		 * Umrechnung der Gre der Zelle von der Darstellung auf dem Bildschirm
		 * in die Darstellung auf dem Drucker, inkl. eines Skalierungsfaktors.
		 * 
		 * @param oldSize
		 *            die Ursprngliche Gre der Zelle.
		 * @param fromDPI
		 *            DPI Zahl des Ursprungsdevices.
		 * @param toDPI
		 *            DPI Zahl des Zieldevices.
		 * @param scalefactor
		 *            Skalierungsfaktor.
		 * @return die neue Gre der Zelle.
		 */
		private Point convertDPI(final Point oldSize, final Point fromDPI,
				final Point toDPI, final float scalefactor) {
			return new Point(
					(int) (scalefactor * oldSize.x * toDPI.x / fromDPI.x),
					(int) (scalefactor * oldSize.y * toDPI.y / fromDPI.y));
		}

		@Override
		public boolean isGreedyX() {
			return true;
		}

		@Override
		public boolean isGreedyY() {
			return true;
		}

	}

	/**
	 * Erzeugt ein neues Print - Objekt zum Drucken einer Zelle.
	 * 
	 * @param cellRenderer
	 *            der Renderer zum Malen der Zelle.
	 * @param r
	 *            der Zelleninhalt.
	 */
	public GridCellRendererPrint(final GridCellRenderer cellRenderer, final R r) {
		super(new PrintHandler<R>(cellRenderer, r));

	}

}
