/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.adapter;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.SafeRunner;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AeussererStreckenAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.InnererStreckenAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.MessQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.RealerStreckenAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenKnoten;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenTeilAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenZugEintraegeContainer;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenZugEintrag;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.sys.funclib.bitctrl.modell.tmgeoreferenzierungglobal.konfigurationsdaten.KdPunktLiegtAufLinienObjekt;

/**
 * Job zur Berechnung der relativen Position aller Objekte auf einem
 * Streckenzug.
 * 
 * @author BitCtrl Systems GmbH, Schnepel / Hoesel
 * @version $Id: WegeCalculatorJob.java,v 1.5 2014/04/17 15:51:19 Pittner Exp $
 */
public final class WegeCalculatorJob extends Job {

	/**
	 * Abstand zwischen 2 {@link MessQuerschnitt} eines {@link StreckenKnoten}s.
	 */
	private static final int MQ_OFFSET = 150;

	/** Das Streckenprofil. */
	private final StreckenprofilEditorModel model;

	/**
	 * Erzeugt einen neuen {@link Job} zur Berechnung der Positionen innerhalb
	 * des Streckenprofils.
	 * 
	 * @param model
	 *            das Streckenprofil.
	 */
	public WegeCalculatorJob(final StreckenprofilEditorModel model) {
		super("Weg berechnen...");
		this.model = model;
	}

	@Override
	protected IStatus run(final IProgressMonitor monitor) {
		final double wegBisher = 0;
		SafeRunner.run(new ISafeRunnable() {

			@Override
			public void run() throws Exception {
				calc(wegBisher, model.getStreckenZug());
			}

			@Override
			public void handleException(final Throwable exception) {
				Logger.getLogger(getClass().getName()).log(Level.WARNING, "",
						exception);
			}
		});

		return Status.OK_STATUS;
	}

	/**
	 * Berechnet die Positionen der {@link MessQuerschnitt}e,
	 * {@link AnzeigeQuerschnitt}e, {@link StreckenKnoten} usw. eines
	 * {@link StreckenZugEintraegeContainer}s. <strong> ACHTUNG: Diese Methode
	 * enthlt eine Rekursion.</strong>
	 * 
	 * @param wegBisher
	 *            der Weg, der bis hierher schon zurck gelegt wurde
	 * @param streckenZugEintragContainer
	 *            ein {@link StreckenZugEintraegeContainer}, der weitere Objekte
	 *            enthlt.
	 * @return der Weg, der nach dem bergebenen
	 *         {@link StreckenZugEintraegeContainer} zurck gelegt wurde und
	 *         damit der Beginn des nchsten
	 *         {@link StreckenZugEintraegeContainer}s ist.
	 */
	private double calc(
			final double wegBisher,
			final StreckenZugEintraegeContainer<? extends StreckenZugEintrag> streckenZugEintragContainer) {
		double aktuellerWegPunkt = wegBisher;
	
		final List<StreckenZugEintrag> streckenZugEintraegeListe = new ArrayList<StreckenZugEintrag>();
		streckenZugEintraegeListe.addAll(streckenZugEintragContainer
				.getStreckenZugEintraege());

		for (final StreckenZugEintrag eintrag : streckenZugEintraegeListe) {
			if (eintrag != null) {
			  
				eintrag.setWegBisher(aktuellerWegPunkt);				

				if (eintrag instanceof RealerStreckenAbschnitt<?>) {
					final List<MessQuerschnitt> mqList = new ArrayList<MessQuerschnitt>();
					mqList.addAll(((RealerStreckenAbschnitt<?>) eintrag)
							.getMessQuerschnitte());

					for (final MessQuerschnitt mq : mqList) {
						// ein MQ selber hat keine Lnge
						final KdPunktLiegtAufLinienObjekt.Daten mqwegeDaten = mq.getMessQuerschnitt().getKdPunktLiegtAufLinienObjekt().getDatum();
            if (mqwegeDaten != null)
            {
              if (eintrag instanceof AeussererStreckenAbschnitt) 
              {
                mq.setWegBisher(aktuellerWegPunkt + mqwegeDaten.getOffset().doubleValue());
              } 
              else if (eintrag instanceof InnererStreckenAbschnitt) 
              {
                mq.setWegBisher(aktuellerWegPunkt + mqwegeDaten.getOffset().doubleValue());
              }              
            }				
					}

          final List<AnzeigeQuerschnitt> aqList = new ArrayList<AnzeigeQuerschnitt>();
          aqList.addAll(((RealerStreckenAbschnitt<?>) eintrag)
              .getAnzeigeQuerschnitt());

          for (final AnzeigeQuerschnitt aq : aqList) {
            // ein AQ selber hat keine Lnge
            final KdPunktLiegtAufLinienObjekt.Daten aqwegeDaten = aq.getModellObjekt()
                .getKdPunktLiegtAufLinienObjekt().getDatum();

            if (aqwegeDaten != null)
            {
              if (eintrag instanceof AeussererStreckenAbschnitt) 
              {
                aq.setWegBisher(aktuellerWegPunkt + aqwegeDaten.getOffset().doubleValue());
              } 
              else if (eintrag instanceof InnererStreckenAbschnitt) 
              {
                aq.setWegBisher(aktuellerWegPunkt + aqwegeDaten.getOffset().doubleValue());
              }              
            }          
          }

					if (eintrag instanceof AeussererStreckenAbschnitt) {
						final AeussererStreckenAbschnitt abschnitt = (AeussererStreckenAbschnitt) eintrag;
						// Streckenteilabschnitte
						double tempWeg = eintrag.getWegBisher();
						final List<StreckenTeilAbschnitt> list = new ArrayList<StreckenTeilAbschnitt>();
						list.addAll(abschnitt.getStreckenTeilAbschnitt());
						for (final StreckenTeilAbschnitt teilAbschnitt : list) {
							teilAbschnitt.setWegBisher(tempWeg);
							if (teilAbschnitt.getLaenge() != null) {
								tempWeg += teilAbschnitt.getLaenge();
							}
						}
					}
				}

				if (eintrag instanceof StreckenZugEintraegeContainer<?>) {
					final double wegBisher2 = calc(aktuellerWegPunkt,
							(StreckenZugEintraegeContainer<?>) eintrag);
						aktuellerWegPunkt = wegBisher2;
				} 
			}
		
			if (eintrag instanceof StreckenKnoten) {
        StreckenKnoten sk = (StreckenKnoten)eintrag;
        final Double laenge = sk.getLaenge();
	      if (laenge != null) {	        	        
	        aktuellerWegPunkt += laenge;
	      }
	    }

      if (eintrag instanceof AeussererStreckenAbschnitt) {          
        AeussererStreckenAbschnitt ass = (AeussererStreckenAbschnitt)eintrag;        
        final Double laenge = ass.getLaenge();
        if (laenge != null) {                    
          aktuellerWegPunkt += laenge;
        }
      }

//      if (eintrag instanceof Strecke) {          
//        Strecke s = (Strecke)eintrag;        
//        final Double laenge = s.getLaenge();
//        if (laenge != null) {                    
//            aktuellerWegPunkt += laenge;
//        }
//      }
		}
	
		return aktuellerWegPunkt;
	}
}
