/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.util;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.Activateable;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.ActivateableContainer;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.ActivationState;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AlternativeStreckenAbschnitteContainer;

/**
 * Hilsklasse zum Ermitteln des {@link ActivationState}s eines
 * {@link Activateable}s.
 * 
 * @author BitCtrl Systems GmbH, Schnepel/Hoesel
 * @version $Id: Activateables.java,v 1.3 2014/04/17 15:51:09 Pittner Exp $
 */
public final class Activateables {

	/**
	 * Konstruktor verstecken.
	 */
	private Activateables() {

	}

	/**
	 * Setzt den Status dieses {@link Activateable}s entsprechend dessen
	 * Kindern.
	 * 
	 * @param activateable
	 *            das Parent {@link Activateable}.
	 * @param active
	 *            der zu setzende {@link ActivationState}.
	 */
	public static void dispatchSetActive(final Activateable activateable,
			final ActivationState active) {
		setActive(activateable, activateable.getActivateableContainer(), active);
		setCalcActive(activateable);
	}

	/**
	 * Berechnet und setzt den aktuellen Status Geht dann die Hierachie nach
	 * oben.
	 * 
	 * @param activateable
	 *            das {@link Activateable}, dessen {@link ActivationState} neu
	 *            berechnet und gesetzt werden soll.
	 */
	private static void setCalcActive(final Activateable activateable) {
		activateable.setInternalState(calcActive(activateable));
		setCalcActiveContainer(activateable, activateable
				.getActivateableContainer());
	}

	/**
	 * Der geht die Hierarchie nach Oben und ermittelt den
	 * {@link ActivationState}.
	 * 
	 * @see {@link Activateables#calcActive(Activateable)}
	 * 
	 * @param activateable
	 *            der Ausgangspunkt.
	 * @param activateableContainer
	 *            das Container - Objekt
	 */
	private static void setCalcActiveContainer(
			final Activateable activateable,
			final ActivateableContainer<? extends Activateable> activateableContainer) {
		if (activateableContainer != null) {
			if (ActivationState.ACTIVE.equals(activateable.getState())
					|| !(activateableContainer.getActivateableContainer() instanceof AlternativeStreckenAbschnitteContainer<?>)) {
				activateableContainer.setInternalState(activateable.getState());
			}
			setCalcActive(activateableContainer);
		}
	}

	/**
	 * Setzt den Status des Parameters activateable auf den uebergebenen Status
	 * im Parameter active. Der das eltern-Element wird in activateableContainer
	 * uebergeben. Dispatcher.
	 * 
	 * @param activateable
	 *            das Objekt, dessen Aktivierungsstatus gesetzt werden soll.
	 * @param activateableContainer
	 *            das Eltern-Element des {@link Activateable}s
	 * @param active
	 *            der neue {@link ActivationState} des {@link Activateable}s.
	 */
	private static void setActive(
			final Activateable activateable,
			final ActivateableContainer<? extends Activateable> activateableContainer,
			final ActivationState active) {

		if (activateable instanceof ActivateableContainer<?>) {
			final ActivateableContainer<?> container = (ActivateableContainer<?>) activateable;
			activateable.setInternalState(active);

			for (final Activateable a : container.getActivateables()) {
				setActive(a, container, active);
			}
			activateable.setInternalState(calcActive(activateable));
		} else {
			activateable.setInternalState(active);
			activateable.setInternalState(calcActive(activateable));
		}
	}

	/**
	 * Ermittelt (berechnet) den Aktivierungs-Status des {@link Activateable}s,
	 * anhand der Stati der Kinder. Dispatcher.
	 * 
	 * @param activateable
	 *            das {@link Activateable}, dessen {@link ActivationState}
	 *            ermittelt werden soll.
	 * @return der {@link ActivationState} des {@link Activateable}s.
	 */
	private static ActivationState calcActive(final Activateable activateable) {
		if (activateable instanceof ActivateableContainer<?>) {
			final Set<ActivationState> states = usedActivationStates(((ActivateableContainer<?>) activateable)
					.getActivateables());
			if (null == states || states.isEmpty()) {
				return activateable.getState();
			} else {
				return States.mergeStates(states);
			}
		}

		return activateable.getState();
	}

	/**
	 * Liefert die Menge der {@link ActivationState}s, die in der bergebenen
	 * Menge von {@link Activateable}s anzutreffen sind.
	 * 
	 * @param col
	 *            eine Menge von {@link Activateable}s.
	 * @return die Menge der {@link ActivationState}s der {@link Activateable}s.
	 */
	private static Set<ActivationState> usedActivationStates(
			final Collection<? extends Activateable> col) {
		final Set<ActivationState> result = new HashSet<ActivationState>();
		for (final Activateable a : col) {
			result.add(a.getState());
		}
		return result;
	}

}
