/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.util;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.ActivationState;

/**
 * Util Klasse zur Ermittlung der Status eines
 * {@link de.bsvrz.buv.plugin.streckenprofil.sts.a8.util.Activateables} eines
 * Streckenprofils.
 * 
 * @author BitCtrl Systems GmbH, Schnepel/Hoesel
 * @version $Id: States.java,v 1.3 2014/04/17 15:51:09 Pittner Exp $
 */
public final class States {

	/**
	 * Menge aller {@link ActivationState}s, die als "aktiviert" betrachtet
	 * werden.
	 */
	public static final Set<ActivationState> ACTIVE_STATES = new HashSet<ActivationState>();

	/**
	 * Menge aller {@link ActivationState}, die als "nicht aktiviert" betrachtet
	 * werden.
	 */
	public static final Set<ActivationState> INACTIVE_STATES = new HashSet<ActivationState>();

	static {
		ACTIVE_STATES.add(ActivationState.ACTIVE);
		ACTIVE_STATES.add(ActivationState.SEMI_ACTIVE);
		INACTIVE_STATES.add(ActivationState.INACTIVE);
	}

	/**
	 * Konstruktor verstecken.
	 */
	private States() {

	}

	/**
	 * Ermittelt den {@link ActivationState} ber eine Menge von
	 * {@link ActivationState}s.
	 * 
	 * @param states
	 *            eine Menge von {@link ActivationState}s.
	 * @return der zusammengefate {@link ActivationState}.
	 */
	public static ActivationState mergeStates(
			final Collection<ActivationState> states) {
		if (states.contains(ActivationState.SEMI_ACTIVE)) {
			return ActivationState.SEMI_ACTIVE;
		} else {
			final boolean a = !Collections.disjoint(ACTIVE_STATES, states);
			final boolean i = !Collections.disjoint(INACTIVE_STATES, states);
			if ((a && !i) || (!a && i)) {
				return a ? ActivationState.ACTIVE : ActivationState.INACTIVE;
			} else {
				return ActivationState.SEMI_ACTIVE;
			}
		}
	}

	/**
	 * Konvertiert ein {@link Boolean} in einen {@link ActivationState}.
	 * 
	 * @param active
	 *            <code>true</code>, wenn das Objekt "aktiviert" ist, sonst
	 *            <code>false</code>
	 * @return der daraus resultieren {@link ActivationState}.
	 */
	public static ActivationState convert(final boolean active) {
		return active ? ActivationState.ACTIVE : ActivationState.INACTIVE;
	}
}
