/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.util;

import java.util.Set;

import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.sys.funclib.bitctrl.modell.metamodellglobal.attribute.AttJaNein;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.konfigurationsdaten.KdAeusseresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.konfigurationsdaten.KdInneresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.konfigurationsdaten.KdInneresStrassenSegmentHauptFahrBeziehung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.konfigurationsdaten.KdStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.AeusseresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.InneresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Strasse;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenKnoten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenSegment;

/**
 * Helper Klasse zur Ermittlung der
 * {@link de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt}e, die in einem
 * Streckenprofil vorkommen. Vorbedingung zur Verwendung dieser Klasse ist, das
 * der {@link de.bsvrz.sys.funclib.bitctrl.modell.util.cache.MqCache} und der
 * {@link de.bsvrz.sys.funclib.bitctrl.modell.util.cache.NetzCache}
 * initialisiert sind.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: SystemObjekte.java,v 1.3 2014/04/17 15:51:10 Pittner Exp $
 */
public final class SystemObjekte {

	/**
	 * Konstruktor verstecken.
	 */
	private SystemObjekte() {

	}

	/**
	 * Liefert alle {@link MessQuerschnittAllgemein}e, die zu einem
	 * {@link StrassenSegment} gehren.
	 * 
	 * @param segment
	 *            das {@link StrassenSegment}, dessen MQs gesucht werden.
	 * @return Menge mit allen MQs des {@link StrassenSegment}es.
	 */
	public static Set<MessQuerschnittAllgemein> messQuerschnitte(
			final StrassenSegment segment) {
		return StartupPlugin.getDefault().getDefaultMqCache()
				.getMessquerschnitte(segment);
	}

	/**
	 * Liefert die {@link Strasse} zu der das bergebene {@link StrassenSegment}
	 * gehrt.
	 * 
	 * @param segment
	 *            das {@link StrassenSegment}, dessen {@link Strasse} gesucht
	 *            wird.
	 * @return die {@link Strasse}, zu der das {@link StrassenSegment} gehrt,
	 *         oder <code>null</code>, wenn in der Konfiguration nicht versorgt.
	 */
	public static Strasse gehoertZuStrasse(final StrassenSegment segment) {
		Strasse result = null;
		final KdStrassenSegment.Daten d = segment.getKdStrassenSegment()
				.getDatum();
		if (d != null) {
			result = d.getGehoertZuStrasse();
		}
		return result;
	}

	/**
	 * Ermittelt die {@link AttTmcRichtung} des {@link AeusseresStrassenSegment}
	 * s.
	 * 
	 * @param segment
	 *            das {@link AeusseresStrassenSegment}.
	 * @return die {@link AttTmcRichtung}.
	 */
	public static AttTmcRichtung richtung(final AeusseresStrassenSegment segment) {
		AttTmcRichtung result = null;
		final KdAeusseresStrassenSegment.Daten datum = segment
				.getKdAeusseresStrassenSegment().getDatum();
		if (datum != null) {
			result = datum.getTmcRichtung();
		}
		return result;
	}

	/**
	 * Ermittelt den {@link StrassenKnoten}, an dem das
	 * {@link AeusseresStrassenSegment} beginnt.
	 * 
	 * @param segment
	 *            das {@link AeusseresStrassenSegment}.
	 * @return der {@link StrassenKnoten}, an dem das
	 *         {@link AeusseresStrassenSegment} beginnt.
	 */
	public static StrassenKnoten vonKnoten(
			final AeusseresStrassenSegment segment) {
		return segment.getKdAeusseresStrassenSegment().getDatum()
				.getVonKnoten();
	}

	/**
	 * Ermittelt den {@link StrassenKnoten}, an dem das
	 * {@link AeusseresStrassenSegment} endet.
	 * 
	 * @param segment
	 *            das {@link AeusseresStrassenSegment}.
	 * @return der {@link StrassenKnoten}, an dem das
	 *         {@link AeusseresStrassenSegment} endet.
	 */
	public static StrassenKnoten nachKnoten(
			final AeusseresStrassenSegment segment) {
		return segment.getKdAeusseresStrassenSegment().getDatum()
				.getNachKnoten();
	}

	/**
	 * Liefert das {@link AeusseresStrassenSegment}, das dem ebergebenen
	 * {@link InneresStrassenSegment} folgt.
	 * 
	 * @param segment
	 *            des {@link InneresStrassenSegment},dessen Nachfolger gesucht
	 *            wird.
	 * @return das Nachfolger - {@link AeusseresStrassenSegment}, oder
	 *         <code>null</code>, wenn nicht in der Konfiguration versorgt.
	 */
	public static AeusseresStrassenSegment nachStrassenSegment(
			final InneresStrassenSegment segment) {
		AeusseresStrassenSegment result = null;
		final KdInneresStrassenSegment.Daten d = segment
				.getKdInneresStrassenSegment().getDatum();
		if (d != null) {
			result = d.getNachStrassenSegment();
		}
		return result;
	}

	/**
	 * Liefert das {@link AeusseresStrassenSegment}, an das dieses
	 * {@link InneresStrassenSegment} grenzt.
	 * 
	 * @param segment
	 *            das {@link InneresStrassenSegment}, dessen Vorgnger gesucht
	 *            wird.
	 * @return das Vorgnger - {@link AeusseresStrassenSegment}, oder
	 *         <code>null</code>, wenn nicht in der Konfiguration versorgt.
	 */
	public static AeusseresStrassenSegment vonStrassenSegment(
			final InneresStrassenSegment segment) {
		AeusseresStrassenSegment result = null;
		final KdInneresStrassenSegment.Daten d = segment
				.getKdInneresStrassenSegment().getDatum();
		if (d != null) {
			result = d.getVonStrassenSegment();
		}
		return result;
	}

	/**
	 * Liefert den {@link StrassenKnoten}, zu dem das bergebene
	 * {@link InneresStrassenSegment} gehrt.
	 * 
	 * @param inneres
	 *            das {@link InneresStrassenSegment}.
	 * @return der {@link StrassenKnoten}, zu dem das bergebene
	 *         {@link InneresStrassenSegment} gehrt.
	 */
	public static StrassenKnoten knoten(final InneresStrassenSegment inneres) {
		return StartupPlugin.getDefault().getDefaultNetzCache()
				.getStrassenKnoten(inneres);
	}

	/**
	 * Ermittelt, ob es sich bei dem bergebenen {@link InneresStrassenSegment}
	 * im die Hauptfahrbeziehung handelt.
	 * 
	 * @param inneres
	 *            das {@link InneresStrassenSegment}
	 * @return <code>true</code>, wenn das {@link InneresStrassenSegment} eine
	 *         Hauptfahrbeziehung darstellt und <code>false</code>,wenn nicht.
	 *         Hier kann allerdings auch <code>null</code> zurckkommen, nmlich
	 *         ,wenn das Datum in der Konfiguration nicht versorgt ist.
	 */
	public static Boolean isHauptFahrbeziehung(
			final InneresStrassenSegment inneres) {
		Boolean result = null;
		final KdInneresStrassenSegmentHauptFahrBeziehung.Daten d = inneres
				.getKdInneresStrassenSegmentHauptFahrBeziehung().getDatum();
		if (d != null) {
			result = d.getDurchgehendeHauptFahrBeziehung().equals(
					AttJaNein.ZUSTAND_1_JA);
		}
		return result;
	}
}
