/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.autarkprogramm;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.MouseWheelHandler;
import org.eclipse.gef.MouseWheelZoomHandler;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaAnsichtProvider;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaEinstellungen;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DoModelHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.views.SbaScalableRootEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.DarstellungEditPartFactory;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.ursacheneinheit.verwaltung.DaVKommunikation;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse realisert die Darstellung und Bearbeitung des Schaltbild-Parameters (Einzelmassnahme) einer
 * Ursacheneinheit. Diese wird in einer Registerkarte der Sicht {@code EinzelmassnahmenViewParameter} verwendet.
 * 
 * @author Pittner
 * 
 */
public class CompositeSchaltbildVersorgungAutarkProgramm extends Composite
{
  /**
   * Die Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection;

  private Composite _composite2;

  private Composite _composite21;

  /**
   * Der GEF-Viewer
   */
  private GraphicalViewer graphicalViewer;

  /**
   * Datenfeld mit den aktuellen Einstellungen
   */
	private Data _data;

  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  private static final String _attributgruppe = "atg.sswSbaAutarkprogramm";

  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspekt = "asp.parameterSoll";

  /**
   * Konstruktor
   * 
   * @param parent
   *          das Elternobjekt
   * @param style
   *          der Stil wie das {@code Composite} dargestellt werden soll
   * @param so
   *          die Ursacheneinheit
   */
  public CompositeSchaltbildVersorgungAutarkProgramm(Composite parent, int style, SystemObject so)
  {
    super(parent, style);
    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    createInhaltTab(parent, so);

  }

  /**
   * Initialisiert den Inhalt des {@code TabItem}
   * 
   * @param tabItemComposite
   *          das Eltern Objekt
   * @param so
   *          die Ursacheneinheit
   */
  private void createInhaltTab(Composite tabItemComposite, SystemObject so)
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    
    createComposite2(tabItemComposite);

    aktualisiereBaum(so);
    
    setzeAlleAnzeigen(AnzeigeInhaltTyp.BELIEBIG_BI);
  
    Data item = _data;
    aktualisiereDarstellung(item);

  }

  /**
   * This method initializes _composite2
   * 
   */
  private void createComposite2(Composite parent)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _composite2 = new Composite(parent, SWT.NONE);
    _composite2.setLayout(new GridLayout());
    _composite2.setLayoutData(gridData);

    createGraphicalViewer();
  }

  /**
   * GraphicalViewer fr die Darstellung
   */
  private void createGraphicalViewer()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _composite21 = new Composite(_composite2, SWT.BORDER);
    _composite21.setLayout(new FillLayout());
    _composite21.setLayoutData(gridData);

    /*
     *  Ansicht in einen extra Viewer packen.
     */
    graphicalViewer = new ScrollingGraphicalViewer();
    graphicalViewer.createControl(_composite21);

    /*
     *  XXX: Wichtig - Damit wei der Viewer welchen EditPart er fr ein
     *  Modell Objekt benutzen soll.
     */
    graphicalViewer.setEditDomain(new EditDomain());
    graphicalViewer.setRootEditPart(new SbaScalableRootEditPart());
    graphicalViewer.setEditPartFactory(new DarstellungEditPartFactory());
    
    // Hier wird der Viewer global mit dem Kontext fr die Parameterdarstellung versorgt, da in der Legende der Kontext 
    // fr Fahrstreifen und Detektor bentigt wird, diese aber vin der Emf-Modelldefinition keinen Kontext enthalten. 
    graphicalViewer.getControl().setData("kontext", 5);
    
    
//    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
//    String name = store.getString(DefsPreference.P_DARSTELLUNG_PARAMETER);    
//    if (name != null && name.length() > 0){
//      graphicalViewer.setContents(EcoreUtil.copy(SbaEinstellungen.INSTANCE.getAnsicht(name)));    
//    }

    Ansicht ansicht = SbaAnsichtProvider.getInstanz().getAnsichtKopie(DefsPreference.P_DARSTELLUNG_PARAMETER);
    if (ansicht != null)
      graphicalViewer.setContents(ansicht);    
    
    final ScalableRootEditPart rootEditPart = (ScalableRootEditPart)graphicalViewer.getRootEditPart();

//    final ZoomManager zoomManager = rootEditPart.getZoomManager();
//    zoomManager.setZoomLevels(new double[] { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0 });
//    graphicalViewer.setProperty(MouseWheelHandler.KeyGenerator.getKey(SWT.NONE), MouseWheelZoomHandler.SINGLETON);

    if (DaVKommunikationDobj.getInstanz().holeDaVVerbindung() != null)
    {

      AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();
//      Ansicht ansicht = contents.getModel();

      StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
      darstellung.setSpaltenBorder(true);

      contents = (AnsichtEditPart)graphicalViewer.getContents();
      ansicht = contents.getModel();
      darstellung = (StilisierteDarstellung)ansicht.getDarstellung();
    }

    graphicalViewer.getControl().addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDoubleClick(MouseEvent e)
      {
        ISelection selection = graphicalViewer.getSelection();
        DoModelHelper.holeDoppelKlickAktionAusViewerSelection(selection, _data);
      }
    });
  }

  /**
   * 
   */
  /**
   * Vorbelegung aller Anzeigen mit einer Inhalt
   * @param inhalt
   */
  private void setzeAlleAnzeigen(AnzeigeInhaltTyp  inhalt)
  {
    AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {
      EList<DoModel> dos = ebene.getDoObjekte();
      for (DoModel dom : dos)
      {
        if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
        {
          AnzeigeQuerschnittModelTypAbstract model = (AnzeigeQuerschnittModelTypAbstract)dom;
          EList<AnzeigeModelTypAbstract> komponenten = model.getKomponenten();
          for (AnzeigeModelTypAbstract anzeige : komponenten)
          {
            EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
            final SetCommand cmd = new SetCommand(anzeige, feature, inhalt);
            getCommandStack().execute(cmd);
          }
        }
      }
    }
  }

  /**
   * Aktualisert die Darstellung im {@code GraphicalViewer} mit dem Daten des uebergebenen {@code SystemObject}
   * 
   * @param die
   *          Ursacheneinheit
   */
  protected void aktualisiereBaum(SystemObject so)
  {
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(_attributgruppe);
    Aspect asp = _connection.getDataModel().getAspect(_aspekt);
    ResultData result = _connection.getData(so, new DataDescription(atg, asp), 1500);

    Data data = null;
    if (result.hasData())
    	data = result.getData();
    
    if (data == null)
	    data = _connection.createData( atg );
    
    if (data != null)
    {
      _data = data.createModifiableCopy();
    }
  }

  /**
   * Holt das {@code DoModel} aus dem Parameter atg.sswSbaEinzelMassnahme und benachrichtigt den {@code GraphicalViewer}
   * zur Darstellung der Aenderung
   * 
   * @param die
   *          Daten aus der Atg
   */
  private void aktualisiereDarstellung(Data item)
  {
    if (item != null)
    {
    	Array array = item.getArray("Anzeige");
    	for (int i=0; i<array.getLength(); ++i)
    	{
        Data item2 = array.getItem(i);
        SystemObject soAnzeige = item2.getItem("Anzeige").asReferenceValue().getSystemObject();

        if (soAnzeige != null)
        {
//          SystemObject soAnzeigeInhalt = item2.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue()
//              .getSystemObject();
//          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.wvzInhalt");
//          Data cd = soAnzeigeInhalt.getConfigurationData(atg);
//          String bi = cd.getItem("Bildinhalt").asTextValue().getText();
//          AnzeigeInhaltTyp inhalt = AnzeigeInhaltTyp.setBildInhalt(bi, null);

          AnzeigeModelTypAbstract doModel = DoModelHelper.holeAnzeigeModelTypAbstractVonSystemObject(
              (AnsichtEditPart)graphicalViewer.getContents(), soAnzeige,
              DoModelHelper.ENUM_DARSTELLUNG_KONTEXT.PARAMETER);
          
          if (doModel != null)
          {
            SystemObject soAnzeigeInhalt = item2.getItem("Schaltbild").getItem("AnzeigeInhalt").asReferenceValue()
            .getSystemObject();
            
            if (soAnzeigeInhalt != null) {
              AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.wvzInhalt");
              Data cd = soAnzeigeInhalt.getConfigurationData(atg);
              String bi = cd.getItem("Bildinhalt").asTextValue().getText();
              AnzeigeInhaltTyp inhalt = AnzeigeInhaltTyp.setBildInhalt(bi, null); 
              
              EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
              final SetCommand cmd = new SetCommand(doModel, feature, inhalt);
              getCommandStack().execute(cmd);
            }
            else {
              Data item3 = item2.getItem("Schaltbild").getItem("DynamischerWechseltext");
              if (item3 != null) {
                VisuAnzeigeKapsel kapsel = DaVHelper.holeDynamischerWechselTextAusData(item3);           
                EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                final SetCommand cmd = new SetCommand(doModel, feature, kapsel);
                getCommandStack().execute(cmd);
              }
            }
          }
        }
      }
    }
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {

    CommandStack stack = graphicalViewer.getEditDomain().getCommandStack();

    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  /**
   * Holt den GEF-Viewer
   * 
   * @return viewer
   */
  public GraphicalViewer getGraphicalViewer()
  {
    return graphicalViewer;
  }

  /**
   * Setzt den GEF-Viewer
   * 
   * @param graphicalViewer
   */
  public void setGraphicalViewer(GraphicalViewer graphicalViewer)
  {
    this.graphicalViewer = graphicalViewer;
  }

	/**
   * Methode liefert die Datenstruktur mit den aktuell eingestellten Daten zurck
   * @return data
   */
  public Data getData()
  {
  	return _data;
  }
}
