package de.bsvrz.buv.plugin.ursacheneinheit.defaultParameter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.ursacheneinheit.Activator;
import de.bsvrz.buv.plugin.ursacheneinheit.verwaltung.VersorgungsArt;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.sys.funclib.dataSerializer.NoSuchVersionException;

public class DefaultParameterMultiHandler
{
  Button _button;
  IDefaultParameterMultiVersorgung _versorgung;
  
  public DefaultParameterMultiHandler(Composite parent, IDefaultParameterMultiVersorgung versorgung) {
    
    switch (versorgung.getVersorgungsArt()) {
        case EinReiterProUrsachenEinheit:
        case EinReiterFuerAlleUrsachenEinheiten:
          _versorgung = versorgung;  
          break;
        default:
          throw new IllegalStateException("Nicht untersttzte Versorgungsart");
    }

    _button = new Button( parent, SWT.NONE );
    _button.setText("Defaultparam.");
    
    if (versorgung.getVersorgungsArt() == VersorgungsArt.EinReiterFuerAlleUrsachenEinheiten) {
      _button.setToolTipText("Setzen aller Parameterwerte der ausgewhlten Objekte"
          + " (nicht: Zuordnungen von MQ/UFD) auf die Werte der"
          + " entsprechenden Defaultparameter");  
    }
    else {
      _button.setToolTipText("Setzen aller Parameterwerte des aktuellen Objekts"
          + " (nicht: Zuordnungen von MQ/UFD) auf die Werte der"
          + " entsprechenden Defaultparameter");
    }
   
    _button.addSelectionListener(new SelectionListener());
  }
  
  private void buttonPressed() {
    System.out.println("_versorgung.getVersorgungsArt():" + _versorgung.getVersorgungsArt());
    System.out.println("_versorgung.getRootObjektPid():" + _versorgung.getRootObjektPid());
    System.out.println("_versorgung.getParameterAtgPids():" + _versorgung.getParameterAtgPids());
  
    String[] atgPids = _versorgung.getParameterAtgPids();
    Data[] daten = new Data[atgPids.length];
    Arrays.fill(daten, null);
    Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
    List<String> nichtGefundeneAtgs = new ArrayList<String>();
    
    try {
      for (int i = 0; i < daten.length; i++) {
        daten[i] = DefaultParameterTools.getData(_versorgung.getRootObjektPid(), atgPids[i]);
        if (daten[i] == null) {
          nichtGefundeneAtgs.add(atgPids[i]);  
        }
      }
    }
    catch (NoSuchVersionException e) {
      String msg = "Versions-Fehler beim Deserialisieren von Defaultparametern fr " + _versorgung.getRootObjektPid();
      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
      MessageBox msgBox = new MessageBox(shell, SWT.ICON_ERROR);
      msgBox.setText("Fehler");
      msgBox.setMessage(msg + ":\n" + e.getMessage());
      msgBox.open();
      return;
    }
    catch (IOException e) {
      String msg = "I/O-Fehler beim Lesen von Defaultparametern fr " + _versorgung.getRootObjektPid();
      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
      MessageBox msgBox = new MessageBox(shell, SWT.ICON_ERROR);
      msgBox.setText("Fehler");
      msgBox.setMessage(msg + ":\n" + e.getMessage());
      msgBox.open();
      return;
    }
      
    if (nichtGefundeneAtgs.size() < atgPids.length) {
      // Mindestens 1 Datensatz gefunden
      MessageBox msgBox = new MessageBox(shell, SWT.ICON_QUESTION | SWT.OK | SWT.CANCEL);
      msgBox.setText("Defaultparameter setzen");
      if (_versorgung.getVersorgungsArt() == VersorgungsArt.EinReiterFuerAlleUrsachenEinheiten) {
        msgBox.setMessage("Sollen die Parameterwerte der ausgewhlten  Objekte (nicht: Zuordnungen von MQ/UFD) auf die Werte der entsprechenden Defaultparameter gesetzt werden?");  
      }
      else {
        msgBox.setMessage("Sollen die Parameterwerte des aktuellen Objekts (nicht: Zuordnungen von MQ/UFD) auf die Werte der entsprechenden Defaultparameter gesetzt werden?");  
      }
      
      if (msgBox.open() == SWT.OK) {
        _versorgung.setData(daten);  
      }
    }
    else {
      // Kein Datensatz gefunden
      MessageBox msgBox = new MessageBox(shell, SWT.ICON_WARNING);
      msgBox.setText("Defaultparameter nicht gefunden");
      msgBox.setMessage("Kein Defaultparameter gefunden fr " +  _versorgung.getRootObjektPid());
      msgBox.open();
    }
    
    System.out.println("Deserialisierte Daten:");
    for (Data data: daten) {
      System.out.println(data);
    }
  }
  
  private class SelectionListener extends SelectionAdapter {
    
    public void widgetSelected(SelectionEvent e) {
      buttonPressed();
    }
    
  }
  
}
  
