package de.bsvrz.buv.plugin.ursacheneinheit.defaultParameter;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import de.bsvrz.buv.plugin.ursacheneinheit.verwaltung.DaVKommunikation;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.dataSerializer.Deserializer;
import de.bsvrz.sys.funclib.dataSerializer.NoSuchVersionException;
import de.bsvrz.sys.funclib.dataSerializer.SerializingFactory;

public class DefaultParameterTools
{
  
  /**
   * 
   * @param soPid Pid des Systemobjekts
   * @param atgPid Pid der ATG
   * @return den deserialisierten Defaultparameter-Datensatz fr bergebene(s) Objekt/ATG, 
   * oder <code>null</code>, wenn kein Datensatz gefunden oder keine DaV-Verbindung
   * @throws NoSuchVersionException
   * @throws IOException
   */
  public static Data getData(String soPid, String atgPid) throws NoSuchVersionException, IOException {
    Data result = null;
    
    ClientDavInterface connection = DaVKommunikation.getInstanz().getConnection();
    if (connection != null) {
      DataModel dataModel = connection.getDataModel();
      SystemObject so = dataModel.getObject(soPid);
      AttributeGroup atgParam = dataModel.getAttributeGroup(atgPid);
      AttributeGroup atgDefault = dataModel.getAttributeGroup("atg.defaultParameterdatenstze");
      
      Data satz = findeDefaultSatz(so, atgDefault, atgParam);
      
      if (satz != null) {
        
        byte[] datensatz = satz.getScaledArray("datensatz").getByteArray();
        if (datensatz != null) {
          int serialisierer = satz.getUnscaledValue("serialisierer").intValue();
          InputStream inputStream = new ByteArrayInputStream(datensatz);
          
          try
          {
            Deserializer deserialisierer = SerializingFactory.createDeserializer(serialisierer, inputStream);
            result = deserialisierer.readData(atgParam, dataModel);
          }
          finally {
            try
            {
              inputStream.close();
            }
            catch (IOException e)
            {
              //do nothing
            }
          }
        }
      }
    }
    return result;
  }
    
  private static Data findeDefaultSatz(SystemObject so, AttributeGroup atgDefault, AttributeGroup atgParam) {
    
    Data result = null;
    
    Data data = so.getConfigurationData(atgDefault);
    if (data != null) {
      result = findeSatzInDaten(so, data, atgParam);
    }
    
    if (result == null) {
      result = findeDefaultSatzInObertyp(so, atgDefault, atgParam);
    }
    
    return result;
  }
  
  private static Data findeDefaultSatzInObertyp(SystemObject so, AttributeGroup atgDefault, AttributeGroup atgParam) {
    
    Data result = null;
    
    if (so instanceof SystemObjectType) {
      SystemObjectType typ = (SystemObjectType)so;
      if (!typ.isBaseType()) {
        for (SystemObjectType oberTyp: typ.getSuperTypes()) {
          Data data = findeDefaultSatz(oberTyp, atgDefault, atgParam);
          if (data != null) {
            result = data;
            break;
          }
        }  
      }
    }
    else {
      result = findeDefaultSatz(so.getType(), atgDefault, atgParam);
    }
   
    return result;
  }
  
  private static Data findeSatzInDaten(SystemObject so, Data data, AttributeGroup atg) {
    Data result = null;
    
    Array array = data.getArray("Default-Parameterdatensatz");
        
    if (array != null) {
      SystemObjectType typ = so instanceof SystemObjectType ? (SystemObjectType)so : so.getType();
      
      for (int i = 0; i < array.getLength(); i++) {
        Data element = array.getItem(i);
        SystemObject soTyp = element.getReferenceValue("typ").getSystemObject();
        if (typ.equals(soTyp) || soTyp instanceof SystemObjectType && typ.inheritsFrom((SystemObjectType)soTyp)) {
          SystemObject paramAtg = element.getReferenceValue("attributgruppe").getSystemObject();
          if (atg.equals(paramAtg)) {
            System.out.println("Default-Parameter-Eintrag gefunden bei: " + so.getPid());
            System.out.println(element);
            result = element;
            break;
          }
        }
      }
    }
    
    return result;
  }
  
}
