/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.externetrigger;

/**
 * Enum zur Kodierung der Windrichtung
 * @author ObertM
 */
public enum Richtung
{
  /**
   * Nord
   */
  NORD                     ("Nord",                      0),
  /**
   * Nord/Ost
   */
  NORDOST                  ("Nord/Ost",                 45),
  /**
   * Ost
   */
  OST                      ("Ost",                      90),
  /**
   * Ost/Sd
   */
  OSTSUED                  ("Ost/Sd",                 135),
  /**
   * Sd
   */
  SUED                     ("Sd",                     180),
  /**
   * Sd/West
   */
  SUEDWEST                 ("Sd/West",                225),
  /**
   * West
   */
  WEST                     ("West",                    270),

  ;
  
  /**
   * Kennung
   */
  private String _kennung;
  
  /**
   * Kodierung im DaV
   */
  private int _kodierung;
  
  /**
   * Konstruktor der Klasse
   * @param kennung Kennung
   * @param kodierung Kodierung
   */
  Richtung (String kennung, int kodierung)
  {
  	_kennung = kennung;
  	
  	_kodierung = kodierung;
  }
 
  /**
   * Methode liefert die Richtung zu einer Kodierung
   * @param kodierung Kodierung
   * @return Richtung
   */
  public static Richtung getRichtung (int kodierung)
  {
  	for (Richtung s : Richtung.values())
  		if (s.getKodierung() == kodierung)
  			return s;
  	
  	return null;
  }

  /**
   * Methode liefert die Richtung zu einer Kennung
   * @param kennung Kennung
   * @return Richtung
   */
  public static Richtung getRichtung (String kennung)
  {
  	for (Richtung s : Richtung.values())
  		if (s.getKennung().equals(kennung))
  			return s;
  	
  	return null;
  }

	/**
   * Methode liefert kennung zurck
   * @return kennung
   */
  public String getKennung()
  {
  	return _kennung;
  }

	/**
   * Methode liefert kodierung zurck
   * @return kodierung
   */
  public int getKodierung()
  {
  	return _kodierung;
  }
}
