/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.externetrigger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.ursacheneinheit.Activator;
import de.bsvrz.buv.plugin.ursacheneinheit.defaultParameter.DefaultParameterHandler;
import de.bsvrz.buv.plugin.ursacheneinheit.defaultParameter.IDefaultParameterVersorgung;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.buv.plugin.ursacheneinheit.verwaltung.VersorgungsArt;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog;

/**
 * View zum Versorgen der Parameter einer Ursacheneinheit vom Typ "typ.sswSbaUeExterneTrigger"
 * @author ObertM
 */
public class ViewParameterExterneTrigger extends ViewPart implements IDefaultParameterVersorgung
{
	/**
	 * eindeutige ID der View
	 */
  public static final String ID = "de.bsvrz.buv.plugin.ursacheneinheit.parameter.typ.sswSbaUeExterneTrigger"; 

  /**
   * Schalter fr Debug Ausgaben
   */
  private static final boolean _isDebug = false;

  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  public static final String _pidAtgAlg = "atg.sswSbaUeExterneTrigger";

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();
  
  /**
   * Aufrufendes Compoiste
   */
  private Composite _parent;
  
  /**
   * Compoiste fr die Reiter
   */
  private CTabFolder _tabFolder = null;

  /**
   * Hauptcomposite
   */
  private Composite _composite = null;

  /**
   * Composite mit den Buttons
   */
  private Composite _compositeBottom;

  /**
   * Button "OK" 
   */
  private Button _buttonOk;

  /**
   * Button "bernehmen"
   */
  private Button _buttonUebernehemen;

  /**
   * Button "Abbrechen"
   */
  private Button _buttonAbbrechen;

  /**
   * Button Hilfe
   */
  private Button _buttonHilfe;

  /**
   * Button "Fr alle bernehmen"
   */
  private Button _buttonAlle;

  /**
   * Memento
   */
  private IMemento _memento = null;

  /**
   * Second View ID
   */
  private String _secondViewId;

  /**
   * Verbindung zum DaV
   */
  private ClientDavInterface _connection;

  /**
   * Ergebnisdaten des DaV fr die Algorithmus spezifischen Parameter in einer Hashmap. Als Key dient die Pid
   * des Systemobjekts, als Value das Ergebnis aus dem DaV
   */
  private HashMap<String,ResultData> _resultDataAlg = new HashMap<String, ResultData>();

  /**
   * Liste mit den mglichen Triggern
   */
  private List<SystemObject> _listeMoeglicherTrigger = null;

  /**
   * Liste mit den mglichen Aspekten
   */
  private List<SystemObject> _listeMoeglicherAspekte = null;

  /**
   * Liste mit den mglichen Objekten
   */
	private List<SystemObject> _listeMoeglicherObjekte;

	/**
	 * Liste mit den mglichen Attributgruppen
	 */
	private List<SystemObject> _listeMoeglicherAttributGruppen;

  @Override
  public void createPartControl( Composite parent )
  {
    if ( _isDebug )
      System.out.println( "--> ViewParameter: createPartControl" );

    _parent = parent;

    //Fr die F1 Taste
    
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _parent, Activator.PLUGIN_ID + ".parameterExterneTrigger" );

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    _composite = new Composite( parent, SWT.NONE );
    _composite.setLayout( gridLayout );
    _composite.setLayoutData( gridData );

    // Darstellung aufbauen
    
    createCompositeTab();
    createCompositeBottom();

    restoreState();

    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    List<SystemObject> list = new ArrayList<SystemObject>();

    if ( _secondViewId.contains( ";" ) )
    {

      String[] split = _secondViewId.split( ";" );

      for (String s : split)
      {
        SystemObject so = _connection.getDataModel().getObject( s );
        list.add( so );
      }

    }
    else
    {
      SystemObject so = _connection.getDataModel().getObject( _secondViewId );
      list.add( so );
    }

    if ( _isDebug )
      System.out.println( "Listener anmelden!" );

    if ( _isDebug )
    {
      System.out.println( "Anzahl Tabs:" + list.size() );

      for (SystemObject so : list)
      {
        System.out.println( "\tPids: " + so.getPid() );
      }
    }
    
    // Reiter erstellen

    erzeugeTabItems( list );
    
    leseDatenVonDav(list);

    _buttonAlle.setEnabled(list.size() > 1);
    
  }

  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }

  /**
   * Methode liest die bentigten Parameter direkt vom DaV
   * @param liste Liste mit Systemobjekten die versorgt werden sollen
   */
  private void leseDatenVonDav (List<SystemObject> liste)
  {
  	System.out.println("ViewParameterSeitenWind.leseDatenVonDav()");
  	
  	for (int i=0; i<liste.size(); ++i)
  	{
  		String pidObj = liste.get(i).getPid();
  		
  		ResultData result = null;
  		
  		/*
  		 * Parameter Algorithmus
  		 */
  		result = DaVKommunikation.getInstanz().leseDatenDirektVonDaV(pidObj, _pidAtgAlg, "asp.parameterSoll");
  		_resultDataAlg.put (pidObj, result);
  		
  		verarbeiteDavDaten(pidObj);
		}
  }

  /**
   * This method initializes compositeTab (Bereich ber der ButtonLeiste)
   * 
   */
  private void createCompositeTab()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder(_composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridData3);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
        // TODO Auto-generated method stub

      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();        
        CTabItem itemSel = folder.getSelection();
        
        Composite cSel = (Composite)itemSel.getControl();
        
        if (((Composite)cSel).getChildren().length > 0)
        {
          CompositeParameterGesamt compParSel = (CompositeParameterGesamt)cSel.getChildren()[0];
          CompositeParameterAlgorithmus[] composites = compParSel.getCompositeAlg();
          for (CompositeParameterAlgorithmus c : composites)
          	if (c != null && c.getTvParameter() != null)
          		c.getTvParameter().refresh();
        }
      }

    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<SystemObject> list )
  {
    for (SystemObject so : list)
    {
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

      String tabName = Tools.getName(so);
      tabItem.setText( tabName );
      tabItem.setData( tabName, so.getPid() );

      GridData gridData2 = new GridData();
      gridData2.horizontalAlignment = GridData.FILL;
      gridData2.grabExcessHorizontalSpace = true;
      gridData2.grabExcessVerticalSpace = true;
      gridData2.verticalAlignment = GridData.FILL;

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData2 );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
  }

  /**
   * Reiter fr Untergruppen des SystemObjekts erstellen
   * 
   * @param so
   *          SystemObjekt
   * @param ug
   *          Name der Untergruppe
   * @param pos
   *          Position des Reiters
   */
  public CTabItem erzeugeTabItems( SystemObject so, String ug, int pos )
  {
    CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE, pos );

    String tabName = Tools.getName(so);
    tabItem.setText( tabName );
    tabItem.setData( tabName, so.getPid() );

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    Composite c1 = new Composite( _tabFolder, SWT.NONE );
    c1.setLayout( new GridLayout() );
    c1.setLayoutData( gridData2 );

    tabItem.setControl( c1 );

    return tabItem;
  }

  /**
   * This method initializes compositeBottom (ButtonLeiste)
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    _compositeBottom = new Composite( _composite, SWT.NONE );
    _compositeBottom.setLayoutData( gridData3 );
    _compositeBottom.setLayout( rowLayout );

    new DefaultParameterHandler(_compositeBottom, this);
    
    _buttonAlle = new Button( _compositeBottom, SWT.NONE );
    _buttonAlle.setText( "Alle setzen" );
    _buttonAlle.setToolTipText( "Einstellungen bei allen ausgewhlten gleichartigen AQs setzen" );
    _buttonAlle.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionAlle();
      }
    } );

    _buttonOk = new Button( _compositeBottom, SWT.NONE );
    _buttonOk.setText( "Ok" );
    _buttonOk.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionSchalten();
      }
    } );

    _buttonUebernehemen = new Button( _compositeBottom, SWT.NONE );
    _buttonUebernehemen.setText( "bernehmen" );
    _buttonUebernehemen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionUebernehmen();
      }
    } );

    _buttonAbbrechen = new Button( _compositeBottom, SWT.NONE );
    _buttonAbbrechen.setText( "Abbrechen" );
    _buttonAbbrechen.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        dispose();
      }

    } );
    _buttonHilfe = new Button( _compositeBottom, SWT.NONE );
    _buttonHilfe.setText( "Hilfe" );
    _buttonHilfe.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionHilfe();
      }

    } );
  }

  protected void aktionHilfe()
  {
	String contextId = Activator.PLUGIN_ID + ".parameterExterneTrigger";  
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  protected void aktionSchalten()
  {
    if ( schaltenUndParametrieren() )
    {
      dispose();
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  protected void aktionUebernehmen()
  {
    if ( schaltenUndParametrieren() )
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  @Override
  public void init( IViewSite site, IMemento memento ) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init( site, memento );
  }

  private void restoreState()
  {

    if ( _memento == null )
    {
      return;
    }

    _secondViewId = _memento.getString( "pidMemento" );

  }

  @Override
  public void saveState( IMemento memento )
  {
    memento.putString( "pidMemento", _secondViewId );
    super.saveState( memento );
  }
  
  /**
   * Schalten und Parametrieren im DAV durchfhren
   * 
   * @return
   */
  private Boolean schaltenUndParametrieren()
  {
    List<ResultData> list = new ArrayList<ResultData>();
    
  	list.addAll(erzeugeResultDataAlg());

    UrlasserDialog urlasserDialog = new UrlasserDialog( Display.getCurrent().getActiveShell() );
    Boolean schlieenOk = (urlasserDialog.anzeigen( list )).getEingabeOk();

    return schlieenOk;
  }

  /**
   * Methode erzeugt die Resultdaten fr den DaV fr alle Reiter fr die Algorithmus Parameter
   * @return Resultdaten fr den DaV
   */
  private List<ResultData>  erzeugeResultDataAlg ()
  {
    List<ResultData> list = new ArrayList<ResultData>();
    
    DataModel dataModel = _connection.getDataModel();

    Aspect asp = dataModel.getAspect( "asp.parameterVorgabe" );
    AttributeGroup atg = dataModel.getAttributeGroup( _pidAtgAlg );
    
    CTabItem[] items = _tabFolder.getItems();

    for (CTabItem ti : items)
    {
    	Data data = erzeugeAttributGruppeAlg (ti);
      
      String pid = (String) ti.getData( ti.getText() );
      SystemObject so = dataModel.getObject( pid );
       
      ResultData rd1 = new ResultData( so, new DataDescription( atg, asp ), new Date().getTime(), data );
      list.add( rd1 );
    }

    return list;
  }

  /**
   * Methode erzeugt ein Datenfeld mit den Daten eines bergebenen Reiters
   * @param itemSel auszuwertender Reiter
   * @return Datenfeld
   */
  private Data erzeugeAttributGruppeAlg (CTabItem itemSel)
  {
    DataModel dataModel = _connection.getDataModel();

    AttributeGroup atg = dataModel.getAttributeGroup( _pidAtgAlg );
    Data data = _connection.createData( atg );
    
    Composite c = (Composite) itemSel.getControl();
    CompositeParameterGesamt compPar = (CompositeParameterGesamt) c.getChildren()[0];

    CompositeParameterAlgorithmus[] ca = compPar.getCompositeAlg();
    erzeugeAttributGruppeAllgemein (ca, data);

    CompositeParameterAlgorithmusAlternative[] caAlt = compPar.getCompositeAlgAlt();
    erzeugeAttributGruppeAlternative (caAlt, data.getItem("AlternativeProgrammSchaltung"));

    return data;
  }
    
	private void erzeugeAttributGruppeAllgemein (CompositeParameterAlgorithmus[] ca, Data data)
  {
    DataModel dataModel = _connection.getDataModel();

    for (int i=0; i<ca.length; ++i)
    {
      Data data1 = data;

      CompositeParameterAlgorithmus cAlg = ca [i];
      
      if (cAlg != null)
      {
      	/*
      	 * Spinner
      	 */
	      for (int j=0; j<cAlg.getAnzahlSpinnerItems(); ++j)
	      {
	        String item = cAlg.getItemSpinnerName(j);
	        float f = getWert(cAlg.getSpinner(j));
	        data1.getScaledValue(item).set(f);
	      }

	      /*
	       * ComboBoxen
	       */
	      for (int j=0; j<cAlg.getAnzahlComboBoxItems(); ++j)
	      {
	        String item = cAlg.getItemComboBoxName(j);
	        
	        Combo cb = cAlg.getComboBox(j);
	        String selected = cb.getItem( cb.getSelectionIndex() );

	        /*
	         * Trigger
	         */
	        if (j==0)
	        {
	        	SystemObject soTrigger = null;
	          if (!selected.equals(Tools._undefiniert))
	          {
	          	for (SystemObject so : _listeMoeglicherTrigger)
	          		if (Tools.getName(so).equals(selected))
	          			soTrigger = so;
	          }
	          data1.getReferenceValue(item).setSystemObject(soTrigger);
	        }

	        /*
	         * Aspekt
	         */
	        if (j==1)
	        {
	        	SystemObject soAspekt = null;
	          if (!selected.equals(Tools._undefiniert))
	          {
	          	for (SystemObject so : _listeMoeglicherAspekte)
	          		if (Tools.getName(so).equals(selected))
	          			soAspekt = so;
	          }
	          data1.getReferenceValue(item).setSystemObject(soAspekt);
	        }

	        /*
	         * sofortigeNeuberechnung bzw. ManahmeLschen
	         */
	        if (j == 2 || j == 3)
	        {
	          data1.getTextValue(item).setText(selected);	
	        }
	      }
	      
	      System.out
            .println("ViewParameterSeitenWind.erzeugeAttributGruppeAlg() " + data.toString());
      }
    }
  }

  private void erzeugeAttributGruppeAlternative(CompositeParameterAlgorithmusAlternative[] ca, Data data)
  {
    for (int i=0; i<ca.length; ++i)
    {
      Data data1 = data;

      CompositeParameterAlgorithmusAlternative cAlg = ca [i];
      
      if (cAlg != null)
      {
      	/*
      	 * Spinner
      	 */
	      for (int j=0; j<cAlg.getAnzahlSpinnerItems(); ++j)
	      {
	        String item = cAlg.getItemSpinnerName(j);
	        Integer i1 = cAlg.getSpinner(j).getSelection();
	        data1.getUnscaledValue(item).set(i1);
	      }

      	/*
      	 * Texteingaben
      	 */
	      for (int j=0; j<cAlg.getAnzahlTextItems(); ++j)
	      {
	        String item = cAlg.getItemText(j);
	        data1.getTextValue(item).setText(cAlg.getText(j).getText());
	      }

	      /*
	       * ComboBoxen
	       */
	      for (int j=0; j<cAlg.getAnzahlComboBoxItems(); ++j)
	      {
	        String item = cAlg.getItemComboBoxName(j);
	        
	        Combo cb = cAlg.getComboBox(j);
	        String selected = cb.getItem( cb.getSelectionIndex() );

	        /*
	         * Objekte
	         */
	        if (j==0)
	        {
	        	SystemObject soWS = null;
	          if (!selected.equals(Tools._undefiniert))
	          {
	          	for (SystemObject so : _listeMoeglicherObjekte)
	          		if (Tools.getName(so).equals(selected))
	          			soWS = so;
	          }
	          data1.getReferenceValue(item).setSystemObject(soWS);
	        }

	        /*
	         * Attributruppen
	         */
	        if (j==1)
	        {
	        	SystemObject soWR = null;
	          if (!selected.equals(Tools._undefiniert))
	          {
	          	for (SystemObject so : _listeMoeglicherAttributGruppen)
	          		if (Tools.getName(so).equals(selected))
	          			soWR = so;
	          }
	          data1.getReferenceValue(item).setSystemObject(soWR);
	        }

	        /*
	         * Aspekte
	         */
	        if (j==2)
	        {
	        	SystemObject soWR = null;
	          if (!selected.equals(Tools._undefiniert))
	          {
	          	for (SystemObject so : _listeMoeglicherAspekte)
	          		if (Tools.getName(so).equals(selected))
	          			soWR = so;
	          }
	          data1.getReferenceValue(item).setSystemObject(soWR);
	        }
	      }
	      
	      System.out
            .println("ViewParameterSeitenWind.erzeugeAttributGruppeAlg() " + data.toString());
      }
    }
  }

  /**
   * Methode liefert den Wert einers Spinners
   * @param spinner Spinner
   * @return eingestellter Wert
   */
  public float getWert (Spinner spinner)
  {
    Integer i1 = spinner.getSelection();
    int digits = spinner.getDigits();
    
  	double f1 = (i1 / Math.pow(10, digits));
  	
  	return (float) f1;
  }

  /**
   * Methode zum Verarbeiten der Daten des DaV
   * @param pidObj Pid des Systemobjekts
   */
  private void verarbeiteDavDaten (String pidObj)
  {
    if ( _isDebug )
      System.out.println( "--> ViewParameter: VerarbeiteDavDaten" );
    
    /*
     *  einmalig alle mglichen Triggern bestimmen
     */
    
    if ( _listeMoeglicherTrigger == null )
    	_listeMoeglicherTrigger = Tools.getSystemObjekte("typ.sswSbaExterneTrigger"); 
    		
    /*
     *  einmalig alle mglichen Aspekte bestimmen
     */
    
    if ( _listeMoeglicherAspekte == null )
    	_listeMoeglicherAspekte = Tools.getSystemObjekte("typ.aspekt");
    
    /*
     *  einmalig alle mglichen Objekte bestimmen
     */
    
    if ( _listeMoeglicherObjekte == null )
      _listeMoeglicherObjekte = Tools.getSystemObjekte("typ.konfigurationsObjekt");
    
    System.out.println("ViewParameterExterneTrigger.verarbeiteDavDaten() " + _listeMoeglicherObjekte.size());

    /*
     *  einmalig alle mglichen Attributgruppen bestimmen
     */
    
    if ( _listeMoeglicherAttributGruppen == null )
    	_listeMoeglicherAttributGruppen = _connection.getDataModel().getType( "typ.attributgruppe" ).getElements();

    /*
     *  alle Reiter zur Pid ermitteln
     */
    CTabItem[] items = _tabFolder.getItems();
    List<CTabItem> itemPidList = new ArrayList<CTabItem>();
    for (CTabItem ti : items)
    {
      String pid = (String) ti.getData( ti.getText() );

      if ( pid == pidObj )
      {
        itemPidList.add( ti );
      }
    }

    if ( itemPidList != null )
    {
      CTabItem item = null;
  
      for (CTabItem ti : itemPidList)
      {
        item = ti;

        Composite c = (Composite) item.getControl();
        Control[] children = c.getChildren();
        CompositeParameterGesamt compPar = null;

        if ( children.length == 0 )
        {
          compPar = new CompositeParameterGesamt( c, SWT.None, _resultDataAlg.get(pidObj), 
          		Tools.bestimmeObjektNamen (_listeMoeglicherTrigger, true), 
          		Tools.bestimmeObjektNamen (_listeMoeglicherAspekte, true),
          		Tools.bestimmeObjektNamen (_listeMoeglicherObjekte, true),
          		Tools.bestimmeObjektNamen (_listeMoeglicherAttributGruppen, true));
          
          compPar.layout( true );
          c.layout( true );

        }
        else
        {
          compPar = (CompositeParameterGesamt) children[0];
          compPar.update( _resultDataAlg.get(pidObj) );
          
          compPar.layout( true );
          c.layout( true );
        }
      }
    }
  }

  @Override
  public void dispose()
  {
    super.dispose();
//    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener( this );
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if ( page != null )
    {
      page.hideView( this );
    }
  }

  /**
   * Methode bernimmt die eingestellten Parameter fr alle Tabs. Hierzu werden die Tabellenviewer des
   * angewhlten Tab an die anderen Tabs bergeben.
   */
  protected void aktionAlle()
  {
    List<ResultData> list = new ArrayList<ResultData>();
    
    DataModel dataModel = _connection.getDataModel();

    Aspect asp = dataModel.getAspect( "asp.parameterVorgabe" );
    AttributeGroup atg = dataModel.getAttributeGroup( _pidAtgAlg );
    
    /*
     * aktueller Reiter einlesen
     */
    CTabItem itemSel = _tabFolder.getSelection();
    Data data = erzeugeAttributGruppeAlg(itemSel);

    /*
     * Fr alle Reiter die Attributgruppe erzeugen
     */
    CTabItem[] items = _tabFolder.getItems();

    for (CTabItem ti : items)
    {
      String pid = (String) ti.getData( ti.getText() );
      SystemObject so = dataModel.getObject( pid );

      ResultData rd1 = new ResultData( so, new DataDescription( atg, asp ), new Date().getTime(), data );
      list.add(rd1);
    }
     
    /*
     * Daten umkopieren
     */
    
    for (ResultData rData : list)
    {
    	String pid = rData.getObject().getPid();
    	_resultDataAlg.put(pid, rData);
    	
    	verarbeiteDavDaten(pid);
    }
  }
  
  @Override
  public String getParameterAtgPid()
  {
    return _pidAtgAlg;
  }

  @Override
  public String getRootObjektPid()
  {
    String result = null;
    
    switch (getVersorgungsArt()) {
      case EinReiterProUrsachenEinheit:
        CTabItem item = _tabFolder.getSelection();
        if (item != null) {
          result = (String)item.getData(item.getText());
        }  
        break;
//      case EinReiterFuerAlleUrsachenEinheiten:
//        for (SystemObject so : _ursachenEinheiten)
//        {
//          if (result == null) {
//            result = so.getType().getPid();
//          }
//          else {
//            if (!result.equals(so.getType().getPid())) {
//              result = null;
//              break;
//            }
//          }
//        }
//        break;
    }
    
    return result;
  }

  @Override
  public VersorgungsArt getVersorgungsArt()
  {
    return VersorgungsArt.EinReiterProUrsachenEinheit;
  }
  
  @Override
  public void setData(Data data) {
    CTabItem item = _tabFolder.getSelection();
    if (item != null) {
      Composite c = (Composite) item.getControl();
      Control[] children = c.getChildren();
      
      if ( children.length > 0 )
      {
        CompositeParameterGesamt compPar = (CompositeParameterGesamt) children[0];
        
        String pid = (String)item.getData(item.getText());
      
        ResultData result = _resultDataAlg.get(pid);
        result = new ResultData(result.getObject(), result.getDataDescription(),
            result.getDataTime(), data);
        _resultDataAlg.put(pid, result);
        
        compPar.update( result );
      }
    }  
  }
  
}
