/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.fremdanlage;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.bsvrz.buv.plugin.ursacheneinheit.lkwueberholverbotnaesse.AuswahlObjekte;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Darstellung des Fensters zur Parameternderung mit den Messquerschnitten und den Algorithmus spezifischen
 * Parametern
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Algorithmus spezifische Parameter
   */
  private ResultData _resultDataAlg;

  /**
   * Liste mit den mglichen Anzeigequerschnitten
   */
  private List<String> _listeMoeglicherAnzeigeQuerschnitte;

  /**
   * Liste mit den mglichen Anzeigen
   */
  private List<String> _listeMoeglicherAnzeigen;

  /**
   * Hauptcomposite
   */
  private Composite _composite;

	/**
	 * Instanz zur Auswahl der Anzeigequerschnitte
	 */
	private AuswahlObjekte _auswahlObjekteAnzeigeQuerschnitte;

	/**
	 * Instanz zur Auswahl der Anzeigen
	 */
	private AuswahlObjekte _auswahlObjekteAnzeigen;

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataAlg Ergebnisdaten der Algorithmus spezifischen Parameter
	 * @param listMoeglicheAnzeigeQuerschnitte Liste mit den Namen der mglichen Anzeigequerschnitte
	 * @param listMoeglicheAnzeigen Liste mit den Namen der mglichen Anzeigen
	 */
  CompositeParameterGesamt(Composite parent, int style, ResultData resultDataAlg, List<String> listMoeglicheAnzeigeQuerschnitte, List<String> listMoeglicheAnzeigen)
  {
    super(parent, style);
    
    _resultDataAlg = resultDataAlg;
    
    _listeMoeglicherAnzeigeQuerschnitte = listMoeglicheAnzeigeQuerschnitte;
    
    _listeMoeglicherAnzeigen = listMoeglicheAnzeigen;
    
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

  	_auswahlObjekteAnzeigeQuerschnitte = new AuswahlObjekte(_composite, "Anzeigequerschnitte");
  	_auswahlObjekteAnzeigen            = new AuswahlObjekte(_composite, "Anzeigen");

    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataAlg Ergebnis Algorithmus
   */
  public synchronized void update (ResultData resultDataAlg)
  {
    this._resultDataAlg = resultDataAlg;

    updateData ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultDataAlg != null)
    {
    	updateDataAnzeigeQuerschnitte();
    	
    	updateDataAnzeigen();
    }
  }
  
  /**
   * Updatefunktion ber die Anzeigequerschnitte
   */
	private void updateDataAnzeigeQuerschnitte()
  {
		List<String> angewaehlteObjekte = new ArrayList<String>();
		
    if ( _resultDataAlg != null)
    {
      if ( _resultDataAlg.hasData() )
      {
      	Data data = _resultDataAlg.getData();
      	
        ReferenceArray arrayAQ = data.getReferenceArray ("AnzeigeQuerschnitt" );
        for (int k = 0; k < arrayAQ.getLength(); k++)
        {
          String text = "";
          SystemObject so = arrayAQ.getReferenceValue(k).getSystemObject();
          if ( so != null )
            text = Tools.getName(so);
          
          angewaehlteObjekte.add(text);
        }
      }
    }
    
    _auswahlObjekteAnzeigeQuerschnitte.setAngewaehlteObjekte(_listeMoeglicherAnzeigeQuerschnitte, angewaehlteObjekte);
  }

  /**
   * Updatefunktion ber die Anzeigen
   */
	private void updateDataAnzeigen()
  {
		List<String> angewaehlteObjekte = new ArrayList<String>();
		
    if ( _resultDataAlg != null)
    {
      if ( _resultDataAlg.hasData() )
      {
      	Data data = _resultDataAlg.getData();
      	
        ReferenceArray arrayA = data.getReferenceArray ("Anzeige" );
        for (int k = 0; k < arrayA.getLength(); k++)
        {
          String text = "";
          SystemObject so = arrayA.getReferenceValue(k).getSystemObject();
          if ( so != null )
            text = Tools.getName(so);
          
          angewaehlteObjekte.add(text);
        }
      }
    }
    
    _auswahlObjekteAnzeigen.setAngewaehlteObjekte(_listeMoeglicherAnzeigen, angewaehlteObjekte);
  }

  /**
   * Methode liefert die Liste der angewhlten Anzeigequerschnitte
   * @return Liste der angewhlten Anzeigequerschnitte
   */
  public org.eclipse.swt.widgets.List getZugeordneteAnzeigeQuerschnitte()
  {
    return _auswahlObjekteAnzeigeQuerschnitte.getZugeordneteObjekte();
  }

  /**
   * Methode liefert die Liste der angewhlten Anzeigen
   * @return Liste der angewhlten Anzeigequerschnitte
   */
  public org.eclipse.swt.widgets.List getZugeordneteAnzeigen()
  {
    return _auswahlObjekteAnzeigen.getZugeordneteObjekte();
  }
}
