/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.laerm;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import de.bsvrz.dav.daf.main.ResultData;

/**
 * Darstellung des Fensters zur Parameternderung mit den Messquerschnitten und den Algorithmus spezifischen
 * Parametern
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Algorithmus spezifische Parameter
   */
  private ResultData[] _resultDataAlg;

  /**
   * Liste mit den mglichen Umfelddatensensoren
   */
  private List<String> _listeMoeglicherUmfeldDatenSensoren;

  /**
   * Liste mit den mglichen Messquerschnitte
   */
  private List<String> _listeMoeglicherMessQuerschnitte;

  /**
   * Liste mit den mglichen Ereignistypen
   */
  private List<String> _listeMoeglicherEreignisTypen;

  /**
   * Hauptcomposite
   */
  private Composite _composite;

  /**
   * Composite mit den Reitern
   */
  private CTabFolder _tabFolder = null;

  /**
   * Liste mit den unterschiedlichen Parametergruppen
   */
	private ArrayList<String> _list;

	private Group _groupParameter;

	private CompositeParameterLaerm _compositeParameterLaerm;

	private CompositeParameterLaermBerechnung _compositeParameterLaermBerechnung;

	/**
	 * Konstruktor der Klasse
	 * @param parent aufrufendes Composite
	 * @param style Stil
	 * @param resultDataAlg Ergebnisdaten
	 * @param listMoeglicheUmfeldDatenSensoren Liste mit den Namen mglicher Sensoren
	 * @param listMoeglicheMessQuerschnitte Liste mit den Namen mglicher Messquerschnitte
	 * @param listMoeglicheEreignisTypen Liste mit den Namen mglicher Ereignistypen
	 */
  CompositeParameterGesamt(Composite parent, int style, ResultData[] resultDataAlg, List<String> listMoeglicheUmfeldDatenSensoren, List<String> listMoeglicheMessQuerschnitte, List<String> listMoeglicheEreignisTypen)
  {
    super(parent, style);
    
    _resultDataAlg = resultDataAlg;
    
    _listeMoeglicherUmfeldDatenSensoren = listMoeglicheUmfeldDatenSensoren;
    
    _listeMoeglicherMessQuerschnitte = listMoeglicheMessQuerschnitte;

    _listeMoeglicherEreignisTypen = listMoeglicheEreignisTypen;

    initialize();
  }
  
	/**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    createCompositeTab();
    
    /*
     * Reiter anlegen fr die unterschiedlichen Parameter
     */
    _list = new ArrayList<String>();
    
    _list.add ("Lrm");
    _list.add ("Lrmberechnung");
   
    erzeugeTabItems (_list);

    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   */
  public synchronized void update (ResultData[] resultDataAlg)
  {
    this._resultDataAlg = resultDataAlg;

    updateData ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
  	System.out.println("CompositeParameterGesamt.updateData()");
  	
    if (_resultDataAlg [0] != null)
    {
    	updateLaerm (_resultDataAlg [0]);
    }

    if (_resultDataAlg [1] != null)
    {
    	updateLaermBerechnung (_resultDataAlg [1]);
    }
  }

  /**
   * Methode zum Aktualisieren der Parameter "Lrm"
   * @param resultDataAlgorithmus Ergebnisdaten aus dem DaV
   * @param resultDataAlg Ergebnisdaten aus dem DaV
   */
  private void updateLaerm (ResultData resultDataAlgorithmus)
  {
    /*
     * Reiter ermmitten
     */

		CTabItem[] items = _tabFolder.getItems();
    CTabItem item = items [0];
      
     if ( item != null )
     {
        Composite c = (Composite) item.getControl();

        if (_compositeParameterLaerm == null)
        {
        	_compositeParameterLaerm = new CompositeParameterLaerm ( c, SWT.None, resultDataAlgorithmus, resultDataAlgorithmus.getData(), _listeMoeglicherUmfeldDatenSensoren, _listeMoeglicherMessQuerschnitte, _listeMoeglicherEreignisTypen);
        	_compositeParameterLaerm.layout( true );
          
          c.layout( true );
        }
        else
        {
        	_compositeParameterLaerm.update ( resultDataAlgorithmus, resultDataAlgorithmus.getData());
        	_compositeParameterLaerm.layout( true );
        	
          c.layout( true );
        }
     }
  }

  /**
   * Methode zum Aktualisieren der Parameter "Lrm Berechnung"
   * @param resultDataAlgorithmus Ergebnisdaten aus dem DaV
   * @param resultDataBewertung Ergebnisdaten aus dem DaV
   */
  private void updateLaermBerechnung (ResultData resultDataAlgorithmus)
  {
    /*
     * Reiter ermmitten
     */

		CTabItem[] items = _tabFolder.getItems();
    CTabItem item = items [1];
      
     if ( item != null )
     {
        Composite c = (Composite) item.getControl();

        if (_compositeParameterLaermBerechnung == null)
        {
        	_compositeParameterLaermBerechnung = new CompositeParameterLaermBerechnung( c, SWT.None, resultDataAlgorithmus, resultDataAlgorithmus.getData());
        	_compositeParameterLaermBerechnung.layout( true );
          
          c.layout( true );
        }
        else
        {
        	_compositeParameterLaermBerechnung.update ( resultDataAlgorithmus, resultDataAlgorithmus.getData());
        	_compositeParameterLaermBerechnung.layout( true );
        	
          c.layout( true );
        }
     }
  }

  /**
   * Methode zum Anlegen des Composite fr die Tabelle
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;

    _groupParameter = new Group(_composite, SWT.NONE);
    _groupParameter.setText("Parameter");
    _groupParameter.setLayout(new GridLayout(1, false));
    _groupParameter.setLayoutData(gridDataGroup);
  
    _tabFolder = new CTabFolder(_groupParameter, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);
    _tabFolder.addSelectionListener(new SelectionListener() {

			@Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();        
        CTabItem itemSel = folder.getSelection();
        
        Composite cSel = (Composite)itemSel.getControl();
        
        /*
         * Damit die Tabellen in den einzelnen Parameterdialogen richtig dargestellt werden 
         * (in der richtigen Hhe der Zeilen) muss beim Anwhlen eines Reiters mit so einer Tabelle
         * die Tabelle aktualisiert werden.
         */
        if (((Composite)cSel).getChildren().length > 0)
        {
          if (itemSel.getText().equals(_list.get(0)))
          	if (_compositeParameterLaerm != null)
          	  _compositeParameterLaerm.refreshTabelle();
          
          if (itemSel.getText().equals(_list.get(1)))
          	if (_compositeParameterLaermBerechnung != null)
          	  _compositeParameterLaermBerechnung.refreshTabelle();
        }
      }
    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<String> list )
  {
  	for (int i=0; i<list.size(); ++i)
  	{
      String s = list.get(i);
     
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

      String tabName = s;
      tabItem.setText( tabName );
      tabItem.setData( tabName, s );

      GridData gridData2 = new GridData();
      gridData2.horizontalAlignment = GridData.FILL;
      gridData2.grabExcessHorizontalSpace = true;
      gridData2.grabExcessVerticalSpace = true;
      gridData2.verticalAlignment = GridData.FILL;

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData2 );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
    
  }

	/**
   * Methode liefert das Composite zum Versorgen der Parameter "Stau Belegung" 
   * @return Composite
   */
	public CompositeParameterLaerm getCompositeParameterLaerm ()
  {
	  return _compositeParameterLaerm;
  }

	/**
   * Methode liefert compositeParameterStauGeschwindigkeit zurck
   * @return compositeParameterStauGeschwindigkeit
   */
  public CompositeParameterLaermBerechnung getCompositeParameterLaermBerechnung()
  {
  	return _compositeParameterLaermBerechnung;
  }
}
