/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.stau;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

import de.bsvrz.dav.daf.main.Data;

/**
 * Klasse erzeugt eine Gruppe mit Eingabeelementen die es ermglichen die Werte einzugeben die fr
 * die Bewertung der Verkehrsstufen bentigt werden
 * @author ObertM
 *
 */
public class CompositeBewertungVerkehrsStufe
{
	/**
	 * Eingabefelder "Nicht-Stau"
	 */
	private EingabeElemente _nichtStau;

	/**
	 * Eingabefelder "Stau"
	 */
	private EingabeElemente _stau;

	/**
	 * Methode erzeugt eine Gruppe mit den Eingabeelementen
	 * @param parent Parent Composite
	 */
  public void createGroupBewertung (Composite parent)
  {
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.BEGINNING;

    Group groupBewertung = new Group(parent, SWT.NONE);
    groupBewertung.setText("Bewertung");
    groupBewertung.setLayout(new GridLayout(3, false));
    groupBewertung.setLayoutData(gridData2);

  	createGroupStau     (groupBewertung);
  	createGroupNichtStau(groupBewertung);
  	
  }

  /**
   * Methode erzeugt die Gruppe "Nicht Stau" mit den dazugehrenden Eingabeelementen
   * @param parent Parent Composite
   */
  private void createGroupNichtStau (Composite parent)
  {
  	_nichtStau = new EingabeElemente(parent, "Nicht Stau");
  }

  /**
   * Methode erzeugt die Gruppe "Stau" mit den dazugehrenden Eingabeelementen
   * @param parent Parent Composite
   */
  private void createGroupStau (Composite parent)
  {
  	_stau = new EingabeElemente(parent, "Stau");
  }
  
  /**
   * Update Methode 
   * @param data Daten vom DaV
   */
	public void update(Data data)
  {
	  _nichtStau.update(data.getItem("Nicht-Stau"));
	  _stau.update(data.getItem("Stau"));
  }
	
	/**
	 * Methode zum Erzeugen einer Attributgruppe mit den eingestellten Werten
	 * @param data Datenfeld im DaV in das die Werte eingetragen werden
	 */
	public void erzeugeAttributGruppe (Data data)
	{
		_nichtStau.erzeugeAttributGruppe (data.getItem("Nicht-Stau"));
		_stau.erzeugeAttributGruppe (data.getItem("Stau"));
	}
	
	/**
	 * Klasse zum Erzeugen einer Gruppe von Eingabeelementen fr die Bewertung der Verkehrsstufen
	 * @author ObertM
	 */
	private class EingabeElemente 
	{
		/**
		 * Spinner "freier Verkehr"
		 */
		private Spinner _spinnerFV;
		
		/**
		 * Spinner "dichter Verkehr"
		 */
		private Spinner _spinnerDV;
		
		/**
		 * Spinner "zhfliessender Verkehr"
		 */
		private Spinner _spinnerZV;
		
		/**
		 * Spinner "Stau"
		 */
		private Spinner _spinnerStau;

		/**
		 * Konstruktor der Klasse
		 * @param parent Parent Composite
		 * @param name Bezeichnung der Gruppe mit den Eingabeelementen
		 */
		public EingabeElemente (Composite parent, String name)
		{
	    GridData gridData2 = new GridData();
	    gridData2.verticalAlignment = GridData.BEGINNING;
	
	  	Group group = new Group(parent, SWT.NONE);
	  	group.setText(name);
	  	group.setLayout(new GridLayout(2, false));
	  	group.setLayoutData(gridData2);
	    
	  	_spinnerFV   = createSpinner(group, "Freier Verkehr");
	  	_spinnerDV   = createSpinner(group, "Dichter Verkehr");
	  	_spinnerZV   = createSpinner(group, "Zhfliessender Verkehr");
	  	_spinnerStau = createSpinner(group, "Stau");
		}
		
		/**
		 * Methode legt ein Label fr eine Beschriftung und einen Spinner in einem Composite an
		 * @param parent Parent Composite
		 * @param name Beschriftung
		 * @return Spinner
		 */
		private Spinner createSpinner (Composite parent, String name)
		{
	    Label l = new Label(parent, SWT.NONE);
	    l.setText(name);
	    
	  	Spinner spinner = new Spinner(parent, SWT.BORDER);
	  	spinner.setSelection(0);
	  	spinner.setDigits(2);
	  	spinner.setMinimum(0);
	  	spinner.setMaximum(100);
	  	
	  	return spinner;
		}
		
		/**
		 * Update Methode zum Aktualisieren der Eingabeelemente
		 * @param data Daten von DaV
		 */
		public void update (Data data)
		{
      updateSpinner(_spinnerFV,   data.getScaledValue("freierVerkehr").floatValue());		  	
      updateSpinner(_spinnerDV,   data.getScaledValue("dichterVerkehr").floatValue());		  	
      updateSpinner(_spinnerZV,   data.getScaledValue("zhflieend").floatValue());		  	
      updateSpinner(_spinnerStau, data.getScaledValue("stau").floatValue());		  	
		}

		/**
		 * Methode zum Erzeugen einer Attributgruppe mit den eingestellten Werten
		 * @param data Datenfeld im DaV in das die Werte eingetragen werden
		 */
		public void erzeugeAttributGruppe(Data data)
    {
	    data.getScaledValue("freierVerkehr").set(getWert(_spinnerFV));
	    data.getScaledValue("dichterVerkehr").set(getWert(_spinnerDV));
	    data.getScaledValue("zhflieend").set(getWert(_spinnerZV));
	    data.getScaledValue("stau").set(getWert(_spinnerStau));
    }

		/**
		 * Methode zum Setzen eines Spinners auf einen bestimmten Wert
		 * @param spinner Spinner
		 * @param wert zu setzender Wert
		 */
		private void updateSpinner (Spinner spinner, float wert)
		{
	    int digits = spinner.getDigits();
		  int iWert = (int) (wert * (int) Math.pow(10, digits));
		  spinner.setSelection(iWert);
		}
		
	  /**
	   * Methode liefert den Wert einers Spinners
	   * @param spinner Spinner
	   * @return eingestellter Wert
	   */
	  public float getWert (Spinner spinner)
	  {
	    Integer i1 = spinner.getSelection();
	    int digits = spinner.getDigits();
	    
	  	double f1 = (i1 / Math.pow(10, digits));
	  	
	  	return (float) f1;
	  }
	}
}
