/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.stau;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.ursacheneinheit.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Dient der Kommunikation mit dem DaV bzgl. der Harmonisierung.
 * 
 * @author Krause
 * 
 */
public class DaVKommunikation implements ClientReceiverInterface
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private ClientDavInterface _connection = null;

//  private Vector _vector = new Vector();

  /**
   * Singleton-Instanz
   */
  private static DaVKommunikation _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static DaVKommunikation getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DaVKommunikation();
    }
    return _instanz;
  }

  /**
   * Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private DaVKommunikation()
  {

    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt werden.
     */
    IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer() {
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _connection = null;
      }

      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        // Anmelden, wenn die Verbindung aufgebaut wurde
        _connection = e.getDaVVerbindung();
      }

    };

    if (_connection != null)
    {
      // Diagram diagram = DiagramFactory.getInstanz().getDiagram();
      // Diagram diagram2 = DiagramFactory.getInstanz().getDiagram2();
      //      
      // List figures = diagram.getFigures();
      // figures.addAll(diagram2.getFigures());
      //
      // _vector.addAll(figures);
    }
    else
      _debug.error("Plugin Automatikprogramme/Harmonisierung : Connection ist null!");

  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeDaVVerbindung(ClientDavInterface connnection)
  {
    this._connection = connnection;
  }

  /**
   * Holt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public ClientDavInterface holeDaVVerbindung()
  {
    return _connection;
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet 
   * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV( Long objId, String atgPid, String aspPid )
  {
    if ( objId == null || atgPid == null || aspPid == null )
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject( objId );
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup( atgPid );
    Aspect asp = _connection.getDataModel().getAspect( aspPid );
    DataDescription dd = new DataDescription( atg, asp );

    _connection.unsubscribeReceiver( this, obj, dd );
  }

  /**
   * Methode die die Liste der Objekte objs beim Datenverteiler abmeldet 
   * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
   * 
   * @param objs
   *          Lister der Ids der Objekte
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV( List objs, String atgPid, String aspPid )
  {
    if (objs == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, objs, dd);
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet 
   * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(String objPid, String atgPid, String aspPid, ReceiverRole role)
  {
    if (objPid == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objPid);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode die die Liste der Objekte objs beim Datenverteiler anmeldet 
   * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
   * 
   * @param objs
   *          Liste der Pids der Objekte
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(List objs, String atgPid, String aspPid, ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, objs, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode die die Liste der Objekte objs beim Datenverteiler anmeldet 
   * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid.
   * 
   * @param objs
   *          Liste der Pids der Objekte
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeStringsLeseDaV(List<String> objs, String atgPid, String aspPid, ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    List<SystemObject> sos = new ArrayList<SystemObject>();

    for (String s : objs)
    {
      SystemObject so = _connection.getDataModel().getObject(s);

      if (so != null)
        sos.add(so);
      else
      {
        // System.out.println(s + " : nicht in der Konfiguration vorhanden");
      }
    }

    _connection.subscribeReceiver(this, sos, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Verarbeitet smtliche Updates
   */
  public void update(ResultData[] results)
  {
    
    for (int i = 0; i < results.length; i++)
    {
      final ResultData result = results[i];
      
      //fr DAV-Testausgaben
      if ( false )
      {
        System.out.println( result );
        
        if ( result.hasData() )
        {
          Data data = result.getData();
          System.out.println( "=>" + data );
                 
          Array aMQ = data.getArray( "MQ" );
          System.out.println( "\t->Anz. Ele:" + aMQ.getLength() );
          for (int j = 0; j < aMQ.getLength(); j++)
          {
            Data item = aMQ.getItem( j );
            System.out.println( "\t-->>" + item );

            Array aMQs = item.getArray( "MessQuerschnitt" );
            System.out.println( "\t\t->Anz. Ele:" + aMQs.getLength() );
            for (int k = 0; k < aMQs.getLength(); k++)
            {
              Data itemMQ = aMQs.getItem( k );
              System.out.println( "\t\t-->>" + itemMQ );

            }

            Data dataPar = item.getItem( "Parameter" );
            System.out.println( "\t\t\t->" + dataPar );

            Data dataQBEin = dataPar.getItem( "QBEin" );
            System.out.println( "\t\t\t->>" + dataQBEin );
          }

        }
      }
      
      Display.getDefault().asyncExec(new Runnable() {
        public void run()
        {
          if (result != null)
            DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
          else
            System.err.println("DAVKommunikation:update: Ein Result ist leer!");

        }
      });

    }
  }

  /**
   * Bestimmt den Default-Bereich des DaV
   * 
   * @param connection
   * @return
   */
  public ConfigurationArea bestimmeDefaultBereich(ClientDavInterface connection)
  {
    Data data = connection.getDataModel().getConfigurationAuthority().getConfigurationData(
        connection.getDataModel().getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften"));

    if (data != null)
    {
      TextArray array = data.getTextArray("defaultBereich");

      if (array.getLength() == 1)
      {
        return (ConfigurationArea)connection.getDataModel().getObject(array.getText(0));
      }
    }

    return null;
  }

  /**
   * Prft, ob das Objekt mit der PID objPid vorhanden ist
   * 
   * @param objPid
   *          Pid des Objekts
   * @return
   */
  public boolean isObjektVorhanden(String objPid)
  {

    SystemObject object = _connection.getDataModel().getObject(objPid);

    if (object != null)
      return true;

    return false;
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet 
   * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid
   * und der Rolle des normalen Empfngers.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   */
  public void anmeldeDaten(String objPid, String atgPid, String aspPid)
  {
    anmeldeLeseDaV(objPid, atgPid, aspPid, ReceiverRole.receiver());

  }

  /**
   * Methode die die Liste der Objekte objs beim Datenverteiler anmeldet 
   * zum Lesen der Attibutgruppe atgPid unter dem Aspekt aspPid
   * und der Rolle des normalen Empfngers.
   * 
   * @param objs
   *          Liste der Pids der Objekte
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   */
  public void anmeldeDaten(List objs, String atgPid, String aspPid)
  {
    anmeldeLeseDaV(objs, atgPid, aspPid, ReceiverRole.receiver());
    
  }

  /**
   * Methode liest fr das Objekt mit der PID objPid die Attibutgruppe atgPid 
   * unter dem Aspekt aspPid direkt vom DaV und ruft sofort die Auswertemethode auf
   * (nicht ber den Thread) 
   * @param objPid Pid des Objekts 
   * @param atgPid Attributgruppe die angemeldet werden soll
   * @param aspPid Apekt der angemeldet werden soll
   */
  public ResultData leseDatenDirektVonDaV (String objPid, String atgPid, String aspPid)
  {
    ClientDavInterface verb = _connection;

    AttributeGroup atg = verb.getDataModel().getAttributeGroup(atgPid);

    Aspect asp = verb.getDataModel().getAspect(aspPid);

    DataDescription dd = new DataDescription(atg, asp);

    SystemObject obj = verb.getDataModel().getObject(objPid); 

    try
    {
      ResultData data = verb.getData( obj, dd, 10000 );
      
      ResultData[] results = new ResultData [] {data};
      
      if (results.length == 1)
      	return results [0];
    }
    catch (Exception e) 
    {
      e.printStackTrace();
    }
    
    return null;
  }

}
