/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.configurationAreas.compsites;

import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.ConfigurationAreaModelRecord;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.DynamicObjectTypesDetermination;

/**
 * Mit Hilfe dieses {@link Composite}s lsst sich eine Zuordnung von Objekttyp
 * zu Konfigurationsbereich festlegen.
 * 
 * @author HofmannJ
 * 
 */
public class CompositeAssociationObjectTypeConfigurationArea extends Composite
{
	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _connection = null;

	/**
	 * GTM Datenbasis
	 */
	private GtmDatenBasis _gtmDatenBasis = null;

	/**
	 * Linke Gruppe zum Auswhlen des ObjektTyps
	 */
	private Group _left = null;

	/**
	 * Rechte Gruppe zum Auswhlen des Konfigurationsbereiches
	 */
	private Group _right = null;

	/**
	 * Objekttypwhler
	 */
	private CompositeObjectTypeChooser _objectTypeChooser = null;

	/**
	 * Konfigurationsbereichswhler
	 */
	private CompositeConfigurationAreaChooser _configurationAreaChooser = null;

	/**
	 * Konstruktor
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll.
	 * @param style
	 *          SWT-Stil
	 * @param connection
	 *          Verbindung zum DaV
	 * @param gtmDatenBasis
	 *          {@link GtmDatenBasis}
	 */
	public CompositeAssociationObjectTypeConfigurationArea(Composite parent,
			int style, ClientDavInterface connection, GtmDatenBasis gtmDatenBasis)
	{
		super(parent, style);
		this._connection = connection;
		this._gtmDatenBasis = gtmDatenBasis;

		createContent();
	}

	/**
	 * Erzeugt den Inhalt des {@link Composite}s.
	 */
	private void createContent()
	{
		SashForm form = new SashForm(this, SWT.HORIZONTAL);

		createLeft(form);
		createRight(form);
		form.setWeights(new int[] { 50, 50 });

		/*
		 * Layout
		 */
		this.setLayout(new FillLayout());
	}

	/**
	 * Erzeugt die linke Gruppe zur Darstellung aller bekannten Objekttypen.
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll.
	 * @return Liefert die erzeugte {@link Group} zurck.
	 */
	private Group createLeft(Composite parent)
	{
		if (_left == null)
		{
			_left = new Group(parent, SWT.NONE);
			_left.setText("Pid Objekttyp:");

			DynamicObjectTypesDetermination d = new DynamicObjectTypesDetermination();
			LinkedList<SystemObjectType> dynObjTypes = d.determineDynObjTypes();
			_objectTypeChooser = new CompositeObjectTypeChooser(_left, SWT.NONE,
					_connection, dynObjTypes);

			/**
			 * Layout
			 */
			GridLayout gl = new GridLayout(1, false);
			_left.setLayout(gl);

			GridData gdObjTypChooser = new GridData();
			gdObjTypChooser.grabExcessHorizontalSpace = true;
			gdObjTypChooser.horizontalAlignment = GridData.FILL;
			gdObjTypChooser.grabExcessVerticalSpace = true;
			gdObjTypChooser.verticalAlignment = GridData.FILL;
			_objectTypeChooser.setLayoutData(gdObjTypChooser);
		}

		return _left;
	}

	/**
	 * Erzeugt die rechte Gruppe zur Darstellung aller verfgbaren
	 * Konfigurationsbereiche.
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll.
	 * @return Liefert die erzeugte {@link Group} zurck.
	 */
	private Group createRight(Composite parent)
	{
		if (_right == null)
		{
			_right = new Group(parent, SWT.NONE);
			_right.setText("Pid Konfigurationsbereich:");

			_configurationAreaChooser = new CompositeConfigurationAreaChooser(_right,
					SWT.NONE, _connection, EnumSelection.SINGLE);
			/**
			 * Layout
			 */
			GridLayout gl = new GridLayout(1, false);
			_right.setLayout(gl);

			GridData gdConfigurationAreaChooser = new GridData();
			gdConfigurationAreaChooser.grabExcessHorizontalSpace = true;
			gdConfigurationAreaChooser.horizontalAlignment = GridData.FILL;
			gdConfigurationAreaChooser.grabExcessVerticalSpace = true;
			gdConfigurationAreaChooser.verticalAlignment = GridData.FILL;
			_configurationAreaChooser.setLayoutData(gdConfigurationAreaChooser);
		}

		return _right;
	}

	/**
	 * Liefert die Pid des ersten, selektierten Objekttyps.
	 * 
	 * @return Pid des ersten, selektierten Objekttyps
	 */
	public String getFirstSelectedObjectTypePid()
	{
		String re = null;
		if (_objectTypeChooser.getSelectedObjectTypesPids().size() > 0)
		{
			re = _objectTypeChooser.getSelectedObjectTypesPids().getFirst();
		}
		return re;
	}

	/**
	 * Liefert die Pid des ersten, selektierten Konfigurationsbereiches.
	 * 
	 * @return Pid des ersten, selektierten Konfigurationsbereiches
	 */
	public String getFirstSelectedConfigurationAreaPid()
	{
		String re = null;

		LinkedList<String> selectedConfigurationAreasPids = _configurationAreaChooser
				.getSelectedConfigurationAreasPids();
		if (selectedConfigurationAreasPids.size() > 0)
		{
			re = selectedConfigurationAreasPids.getFirst();
		}

		return re;
	}

	/**
	 * Setzt die Vorselektion von Objekttypen und Konfigurationsbereich. Die
	 * Auswahl des Objekttyps wird deaktiviert.
	 * 
	 * @param record
	 */
	public void setPreselectionRecord(ConfigurationAreaModelRecord record)
	{
		if (record != null)
		{
			if ((record.getPidObjectType() != null)
					&& (record.getPidConfigurationArea() != null))
			{
				setSelectedObjectType(record.getPidObjectType());				
				setSelectedConfigurationArea(record.getPidConfigurationArea());
				setObjectChooserEditable(false);
			}
		}
	}

	/**
	 * Setzt einen zu selektierenden Objekttyp an Hand seiner Pid.
	 * 
	 * @param pidObjectType
	 *          Pid des zu selektierenden Objekttyps.
	 */
	public void setSelectedObjectType(String pidObjectType)
	{
		if (pidObjectType != null)
		{
			LinkedList<String> objTypePids = new LinkedList<String>();
			objTypePids.add(pidObjectType);
			_objectTypeChooser.setSelectedObjectTypesPids(objTypePids);
		}
	}

	/**
	 * Setzt einen zu selektierenden Konfigurationsbereich an Hand seiner Pid.
	 * 
	 * @param pidConfigurationArea
	 *          Pid des zu selektierenden Konfigurationsbereiches.
	 */
	public void setSelectedConfigurationArea(String pidConfigurationArea)
	{
		if (pidConfigurationArea != null)
		{
			LinkedList<String> confAreaPids = new LinkedList<String>();
			confAreaPids.add(pidConfigurationArea);
			_configurationAreaChooser.setSelectedConfigurationAreasPids(confAreaPids);
		}
	}

	/**
	 * Setzt die Editierbarkeit der Liste zur Auswahl von Objekttypen.
	 * 
	 * @param editable
	 *          Wenn <code>true</code>, dann ist die Liste editierbar, bei
	 *          <code>false</code> nicht.
	 */
	public void setObjectChooserEditable(boolean editable)
	{
		_objectTypeChooser.setEditable(editable);
	}

	/**
	 * Prft, ob die Liste zur Auswahl von Objekttypen editierbar ist oder nicht.
	 * 
	 * @return Liefert <code>true</code>, wenn die Liste editierbar ist, sonst
	 *         <code>false</code>.
	 */
	public boolean isEditable()
	{
		return _objectTypeChooser.isEditable();
	}

}
