/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.kontextmenu;


import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.MenuManager;

/**
 * Klasse um Menelemente anzulegen
 * 
 * @author Krause
 * 
 */
public class MenuElement extends MenuManager
{
  /**
   * ID
   */
  private String m_id;

  /**
   * Name
   */
  private String m_name;

  /**
   * Elternteil diese Mens
   */
  private MenuElement m_elternMenu;
  
  /**
   * Kind, das als Default selektiert wird
   */
  private Action m_defaultKind;
  
  /**
   * Liste der Kinder
   */
  private LinkedList<Object> m_listeKinder;

  /**
	 * Konstruktor um ein Men anzulegen
	 * 
	 * @param elternMenu
	 *            Men, zu dem dieses Men hinzugefgt werden soll
	 * @param id
	 *            ID des Mens
	 * @param name
	 *            anzuzeigender Name des Mens
	 */
  public MenuElement( MenuElement elternMenu, String id, String name )
  {
    super( name );
    
    m_id = id;
    m_name = name;
    m_listeKinder = new LinkedList<Object>();

    if ( elternMenu != null )
    {
      m_elternMenu = elternMenu;
      elternMenu.add( this );
    }

  }

  /**
   * Fgt ein Men hinzu
   * 
   * @param element Submen
   */
  public void add(IContributionItem element)
  {   
    super.add( element );
    m_listeKinder.add( element );
  }

  /**
   * Fgt ein Aktion hinzu
   * 
   * @param element Aktion
   * @param defaultElement Aktion soll als Default selektiert werden
   */
  public void add(Action element, boolean defaultElement)
  {   
    super.add( element );
    m_listeKinder.add( element );
    
    //DefaultAktion setzen
    if ( defaultElement && m_defaultKind == null )  
  	{
  		m_defaultKind = element;
  	  m_defaultKind.setChecked( true );
  	  
  	  if (m_elternMenu != null && m_elternMenu.getDefaultChild() == null)
  	  {
  	    m_elternMenu.setDefaultChild( m_defaultKind );
  	  }
  	}

  }

  /**
   * Setzen des Default Kindes
   */
  public boolean setDefaultChild(Action defaultKind)
  {
    if (m_listeKinder.contains( defaultKind ))
    {
      m_defaultKind = defaultKind;
      return true;
    }
    return false;
  }
  
  /**
   * Liefert das als Default gesetzte Kind
   */
  public Action getDefaultChild()
  {
    if (m_defaultKind == null && m_listeKinder != null && m_listeKinder.size() > 0)
    {
      //falls dies nicht mit der Methode add erfolgt ist
      for (Iterator kinder = m_listeKinder.iterator(); kinder.hasNext();)
      {
        Object objekt = (Object) kinder.next();
        if (objekt instanceof MenuElement )
        {
          MenuElement subMenu = (MenuElement) objekt;
          Action defaultKind = subMenu.getDefaultChild();
          if (defaultKind != null)
          {
            m_defaultKind = defaultKind;
            return m_defaultKind;
          }
        }
      }
    }
    return m_defaultKind;
  }
  
  /**
   * Liefert die Kinder des Mens
   */
  public LinkedList<Object> getChildren()
  {
    return m_listeKinder;
  }
  
}
