/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.navigatorView;


import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.objektzusammenstellung.ObjektSpeicher;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.navigator.Activator;
import de.bsvrz.sba.plugin.navigator.dnd.ZugZuhoererSystemObjekt;
import de.bsvrz.sba.plugin.navigator.kontextmenu.KontextMenuInhalt;
import de.bsvrz.sba.plugin.navigator.navigatorModell.INavigatorElement;
import de.bsvrz.sba.plugin.navigator.navigatorModell.NavigatorElement;
import de.bsvrz.sba.plugin.navigator.navigatorModell.NavigatorElement.KATEGORIE;

/**
 * Implementiert den Navigator innerhalb eines {@link TreeViewer}.
 * 
 * Dies beinhaltet die Bereitstellung eines Views sowie die Bereitstellung eines
 * Kontextmens mit den entsprechenden Editierfunktionen und die
 * Kontroll-Buttons hierzu.
 * 
 * @author Krause
 * 
 */

public class NavigatorView extends ViewPart
{
  /**
   * ID der Navigatorview
   */
  public static final String ID = "de.bsvrz.sba.plugin.navigator.Navigatorverwaltung";

  /**
   * Debug-Flag
   */
  private static final boolean m_debug = false;

  /**
   * Registry fr die verwendeten Bilder
   */
  private static MyImageRegistry m_pluginImageReg;

  /**
   * Basiscomposite fr die Darstellung
   */
  private Composite m_top;
  
  /**
   * Treeviewer fr den Navigator
   */
  public TreeViewer m_treeViewer;

  // /**
  // * aktuell aufgeklapptes Level des Navigators
  // */
  // private int m_aktuellerLevel;

  /**
   * Lade Daten aus dem Datenverteiler fr den Navigatorbaum
   */
  private LadeBaumDaten m_ladeBaumDaten;

  /**
   * Wurzel-Element des Navigator-Baums
   */
  public INavigatorElement m_root;
  
  /**
   * Speicher mit den Prferenzen (Navigatoreinstellungen)
   */
  private IPreferenceStore m_preferenceStore;
  
  /**
   * Liste der selektierten Systemobjekte.
   */
  private List<SystemObject> _listeDaten = new LinkedList<SystemObject>();

  private ZugZuhoererSystemObjekt _zugZuhoererSystemObjekt;

  /**
   * Konstruktor
   */
  public NavigatorView()
  {
    super();

    // ImageRegistry fllen
    fillImageRegistry();
    
    // Hole den Prferenzen Speicher
    m_preferenceStore = Activator.getDefault().getPreferenceStore();

  }

  @Override
  public void createPartControl( Composite parent )
  {

    m_top = new Composite(parent, SWT.NONE);  
    
    // Erzeuge Navigator-Baum
    erzeugeBaum( m_top );

    // Erzeuge Sorter, Toolbar, Kontextmens.
    erzeugeSorter();
    erzeugeToolbar();
    erzeugeKontextMenue();

    ladeNavigator();
    
    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( m_top, Activator.PLUGIN_ID + ".navigator" );

    /*
     * Layout
     */
    GridLayout glTop = new GridLayout(1, false);
    m_top.setLayout( glTop );
    
    GridData gdTree = new GridData();
    gdTree.grabExcessHorizontalSpace = true;
    gdTree.horizontalAlignment = GridData.FILL;
    gdTree.grabExcessVerticalSpace = true;
    gdTree.verticalAlignment = GridData.FILL;
    m_treeViewer.getControl().setLayoutData( gdTree );

  }

  /**
   * ImageRegistry fllen
   */
  private void fillImageRegistry()
  {   
//    m_pluginImageReg = MyImageRegistry.getInstance();
    m_pluginImageReg = new MyImageRegistry();
    
    m_pluginImageReg.putImage( "add", "icons/actions/add_16x16.png" );
    m_pluginImageReg.putImage( "remove", "icons/actions/remove.png" );
    m_pluginImageReg.putImage( "exit", "icons/actions/exit.png" );
    m_pluginImageReg.putImage( "collapse", "icons/actions/tree_collapse.png" );
    m_pluginImageReg.putImage( "expand", "icons/actions/tree_expand.png" );
    m_pluginImageReg.putImage( "collapseAll", "icons/actions/tree_collapseAll.png" );
    m_pluginImageReg.putImage( "expandAll","icons/actions/tree_expandAll.png" );
    m_pluginImageReg.putImage( "expandEqual", "icons/actions/tree_expandEqual.png" );
    m_pluginImageReg.putImage( "expandEqual", "icons/actions/tree_expandEqual.png" );
    m_pluginImageReg.put( "icons/items/unbekannt.png", PlatformUI.getWorkbench().getSharedImages().getImage(
    ISharedImages.IMG_OBJS_WARN_TSK ) );
        
  }

  /**
   * Liefert die ImageRegistry mit den zwischengespeicherten Bildern
   */
  public static MyImageRegistry getMyImageRegistry()
  {
    return m_pluginImageReg;
  }
  
  /**
   * Liefert die Baumstruktur zum Navigator
   * 
   * @param parent
   *          the parent control
   */
  private void erzeugeBaum( Composite parent )
  {
    // zur Performance Verbesserung eventuell VIRTUAL verwenden
    // m_treeViewer = new TreeViewer( parent, SWT.VIRTUAL );
    m_treeViewer = new TreeViewer( parent );
    m_treeViewer.setContentProvider( new NavigatorContentProvider() );
    m_treeViewer.setLabelProvider( new NavigatorLabelProvider() );
    m_treeViewer.setUseHashlookup( true );
    m_treeViewer.addSelectionChangedListener(new ISelectionChangedListener(){

      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        // TODO Auto-generated method stub
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        
        Object firstElement = selection.getFirstElement();
//<<<<<<< NavigatorView.java
//         
//=======
//        
////        System.out.println(firstElement.getClass().getName());
//        
//      
//        
//        
//>>>>>>> 1.12
        if (firstElement instanceof NavigatorElement)
        {
          
          NavigatorElement navElement = (NavigatorElement)firstElement;
          
          List<SystemObject> liste = new LinkedList<SystemObject>();
          @SuppressWarnings("unchecked")
          Iterator it = selection.iterator();
          while (it.hasNext())
          {
            NavigatorElement ne = (NavigatorElement)it.next();
            LinkedList<SystemObject> objekte = ne.getObjekte();            
            
            for (SystemObject so : objekte)
            {
//              System.out.println("so: " + so);
              liste.add(so);
//              if (!ObjektSpeicher.getInstanz().getSystemObjekte().contains(so)){
//                ObjektSpeicher.getInstanz().hinzufuegenSystemObjekt(so);                 
//              }
            }
            
            ObjektSpeicher.getInstanz().setSystemObjekte(liste);
            
            
          }
        }
      }
      
    });

    /*
     * NavigatorDaten aus dem Datenverteiler aus Performance-Grnden erst nach
     * der Baumerstellung laden
     */
    INavigatorElement root = new NavigatorElement( null, null, null, null );
    new NavigatorElement( root, "Daten werden geladen", KATEGORIE.unbekannt, null );

    if ( root != null )
      m_treeViewer.setInput( root );
    
    List<SystemObject> liste = ObjektSpeicher.getInstanz().getSystemObjekte();
    
    int ops = DND.DROP_COPY | DND.DROP_MOVE;      
    _zugZuhoererSystemObjekt = new ZugZuhoererSystemObjekt(m_treeViewer, liste);
    Transfer[] dragTransfers = new Transfer[] { TextTransfer.getInstance() };
    m_treeViewer.addDragSupport(ops, dragTransfers, _zugZuhoererSystemObjekt);
    
    
  }

  /**
   * Liefert die Sortierreihenfolge nach Kategorien
   */
  public void erzeugeSorter()
  {
    // Baum sortieren nach Label
    // m_treeViewer.setSorter( new ViewerSorter() );

    // Baum sortieren mit Kategorien
    m_treeViewer.setSorter( new ViewerSorter()
    {
      /**
       * Sortierung der Elemente in der Art, dass bestimmte Kategorien
       * entsprechend ihrer Nummerierung vor anderen kommen
       */
      public int category( Object element )
      {
      	//Position ber die Prferenzes
      	String elemTitel = ((NavigatorElement) element).getName();
        int position = 0;
        position = m_preferenceStore.getInt( Activator.PLUGIN_ID + ":" + elemTitel );
      	if ( position != 0 )
        	return position;
      	
      	//ansonsten, Position ber die Kategorie
        KATEGORIE kategorie = ((NavigatorElement) element).getKategorie();

				if (kategorie == null)
					return 10;

				switch( kategorie )
				{
					case Wetterstation:
						return 1;
					case KonfigBereich:
						return 2;
					case Anzeigequerschnitte:
						return 3;
					case Messquerschnitte:
						return 4;
					case Kalender:
						return 5;
					case Ordner:
						return 6;
				}
				return 10;
      }
    } );
  }

  /**
	 * Liefert die Toolbar mit den Kontroll-Aktionen zum Navigator
	 */
  private void erzeugeToolbar()
  {
    // m_aktuellerLevel = 1;

    // // BaumElement hinzufgen
    // IAction addAction = new Action("Add Element") {
    // public void run() {
    // addElement();
    // }
    // };
    // addAction.setToolTipText("Einfgen");
    // addAction.setImageDescriptor(Activator.getMyImageRegistry().getDescriptor(
    // "add" ));
    //    
    // // BaumElement entfernen
    // IAction removeAction = new Action("Delete") {
    // public void run() {
    // removeSelected();
    // }
    // };
    // removeAction.setToolTipText("Lschen");
    // removeAction.setImageDescriptor(Activator.getMyImageRegistry().getDescriptor(
    // "remove" ));
    //    
    // // Beenden
    // IAction actionBeenden = new Action("Exit") {
    // public void run()
    // {
    // //dies ist aber gefhrlich, da so kein sauberer Logout erfolgt
    // //getViewSite().getShell().close();
    // }
    // };
    // actionBeenden.setToolTipText("Beenden");
    // actionBeenden.setImageDescriptor(Activator.getMyImageRegistry().getDescriptor(
    // "exit"));

    // selektierte Baumknoten verstecken
    IAction actionCollapse = new Action( "Collapse" )
    {
      public void run()
      {
        actionCollapseActualSelection();
      };
    };
    actionCollapse.setToolTipText( "Kindknoten verstecken" );
    actionCollapse.setImageDescriptor( m_pluginImageReg.getDescriptor( "collapse" ) );

    // selektierte Baumknoten expandieren
    IAction actionExpand = new Action( "Expand" )
    {
      public void run()
      {
        // m_aktuellerLevel++;
        // m_treeViewer.getTree().setRedraw( false );
        // m_treeViewer.expandToLevel( m_aktuellerLevel );
        // m_treeViewer.getTree().setRedraw( true );

        actionExpandActualSelection();
      };
    };
    actionExpand.setToolTipText( "Kindknoten anzeigen" );
    actionExpand.setImageDescriptor( m_pluginImageReg.getDescriptor( "expand" ) );

    // alle Kindknoten verstecken
    IAction actionCollapseAll = new Action( "CollapseAll" )
    {
      public void run()
      {
        m_treeViewer.getTree().setRedraw( false );
        m_treeViewer.collapseAll();
        m_treeViewer.getTree().setRedraw( true );
        // m_aktuellerLevel = 1;
      };
    };
    actionCollapseAll.setToolTipText( "alle Kindknoten verstecken" );
    actionCollapseAll.setImageDescriptor( m_pluginImageReg.getDescriptor( "collapseAll" ) );

    // alle Kindknoten anzeigen
    IAction actionExpandAll = new Action( "ExpandAll" )
    {
      public void run()
      {
        // Informiert den Baum mit dem Neuzeichnen bis zur Fertigstellung zu
        // warten
        m_treeViewer.getTree().setRedraw( false );
        m_treeViewer.expandAll();
        // m_aktuellerLevel = 10;
        m_treeViewer.getTree().setRedraw( true );
      };
    };
    actionExpandAll.setToolTipText( "alle Kindknoten anzeigen" );
    actionExpandAll.setImageDescriptor( m_pluginImageReg.getDescriptor( "expandAll" ) );

    // // alle gleichen Baumknoten expandieren
    // IAction actionExpandEqual = new Action("ExpandEqual") {
    // public void run()
    // {
    // m_aktuellerLevel++;
    // m_treeViewer.getTree().setRedraw( false );
    // m_treeViewer.expandToLevel( m_aktuellerLevel );
    // m_treeViewer.getTree().setRedraw( true );
    // };
    // };
    // actionExpandEqual.setToolTipText("Baumknoten gleichen Typs expandieren");
    // actionExpandEqual.setImageDescriptor(Activator.getMyImageRegistry().getDescriptor(
    // "expandEqual"));

    // ToolBar erzeugen
    IToolBarManager toolBarManager = getViewSite().getActionBars().getToolBarManager();

    // toolBarManager.add(addAction);
    //    
    // toolBarManager.add(removeAction);
    //    
    // toolBarManager.add(actionBeenden);

    toolBarManager.add( actionCollapse );

    toolBarManager.add( actionExpand );

    toolBarManager.add( actionCollapseAll );

    toolBarManager.add( actionExpandAll );

  }

  // /** Add a new item to the first selected domain object.
  // * If nothing is selected add to the root.
  // */
  // protected void addElement()
  // {
  // INavigatorElement receivingElement = null;
  // if (m_treeViewer.getSelection().isEmpty())
  // {
  // receivingElement = m_root;
  // }
  // else
  // {
  // IStructuredSelection selection = (IStructuredSelection)
  // m_treeViewer.getSelection();
  // INavigatorElement selectedDomainObject = (INavigatorElement)
  // selection.getFirstElement();
  // if (!(selectedDomainObject instanceof INavigatorElement))
  // {
  // receivingElement = (INavigatorElement)selectedDomainObject.getParent();
  // }
  // else
  // {
  // receivingElement = (INavigatorElement)selectedDomainObject;
  // }
  // }
//   receivingElement.add(NavigatorElementMitStati.newElement());
  // }
  //  
  // /** Remove the selected domain object(s).
  // * If multiple objects are selected remove all of them.
  // *
  // * If nothing is selected do nothing. */
  // protected void removeSelected()
  // {
  // if (m_treeViewer.getSelection().isEmpty()) {
  // return;
  // }
  // IStructuredSelection selection = (IStructuredSelection)
  // m_treeViewer.getSelection();
  // /* Tell the tree to not redraw until we finish
  // * removing all the selected children. */
  // m_treeViewer.getTree().setRedraw(false);
  // for (Iterator iterator = selection.iterator(); iterator.hasNext();)
  // {
  // INavigatorElement element = (INavigatorElement) iterator.next();
  // INavigatorElement parent = (INavigatorElement)element.getParent();
  // parent.remove(element);
  // }
  // m_treeViewer.getTree().setRedraw(true);
  // }

  /**
   * Kollabiert die aktuelle Selektion im Baum
   */
  private void actionCollapseActualSelection()
  {
    if ( m_treeViewer != null )
    {
      LinkedList<INavigatorElement> expandedElem = new LinkedList<INavigatorElement>();

      ISelection selection = m_treeViewer.getSelection();
      if ( selection instanceof StructuredSelection )
      {
        StructuredSelection ss = (StructuredSelection) selection;

        Iterator itExpElem = ss.iterator();
        while (itExpElem.hasNext())
        {
          Object obj = itExpElem.next();

          if ( obj instanceof INavigatorElement )
          {
            INavigatorElement elem = (INavigatorElement) obj;
            if ( elem.hasChildren() )
            {
              if ( !expandedElem.contains( elem ) )
              {
                expandedElem.add( elem );
              }
            }
            else if ( elem.hasParent() )
            {
              INavigatorElement parentElem = (INavigatorElement) elem.getParent();
              if ( !expandedElem.contains( parentElem ) )
              {
                expandedElem.add( parentElem );
              }
            }
          }
        }
      }

      if ( expandedElem.size() > 0 )
      {
        m_treeViewer.getTree().setRedraw( false );
        Iterator<INavigatorElement> it = expandedElem.iterator();
        while (it.hasNext())
        {
          INavigatorElement elem = it.next();
          m_treeViewer.collapseToLevel( elem, AbstractTreeViewer.ALL_LEVELS );
        }
        m_treeViewer.getTree().setRedraw( true );
      }

    }
  }

  /**
   * Expandiert die aktuelle Selektion im Baum
   */
  private void actionExpandActualSelection()
  {
    if ( m_treeViewer != null )
    {
      LinkedList<INavigatorElement> collapsedNodes = new LinkedList<INavigatorElement>();

      ISelection selection = m_treeViewer.getSelection();
      if ( selection instanceof StructuredSelection )
      {
        StructuredSelection ss = (StructuredSelection) selection;

        @SuppressWarnings("unchecked")
        Iterator itColBest = ss.iterator();
        while (itColBest.hasNext())
        {
          Object obj = itColBest.next();

          if ( obj instanceof INavigatorElement )
          {
            INavigatorElement elem = (INavigatorElement) obj;
            if ( elem.hasChildren() )
            {
              collapsedNodes.add( elem );
            }
          }
        }
      }

      if ( collapsedNodes.size() > 0 )
      {
        m_treeViewer.getTree().setRedraw( false );
        Iterator<INavigatorElement> it = collapsedNodes.iterator();
        while (it.hasNext())
        {
          INavigatorElement colNode = it.next();
          m_treeViewer.expandToLevel( colNode, AbstractTreeViewer.ALL_LEVELS );
        }
        m_treeViewer.getTree().setRedraw( true );
      }
    }
  }

  /**
   * Liefert die KontextMens zum Navigator
   */
  private void erzeugeKontextMenue()
  {

    // MenuManager anfertigen
    MenuManager kontextMenuMgr = new MenuManager( "#PopupMenu" );
    // Menpunkte nicht bis zum nchsten Aufruf aufbewahren
    kontextMenuMgr.setRemoveAllWhenShown( true );
    
    // Seit Eclipse 3.5 funktioniert dies nicht mehr, alle Items werden nun manuell 
    // in fuelleKontextMenu geloescht! 
    kontextMenuMgr.setRemoveAllWhenShown( false );
    // Beim Aufruf Men dynamisch erzeugen
    kontextMenuMgr.addMenuListener( new IMenuListener()
    {

      // Kontextmenu dynamisch erstellen
      // --------------------------------
      public void menuAboutToShow( IMenuManager manager )
      {
        fuelleKontextMenu( manager );
      }
    } );

    // Men erzeugen - Viewer wird dabei dem Men bekannt gemacht
    Menu menu = kontextMenuMgr.createContextMenu( m_treeViewer.getControl() );
    // Das Men dem Baum bekannt machen
    m_treeViewer.getControl().setMenu( menu );
    // Mit der ViewSite registrieren
    this.getSite().registerContextMenu( kontextMenuMgr, m_treeViewer );

    /*
     * Hinzufgen eines SelektionListeners zum Baum, der auf Doppelklicks
     * reagiert. Es wird das selektierte Objekt der entsprechenden Zelle als
     * String zurckgeliefert und die selektierte Action des KontextMenu
     * ausgefhrt. Ist im KontextMenu keine Action selektiert, erscheint ein
     * entsprechender Hinweis.
     */
    m_treeViewer.getTree().addSelectionListener( new SelectionAdapter()
    {
      public void widgetDefaultSelected( SelectionEvent e )
      {
        IStructuredSelection is = (IStructuredSelection) m_treeViewer.getSelection();
        Object erstesElement = is.getFirstElement();

        KontextMenuInhalt kontextMenu = null;
        if ( erstesElement instanceof NavigatorElement )
        {
          NavigatorElement selElem = (NavigatorElement) erstesElement;
          kontextMenu = new KontextMenuInhalt( selElem.getKategorie(), null, null );
          // anzuzeigende Objekte fllen
          kontextMenu.addObjekte( selElem.getObjekte() );
        }

        if ( kontextMenu.selektierteAktionAusfuehren() )
        {
          if ( m_debug )
            System.out.println( "Aktion zum Doppelklick ausgefhrt!" );
        }
        else
        {
          MessageBox mb = new MessageBox( PlatformUI.getWorkbench().getDisplay().getActiveShell(),
              SWT.APPLICATION_MODAL | SWT.ICON_ERROR | SWT.OK );
          mb.setText( "Fehler:" );
          mb.setMessage( "Es ist zur Zeit keine Aktion hinterlegt, " + "die per Doppelklick ausgefhrt werden kann!" );
          mb.open();
        }
      }
    } );

  }

  /**
   * Diese Methode prft die gleichzeitige Auswahl der Actions und fllt das
   * Kontextmen mit den registrierten {@link Action}s.
   * 
   * @param manager
   *          Der {@link IMenuManager}, dessen Kontextmen gefllt werden soll.
   */
  private void fuelleKontextMenu( IMenuManager manager )
  {
	// Seit Eclipse 3.5 funktioniert setRemoveAllWhenShown(true) nicht mehr, 
	// alle Items werden nun manuell geloescht   
	manager.removeAll();  
	  
    IStructuredSelection selection = (IStructuredSelection) m_treeViewer.getSelection();
    if ( selection.isEmpty() )
      return;

    // Prfung, ob alle ausgewhlten Elemente vom gleichen Typ(Kategorie) sind
    NavigatorElement erstesElement = (NavigatorElement) selection.getFirstElement();
    KATEGORIE kategorie = erstesElement.getKategorie();
    for (Iterator iterator = selection.iterator(); iterator.hasNext();)
    {
      NavigatorElement selElement = (NavigatorElement) iterator.next();

      if ( selElement.getKategorie() != kategorie )
      {
        //Die Kategorien sind nicht gleich ==> kein Kontextmen
        return;
      }
    }// for

    //wird eventuell spter mal bentigt:
    // KontextMenu zum ausgewhlten Element erstellen
    // an Hand der Kategorie der ausgewhlten Navigatorelemente

    // // und der Kategorie des Elternelements (z.B. beim KonfigBereich
    // // notwendig)
    // NavigatorElementMitStati parent = (NavigatorElementMitStati)
    // firstElement.getParent();
    // KontextMenuInhalt kontextMenu = new KontextMenuInhalt(
    // firstElement.getKategorie(), parent.getKategorie(), null );

    KontextMenuInhalt kontextMenu = new KontextMenuInhalt( erstesElement.getKategorie(), null, null );

    // anzuzeigende Objekte fllen
    for (Iterator iterator = selection.iterator(); iterator.hasNext();)
    {
      NavigatorElement selElem = (NavigatorElement) iterator.next();	
      kontextMenu.addObjekte( selElem.getObjekte() );
    }

    // Kontextmenu anzeigen
    LinkedList<Object> listeKontextMenu = kontextMenu.getKontextListe();

    for (Iterator iterator = listeKontextMenu.iterator(); iterator.hasNext();)
    {
      Object object = (Object) iterator.next();
      if ( object instanceof Action )
      {	
        Action action = (Action) object;
        manager.add( action );
      }
      if ( object instanceof MenuManager || object instanceof Separator )
      {	
        IContributionItem ici = (IContributionItem) object;
        manager.add( ici );
      }
    }// for

  }

  /**
   * Laden der Daten fr den Navigatorbaum
   */
  public void ladeNavigator()
  {
    // Starte Job beim 1.Mal
    if ( m_ladeBaumDaten == null )
    {
      // Lade Daten aus dem Datenverteiler fr den Navigatorbaum,
      // die bereits ber den Starter vorgeladen werden
      m_ladeBaumDaten = LadeBaumDaten.getInstance();

      if (m_debug)
        System.out.println( "View: " + m_ladeBaumDaten.hashCode() );
 
      // wenn bereits geladen
      if ( m_ladeBaumDaten.isDatenGeladen() ){
        aktualisiereView( m_ladeBaumDaten.getRoot() );
      }

      // warten bis das Laden fertig ist
      m_ladeBaumDaten.addListener( new ILadeDatenListener()
      {

        @Override
        public void ladenFertig( INavigatorElement root )
        {
//          if (root != null)            
          aktualisiereView( root );
        }
      } );
    }
    else
    {
      // Daten fr den Navigator neu laden (bei Preferences)
      m_ladeBaumDaten.start();
    }

  }

  private void aktualisiereView( INavigatorElement root )
  {
    m_root = root;
    if (m_debug)
      System.out.println("Baum aktualisieren mit " + m_root);

    PlatformUI.getWorkbench().getDisplay().syncExec( new Runnable()
    {

      @Override
      public void run()
      {
        if ( m_root != null )
        {
          m_treeViewer.setInput( m_root );
        }
      }

    } );
  }

  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (m_top != null)
      m_top.setFocus();

  }

  @Override
  public void dispose()
  {
    m_ladeBaumDaten.dispose();
    super.dispose();
  }

}
