/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.daten;

public enum FahrstreifenLage
{
  /**
   * Unbekannt
   */
  UNBEKANNT ("unbekannt", -1),
  
  /**
   * Hauptfahrstreifen
   */
  HFS ("HFS", 0),
  
  /**
   * 1. berholfahrstreifen.
   */
  UEFS1 ("FS1", 1),

  /**
   * 2. berholfahrstreifen.
   */
  UEFS2 ("FS2", 2),

  /**
   * 3. berholfahrstreifen.
   */
  UEFS3 ("FS3", 3),

  /**
   * 4. berholfahrstreifen.
   */
  UEFS4 ("FS4", 4),

  /**
   * 5. berholfahrstreifen.
   */
  UEFS5 ("FS5", 5),

  /**
   * 6. berholfahrstreifen.
   */
  UEFS6 ("FS6", 6),

  /**
   * Standstreifen
   */
  STNDS ("Standstreifen", 7);
  
  /**
   * Kodierung der Lage im DaV als ganze Zahl
   */
  private int m_kodierung;
  
  /**
   * Kennung des Fahsrstreifens
   */
  private String m_kennung;
  
  /**
   * Konstruktor der Klasse
   * @param kodierung Kodierung der Enum-Werte im DaV
   */
  private FahrstreifenLage (String kennung, int kodierung)
  {
  	m_kennung = kennung;
  	
    m_kodierung = kodierung;  
  }
  
  /**
   * Methode liefert den Wert der zu einer Kodierung im DaV gehrt
   * @param kodierung Kodierung deren Wert gesucht wird
   * @return Wert, bei einer falschen Kodierung: 'UNBEKANNT'
   */
  public static FahrstreifenLage  getValue (int kodierung)
  {
    for (FahrstreifenLage s: FahrstreifenLage.values())
    {
      if (s.m_kodierung == kodierung)
        return s;
    }
    
    return UNBEKANNT;
  }

  /**
   * Methode liefert den Wert der zu einer Kodierung im DaV gehrt
   * @param kodierung Kodierung deren Wert gesucht wird
   * @return Wert, bei einer falschen Kodierung: 'UNBEKANNT'
   */
  public static FahrstreifenLage  getValue (String kennung)
  {
    for (FahrstreifenLage s: FahrstreifenLage.values())
    {
      if (s.m_kennung.equals(kennung))
        return s;
    }
    
    return UNBEKANNT;
  }

  /**
   * Methode liefert die Variable m_kodierung zurck
   * @return Variable m_kodierung
   */
  public int getKodierung()
  {
    return m_kodierung;
  }

	/**
   * Methode liefert kennung zurck
   * @return kennung
   */
  public String getKennung()
  {
  	return m_kennung;
  }
}
