/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.daten;

/**
 * Klasse beschreibt die Eigenschaft ob in einer Vorgnger/Nachfolger Beziehung die
 * direkten Vorgnger/Nachfolger oder die Vorgnger/Nachfolger gleichen Typs verwendet
 * werden sollen.
 * @author ObertM
 *
 */
public enum VorgaengerNachfolgerArt
{
  /**
   * Direkte Vorgnger/Nachfolger
   */
  DIREKT ("Direkt", 0),
  
  /**
   * Vorgnger/Nachfolger gleichen Typs
   */
  GLEICHERTYP ("Gleicher Typ", 1);
  
  
  /**
   * Kodierung der Lage im DaV als ganze Zahl
   */
  private int m_kodierung;
  
  /**
   * Kennung
   */
  private String m_kennung;
  
  /**
   * Konstruktor der Klasse
   * @param kodierung Kodierung der Enum-Werte im DaV
   */
  private VorgaengerNachfolgerArt (String kennung, int kodierung)
  {
  	m_kennung = kennung;
  	
    m_kodierung = kodierung;  
  }
  
  /**
   * Methode liefert den Wert der zu einer Kodierung im DaV gehrt
   * @param kodierung Kodierung deren Wert gesucht wird
   * @return Wert, bei einer falschen Kodierung: 'GLEICHERTYP'
   */
  public static VorgaengerNachfolgerArt getArt (int kodierung)
  {
    for (VorgaengerNachfolgerArt s: VorgaengerNachfolgerArt.values())
    {
      if (s.m_kodierung == kodierung)
        return s;
    }
    
    return GLEICHERTYP;
  }

  /**
   * Methode liefert den Wert der zu einer Kodierung im DaV gehrt
   * @param kodierung Kodierung deren Wert gesucht wird
   * @return Wert, bei einer falschen Kodierung: 'GLEICHERTYP'
   */
  public static VorgaengerNachfolgerArt getArt (String kennung)
  {
    for (VorgaengerNachfolgerArt s: VorgaengerNachfolgerArt.values())
    {
      if (s.m_kennung.equals(kennung))
        return s;
    }
    
    return GLEICHERTYP;
  }

  /**
   * Methode liefert die Variable m_kodierung zurck
   * @return Variable m_kodierung
   */
  public int getKodierung()
  {
    return m_kodierung;
  }

	/**
   * Methode liefert kennung zurck
   * @return kennung
   */
  public String getKennung()
  {
  	return m_kennung;
  }
}
