package de.bsvrz.sba.plugin.parameter.steuerungskern.konfig;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.FahrtRichtung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeElementFigure;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.EAnzeigeTypen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Methode zum Speichern der bentigten Konfigurationsdaten
 * @author ObertM
 *
 */
public class Konfig
{
	/**
	 * Vektor mit den Systemobjekte vom Typ 'typ.anzeige'
	 */
	private Vector<SystemObject> _anzeigen = new Vector<SystemObject>();

	/**
	 * Vektor mit den Systemobjekte vom Typ 'typ.anzeigeQuerschnitt'
	 */
	private Vector<SystemObject> _anzeigeQuerschnitte = new Vector<SystemObject>();

	/**
	 * Hashmap mit der Zuordnung 'typ.anzeige' --> 'typ.sbaAnzeige'. Als Key dient das Systemobjekt
	 * der Anzeige als Value eine Instanz der SBAAnzeige
	 */
	private HashMap<SystemObject, SBAAnzeige> _zuordnungSBAAnzeige = new HashMap<SystemObject, SBAAnzeige>();

	/**
	 * Hashmap mit der Zuordnung 'typ.anzeigeQuerschnitt' --> 'typ.sbaAnzeigeQuerschnitt'. Als Key dient das 
	 * Systemobjekt des Anzeigequerschnitts, als Value eine Instanz des SBAAnzeigeQuerschnitts
	 */
	private HashMap<SystemObject, SBAAnzeigeQuerschnitt> _zuordnungSBAAnzeigeQuerschnitt = new HashMap<SystemObject, SBAAnzeigeQuerschnitt>();
	
	/**
	 * Hashmap mit der Zuordnung der Anzeigen zu den Anzeigequerschnitten. Als Key dient die Pid eines Anzeigequerschnitts,
	 * als Value eine Liste mit den Pids der zu Anzeigequerschnitt gehrenden Anzeigen
	 */
	private HashMap<String, List<String>> _zuordnungAnzeigenAnzeigeQuerschnitt = new HashMap<String, List<String>>();

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _connection;
	
  /**
   * Einzige Instanz der Klasse
   */
  private static Konfig _instance = null;

  /**
   * Methode leifert die einzige Instanz der Klasse
   * @return einzige Instanz dieser Klasse.
   */
  public static Konfig getInstance()
  {
  	if (_instance == null)
  	{
  		synchronized (Konfig.class)
      {
  			if (_instance == null)
  			{
  				_instance = new Konfig();
  			}
      }
  	}
    return _instance;
  }

  /**
   * Privater Konstruktor der Klasse
   */
  private Konfig()
  {
  	inialialisiere ();
  }

  /**
   * Methode zum Inialisieren der Konfiguration
   */
	private void inialialisiere()
  {
		_connection = DaVVerbindung.getInstanz().getDaVVerbindung();
			
		// Auslesen der bentigten Anzeigequerschnitte aus dem DaV
		
		List<SystemObject> querschnitte = VerwaltungMoeglicheAnzeigeQuerschnitte.getInstance().getAnzeigeQuerschnitte();

  	TreeMap<String, SystemObject> sorter = new TreeMap<String, SystemObject>();
  	for (SystemObject so : querschnitte)
  		sorter.put(so.getName(), so);

	  _anzeigeQuerschnitte.clear();
	  for (SystemObject so : sorter.values())
	  {
	  	_anzeigeQuerschnitte.add(so);
	  	_zuordnungSBAAnzeigeQuerschnitt.put(so, null);
	  }

		// Bestimmen der bentigten Anzeigen in Abhngigkeit der Anzeigequerschnitte
		
  	sorter.clear();
    for (SystemObject so : querschnitte)
    	bestimmeAnzeigen(so, sorter);

    _anzeigen.clear();
    for (SystemObject so : sorter.values())
    {
    	_anzeigen.add(so);
    	_zuordnungSBAAnzeige.put(so, null);
    }

    // Bestimmen der SBA Anzeigen
    
		bestimmeSBAAnzeigen();

    // Bestimmen der SBA Anzeigequerschnitte

		bestimmeSBAAnzeigeQuerschnitte ();
		
 		pruefePlausibilitaet();
  }
	
	/**
	 * Plausibilittsprfung
	 */
	private void pruefePlausibilitaet()
  {
		/*
		 * Prfen ob jeder Anzeige eine SBA Anzeige zugeordnet ist
		 */
		Vector<SystemObject> anzeigen = getAnzeigen();
		for (SystemObject a : anzeigen)
		{
			if (_zuordnungSBAAnzeige.get(a) == null)
				System.err.println("Anzeige " + a.getName() + " hat keine Zuordnung zu einer sbaAnzeige");
//			else
//				System.out.println("Anzeige " + a.getName() + " ---> " + _zuordnungSBAAnzeige.get(a).getWzgTyp().getPid());
		}
		
		/*
		 * Prfen ob jedem AnzeigeQuerschnitt ein SBA AnzeigeQuerschnitt zugeordnet ist
		 */
		Vector<SystemObject> aqs = getAnzeigeQuerschnitte();
		for (SystemObject aq : aqs)
		{
			if (_zuordnungSBAAnzeigeQuerschnitt.get(aq) == null)
				System.err.println("Anzeigequershcnitt " + aq.getName() + " hat keine Zuordnung zu einem sbaAnzeigeQuerschnitt");
//			else
//				System.out.println("Anzeige " + a.getName() + " ---> " + _zuordnungSBAAnzeige.get(a).getWzgTyp().getPid());
		}

  }

	/**
	 * Methode bestimmt die im System definierten Systemobjekte vom Typ 'typ.sswSbaAnzeige'
	 */
  private void bestimmeSBAAnzeigen ()
  {
  	List<SystemObject> objekte = Tools.getSystemObjekte("typ.sswSbaAnzeige");
  	
  	for (SystemObject so : objekte)
  	{
  		SBAAnzeige sbaAnzeige = new SBAAnzeige(so);
  		
  		SystemObject anzeige = sbaAnzeige.getAnzeige();
  		
  		if (anzeige != null)
  		{
  			if (_zuordnungSBAAnzeige.get(anzeige) != null)
  				System.err.println("Anzeige " + anzeige.getName() + " doppelte Zuordnung ");
  			
  			_zuordnungSBAAnzeige.put (sbaAnzeige.getAnzeige(), sbaAnzeige);
  		}
  	}
  }

	/**
	 * Methode bestimmt die im System definierten Systemobjekte vom Typ 'typ.sswSbaAnzeigeQuerschnitt'
	 */
  private void bestimmeSBAAnzeigeQuerschnitte ()
  {
  	List<SystemObject> objekte = Tools.getSystemObjekte("typ.sswSbaAnzeigeQuerschnitt");
  	
  	for (SystemObject so : objekte)
  	{
  		SBAAnzeigeQuerschnitt sbaAnzeigeQuerschnitt = new SBAAnzeigeQuerschnitt(so);
  		
  		SystemObject aq = sbaAnzeigeQuerschnitt.getAnzeigeQuerschnitt();
  		
  		if (aq != null)
  		{
  			if (_zuordnungSBAAnzeigeQuerschnitt.get(aq) != null)
  				System.err.println("AQ " + aq.getName() + " doppelte Zuordnung ");
  			
  			_zuordnungSBAAnzeigeQuerschnitt.put (aq, sbaAnzeigeQuerschnitt);
  		}
  	}
  }

	/**
	 * Methode liefert eine Bezeichnung fr einen Aspekt. Die Bezeichnung setzt sich
	 * zusammen aus Name + Pid
	 * @param objekt Systemobjekt des Aspekts
	 * @return Bezeichnung 
	 */
	public String getBezeichnungSchaltgrund (SystemObject objekt)
	{
		return objekt.getName() + " (" + objekt.getPid() + ")";
	}

  /**
   * Methode liefert die Objekte eines bestimmten Typs
   * @param pidTyp Pid des gesuchten Typs
   * @return Liste mit Objekten dieses Typs
   */
  public List<SystemObject> bestimmeObjekteImDaV (String pidTyp)
  {
    SystemObjectType type = _connection.getDataModel().getType(pidTyp);
    if (type == null)
      throw new IllegalArgumentException("Kein Typ " + pidTyp + " definiert !");
      
    return type.getElements();
  }

  /**
   * Methode liefert den WZG Typ zu einer Anzeige (Objekt vom Typ  "typ.anzeige") 
   * @param pidAnzeige Pid der Anzeige
   * @return WZG Typ
   */
  public SystemObject getWzgTypVonAnzeige (String pidAnzeige)
  {
  	SystemObject anzeige = getAnzeigeByPid(pidAnzeige);
  	if (anzeige != null)
  	{
	    SBAAnzeige sbaAnzeige = _zuordnungSBAAnzeige.get(anzeige);
	    if (sbaAnzeige != null)
	    	return sbaAnzeige.getWzgTyp();
  	}
  	
  	return null;
  }

  /**
   * Methode liefert die WZG Typen zu einem Anzeigequerschnitt (Objekt vom Typ  "typ.anzeigeQuerschnitt") 
   * @param pidAnzeigeQuerschnitt Pid des Anzeigequerschnitts
   * @return Liste mit WZG Typen 
   */
  public Vector<SystemObject> getWzgTypenVonAnzeigeQuerschnitt (String pidAnzeigeQuerschnitt)
  {
  	Vector<SystemObject> l = new Vector<SystemObject>();
  	
  	if (_zuordnungAnzeigenAnzeigeQuerschnitt.containsKey(pidAnzeigeQuerschnitt))
  	{
  		List<String> anzeigen = _zuordnungAnzeigenAnzeigeQuerschnitt.get(pidAnzeigeQuerschnitt);
  		for (String pidAnzeige : anzeigen)
  		{
  			SystemObject wzgTyp = getWzgTypVonAnzeige(pidAnzeige);
  			
  			if ((wzgTyp != null) && (!l.contains(wzgTyp)))
  				l.add(wzgTyp);
  		}
  	}
  	
  	return l;
  }

  
  /**
   * Methode bestimmt die zu einem Anzeigequerschnitt gehrenden Anzeigen
   * @param so SystemObjekt vom Typ "typ.anzeigeQuerschnitt"
   */
  private void bestimmeAnzeigen (SystemObject so, TreeMap<String, SystemObject> sorter )
  {
    // Die Zuordnung Anzeigequerschnitt zu den Anzeigen ist in einer Menge 'Anzeigen' festgelegt.
    // Daher muss nun fr jeden Anzeigequerschnitt die zugeordneten Anzeigen ausgelesen werden
    // und in umgekehrter Zuordnung gespeichert werden.
    
    ConfigurationObject anzeigeQuerschnitt = (ConfigurationObject) so;
  
    if (anzeigeQuerschnitt != null)
    {
    	if (!_zuordnungAnzeigenAnzeigeQuerschnitt.containsKey(so.getPid()))
    	  _zuordnungAnzeigenAnzeigeQuerschnitt.put(so.getPid(), new ArrayList<String>());	

    	List<String> zuordnung = _zuordnungAnzeigenAnzeigeQuerschnitt.get(so.getPid());
    	
      NonMutableSet menge = anzeigeQuerschnitt.getNonMutableSet("Anzeigen");
      if (menge != null)
      {
        List<SystemObject> elemente = menge.getElements();
        for (int i = 0; i < elemente.size(); i++)
        {
          ConfigurationObject anzeige = (ConfigurationObject)elemente.get(i);
          zuordnung.add(anzeige.getPid());

          sorter.put (((SystemObject) anzeige).getName(), anzeige);
        }
      }
    }
  }

  /**
   * Methode liefert eine Liste mit den Pids der Anzeigequerschnitte
   * @return Liste mit den Pids der Anzeigequerschnitte
   */
  public List<String> getPidsAnzeigeQuerschnitte ()
  {
    List<String> l = new ArrayList<String>();
    for (SystemObject so : _anzeigeQuerschnitte)
    	l.add(so.getPid());
    
    return l;
  }

  /**
   * Methode liefert den Aspekt zu einer bestimmten Pid
   * @param pidAspekt Pid des Aspekts
   * @return Aspekt
   */
  public Aspect getAspekt (String pidAspekt)
  {
  	if (_connection != null && pidAspekt != null)
  	{
  		SystemObject so = _connection.getDataModel().getObject(pidAspekt);
  		return (Aspect) so;
  	}
  	
  	return null;
  }

  /**
   * Methode liefert den Anzeigequerschnitt zu einem bestimmten Namen
   * @param nameAnzeigeQuerschnitt Name des Anzeigequerschnitts
   * @return Anzeigequerschnitt
   */
  public SystemObject getAnzeigeQuerschnittByName (String nameAnzeigeQuerschnitt)
  {
  	for (SystemObject so : _anzeigeQuerschnitte)
  	{
  		System.out.println(so.getName());
      if (so.getName().equals(nameAnzeigeQuerschnitt))
      	return so;
  	}
  	
  	System.out.println("nicht gefunden");
  	return null;
  }

  /**
   * Methode liefert den Anzeigequerschnitt zu einer bestimmten Pid
   * @param pidAnzeigeQuerschnitt Pid des Anzeigequerschnitts
   * @return Anzeigequerschnitt
   */
  public SystemObject getAnzeigeQuerschnittByPid (String pidAnzeigeQuerschnitt)
  {
  	for (SystemObject so : _anzeigeQuerschnitte)
      if (so.getPid().equals(pidAnzeigeQuerschnitt))
      	return so;
  	
  	return null;
  }

  public SystemObject getSystemObject (String pid)
  {
  	if (_connection != null && pid != null)
  	{
  		SystemObject so = _connection.getDataModel().getObject(pid);
  		return so;
  	}
  	
  	return null;
  }
  
  /**
   * Methode liefert einen Vektor mit den Anzeigequerschnitten
   * @return Vektor mit den Anzeigequerschnitten
   */
	public Vector<SystemObject> getAnzeigeQuerschnitte()
  {
		Vector<SystemObject> v = new Vector<SystemObject>();
    v.addAll(_anzeigeQuerschnitte);
    		
	  return v;
  }

  /**
   * Methode liefert einen Vektor mit den Anzeigequerschnitten pro Fahrtrichtung
   * @return Vektor mit den Anzeigequerschnitten
   */
	public Vector<SystemObject> getAnzeigeQuerschnitte(FahrtRichtung fahrtrichtung)
  {
		Vector<SystemObject> v = new Vector<SystemObject>();

		for (SystemObject so : _anzeigeQuerschnitte)
		{
			if (so.getPid().contains(fahrtrichtung.getKuerzel()))
			  v.add(so);
		}
    		
	  return v;
  }

	/**
	 * Methode liefert den Anzeigequerschnitt zu einer Anzeige
	 * @param anzeige
	 */
	public SystemObject getAnzeigeQuerschnittZuAnzeige (SystemObject anzeige)
  {
		Iterator<Entry<String, List<String>>> it = _zuordnungAnzeigenAnzeigeQuerschnitt.entrySet().iterator();
		while (it.hasNext())
		{
			Entry<String, List<String>> entry = it.next();

			String pidAQ = entry.getKey();
			List<String> pidAnzeigen = entry.getValue();
			
			for (String s : pidAnzeigen)
				if (s.equals(anzeige.getPid()))
					return getSystemObject(pidAQ);
		}

		return null;
  }

	/**
	 * Methode liefert einen Vektor mit den Systemobjekten vom Typ "typ.anzeige"
	 * @return Vektor mit Systemobjekten
	 */
	public Vector<SystemObject> getAnzeigen ()
	{
		Vector<SystemObject> v = new Vector<SystemObject>();
		v.addAll(_anzeigen);
		
		return v;
	}

  /**
   * Methode liefert die Anzeige zu einem bestimmten Namen
   * @param nameAnzeige Name der Anzeige
   * @return Anzeige
   */
  public SystemObject getAnzeigeByName (String nameAnzeige)
  {
  	for (SystemObject so : _anzeigen)
      if (so.getName().equals(nameAnzeige))
      	return so;
  	
  	return null;
  }

  /**
   * Methode liefert die Anzeige zu einer bestimmten Pid
   * @param pidAnzeige Pid der Anzeige
   * @return Anzeige
   */
  public SystemObject getAnzeigeByPid (String pidAnzeige)
  {
  	for (SystemObject so : _anzeigen)
      if (so.getPid().equals(pidAnzeige))
      	return so;
  	
  	return null;
  }

  /**
   * Methode liefert den WZG-Typ zu einem bestimmten Namen
   * @param nameWZGTyp Name des Typs
   * @return WZG Typ
   */
  public SystemObject getWZGTypByName (String nameWZGTyp)
  {
    List<SystemObject> typen = bestimmeObjekteImDaV("typ.sswSbaAnzeigeTyp");

  	for (SystemObject so : typen)
      if (so.getName().equals(nameWZGTyp) || Tools.getBezeichnungAnzeigeTyp(so).equals(nameWZGTyp))
      	return so;
  	
  	return null;
  }

  /**
   * Methode liefert den WZG-Typ zu einer bestimmten Pid
   * @param Pid des WZG Typs
   * @return WZG Typ
   */
	public SystemObject getWZGTypByPid(String pid)
  {
		if (_connection != null)
		{
			return _connection.getDataModel().getObject(pid);
		}
	  return null;
  }
	
	public String getRichtung (SystemObject anzeigeQuerschntitt)
	{
		SBAAnzeigeQuerschnitt sbaAQ = _zuordnungSBAAnzeigeQuerschnitt.get(anzeigeQuerschntitt);
		if (sbaAQ != null)
		  return sbaAQ.getRichtung();
		
		return null;
	}
	
	public List<SystemObject> bestimmeMoeglicheBilder (SystemObject wzgTyp)
	{
		List<SystemObject> l = new ArrayList<SystemObject>();

		if (wzgTyp != null)
		{
  	  AnzeigeElementTyp aet = AnzeigeElementTyp.getTypByPid(wzgTyp.getPid());
  	  if (aet != null)
  	  {
        List<AnzeigeInhaltTyp> moeglicheBilder = AnzeigeElementFigure.createAnzeigeInhalte(aet);
      
	      for (AnzeigeInhaltTyp bild : moeglicheBilder)
	        l.add(_connection.getDataModel().getObject(bild.getBildInhalt()));
  	  }
    }
		
    return l;
	}

  /**
   * Methode liefert die Pid der SBA
   * @return Pid der SBA
   */
	public static String getPidSBA()
  {
	  return ProjektSpezifischeEinstellungen.getPidSba();
  }
}
