package de.bsvrz.sba.plugin.parameter.steuerungskern.konfig;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;

import de.bsvrz.buv.rw.basislib.berechtigung.SystemObjekteEreignis;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix.InhaltMitTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeTypEigenschaften;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.DefaultEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.EAnzeigeTypen;

public class VerwaltungWvzInhalte
{
	/**
	 * Einzige Instanz der Klasse
	 */
	private static VerwaltungWvzInhalte _instance = null;

	/**
	 * Zuordnung der Bildinhalten zu den WvzInhalten in Abhngigkeit vom
	 * Anzeigetyp. Hierzu werden 2 in einander verschachtelte Hashmaps verwendet.
	 * Die innere Hashmap speichert die Zuordnung Bildinhalt zu Pid des
	 * WvzInhalts, die uere Hashamp den Bezug zum Anzeigetyp.
	 */
	private HashMap<EAnzeigeTypen, HashMap<AnzeigeInhaltTyp, String>> _zuordnungProAnzeigeTyp = new HashMap<EAnzeigeTypen, HashMap<AnzeigeInhaltTyp, String>>();

	/**
	 * Hashmap mit der Zuordnung WvzInhalt -> Bildinhalt. Als Key wird die Pid des
	 * WVZInhalts verwendet, als Value ein Objekt vom Typ AnzeigeInhaltTyp. Diese
	 * Hashmap wird fr den schnellen Zugriff auf die Bildinhalte in Abhngigkeit
	 * von der Pid des WvzInhalts bentigt.
	 */
	private ConcurrentHashMap<String, AnzeigeInhaltTyp> _zuordnungPidWvzInhaltZuBildInhalt = new ConcurrentHashMap<String, AnzeigeInhaltTyp>();

	/**
	 * Hasmap zum Speichern welche Bildinhalte bei welchem Anzeigetyp geschaltet
	 * werden knnen.
	 */
	private ConcurrentHashMap<EAnzeigeTypen, Vector<AnzeigeInhaltTyp>> _zuordnungNormal = new ConcurrentHashMap<EAnzeigeTypen, Vector<AnzeigeInhaltTyp>>();

	private ConcurrentHashMap<EAnzeigeTypen, Vector<AnzeigeInhaltTyp>> _zuordnungNegiert = new ConcurrentHashMap<EAnzeigeTypen, Vector<AnzeigeInhaltTyp>>();

	/**
	 * Eigenschaften der Anzeigetypen
	 */
	private Vector<AnzeigeTypEigenschaften> _anzeigeTypenEigenschaften;

	/**
	 * Pid des Anzeigetyp unabhngigen Bildinhalts "Beliebig"
	 */
	private String _pidBeliebig;

	/**
	 * Pid des Anzeigetyp unabhngigen Bildinhalts "Don't care"
	 */
	private String _pidDontCare;

	/**
	 * Pid des Anzeigetyp unabhngigen Bildinhalts "Beibehalten"
	 */
	private String _pidBeibehalten;

	/**
	 * WVZ-Inhalt "Beliebig"
	 */
	private WVZInhalt _wvzInhaltBeliebig;

	/**
	 * WVZ-Inhalt "Don't care"
	 */
	private WVZInhalt _wvzInhaltDontCare;

	/**
	 * WVZ-Inhalt "Beibehalten"
	 */
	private WVZInhalt _wvzInhaltBeibehalten;

	/**
	 * Methode liefert die einzige Instanz der Klasse
	 * 
	 * @return einzige Instanz der Klasse
	 */
	public static VerwaltungWvzInhalte getInstance()
	{
		if (_instance == null)
		{
			synchronized (VerwaltungWvzInhalte.class)
			{
				if (_instance == null)
					_instance = new VerwaltungWvzInhalte();
			}
		}

		return _instance;
	}

	/**
	 * Privater Konstruktor der Klasse
	 */
	private VerwaltungWvzInhalte()
	{
		initialisiere();
	}

	/**
	 * Methode initialisiert die bentigten Datenstrukturen
	 */
	private void initialisiere()
	{
		// Bestimmen der speziellen WVZ-Inhalte

		bestimmeSpezialInhalte();

		// Anzeigetypen initialisieren

		_anzeigeTypenEigenschaften = bestimmeAnzeigeTypen();

		// WvzInhalte initialisieren

		bestimmeWvzInhalte();
	}

	/**
	 * Methode zur Definition der speziellen WVZ-Inhalt wie Beibehalten, Beliebig,
	 * Don't care, etc.
	 */
	private void bestimmeSpezialInhalte()
	{
		VerwaltungSpezialWVZInhalte v = VerwaltungSpezialWVZInhalte
		    .getInstance();

		// Beibehalten

		SystemObject so = v.getBeibehalten();
		if (so != null)
		{
			_pidBeibehalten = so.getPid();
			_wvzInhaltBeibehalten = new WVZInhalt(_pidBeibehalten);
			bestimmeBildInhalt(_pidBeibehalten);
		}

		// Beliebig

		so = v.getBeliebig();
		if (so != null)
		{
			_pidBeliebig = so.getPid();
			_wvzInhaltBeliebig = new WVZInhalt(_pidBeliebig);
			bestimmeBildInhalt(_pidBeliebig);
		}

		// Dont't care

		so = v.getDontCare();
		if (so != null)
		{
			_pidDontCare = so.getPid();
			_wvzInhaltDontCare = new WVZInhalt(_pidDontCare);
			bestimmeBildInhalt(_pidDontCare);
		}
	}

	/**
	 * Methode bestimmt die bentigten WVZ-Inhalte
	 */
	private void bestimmeWvzInhalte()
	{
		/*
		 * Loop ber die bentigten Typen
		 */
		for (AnzeigeTypEigenschaften ate : _anzeigeTypenEigenschaften)
		{
//			System.out.println("---> VerwaltungWvzInhalte.bestimmeWvzInhalte() " + ate.getAnzeigeTyp());
			
					
			Vector<AnzeigeInhaltTyp> normal  = new Vector<AnzeigeInhaltTyp>();
			Vector<AnzeigeInhaltTyp> negiert = new Vector<AnzeigeInhaltTyp>();

			Vector<SystemObject> inhalte = ate.getInhalte();

			// getWvzInhalte (ate.getTyp());

			if (!_zuordnungProAnzeigeTyp.containsKey(ate.getTyp()))
				_zuordnungProAnzeigeTyp.put(ate.getTyp(),
				    new HashMap<AnzeigeInhaltTyp, String>());

			HashMap<AnzeigeInhaltTyp, String> aktuell = _zuordnungProAnzeigeTyp
			    .get(ate.getTyp());

			for (SystemObject soWvzInhalt : inhalte)
			{
				String pidWvzInhalt  = soWvzInhalt.getPid();
				AnzeigeInhaltTyp ait = bestimmeBildInhalt(pidWvzInhalt);

//				System.out.println("VerwaltungWvzInhalte.bestimmeWvzInhalte() " + pidWvzInhalt);
				
				if (ait != null)
				{
					if (!ait.isNegiert())
						normal.add(ait);
					else
						negiert.add(ait);

					aktuell.put(ait, pidWvzInhalt);
				}
			}

			_zuordnungNormal.put(ate.getTyp(), normal);
			_zuordnungNegiert.put(ate.getTyp(), negiert);
		}
	}

	/**
	 * Methode bestimmt die bentigten Anzeigetypen
	 * 
	 * @return Liste mit den Pids der bentigten Anzeigetypen
	 */
	private Vector<AnzeigeTypEigenschaften> bestimmeAnzeigeTypen()
	{
		Vector<AnzeigeTypEigenschaften> eigenschaften = null;

		// Lesen des Parameters aus dem DaV

		eigenschaften = VerwaltungAnzeigeTypMitWVZInhalt.getInstance()
		    .getEigenschaften();

		// Pids konnten nicht aus DaV gelesen werden
		// --> Projektspezifische Einstellungen vornehmen

		if (eigenschaften == null)
			eigenschaften = DefaultEinstellungen.getDefaultAnzeigeTypen();

		return eigenschaften;
	}

	/**
	 * Methode liefert den AnzeigeInhaltTyp zu einem WvzInhalt
	 * 
	 * @param pidWvzInhalt
	 *          Pid des WvzInhalts
	 * @return AnzeigeInhaltTyp
	 */
	public AnzeigeInhaltTyp getAnzeigeInhaltTyp(String pidWvzInhalt)
	{
//		System.out.println("VerwaltungWvzInhalte.getAnzeigeInhaltTyp() ---> " + pidWvzInhalt);
		
		if (_zuordnungPidWvzInhaltZuBildInhalt.containsKey(pidWvzInhalt))
			return _zuordnungPidWvzInhaltZuBildInhalt.get(pidWvzInhalt);

		return null;
	}

	/**
	 * Methode bestimmt den Bildinhalt zu diesem WVZ-Inhalt
	 * 
	 * @param pidWvzInhalt
	 *          Pid des WVZ-Inhalts
	 * @return Bildinhalt
	 */
	private AnzeigeInhaltTyp bestimmeBildInhalt(String pidWvzInhalt)
	{
		AnzeigeInhaltTyp ait = null;
		String inhalt = null;

		ClientDavInterface connection = DaVVerbindung.getInstanz()
		    .getDaVVerbindung();
		DataModel dataModel = connection.getDataModel();

		SystemObject so = dataModel.getObject(pidWvzInhalt);
		if (so != null)
		{
			// Konfiguration auslesen

			AttributeGroup atg = dataModel.getAttributeGroup("atg.wvzInhalt");
			if (atg != null)
			{
				Data confData = so.getConfigurationData(atg);

				if (confData != null)
				{
					inhalt = confData.getTextValue("Bildinhalt").getText();
				}
			}
		}

		if (inhalt != null)
		{
			ait = AnzeigeInhaltTyp.getTyp(inhalt);

			if (ait != null)
			{
//				System.out.println("VerwaltungWvzInhalte.bestimmeBildInhalt() " + pidWvzInhalt + " --> " + ait);
				
				_zuordnungPidWvzInhaltZuBildInhalt.put(pidWvzInhalt, ait);
			}
		}

		return ait;
	}

	/**
	 * Methode liefert die Bildinhalte zu einem Anzeigetyps
	 * 
	 * @param typ
	 *          Anzeigetyp
	 * @param mitBeliebig
	 *          mit dem Inhalt beliebig
	 * @param mitDontCare
	 *          mit dem Inhalt don't care
	 * @param mitNegiertenInhalten
	 *          mit den negierten Inhalten
	 * @return Liste mit Bildinhalten
	 */
	public List<AnzeigeInhaltTyp> getAnzeigeInhaltTypen(AnzeigeElementTyp typ,
	    boolean mitBeliebig, boolean mitDontCare, boolean mitNegiertenInhalten)
	{
		List<AnzeigeInhaltTyp> l = new ArrayList<AnzeigeInhaltTyp>();

		if (mitBeliebig)
			l.add(AnzeigeInhaltTyp.BELIEBIG);

		if (mitDontCare)
			l.add(AnzeigeInhaltTyp.DONT_CARE);

		if (_zuordnungNormal.containsKey(typ.getTyp()))
			l.addAll(_zuordnungNormal.get(typ.getTyp()));

		if (mitNegiertenInhalten)
		{
			if (_zuordnungNegiert.containsKey(typ.getTyp()))
				l.addAll(_zuordnungNegiert.get(typ.getTyp()));
		}

		return l;
	}

	public List<AnzeigeInhaltTyp> getAlleBildinhalte(boolean mitBeliebig,
	    boolean mitDontCare, boolean mitNegiertenInhalten)
	{
		List<AnzeigeInhaltTyp> l = new ArrayList<AnzeigeInhaltTyp>();

		if (mitBeliebig)
			l.add(AnzeigeInhaltTyp.BELIEBIG);

		if (mitDontCare)
			l.add(AnzeigeInhaltTyp.DONT_CARE);

		for (AnzeigeElementTyp typ : AnzeigeElementTyp.getVersorgteTypen())
		{
			List<AnzeigeInhaltTyp> inhalte = getAnzeigeInhaltTypen(typ, false, false,
			    mitNegiertenInhalten);
			for (AnzeigeInhaltTyp inhalt : inhalte)
				l.add(inhalt);
		}

		return l;
	}

	/**
	 * Methode liefert eine Liste mit Systemobjekten vom Typ 'typ.wvzInhalt'.
	 * Diese Liste enthlt alle WVZ-Inhalte die als zu prfende Bilder in Frage
	 * kommen.
	 * 
	 * @return Liste mit WVZ-Inhalten
	 */
	public List<SystemObject> getZuPruefendeWvzInhalte()
	{
		boolean mitBeliebig = true;
		boolean mitDontCare = false;
		boolean mitBeibehalten = false;
		boolean mitNegiert = true;

		return getSystemObjekteWvzInhalte(mitBeliebig, mitDontCare, mitBeibehalten,
		    mitNegiert);
	}

	/**
	 * Methode liefert eine Liste mit Systemobjekten vom Typ 'typ.wvzInhalt'.
	 * Diese Liste enthlt alle WVZ-Inhalte die als zu schaltende Bilder in Frage
	 * kommen.
	 * 
	 * @return Liste mit Systemobjekten
	 */
	public List<SystemObject> getZuSchaltendeWvzInhalte()
	{
		boolean mitBeliebig = false;
		boolean mitDontCare = false;
		boolean mitBeibehalten = true;
		boolean mitNegiert = false;

		return getSystemObjekteWvzInhalte(mitBeliebig, mitDontCare, mitBeibehalten,
		    mitNegiert);
	}

	/**
	 * Methode liefert eine Liste mit Systemobjekten vom Typ 'typ.wvzInhalt' die
	 * bei einem bestimmten Anzeigetyp geschaltet werden knnen.
	 * 
	 * @param anzeigeTyp
	 *          {@link SystemObjekteEreignis} vom Typ 'typ.sswSbaAbzeigeTyp'
	 * @return Liste mit Systemobjekten
	 */
	public List<SystemObject> getZuSchaltendeWvzInhalte(SystemObject anzeigeTyp)
	{
		boolean mitBeliebig = false;
		boolean mitDontCare = false;
		boolean mitBeibehalten = true;
		boolean mitNegiert = false;

		return getSystemObjekteWvzInhalte(anzeigeTyp, mitBeliebig, mitDontCare,
		    mitBeibehalten, mitNegiert);
	}

	/**
	 * Methode liefert eine Liste mit Systemobjekten vom Typ 'typ.wvzInhalt'.
	 * 
	 * @param mitBeliebig
	 *          mit WVZ-Inhalt 'beliebig'
	 * @param mitDontCare
	 *          mit WVZ-Inhalt 'don't care'
	 * @param mitBeibehalten
	 *          mit WVZ-Inhalt 'beibehalten'
	 * @param mitNegiertenInhalten
	 *          mit negierten WVZ-Inhalten
	 * @return Liste mit Systemobjekten
	 */
	public List<SystemObject> getSystemObjekteWvzInhalte(boolean mitBeliebig,
	    boolean mitDontCare, boolean mitBeibehalten, boolean mitNegiertenInhalten)
	{
		List<SystemObject> l = new ArrayList<SystemObject>();

		if (mitBeliebig)
			l.add(Konfig.getInstance().getSystemObject(_pidBeliebig));

		if (mitDontCare)
			l.add(Konfig.getInstance().getSystemObject(_pidDontCare));

		if (mitBeibehalten)
			l.add(Konfig.getInstance().getSystemObject(_pidBeibehalten));

		VerwaltungAnzeigeTypMitWVZInhalt v = VerwaltungAnzeigeTypMitWVZInhalt
		    .getInstance();
		Vector<AnzeigeTypEigenschaften> eigenschaften = v.getEigenschaften();
		for (AnzeigeTypEigenschaften ate : eigenschaften)
		{
			Vector<SystemObject> objekte = ate.getInhalte();
			for (SystemObject so : objekte)
			{
				if (l != null && !l.contains(so))
				{
					AnzeigeInhaltTyp inhalt = getAnzeigeInhaltTyp(so.getPid());

					if ((inhalt != null) && (inhalt.isNegiert()))
					{
						if (mitNegiertenInhalten)
							l.add(so);
					} else
					{
						l.add(so);
					}
				}
			}
		}

		return l;
	}

	/**
	 * Methode liefert eine Liste mit Systemobjekten vom Typ 'typ.wvzInhalt'.
	 * 
	 * @param anzeigeTyp
	 *          {@link SystemObjekteEreignis} vom Typ 'typ.sswSbaAbzeigeTyp'
	 * @param mitBeliebig
	 *          mit WVZ-Inhalt 'beliebig'
	 * @param mitDontCare
	 *          mit WVZ-Inhalt 'don't care'
	 * @param mitBeibehalten
	 *          mit WVZ-Inhalt 'beibehalten'
	 * @param mitNegiertenInhalten
	 *          mit negierten WVZ-Inhalten
	 * @return Liste mit Systemobjekten
	 */
	public List<SystemObject> getSystemObjekteWvzInhalte(SystemObject anzeigeTyp,
	    boolean mitBeliebig, boolean mitDontCare, boolean mitBeibehalten,
	    boolean mitNegiertenInhalten)
	{
		List<SystemObject> l = new ArrayList<SystemObject>();

		if (mitBeliebig)
			l.add(Konfig.getInstance().getSystemObject(_pidBeliebig));

		if (mitDontCare)
			l.add(Konfig.getInstance().getSystemObject(_pidDontCare));

		if (mitBeibehalten)
			l.add(Konfig.getInstance().getSystemObject(_pidBeibehalten));

		AnzeigeElementTyp typ = AnzeigeElementTyp.getTypBySystemObject(anzeigeTyp);
		if (typ != null)
		{
			VerwaltungAnzeigeTypMitWVZInhalt v = VerwaltungAnzeigeTypMitWVZInhalt
			    .getInstance();
			AnzeigeTypEigenschaften ate = v.getEigenschaft(typ);
			if (ate != null)
			{
				Vector<SystemObject> objekte = ate.getInhalte();
				for (SystemObject so : objekte)
				{
					if (l != null && !l.contains(so))
					{
						AnzeigeInhaltTyp inhalt = getAnzeigeInhaltTyp(so.getPid());

						if ((inhalt != null) && (inhalt.isNegiert()))
						{
							if (mitNegiertenInhalten)
								l.add(so);
						} else
						{
							l.add(so);
						}
					}
				}
			}
		}

		return l;
	}

	/**
	 * Methode liefert die Pid zu einem Anzeigetyp
	 * 
	 * @param typ
	 *          Typ
	 * @return Pid des Anzeigetyps
	 */
	public String getPidAnzeigeTyp(EAnzeigeTypen typ)
	{
		for (AnzeigeTypEigenschaften ate : _anzeigeTypenEigenschaften)
			if (ate.getTyp() == typ)
				return ate.getAnzeigeTyp().getPid();

		return null;
	}

	/**
	 * Methode liefert das Defaultbild zu einem Anzeigetyp
	 * 
	 * @param typ
	 *          Anzeigetyp
	 * @return Defaultbild
	 */
	public AnzeigeInhaltTyp getDefaultBild(EAnzeigeTypen typ)
	{
		for (AnzeigeTypEigenschaften ate : _anzeigeTypenEigenschaften)
			if (ate.getTyp() == typ)
			{
				String pid = ate.getDefaultBild().getPid();

//				System.out.println("VerwaltungWvzInhalte.getDefaultBild() " + pid);
				
				if (pid != null)
					return _zuordnungPidWvzInhaltZuBildInhalt.get(pid);
			}

		return null;
	}

	/**
	 * Methode liefert die Information ob die bergebene Pid (Pid vom Typ
	 * typ.wvzInhalt) dem Bildinhalt "Dunkel" entspricht.
	 * 
	 * @param pidWvzInhalt
	 *          Pid des zu prfenden WVZ-Inhalts
	 * @return Ergebnis der Prfung
	 */
	public boolean isDunkel(String pidWvzInhalt)
	{
		if (_zuordnungPidWvzInhaltZuBildInhalt.containsKey(pidWvzInhalt))
		{
			AnzeigeInhaltTyp ait = _zuordnungPidWvzInhaltZuBildInhalt
			    .get(pidWvzInhalt);
			return ait == AnzeigeInhaltTyp.DUNKEL;
		}

		return false;
	}

	/**
	 * Methode liefert die Information ob die bergebene Pid (Pid vom Typ
	 * typ.wvzInhalt) dem Bildinhalt "Beibehalten" entspricht.
	 * 
	 * @param pidWvzInhalt
	 *          Pid des zu prfenden WVZ-Inhalts
	 * @return Ergebnis der Prfung
	 */
	public boolean isBeibehalten(String pidWvzInhalt)
	{
		if (_zuordnungPidWvzInhaltZuBildInhalt.containsKey(pidWvzInhalt))
		{
			AnzeigeInhaltTyp ait = _zuordnungPidWvzInhaltZuBildInhalt
			    .get(pidWvzInhalt);
			return ait == AnzeigeInhaltTyp.BEIBEHALTEN;
		}

		return false;
	}

	/**
	 * Methode liefert die Information ob die bergebene Pid (Pid vom Typ
	 * typ.wvzInhalt) dem Bildinhalt "Beliebig" entspricht.
	 * 
	 * @param pidWvzInhalt
	 *          Pid des zu prfenden WVZ-Inhalts
	 * @return Ergebnis der Prfung
	 */
	public boolean isBeliebig(String pidWvzInhalt)
	{
		if (_zuordnungPidWvzInhaltZuBildInhalt.containsKey(pidWvzInhalt))
		{
			AnzeigeInhaltTyp ait = _zuordnungPidWvzInhaltZuBildInhalt
			    .get(pidWvzInhalt);
			return ait == AnzeigeInhaltTyp.BELIEBIG;
		}

		return false;
	}

	/**
	 * Methode liefert die Pid des WVZ-Inhalts die innerhalb eines Anzeigetyps zu
	 * einem Bildinhalt gehrt
	 * 
	 * @param anzeigeTyp
	 *          Anzeigetyp
	 * @param inhalt
	 *          Bildinhalt
	 * @return Pid
	 */
	public String getPidWvzInhalt(AnzeigeElementTyp anzeigeTyp,
	    AnzeigeInhaltTyp inhalt)
	{
		if (inhalt == AnzeigeInhaltTyp.BELIEBIG)
			return _pidBeliebig;

		if (inhalt == AnzeigeInhaltTyp.BEIBEHALTEN)
			return _pidBeibehalten;

		if (inhalt == AnzeigeInhaltTyp.DONT_CARE)
			return _pidDontCare;

		EAnzeigeTypen typ = anzeigeTyp.getTyp();

		HashMap<AnzeigeInhaltTyp, String> versorgt = _zuordnungProAnzeigeTyp
		    .get(typ);

		if (versorgt != null)
			return versorgt.get(inhalt);

		return null;
	}

	/**
	 * Methode liefert beliebig zurck
	 * 
	 * @return beliebig
	 */
	public WVZInhalt getBeliebig()
	{
		return _wvzInhaltBeliebig;
	}

	/**
	 * Methode liefert wvzInhaltDontCare zurck
	 * 
	 * @return wvzInhaltDontCare
	 */
	public WVZInhalt getWvzInhaltDontCare()
	{
		return _wvzInhaltDontCare;
	}

	/**
	 * Methode setzt die Variable wvzInhaltDontCare
	 * 
	 * @param wvzInhaltDontCare
	 *          zu bernehmender Wert fr wvzInhaltDontCare
	 */
	public void setWvzInhaltDontCare(WVZInhalt wvzInhaltDontCare)
	{
		_wvzInhaltDontCare = wvzInhaltDontCare;
	}

	/**
	 * Methode liefert einen WVZ-Inhalt zu einem Bildinhalt in Abhngigkeit des
	 * Anzeigetyps
	 * 
	 * @param anzeigeTyp
	 *          Anzeigetyp
	 * @param bildInhalt
	 *          Bildinhalt
	 * @return WvzInhalt
	 */
	public WVZInhalt getWvzInhalt(AnzeigeElementTyp anzeigeTyp, String bildInhalt)
	{
		AnzeigeInhaltTyp ait = AnzeigeInhaltTyp.getTyp(bildInhalt);
		
//		System.out.println("VerwaltungWvzInhalte.getWvzInhalt() " + anzeigeTyp + " " + bildInhalt + 
//				" " + ait);

		if (ait == AnzeigeInhaltTyp.BELIEBIG)
			return getWvzInhaltBeliebig();

		if (ait == AnzeigeInhaltTyp.BEIBEHALTEN)
			return getWvzInhaltBeibehalten();

		if (ait == AnzeigeInhaltTyp.DONT_CARE)
			return getWvzInhaltDontCare();

		String pidWvzInhalt = getPidWvzInhalt(anzeigeTyp, ait);
		if (pidWvzInhalt != null)
			return new WVZInhalt(pidWvzInhalt);

		return null;
	}

	/**
	 * Methode liefert einen Vektor mit WvzInhalten der den bergebenen
	 * Bildinhalten entspricht
	 * 
	 * @param inhalte
	 *          Vector mit Bildinhalten
	 * @return Vektor mit WvzInhalten
	 */
	public Vector<WVZInhalt> getWvzInhalte(Vector<InhaltMitTyp> inhalte)
	{
		Vector<WVZInhalt> v = new Vector<WVZInhalt>();

		for (InhaltMitTyp imt : inhalte)
			v.add(getWvzInhalt(imt));

		return v;
	}

	/**
	 * Methode liefert einen WvzInhalt zu einem Bildinhalt. Als bergabeparameter
	 * wird eine Instanz der Klasse {@link InhaltMitTyp} verwendet.
	 * 
	 * @param inhaltMitTyp
	 *          Bildinhalt
	 * @return WvzInhalt
	 */
	public WVZInhalt getWvzInhalt(InhaltMitTyp inhaltMitTyp)
	{
		AnzeigeElementTyp anzeigeTyp = inhaltMitTyp.getTyp();

		String bildInhalt = inhaltMitTyp.getInhalt();

		return getWvzInhalt(anzeigeTyp, bildInhalt);
	}

	/**
	 * Methode liefert einen WvzInhalt der dem bergeben Bildinhalt entspricht
	 * 
	 * @param anzeigeTyp
	 *          Typ der Anzeige
	 * @param inhalt
	 *          Bildinhalt
	 * @return WvzInhalt
	 */
	public WVZInhalt getWvzInhalt(AnzeigeElementTyp anzeigeTyp,
	    AnzeigeInhaltTyp inhalt)
	{
		if (anzeigeTyp != null && inhalt != null)
			return getWvzInhalt(anzeigeTyp, inhalt.getBildInhalt());

		return null;
	}

	/**
	 * Methode liefert das Defaultbild zu einem Anzeigetyp
	 * 
	 * @param anzeigeTyp
	 *          Anzeigetyp
	 * @return Defaultbild
	 */
	public WVZInhalt getDefaultWvzInhalt(AnzeigeElementTyp anzeigeTyp)
	{
		AnzeigeInhaltTyp defaultBild = anzeigeTyp.getDefaultBild();
		String pidWvzInhalt = VerwaltungWvzInhalte.getInstance().getPidWvzInhalt(
		    anzeigeTyp, defaultBild);

//		System.out.println("VerwaltungWvzInhalte.getDefaultWvzInhalt() " + anzeigeTyp + " " 
//				+ pidWvzInhalt);
		
		return getWvzInhalt(anzeigeTyp, pidWvzInhalt);
	}

	/**
	 * Methode liefert wvzInhaltBeibehalten zurck
	 * 
	 * @return wvzInhaltBeibehalten
	 */
	public WVZInhalt getWvzInhaltBeibehalten()
	{
		return _wvzInhaltBeibehalten;
	}

	/**
	 * Methode liefert wvzInhaltBeliebig zurck
	 * 
	 * @return wvzInhaltBeliebig
	 */
	public WVZInhalt getWvzInhaltBeliebig()
	{
		return _wvzInhaltBeliebig;
	}

	/**
	 * Methode prft ob zu einem Bildinhalt ein WvzInhalt definiert ist
	 * 
	 * @param typ
	 *          Typ
	 * @param inhalt
	 *          Bildinhalt
	 * @return Ergebnis der Prfung
	 */
	public boolean isVersorgt(AnzeigeElementTyp typ, AnzeigeInhaltTyp inhalt)
	{
		return getWvzInhalt(typ, inhalt) != null;
	}

	/**
	 * Methode liefert eine Liste mit Systemobjekten vom Typ
	 * (typ.sswSbaAnzeigeTyp)
	 * 
	 * @return Liste mit Systemobjekten
	 */
	public List<SystemObject> getAnzeigeTypen()
	{
		List<SystemObject> l = new ArrayList<SystemObject>();

		VerwaltungAnzeigeTypMitWVZInhalt v = VerwaltungAnzeigeTypMitWVZInhalt
		    .getInstance();
		Vector<AnzeigeTypEigenschaften> eigenschaften = v.getEigenschaften();
		for (AnzeigeTypEigenschaften ate : eigenschaften)
		{
			if (ate.getAnzeigeTyp() != null)
				l.add(ate.getAnzeigeTyp());
		}

		return l;
	}
}
