package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallandereszeichen;

/**
 * Definition der zu schaltenden Anzeigen
 * @author Swarco Traffic Systems GmbH
 */
public enum SelektionAnzeige
{
  /**
   * Keine Anzeige
   */
  KEINE ("Keine", 0),
  
  /**
   * Aktuelle Anzeige
   */
  AKTUELLE ("Aktuelle", 1),
  
  /**
   * Alle Anzeigen gleichen Typs
   */
  ALLE ("Alle", 2);
  
  private int m_kodierung;
  
  /**
   * Bezeichnung
   */
	private String m_bezeichnung;
  
  /**
   * Konstruktor der Klasse
   * @param kodierung Kodierung der Enum-Werte im DaV
   */
  SelektionAnzeige (String bezeichnung, int kodierung)
  {
  	m_bezeichnung = bezeichnung;
  	
    m_kodierung = kodierung;  
  }
  
  /**
   * Methode liefert den Wert der zu einer Kodierung im DaV gehrt
   * @param kodierung Kodierung deren Wert gesucht wird
   * @return Wert, bei einer falschen Kodierung: 'keineAnzeige'
   */
  public static SelektionAnzeige getValue (int kodierung)
  {
    for (SelektionAnzeige s: SelektionAnzeige.values())
    {
      if (s.m_kodierung == kodierung)
        return s;
    }
    
    return KEINE;
  }

  /**
   * Methode liefert die Variable m_kodierung zurck
   * @return Variable m_kodierung
   */
  public int getKodierung()
  {
    return m_kodierung;
  }

	/**
   * Methode liefert bezeichnung zurck
   * @return bezeichnung
   */
  public String getBezeichnung()
  {
  	return m_bezeichnung;
  }

  /**
   * Methode liefert die Selektion, die zu einer Bezeichnung passe
   * @param bezeichnung Bezeichnung der Selektion
   * @return Selektion
   */
	public static SelektionAnzeige getSelektion(String bezeichnung)
  {
		for (SelektionAnzeige sa : values())
			if (sa.getBezeichnung().equals(bezeichnung))
				return sa;
		
	  return null;
  }
};
