package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausschaltregel;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Dambach Werke GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private Vector<RegelAusschaltRegelNeu> m_parameter = new Vector<RegelAusschaltRegelNeu> ();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public ListeMitRegeln ()
  {
  }
  
  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelAusschaltRegelNeu> getParameter()
  {
    Vector<RegelAusschaltRegelNeu> v = new Vector<RegelAusschaltRegelNeu>();
    synchronized (m_parameter)
    {
      for (RegelAusschaltRegelNeu r : m_parameter)
        v.add( r );
    }
    return v;
  }

  /**
   * Methode setzt die Variable parameter
   * @param parameter Wert fr parameter
   */
  public void setParameter(Vector<RegelAusschaltRegelNeu> parameter)
  {
    synchronized (m_parameter)
    {
      m_parameter = parameter;
    } 
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaRegelAusschaltRegel'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    Array a = data.getArray( "Regeln" );
    int anzahl = a.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelAusschaltRegel r = new RegelAusschaltRegel ();
        r.auswerteAttributgruppe( a.getItem( x ), schaltgrundAllgemein );
        
        m_parameter.addAll( umwandleParameter( r ));
      }
    }
  }
  
  /**
   * Methode wandelt einen Parametersatz in das neue Format um
   * @param regel Regel die umgewandelt werden soll
   * @return Vektor mit Regeln im neuen Format
   */
  public Vector<RegelAusschaltRegelNeu> umwandleParameter ( RegelAusschaltRegel regel )
  {
  	Vector<RegelAusschaltRegelNeu> v = new Vector<RegelAusschaltRegelNeu>();
  	
  	WVZInhalt x = regel.getAnzeigenInhalt();
  	Vector<FuellRegel> regeln = regel.getFuellRegeln();
  	
  	for (FuellRegel fr : regeln)
  	{
  		SystemObject a = fr.getAnzeigeTyp();
  		WVZInhalt y = fr.getAngeforderterInhalt();
  		int z = fr.getPrioritaet();
  		Aspect schaltgrund = regel.getSchaltgrundRegel();
  		
  		v.add (new RegelAusschaltRegelNeu (x, a, y, z, schaltgrund));
  	}
  	
  	return v;
  }

  /**
   * Methode fgt eine Regel hinzu
   * @param x zu prfende Anzeige
   * @param a Anzeigetyp
   * @param y zu schaltende Anzeige
   * @param z Prioritt
   * @param schaltgrund Schaltgrund
   */
  public void addRegel (WVZInhalt x, SystemObject a,  WVZInhalt y, int z, Aspect schaltgrund )
  {
    synchronized (m_parameter)
    {
    	RegelAusschaltRegelNeu r = new RegelAusschaltRegelNeu (x, a, y, z, schaltgrund);
      m_parameter.add( r );
    }
  }

  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      for (RegelAusschaltRegelNeu r : m_parameter)
        s += r.toString() + "\n";
    }
    
    return s;
  }
}
