package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.bedienung;

import java.util.List;
import java.util.TreeMap;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeTypEigenschaften;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.DefaultEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.EAnzeigeTypen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Composite zum Versorgen der Parameter Bedienung - "Anzeige Typen mit WVZ-Inhalten" 
 * @author ObertM
 *
 */
public class CompositeAnzeigeTypen extends Composite
{
	/**
	 * Kennung undefiniert
	 */
	private final static String _undefiniert = "undefiniert";

	/**
	 * Liste mit den mglichen WVZ-Inhalten
	 */
	private List<SystemObject> _wvzInhalte = new Vector<SystemObject>();

  /**
   * Liste mit den mglichen Anzeigetypen	
   */
	private List<SystemObject> _anzeigeTypen;

	/**
	 * Parameter Anzeigequerschnitte
	 */
	private ConcurrentHashMap<EAnzeigeTypen, AnzeigeTypEigenschaften> _parameter = new ConcurrentHashMap<EAnzeigeTypen, AnzeigeTypEigenschaften>();

	/**
	 * Combobox Anzeigetyp
	 */
	private Combo _comboAnzeigeTyp;

	/**
	 * Combobox Systemobjekt
	 */
	private Combo _comboSystemObjekt;

	/**
	 * Combobox defaultbild
	 */
	private Combo _comboDefaultBild;

	/**
	 * Feld mit den Bezeichnungen der WVZ-Inhalte
	 */
	private String[] _wvzInhalteAlsText;

	/**
	 * Feld mit den Bezeichnungen der Anzeigetypen
	 */
	private String[] _anzeigeTypenAlsText;

	/**
	 * Aktuell eingestellter Anzeigetyp
	 */
	private EAnzeigeTypen _aktuellerTyp;

	/**
	 * Zuordnung der Rckmeldungen
	 */
	private CompositeZuordnung _zuordnung;

  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil
   * @param parameter aktuelle Parameter
   */
	CompositeAnzeigeTypen(Composite parent, int style, ParameterBedienung parameter) 
  {
    super(parent, style);
    
    // Defaultparameter fr alle Anzeigetypen anlegen
    
    for (EAnzeigeTypen eat : EAnzeigeTypen.values())
    	_parameter.put(eat, new AnzeigeTypEigenschaften (eat));
    
    // bisherige Parameter auswerten
    
    for (AnzeigeTypEigenschaften ate : parameter.getAnzeigeTypen())
    	_parameter.put (ate.getTyp(), ate.getCopy());

    // bentigte Objekte bestimmen
    
    bestimmeMoeglicheObjekte ();

    initialize();
    
    aendereTyp();
  }

	/**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;

    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);

		GridData gridDataCombo = new GridData();
		gridDataCombo.horizontalAlignment = GridData.FILL;
		gridDataCombo.grabExcessHorizontalSpace = true;

    /*
     * Combobox mit der Auswahlmglichkeit des zugeordneten Systemobjekts
     */
		int anzahl = EAnzeigeTypen.values().length;
		String[] sa = new String[anzahl];
		for (int j = 0; j < anzahl; ++j)
			sa[j] = EAnzeigeTypen.values()[j].toString();
		
		ComboMitText cmt1 = new ComboMitText(this, SWT.BORDER);
		cmt1.setLayoutData(gridDataCombo);
		cmt1.initGUI("Anzeigetyp", sa);

    _comboAnzeigeTyp = cmt1.getCombo();
		_comboAnzeigeTyp.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				aendereTyp ();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});

    /*
     * Combobox mit der Auswahlmglichkeit des zugeordneten Systemobjekts
     */
		ComboMitText cmt2 = new ComboMitText(this, SWT.BORDER);
		cmt2.setLayoutData(gridDataCombo);
		cmt2.initGUI("zugeordnetes Systemobjekt:", _anzeigeTypenAlsText);
		
    _comboSystemObjekt = cmt2.getCombo();

    /*
     * Combobox mit dem Defaultbild
     */
		ComboMitText cmt3 = new ComboMitText(this, SWT.BORDER);
		cmt3.setLayoutData(gridDataCombo);
		cmt3.initGUI("Defaultbild:", _wvzInhalteAlsText);
		   
    _comboDefaultBild = cmt3.getCombo();

    /*
     * Composite mit der Zuordnung der WVZ-Anzeigen
     */
		GridData gridDataZuordnung = new GridData();
		gridDataZuordnung.horizontalAlignment = GridData.FILL;
		gridDataZuordnung.grabExcessHorizontalSpace = true;
		gridDataZuordnung.verticalAlignment = GridData.FILL;
		gridDataZuordnung.grabExcessVerticalSpace = true;

  	_zuordnung = new CompositeZuordnung(this, SWT.BORDER, "mgliche WVZ-Inhalte:", "zugeordnete WVZ-Inhalte:");
  	_zuordnung.setLayoutData(gridDataZuordnung);
    
  	/*
  	 * Composite mit dem Defaultbutton 
  	 */
    createCompositeDefaultButton (this);
  }

  /**
   * Methode bestimmt die im Datenverteiler definierten Objekte
   */
	private void bestimmeMoeglicheObjekte()
  {
		// WVZ-Inhalte
		
		_wvzInhalte = Tools.getSystemObjekte("typ.wvzInhalt");
		_wvzInhalteAlsText = bestimmeNamen (_wvzInhalte, true);
		
		// AnzeigeTypen
		
		_anzeigeTypen = Tools.getSystemObjekte("typ.sswSbaAnzeigeTyp");
		_anzeigeTypenAlsText = bestimmeNamen (_anzeigeTypen, true);
  }

	/**
	 * Methode bestimmt die Namen der anzuzeigenden Systemobjekte
	 * @param objekte Systemobjekte
	 * @return Liste mit den Namen
	 */
	private String[] bestimmeNamen (java.util.List<SystemObject> objekte, boolean mitUndefiniert)
  {
		TreeMap<String, SystemObject> sort = new TreeMap<String, SystemObject>();
		for (SystemObject so : objekte)
			sort.put(so.toString(), so);
		
		int anzahl = sort.size();
		if (mitUndefiniert)
		  anzahl++;
		
		String a [] = new String [anzahl];
		
		int index = 0;
		if (mitUndefiniert)
			a [index++] = _undefiniert;

		for (String s : sort.keySet())
			a [index++] = s;
		
		return a;
  }

	/**
	 * Methode initialisiert die Eingabeelemente fr einen bestimmen Anzeigetyp
	 * @param ate Eigenschaften des Typs
	 */
  private void aktualisiereEingabeElemente (AnzeigeTypEigenschaften ate)
  {
  	aktualisiereComboBox (_comboSystemObjekt, ate.getAnzeigeTyp());

  	aktualisiereComboBox (_comboDefaultBild, ate.getDefaultBild());

  	_zuordnung.getMoeglicheObjekte().clear();
  	_zuordnung.getZugewieseneObjekte().clear();
  	
  	for (SystemObject so : _wvzInhalte)
  		_zuordnung.getMoeglicheObjekte().add(so);
  	
  	for (SystemObject so : ate.getInhalte())
  	{
  		_zuordnung.getZugewieseneObjekte().add(so);
  		_zuordnung.getMoeglicheObjekte().remove(so);
  	}

  	_zuordnung.refresh ();
  }  

  private AnzeigeTypEigenschaften leseEingabeElemente ()
  {
    AnzeigeTypEigenschaften ate = new AnzeigeTypEigenschaften (_aktuellerTyp);
       
    int index = _comboSystemObjekt.getSelectionIndex();
    ate.setAnzeigeTyp(bestimmeSystemObject(_anzeigeTypen, _anzeigeTypenAlsText [index]));
    
    index = _comboDefaultBild.getSelectionIndex();
    ate.setDefaultBild(bestimmeSystemObject(_wvzInhalte, _wvzInhalteAlsText [index]));
    
    Vector<SystemObject> inhalte = new Vector<SystemObject>();
    for (Object so : _zuordnung.getZugewieseneObjekte())
    	inhalte.add((SystemObject) so);

    ate.setInhalte(inhalte);
    
    return ate;
  }


  private SystemObject bestimmeSystemObject (java.util.List<SystemObject> objekte, String vgl)
  {
  	for (SystemObject so : objekte)
  		if (so.toString().equals(vgl))
  			return so;
  	
  	return null;
  }

	/**
	 * Methode erzeugt das Composite mit dem Default Button
	 */
	private void createCompositeDefaultButton(Composite parent)
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		
		Composite composite = new Composite(parent, SWT.BORDER);
		composite.setLayout(new GridLayout(1, false));
		composite.setLayoutData(gridData1);

		Button buttonDefault = new Button(composite, SWT.NONE);
		
		buttonDefault.setText("Defaultwerte");
		buttonDefault.addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseDown(MouseEvent e)
			{
				super.mouseDown(e);
				
				defaultWerte();
			}
		});
	}

	/**
	 * Methode definiert die Defaultwerte
	 */
	private void defaultWerte ()
	{
		_parameter.clear();
		
    // Defaultparameter fr alle Anzeigetypen anlegen
    
    for (EAnzeigeTypen eat : EAnzeigeTypen.values())
    	_parameter.put(eat, new AnzeigeTypEigenschaften (eat));
    
    // bisherige Parameter auswerten
    
		for (AnzeigeTypEigenschaften eigenschaften : DefaultEinstellungen.getDefaultAnzeigeTypen())
			_parameter.put(eigenschaften.getTyp(), eigenschaften);
		
		aktualisiereEingabeElemente (_parameter.get(_aktuellerTyp));
	}

  /**
   * Methode liefert die aktuell eingestellten Parameter
   * @return aktuell eingestellten Parameter
   */
	public Vector<AnzeigeTypEigenschaften> getParameter()
  {
	  AnzeigeTypEigenschaften aktuell = leseEingabeElemente();
	  _parameter.put(_aktuellerTyp, aktuell);
	  
    Vector<AnzeigeTypEigenschaften> objekte = new Vector<AnzeigeTypEigenschaften>();
    
    for (AnzeigeTypEigenschaften ate : _parameter.values())
    {
    	if (ate.getAnzeigeTyp() != null)
    	  objekte.add(ate);
    }
    
  	return objekte;
  }

	/**
	 * Methode aktualisiert die View
	 */
	public void refresh()
  {
  }
	
	/**
	 * Methode bernimmt die aktuell eingestellten Werte fr den bisher gltigen
	 * Anzeigetyp. Danach wird der neue Anzeigetyp bestimmt und die Werte dieses
	 * Anzeigetyps eingeblendet.
	 */
	protected void aendereTyp()
  {
		/*
		 * Aktuelle Einstellungen bernehmen
		 */
    if (_aktuellerTyp != null)
    {
    	AnzeigeTypEigenschaften tmp = leseEingabeElemente(); 
    	_parameter.put (_aktuellerTyp, tmp);
    }
		
		/*
		 * Neuen Anzeigetyp bestimmen
		 */
		int index = _comboAnzeigeTyp.getSelectionIndex();
		if (index == -1)
			return;
		
		EAnzeigeTypen typ = EAnzeigeTypen.values()[index];
		
		_aktuellerTyp = typ;
		
		AnzeigeTypEigenschaften ate = _parameter.get(typ);
		if (ate != null)
      aktualisiereEingabeElemente(ate);
  }
	
	/**
	 * Methode selektiert in einer Combobox einen Text 
	 * @param combo Combobox
	 * @param selektiert selektierter Text
	 */
  private void aktualisiereComboBox(Combo combo,
      SystemObject selektiert)
  {
  	int index = -1;
  	String[] items = combo.getItems();

  	String vgl = _undefiniert;
  	if (selektiert != null)
  		vgl = selektiert.toString();
  	
  	for (int i=0; i<items.length; ++i)
  		if (items[i].equals(vgl))
  			index = i;
  	
  	combo.select(index);
  }
  
  /**
   * Klasse realisiert ein Compoiste mit einem Text und einer Combobox
   * @author ObertM
   *
   */
  private class ComboMitText extends Composite 
  {
		private Combo _combo;

		public ComboMitText(Composite parent, int style)
    {
	    super(parent, style);
    }
		
		public void initGUI (String header, String [] values)
		{
			setLayout(new GridLayout(1, false));

			GridData gdTextItem = new GridData();
			gdTextItem.horizontalAlignment = GridData.FILL;
			gdTextItem.grabExcessHorizontalSpace = true;
			gdTextItem.verticalAlignment = GridData.BEGINNING;

			GridData gdComboItem = new GridData();
			gdComboItem.horizontalAlignment = GridData.FILL;

			if (header != null)
			{
			  Label dummy = new Label(this, SWT.LEFT);
			  dummy.setText(header);
			  dummy.setLayoutData(gdTextItem);
			}

			/*
			 * Combobox Anzeigetyp
			 */
			_combo = new Combo(this, SWT.BORDER);
			_combo.setLayoutData(gdComboItem);

			_combo.setItems(values);
			_combo.select(0);
		}

		/**
     * Methode liefert combo zurck
     * @return combo
     */
    public Combo getCombo()
    {
    	return _combo;
    }
  }
}
