package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.monotonie;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;

/**
 * Klasse erzeugt die Defaultregeln fr die Monotonie. Dabei werden Regeln erzeugt die
 * prfen ob:<br>
 * - links eine hhere Geschwindigkeit steht wie rechts<br>
 * - eine Aufhebung gegen eine Geschwindigkeit geschaltet wird<br>
 * - eine Geschwindigkeit gegen Dunkel geschaltet wird<br>
 * - eine Aufhebung gegen Dunkel geschaltet wird
 * @author ObertM
 *
 */
public class DefaultWerte
{
	private AnzeigeInhaltTyp _dunkel;
	private AnzeigeInhaltTyp _km40;
	private AnzeigeInhaltTyp _km50;
	private AnzeigeInhaltTyp _km60;
	private AnzeigeInhaltTyp _km80;
	private AnzeigeInhaltTyp _km100;
	private AnzeigeInhaltTyp _km120;
	private AnzeigeInhaltTyp _km130;
	
	private AnzeigeInhaltTyp _endeKm40;
	private AnzeigeInhaltTyp _endeKm50;
	private AnzeigeInhaltTyp _endeKm60;
	private AnzeigeInhaltTyp _endeKm80;
	private AnzeigeInhaltTyp _endeKm100;
	private AnzeigeInhaltTyp _endeKm120;
	private AnzeigeInhaltTyp _endeKm130;
	
	private AnzeigeInhaltTyp _endeAllerSV;
	private AnzeigeInhaltTyp _kreuz;
	private AnzeigeInhaltTyp _pfl;
	private AnzeigeInhaltTyp _pfr;
	private AnzeigeInhaltTyp _stau;
	
	private Aspect _schaltgrund;
	private VerwaltungZuordung _verwaltungGeschwindigkeiten;
	
	private Vector<AnzeigeInhaltTyp> _moeglicheWerteX;
	private Vector<AnzeigeInhaltTyp> _moeglicheWerteY;
	private Vector<AnzeigeInhaltTyp> _sonstigeZeichen;

	/**
	 * Parameterloser Konstruktor der Klasse
	 */
	public DefaultWerte ()
	{
		initialisiere ();
	}
	
  private void initialisiere()
  {
  	_dunkel    = AnzeigeInhaltTyp.DUNKEL;

  	_km40      = AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_40;
  	_km50      = AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_50;
  	_km60      = AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_60;
  	_km80      = AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_80;
  	_km100     = AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_100;
  	_km120     = AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_120;
  	_km130     = AnzeigeInhaltTyp.TLS_ZULAESSIGE_HOECHST_130;

  	_endeKm40  = AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_40;
  	_endeKm50  = AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_50;
  	_endeKm60  = AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_60;
  	_endeKm80  = AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_80;
  	_endeKm100 = AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_100;
  	_endeKm120 = AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_120;
  	_endeKm130 = AnzeigeInhaltTyp.TLS_ENDE_ZULAESSIGE_HOECHST_130;

  	_endeAllerSV = AnzeigeInhaltTyp.TLS_ENDE_ALLER_STRECKENVERBOTE;
  	_kreuz       = AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_GESPERRT;
  	_pfl         = AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_LINKS;
  	_pfr         = AnzeigeInhaltTyp.TLS_FAHRSTREIFEN_WECHSELN_NACH_RECHTS;
  	_stau        = AnzeigeInhaltTyp.TLS_STAU;
 
  	_schaltgrund = VerwaltungParaMonotonie.getInstance().getSchaltgrundAllgemein();
  	
  	_verwaltungGeschwindigkeiten = new VerwaltungZuordung();
  	
  	_verwaltungGeschwindigkeiten.addZuordnung(_km40,   40);
  	_verwaltungGeschwindigkeiten.addZuordnung(_km50,   50);
  	_verwaltungGeschwindigkeiten.addZuordnung(_km60,   60);
  	_verwaltungGeschwindigkeiten.addZuordnung(_km80,   80);
  	_verwaltungGeschwindigkeiten.addZuordnung(_km100, 100);
  	_verwaltungGeschwindigkeiten.addZuordnung(_km120, 120);
  	_verwaltungGeschwindigkeiten.addZuordnung(_km130, 130);

  	// Werte fr X
  	
  	_moeglicheWerteX = new Vector<AnzeigeInhaltTyp>();
  	
    add (_moeglicheWerteX, AnzeigeElementTyp.WZG_A, _km40);
    add (_moeglicheWerteX, AnzeigeElementTyp.WZG_A, _km50);
    add (_moeglicheWerteX, AnzeigeElementTyp.WZG_A, _km60);
    add (_moeglicheWerteX, AnzeigeElementTyp.WZG_A, _km80);
    add (_moeglicheWerteX, AnzeigeElementTyp.WZG_A, _km100);
    add (_moeglicheWerteX, AnzeigeElementTyp.WZG_A, _km120);
    
  	_moeglicheWerteY = new Vector<AnzeigeInhaltTyp>();
  	
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _dunkel);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _km40);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _km50);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _km60);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _km80);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _km100);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _km120);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _km130);

    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _endeKm40);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _endeKm50);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _endeKm60);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _endeKm80);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _endeKm100);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _endeKm120);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _endeKm130);
    add (_moeglicheWerteY, AnzeigeElementTyp.WZG_A, _endeAllerSV);

  	/*
  	 * Sonstige Zeichen sind all die Zeichen, die gegen Dunkel gepft werden sollen und in den
  	 * obigen Regeln nicht enthalten sind.
  	 */
  	_sonstigeZeichen = new Vector<AnzeigeInhaltTyp>();
  	
    add (_sonstigeZeichen, AnzeigeElementTyp.WZG_A, _endeKm40);
    add (_sonstigeZeichen, AnzeigeElementTyp.WZG_A, _endeKm50);
    add (_sonstigeZeichen, AnzeigeElementTyp.WZG_A, _endeKm60);
    add (_sonstigeZeichen, AnzeigeElementTyp.WZG_A, _endeKm80);
    add (_sonstigeZeichen, AnzeigeElementTyp.WZG_A, _endeKm100);
    add (_sonstigeZeichen, AnzeigeElementTyp.WZG_A, _endeKm120);
    add (_sonstigeZeichen, AnzeigeElementTyp.WZG_A, _endeKm130);
    add (_sonstigeZeichen, AnzeigeElementTyp.WZG_A, _endeAllerSV);
    add (_sonstigeZeichen, AnzeigeElementTyp.WZG_A, _stau);
  }

  /**
   * Methode prft ob ein Bildinhalt versorgt ist und fgt diesen wenn versorgt einem
   * Vektor hinzu
   * @param vektor Vektor	
   * @param wzgTyp Anzeigetyp
   * @param inhalt Inhalt
   */
	private void add(Vector<AnzeigeInhaltTyp> vektor,
      AnzeigeElementTyp wzgTyp, AnzeigeInhaltTyp inhalt)
  {
    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
    
    if (verw.getWvzInhalt (wzgTyp, inhalt.getBildInhalt()) != null)
    	vektor.add(inhalt);
  }

	/**
   * Methode generiert automatisch alle Regeln fr eine 0 km/h Geschwindigkeitsdiffernze
   */
  public Vector<RegelMonotonie> getRegelnGeschwDiff0KmH ()
  {
		Vector<RegelMonotonie> regeln = new Vector<RegelMonotonie>();

    setzeRegeln (regeln, 0);
    
	  return regeln;
  }

	/**
   * Methode generiert automatisch alle Regeln fr eine 20 km/h Geschwindigkeitsdiffernze
   */
  public Vector<RegelMonotonie> getRegelnGeschwDiff20KmH ()
  {
		Vector<RegelMonotonie> regeln = new Vector<RegelMonotonie>();

    setzeRegeln (regeln, 20);
    
	  return regeln;
  }

  /**
   * Methode erzeugt die bntigten Regeln
   * @param regeln Regeln
   * @param differenz mnax. zulssige Geschwindigkeitsdifferenz 
   */
  private void setzeRegeln (Vector<RegelMonotonie> regeln, int differenz)
  {
  	setzeVorgaengerBeziehung (regeln, differenz);
  	
  	setzeNachfolgerBeziehung(regeln);

  	setzeSonstigeRegeln (regeln);
  }

  /**
   * Methode erstellt Regeln in denen geprft wird, ob Zeichen gegen dunkel geschaltet 
   * werden
   * @param regeln Regeln
   */
  private void setzeSonstigeRegeln(Vector<RegelMonotonie> regeln)
  {
  	Vector<AnzeigeInhaltTyp> werteFuerX = (Vector<AnzeigeInhaltTyp>) _sonstigeZeichen.clone();
  	
    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();

    WVZInhalt wy = verw.getWvzInhalt (AnzeigeElementTyp.WZG_A, _dunkel.getBildInhalt());

  	for (AnzeigeInhaltTyp x : werteFuerX)
  	{
	    WVZInhalt wx = verw.getWvzInhalt (AnzeigeElementTyp.WZG_A, x.getBildInhalt());
	    
	  	regeln.add (new RegelMonotonie (_schaltgrund, 
          ArbeitsRichtungVN.VORGAENGER, wx, wy, wx)); 
	  	
	  	regeln.add (new RegelMonotonie (_schaltgrund, 
          ArbeitsRichtungVN.NACHFOLGER, wx, wy, wx));
  	}
  }

	/**
   * Methode setzt die Vorgngerbezihung zwischen. Dabei wird geprft dass die linke Anzeige
   * zur rechten Anzeige eine max. Geschwindigkeitsdiffernz nicht berschreitet
   * @param regeln Regeln
   * @param differenz max. Geschwindigkeitsdifferenz
   */
  private void setzeVorgaengerBeziehung(Vector<RegelMonotonie> regeln, int differenz)
  {
  	/*
  	 * Vorgngerbeziehung (von rechts nach links)
  	 * Prfen ob links eine hhere Geschwindigkeit wie rechts steht
  	 */
  	
  	Vector<AnzeigeInhaltTyp> werteFuerX = (Vector<AnzeigeInhaltTyp>) _moeglicheWerteX.clone();
  	Vector<AnzeigeInhaltTyp> werteFuerY = (Vector<AnzeigeInhaltTyp>) _moeglicheWerteY.clone();
  	
    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();

  	for (AnzeigeInhaltTyp x : werteFuerX)
  	{
  		if (werteFuerY.contains(x))
  			werteFuerY.remove(x);
  		
  		for (AnzeigeInhaltTyp y : werteFuerY)
  		{
  			int v1 = _verwaltungGeschwindigkeiten.getGeschwindigkeit(x);
  			int v2 = _verwaltungGeschwindigkeiten.getGeschwindigkeit(y);
  			
  			if (Math.abs(v2 - v1) > differenz)
  			{
  				AnzeigeInhaltTyp z = x;

  				/*
  				 * Nur wenn Differenz > 0 ist und fr Y eine Geschwindigkeitvorgabe gefunden wurde
  				 */
  				if (differenz > 0 && v2 > 0)
  				{
  					AnzeigeInhaltTyp nochZulaessig = null;	
  				
  					/*
  					 * In 10er Schritten Geschwindigkeit herabsetzen bis eine entsprechend
  					 * zulssige Anzeige gefunden wird
  					 */
  					do
  					{
		  				v2 = v2 - 10;
		  				
		  				if (Math.abs(v2 - v1) <= differenz)
		  				  nochZulaessig = _verwaltungGeschwindigkeiten.getAnzeigeInhaltTyp (v2);
		  				
		  				if (v2 < 0) // Sicherheitshalber
		  					break;
  					}
  					while ((nochZulaessig == null) || (v2 <= v1));
  					
  					if (nochZulaessig != null)
  						z = nochZulaessig;
  				}
  				
    	    WVZInhalt wx = verw.getWvzInhalt (AnzeigeElementTyp.WZG_A, x.getBildInhalt());
    	    WVZInhalt wy = verw.getWvzInhalt (AnzeigeElementTyp.WZG_A, y.getBildInhalt());
    	    WVZInhalt wz = verw.getWvzInhalt (AnzeigeElementTyp.WZG_A, z.getBildInhalt());

			  	regeln.add (new RegelMonotonie (_schaltgrund, 
		          ArbeitsRichtungVN.VORGAENGER, wx, wy, wz));
  			}
  		}
  	}
  }

  /**
   * Methode setzt die Vorgngerbezihung zwischen. Dabei wird geprft dass die linke Anzeige
   * zur rechten Anzeige eine max. Geschwindigkeitsdiffernz nicht berschreitet
   * @param regeln Regeln
   * @param differenz max. Geschwindigkeitsdifferenz
   */
  private void setzeNachfolgerBeziehung(Vector<RegelMonotonie> regeln)
  {
  	/*
  	 * Nachfolgerbeziehung (von links nach rechts)
  	 * Prfen ob rechts eine hhere Geschwindigkeit wie links steht
  	 */
  	
  	Vector<AnzeigeInhaltTyp> werteFuerX = (Vector<AnzeigeInhaltTyp>) _moeglicheWerteX.clone();
  	Vector<AnzeigeInhaltTyp> werteFuerY = (Vector<AnzeigeInhaltTyp>) _moeglicheWerteY.clone();
  	
    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();

  	for (AnzeigeInhaltTyp x : werteFuerX)
  	{
  		if (werteFuerY.contains(x))
  			werteFuerY.remove(x);
  		
  		for (AnzeigeInhaltTyp y : werteFuerY)
  		{
  	    WVZInhalt wx = verw.getWvzInhalt (AnzeigeElementTyp.WZG_A, x.getBildInhalt());
  	    WVZInhalt wy = verw.getWvzInhalt (AnzeigeElementTyp.WZG_A, y.getBildInhalt());

		  	regeln.add (new RegelMonotonie (_schaltgrund, 
	          ArbeitsRichtungVN.NACHFOLGER, wx, wy, wx)); 
  		}
  	}
  }

  /**
   * Klasse zum Speichern der Zuordnung WVZ-Inhalt - zugeordnete Geschwindigkeit
   * @author ObertM
   *
   */
  private class VerwaltungZuordung
  {
  	/**
  	 * Hashmap mit der Zuordnung WVZ-Inhalt - Geschwindigkeit
  	 */
  	private HashMap<AnzeigeInhaltTyp, Integer> _zuordnungen = new HashMap<AnzeigeInhaltTyp, Integer>();
  	
  	/**
  	 * Methode zum Hinzufgen einer Zuordnung
  	 * @param inhalt Inhalt
  	 * @param geschwindigkeit Geschwindigkeit
  	 */
  	public void addZuordnung (AnzeigeInhaltTyp inhalt, int geschwindigkeit)
  	{
  		_zuordnungen.put(inhalt, geschwindigkeit);
  	}
  	
  	/**
  	 * Methode liefert den AnzeigeInhaltTyp zu einer Geschwindigkeit
  	 * @param geschwindigkeit Geschwindigkeit
  	 * @return Anzeigeinhalttyp
  	 */
  	public AnzeigeInhaltTyp getAnzeigeInhaltTyp(int geschwindigkeit)
    {
  		Iterator<Entry<AnzeigeInhaltTyp, Integer>> it = _zuordnungen.entrySet().iterator();
  		while (it.hasNext())
  		{
  			Entry<AnzeigeInhaltTyp, Integer> entry = it.next();
  			if (entry.getValue() == geschwindigkeit)
  				return entry.getKey();
  		}
  		
	    return null;
    }

		/**
  	 * Methode liefert die Geschwindigkeit zu einem Inhalt
  	 * @param inhalt Inhalt
  	 * @return zugeordnete Geschwindigkeit, wenn nicht definiert dann 0 
  	 */
  	public int getGeschwindigkeit (AnzeigeInhaltTyp inhalt)
  	{
  		if (_zuordnungen.containsKey(inhalt))
  			return _zuordnungen.get(inhalt);
  		
  		return 0;
  	}
  }
}
