package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.monotonie;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Dambach Werke GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private Vector<RegelMonotonie> m_parameter = new Vector<RegelMonotonie> ();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public ListeMitRegeln ()
  {
  }
  
  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelMonotonie> getParameter()
  {
    Vector<RegelMonotonie> v = new Vector<RegelMonotonie>();
    synchronized (m_parameter)
    {
      for (RegelMonotonie r : m_parameter)
        v.add( r );
    }
    return v;
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaMonotonieProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    Array a = data.getArray( "Regeln" );
    int anzahl = a.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelMonotonie r = new RegelMonotonie ();
        r.auswerteAttributgruppe( a.getItem( x ), schaltgrundAllgemein );
        
        m_parameter.add( r );
      }
    }
  }
  
  /**
   * Methode fgt eine Regel hinzu
   * @param x Anzeigeinhalt der aktuell betrachteten Anzeige 
   * @param y Anzeigeinhalt des Vorgnger/Nachfolgers
   * @param z Angeforderter Anzeigeinhalt
   * @param richtung Richtung in der die Regel arbeitet
   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
   */
  public void addRegel (WVZInhalt x, WVZInhalt y, WVZInhalt z, ArbeitsRichtungVN richtung, Aspect schaltgrund )
  {
    synchronized (m_parameter)
    {
      RegelMonotonie r = new RegelMonotonie ();
      
      r.setSchaltgrundRegel( schaltgrund );
      r.setAnzeigenInhalt( x );
      r.setAnzeigenInhaltVorgaengerNachfolger( y );
      r.setAngeforderterAnzeigenInhalt( z );
      r.setRichtung( richtung );
      
      m_parameter.add( r );
    }
  }

  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      for (RegelMonotonie r : m_parameter)
        s += r.toString() + "\n";
    }
    
    return s;
  }

  /**
   * Methode setzt die Variable parameter
   * @param parameter Wert fr parameter
   */
  public void setParameter(Vector<RegelMonotonie> parameter)
  {
    synchronized (m_parameter)
    {
      m_parameter.clear();
      
      for (RegelMonotonie r : parameter)
         m_parameter.add(r);
    } 
  }
}
