package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.monotonieKreuzPfeil;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.CompositeTools;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Darstellung des Fensters zur Versorgung der Parameter
 */
public class CompositeParameterMonotonieKreuzPfeil extends Composite
{
	/**
	 * max. Anzahl versorgbarer Bilder
	 */
	private final int _anzBilder = 10;
	
	/**
	 * max. Anzahl Bilder in einer Zeile
	 */
	private final int _anzBilderProZeile = 5;
	
	/**
	 * Aktuelles Composite
	 */
  private Composite _composite;

  /**
   * Composite der Tabelle
   */
  private Composite _compositeTabelle;

  /**
   * Gruppe mit den Regeln
   */
  private Group _groupRegeln;

  /**
   * Tabelviewer der Tabelle
   */
  private TableViewer _tvAnzeige;

  /**
   * Aktuelle Parameter
   */
  private ListeMitRegeln _aktuelleParameter;

  /**
   * Composite fr die Buttons
   */
	private Composite _compositeButtons;

	/**
	 * Button Hinzufgen
	 */
	private Button _buttonHinzufuegen;

	/**
	 * Button Lschen
	 */
	private Button _buttonLoeschen;

	/**
	 * Tabellenmodell
	 */
	private TableModel[] _tableModel;
	
	/**
	 * Anzeigetyp fr den die Parameter versorgt werden
	 */
	private AnzeigeElementTyp _anzeigeTyp;

	private Button _buttonAlleLoeschen;

	private Button _buttonDefault;

  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil des Fenstern
   * @param aktuell Aktuelle Parameter
   * @param pidAnzeigeTyp Pid des Anzeigetyps
   */
  CompositeParameterMonotonieKreuzPfeil (Composite parent, int style, ListeMitRegeln aktuell, String pidAnzeigeTyp)
  {
    super (parent, style);
    
    this._aktuelleParameter = erzeugeKopie(aktuell);
    
    _anzeigeTyp = AnzeigeElementTyp.getTypByPid(pidAnzeigeTyp);
    
    initialize();
  }

  /**
   * Methode legt eine Arbeitskopie der akutellen Daten an
   * @param aktuell aktuelle Daten
   * @return Arbeitskopie
   */
  private ListeMitRegeln erzeugeKopie(ListeMitRegeln aktuell)
  {
	  ListeMitRegeln kopie = new ListeMitRegeln();
	  
	  Vector<RegelMonotonieKreuzPfeil> parameter = aktuell.getParameter();
	  for (RegelMonotonieKreuzPfeil regel : parameter)
	  {
	  	List<WVZInhalt> x = new ArrayList<WVZInhalt>();
	  	for (WVZInhalt inhalt : regel.getAnzeigenInhalt())
	  		x.add(inhalt);

	  	List<WVZInhalt> y = new ArrayList<WVZInhalt>();
	  	for (WVZInhalt inhalt : regel.getAnzeigenInhaltVorgaengerNachfolgerLaengs())
	  		y.add(inhalt);
	  	
	  	List<WVZInhalt> z = new ArrayList<WVZInhalt>();
	  	for (WVZInhalt inhalt : regel.getAnzeigenInhaltVorgaengerNachfolgerQuer())
	  		z.add(inhalt);

	  	kopie.addRegel(x, y, z, regel.getRichtungLaengs(), regel.getRichtungQuer(), regel.getSchaltgrundRegel());
	  }
	  
	  return kopie;
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
 
    erzeugeKommentar();
    
    erzeugeGruppeRegeln();
    
    Table table = _tvAnzeige.getTable();
    _tableModel = createModel1(table);
    
    if ( _tableModel != null )
      _tvAnzeige.setInput(_tableModel); 
    
    createButtons();
    
    updateData(_aktuelleParameter);
  }

  /**
   * Methode blendet einen Kommentar zu dem Parametersatz ein
   */
  private void erzeugeKommentar ()
  {
  	String kommentar = "Monotonie Kreuz/Pfeil:\n\n" + 
    "Wenn Anzeigeinhalt in \"X\"\n" +
    "und Inhalt des Vorgngers/Nachfolgers in Lngsrichtung in \"Y\"\n" + 
    "und Inhalt des Vorgngers/Nachfolgers in Querrichtung in \"Z\"\n" +
    "dann an der aktuelle Anzeige den Inhalt des Vorgngers/Nachfolgers in Querrichtung anfordern.\n\n" +
  	"Die Regel kann ber einen Hauptschalter abgeschaltet werden.\n\n" +
  	"";

    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    
    Group groupKommentar = new Group (_composite, SWT.NONE);
    groupKommentar.setLayout(new FillLayout());
    groupKommentar.setLayoutData(gridData1);

//    groupKommentar.setText("Erklrung");
    
    Label text = new Label (groupKommentar, SWT.NONE);
    text.setText(kommentar);
  }

  /**
   * Methode bernimmt die Daten und zeigt diese in der Tabelle an
   */
  private void updateData (ListeMitRegeln parameter)
  {
  	int anzahlParameter = parameter.getParameter().size();
  	
  	erzeugeTabellenModell (anzahlParameter);

  	for (int j=0; j<anzahlParameter; ++j)
  	{
      TableModel tm = _tableModel [j];    	
    	RegelMonotonieKreuzPfeil aktuell = parameter.getParameter().get(j);
    	
    	/*
    	 * Lschen
    	 */
    	tm.setSelectedLoeschen("false");
    	
    	/*
    	 * Schaltgrund
    	 */
    	
    	if (aktuell.getSchaltgrundRegel() == null)
    	{
    	  tm.setSelectedSchaltgrund("");
    	}
    	else
    	{
  	    tm.setSelectedSchaltgrund(aktuell.getSchaltgrundRegel().getName());
    	}
    	
    	/*
    	 * Richtung Lngs
    	 */
  	  tm.setSelectedRichtungLaengs (aktuell.getRichtungLaengs().getName());

    	/*
    	 * Richtung Quer
    	 */
  	  tm.setSelectedRichtungQuer (aktuell.getRichtungQuer().getName());

  	  /*
  	   * X
  	   */
  	  erzeugeBilder (tm.getX(), aktuell.getAnzeigenInhalt());

  	  /*
  	   * Y
  	   */
  	  erzeugeBilder (tm.getY(), aktuell.getAnzeigenInhaltVorgaengerNachfolgerLaengs());

  	  /*
  	   * Z
  	   */
  	  erzeugeBilder (tm.getZ(), aktuell.getAnzeigenInhaltVorgaengerNachfolgerQuer());
    }

  	_buttonAlleLoeschen.setEnabled (anzahlParameter != 0);
  	pruefeObLoeschenAngewaehlt();
  }

  /**
   * Methode erzeugt in einem Composite die versorgten Bilder
   * @param c Composite
   * @param regel
   */
  private void erzeugeBilder (Composite c, List<WVZInhalt> wi)
  {
	  Control[] children = c.getChildren();
	  Label [] im = new Label [children.length];
	  for (int i=0; i<children.length; ++i)
	  	im [i] = (Label) children [i];
	  
	  int max = (wi.size() > _anzBilder ? _anzBilder : wi.size());
	  
	  for (int i=0; i<max; ++i)
	  	Tools.setzeBildInhalte(im[i], wi.get(i), _anzeigeTyp, true);
	  
		WVZInhalt dontCare = VerwaltungWvzInhalte.getInstance().getWvzInhalt (_anzeigeTyp, AnzeigeInhaltTyp.DONT_CARE);

	  for (int i=wi.size(); i<_anzBilder; ++i)
	  	Tools.setzeBildInhalte(im[i], dontCare, _anzeigeTyp, true);
  }
  
  /**
   * Methode erzeugt ein Tabellenmodell einer bestimmten Gre. Dabei werden bereits vorhandene
   * Composite in der Tabelle weiter verwendet
   * @param anzahlParameter Gre des erzeugten Tabellenmodells
   */
  private void erzeugeTabellenModell (int anzahlParameter)
  {
  	/*
  	 * Hat sich die Gre gendert
  	 */
  	if (_tableModel.length != anzahlParameter)
  	{
  		/*
  		 * Tabelle erweitern
  		 */
  		if (_tableModel.length < anzahlParameter)
  		{
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  
  		  for (int i=0; i<_tableModel.length; ++i)
  		  	v.add(_tableModel [i]);
  		  
  		  for (int i=_tableModel.length; i<anzahlParameter; ++i)
  		  	v.add (erzeugeZeileInTabelle(_tvAnzeige.getTable()));
  		  
  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		/*
  		 * Tabelle reduzieren
  		 */
  		else if (_tableModel.length > anzahlParameter)
  		{
  			/*
  			 * Nicht mehr bentigte Elemente lschen (in umgekehrter Reihenfolge)
  			 */
  			
  			for (int i = _tableModel.length - 1; i > anzahlParameter - 1; i--)
  			{
  				_tableModel [i].getZ().dispose();
  				_tableModel [i].getY().dispose();
  				_tableModel [i].getX().dispose();
  				_tableModel [i].getRichtungQuer().dispose();
  				_tableModel [i].getRichtungLaengs().dispose();
  				_tableModel [i].getSchaltgrund().dispose();
  				_tableModel [i].getLoeschen().dispose();
  			}
  				
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  for (int i=0; i<anzahlParameter; ++i)
  		  	v.add(_tableModel [i]);

  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		
  	  _tvAnzeige.setInput(_tableModel);
  	  _tvAnzeige.getTable().layout(true);
  	}
  }
  
  /**
   * Methode erzeugt die Gruppe mit den Regeln
   */
  private void erzeugeGruppeRegeln()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupRegeln = new Group(_composite, SWT.NONE);
    _groupRegeln.setText("Regeln");
    
    _groupRegeln.setLayout(fillLayout1);
    _groupRegeln.setLayoutData(gridData2);

    erzeugeTabelle();
  }

  /**
   * Methode erzeugt das Composite mit den Buttons
   */
  private void createButtons()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    _compositeButtons = new Composite( _composite, SWT.NONE );
    _compositeButtons.setLayoutData( gridData3 );
    _compositeButtons.setLayout( rowLayout );

    _buttonHinzufuegen = new Button( _compositeButtons, SWT.NONE );
    _buttonHinzufuegen.setText( "Regel hinzufgen" );
    _buttonHinzufuegen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelHinzufuegen();
      }
    } );

    _buttonLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonLoeschen.setText( "Selektierte Regeln loeschen" );
    _buttonLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelLoeschen();
      }

    } );
    _buttonLoeschen.setEnabled(false);

    _buttonAlleLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonAlleLoeschen.setText( "Alle Regeln loeschen" );
    _buttonAlleLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        aktionAlleRegelLoeschen();
      }
    } );
    
    if (_anzeigeTyp == AnzeigeElementTyp.WZG_A)
    {
	    _buttonDefault = new Button( _compositeButtons, SWT.NONE );
	    _buttonDefault.setText( "Defaultwerte" );
	    _buttonDefault.addMouseListener( new MouseAdapter()
	    {
	      @Override
	      public void mouseDown( MouseEvent e )
	      {
	        aktionDefault ();
	      }
	    } );
    }
  }

	/**
   * Methode zum Lschen aller Meldungen
   */
  protected void aktionAlleRegelLoeschen()
  {
		Vector<RegelMonotonieKreuzPfeil> regeln = new Vector<RegelMonotonieKreuzPfeil>();
    _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);
  }
  
  /**
   * Methode generiert automatisch die Defaultwerte
   */
  protected void aktionDefault()
  {
		Vector<RegelMonotonieKreuzPfeil> regeln = new Vector<RegelMonotonieKreuzPfeil>();

	  DefaultWerte dw = new DefaultWerte ();
	  regeln = dw.getDefaultWerte(_anzBilder);

    _aktuelleParameter.setParameter(regeln);
    updateData(_aktuelleParameter);
  }

  /**
   * Methode prft ob Regeln zum Lschen ausgewhlt sind
   */
  private void pruefeObLoeschenAngewaehlt ()
  {
  	boolean angewaehlt = false;
  	
		for (TableModel tm : _tableModel)
			if (tm.getSelectedLoeschen().equals("true"))
				angewaehlt = true;
		
		_buttonLoeschen.setEnabled(angewaehlt);
  }
  
  /**
   * Methode zum Lschen der Regeln
   */
	private void aktionRegelLoeschen()
  {
		Vector<RegelMonotonieKreuzPfeil> regeln = leseAktuelleEinstellungen().getParameter();
		
    Vector<RegelMonotonieKreuzPfeil> loeschen = new Vector<RegelMonotonieKreuzPfeil>();
    
    for (int i=0; i<_tableModel.length; ++i)
    	if (!_tableModel [i].getSelectedLoeschen().equals("false"))
    		loeschen.add(regeln.get(i));

    for (RegelMonotonieKreuzPfeil r : loeschen)
    {
    	regeln.remove(r);
    }
    
    _aktuelleParameter.setParameter(regeln);
    
    updateData(_aktuelleParameter);
  }

	/**
	 * Aktion Regel hinzufgen
	 */
	private void aktionRegelHinzufuegen()
  {
		Vector<RegelMonotonieKreuzPfeil> regeln = leseAktuelleEinstellungen().getParameter();
    _aktuelleParameter.setParameter(regeln);

		Aspect a = VerwaltungParaMonotonieKreuzPfeil.getInstance().getSchaltgrundAllgemein();
		
		WVZInhalt dontCare = VerwaltungWvzInhalte.getInstance().getWvzInhalt (_anzeigeTyp, AnzeigeInhaltTyp.DONT_CARE);

		List<WVZInhalt> x = new ArrayList<WVZInhalt>();
		List<WVZInhalt> y = new ArrayList<WVZInhalt>();
		List<WVZInhalt> z = new ArrayList<WVZInhalt>();
		
		for (int i=0; i<_anzBilder; ++i)
		{
			x.add(dontCare);
			y.add(dontCare);
			z.add(dontCare);
		}

		_aktuelleParameter.addRegel (x, y, z, ArbeitsRichtungVN.NACHFOLGER, ArbeitsRichtungVN.NACHFOLGER, a);
		
		updateData(_aktuelleParameter);
  }

	/**
	 * Methode erzeugt das Composite mit der Tabelle
	 */
  private void erzeugeTabelle ()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    
    _compositeTabelle = new Composite(_groupRegeln, SWT.None);
    _compositeTabelle.setLayout(fillLayout1);
    
    _tvAnzeige = new TableViewer(_compositeTabelle, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAnzeige.setContentProvider(new TableContentProvider());
    
    int breiteWVZ = Tools.getMaxBreiteBild() * _anzBilderProZeile;
    
    String [] header = new String []{"Lschen", "Schaltgrund", "Lngsrichtung", "Querrichtung", "X", "Y", "Z",};
    int    [] breite = new int [] {60, 200, 100, 100, breiteWVZ, breiteWVZ, breiteWVZ};

    for (int i=0; i<header.length; ++i)
    {
      TableViewerColumn column = new TableViewerColumn(_tvAnzeige, SWT.NONE);
      column.getColumn().setWidth(breite [i]);
      column.getColumn().setText(header [i]);
      column.setLabelProvider(new MyCellLabelProvider());
    }

    // resize the row height using a MeasureItem listener
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
          	
          	int anzZeilen = _anzBilder / _anzBilderProZeile;
          	if ((_anzBilder % _anzBilderProZeile) != 0)
          		anzZeilen++;
          	
            event.height = Tools.getZeilenHoehe(_anzeigeTyp) * anzZeilen;
            break;

          default:
        }
      }
    };
    
    _tvAnzeige.getTable().addListener(SWT.MeasureItem, listener);
    _tvAnzeige.getTable().setLinesVisible(true);
    _tvAnzeige.getTable().setHeaderVisible(true);
  }

  /**
   * Klasse realsiert den TableContentProvider fr die Tabelle
   * @author ObertM
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
  	/**
  	 * Tabelviewer
  	 */
    private TableViewer tableViewer;

    /**
     * Methode liefert die Elemente der Tabelle
     */
    public Object[] getElements(Object inputElement)
    {
			Object[] re = null;

			if (inputElement instanceof TableModel [])
			{
        re = (TableModel[]) inputElement;
			}
			
			return re;
    }

    /**
     * Dispose
     */
    public void dispose()
    {
    }

    /**
     * Methode wird aufgerufen wenn sich der Inhalt der Tabelle gendert hat
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.deleteObserver(this);
        }
      }
      
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.addObserver(this);
        }
      }
    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();
      }
    }
  }

  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * @param table
   * @return TabellenModell
   */
  private TableModel[] createModel1 (Table table)
  {
    int tabLength = getAnzahlParameter();
    int elIndex   = 0;
    
    TableModel[] elements = new TableModel [tabLength];
    
    if (elements.length == 0)
    	return elements;

    for (int x=0; x<getAnzahlParameter(); ++x)
    {
	    elements[elIndex] = erzeugeZeileInTabelle(table);
	    elIndex++;
    }
    
    return elements;
  }

  /**
   * Methode erzeugt ein Tabellenmodell fr eine Zeile in der Tabelle
   * @param table Tabelle 
   * @return Tabellenmodell fr eine Zeile
   */
  private TableModel erzeugeZeileInTabelle (Table table)
  {
    /*
     * Button Lschen
     */
    Composite comp0 = CompositeTools.erzeugeCompositeLoeschen(table);
    Button b = (Button) comp0.getChildren()[0];

    b.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				pruefeObLoeschenAngewaehlt();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
    
    /*
     * Combobox Schaltgrund    
     */
    String[] namen = Tools.getNamenDerAspekte();

    Composite comp1 = CompositeTools.erzeugeCompositeCombo(table, namen); 
    
    /*
     * Combobox Richtung lngs    
     */
    namen = Richtung.ArbeitsRichtungVN.getNamen();

    Composite comp2 = CompositeTools.erzeugeCompositeCombo(table, namen); 
    	
    /*
     * Combobox Richtung    
     */
    Composite comp3 = CompositeTools.erzeugeCompositeCombo(table, namen); 

    /*
     * X
     */
    Composite comp4 = CompositeTools.erzeugeCompositeWVZ(table, _anzeigeTyp, _anzBilder, _anzBilderProZeile);
    
    /*
     * Y
     */
    Composite comp5 = CompositeTools.erzeugeCompositeWVZ(table, _anzeigeTyp, _anzBilder, _anzBilderProZeile);
    
    /*
     * Z
     */
    Composite comp6 = CompositeTools.erzeugeCompositeWVZ(table, _anzeigeTyp, _anzBilder, _anzBilderProZeile);
   
    return new TableModel( comp0, comp1, comp2, comp3, comp4, comp5, comp6 );
  }
  
  /**
   * Methode liefert die Anzahl der Parameter
   * @return Anzahl der Parameter
   */
  public int getAnzahlParameter()
  {
    return _aktuelleParameter.getParameter().size();
  }
  
  /**
   * Methode liefert den Tabelviewer
   * @return
   */
  public TableViewer getTvAnzeige()
  {
    return _tvAnzeige;
  }

  /**
   * Renderer fr die Tabelle
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider 
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

//      if (item.getData("EDITOR") != null)
//      {
//        TableEditor editor = (TableEditor)item.getData("EDITOR");
//        editor.dispose();
//      }

      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getLoeschen();
          break;
          
        case 1:
          comp = model.getSchaltgrund();
          break;
          
        case 2:
          comp = model.getRichtungLaengs();
          break;

        case 3:
          comp = model.getRichtungQuer();
          break;

        case 4:
          comp = model.getX();
          break;
          
        case 5:
          comp = model.getY();
          break;

        case 6:
          comp = model.getZ();
          break;

        default:
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);

    }
  }

  /**
   * Methode liefert die eingestellten Parameter
   * @return eingestellte Parameter
   */
	public ListeMitRegeln getParameter()
  {
 		aktionRegelLoeschen();
 		return leseAktuelleEinstellungen();
  }
		
	private ListeMitRegeln leseAktuelleEinstellungen ()
	{
		ListeMitRegeln regeln = new ListeMitRegeln();
		
		for (int j=0; j<_tableModel.length; ++j)
		{
      TableModel tm = _tableModel [j];
      
      /*
       * Schaltgrund
       */
      
      String nameSchaltgrund = tm.getSelectedSchaltgrund();
      Aspect schaltgrund = Tools.getAspekt(nameSchaltgrund);
      
      /*
       * Richtung Lngs 
       */
      String nameRichtungLaengs = tm.getSelectedRichtungLaengs();
      ArbeitsRichtungVN richtungLaengs = ArbeitsRichtungVN.getRichtung (nameRichtungLaengs);

      /*
       * Richtung Quer 
       */
      String nameRichtungQuer = tm.getSelectedRichtungQuer();
      ArbeitsRichtungVN richtungQuer = ArbeitsRichtungVN.getRichtung (nameRichtungQuer);

      /*
       * X
       */
      List<WVZInhalt> x = leseInhalte(tm.getSelectedX());
      
      /*
       * Y
       */
      List<WVZInhalt> y = leseInhalte(tm.getSelectedY());
      
      /*
       * Z
       */
      List<WVZInhalt> z = leseInhalte(tm.getSelectedZ());

      regeln.addRegel(x, y, z, richtungLaengs, richtungQuer, schaltgrund);
		}

		return regeln;
  }

	/**
	 * Methode liest die Namen der Inhalte und erzeugt daraus die WVZ-Inhalte
	 * @param inhalte Namen der Inhalte
	 * @return Liste mit WVZ-Inhalten
	 */
	private List<WVZInhalt> leseInhalte (String[] inhalte)
	{
    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
    
    List<String> tmp = new ArrayList<String>();
    
    for (String s : inhalte)
    {
    	if (s != null && !s.equals(AnzeigeInhaltTyp.DONT_CARE.getBildInhalt()))
    		if (!tmp.contains(s))
    			tmp.add(s);
    }

    List<WVZInhalt> l = new ArrayList<WVZInhalt>();
    for (String s : tmp)
    	l.add (verw.getWvzInhalt (_anzeigeTyp, s));
    
    return l;
	}
	
	/**
	 * Methode bernimmt die Daten eines anderen Folders
	 * @param tvAnzeige Tableviewer der zu bernhmenden Tabelle
	 */
	public void setTvAnzeige (TableViewer tvAnzeige)
	{
    VerwaltungWvzInhalte verw = VerwaltungWvzInhalte.getInstance();
    
    TableModel[] modelSet = (TableModel[]) tvAnzeige.getInput();

    Vector<RegelMonotonieKreuzPfeil> v = new Vector<RegelMonotonieKreuzPfeil>();

    for (int i=0; i<modelSet.length; ++i)
    {
      String nameSchaltgrund = modelSet [i].getSelectedSchaltgrund();
      Aspect schaltgrund = Tools.getAspekt(nameSchaltgrund);

      String nameRichtungLaengs = modelSet [i].getSelectedRichtungLaengs();
      ArbeitsRichtungVN rL = ArbeitsRichtungVN.getRichtung (nameRichtungLaengs);

      String nameRichtungQuer = modelSet [i].getSelectedRichtungQuer();
      ArbeitsRichtungVN rQ = ArbeitsRichtungVN.getRichtung (nameRichtungQuer);

  	  List<WVZInhalt> x = new ArrayList<WVZInhalt>();
  	  for (String s : modelSet [i].getSelectedX())
  	  	x.add(verw.getWvzInhalt(_anzeigeTyp, s));

  	  List<WVZInhalt> y = new ArrayList<WVZInhalt>();
  	  for (String s : modelSet [i].getSelectedY())
  	  	y.add(verw.getWvzInhalt(_anzeigeTyp, s));

  	  List<WVZInhalt> z = new ArrayList<WVZInhalt>();
  	  for (String s : modelSet [i].getSelectedZ())
  	  	z.add(verw.getWvzInhalt(_anzeigeTyp, s));

  	  v.add (new RegelMonotonieKreuzPfeil (schaltgrund, rL, rQ, x, y, z));
    }
    
    _aktuelleParameter.setParameter(v);
    updateData(_aktuelleParameter);
  }
}
