package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.trichterung;

import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.Activator;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;
import de.bsvrz.sys.funclib.debug.Debug;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog;

/**
 * View zum ndern der Parameter "Trichterung"
 */
public class ViewParameterTrichterung extends ViewPart 
{
  public static final String ID = "de.bsvrz.sba.plugin.parameter.steuerungskern.view.ViewParameterTrichterung";
  
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Folder fr die Tabs
   */
  private CTabFolder _tabFolder = null;

  /**
   * Composite
   */
  private Composite _composite = null;

  /**
   * Composite mit den Buttons
   */
  private Composite _compositeBottom;

  /**
   * Button "Fr alle bernehmen"
   */
  private Button buttonAlle;
  
  /**
   * Button "Speichern"
   */
  private Button buttonOk;

  /**
   * Button "bernehmen"  
   */
  private Button buttonUebernehemen;

  /**
   * Button "Abbrechen"
   */
  private Button buttonAbbrechen;

  /**
   * Button "Hilfe"
   */
  private Button buttonHilfe;

  /**
   * Memento
   */
  private IMemento _memento = null;

  /**
   * Zweite ID des Views
   */
  private String _secondViewId;

  /**
   * Verbindung zum DaV
   */
  private ClientDavInterface _connection;

  /**
   * Anzuzeigende Objekte
   */
  private List<SystemObject> _objekte = new ArrayList<SystemObject>();

  /**
   * Listener
   */
  private PropertyChangeListener m_listener;
  
  @Override
  public void createPartControl( Composite parent )
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    
    _composite = new Composite( parent, SWT.NONE );
    _composite.setLayout( gridLayout );
    _composite.setLayoutData( gridData );
    
    //Fr die F1 Taste
    
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _composite, Activator.PLUGIN_ID + ".parameter" );   
    
    _connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    
    // Darstellung aufbauen
    
    createCompositeTab();
    createCompositeButtons();

    restoreState();

   _objekte.clear();

    if ( _secondViewId.contains( ";" ) )
    {

      String[] split = _secondViewId.split( ";" );

      for (String s : split)
      {
      	if (_connection != null)
      	{
	        SystemObject so = _connection.getDataModel().getObject( s );
	        _objekte.add( so );
      	}
      }

    }
    else
    {
    	if (_connection != null)
    	{
        SystemObject so = _connection.getDataModel().getObject( _secondViewId );
        _objekte.add( so );
    	}
    }

    erzeugeTabItems( _objekte );
    
    verarbeiteDaten();
  }

  @Override
  public void setFocus()
  {
    if (_composite != null)
      _composite.setFocus();
  }

  /**
   * Methode erzeugt das Composite mit den Foldern
   */
  private void createCompositeTab()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder( _composite, SWT.NONE );
    _tabFolder.setLayout( new GridLayout() );
    _tabFolder.setLayoutData( gridData3 );
    
    _tabFolder.addSelectionListener( new SelectionListener()
		{
      @Override
      public void widgetSelected( SelectionEvent e )
      {
        CTabFolder folder = (CTabFolder) e.getSource();
        CTabItem itemSel = folder.getSelection();

        Composite cSel = (Composite) itemSel.getControl();

        if ( cSel.getChildren() != null && cSel.getChildren().length > 0 )
        {
        	CompositeAnzeigeTypen compParSel = (CompositeAnzeigeTypen) cSel.getChildren()[0];
        	compParSel.aktualisiereAlleFolder();
        }
      }

			@Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }
    } );
  }


  /**
   * Methode erzugt das Composite mit den Buttons
   */
  private void createCompositeButtons()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    _compositeBottom = new Composite( _composite, SWT.NONE );
    _compositeBottom.setLayoutData( gridData3 );
    _compositeBottom.setLayout( rowLayout );

    buttonAlle = new Button( _compositeBottom, SWT.NONE );
    buttonAlle.setText( "Alle setzen" );
    buttonAlle.setToolTipText( "Einstellungen bei allen ausgewhlten gleichartigen AQs setzen" );
    buttonAlle.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionAlle();
      }
    } );
   
    buttonOk = new Button( _compositeBottom, SWT.NONE );
    buttonOk.setText( "Ok" );
    buttonOk.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionSchalten();
      }
    } );

    buttonUebernehemen = new Button( _compositeBottom, SWT.NONE );
    buttonUebernehemen.setText( "bernehmen" );
    buttonUebernehemen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionUebernehmen();
      }
    } );

    buttonAbbrechen = new Button( _compositeBottom, SWT.NONE );
    buttonAbbrechen.setText( "Abbrechen" );
    buttonAbbrechen.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        dispose();
      }

    } );
    buttonHilfe = new Button( _compositeBottom, SWT.NONE );
    buttonHilfe.setText( "Hilfe" );
    buttonHilfe.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionHilfe();
      }

    } );
  }

  /**
   * Zeigt die Hilfe an.
   */
  protected void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".parameter";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }
  
  /**
   * Methode bernimmt die eingestellten Parameter fr alle Tabs. Hierzu werden die Tabellenviewer des
   * angewhlten Tab an die anderen Tabs bergeben.
   */
  protected void aktionAlle()
  {
//  	System.out.println("ViewParameterTrichterung.aktionAlle()");
  	
    /*
     * aktueller Reiter
     */
    CTabItem itemSel = _tabFolder.getSelection();

    Composite compSel = null;

    if ( itemSel == null )
      compSel = (Composite) _tabFolder.getItems()[0].getControl();
    else
      compSel = (Composite) itemSel.getControl();

    if ( compSel.getChildren() != null )
    {
    	CompositeAnzeigeTypen compParSel = (CompositeAnzeigeTypen) compSel.getChildren()[0];

    	HashMap<String, TableViewer> modelle = compParSel.getTableViewer();
    	
      CTabItem[] items = _tabFolder.getItems();
      for (CTabItem item : items)
      {
        if ( !item.equals( itemSel ) )
        {
          Composite c = (Composite) item.getControl();
  
          CompositeAnzeigeTypen compPar = (CompositeAnzeigeTypen) c.getChildren()[0];
          compPar.setTableViewer(modelle);
        }
      }
    }
    else
      System.out.println("Kein selektierter Tab!!!");
  }

  /**
   * bernehmen der Parameter und Beenden des Views
   */
  protected void aktionSchalten()
  {
    if ( speichernDerParameterInDaV() )
    {
      dispose();
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  /**
   * bernehmen der Parameter
   */
  protected void aktionUebernehmen()
  {
    if ( speichernDerParameterInDaV() )
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  @Override
  public void init( IViewSite site, IMemento memento ) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init( site, memento );
  }

  private void restoreState()
  {
    if ( _memento == null )
    {
      return;
    }

    _secondViewId = _memento.getString( "pidMemento" );

  }

  @Override
  public void saveState( IMemento memento )
  {
    memento.putString( "pidMemento", _secondViewId );
    super.saveState( memento );
  }

  /**
   * Methode speichert die Parameter im DaV
   * @return
   */
  private Boolean speichernDerParameterInDaV()
  {   
  	Boolean schliessenOk = false;
  	
  	/*
  	 * Daten der einzelnen Tabs einlesen
  	 */

	  CTabItem[] items = _tabFolder.getItems();
	  for (CTabItem item : items)
	  {
	    Composite c = (Composite)item.getControl();
	    CompositeAnzeigeTypen compPar = (CompositeAnzeigeTypen) c.getChildren()[0];

      String pid = (String)item.getData(item.getText());

	    RegelnProAnzeigeTyp regeln = compPar.getParameter();
	    
	    VerwaltungParaTrichterung.getInstance().setParameter (pid, regeln);
	  }

	  /*
	   * Attributgruppe zusammenstellen und an den DaV senden
	   */
	  
	  Data data = VerwaltungParaTrichterung.getInstance().getData();
	  
    if (data != null)
    {
      SystemObject sysObj = _connection.getDataModel().getObject(Konfig.getInstance().getPidSBA());
      
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup ("atg.sswSbaTrichterung");
      
      Aspect asp = _connection.getDataModel().getAspect("asp.parameterVorgabe");
      
      DataDescription dd = new DataDescription(atg, asp);

      ResultData result = new ResultData( sysObj, dd, new Date ().getTime(), data );

      List<ResultData> list = new ArrayList<ResultData>();
      list.add (result);

      UrlasserDialog urlasserDialog = new UrlasserDialog( Display.getCurrent().getActiveShell() );
      schliessenOk = (urlasserDialog.anzeigen( list )).getEingabeOk();
    }

    return schliessenOk;
  }

  @Override
  public void dispose()
  {
    super.dispose();

    VerwaltungParaTrichterung.getInstance().removePropertyChangeListener(m_listener);
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if ( page != null )
    {
      page.hideView( this );
    }
  }

  /**
   * Methode aktualisiert die einzelnen Tabs
   */
  private void verarbeiteDaten()
  {
    for (SystemObject so : _objekte)
    {
      verarbeiteDaten(so);        
    }
    
    if ( _tabFolder.getItemCount() <= 1 )
      buttonAlle.setEnabled( false );
    else
    {
    	/*
    	 * Prfen ob alle offenen Objekte die selben Anzeigetypen haben, nur dann darf der Button 
    	 * aktiviert werden.
    	 */
    	
    	Vector<SystemObject> vgl = null;
    	
    	boolean identisch = true;
    	
    	for (SystemObject so : _objekte)
    	{
    	  Vector<SystemObject> typen = Tools.bestimmeAnzeigeTypen(so.getPid());
    	  if (vgl == null)
    	  	vgl = typen;
    	  else
    	  {
    	    if (typen.size() != vgl.size())
    	    	identisch = false;
    	    
    	    for (SystemObject typ : typen)
    	    	if (!vgl.contains(typ))
    	    		identisch = false;
    	  }
    	}
    	
      buttonAlle.setEnabled( identisch );
    }

  }
  
  /**
   * Methode aktualisiert einen Tab
   * @param so Systemobjekt fr den ein Tab angelegt wurde
   */
  private void verarbeiteDaten(SystemObject so)
  {
    CTabItem[] items = _tabFolder.getItems();
    
    // Tab zum Systemobject ermitteln
    
    CTabItem item = null;
    for (CTabItem ti : items)
    {
      String pid = (String)ti.getData(ti.getText());

      if (pid == so.getPid())
      {
        item = ti;
        break;
      }
    }

    // Tab gefunden
    
    if (item != null)
    {
      RegelnProAnzeigeTyp aktuell = VerwaltungParaTrichterung.getInstance().getParameter (so.getPid());
      
      Composite c = (Composite)item.getControl();
      
      Control[] children = c.getChildren();

      // Tab neu anlegen, wenn noch leer
      
      if (children.length == 0)
      {
//      	Group xx = new Group(c, SWT.BORDER);
      	
        CompositeAnzeigeTypen compPar = new CompositeAnzeigeTypen( c, SWT.NONE, aktuell, (String) so.getPid() );
        compPar.layout( true );
        c.layout( true );
      }
    }
  }

  /**
   * Methode erzeugt die einzelnen Folder
   * @param list List mit Objekten fr die Folder angelegt werden
   */
  public void erzeugeTabItems( List<SystemObject> list )
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    
    for (SystemObject so : list)
    {
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );
      tabItem.setText( so.getName() );
      tabItem.setData( so.getName(), so.getPid() );

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
  }
}
