package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dialog.AnzeigeInhaltTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungWvzInhalte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaften;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Fenster zum Versorgen der Algorithmus abhngigen Parameter
 * @author ObertM
 */
public class CompositeParameterEigenschaft1 extends Composite
{
  /**
   * Einheiten des DAV fr die Spinner
   */
  private static enum Einheit { KEIN };
  
  /**
   * ParameterElemente des DAV, die als Spinner angezeigt werden, die Namen mssen identisch sein
   * mit den Namen der Items der Attributgruppe im DaV 
   */
  private static final String[] _parItemSpin = 
  {
  };
  
  /**
   * ParameterElemente des DAV, die als ComboBox angezeigt werden, die Namen mssen identisch sein
   * mit den Namen der Items der Attributgruppe im DaV 
   */
  private static final String[] _parItemCombo = 
  {
  	"Kriterium",
  };

  /**
   * Gesmatcomposite
   */
  private Composite _composite;

  /**
   * Composite (Gruppe) fr die Parameter
   */
  private Group _groupParameter;

  /**
   * Composite mit der Tabelle
   */
  private Composite _compositeTabelle = null;

  /**
   * Compoiste fr die Tabelle
   */
  private Composite _composite3 = null;

  /**
   * Table Viewer
   */
  private TableViewer _tvParameter;;

  /**
   * Spinner
   */
  private Spinner[] _spinnerPar;

  /**
   * ComboBoxen
   */
  private Combo[] _comboPar = new Combo [100];

  /**
   * Universal Regel
   */
	private RegelUniversal _aktuell;

	/**
	 * Tabellenmodell
	 */
	private TableModel[] _tableModel = new TableModel [0];

	/**
	 * Liste mit den mglichen Anzeigeinhalten
	 */
	private List<SystemObject> _listMoeglicheInhalte;

	/**
	 * Systemobjekt fr einen beliebigen Inhalt
	 */
	private SystemObject _beliebig;

  /**
   * Konstruktor der Klasse
   * @param parent Aufrufendes Composite
   * @param style Stil
   * @param result Ergebnis vom Dav
   * @param data Datenfeld innerhalb des Ergebnisses welches ausgewertet werden soll
   */
  CompositeParameterEigenschaft1(Composite parent, int style, RegelUniversal regel)
  {
    super(parent, style);
    
    _aktuell = regel;
 
    VerwaltungWvzInhalte v = VerwaltungWvzInhalte.getInstance();
    
    _listMoeglicheInhalte = v.getZuPruefendeWvzInhalte();
    
    _beliebig = v.getBeliebig().getAnzeigenInhalt();
  
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    erzeugeKommentar();
    	
    createGroupParameter();

//    Table table = _tvParameter.getTable();
//    TableModel[] input = createModel(table);
//    _tvParameter.setInput(input);

    updateData();
  }

  /**
   * Methode blendet einen Kommentar zu dem Parametersatz ein
   */
  private void erzeugeKommentar ()
  {
  	String kommentar = "Universalregel Eigenschaft 1:\n\n" + 
  	                   "Anzeigeinhalt ist \"X\" = ja/nein/beliebig\n\n";
  	
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    
    Group groupKommentar = new Group (_composite, SWT.NONE);
    groupKommentar.setLayout(new FillLayout());
    groupKommentar.setLayoutData(gridData1);

//    groupKommentar.setText("Erklrung");
    
    Label text = new Label (groupKommentar, SWT.NONE);
    text.setText(kommentar);
  }

  /**
   * Updatefunktion fr Aufruf von aussen
   * @param result Ergebnis vom Dav
   * @param data Datenfeld innerhalb des Ergebnisses welches ausgewertet werden soll
   */
  public void update (RegelUniversal regel)
  {
    _aktuell = regel;
    
    updateData();
    
  }

  private int bestimmeAnzahlEingabeFelder ()
  {
    return _aktuell.getEigenschaften().size();	
  }
  
  /**
   * Updatefunktion
   */
  private void updateData ()
  {
    Table table = _tvParameter.getTable();
    TableModel[] input = createModel(table);
    _tvParameter.setInput(input);
  	
  	Vector<UniversalEigenschaften> eigenschaften = _aktuell.getEigenschaften();
  	
  	/*
  	 * Kriterium (es wird nur das erste Kriterium ausgewertet, da das Kriterium fr alle Anzeigeninhalt
  	 * gleich sein muss (es soll ja eine Anzeigenkombination verglichen werden)
  	 */
  	
  	JaNeinBeliebig jnb = JaNeinBeliebig.BELIEBIG;
  	if (eigenschaften.size() > 0)
  		jnb = eigenschaften.get(0).getEigenschaftAnzeigenInhalt().getVergleich();

		Combo cb = (Combo) _comboPar [0];
	  for (int k = 0; k < cb.getItemCount(); k++)
	  {
	    if (jnb.getKennung().equals(cb.getItem( k )))
	      cb.select(k);      
	  }

	  /*
	   * Anzeigeninhalte
	   */
  	
	  for (int i=0; i<eigenschaften.size(); ++i)
	  {
      WVZInhalt inhalt = eigenschaften.get(i).getEigenschaftAnzeigenInhalt().getAnzeigenInhalt();
      SystemObject so = inhalt.getAnzeigenInhalt();

      	
      String name = Tools.getName(_beliebig);
      if (so != null)
      	name = Tools.getName(so);

  		cb = (Combo) _comboPar [i + 1];
  		if (cb == null)
  		{
  		  	
  		}
  		
  	  for (int k = 0; k < cb.getItemCount(); k++)
  	  {
  	  	if (name != null)
	  	    if (name.equals(cb.getItem( k )))
	  	      cb.select(k);      
  	  }
	  }

    getTvParameter().refresh();
    layout(true);
  }

  /**
   * Methode zum Anlegen des Composite mit den Parametern
   */
  private void createGroupParameter()
  {
    FillLayout fillLayout2 = new FillLayout();
    fillLayout2.type = org.eclipse.swt.SWT.VERTICAL;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupParameter = new Group(_composite, SWT.NONE);
    _groupParameter.setText("Parameter");
    _groupParameter.setLayout(fillLayout2);
    _groupParameter.setLayoutData(gridData2);
    
    //GridLayout mit Anzahl Spalten
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    gridLayout.marginHeight = 5;
    gridLayout.marginWidth = 5;
    
    _compositeTabelle = new Composite(_groupParameter, SWT.NONE);
    _compositeTabelle.setLayout(gridLayout);

    erzeugeTabelle();
  }

  /**
   * Tabelle fr die Parameter erzeugen
   */
  private void erzeugeTabelle()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    FillLayout fillLayout = new FillLayout();
    fillLayout.type = org.eclipse.swt.SWT.HORIZONTAL; 

    _composite3 = new Composite(_compositeTabelle, SWT.None);
    _composite3.setLayout(fillLayout);
    _composite3.setLayoutData( gridData );
    
    _tvParameter = new TableViewer(_composite3, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvParameter.setContentProvider(new TableContentProvider());

    TableViewerColumn column1 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column1.getColumn().setWidth(100);
    column1.getColumn().setText("Attribut");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column2.getColumn().setWidth(400);
    column2.getColumn().setText("Wert");
    column2.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column3 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column3.getColumn().setWidth(100);
    column3.getColumn().setText("");
    column3.setLabelProvider(new MyCellLabelProvider());

    /*
     * Zeilenhhe anpassen, nachdem die Daten eingegangen sind
     */
    _tvParameter.getTable().addListener(SWT.MeasureItem, new Listener() 
    {
      @Override
      public void handleEvent(Event event)
      {
        event.height = 29;
      }
    });

    _tvParameter.getTable().setLinesVisible(true);
    _tvParameter.getTable().setHeaderVisible(true);
  }

  /**
   * Contentprovider fr die Tabelle
   * @author ObertM
   *
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
    private TableViewer tableViewer;

    /*
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement)
    {
      return (TableModel[])inputElement;
    }

    /*
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {

    }

    /*
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
     *      java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.deleteObserver(this);
          }

        }

      }
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.addObserver(this);
          }
        }

      }

    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();

      }
    }
  }

  /**
   * Tabellenmodell fr die Parameter erzeugen
   * @param table Tabelle
   * @return TabellenModell
   */
  private TableModel[] createModel( Table table )
  {
  	int anzahl = bestimmeAnzahlEingabeFelder();
  	
  	/*
  	 * Anzahl besteht aus 1 Zeile fr Vergleich + eine Zeile pro Anzeige
  	 */
  	int benoetigt = anzahl + 1;
  	
  	if (_tableModel != null)
	  	if (benoetigt == _tableModel.length)
	  		return _tableModel;
  	
		/*
		 * Tabelle erweitern
		 */
		if (_tableModel.length < benoetigt)
		{
		  Vector<TableModel> v = new Vector<TableModel>();
		  
		  for (int i=0; i<_tableModel.length; ++i)
		  	v.add(_tableModel [i]);
		  
		  for (int i=_tableModel.length; i<benoetigt; ++i)
		  {
		  	if (i == 0)
		  		v.add (erzeugeVergleichsOperatorInTabelle (table));
		  	else
		  	  v.add (erzeugeEingenschaftInTabelle (table, i - 1));
		  }
		  
		  _tableModel = new TableModel [benoetigt];
		  for (int i=0; i<v.size(); ++i)
		  	_tableModel [i] = v.get(i);
		}
		
		/*
		 * Tabelle reduzieren
		 */
		else if (_tableModel.length > benoetigt)
		{
			/*
			 * Nicht mehr bentigte Elemente lschen
			 */
			
			for (int i = _tableModel.length - 1; i > benoetigt - 1; i--)
			{
				_tableModel [i].getEinheit().dispose();
				_tableModel [i].getAttribut().dispose();
				_tableModel [i].getBezeichnung().dispose();
			}
				
		  Vector<TableModel> v = new Vector<TableModel>();
		  for (int i=0; i<benoetigt; ++i)
		  	v.add(_tableModel [i]);

		  _tableModel = new TableModel [benoetigt];
		  for (int i=0; i<v.size(); ++i)
		  	_tableModel [i] = v.get(i);
		}

    return _tableModel;
  }

  /**
   * Methode erzeugt das Tabellenmodell fr die eigentliche zu vergleichende Eigenschaft
   * @param table Tabelle
   * @return Tabellemodell
   */
  private TableModel erzeugeEingenschaftInTabelle(Table table, int index)
  {
    return createWVZInhaltEingabe(table, index, "Anzeige " + (index+1) );
  }

  /**
   * Methode erzeugt das Tabellenmodell fr den Vergleichsoperator
   * @param table Tabelle
   * @return Tabellemodell
   */
	private TableModel erzeugeVergleichsOperatorInTabelle (Table table)
  {
    /*
     * Ja/Nein/Beliebig
     */
    List<String> jaNeinBeliebig = new ArrayList<String>();
    for (JaNeinBeliebig jnb : JaNeinBeliebig.values())
    	jaNeinBeliebig.add (jnb.getKennung());

    /*
     * ComboBoxen erzeugen
     */
    return createComboElement(table, 0, _parItemCombo[0], Einheit.KEIN, jaNeinBeliebig );
  }

	/**
	 * Methode erzeugt ein Tabellenmodell fr die Eingabe eines WVZ Inhalts
	 * @param table Tabelle
	 * @param intInhalt Index des Eingabeelements
	 * @param item Name des Attributs
	 * @return Tabellenmodell
	 */
	private TableModel createWVZInhaltEingabe (Table table, int intInhalt, String item)
  {
    return createComboElement(table, intInhalt + 1, item, Einheit.KEIN, Tools.bestimmeObjektNamen(_listMoeglicheInhalte) );
  }
  
  /**
   * ein Parameter-Element des TabellenModells erzeugen
   * 
   * @param table       Tabelle
   * @param intCombo    Index der ComboBox
   * @param item        Bezeichnung des Parameters
   * @param einheit     Einheit des Parameters
   * @param anzeigeQuerschnitte       mgliche Werte der ComboBox
   * @return
   */
  private TableModel createComboElement( Table table, int intCombo, String item, Einheit einheit, List<String> anzeigeQuerschnitte )
  {
  	/*
  	 * Composite 0: Beschriftung 
  	 */
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessVerticalSpace = true;
    
    Composite comp0 = new Composite(table, SWT.NONE);
    comp0.setBackground(table.getBackground());
    comp0.setBackgroundMode(SWT.INHERIT_DEFAULT);
    comp0.setLayoutData(gd0);
    comp0.setLayout(new GridLayout());
    
    Label label = new Label(comp0, SWT.NONE);
    label.setText( item );

    /*
     * Composite 1: Combobox
     */
    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;
    gd1.verticalAlignment = GridData.END;

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    Composite comp1 = new Composite( table, SWT.NONE );
    comp1.setBackground( table.getBackground() );
    comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp1.setLayoutData( gd1 );
    comp1.setLayout( new GridLayout() );
    
    _comboPar[intCombo] = new Combo( comp1, SWT.BORDER );
    _comboPar[intCombo].setLayoutData( gdTableItem );
    
    String[] t = new String [anzeigeQuerschnitte.size()];
    for (int i=0; i<anzeigeQuerschnitte.size(); ++i)
    	t[i] = anzeigeQuerschnitte.get(i);
    	
    _comboPar[intCombo].setItems(t);
    _comboPar[intCombo].select(0);

    /*
     *  Composite 2: Einheit
     */
    
    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.verticalAlignment = GridData.CENTER;
    gd2.grabExcessVerticalSpace = true;

    Composite comp2 = new Composite(table, SWT.NONE);
    comp2.setBackground(table.getBackground());
    comp2.setBackgroundMode(SWT.INHERIT_DEFAULT);
    comp2.setLayoutData(gd2);
    comp2.setLayout(new GridLayout());
    Label label2 = new Label(comp2, SWT.NONE);
    label2.setText("");
    
    return new TableModel(comp0, comp1, comp2);
  }

  /**
   * Methode liefert den TableViewer
   * @return TableViewer
   */
  public TableViewer getTvParameter()
  {
    return _tvParameter;
  }

  /**
   * Methode liefert den Spinner zu einem Index
   * @param index Index des Spinners
   * @return Spinner
   */
  public Spinner getSpinner(int index)
  {
    return _spinnerPar [index];
  }

  /**
   * Methode liefert die ComboBox zu einem Index
   * @param index Index des Spinners
   * @return Spinner
   */
  public Combo getComboBox (int index)
  {
    return _comboPar [index];
  }

  /**
   * Methode liefert den Namen eines Items eines Spinners zu einem Index
   * @param index Index
   * @return Name des Items
   */
  public String getItemSpinnerName (int index)
  {
  	return _parItemSpin [index];
  }

  /**
   * Methode liefert den Namen eines Items einer ComboBox zu einem Index
   * @param index Index
   * @return Name des Items
   */
  public String getItemComboBoxName (int index)
  {
  	return _parItemCombo [index];
  }

  /**
   * Labelprovider
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

      if (item.getData("EDITOR") != null)
      {
        TableEditor editor = (TableEditor)item.getData("EDITOR");
        editor.dispose();
      }


      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getBezeichnung();
          break;
        case 1:
          comp = model.getAttribut();
          break;
        case 2:
          comp = model.getEinheit();
          break;
        default:
      }
      TableEditor editor = new TableEditor(item.getParent());
      
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);
    }
  }
  
  /**
   * Methode liefert die Anzahl der verwendeten Items
   * @return Anzahl 
   */
  public int getAnzahlSpinnerItems ()
  {
  	return _parItemSpin.length;
  }

  /**
   * Methode liefert die Anzahl der verwendeten Items
   * @return Anzahl 
   */
  public int getAnzahlComboBoxItems ()
  {
  	return _parItemCombo.length;
  }

  /**
   * Methode refresht die Tabelle
   */
	public void refresh()
  {
		if (_tvParameter != null)
	    _tvParameter.refresh();
//		{
//	    Table table = _tvParameter.getTable();
//      TableModel[] input = createModel(table);
//      
//      updateData();
//      
//      _tvParameter.setInput(input);
//	    _tvParameter.refresh();
//		}
  }

	/**
	 * Methode zum bernehmen der eingestellten Daten
	 */
	public void uebernehmeDaten ()
  {
    Vector<UniversalEigenschaften> eigenschaften = _aktuell.getEigenschaften();
    
    /*
     * Kriterium
     */
    Combo cb = _comboPar [0];
    String selected = cb.getItem( cb.getSelectionIndex() );
    JaNeinBeliebig jnb = JaNeinBeliebig.getWert(selected);
    
    for (UniversalEigenschaften ue : eigenschaften)
    {
    	ue.getEigenschaftAnzeigenInhalt().setVergleich(jnb);
    }
    
    /*
     * Schaltbilder
     */
    for (int i=0; i<eigenschaften.size(); ++i)
    {
      cb = _comboPar [i + 1];
      
      WVZInhalt x = VerwaltungWvzInhalte.getInstance().getWvzInhaltBeliebig();
      
      if (cb != null)  // wenn noch nicht angelegt
      {
        selected = cb.getItem( cb.getSelectionIndex() );
	      for (SystemObject so : _listMoeglicheInhalte)
	      	if (Tools.getName(so).equals(selected))
	      		x = new WVZInhalt(so);
      }
      	
    	eigenschaften.get(i).getEigenschaftAnzeigenInhalt().setAnzeigenInhalt(x);
    }
  }
}
