package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungQL;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaften;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Swarco Traffic Systems GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private Vector<RegelUniversal> m_parameter = new Vector<RegelUniversal> ();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public ListeMitRegeln ()
  {
  }
  
  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelUniversal> getRegeln()
  {
    Vector<RegelUniversal> v = new Vector<RegelUniversal>();
    synchronized (m_parameter)
    {
      for (RegelUniversal r : m_parameter)
        v.add( r );
    }
    return v;
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaRegelUniversal'
   * @param a Daten vom DaV
   */
  public void auswerteAttributgruppe (Array a, Aspect schaltgrundAllgemein )
  {
    int anzahl = a.getLength();
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelUniversal r = new RegelUniversal ();
        r.auswerteAttributgruppe( a.getItem( x ), schaltgrundAllgemein  );
        
        m_parameter.add( r );
      }
    }
  }
  
  /**
   * Methode fgt eine Regel hinzu
   * @param richtungVN Arbeitet Regel in Vorgnger- oder Nachfolgerrichtung 
   * @param richtungQL Arbeitet Regel in Quer- oder Lngsrichtung
   * @param anzeigen Reihe von Anzeigen welche die folgenden Eigenschaften aufweist
   * @param eigenschaften Eigenschaften fr die Regel
   * @param angeforderteInhalte angeforderte Inhalte der Anzeigen
   * @param alternativerAnzeigenTyp Anzeigentyp fr Alterntivschaltung
   * @param alternativerAnzeigenInhalt alternativer Anzeigeninhalt
   * @param schaltgrund Spezieller Schaltgrund fr die Regel
   */
  public void addRegel (
                         ArbeitsRichtungVN richtungVN,
                         ArbeitsRichtungQL richtungQL,
                         Vector<SystemObject> anzeigen,
                         Vector<UniversalEigenschaften> eigenschaften,
                         Vector<WVZInhalt> angeforderteInhalte,
                         SystemObject alternativerAnzeigenTyp,
                         WVZInhalt alternativerAnzeigenInhalt,
                         Aspect schaltgrund 
                       )
  {
    RegelUniversal ru = new RegelUniversal ();  
    
    ru.setRichtungVN                ( richtungVN );
    ru.setRichtungQL                ( richtungQL );
    
    ru.setAnzeigen                  ( anzeigen );
    ru.setEigenschaften             ( eigenschaften );
    ru.setAngeforderteInhalte       ( angeforderteInhalte );
    ru.setAlternativerAnzeigenTyp   ( alternativerAnzeigenTyp );
    ru.setAlternativerAnzeigenInhalt( alternativerAnzeigenInhalt );
    ru.setSchaltgrundRegel          ( schaltgrund );
    
    m_parameter.add( ru );
  }

  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      for (RegelUniversal r : m_parameter)
        s += r.toString() + "\n";
    }
    
    return s;
  }

  /**
   * Methode fgt eine Regel hinzu
   * @param regel Regel
   */
	public void addRegel(RegelUniversal regel)
  {
		synchronized (m_parameter)
    {
	    m_parameter.add(regel);   
    }
  }
}
