/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.JaNeinBeliebig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse realisiert folgende Eigenschaft der Universalregel:<br>
 * Anzeigeninhalt ist "X" = ja/nein/beliebig
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 */
public class UniversalEigenschaft1
{
  /**
   * Variable zum Speichern des Anzeigeinhalts 
   */
  private WVZInhalt m_anzeigenInhalt = new WVZInhalt();
  
  /**
   * Vergleichsoperator  
   */
  private JaNeinBeliebig m_vergleich = JaNeinBeliebig.BELIEBIG;

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public UniversalEigenschaft1 ()
  {
  }

  /**
   * Methode zum Auswerten der Attributliste 'atl.sswSbaRegelUniversalEigenschaft1'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe( Data data )
  {
    // Kriterium
    
    String jaNeinBeliebig = data.getItem( "Kriterium" ).asTextValue().getText();
    if (jaNeinBeliebig.equals( "Ja" ))
      m_vergleich = JaNeinBeliebig.JA;
    else if (jaNeinBeliebig.equals(  "Nein" ))
      m_vergleich = JaNeinBeliebig.NEIN;
    else
      m_vergleich = JaNeinBeliebig.BELIEBIG;
    
    // Anzeigeinhalt
    
    m_anzeigenInhalt = new WVZInhalt ();
    m_anzeigenInhalt.auswerteAttributgruppe( data.getItem( "Anzeigeinhalt" ));
  }

  /**
   * Methode liefert die Variable m_anzeigenInhalt zurck
   * @return Variable m_anzeigenInhalt
   */
  public WVZInhalt getAnzeigenInhalt()
  {
    return m_anzeigenInhalt;
  }

  /**
   * Methode setzt die Variable m_anzeigenInhalt
   * @param anzeigenInhalt Wert auf den die Variable m_anzeigenInhalt gesetzt werden soll
   */
  public void setAnzeigenInhalt( WVZInhalt anzeigenInhalt )
  {
    m_anzeigenInhalt = anzeigenInhalt;
  }

  /**
   * Methode liefert die Variable m_vergleich zurck
   * @return Variable m_vergleich
   */
  public JaNeinBeliebig getVergleich()
  {
    return m_vergleich;
  }

  /**
   * Methode setzt die Variable m_vergleich
   * @param vergleich Wert auf den die Variable m_vergleich gesetzt werden soll
   */
  public void setVergleich( JaNeinBeliebig vergleich )
  {
    m_vergleich = vergleich;
  }
  
  /**
   * Methode gibt den Inhalt der Klasse als String aus
   */
  @Override
  public String toString()
  {
    String s = "Anzeigeninhalt ist " + Tools.getWVZInhalt( m_anzeigenInhalt ) + " = " + m_vergleich + "\n";
    return s;
  }
}
