package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.zusammenfassen;

import java.util.HashMap;
import java.util.Vector;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Darstellung eines Composite mit unterschiedlichen Foldern fr die einzelnen
 * Anzeigetypen
 */
public class CompositeAnzeigeTypen extends Composite
{  
	/**
	 * Aktuelle Parameter
	 */
  private RegelnProAnzeigeTyp _aktuelleParameter;

  /**
   * Instanz mit den Foldern
   */
  private CTabFolder _tabFolder = null;

  /**
   * Composite
   */
  private Composite _composite;

  /**
   * Pid des auslsenden Objekts
   */
	private String _pid;

	/**
	 * Konstuktor der Klasse
	 * @param parent ausfrufendes Composite
	 * @param style Stil
	 * @param aktuell aktuelle Parameter
	 * @param pid Pid des Objekts mit den Parametern
	 */
  public CompositeAnzeigeTypen (Composite parent, int style, RegelnProAnzeigeTyp aktuell, String pid)
  {
    super (parent, style);
    
    _pid = pid;
    
    this._aktuelleParameter = aktuell;
    
    initialize();
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);

    /*
     * Bestimmen der bentigten Anzeigetypen, fr jeden Anzeigetyp wird ein eigener Tab angelgt
     */
    
    Vector<SystemObject> anzeigeTypen = Tools.bestimmeAnzeigeTypen(_pid);
    
    createCompositeTab();
    erzeugeTabItems (anzeigeTypen);
    
    /*
     * Daten einblenden
     */
    
    for (SystemObject so : anzeigeTypen)
      verarbeiteDaten(so);
    
  }
    
  /**
   * Methode legt das Composite mit den Foldern an
   */
  private void createCompositeTab()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder( _composite, SWT.NONE );
    _tabFolder.setLayout( new GridLayout() );
    _tabFolder.setLayoutData( gridData3 );

    _tabFolder.addSelectionListener( new SelectionListener()
    {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder) e.getSource();
        CTabItem itemSel = folder.getSelection();

        aktualisiereFolder(itemSel);
      }
      
      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }
    });
  }

  /**
   * Methode zum Aktualisieren aller Folder
   */
  public void aktualisiereAlleFolder ()
  {
    CTabItem[] items = _tabFolder.getItems();
    for (CTabItem itemSel : items)
    	aktualisiereFolder(itemSel);
  }

  /**
   * Methode zum Aktualisieren eines Folder
   * @param itemSel selektierter Folder
   */
  public void aktualisiereFolder (CTabItem itemSel)
  {
    Composite cSel = (Composite) itemSel.getControl();

    if ( cSel.getChildren() != null && cSel.getChildren().length > 0 )
    {
      CompositeParameterZusammenfassen compParSel = (CompositeParameterZusammenfassen) cSel.getChildren()[0];

      TableViewer tvParameter = compParSel.getTvAnzeige();          
      tvParameter.refresh();
    }
  }
  
  /**
   * Methode erzeugt die Tabs fr die einzelnen Anzeigetypen
   * @param anzeigeTypen Anzeigetypen
   */
  public void erzeugeTabItems( Vector<SystemObject> anzeigeTypen )
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    
    for (SystemObject so : anzeigeTypen)
    {
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );
      tabItem.setText( Tools.getBezeichnungAnzeigeTyp(so) );
      tabItem.setData( Tools.getBezeichnungAnzeigeTyp(so) , so.getPid() );

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
  }
  
  /**
   * Methode blendet die aktuellen Parameter in einem Foldern ein
   * @param so Systemobjekt dessen Folder aktualisiert werden soll
   */
  private void verarbeiteDaten(SystemObject so)
  {
    CTabItem[] items = _tabFolder.getItems();
    
    // Tab zum Systemobject ermitteln
    
    CTabItem item = null;
    for (CTabItem ti : items)
    {
      String pid = (String)ti.getData(ti.getText());

      if (pid == so.getPid())
      {
        item = ti;
        break;
      }
    }

    // Tab gefunden
    
    if (item != null)
    {
      ListeMitRegeln aktuell = _aktuelleParameter.getParameter(so);
      
      Composite c = (Composite)item.getControl();
      
      Control[] children = c.getChildren();

      // Tab neu anlegen, wenn noch leer
      
      if (children.length == 0)
      {
        CompositeParameterZusammenfassen compPar = new CompositeParameterZusammenfassen ( c, SWT.None, aktuell, (String) so.getPid() );
        
        compPar.layout( true );
        c.layout( true );
      }
    }
  }

  /**
   * Methode liefert die aktuell eingestellten Parameter
   * @return aktuell eingestellte Parameter
   */
	public RegelnProAnzeigeTyp getParameter()
  {
		RegelnProAnzeigeTyp regeln = new RegelnProAnzeigeTyp();
		
	  CTabItem[] items = _tabFolder.getItems();
	  for (CTabItem item : items)
	  {
	    Composite c = (Composite)item.getControl();
	    CompositeParameterZusammenfassen compPar = (CompositeParameterZusammenfassen) c.getChildren()[0];

      String pid = (String)item.getData(item.getText());

	    ListeMitRegeln ergebnis = compPar.getParameter();
	    regeln.addParameter(Tools.getAnzeigeTyp(pid), ergebnis);
	  }
	  
	  return regeln;
  }
	
  /**
   * Methode liefert die Tabellenviewer der einzlnen Tabellen	
   * @return Tabellenviewer der einzlnen Tabellen
   */
	public HashMap<String, TableViewer> getTableViewer ()
	{
		HashMap<String, TableViewer> tableViewer = new HashMap<String, TableViewer>();

		CTabItem[] items = _tabFolder.getItems();
	  for (CTabItem item : items)
	  {
	    Composite c = (Composite)item.getControl();
	    CompositeParameterZusammenfassen compPar = (CompositeParameterZusammenfassen) c.getChildren()[0];
	    
	    TableViewer viewer = compPar.getTvAnzeige();
      String pid = (String)item.getData(item.getText());
      
      tableViewer.put(pid, viewer);
	  }
		
		return tableViewer;
	}

	/**
	 * Methode zum bernehmen der Tabellenviewer der einzelnen Tabellen
	 * @param tableViewer zu bernehmende Viewer
	 */
	public void setTableViewer (HashMap<String, TableViewer> tableViewer)
	{
		CTabItem[] items = _tabFolder.getItems();
	  for (CTabItem item : items)
	  {
	    Composite c = (Composite)item.getControl();
	    CompositeParameterZusammenfassen compPar = (CompositeParameterZusammenfassen) c.getChildren()[0];
	    
      String pid = (String)item.getData(item.getText());
	    TableViewer viewer = tableViewer.get(pid);
	    
	    if (viewer != null)
	    	compPar.setTvAnzeige(viewer);
	  }
	}

}
