/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.steuerungskern;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.TimeValue;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;

/**
 * Klasse verwaltet die Parameter Zeitraster
 * @author ObertM
 *
 */
public class VerwaltungParaZeitraster
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungParaZeitraster _instance;

	/**
   * Pid der Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaZeitRaster";
  
  /**
   * Parametersatz
   */
  private List<ParaZeitraster> _parameter = new ArrayList<ParaZeitraster>();
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Pid der SBA
   */
	private String _pidSBA;

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaZeitraster ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();

    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
  }
  
  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaZeitraster getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaZeitraster.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaZeitraster ();
        }
      }
    }
    
    return _instance;
  }

  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
  }

  /**
   * Methode liefert die Parameter Zeitraster fr eine Instanz
   * @param instanz Instanz
   * @return Zeitraster
   */
  public ParaZeitraster getParameter (String instanz)
  {
  	synchronized (_parameter)
    {
	    for (ParaZeitraster para : _parameter)
	    	if (para.getInstanzen().contains (instanz))
	    		return para;
	    
	    return new ParaZeitraster(instanz, 0, 0);
    }
  }

  /**
   * Methode bernimmt das Zeitraster und den Offset fr eine Instanz des Steuerungskerns
   * @param instanz Instanz
   * @param paraZeitraster zu setzende Parameter
   */
  public void setZeitraster (String instanz, ParaZeitraster paraZeitraster)
  {
  	boolean neuAnlegen = true;
  	
  	synchronized (_parameter)
    {
	    for (ParaZeitraster para : _parameter)
	    {
	    	/*
	    	 * Prfen ob die Instanz bereits angelegt ist
	    	 */
	    	if (para.getInstanzen().contains(instanz))
	    	{
	    		if ((para.getZeitraster() == paraZeitraster.getZeitraster()) &&
	    				(para.getOffset()     == paraZeitraster.getOffset()))
	    		{
	    			/*
	    			 * Anzahl stimmt bereits, keine weitere Aktion notwendig
	    			 */
	    			
	    			neuAnlegen = false;
	    		}
	    		else
	    		{
	    			/*
	    			 * Anzahl stimmt nicht, Instanz aus Parametersatz entfernen
	    			 */
	    			
	    			para.getInstanzen().remove(instanz);
	    		}
	    	}
	    	
	    	/*
	    	 * Prfen ob der Wert bereits versorgt ist
	    	 */
	    
	    	if ((para.getZeitraster() == paraZeitraster.getZeitraster()) &&
	    			(para.getOffset()     == paraZeitraster.getOffset()))
	    	{
	    		if (!para.getInstanzen().contains(instanz))
	    			para.getInstanzen().add(instanz);
	    		
	    		neuAnlegen = false;
	    	}
	    }
  	
	  	if (neuAnlegen)
	  	{
	      _parameter.add(new ParaZeitraster(instanz, paraZeitraster.getZeitraster(), paraZeitraster.getOffset()));  		
	  	}
	  	
	  	/*
	  	 * Aufrumen
	  	 */
	  	List<ParaZeitraster> leer = new ArrayList<ParaZeitraster>();
	  	for (ParaZeitraster para : _parameter)
	  		if (para.getInstanzen().size() == 0)
	  			leer.add(para);
	  	
	  	for (ParaZeitraster para : leer)
	  		_parameter.remove(para);
    }
  }
  
  /**
   * Auswerten der vom DaV empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen( ResultData[] results )
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      // Parameter Iterationen
      
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
        auswerteAttributgruppe( dat );

    } // for (int i = 0; i < arg0.length; i++)
  }
  
  /**
   * Methode zum Auswerten der vom DaV empfangenen Attributgruppe 
   * @param data vom DaV empangender Datensatz
   */
  public void auswerteAttributgruppe( ResultData data )
  {
  	synchronized (_parameter)
    {
  		_parameter.clear();
  		
	    if (data.hasData())
	    {
	      Array zuordnungen = data.getData().getArray( "Zuordnungen" );
	      if ((zuordnungen != null) && (zuordnungen.getLength() > 0))
	      {
	        for (int j = 0; j < zuordnungen.getLength(); ++j)
	        {
	          Data zuordnung = zuordnungen.getItem( j );
	
	        	Vector<String> instanzen = new Vector<String>();
	
	          Array funktionen = zuordnung.getArray( "Funktionen" );
	          for (int x = 0; x < funktionen.getLength(); ++x)
	          	instanzen.add(funktionen.getItem(x).asTextValue().getText());
	          
            TimeValue wert = zuordnung.getItem( "ZeitRaster" ).asTimeValue();
            long zeitraster = wert.getMillis();
            zeitraster /= 1000; // In Sekunden umwandeln
        
            long offset = zuordnung.getItem( "Offset" ).asUnscaledValue().longValue();
            if (offset < 0)
              offset = 0;
	          
	          _parameter.add(new ParaZeitraster (instanzen, zeitraster, offset));
	        }
	      }
	    }
    }
    
    benachrichtigeListener();
  }
 
  /**
   * Methode liefert eine Liste mit den versorgten Instanzen
   * @return Liste mit den versorgten Instanzen
   */
  public List<String> getInstanzen ()
  {
  	List<String> l = new ArrayList<String>();
  	
  	synchronized (_parameter)
    {
	    for (ParaZeitraster para : _parameter)
	    {
	    	for (String s : para.getInstanzen())
	    		if (!l.contains(s))
	    			l.add(s);
	    }
    }
  	
  	return l;
  }

  /**
   * Methode zum Erzeugen einer Attributgruppe mit den Parametern
   * @return Attributgruppe mit den Parametern
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;
    
    String pidAtg = _pidAtg;
    
    // Attributgruppe zusammensetzen und versenden

    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(pidAtg);
    Data data = connection.createData(atg );

    Array a = data.getArray("Zuordnungen");
    a.setLength(_parameter.size());
    
    for (int i=0; i<_parameter.size(); ++i)
    {
    	Data d = a.getItem(i);
    	ParaZeitraster p = _parameter.get(i);
    
    	Array f = d.getArray("Funktionen");
    	f.setLength(p.getInstanzen().size());
    	
    	for (int j=0; j<p.getInstanzen().size(); ++j)
    		f.getTextValue(j).setText(p.getInstanzen().get(j));
    	
    	d.getTimeValue("ZeitRaster").setMillis(p.getZeitraster() * 1000); // In Millisekunden umwandeln
    	d.getUnscaledValue("Offset").set(p.getOffset());
    }
    
    return data;
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }

  /**
   * Methode entfernt die Parameter einer Instanz
   * @param instanz zu lschende Instanz
   */
	public void remove(String instanz)
  {
  	synchronized (_parameter)
    {
	    for (ParaZeitraster para : _parameter)
	    	if (para.getInstanzen().contains(instanz))
     			para.getInstanzen().remove(instanz);
    }

  	/*
  	 * Aufrumen
  	 */
  	List<ParaZeitraster> leer = new ArrayList<ParaZeitraster>();
  	for (ParaZeitraster para : _parameter)
  		if (para.getInstanzen().size() == 0)
  			leer.add(para);
  	
  	for (ParaZeitraster para : leer)
  		_parameter.remove(para);
  }

}
