package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.allgemeineVerriegelungsMatrix;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Dambach Werke GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private Vector<RegelAllgemeineVerriegelungsMatrix> m_parameter = new Vector<RegelAllgemeineVerriegelungsMatrix> ();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public ListeMitRegeln ()
  {
  }
  
  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelAllgemeineVerriegelungsMatrix> getParameter()
  {
    Vector<RegelAllgemeineVerriegelungsMatrix> v = new Vector<RegelAllgemeineVerriegelungsMatrix>();
    synchronized (m_parameter)
    {
      for (RegelAllgemeineVerriegelungsMatrix r : m_parameter)
        v.add( r );
    }
    return v;
  }

  /**
   * Methode setzt die Variable parameter
   * @param parameter Wert fr parameter
   */
  public void setParameter(Vector<RegelAllgemeineVerriegelungsMatrix> parameter)
  {
    synchronized (m_parameter)
    {
      m_parameter = parameter;
    } 
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaAllgemeineVerriegelungsMatrixProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    Array a = data.getArray( "Regeln" );
    int anzahl = a.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelAllgemeineVerriegelungsMatrix r = new RegelAllgemeineVerriegelungsMatrix ();
        r.auswerteAttributgruppe( a.getItem( x ), schaltgrundAllgemein );
        
        m_parameter.add( r );
      }
    }
  }

  /**
   * Methode fgt eine Regel hinzu
   * @param xyz zu prfende Zeicehnfolgen 
   * @param richtung Richtung in der die Regel arbeitet
   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
   */
  public void addRegel (Vector<WVZInhalt> xyz, ArbeitsRichtungVN richtung, Aspect schaltgrund )
  {
    synchronized (m_parameter)
    {
      RegelAllgemeineVerriegelungsMatrix r = new RegelAllgemeineVerriegelungsMatrix ();
      
      r.setSchaltgrundRegel( schaltgrund );
      r.setZuPrfendeZeichenFolge( xyz );
      r.setRichtung( richtung );
      
      m_parameter.add( r );
    }
  }

  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      for (RegelAllgemeineVerriegelungsMatrix r : m_parameter)
        s += r.toString() + "\n";
    }
    
    return s;
  }

  /**
   * Methode fgt eine regel hinzu
   * @param regel Regel
   */
	public void addRegel(RegelAllgemeineVerriegelungsMatrix regel)
  {
		synchronized (m_parameter)
    {
      m_parameter.add(regel);	    
    }
  }
}
