package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.spezielleVerriegelungsMatrix;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;

/**
 * Klasse enthlt eine Regel fr die Regelklasse "Verriegelungsmatrix", Regel "Spezielle Verriegelungsmatrix".<br>  
 * @author Swarco Traffic Systems GmbH
 */
public class RegelSpezielleVerriegelungsMatrix
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;
  
  /**
   * Verriegelte Zeichenfolge
   */
  private List<Verriegelung> m_verriegelungen = new ArrayList<Verriegelung>();
  
  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelSpezielleVerriegelungsMatrix ()
  {
  }
  
  /**
   * Konstruktor der Klasse
   * @param verriegelungen Verriegelungen
   * @param schaltgrund Schaltgrund
   */
  public RegelSpezielleVerriegelungsMatrix (List<Verriegelung> verriegelungen, Aspect schaltgrund)
  {
    m_verriegelungen = verriegelungen;
    
    m_schaltgrundRegel = schaltgrund;
  }
  
  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaRegelSpezielleVerriegelungsMatrix'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    
    if (m_schaltgrundRegel == null)
      m_schaltgrundRegel = schaltgrundAllgemein;
    
    Array a = data.getArray( "Verriegelungen" );
    int anzahl = a.getLength();
    
    synchronized (m_verriegelungen)
    {
      m_verriegelungen.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d = a.getItem( x );
        Verriegelung v = new Verriegelung();
        v.auswerteAttributgruppe( d );
        
        m_verriegelungen.add( v );
      }
    }
  }

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * @param schaltgrundRegel Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel ) 
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }
  
  /**
   * Methode liefert die Variable m_verriegelungen zurck
   * @return Variable m_verriegelungen
   */
  public List<Verriegelung> getVerriegelungen()
  {
  	List<Verriegelung> l = new ArrayList<Verriegelung>();
  	
  	synchronized (m_verriegelungen)
    {
	    for (Verriegelung v : m_verriegelungen)
	    	l.add(v);
    }
    return l;
  }

  /**
   * Methode setzt die Variable m_verriegelungen
   * @param verriegelungen Wert auf den die Variable m_verriegelungen gesetzt werden soll
   */
  public void setVerriegelungen( List<Verriegelung> verriegelungen )
  {
  	synchronized (m_verriegelungen)
    {
	    m_verriegelungen.clear();
	    for (Verriegelung v : verriegelungen)
	    	m_verriegelungen.add(v);
    }
  }

  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += "Verriegelungen\n";
    for (Verriegelung v : m_verriegelungen)
      s += "-" + v.toString() + "\n";

    return s;
  }
  
  /**
   * Methode liefert den WVZ Inhalt einer Anzeige sofern diese innerhalb der Verriegelung 
   * versorgt ist. Ansonsten wird null zurckgeliefert
   * @param pidAnzeige Pid der Anzeige
   * @return WVZ Inhalt
   */
  public WVZInhalt getVerriegelung (String pidAnzeige)
  {
  	for (Verriegelung v : m_verriegelungen)
  		if (v.getAnzeige() != null && v.getAnzeige().getPid().equals(pidAnzeige))
  			return v.getWvzInhalt();
  	
  	return null;
  }

  /**
   * Methode prft ob fr eine Anzeige ein Befehl versorgt ist
   * @param anzeige Anzeige
   * @return Information ob fr die Anzeige ein Befehl versorgt ist
   */
	public boolean isAnzeigeVersorgt(SystemObject anzeige)
  {
		return getVerriegelung (anzeige.getPid()) != null;
  }
}
