package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.spezielleVerriegelungsMatrix;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse speichert eine Zuordnung Anzeige - WVZ-Inhalt fr die Parameter
 * 'Spezielle Verriegelungsmatrix'
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 */
public class Verriegelung
{
  /**
   * Anzeige die geprft werden soll.
   */
  private SystemObject m_anzeige;
  
  /**
   * Verriegelter Inhalt der Anzeige
   */
  private WVZInhalt m_wvzInhalt = new WVZInhalt();
  
  /**
   * Parameterloser Konstruktor der Klasse
   */
  public Verriegelung ()
  {
    
  }

  /**
   * Konstruktor der Klasse
   * @param anzeige Anzeige die geprft werden soll
   * @param inhalt Verriegelter Inhalt der Anzeige
   */
  public Verriegelung (SystemObject anzeige, WVZInhalt inhalt)
  {
    m_anzeige = anzeige;
    
    m_wvzInhalt = inhalt;
  }
  
  /**
   * Methode liefert die Variable m_anzeige zurck
   * @return Variable m_anzeige
   */
  public SystemObject getAnzeige()
  {
    return m_anzeige;
  }

  /**
   * Methode setzt die Variable m_anzeige
   * @param anzeige Wert auf den die Variable m_anzeige gesetzt werden soll
   */
  public void setAnzeige( SystemObject anzeige )
  {
    m_anzeige = anzeige;
  }

  /**
   * Methode liefert die Variable m_wvzInhalt zurck
   * @return Variable m_wvzInhalt
   */
  public WVZInhalt getWvzInhalt()
  {
    return m_wvzInhalt;
  }

  /**
   * Methode setzt die Variable m_wvzInhalt
   * @param wvzInhalt Wert auf den die Variable m_wvzInhalt gesetzt werden soll
   */
  public void setWvzInhalt( WVZInhalt wvzInhalt )
  {
    m_wvzInhalt = wvzInhalt;
  }
  
  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaVerriegelungen'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data)
  {
    m_anzeige = data.getReferenceValue( "Anzeige" ).getSystemObject();
    
    m_wvzInhalt = new WVZInhalt();
    
    Data d = data.getItem( "Zeichenfolge" );
    m_wvzInhalt.auswerteAttributgruppe( d );
  }

  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += Tools.getObjektName( m_anzeige ) + " - " + Tools.getWVZInhalt( m_wvzInhalt );
    
    return s;
  }
}
