package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verschiebungen.verschieben;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

/**
 * Tabellen Modell fr die Parameter "Verschieben"
 * @author ObertM
 *
 */
public class TableModel extends Observable implements Observer
{
	/**
	 * Merker zum Lschen der Zeile
	 */
	private Composite _loeschen;
	
	/**
	 * Schaltgrund
	 */
  private Composite _schaltgrund;
  
  /**
   * Arbeitsrichtung
   */
  private Composite _richtung;

  /**
   * Parameter X
   */
  private Composite _x;

  /**
   * Parameter Schaltgrnde
   */
  private Composite _schaltgruende;

  /**
   * Parameter Ebene
   */
  private Composite _ebene;

  /**
   * Parameter abhaengigeAnzeigen
   */
  private Composite _abhaengigeAnzeigen;
  
  /**
   * Konstruktor der Klasse 
   * @param loeschen Lschen
   * @param schaltgrund Schaltgrund der Regel
   * @param richtung Arbeitsrichtung
   * @param x zu prfender Anzeigeinhalt
   * @param schaltgruende Schaltgrnde
   * @param abhaengigeAnzeigen abhngige Anzeigen mit verschieben
   */
  public TableModel(Composite loeschen, Composite schaltgrund, Composite richtung, Composite x, Composite schaltgruende, Composite ebene, Composite abhaengigeAnzeigen)
  {
  	this._loeschen = loeschen;
    this._schaltgrund = schaltgrund;
    this._richtung = richtung;   
    this._x = x;    
    this._schaltgruende = schaltgruende;    
    this._ebene = ebene;
    this._abhaengigeAnzeigen = abhaengigeAnzeigen;    
  }

  /**
   * Methode liefert den eingestellten Wert des Composite Lschen
   * @return eingestellter Wert
   */
  public String getSelectedLoeschen()
  {
    return getSelectedWert(_loeschen);    
  }

  /**
   * Methode liefert den eingestellten Wert des Composite Schaltgrund
   * @return eingestellter Wert
   */
  public String getSelectedSchaltgrund ()
  {
    return getSelectedWert(_schaltgrund);    
  }

  /**
   * Methode liefert den eingestellten Wert des Composite Richtung
   * @return eingestellter Wert
   */
  public String getSelectedRichtung ()
  {
    return getSelectedWert(_richtung);    
  }

  /**
   * Methode liefert den eingestellten Wert des Composite X
   * @return eingestellter Wert
   */
  public String getSelectedX ()
  {
    return getSelectedWert(_x);    
  }

  /**
   * Methode liefert die eingestellten Werte des Composite Schaltgrnde
   * @return eingestellte Werte
   */
  public String[] getSelectedSchaltgruende()
  {
  	Control[] controls = _schaltgruende.getChildren();
  	String [] selected = new String [controls.length];
  	
  	for (int i=0; i<controls.length; ++i)
  	  selected [i] = getSelectedWert(controls [i]);
  	
    return selected;    
  }

  /**
   * Methode liefert die eingestellte Ebene 
   * @return eingestellte Ebene
   */
  public int getSelectedEbene()
  {
    Control control = _ebene.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int ebene = sp.getSelection();
      return ebene;
    }
    else
      return 0;
  }

  /**
   * Methode liefert den eingestellten Wert des Composite Abhngige Anzeigen
   * @return eingestellter Wert
   */
  public String getSelectedAbhaengigeAnzeigen ()
  {
    return getSelectedWert(_abhaengigeAnzeigen);    
  }

  /**
   * Methode setzt den Wert des Compiste Ebene
   * @param selWert zu setzender Wert
   */
  public void setSelectedEbene(String selWert)
  {
    setSelectedWert(_ebene, selWert, null);
  }

  /**
   * Methode setzt den Wert des Compiste Lschen
   * @param selWert zu setzender Wert
   */
  public void setSelectedLoeschen(String selWert)
  {
    setSelectedWert(_loeschen, selWert, null);
  }

  /**
   * Methode setzt den Wert des Compiste Lschen
   * @param selWert zu setzender Wert
   */
  public void setSelectedSchaltgrund(String selWert)
  {
    setSelectedWert(_schaltgrund, selWert, null);
  }

  /**
   * Methode setzt den Wert des Compiste Lschen
   * @param selWert zu setzender Wert
   */
  public void setSelectedRichtung(String selWert)
  {
    setSelectedWert(_richtung, selWert, null);
  }

  /**
   * Methode setzt den Wert des Compiste X
   * @param selWert zu setzender Wert
   */
  public void setSelectedX(Image selImage)
  {
    setSelectedWert(_x, null, selImage);
  }

  /**
   * Methode setzt die Werte des Compiste Schaltgruende
   * @param selWert zu setzender Wert
   */
  public void setSelectedSchaltgruende (String[] selWert)
  {
  	Control[] controls = _schaltgruende.getChildren();
  	
  	for (int i=0; i<controls.length; ++i)
  	  setSelectedWert (controls [i], selWert [i], null);
  }

  /**
   * Methode setzt den Wert des Compiste Abhngige Anzeigen
   * @param selWert zu setzender Wert
   */
  public void setSelectedAbhaengeAnzeigen (String selWert)
  {
    setSelectedWert(_abhaengigeAnzeigen, selWert, null);
  }
  
  /**
   * Methode liefert den eingestellten Wert des ersten Control Elements eines Composite
   * als String zurck.
   * @param c Composite
   * @return Wert als String
   */
  public String getSelectedWert(Composite c)
  {
  	Control control = c.getChildren()[0];

  	return getSelectedWert(control);
  }

  /**
   * Methode liefert den eingestellten Wert eines Control Elements 
   * als String zurck.
   * @param control Control Element
   * @return Wert als String
   */
  public String getSelectedWert(Control control)
  {
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int spSel = sp.getSelection();
      return String.valueOf(spSel);
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      String cbSel = cb.getItem( cb.getSelectionIndex() );
      return cbSel;
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        boolean chSel = ch.getSelection();
        return String.valueOf(chSel);
      }
      else
        return "";
    }
    else if ( control instanceof Label )
    {
      Label im = (Label)control;
      String imSel = (String)im.getData();
      return imSel;
    }
    else
      return "";
  }

  /**
   * Methode setzt den Wert des ersten Control Elements eines Composite
   * @param c Composite
   */
  public void setSelectedWert(Composite c, String selWert, Image selImage)
  {
    Control control = c.getChildren()[0];

    setSelectedWert(control, selWert, selImage);
  }

  /**
   * Methode setzt den Wert eines Control Elements eines Composite
   * @param c Composite
   */
  public void setSelectedWert(Control control, String selWert, Image selImage)
  {
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      try
      {
        sp.setSelection(Integer.valueOf( selWert ));
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      for (int k = 0; k < cb.getItemCount(); k++)
      {
        if (selWert.equals(cb.getItem( k )))
          cb.select(k);      
      }
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        ch.setSelection( Boolean.valueOf( selWert ) );
      }
    }
    else if ( control instanceof Label )
    {
        Label im = (Label)control;
        im.setData( selWert );
        im.setImage( selImage ); 
    }
  }

  @Override
  public void update(Observable o, Object arg)
  {
    fireEvent();
  }

  public void fireEvent()
  {
    setChanged();
    notifyObservers();
  }

	/**
   * Methode liefert loeschen zurck
   * @return loeschen
   */
  public Composite getLoeschen()
  {
  	return _loeschen;
  }

	/**
   * Methode setzt die Variable loeschen
   * @param loeschen zu bernehmender Wert fr loeschen
   */
  public void setLoeschen(Composite loeschen)
  {
  	_loeschen = loeschen;
  }

	/**
   * Methode liefert schaltgrund zurck
   * @return schaltgrund
   */
  public Composite getSchaltgrund()
  {
  	return _schaltgrund;
  }

	/**
   * Methode setzt die Variable schaltgrund
   * @param schaltgrund zu bernehmender Wert fr schaltgrund
   */
  public void setSchaltgrund(Composite schaltgrund)
  {
  	_schaltgrund = schaltgrund;
  }

	/**
   * Methode liefert richtung zurck
   * @return richtung
   */
  public Composite getRichtung()
  {
  	return _richtung;
  }

	/**
   * Methode setzt die Variable richtung
   * @param richtung zu bernehmender Wert fr richtung
   */
  public void setRichtung(Composite richtung)
  {
  	_richtung = richtung;
  }

	/**
   * Methode liefert x zurck
   * @return x
   */
  public Composite getX()
  {
  	return _x;
  }

	/**
   * Methode setzt die Variable x
   * @param x zu bernehmender Wert fr x
   */
  public void setX(Composite x)
  {
  	_x = x;
  }

	/**
   * Methode liefert schaltgruende zurck
   * @return schaltgruende
   */
  public Composite getSchaltgruende()
  {
  	return _schaltgruende;
  }

	/**
   * Methode setzt die Variable schaltgruende
   * @param schaltgruende zu bernehmender Wert fr schaltgruende
   */
  public void setSchaltgruende(Composite schaltgruende)
  {
  	_schaltgruende = schaltgruende;
  }

	/**
   * Methode liefert abhaengigeAnzeigen zurck
   * @return abhaengigeAnzeigen
   */
  public Composite getAbhaengigeAnzeigen()
  {
  	return _abhaengigeAnzeigen;
  }

	/**
   * Methode setzt die Variable abhaengigeAnzeigen
   * @param abhaengigeAnzeigen zu bernehmender Wert fr abhaengigeAnzeigen
   */
  public void setAbhaengigeAnzeigen(Composite abhaengigeAnzeigen)
  {
  	_abhaengigeAnzeigen = abhaengigeAnzeigen;
  }

	/**
   * Methode liefert ebene zurck
   * @return ebene
   */
  public Composite getEbene()
  {
  	return _ebene;
  }

	/**
   * Methode setzt die Variable ebene
   * @param ebene zu bernehmender Wert fr ebene
   */
  public void setEbene(Composite ebene)
  {
  	_ebene = ebene;
  }
}
