package de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen;

import java.util.HashMap;
import java.util.Vector;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.abhaengigkeiten.Abhaengigkeiten;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen.SBA;

public class DefaultEinstellungen
{
  /**
   * Methode liefert einen Vektor mit den abhngigen Anzeigen fr einen AQ
   * @param soAQ Systemobjekt des AQ
   * @return Vektor mit Abhngigkeiten
   */
  public static Vector<Abhaengigkeiten> getDefaultAbhaengigkeiten (SystemObject soAQ)
  {
		IProjektEinstellungen ipe = getProjektEinstellungen();
		if (ipe != null)
		{
			return ipe.getDefaultAbhaengigkeiten(soAQ);
		}
		
		return new Vector<Abhaengigkeiten>();
  }
  
  /**
   * Methode liefert eine Feld mit Systemobjektend er Anzeigen die die Defaultquerbeziehung
   * eines AQ darstellen. Die Richtung ist dabei von links nach rechts und von oben nach unten.
   * @param soAQ Systemobejkt des AQ
   * @return Feld mit Systemobjekten
   */
  public static SystemObject[] getDefaultQuerBeziehung(SystemObject soAQ)
  {
		IProjektEinstellungen ipe = getProjektEinstellungen();
		if (ipe != null)
		{
			return ipe.getDefaultQuerBeziehung(soAQ);
		}
		
		return new SystemObject [0];
  }

	/**
	 * Methode liefert eine Vector mit den im Projekt zu verwendenden Anzeigetypen
	 * @return Liste mit Anzeigetypen
	 */
  public static Vector<AnzeigeTypEigenschaften> getDefaultAnzeigeTypen ()
  {
		IProjektEinstellungen ipe = getProjektEinstellungen();
		if (ipe != null)
		{
			return ipe.getDefaultAnzeigeTypen();
		}
		
		return new Vector<AnzeigeTypEigenschaften>();
  }

	/**
	 * Methode liefert eine Hashmap mit den verwendeten Anzeigequerschnitten.
	 * @return Hashmap mit den Anzeigequerschnitten, als Key dient die Pid, als Value der Typ des Anzeigequerschnitts
	 */
  public static HashMap<String, EAnzeigeQuerschnittTyp> getDefaultAnzeigeQuerschnitte ()
  {
		IProjektEinstellungen ipe = getProjektEinstellungen();
		if (ipe != null)
		{
			return ipe.getDefaultAnzeigeQuerschnitte();
		}
		
		return new HashMap<String, EAnzeigeQuerschnittTyp>();
  }

  /**
   * Methode liefert die Instanz mit den projektspezifischen Einstellungen in
   * Abhngigkeit von der aktuellen SBA
   * @return Instanz
   */
  public static IProjektEinstellungen getProjektEinstellungen ()
  {
		IProjektEinstellungen ipe = null;
		
		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A14)
		{
			ipe = new DefaultEinstellungenA14();
		}

		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.B27)
		{
			ipe = new DefaultEinstellungenB27();
		}

		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A8)
		{
			ipe = new DefaultEinstellungenA8();
		}

		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A81)
		{
			ipe = new DefaultEinstellungenA81();
		}

		if (ProjektSpezifischeEinstellungen.getInstance().getSba() == SBA.A2)
		{
			ipe = new DefaultEinstellungenA2();
		}

		return ipe;
  }
}
