package de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.abhaengigkeiten.Abhaengigkeiten;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse definiert die Defaulteinstellungen fr die A14. Diese Einstellungen werden nur
 * verwendet, wenn ber den Datenverteiler keine anderen Parameter versorgt sind
 * (atg.sswSbaAnzeigeTypMitWVZInhalt).
 * 
 * @author ObertM
 * 
 */
public class DefaultEinstellungenA14 implements IProjektEinstellungen
{
	/**
	 * Prfix mit der die Pid der AnzeigeTypen (typ.sswSbaAnzeigeTyp) beginnen
	 */
	private static final String _praefixAnzeigeTypen = "sba.anzeigeTyp.A14";

	/**
	 * Prfix mit der die Pid der WvzInhalte (typ.wvzInhalt) beginnen
	 */
	private static final String _praefixWvzInhalte = "wvzInhalt.uz.a14";

	/**
	 * Prfix mit der die Pid der WvzInhalte typ.anzeigeQuerschnitt) beginnen
	 */
	private static final String _praefixAnzeigeQuerschnitt = "aq.A14";

	/**
	 * Methode liefert eine Vector mit den im Projekt zu verwendenden Anzeigetypen
	 * @return Liste mit Anzeigetypen
	 */
	public Vector<AnzeigeTypEigenschaften> getDefaultAnzeigeTypen()
	{
		Vector<AnzeigeTypEigenschaften> l = new Vector<AnzeigeTypEigenschaften>();

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_A,
		    _praefixAnzeigeTypen + ".TypA", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_WZG_A)));

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_B,
		    _praefixAnzeigeTypen + ".TypB", _praefixWvzInhalte + ".Dunkel",
        getWvzInhalte(EAnzeigeTypen.TYP_WZG_B)));

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_C,
		    _praefixAnzeigeTypen + ".TypC", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_WZG_C)));

		return l;
	};

	/**
	 * Methode liefert die Pids der zu verwendenden WvzInhalte (typ.wvzInhalt) in
	 * Abhngigkeit des Anzeigetyps.
	 * 
	 * @param typ
	 *          Anzeigetyp
	 * @return Liste mit Pids
	 */
	private List<String> getWvzInhalte(EAnzeigeTypen typ)
	{
		String praefix = _praefixWvzInhalte;

		List<String> list = new ArrayList<String>();

		if (typ == EAnzeigeTypen.TYP_WZG_A)
		{
			list.add(praefix + ".Dunkel");
			list.add(praefix + ".tlsFahrstreifenGesperrt");
			list.add(praefix + ".tlsFahrstreifenWechselnNachLinks");
			list.add(praefix + ".tlsFahrstreifenWechselnNachRechts");
			list.add(praefix + ".tlsZulaessigeHoechst40");
			list.add(praefix + ".tlsZulaessigeHoechst50");
			list.add(praefix + ".tlsZulaessigeHoechst60");
			list.add(praefix + ".tlsZulaessigeHoechst80");
			list.add(praefix + ".tlsZulaessigeHoechst100");
			list.add(praefix + ".tlsZulaessigeHoechst120");
			list.add(praefix + ".tlsZulaessigeHoechst130");

			list.add(praefix + ".tlsEndeZulaessigeHoechst40");
			list.add(praefix + ".tlsEndeZulaessigeHoechst50");
			list.add(praefix + ".tlsEndeZulaessigeHoechst60");
			list.add(praefix + ".tlsEndeZulaessigeHoechst80");
			list.add(praefix + ".tlsEndeZulaessigeHoechst100");
			list.add(praefix + ".tlsEndeZulaessigeHoechst120");
			list.add(praefix + ".tlsEndeZulaessigeHoechst130");

		} 
		else if (typ == EAnzeigeTypen.TYP_WZG_B)
		{
			list.add(praefix + ".Dunkel");
			list.add(praefix + ".tlsGefahrenstelle");
			list.add(praefix + ".tlsGefahrenstelle");
			list.add(praefix + ".tlsSchleudergefahr");
			list.add(praefix + ".tlsBaustelle");
			list.add(praefix + ".tlsStau");
			list.add(praefix + ".tlsUeberholverbotAllgemein");
			list.add(praefix + ".tlsUeberholverbotLkw");
			list.add(praefix + ".tlsEndeUeberholverbotAllgemein");
			list.add(praefix + ".tlsEndeUeberholverbotLkw");
			list.add(praefix + ".tlsEndeAllerStreckenverbote");
		} 
		else if (typ == EAnzeigeTypen.TYP_WZG_C)
		{
			list.add(praefix + ".Dunkel");
			list.add(praefix + ".tlsStau");
			list.add(praefix + ".tlsStaugefahr");
			list.add(praefix + ".tlsNebel");
			list.add(praefix + ".tlsNaesse");
			list.add(praefix + ".tlsUnfall");
			list.add(praefix + ".tlsSicht");
			list.add(praefix + ".tls500Meter");
			list.add(praefix + ".tls1000Meter");
			list.add(praefix + ".tls1500Meter");
			list.add(praefix + ".tls2000Meter");
			list.add(praefix + ".tls2500Meter");
			list.add(praefix + ".tlsAuf500Meter");
			list.add(praefix + ".tlsAuf1000Meter");
			list.add(praefix + ".tlsAuf1500Meter");
			list.add(praefix + ".tlsAuf2000Meter");
			list.add(praefix + ".tlsAuf2500Meter");
			list.add(praefix + ".tls7Komma5Tonnen");
			list.add(praefix + ".tls-asGlaette");
			list.add(praefix + ".tlsOelspur");
		}

		return list;
	}

	@Override
	public String getPidBeibehalten()
	{
		return _praefixWvzInhalte + ".Beibehalten";
	}

	@Override
	public String getPidBeliebig()
	{
		return _praefixWvzInhalte + ".Beliebig";
	}

	@Override
	public String getPidDunkel()
	{
		return _praefixWvzInhalte + ".Dunkel";
	}

	@Override
  public String getPidDontCare()
  {
		return _praefixWvzInhalte + ".DontCare";
  }
	
	@Override
  public HashMap<String, EAnzeigeQuerschnittTyp> getDefaultAnzeigeQuerschnitte ()
  {
  	HashMap<String, EAnzeigeQuerschnittTyp> zuordnungen = new HashMap<String, EAnzeigeQuerschnittTyp>();

  	// Liste aller Anzeigequerschnitte
  	
  	List<SystemObject> objekte = Tools.getSystemObjekte("typ.anzeigeQuerschnitt");
  	for (SystemObject so : objekte)
  		if (so.getPid().startsWith(_praefixAnzeigeQuerschnitt))
  			zuordnungen.put(so.getPid(), EAnzeigeQuerschnittTyp.A14_TYP1);

  	return zuordnungen;
  }

	@Override
  public Vector<Abhaengigkeiten> getDefaultAbhaengigkeiten(SystemObject soAQ)
  {
  	SystemObject[] anzeigen = Tools.getSystemObjekteAnzeigen(soAQ);
  	
		Vector<Abhaengigkeiten> v = new Vector<Abhaengigkeiten>();

		v.add (new Abhaengigkeiten(anzeigen [2], anzeigen [3])); // B/C

		return v;
  }

	@Override
  public SystemObject[] getDefaultQuerBeziehung(SystemObject soAQ)
  {
  	SystemObject[] anzeigen = Tools.getSystemObjekteAnzeigen(soAQ);

  	Vector<SystemObject> reihenfolge = new Vector<SystemObject>();
  	
  	reihenfolge.add(anzeigen[1]);  // A links
  	reihenfolge.add(anzeigen[2]);  // B
  	reihenfolge.add(anzeigen[3]);  // C
  	reihenfolge.add(anzeigen[0]);  // A rechts
  	
  	return reihenfolge.toArray(new SystemObject[anzeigen.length]);
  }
}
