package de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungMoeglicheAnzeigeQuerschnitte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.abhaengigkeiten.Abhaengigkeiten;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;


/**
 * Klasse definiert die Defaulteinstellungen fr die A8. Diese Einstellungen werden nur
 * verwendet, wenn ber den Datenverteiler keine anderen Parameter versorgt sind
 * (atg.sswSbaAnzeigeTypMitWVZInhalt).
 * 
 * @author ObertM
 * 
 */
public class DefaultEinstellungenA8 implements IProjektEinstellungen
{
	/**
	 * Prfix mit der die Pid der AnzeigeTypen (typ.sswSbaAnzeigeTyp) beginnen
	 */
	private static final String _praefixAnzeigeTypen = "sba.anzeigeTyp.A8.sba_stgt";

	/**
	 * Prfix mit der die Pid der WvzInhalte (typ.wvzInhalt) beginnen
	 */
	private static final String _praefixWvzInhalte = "wvzInhalt.uz.A8.sba_stgt";

	/**
	 * Prfix mit der die Pid der WvzInhalte typ.anzeigeQuerschnitt) beginnen
	 */
	private static final String _praefixAnzeigeQuerschnitt = "aq.A8";
	
	/**
	 * Methode liefert eine Vector mit den im Projekt zu verwendenden Anzeigetypen
	 * @return Liste mit Anzeigetypen
	 */
	public Vector<AnzeigeTypEigenschaften> getDefaultAnzeigeTypen()
	{
		Vector<AnzeigeTypEigenschaften> l = new Vector<AnzeigeTypEigenschaften>();

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_A,
		    _praefixAnzeigeTypen + ".TypA", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_WZG_A)));

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_B,
		    _praefixAnzeigeTypen + ".TypB", _praefixWvzInhalte + ".Dunkel",
        getWvzInhalte(EAnzeigeTypen.TYP_WZG_B)));

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_C,
		    _praefixAnzeigeTypen + ".TypC", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_WZG_C)));


		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_D,
		    _praefixAnzeigeTypen + ".TypD", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_WZG_D)));


		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_BLINKER,
		    _praefixAnzeigeTypen + ".Blinker", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_BLINKER)));


		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_BAQ,
		    _praefixAnzeigeTypen + ".TypBAQ", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_BAQ)));


		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_TAQ,
		    _praefixAnzeigeTypen + ".TypTAQ", _praefixWvzInhalte + ".tlsTAQNeutral",
		    getWvzInhalte(EAnzeigeTypen.TYP_TAQ)));

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WAQ,
		    _praefixAnzeigeTypen + ".TypWAQ", _praefixWvzInhalte + ".tlsWAQPrismaSeite1",
		    getWvzInhalte(EAnzeigeTypen.TYP_WAQ)));

		return l;
	};

	/**
	 * Methode liefert die Pids der zu verwendenden WvzInhalte (typ.wvzInhalt) in Abhngigkeit 
	 * des Anzeigetyps.
	 * @param typ Anzeigetyp
	 * @return Liste mit Pids
	 */
	private List<String> getWvzInhalte (EAnzeigeTypen typ) 
  {
		String praefix = _praefixWvzInhalte;

		List<String> list = new ArrayList<String>();
		
		if (typ == EAnzeigeTypen.TYP_WZG_A)  
		{
			list.add(praefix + ".Dunkel");
			list.add(praefix + ".tlsDunkelStau");
			list.add(praefix + ".DunkelHP");

			list.add(praefix + ".tlsEndeZulaessigeHoechst120");
			list.add(praefix + ".tlsEndeZulaessigeHoechst100");
			list.add(praefix + ".tlsEndeZulaessigeHoechst80");
			list.add(praefix + ".tlsEndeZulaessigeHoechst60");
			list.add(praefix + ".tlsEndeZulaessigeHoechst40");

			list.add(praefix + ".tlsZulaessigeHoechst120");
			list.add(praefix + ".tlsZulaessigeHoechst100");
			list.add(praefix + ".tlsZulaessigeHoechst80");
			list.add(praefix + ".tlsZulaessigeHoechst60");
			list.add(praefix + ".tlsZulaessigeHoechst40");

			list.add(praefix + ".tlsFahrstreifenGesperrt");
			list.add(praefix + ".tlsFahrstreifenWechselnNachLinks");
			list.add(praefix + ".tlsFahrstreifenWechselnNachRechts");
			
			// negierte Inhalte
			
			list.add(praefix + ".nichtDunkel");
			list.add(praefix + ".nichtTlsDunkelStau");
			list.add(praefix + ".nichtDunkelHP");

			list.add(praefix + ".nichtTlsEndeZulaessigeHoechst120");
			list.add(praefix + ".nichtTlsEndeZulaessigeHoechst100");
			list.add(praefix + ".nichtTlsEndeZulaessigeHoechst80");
			list.add(praefix + ".nichtTlsEndeZulaessigeHoechst60");
			list.add(praefix + ".nichtTlsEndeZulaessigeHoechst40");

			list.add(praefix + ".nichtTlsZulaessigeHoechst120");
			list.add(praefix + ".nichtTlsZulaessigeHoechst100");
			list.add(praefix + ".nichtTlsZulaessigeHoechst80");
			list.add(praefix + ".nichtTlsZulaessigeHoechst60");
			list.add(praefix + ".nichtTlsZulaessigeHoechst40");

			list.add(praefix + ".nichtTlsFahrstreifenGesperrt");
			list.add(praefix + ".nichtTlsFahrstreifenWechselnNachLinks");
			list.add(praefix + ".nichtTlsFahrstreifenWechselnNachRechts");
		} 
		else if (typ == EAnzeigeTypen.TYP_WZG_B)  
		{
			list.add(praefix + ".Dunkel");
			list.add(praefix + ".tlsDunkelStau");
			list.add(praefix + ".DunkelHP");

			list.add(praefix + ".tlsEndeUeberholverbotLkw");
			list.add(praefix + ".tlsEndeUeberholverbotAllgemein");
			list.add(praefix + ".tlsEndeAllerStreckenverbote");

			list.add(praefix + ".tlsUeberholverbotLkw");
			list.add(praefix + ".tlsUeberholverbotAllgemein");

			list.add(praefix + ".tlsGlaettegefahr");
			list.add(praefix + ".tlsSchleudergefahr");
			list.add(praefix + ".tlsStau");
			list.add(praefix + ".tlsBaustelle");
			list.add(praefix + ".tlsGefahrenstelle");
			
			// negierte Inhalte
			
			list.add(praefix + ".nichtDunkel");
			list.add(praefix + ".nichtTlsDunkelStau");
			list.add(praefix + ".nichtDunkelHP");

			list.add(praefix + ".nichtTlsEndeUeberholverbotLkw");
			list.add(praefix + ".nichtTlsEndeUeberholverbotAllgemein");
			list.add(praefix + ".nichtTlsEndeAllerStreckenverbote");

			list.add(praefix + ".nichtTlsUeberholverbotLkw");
			list.add(praefix + ".nichtTlsUeberholverbotAllgemein");

			list.add(praefix + ".nichtTlsGlaettegefahr");
			list.add(praefix + ".nichtTlsSchleudergefahr");
			list.add(praefix + ".nichtTlsStau");
			list.add(praefix + ".nichtTlsBaustelle");
			list.add(praefix + ".nichtTlsGefahrenstelle");

		} 
		
		else if (typ == EAnzeigeTypen.TYP_WZG_C)  
		{
			list.add(praefix + ".Dunkel");
			list.add(praefix + ".tlsDunkelStau");
			list.add(praefix + ".DunkelHP");

			list.add(praefix + ".tlsStau");
			list.add(praefix + ".tlsStaugefahr");
			list.add(praefix + ".tlsUnfall");
			list.add(praefix + ".tlsSicht");
			list.add(praefix + ".tls500Meter");
			list.add(praefix + ".tls1000Meter");
			list.add(praefix + ".tls1500Meter");
			list.add(praefix + ".tls2000Meter");
			list.add(praefix + ".tls2500Meter");
			list.add(praefix + ".tls3000Meter");

			list.add(praefix + ".tls1Kilometer");
			list.add(praefix + ".tls1Komma5Kilometer");
			list.add(praefix + ".tls2Kilometer");
			list.add(praefix + ".tls2Komma5Kilometer");
			list.add(praefix + ".tls3Kilometer");

			list.add(praefix + ".tlsAuf500Meter");
			list.add(praefix + ".tlsAuf1000Meter");
			list.add(praefix + ".tlsAuf1500Meter");
			list.add(praefix + ".tlsAuf2000Meter");
			list.add(praefix + ".tlsAuf2500Meter");

			list.add(praefix + ".tls7Komma5Tonnen");
			list.add(praefix + ".tlsTiere");
			list.add(praefix + ".tlsKontrolle");
			list.add(praefix + ".tlsOelspur");
			
			// negierte Inhalte
			
			list.add(praefix + ".nichtDunkel");
			list.add(praefix + ".nichtTlsDunkelStau");
			list.add(praefix + ".nichtDunkelHP");

			list.add(praefix + ".nichtTlsStau");
			list.add(praefix + ".nichtTlsStaugefahr");
			list.add(praefix + ".nichtTlsUnfall");
			list.add(praefix + ".nichtTlsSicht");
			list.add(praefix + ".nichtTls500Meter");
			list.add(praefix + ".nichtTls1000Meter");
			list.add(praefix + ".nichtTls1500Meter");
			list.add(praefix + ".nichtTls2000Meter");
			list.add(praefix + ".nichtTls2500Meter");
			list.add(praefix + ".nichtTls3000Meter");

			list.add(praefix + ".nichtTls1Kilometer");
			list.add(praefix + ".nichtTls1Komma5Kilometer");
			list.add(praefix + ".nichtTls2Kilometer");
			list.add(praefix + ".nichtTls2Komma5Kilometer");
			list.add(praefix + ".nichtTls3Kilometer");

			list.add(praefix + ".nichtTlsAuf500Meter");
			list.add(praefix + ".nichtTlsAuf1000Meter");
			list.add(praefix + ".nichtTlsAuf1500Meter");
			list.add(praefix + ".nichtTlsAuf2000Meter");
			list.add(praefix + ".nichtTlsAuf2500Meter");

			list.add(praefix + ".nichtTls7Komma5Tonnen");
			list.add(praefix + ".nichtTlsTiere");
			list.add(praefix + ".nichtTlsKontrolle");
			list.add(praefix + ".nichtTlsOelspur");
		} 

		else if (typ == EAnzeigeTypen.TYP_BLINKER)  
		{
			list.add(praefix + ".Dunkel");

			list.add(praefix + ".tlsGelbesBlinklichtSynchron");
			list.add(praefix + ".tlsGelbesBlinklichtAsynchron");
			
			// negierte Inhalte
			
			list.add(praefix + ".nichtDunkel");

			list.add(praefix + ".nichtTlsGelbesBlinklichtSynchron");
			list.add(praefix + ".nichtTlsGelbesBlinklichtAsynchron");
		} 

		else if (typ == EAnzeigeTypen.TYP_WZG_D)  
		{
			list.add(praefix + ".Dunkel");
			
			list.add(praefix + ".tlsFahrstreifenHalten");
			
			// negierte Inhalte
			
			list.add(praefix + ".nichtDunkel");

			list.add(praefix + ".nichtTlsFahrstreifenHalten");
		}

		else if (typ == EAnzeigeTypen.TYP_BAQ)  
		{
			list.add(praefix + ".Dunkel");

			list.add(praefix + ".tlsBAQBlockAbfgLkwStop");
			list.add(praefix + ".tlsBAQBlockAbfgLkw");
			
			// negierte Inhalte
			
			list.add(praefix + ".nichtDunkel");

			list.add(praefix + ".nichtTlsBAQBlockAbfgLkwStop");
			list.add(praefix + ".nichtTlsBAQBlockAbfgLkw");

		} 

		else if (typ == EAnzeigeTypen.TYP_TAQ)  
		{
			list.add(praefix + ".tlsTAQNeutral");
			list.add(praefix + ".tlsTAQSeitenstreifenBefahren");
			list.add(praefix + ".tlsTAQSeitenstreifenNichtBefahren");
			list.add(praefix + ".tlsTAQSeitenstreifenRaeumen");
			list.add(praefix + ".tlsTAQNeutralSeite3");
			
			// negierte Inhalte

			list.add(praefix + ".nichtTlsTAQNeutral");
			list.add(praefix + ".nichtTlsTAQSeitenstreifenBefahren");
			list.add(praefix + ".nichtTlsTAQSeitenstreifenNichtBefahren");
			list.add(praefix + ".nichtTlsTAQSeitenstreifenRaeumen");
			list.add(praefix + ".nichtTlsTAQNeutralSeite3");

		} 

		else if (typ == EAnzeigeTypen.TYP_WAQ)  
		{
			list.add(praefix + ".tlsWAQPrismaSeite1");
			list.add(praefix + ".tlsWAQPrismaSeite2");
			list.add(praefix + ".tlsWAQPrismaSeite3");
			list.add(praefix + ".tlsWAQPrismaSeite4");
			
			// negierte Inhalte

			list.add(praefix + ".nichtTlsWAQPrismaSeite1");
			list.add(praefix + ".nichtTlsWAQPrismaSeite2");
			list.add(praefix + ".nichtTlsWAQPrismaSeite3");
			list.add(praefix + ".nichtTlsWAQPrismaSeite4");

		} 

		return list;
  }
	
	@Override
  public String getPidBeibehalten()
  {
	  return _praefixWvzInhalte + ".Beibehalten";
  }

	@Override
  public String getPidBeliebig()
  {
	  return _praefixWvzInhalte + ".Beliebig";
  }

	@Override
  public String getPidDunkel()
  {
	  return _praefixWvzInhalte + ".Dunkel";
  }

	@Override
  public String getPidDontCare()
  {
		return _praefixWvzInhalte + ".DontCare";
  }

	@Override
  public HashMap<String, EAnzeigeQuerschnittTyp> getDefaultAnzeigeQuerschnitte ()
  {
  	HashMap<String, EAnzeigeQuerschnittTyp> zuordnungen = new HashMap<String, EAnzeigeQuerschnittTyp>();

  	// Liste aller Anzeigequerschnitte
  	
  	List<SystemObject> objekte = Tools.getSystemObjekte("typ.anzeigeQuerschnitt");
  	for (SystemObject so : objekte)
  		if (so.getPid().startsWith(_praefixAnzeigeQuerschnitt))
  			zuordnungen.put(so.getPid(), EAnzeigeQuerschnittTyp.UNDEFINIERT);
  		
    // Zuordnen der Anzeigetypen berschreiben

  	zuordnungen.put("aq.A8.6053.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP11d);
  	zuordnungen.put("aq.A8.6052.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP13);
  	zuordnungen.put("aq.A8.6051.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP9);
  	zuordnungen.put("aq.A8.6050.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP7);
  	zuordnungen.put("aq.A8.6049.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP7);
  	zuordnungen.put("aq.A8.6048.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP7);
  	zuordnungen.put("aq.A8.6048.EIN.O",     EAnzeigeQuerschnittTyp.A8_TYP8);
  	zuordnungen.put("aq.A8.6047.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP10d);
  	zuordnungen.put("aq.A8.6046.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP10d);
  	zuordnungen.put("aq.A8.6044.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP16);
  	zuordnungen.put("aq.A8.6041.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP14);
  	zuordnungen.put("aq.A8.6039.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP17);
  	zuordnungen.put("aq.A8.6037.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP15);
  	zuordnungen.put("aq.A8.6033.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP14d);
  	zuordnungen.put("aq.A8.6032.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP17);
  	zuordnungen.put("aq.A8.6030.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP18);
  	zuordnungen.put("aq.A8.6029.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6028.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6026.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6021.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP6d);
  	zuordnungen.put("aq.A8.6017.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6012.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6006.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6005.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP6d);
  	zuordnungen.put("aq.A8.6005.BAQ.O",     EAnzeigeQuerschnittTyp.A8_TYP19);
  	zuordnungen.put("aq.A8.6003.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6002.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6001.HFB.O",     EAnzeigeQuerschnittTyp.A8_TYP6d);

  	zuordnungen.put("aq.A8.6101.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6d);
  	zuordnungen.put("aq.A8.6103.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6104.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6d);
  	zuordnungen.put("aq.A8.6106.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6107.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP11);
  	zuordnungen.put("aq.A8.6108.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6109.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6d);
  	zuordnungen.put("aq.A8.6110.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6111.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6112.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6113.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP7d);
  	zuordnungen.put("aq.A8.6114.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP7d);
  	zuordnungen.put("aq.A8.6115.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6115.EIN.W",     EAnzeigeQuerschnittTyp.A8_TYP8);
  	zuordnungen.put("aq.A8.6116.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6117.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP12d);
  	zuordnungen.put("aq.A8.6118.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP16);
  	zuordnungen.put("aq.A8.6122.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP14);
  	zuordnungen.put("aq.A8.6125.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP14);
  	zuordnungen.put("aq.A8.6126.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP18);
  	zuordnungen.put("aq.A8.6127.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP7d);
  	zuordnungen.put("aq.A8.6128.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6128.EIN.W",     EAnzeigeQuerschnittTyp.A8_TYP13);
  	zuordnungen.put("aq.A8.6130.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6d);
  	zuordnungen.put("aq.A8.6131.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6132.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP6);
  	zuordnungen.put("aq.A8.6133.HFB.W",     EAnzeigeQuerschnittTyp.A8_TYP7d);

   	// prfen ob alle Querschniite versorgt wurden
   	
   	Iterator<Entry<String, EAnzeigeQuerschnittTyp>> it = zuordnungen.entrySet().iterator();
   	while (it.hasNext())
   	{
   		Entry<String, EAnzeigeQuerschnittTyp> entry = it.next();
   		
   		if (entry.getValue() == EAnzeigeQuerschnittTyp.UNDEFINIERT)
   			System.err.println("Fehler: fr Anzeigequerschnitt " + entry.getKey() + 
   					" wurde kein Anzeigequerschnitttyp definiert !");
   	}

  	return zuordnungen;
  }

	@Override
  public Vector<Abhaengigkeiten> getDefaultAbhaengigkeiten(SystemObject soAQ)
  {
  	EAnzeigeQuerschnittTyp typ = VerwaltungMoeglicheAnzeigeQuerschnitte.getAnzeigeQuerschnittTyp(soAQ);
  	
  	SystemObject[] anzeigen = Tools.getSystemObjekteAnzeigen(soAQ);
  	
		Vector<Abhaengigkeiten> v = new Vector<Abhaengigkeiten>();

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP1 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP6 ) ||
   			(typ == EAnzeigeQuerschnittTyp.A8_TYP6d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [4], anzeigen [6])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [3], anzeigen [5])); // B/C rechts
  	}
  	
  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP2 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP7 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP7d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [6], anzeigen [9])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [5], anzeigen [8])); // B/C mitte
  		v.add (new Abhaengigkeiten(anzeigen [4], anzeigen [7])); // B/C rechts
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP3) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP8))
  	{
  		// keine B/C Kombination
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP4) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP9))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [5], anzeigen [7])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [4], anzeigen [6])); // B/C rechts
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP5  ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP10 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP10d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [ 8], anzeigen [12])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [ 7], anzeigen [11])); // B/C mitte links
  		v.add (new Abhaengigkeiten(anzeigen [ 6], anzeigen [10])); // B/C mitte rechts
  		v.add (new Abhaengigkeiten(anzeigen [ 5], anzeigen [ 9])); // B/C rechts
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP11 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP11d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [5], anzeigen [4])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [7], anzeigen [6])); // B/C rechts
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP12 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP12d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [ 7], anzeigen [10])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [ 6], anzeigen [ 9])); // B/C mitte
  		v.add (new Abhaengigkeiten(anzeigen [ 5], anzeigen [ 8])); // B/C rechts
  	}

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP13 )
  	{
  		v.add (new Abhaengigkeiten(anzeigen [ 2], anzeigen [ 3])); // B/C 
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP14 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP14d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [ 6], anzeigen [ 9])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [ 5], anzeigen [ 8])); // B/C mitte
  		v.add (new Abhaengigkeiten(anzeigen [ 4], anzeigen [ 7])); // B/C rechts
  	}

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP15 )
  	{
  		v.add (new Abhaengigkeiten(anzeigen [ 6], anzeigen [ 9])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [ 5], anzeigen [ 8])); // B/C mitte
  		v.add (new Abhaengigkeiten(anzeigen [ 4], anzeigen [ 7])); // B/C rechts
  	}

  	return v;
  }

	@Override
  public SystemObject[] getDefaultQuerBeziehung(SystemObject soAQ)
  {
  	EAnzeigeQuerschnittTyp typ = VerwaltungMoeglicheAnzeigeQuerschnitte.getAnzeigeQuerschnittTyp(soAQ);
  	
  	SystemObject[] anzeigen = Tools.getSystemObjekteAnzeigen(soAQ);

  	Vector<SystemObject> reihenfolge = new Vector<SystemObject>(); 
  	   
  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP1 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP6 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP6d))
  	{
  		reihenfolge.add (anzeigen [2]); // A links
  		reihenfolge.add (anzeigen [4]); // B links
  		reihenfolge.add (anzeigen [6]); // C links
  		reihenfolge.add (anzeigen [1]); // A mitte
  		reihenfolge.add (anzeigen [3]); // B rechts
  		reihenfolge.add (anzeigen [5]); // C rechts
  		reihenfolge.add (anzeigen [0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP6d)
    		reihenfolge.add (anzeigen [7]); // D2
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP2 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP7 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP7d))
  	{
  		reihenfolge.add (anzeigen [3]); // A links
  		reihenfolge.add (anzeigen [6]); // B links
  		reihenfolge.add (anzeigen [9]); // C links
  		reihenfolge.add (anzeigen [2]); // A mitte links
  		reihenfolge.add (anzeigen [5]); // B mitte
  		reihenfolge.add (anzeigen [8]); // C mitte
  		reihenfolge.add (anzeigen [1]); // A mitte rechts
  		reihenfolge.add (anzeigen [4]); // B rechts
  		reihenfolge.add (anzeigen [7]); // C rechts
  		reihenfolge.add (anzeigen [0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP7d)
    		reihenfolge.add (anzeigen [10]); // D2
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP3) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP8))
  	{
  		reihenfolge.add (anzeigen [0]); // A
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP4) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP9))
  	{
  		reihenfolge.add (anzeigen [3]); // A links
  		reihenfolge.add (anzeigen [5]); // B links
  		reihenfolge.add (anzeigen [7]); // C links
  		reihenfolge.add (anzeigen [2]); // A mitte links
  		
  		reihenfolge.add (anzeigen [1]); // A mitte rechts
  		reihenfolge.add (anzeigen [4]); // B rechts
  		reihenfolge.add (anzeigen [6]); // C rechts
  		reihenfolge.add (anzeigen [0]); // A rechts
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP5  ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP10 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP10d))
  	{
  		reihenfolge.add (anzeigen [ 4]); // A links
  		reihenfolge.add (anzeigen [ 8]); // B links
  		reihenfolge.add (anzeigen [12]); // C links
  		reihenfolge.add (anzeigen [ 3]); // A mitte links
  		reihenfolge.add (anzeigen [ 7]); // B mitte links
  		reihenfolge.add (anzeigen [11]); // C mitte links
  		reihenfolge.add (anzeigen [ 2]); // A mitte 
  		reihenfolge.add (anzeigen [ 6]); // B mitte rechts
  		reihenfolge.add (anzeigen [10]); // C mitte rechts
  		reihenfolge.add (anzeigen [ 1]); // A mitte rechts
  		reihenfolge.add (anzeigen [ 5]); // B rechts
  		reihenfolge.add (anzeigen [ 9]); // C rechts
  		reihenfolge.add (anzeigen [ 0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP10d)
    		reihenfolge.add (anzeigen [13]); // D2
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP11 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP11d))
  	{
  		reihenfolge.add (anzeigen [ 3]); // A links
  		reihenfolge.add (anzeigen [ 5]); // B links
  		reihenfolge.add (anzeigen [ 7]); // C links
  		reihenfolge.add (anzeigen [ 2]); // A mitte links
  		reihenfolge.add (anzeigen [ 4]); // B rechts
  		reihenfolge.add (anzeigen [ 6]); // C rechts
  		reihenfolge.add (anzeigen [ 1]); // A mitte rechts 
  		reihenfolge.add (anzeigen [ 0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP11d)
    		reihenfolge.add (anzeigen [8]); // D2
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP12 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP12d))
  	{
  		reihenfolge.add (anzeigen [ 4]); // A links
  		reihenfolge.add (anzeigen [ 7]); // B links
  		reihenfolge.add (anzeigen [10]); // C links
  		reihenfolge.add (anzeigen [ 3]); // A mitte links
  		reihenfolge.add (anzeigen [ 6]); // B mitte
  		reihenfolge.add (anzeigen [ 9]); // C mitte
  		reihenfolge.add (anzeigen [ 2]); // A mitte
  		reihenfolge.add (anzeigen [ 1]); // A mitte rechts
  		reihenfolge.add (anzeigen [ 5]); // B rechts
  		reihenfolge.add (anzeigen [ 8]); // C rechts
  		reihenfolge.add (anzeigen [ 0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP12d)
    		reihenfolge.add (anzeigen [11]); // D2
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP13)
  	{
  		reihenfolge.add (anzeigen [1]); // A links
  		reihenfolge.add (anzeigen [2]); // B 
  		reihenfolge.add (anzeigen [3]); // C 
  		reihenfolge.add (anzeigen [0]); // A mitte
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP14 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP14d))
  	{
  		reihenfolge.add (anzeigen [12]); // TAQ links
  		reihenfolge.add (anzeigen [ 3]); // A links
  		reihenfolge.add (anzeigen [ 6]); // B links
  		reihenfolge.add (anzeigen [ 9]); // C links
  		reihenfolge.add (anzeigen [ 2]); // A mitte links
  		reihenfolge.add (anzeigen [ 5]); // B mitte
  		reihenfolge.add (anzeigen [ 8]); // C mitte
  		reihenfolge.add (anzeigen [ 1]); // A mitte rechts
  		reihenfolge.add (anzeigen [ 4]); // B rechts
  		reihenfolge.add (anzeigen [ 7]); // C rechts
  		reihenfolge.add (anzeigen [ 0]); // A rechts
  		reihenfolge.add (anzeigen [10]); // D
  		reihenfolge.add (anzeigen [11]); // TAQ rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP14d)
    		reihenfolge.add (anzeigen [13]); // D2
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP15 ) 
  	{
  		reihenfolge.add (anzeigen [14]); // Blinker links
  		reihenfolge.add (anzeigen [12]); // TAQ links
  		reihenfolge.add (anzeigen [ 3]); // A links
  		reihenfolge.add (anzeigen [ 6]); // B links
  		reihenfolge.add (anzeigen [ 9]); // C links
  		reihenfolge.add (anzeigen [ 2]); // A mitte links
  		reihenfolge.add (anzeigen [ 5]); // B mitte
  		reihenfolge.add (anzeigen [ 8]); // C mitte
  		reihenfolge.add (anzeigen [ 1]); // A mitte rechts
  		reihenfolge.add (anzeigen [ 4]); // B rechts
  		reihenfolge.add (anzeigen [ 7]); // C rechts
  		reihenfolge.add (anzeigen [ 0]); // A rechts
  		reihenfolge.add (anzeigen [10]); // D
  		reihenfolge.add (anzeigen [13]); // Blinker rechts
  		reihenfolge.add (anzeigen [11]); // TAQ rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP16 ) 
  	{
  		reihenfolge.add (anzeigen [ 2]); // Blinker links
  		reihenfolge.add (anzeigen [ 4]); // TAQ links
  		reihenfolge.add (anzeigen [ 1]); // Blinker mitte
  		reihenfolge.add (anzeigen [ 5]); // WAQ
  		reihenfolge.add (anzeigen [ 0]); // Blinker rechts
  		reihenfolge.add (anzeigen [ 3]); // TAQ rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP17 ) 
  	{
  		reihenfolge.add (anzeigen [ 0]); // Blinker 
  		reihenfolge.add (anzeigen [ 1]); // WAQ
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP18 ) 
  	{
  		reihenfolge.add (anzeigen [ 1]); // Blinker links 
  		reihenfolge.add (anzeigen [ 3]); // TAQ links
  		reihenfolge.add (anzeigen [ 0]); // Blinker rechts
  		reihenfolge.add (anzeigen [ 2]); // TAQ rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP19 ) 
  	{
  		reihenfolge.add (anzeigen [ 1]); // BAQ links 
  		reihenfolge.add (anzeigen [ 0]); // BAQ rechts
    };

    return reihenfolge.toArray(new SystemObject[reihenfolge.size()]);
  }
}
