package de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen;

import java.util.HashMap;
import java.util.Vector;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.abhaengigkeiten.Abhaengigkeiten;


/**
 * Interface zur Definition der projektspezifischen Einstellungen
 * @author ObertM
 */
public interface IProjektEinstellungen
{
	/**
	 * Methode liefert eine Vector mit den im Projekt zu verwendenden Anzeigetypen
	 * @return Liste mit Anzeigetypen
	 */
	public Vector<AnzeigeTypEigenschaften> getDefaultAnzeigeTypen ();
	
	/**
	 * Methode liefert die Pid des WvzInhalts (typ.wvzInhalt) Dunkel
	 * @return Pid
	 */
	public String getPidDunkel ();

	/**
	 * Methode liefert die Pid des WvzInhalts (typ.wvzInhalt) Beibehalten
	 * @return Pid
	 */
	public String getPidBeibehalten ();
	
	/**
	 * Methode liefert die Pid des WvzInhalts (typ.wvzInhalt) Beliebig
	 * @return Pid
	 */
	public String getPidBeliebig ();
	
	/**
	 * Methode liefert die Pid des WvzInhalts (typ.wvzInhalt) Don't Care
	 * @return Pid
	 */
	public String getPidDontCare ();

	/**
	 * Methode liefert eine Hashmap mit den verwendeten Anzeigequerschnitten.
	 * @return Hashmap mit den Anzeigequerschnitten, als Key dient die Pid, als Value der Typ des Anzeigequerschnitts
	 */
  public HashMap<String, EAnzeigeQuerschnittTyp> getDefaultAnzeigeQuerschnitte ();

  /**
   * Methode liefert einen Vektor mit den abhngigen Anzeigen fr einen AQ
   * @param soAQ Systemobjekt des AQ
   * @return Vektor mit Abhngigkeiten
   */
  public Vector<Abhaengigkeiten> getDefaultAbhaengigkeiten(SystemObject soAQ);

  /**
   * Methode liefert eine Feld mit Systemobjektend er Anzeigen die die Defaultquerbeziehung
   * eines AQ darstellen. Die Richtung ist dabei von links nach rechts und von oben nach unten.
   * @param soAQ Systemobejkt des AQ
   * @return Feld mit Systemobjekten
   */
  public SystemObject[] getDefaultQuerBeziehung(SystemObject soAQ);

}
