/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.parametereditor.editor;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;

import de.bsvrz.dav.daf.main.ClientDavInterface;

public class CompositeTabbedParameterEditor extends Composite
{
	private ClientDavInterface _connection = null;
	private TabFolder _folder = null;

	/**
	 * Gibt an, ob die Felder der erzeugten Tab-Seiten editierbar sind, oder
	 * nicht.
	 */
	private boolean _editable = true;

	public CompositeTabbedParameterEditor(Composite parent, int style,
			ClientDavInterface connection)
	{
		super(parent, style);

		this._connection = connection;

		initialize();
	}

	private void initialize()
	{
		// createGroupAusgewaehlteObjekte();
		createFolder();
		setSize(new Point(300, 200));
		setLayout(new GridLayout(1, false));
	}

	/**
	 * Initialisierung des TabFolders
	 * 
	 */
	private void createFolder()
	{
		GridData gridDataTabFolder = new GridData();
		gridDataTabFolder.horizontalAlignment = GridData.FILL;
		gridDataTabFolder.grabExcessHorizontalSpace = true;
		gridDataTabFolder.grabExcessVerticalSpace = true;
		gridDataTabFolder.verticalAlignment = GridData.FILL;
		// _compositeDarstellung = new Composite(this, SWT.BORDER);
		// _compositeDarstellung.setLayout(new GridLayout());
		// _compositeDarstellung.setLayoutData(gridDataDarstellung);

		_folder = new TabFolder(this, SWT.NONE);
		_folder.setLayout(new FillLayout());
		_folder.setLayoutData(gridDataTabFolder);
	}

	private CompositeParameterEditorNeu createTabPage(ParameterEditorKapsel kapsel)
	{
		CompositeParameterEditorNeu in = null;

		if (_folder != null)
		{

			if (_connection != null)
			{
				TabItem tab = new TabItem(_folder, SWT.NONE);
				tab.setText(kapsel.getPidSystemObjekt() + ":"
						+ kapsel.getSimulationsVariante());

				tab.setToolTipText(kapsel.toString() + "\nEditierbar: " + _editable);

				// in = new CompositeParameterEditor(_folder, SWT.None, _connection);
				in = new CompositeParameterEditorNeu(_folder, SWT.NONE, _connection);

				// GridData gdCompositeParametereditorNeu = new GridData();
				// gdCompositeParametereditorNeu.grabExcessHorizontalSpace = true;
				// gdCompositeParametereditorNeu.horizontalAlignment = GridData.FILL;
				// gdCompositeParametereditorNeu.grabExcessVerticalSpace = true;
				// gdCompositeParametereditorNeu.verticalAlignment = GridData.FILL;
				// in.setLayoutData(gdCompositeParametereditorNeu);

				in.setEditable(_editable);
				tab.setControl(in);
				in.setKapsel(kapsel);
			}
		}

		return in;
	}

	protected void erzeugeEditor(ParameterEditorKapsel kapsel)
	{		
		createTabPage(kapsel);
	}

	protected void erzeugeEditoren(LinkedList<ParameterEditorKapsel> kapseln)
	{
		Iterator<ParameterEditorKapsel> it = kapseln.iterator();
		while (it.hasNext())
		{
			erzeugeEditor(it.next());
		}
	}

	protected void sichereEditoren()
	{
		TabItem[] items = _folder.getItems();
		for (TabItem tabItem : items)
		{
			Control control = tabItem.getControl();
			if (control instanceof CompositeParameterEditorNeu)
			{
				CompositeParameterEditorNeu editor = (CompositeParameterEditorNeu) control;
				editor.synchronisiereBearbeitung();
				editor.sichereBearbeiteteDatenAufDaV();
			}
			else
			{
				System.err.println("Keine Instanz von CompositeParameterEditorNeu");
			}
		}
	}

	/**
	 * Liefert eine Liste von {@link ParameterEditorKapsel}n zurck, welche
	 * gesichert werden sollen.
	 * 
	 * @return Liste von {@link ParameterEditorKapsel}n.
	 */
	protected LinkedList<ParameterEditorKapsel> getKapselnZumSichern()
	{
		LinkedList<ParameterEditorKapsel> kapselListe = new LinkedList<ParameterEditorKapsel>();

		TabItem[] items = _folder.getItems();
		for (TabItem tabItem : items)
		{
			Control control = tabItem.getControl();
			if (control instanceof CompositeParameterEditorNeu)
			{
				CompositeParameterEditorNeu editor = (CompositeParameterEditorNeu) control;
				editor.synchronisiereBearbeitung();
				kapselListe.add(editor.getKapsel());
			}
			else
			{
				System.err.println("Keine Instanz von CompositeParameterEditorNeu");
			}
		}

		return kapselListe;
	}

	/**
	 * Liefert den Index des selektierten Tab.
	 * 
	 * @return Index des selektierten Tab.
	 */
	protected int getSelectedTab()
	{
		int re = -1;

		if (_folder != null)
		{
			re = _folder.getSelectionIndex();
		}

		return re;
	}

	/**
	 * Setzt den Index des zu selektierenden Tab.
	 * 
	 * @param idx
	 *          Index des zu selektierenden Tab.
	 */
	protected void setSelektedTab(int idx)
	{
		if (_folder != null)
		{
			if (idx < _folder.getItemCount())
			{
				_folder.setSelection(idx);
			}
		}
	}

	protected void setEditable(boolean editable)
	{
		this._editable = editable;
	}
	
	protected boolean isEditable()
	{
		return _editable;
	}

}
