/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.parametereditor.editor;

import java.util.LinkedList;

import org.eclipse.jface.action.Action;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sba.plugin.actions.parameter.DefaultParameterIds;
import de.bsvrz.sba.plugin.actions.parameter.IParameter;
import de.bsvrz.sba.plugin.actions.parameter.IParameterAction;
import de.bsvrz.sba.plugin.actions.parameter.ParameterRegistry;
import de.bsvrz.sba.plugin.parametereditor.Activator;

/**
 * Liefert die {@link IParameterAction}, mit deren Hilfe sich der
 * Parametereditor im Viewer-Modus ffnen lsst. Es erfolgt nur die Darstellung,
 * ein Editieren ist nicht mglich!<br>
 * Notwendige Parameter, die der Methode
 * {@link ParameterActionOpenParameterViewer#runParameterAction(LinkedList)}
 * mitzuteilen sind:
 * 
 * <ul>
 * <li>Pids der Systemobjekte als String, Komma separiert:
 * {@link DefaultParameterIds#pidSystemObjects}</li>
 * <li>Pid der Attributgruppe: {@link DefaultParameterIds#pidAtg} </li>
 * <li>Pid des Aspektes zum Empfangen der Daten:
 * {@link DefaultParameterIds#pidAspEmpfang}</li>
 * <li>Pid des Aspektes zum Senden der Daten:
 * {@link DefaultParameterIds#pidAspSenden}</li>
 * <li>Simulationsvariante {@link DefaultParameterIds#simVar}</li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class ParameterActionOpenParameterViewer implements IParameterAction
{
	private static final String ID = Activator.PLUGIN_ID + ".OpenParameterViewer";
	private Action _action = null;
	private ParameterRegistry _registry = new ParameterRegistry();

	/**
	 * Konstruktor, initialisiert die Basisparameter.
	 */
	public ParameterActionOpenParameterViewer()
	{
		Parameter parPidSysObj = new Parameter(
				DefaultParameterIds.pidSystemObjects, null, "Pid des SystemObjektes");
		Parameter parPidAtg = new Parameter(DefaultParameterIds.pidAtg, null,
				"Pid der Attributgruppe");
		Parameter parPidAspEmpfang = new Parameter(
				DefaultParameterIds.pidAspEmpfang, "asp.parameterSoll",
				"Pid des Aspekts zum Empfang");
		Parameter parPidAspSenden = new Parameter(DefaultParameterIds.pidAspSenden,
				"asp.parameterVorgabe", "Pid des Aspekts zum Senden");
		Parameter parSimVar = new Parameter(DefaultParameterIds.simVar, null,
				"Simulationsvariante");

		_registry.setParameter(parPidSysObj);
		_registry.setParameter(parPidAtg);
		_registry.setParameter(parPidAspEmpfang);
		_registry.setParameter(parPidAspSenden);
		_registry.setParameter(parSimVar);

		_action = new Action()
		{
			public void run()
			{
				actionOpenParameterEditor();
			}
		};
		_action.setId(ID);
		_action.setText("Parametereditor ffnen");

	}

	/**
	 * Initialisiert den Editor an Hand der in der {@link ParameterRegistry}
	 * hinterlegt Parameter.
	 */
	private void actionOpenParameterEditor()
	{
		String pidsSystemObjects = _registry
				.getParameterValueAsString(DefaultParameterIds.pidSystemObjects);

		String pidAtg = _registry
				.getParameterValueAsString(DefaultParameterIds.pidAtg);
		String pidAspEmpfang = _registry
				.getParameterValueAsString(DefaultParameterIds.pidAspEmpfang);
		String pidAspSenden = _registry
				.getParameterValueAsString(DefaultParameterIds.pidAspSenden);
		short simVar = _registry
				.getParameterValueAsShort(DefaultParameterIds.simVar);

		// String out = "";
		// out = out + "PIDs: " + pidsSystemObjects + "\n";
		// out = out + "Atg: " + pidAtg + "\n";
		// out = out + "AspEmpfang: " + pidAspEmpfang + "\n";
		// out = out + "AspSenden: " + pidAspSenden + "\n";
		// out = out + "SimVar: " + simVar + "\n";
		// System.out.println(out);

		boolean weitermachen = true;
		String msg = "";

		if (pidsSystemObjects == null)
		{
			msg = msg + "Die Angabe der SystemObjekte darf nicht null sein.\n";
			weitermachen = false;
		}
		if (pidAtg == null)
		{
			msg = msg + "Die Angabe der Attributgruppe darf nicht null sein.\n";
			weitermachen = false;
		}
		if (pidAspEmpfang == null)
		{
			msg = msg + "Die Angabe des Aspekts zum Empfang darf nicht null sein.\n";
			weitermachen = false;
		}
		if (pidAspSenden == null)
		{
			msg = msg + "Die Angabe des Aspekts zum Senden darf nicht null sein.\n";
			weitermachen = false;
		}
		if ((simVar < 0) || (simVar > 999))
		{
			msg = msg
					+ "Die Simulationsvariante muss im Bereich von 0 bis 999 liegen.\n";
			weitermachen = false;
		}

		if (weitermachen)
		{
			String[] split = pidsSystemObjects.split(",");
			LinkedList<ParameterEditorKapsel> kapseln = new LinkedList<ParameterEditorKapsel>();

			for (String systemObject : split)
			{
				ParameterEditorKapsel kaps = new ParameterEditorKapsel(systemObject,
						pidAtg, pidAspEmpfang, pidAspSenden, simVar);

				kapseln.add(kaps);
			}

			if (kapseln.size() >= 0)
			{
				OpenViewParameterEditor.openView(kapseln, true, false);
			}
		}
		else
		{
			MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
					.getActiveShell(), SWT.OK | SWT.ICON_INFORMATION);
			mb.setText("Achtung");
			mb.setMessage(msg);
			mb.open();
		}

	}

	@Override
	public Action getParameterAction()
	{
		return _action;
	}

	@Override
	public String getParameterActionId()
	{
		return ID;
	}

	@Override
	public LinkedList<String> getParameterIds()
	{
		LinkedList<String> re = null;

		if (_registry != null)
		{
			synchronized (_registry)
			{
				re = _registry.getParameterIds();
			}
		}

		return re;
	}

	@Override
	public boolean runParameterAction(LinkedList<IParameter> parameters)
	{
		boolean re = false;

		synchronized (this)
		{
			synchronized (_registry)
			{
				_registry.setParameters(parameters);

				if (_action != null)
				{
					_action.run();
					re = true;
				}
			}
		}

		return re;
	}

	/**
	 * Kapselt einen {@link IParameter}
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Parameter implements IParameter
	{
		private String _id = null;
		private Object _value = null;
		private String _info = null;

		public Parameter(String id, Object value, String info)
		{
			this._id = id;
			this._value = value;
			this._info = info;
		}

		@Override
		public String getId()
		{
			return _id;
		}

		@Override
		public String getParameterInfo()
		{
			return _info;
		}

		@Override
		public Object getValue()
		{
			return _value;
		}

		@Override
		public void setValue(Object value)
		{
			this._value = value;
		}

	}

	@Override
	public void clearParameters()
	{
		synchronized (_registry)
		{
			_registry.clearParameters();
		}
	}

	@Override
	public LinkedList<IParameter> getParameters()
	{
		LinkedList<IParameter> re = null;

		synchronized (_registry)
		{
			re = _registry.getParameters();
		}

		return re;
	}

	@Override
	public void setParameters(LinkedList<IParameter> parameters)
	{
		synchronized (_registry)
		{
			_registry.setParameters(parameters);
		}
	}

}
