package de.bsvrz.sba.plugin.zlt.dav;

import java.util.Collection;
import java.util.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sba.plugin.zlt.Defs;

public class DaVKommunikation implements ClientReceiverInterface
{
	private static DaVKommunikation _instance = null;

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _con = null;

	/**
	 * Verwaltung der ZLT Programmaktivierungs Zustnde.
	 */
	private VerwaltungZltProgrammaktivierungZustaende _verwaltungZltProgrammaktivierungZustaende = new VerwaltungZltProgrammaktivierungZustaende();

	private DaVKommunikation() {
		_con = DaVVerbindung.getInstanz().getDaVVerbindung();

		if (_con != null)
		{
			init();
		}
	}

	/**
	 * Liefert die Singleton-Instanz dieser Klasse.
	 * 
	 * @return
	 */
	public static DaVKommunikation getInstance()
	{
		if (_instance == null)
		{
			synchronized (DaVKommunikation.class)
			{
				if (_instance == null)
				{
					_instance = new DaVKommunikation();
				}
			}
		}
		return _instance;
	}

	private void init()
	{
		if (_con != null)
		{
			DataModel dataModel = _con.getDataModel();

			/*
			 * Anmelden auf die Programmaktivierung.
			 */
			SystemObjectType typDeZltPrgakt = dataModel
					.getType(Defs.TYP_DE_ZLT_PROGRAMMAKTIVIERUNG);
			if (typDeZltPrgakt != null)
			{
				List<SystemObject> lSO = typDeZltPrgakt.getObjects();
				
				anmeldeStringsLeseDaV(lSO,
						Defs.ATG_TLS_ZLT_PROGRAMMAKTIVIERUNG, Defs.ASP_TLS_ZUSTAND,
						ReceiveOptions.delta(), ReceiverRole.receiver());
			}
		}
	}

	private void anmeldeStringsLeseDaV(Collection<SystemObject> objs,
			String atgPid, String aspPid, ReceiveOptions options, ReceiverRole role)
	{
		if (_con != null)
		{
			if ((objs != null) && (atgPid != null) && (aspPid != null)
					&& (options != null) && (role != null))
			{
				DataModel dataModel = _con.getDataModel();
				AttributeGroup atg = dataModel.getAttributeGroup(atgPid);
				Aspect asp = dataModel.getAspect(aspPid);

				if ((atg != null) && (asp != null))
				{
					DataDescription dd = new DataDescription(atg, asp);

					_con.subscribeReceiver(this, objs, dd, ReceiveOptions.delta(),
							ReceiverRole.receiver());

				}

			}
		}
	}

	@Override
	public void update(ResultData[] results)
	{
		if (results != null)
		{
			for (ResultData resultData : results)
			{
				// System.out.println("ZLT DaVKom-> " + resultData);

				if ((resultData != null) && (resultData.getObject() != null))
				{
					DataDescription dd = resultData.getDataDescription();
					AttributeGroup atg = dd.getAttributeGroup();
					Aspect asp = dd.getAspect();

					if ((atg != null) && (asp != null))
					{
						if (asp.getPid().equals(Defs.ASP_TLS_ZUSTAND))
						{
							if (atg.getPid().equals(Defs.ATG_TLS_ZLT_PROGRAMMAKTIVIERUNG))
							{
								getVerwaltungZltProgrammaktivierungZustaende()
										.putZltProgrammaktivierung(resultData);
							} 
						}
					}
				}

			}
		}

	}

	public VerwaltungZltProgrammaktivierungZustaende getVerwaltungZltProgrammaktivierungZustaende()
	{
		return _verwaltungZltProgrammaktivierungZustaende;
	}

	public void setVerwaltungZltProgrammaktivierungZustaende(
			VerwaltungZltProgrammaktivierungZustaende verwaltungZltProgrammaktivierungZustaende)
	{
		_verwaltungZltProgrammaktivierungZustaende = verwaltungZltProgrammaktivierungZustaende;
	}

}
