package de.bsvrz.sba.plugin.zlt.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.zlt.Defs;
import de.bsvrz.sba.plugin.zlt.dav.DaVKommunikation;
import de.bsvrz.sba.plugin.zlt.dav.VerwaltungZltProgrammaktivierungZustaende;
import de.bsvrz.sba.plugin.zlt.dav.ZltGrouped;

/**
 * Mit Hilfe dieser View wird die Programmaktivierung dargestellt.
 * 
 * @author Swarco Traffic Systems GmbH
 * 
 */
public class ViewZltProgrammaktivierung extends ViewPart implements PropertyChangeListener
{
  public static final String VIEW_ID = "de.bsvrz.sba.plugin.zlt.views.ViewZltProgrammaktivierung";

  public static final String VIEW_KIND_NORMAL = "NORMAL";

  public static final String VIEW_KIND_PROGRAMM_AKTIVIERT = "PROGRAMM_AKTIVIERT";

  /**
   * Namen der Spalten.
   */
  private static final String[] COL_NAMES = new String[] { "Programm", "Zeitstempel", "Programmnummer", "Status",
      "Programmzustand", "Krzel" };

  /**
   * Initiale Spaltenbreiten.
   */
  private static final int[] COL_WIDTH = new int[] { 250, 125, 125, 150, 125, 125 };

  /*
   * Legt die Reihenfolge der Spalte fest.
   */
  private static final int IDX_COL_SO_PRG = 0;

  private static final int IDX_COL_ZEITSTEMPEL = 1;

  private static final int IDX_COL_PROGRAMMNUMMER = 2;

  private static final int IDX_COL_STATUS = 3;

  private static final int IDX_COL_KUERZEL = 4;

  /**
   * TV zur Darstellunge
   */
  private TableViewer _tv;

  private VC _comparator;

  private ZltPrgStatusFilter _filter;

  @Override
  public void createPartControl(Composite parent)
  {
    parent.setLayout(new FillLayout());

    Table table = new Table(parent, SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    table.setHeaderVisible(true);
    table.setLinesVisible(true);

    _tv = new TableViewer(table);
    _tv.setContentProvider(new CP());
    _tv.setLabelProvider(new LP());

    _comparator = new VC();
    _tv.setComparator(_comparator);

    /*
     * Spalten erzeugen
     */
    for (int i = 0; i < COL_NAMES.length; i++)
    {
      String titelSpalte = COL_NAMES[i];

      if (titelSpalte == null)
      {
        titelSpalte = "-";
      }

      int width = 200;
      if ((i >= 0) && (i < COL_WIDTH.length))
      {
        width = COL_WIDTH[i];
      }

      TableColumn tc = new TableColumn(table, SWT.LEFT);
      tc.setText(titelSpalte);
      tc.setWidth(width);

      tc.addSelectionListener(getSelectionAdapter(tc, i));

    }

    _filter = new ZltPrgStatusFilter();
    _tv.addFilter(_filter);

    String secondaryId = this.getViewSite().getSecondaryId();
    if ((secondaryId != null) && (secondaryId.equals(VIEW_KIND_PROGRAMM_AKTIVIERT)))
    {
      this.setPartName(this.getPartName() + " (akive Programme)");
      doFilter();
    }

    /*
     * Auf Daten anmelden
     */
    DaVKommunikation.getInstance().getVerwaltungZltProgrammaktivierungZustaende().addListener(this);
  }

  private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index)
  {
    SelectionAdapter selectionAdapter = new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        _comparator.setColumn(index);
        int dir = _comparator.getDirection();
        _tv.getTable().setSortDirection(dir);
        _tv.getTable().setSortColumn(column);
        _tv.refresh();
      }
    };
    return selectionAdapter;
  }

  @Override
  public void setFocus()
  {
    _tv.getControl().setFocus();
  }

  @Override
  public void dispose()
  {
    super.dispose();

    DaVKommunikation.getInstance().getVerwaltungZltProgrammaktivierungZustaende().removeListener(this);
  }

  private class CP implements IStructuredContentProvider
  {

    @Override
    public Object[] getElements(Object inputElement)
    {
      Object[] re = null;

      if (inputElement instanceof VerwaltungZltProgrammaktivierungZustaende)
      {
        VerwaltungZltProgrammaktivierungZustaende vew = (VerwaltungZltProgrammaktivierungZustaende)inputElement;
        re = vew.getZltGroups();
      }

      return re;
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  private class LP implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String re = null;

      if (element instanceof ZltGrouped)
      {
        ZltGrouped zltGrouped = (ZltGrouped)element;

        if (zltGrouped != null)
        {
          switch (columnIndex)
          {
            case IDX_COL_SO_PRG:
              SystemObject so = zltGrouped.getSo();
              if (so != null)
              {
                re = so.getNameOrPidOrId();
              }
              break;
            case IDX_COL_ZEITSTEMPEL:
              if (zltGrouped.getZeitstempel() != null)
              {
                re = Defs.SDF.format(zltGrouped.getZeitstempel());
              }
              break;
            case IDX_COL_PROGRAMMNUMMER:
              if (zltGrouped.getProgrammnummer() != null)
              {
                re = zltGrouped.getProgrammnummer().toString();
              }
              break;
            case IDX_COL_STATUS:
              re = zltGrouped.getStatus();
              break;
            case IDX_COL_KUERZEL:
              re = zltGrouped.getKuerzel();
              break;
          }
        }

      }

      if (re == null)
      {
        re = "-";
      }

      return re;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  /**
   * Liefert einen Comparator fr die Tabelle dieser View.
   * 
   * @author Swarco Traffic Systems GmbH
   * 
   */
  private class VC extends ViewerComparator
  {
    private int propertyIndex;

    private static final int DESCENDING = 1;

    private int direction = DESCENDING;

    public VC()
    {
      this.propertyIndex = 0;
      direction = DESCENDING;
    }

    public int getDirection()
    {
      return direction == 1 ? SWT.DOWN : SWT.UP;
    }

    public void setColumn(int column)
    {
      if (column == this.propertyIndex)
      {
        // Same column as last sort; toggle the direction
        direction = 1 - direction;
      }
      else
      {
        // New column; do an ascending sort
        this.propertyIndex = column;
        direction = DESCENDING;
      }
    }

    @Override
    public int compare(Viewer viewer, Object e1, Object e2)
    {
      // return super.compare(viewer, e1, e2);
      int re = 0;

      ZltGrouped g1 = (ZltGrouped)e1;
      ZltGrouped g2 = (ZltGrouped)e2;

      switch (propertyIndex)
      {
        case IDX_COL_SO_PRG:
          re = g1.getSo().getNameOrPidOrId().compareTo(g2.getSo().getNameOrPidOrId());
          break;
        case IDX_COL_ZEITSTEMPEL:
          if ((g1.getZeitstempel() != null) && (g2.getZeitstempel() != null))
          {
            re = g1.getZeitstempel().compareTo(g2.getZeitstempel());
          }
          // else if (g1.getZeitstempel() != null)
          // {
          // re = 1;
          // } else if (g2.getZeitstempel() != null)
          // {
          // re = -1;
          // }
          break;
        case IDX_COL_PROGRAMMNUMMER:
          if ((g1.getProgrammnummer() != null) && (g2.getProgrammnummer() != null))
          {
            re = g1.getProgrammnummer().compareTo(g2.getProgrammnummer());
          }
          else if (g1.getProgrammnummer() != null)
          {
            re = 1;
          }
          else if (g2.getProgrammnummer() != null)
          {
            re = -1;
          }
          break;
        case IDX_COL_STATUS:
          if ((g1.getStatus() != null) && (g2.getStatus() != null))
          {
            re = g1.getStatus().compareTo(g2.getStatus());
          }
          else if (g1.getStatus() != null)
          {
            re = 1;
          }
          else if (g2.getStatus() != null)
          {
            re = -1;
          }
          break;
        case IDX_COL_KUERZEL:
          if ((g1.getKuerzel() != null) && (g2.getKuerzel() != null))
          {
            re = g1.getKuerzel().compareTo(g2.getKuerzel());
          }
          else if (g1.getKuerzel() != null)
          {
            re = 1;
          }
          else if (g2.getKuerzel() != null)
          {
            re = -1;
          }
          break;
      }

      // If descending order, flip the direction
      if (direction == DESCENDING)
      {
        re = -re;
      }

      return re;
    }

  }

  private class ZltPrgStatusFilter extends ViewerFilter
  {
    private String searchString = null;

    public void setSearchText(String s)
    {
      /*
       * Alle Werte fr das Matching verwenden.
       */
      this.searchString = ".*" + s + ".*";
    }

    @Override
    public boolean select(Viewer viewer, Object parentElement, Object element)
    {
      if (searchString == null || searchString.length() == 0)
      {
        return true;
      }

      ZltGrouped group = (ZltGrouped)element;

      if (group.getStatus() != null)
      {
        if (group.getStatus().matches(searchString))
        {
          return true;
        }
      }

      return false;
    }
  }

  @Override
  public void propertyChange(PropertyChangeEvent evt)
  {
    if (evt != null)
    {
      if ((evt.getSource() != null) && (evt.getSource() instanceof VerwaltungZltProgrammaktivierungZustaende))
      {
        final VerwaltungZltProgrammaktivierungZustaende vew = (VerwaltungZltProgrammaktivierungZustaende)evt
            .getSource();

        final Object newValue = evt.getNewValue();

        if ((evt.getPropertyName() == VerwaltungZltProgrammaktivierungZustaende.PROP_ZLT_GROUPED_INIT)
            || (evt.getPropertyName() == VerwaltungZltProgrammaktivierungZustaende.PROP_ZLT_GROUPED_PUT))
        {
          Runnable run = new Runnable() {
            @Override
            public void run()
            {
              _tv.setInput(vew);
            }
          };

          this.getViewSite().getShell().getDisplay().asyncExec(run);

        }
        else if (evt.getPropertyName() == VerwaltungZltProgrammaktivierungZustaende.PROP_ZLT_GROUPED_UPDATE)
        {
          if ((newValue != null) && (newValue instanceof ZltGrouped))
          {
            Runnable run = new Runnable() {
              @Override
              public void run()
              {
                _tv.refresh(newValue);
              }
            };
            this.getViewSite().getShell().getDisplay().asyncExec(run);
          }
        }

        doFilter();
      }

    }
  }

  private void doFilter()
  {

    if (this.getViewSite().getSecondaryId() != null)
    {
      if (this.getViewSite().getSecondaryId().equals(VIEW_KIND_PROGRAMM_AKTIVIERT))
      {
        Runnable run = new Runnable() {

          @Override
          public void run()
          {
            _filter.setSearchText("m aktiviert");
            _tv.refresh();
          }
        };
        this.getViewSite().getShell().getDisplay().asyncExec(run);
      }
    }
  }

}
