/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

/**
 * Umsetzung der Ausprgungen von <code>att.tsfArgoscanZustand</code>.
 * 
 * @author HofmannJ
 * 
 */
public enum E_TSFArgoscanZustand
{
	Ruhezustand(0, "Ruhezustand", "Ruhezustand"), Initialisieren(1,
			"Initialisieren", "Initialisieren"), WarteStartInitialisieren(2,
			"Warte Start Initialisieren", "Warte Start Initialisieren"), TimeoutStartInitialisieren(
			3, "Timeout Start Initialisieren",
			"Timeout Start Initialisieren"), WarteInitialisierenFertig(
			4, "Warte Initialisieren fertig", "Warte Initialisieren fertig"), TimeoutInitialisierenFertig(
			5, "Timeout Initialisieren fertig", "Timeout Initialisieren fertig"), ArgoscanStarten(
			6, "Argoscan starten", "Argoscan starten"), WarteStartArgoscanStarten(7,
			"Warte Start Argoscan starten", "Warte Start Argoscan starten"), TimeoutStartArgoscanStarten(
			8, "Timeout Start Argoscan starten", "Timeout Start Argoscan starten"), WarteFertigArgoscanStarten(
			9, "Warte Fertig Argoscan starten", "Warte Fertig Argoscan starten"), TimeoutFertigArgoscanStarten(
			10, "Timeout Fertig Argoscan starten", "Timeout Fertig Argoscan starten"), ArgoscanStoppen(
			11, "Argoscan stoppen", "Argoscan stoppen"), WarteAufStartArgoscanStoppen(
			12, "Warte auf Start Argoscan stoppen",
			"Warte auf Start Argoscan stoppen"), TimeoutStartArgoscanStoppen(13,
			"Timeout Start Argoscan stoppen", "Timeout Start Argoscan stoppen"), WarteAufFertigArgoscanStoppen(
			14, "Warte auf Fertig Argoscan stoppen",
			"Warte auf Fertig Argoscan stoppen"), TimeoutFertigArgoscanStoppen(15,
			"Timeout Fertig Argoscan stoppen", "Timeout Fertig Argoscan stoppen");

	private int _wert = -1;
	private String _kuerzel = null;
	private String _text = null;

	private E_TSFArgoscanZustand(int wert, String kuerzel, String text)
	{
		this._wert = wert;
		this._kuerzel = kuerzel;
		this._text = text;
	}

	@Override
	public String toString()
	{
		String re = null;

		if (getText() != null)
		{
			re = getText();
		} else if (getKuerzel() != null)
		{
			re = getKuerzel();
		}

		if (re == null)
		{
			re = "-";
		}

		return re;
	}

	public int getWert()
	{
		return _wert;
	}

	public String getKuerzel()
	{
		return _kuerzel;
	}

	public String getText()
	{
		return _text;
	}

	/**
	 * Liefert fr einen int-Wert den entsprechenden Enumerator zur Reprsentation
	 * des Befehls.
	 * 
	 * @param wert
	 * @return
	 */
	public static E_TSFArgoscanZustand getZustand(int wert)
	{
		E_TSFArgoscanZustand re = null;

		if (wert >= 0)
		{
			switch (wert)
			{
			case 0:
				re = Ruhezustand;
				break;
			case 1:
				re = Initialisieren;
				break;
			case 2:
				re = WarteStartInitialisieren;
				break;
			case 3:
				re = TimeoutStartInitialisieren;
				break;
			case 4:
				re = WarteInitialisierenFertig;
				break;
			case 5:
				re = TimeoutInitialisierenFertig;
				break;
			case 6:
				re = ArgoscanStarten;
				break;
			case 7:
				re = WarteStartArgoscanStarten;
				break;
			case 8:
				re = TimeoutStartArgoscanStarten;
				break;
			case 9:
				re = WarteFertigArgoscanStarten;
				break;
			case 10:
				re = TimeoutFertigArgoscanStarten;
				break;
			case 11:
				re = ArgoscanStoppen;
				break;
			case 12:
				re = WarteAufStartArgoscanStoppen;
				break;
			case 13:
				re = TimeoutStartArgoscanStoppen;
				break;
			case 14:
				re = WarteAufFertigArgoscanStoppen;
				break;
			case 15:
				re = TimeoutFertigArgoscanStoppen;
				break;
			}
		}
		
		if(re == null)
		{
			System.err.println(E_TSFArgoscanZustand.class.getSimpleName() + " -> Zustand konnte nicht bestimmt werden fr " + wert  + "!!!");
		}

		return re;
	}
}
