/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

/**
 * Umsetzung der Ausprgungen von <code>att.tsfSteuerungZustand</code>.
 * 
 * @author HofmannJ
 * 
 */
public enum E_TSFSteuerungZustand
{
	Ruhezustand(0, "Ruhezustand", "Ruhezustand"), WarteDelayzeitStartStufe(1,
			"WarteDelayzeitStartStufe",
			"Abwarten Verzgerungszeit bis zum Start der Stufe"), SchalteBefehleBeginnStufe(
			2, "SchalteBefehleBeginnStufe",
			"Befehle zum Beginn der Stufe werden geschaltet"), WarteDelayzeitSchaltbilder(
			3, "WarteDelayzeitSchaltbilder",
			"Abwarten Verzgerungszeit der Schaltbilder"), StarteVideoProgramm(4,
			"StarteVideoProgramm", "Starten des Videoprogramms"), WarteAufEndeVideoProgramm(
			5, "WarteAufEndeVideoProgramm",
			"Abwarten bis zum Ende des Videoprogramms"), WarteAufWeiterSchaltenDurchGUI(
			6, "WarteAufWeiterSchaltenDurchGUI",
			"Warte auf Besttigung des Weiterschaltens durch die Benutzeroberflche"), SchalteBefehleEndeStufe(
			7, "SchalteBefehleEndeStufe", "Schalte Befehle zum Ende der Stufe"), EndeVideoProgrammEmpfangen(
			8, "EndeVideoProgrammEmpfangen",
			"Ende des Videoprogramms wurde empfangen"), TimeoutEndeVideoProgramm(9,
			"TimeoutEndeVideoProgramm", "Timeout Ende des Videoprogramms empfangen.");

	private int _wert = -1;
	private String _kuerzel = null;
	private String _text = null;

	private E_TSFSteuerungZustand(int wert, String kuerzel, String text)
	{
		this._wert = wert;
		this._kuerzel = kuerzel;
		this._text = text;
	}

	@Override
	public String toString()
	{
		String re = null;

		if (getText() != null)
		{
			re = getText();
		} else if (getKuerzel() != null)
		{
			re = getKuerzel();
		}

		if (re == null)
		{
			re = "-";
		}

		return re;
	}

	public int getWert()
	{
		return _wert;
	}

	public String getKuerzel()
	{
		return _kuerzel;
	}

	public String getText()
	{
		return _text;
	}

	/**
	 * Liefert fr einen int-Wert den entsprechenden Enumerator zur Reprsentation
	 * des Befehls.
	 * 
	 * @param wert
	 * @return
	 */
	public static E_TSFSteuerungZustand getBefehl(int wert)
	{
		E_TSFSteuerungZustand re = null;

		if (wert >= 0)
		{
			switch (wert)
			{
			case 0:
				re = Ruhezustand;
				break;
			case 1:
				re = WarteDelayzeitStartStufe;
				break;
			case 2:
				re = SchalteBefehleBeginnStufe;
				break;
			case 3:
				re = WarteDelayzeitSchaltbilder;
				break;
			case 4:
				re = StarteVideoProgramm;
				break;
			case 5:
				re = WarteAufEndeVideoProgramm;
				break;
			case 6:
				re = WarteAufWeiterSchaltenDurchGUI;
				break;
			case 7:
				re = SchalteBefehleEndeStufe;
				break;
			case 8:
				re = EndeVideoProgrammEmpfangen;
				break;
			case 9:
				re = TimeoutEndeVideoProgramm;
				break;
			}
		}

		return re;
	}
}
