/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import de.bsvrz.dav.daf.main.Data;

/**
 * Umsetzung von <code>atl.tsfMQDiagnoseWerte</code>.
 * 
 * @author HofmannJ
 * 
 */
public class M_TSFMQDiagnoseWerte
{
	/**
	 * Feld <code>QB</code>, Aktueller Wert Prognose QB.
	 * att.bemessungsVerkehrsStrkeStunde, PkwE/h, ganze Zahl.
	 */
	private Long _qb = null;

	/**
	 * Feld <code>V</code>. Aktueller Wert Prognose Geschwindigkeit. 0-254 km/h.
	 */
	private Integer _v = null;

	/**
	 * Feld <code>AnzahlMessZyklenEin</code>. Aktuelle Anzahl von Messzyklen fr
	 * die die Einschaltbedingung erfllt ist. Ganze Zahl positiv.
	 */
	private Long _anzahlMessZyklenEin = null;

	/**
	 * Feld <code>AnzahlMessZyklenAus</code>. Anzahl von Messzyklen fr die die
	 * Ausschaltbedingung erfllt ist. Ganze Zahl positiv.
	 */
	private Long _anzahlMessZyklenAus = null;

	public M_TSFMQDiagnoseWerte(Data data)
	{
		if (data != null)
		{
			// QB
			long lQB = data.getUnscaledValue("QB").longValue();
			setQb(lQB);

			// V
			int iV = data.getUnscaledValue("V").intValue();
			setV(iV);

			// AnzahlMessZyklenEin
			long iAnzahlMessZyklenEin = data.getUnscaledValue("AnzahlMessZyklenEin")
					.longValue();
			setAnzahlMessZyklenEin(iAnzahlMessZyklenEin);

			// AnzahlMessZyklenAus
			long iAnzahlMessZyklenAus = data.getUnscaledValue("AnzahlMessZyklenAus")
					.longValue();
			setAnzahlMessZyklenAus(iAnzahlMessZyklenAus);

		}
	}

	public Long getQb()
	{
		return _qb;
	}

	public void setQb(Long qb)
	{
		_qb = qb;
	}

	public Integer getV()
	{
		return _v;
	}

	public void setV(Integer v)
	{
		_v = v;
	}

	public Long getAnzahlMessZyklenEin()
	{
		return _anzahlMessZyklenEin;
	}

	public void setAnzahlMessZyklenEin(Long anzahlMessZyklenEin)
	{
		_anzahlMessZyklenEin = anzahlMessZyklenEin;
	}

	public Long getAnzahlMessZyklenAus()
	{
		return _anzahlMessZyklenAus;
	}

	public void setAnzahlMessZyklenAus(Long anzahlMessZyklenAus)
	{
		_anzahlMessZyklenAus = anzahlMessZyklenAus;
	}

}
