/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.text.SimpleDateFormat;
import java.util.Date;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Daten entsprechend ATG <code>atg.tsfZustandSteuerung</code>.
 * 
 * @author HofmannJ
 * 
 */
public class M_TSFZustandSteuerung implements I_TSFZustand
{
	/**
	 * SystemObjekt vom Typ <code>typ.tsfAbschnitt</code> zur Reprsentation des
	 * Abschnitts.
	 */
	private SystemObject _so = null;

	/**
	 * Befehl der TSF. Attribut Programm, Typ <code>att.tsfBefehl</code>.
	 */
	private E_TSFBefehl _programm = null;

	/**
	 * Stufe innerhalb des Ablaufs der TSF. Attribut Stufe, Typ
	 * <code>att.zahlPositiv</code>
	 */
	private long _stufe = 0;

	/**
	 * Definition mglicher Zustnder der Steuerung whrend einer TSF. Attribut
	 * Zustand, Typ <code>att.tsfSteuerungZustand</code>
	 */
	private E_TSFSteuerungZustand _zustand = null;

	private Date _timestamp = null;

	private static final SimpleDateFormat SDF = new SimpleDateFormat(
			"dd.MM.yyyy HH:mm:ss,SSS");

	/**
	 * SystemObjekt vom Typ <code>typ.tsfAbschnitt</code> zur Reprsentation des
	 * Abschnitts.
	 * 
	 * @return SystemObjekt vom Typ <code>typ.tsfAbschnitt</code> zur
	 *         Reprsentation des Abschnitts.
	 */
	public SystemObject getSo()
	{
		return _so;
	}

	public void setSo(SystemObject so)
	{
		_so = so;
	}

	public E_TSFBefehl getProgramm()
	{
		return _programm;
	}

	public void setProgramm(E_TSFBefehl programm)
	{
		_programm = programm;
	}

	public long getStufe()
	{
		return _stufe;
	}

	public void setStufe(long stufe)
	{
		_stufe = stufe;
	}

	public E_TSFSteuerungZustand getZustand()
	{
		return _zustand;
	}

	public void setZustand(E_TSFSteuerungZustand zustand)
	{
		_zustand = zustand;
	}

	public Date getTimestamp()
	{
		return _timestamp;
	}

	public void setTimestamp(Date timestamp)
	{
		_timestamp = timestamp;
	}

	@Override
	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		sb.append("ZustandSteuerung fr ");
		if (getSo() != null)
		{
			sb.append(getSo().getPid());
		} else
		{
			sb.append("-");
		}
		sb.append(":\n");

		sb.append("Zeitstempel: ");
		if (getTimestamp() != null)
		{
			sb.append(SDF.format(getTimestamp()));
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		sb.append("Programm (TSFBefehl): ");
		if (getProgramm() != null)
		{
			sb.append(getProgramm());
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		sb.append("Stufe: ");
		sb.append(getStufe());
		sb.append("\n");

		sb.append("Zustand (TFSSteuerungZustand)");
		if (getZustand() != null)
		{
			sb.append(getZustand());
		} else
		{
			sb.append("-");
		}

		return sb.toString();
	}

}
