/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.Timer;
import java.util.TimerTask;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.PreferenceInit;
import de.sts.buv.plugin.workflow.tsf.dlg.DialogWeiterschalten;
import de.sts.buv.plugin.workflow.tsf.dlg.IWiedervorlage;
import de.sts.buv.plugin.workflow.tsf.dlg.RegistryWiedervorlagen;

/**
 * Diese Klasse implementiert einen Timer, der nach einer gewissen Zeit die
 * Wiedervorlage des TSF-Weiterschalten Dialoges {@link DialogWeiterschalten}
 * auslst.
 * 
 * @author HofmannJ
 * 
 */
public class WiedervorlageTSFWeiterschalten implements IWiedervorlage
{
	// private static final long DELAY = 2l * 60l * 1000l;

	private M_TSFWeiterSchalten _tsfWeiterSchalten;
	private Timer _timer;
	private Long _startzeitpunkt;

	public WiedervorlageTSFWeiterschalten(M_TSFWeiterSchalten tsfWeiterSchalten)
	{
		this._tsfWeiterSchalten = tsfWeiterSchalten;
		startTimer();
	}

	public M_TSFWeiterSchalten getWeiterSchalten()
	{
		return _tsfWeiterSchalten;
	}

	private void startTimer()
	{
		if ((getWeiterSchalten() != null) && (getWeiterSchalten().getSo() != null))
		{
			TimerTask task = new TimerTask()
			{

				@Override
				public void run()
				{
					aktionWiedervorlegen();
				}
			};

			_timer = new Timer("Timer Wiedervorlage "
					+ getWeiterSchalten().getSo().getPid());

			long delay = PreferenceInit.getDelayWiedervorlageWfWeiterschalten() * 1000l;

			_startzeitpunkt = new Long(System.currentTimeMillis());
			_timer.schedule(task, delay);

			RegistryWiedervorlagen.getInstance().addWiedervorlage(this);
		}
	}

	private void aktionWiedervorlegen()
	{
		if(_timer!=null)
		{
			_timer.cancel();
			_timer = null;
		}
		
		if ((getWeiterSchalten() != null) && (getWeiterSchalten().getSo() != null))
		{
			GUIModelTSFWeiterSchalten model = DaVKomTSFWeiterSchalten.getInstance()
					.getModel();
			M_TSFWeiterSchalten workflow = model.getWorkflow(getWeiterSchalten()
					.getSo());
			if (workflow != null)
			{
				GUIManager_TSFWeiterschalten guiTSFWeiterschaltenManager = DaVKomTSFWeiterSchalten
						.getInstance().getGuiTSFWeiterschaltenManager();

				if (!guiTSFWeiterschaltenManager
						.containsDialogForObject(_tsfWeiterSchalten.getSo()))
				{
					guiTSFWeiterschaltenManager.genFillDlg(workflow);
				} else
				{
					// Dialog bereist offen
				}
			} else
			{
				// Es existiert keine Datenbasis mehr fr den Dialog.
				// Falls so zu ungenau evtl. via Timesamp absichern.
			}
		}

		dispose();
	}

	public void dispose()
	{
		RegistryWiedervorlagen.getInstance().removeWiedervorlage(this);
	}

	@Override
	public String getBeschreibung()
	{
		String re = null;
		M_TSFWeiterSchalten tsfWeiterSchalten = getWeiterSchalten();

		if (tsfWeiterSchalten != null)
		{
			SystemObject so = tsfWeiterSchalten.getSo();
			if (so != null)
			{
				re = "Wiedervorlage Weiterschalten Abschnitt " + so.getNameOrPidOrId();
				E_TSFBefehl prg = tsfWeiterSchalten.getProgramm();
				if (prg != null && prg.equals(E_TSFBefehl.Schnellsperrung))
				  re = "Wiedervorlage Ende-Zustand Abschnitt " + so.getNameOrPidOrId();
			}
		}

		if (re == null)
		{
			re = "";
		}
		return re;
	}

	@Override
	public String getId()
	{
		String re = null;
		M_TSFWeiterSchalten tsfWeiterSchalten = getWeiterSchalten();

		if (tsfWeiterSchalten != null)
		{
			SystemObject so = tsfWeiterSchalten.getSo();
			if (so != null)
			{
				re = "Weiterschalten " + so.getPidOrNameOrId();
        E_TSFBefehl prg = tsfWeiterSchalten.getProgramm();
        if (prg != null && prg.equals(E_TSFBefehl.Schnellsperrung))
          re = "Ende-Zustand " + so.getPidOrNameOrId();
			}
		}

		if (re == null)
		{
			re = "";
		}
		return re;
	}

	@Override
	public long getTimeout()
	{
		long re = PreferenceInit.getDelayWiedervorlageWfWeiterschalten();
		return re;
	}

	@Override
	public void vorzeitigeWiedervorlage()
	{
		if (_timer != null)
		{
//			_timer.cancel();
			aktionWiedervorlegen();
		}
	}

	@Override
	public Long getStartZeitpunkt()
	{
		return _startzeitpunkt;
	}
}
