/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.awt.Toolkit;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.LinkedList;

import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.Activator;
import de.sts.buv.plugin.workflow.tsf.DefsFonts;
import de.sts.buv.plugin.workflow.tsf.ImageGetter;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFAbschnittBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFSteuerung;
import de.sts.buv.plugin.workflow.tsf.dav.Defs;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFProgrammZustand;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFAbschnittBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFAbschnittBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.WrapperULDInfo;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFAbschnittBefehl.WrapperListenerBenachrichtigung;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung.ZustandSammlung;

/**
 * Dialog mit dem eine Schaltung via Workflow ausgelst werden kann. Es wird
 * hierzu ein Abschnittsbefehl ({@link M_TSFAbschnittBefehl}) versendet.<br>
 * <br>
 * Der Button "TSF Abbrechen" darf nur dann aktiv sein, wenn fr den Abschnitt
 * im jeweiligen Bereich gerade eine Schaltung aktiv im Gange ist.
 * 
 * @author HofmannJ
 * 
 */
public class DialogSchalten extends Dialog implements PropertyChangeListener
{
	/**
	 * DEF-Breite
	 */
	private static final int DEF_WIDTH = 600; // 900;

	/**
	 * DEF-Hhe
	 */
	private static final int DEF_HEIGHT = 350; // 250;

	private static Image _kameraImageLinks;

	private static Image _kameraImageRechts;

	/**
	 * Shell
	 */
	private Shell _shell;

	private DialogSchaltenGuiManager _dialogSchaltenGuiManager = null;

	/**
	 * SystemObjekt vom Typ <code>typ.tsfAbschnitt</code>.
	 */
	private SystemObject _soAbschnitt = null;

	private Composite _compAbschnitt;

	private Composite _compTitel;

	private Composite _compCtrl;

	private Composite _compContent;

	/**
	 * Feld zur Darstellung des aktuellen Zustandes des entsprechenden Bereichs.
	 */
	private Text _tAktZustand;

	private ComboViewer _cvMoegZustaende;

	private Button _bSchalten;

	/**
	 * Flag das anzeigt, das nur noch Abgebrochen werden kann. In diesem Fall
	 * stehen die anderen Schaltoptionen nicht mehr zur Verfgung.
	 */
	private boolean _nurAbrechenMoeglich = false;

	private Button _bAbbrechen;

	private Label _lAbschnitt;

	private Composite _compInfoTxt;

	private Label _lInfoTxt;

	public DialogSchalten(Shell parent,
			DialogSchaltenGuiManager dialogSchaltenGuiManager)
	{
		super(parent);
		this._dialogSchaltenGuiManager = dialogSchaltenGuiManager;
	}

	/**
	 * ffnet den Dialog zum Schalten des Abschnitts.
	 * 
	 * @param pidAbschnitt
	 *          PID des zu schaltenden <code>typ.tsfAbschnitt</code>.
	 * @param nurAbbrechenMoeglich
	 *          Flag das anzeigt, das nur noch Abgebrochen werden kann. In diesem
	 *          Fall stehen die anderen Schaltoptionen nicht mehr zur Verfgung.
	 * @return
	 */
	public Object oeffne(String pidAbschnitt, boolean nurAbbrechenMoeglich)
	{
		Object re = null;

		ClientDavInterface daVVerbindung = DaVVerbindung.getInstanz()
				.getDaVVerbindung();
		if (daVVerbindung != null)
		{
			DataModel dataModel = daVVerbindung.getDataModel();
			SystemObject so = dataModel.getObject(pidAbschnitt);
			if (so != null)
			{
				re = oeffne(so, nurAbbrechenMoeglich);
			}
		}

		return re;
	}

	/**
	 * ffnet den Dialog zum Schalten des Abschnitts.
	 * 
	 * @param so
	 *          {@link SystemObject} vom Typ <code>typ.tsfAbschnitt</code> des zu
	 *          schaltenden Abschnitts.
	 * @param nurAbbrechenMoeglich
	 *          Flag das anzeigt, das nur noch Abgebrochen werden kann. In diesem
	 *          Fall stehen die anderen Schaltoptionen nicht mehr zur Verfgung.
	 * @return
	 */
	public Object oeffne(SystemObject so, boolean nurAbbrechenMoeglich)
	{
		this._nurAbrechenMoeglich = nurAbbrechenMoeglich;
		this._soAbschnitt = so;

		this._shell = ShellOnTop.getShell(getParent(), SWT.TITLE | SWT.BORDER);

		_shell.addDisposeListener(new DisposeListener()
		{
			@Override
			public void widgetDisposed(DisposeEvent e)
			{
				dispose();
			}
		});

		/*
		 * GUI Aufbauen
		 */
		initGui(_shell);

		/*
		 * Listener anmelden.
		 */
		DaVKomTSFAbschnittBefehl.getInstance().getModel().addListener(this);

		/*
		 * Daten darstellen
		 */
		// setData(data);

		/*
		 * Validieren
		 */
		validate();

		/*
		 * Zentrieren
		 */

		// Grsse Bildschirm
		Toolkit test = java.awt.Toolkit.getDefaultToolkit();

		/*
		 * Auflsung durch 2 subtrahiert durch die Hlfte der in der Shell
		 * angegebenen Horizontalen, damit der Dialog mittig am Bildschirm erscheint
		 */

		int x = test.getScreenSize().width / 2 - DEF_WIDTH / 2;

		/*
		 * Auflsung durch 2 subtrahiert durch die Hlfte der in der Shell
		 * angegebenen Vertikalen
		 */

		int y = test.getScreenSize().height / 2 - DEF_HEIGHT / 2;

		_shell.setBounds(x, y, DEF_WIDTH, DEF_HEIGHT);

		/*
		 * Statische Positionierung wenn mglich
		 */
		if ((so != null) && (so.getPid() != null))
		{
			Point location = StaticDialogLocation.getInstance().getDlgLoc(
					this.getClass().getName(), so.getPid());
			if (location != null)
			{
				_shell.setLocation(location);
			}
		}

		_shell.open();

		if (isNurAbbrechenMoeglich())
		{
			MessageBox mb = new MessageBox(_shell, SWT.OK | SWT.ICON_WARNING);
			mb.setText("Z.Zt. nur Abbrechen/Schnellsperrung mglich");

			StringBuffer sbMsg = new StringBuffer();
			sbMsg.append("Der Abschnitt ");
			if (getSoAbschnitt() != null)
			{
				sbMsg.append(getSoAbschnitt().getNameOrPidOrId());
			}
			sbMsg.append("\n");
			sbMsg.append("befindet sich z.Zt. in einer aktiven Schaltung.\n");
			sbMsg.append("Solange stehen nur die Optionen \"Abbrechen\" und\n");
			sbMsg.append("\"Schnellsperrung\" zur Verffung!");
			mb.setMessage(sbMsg.toString());

			mb.open();
		}

		Display display = getParent().getDisplay();
		while (!_shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		return null;
	}

	/**
	 * Prft ob nur noch die Schaltflche Abbrechen aktiv sein darf. Dies ist z.B.
	 * dann der Fall, wenn sich der entsprechende Abschnitt bereits in eine
	 * aktiven Schaltung befindet.
	 * 
	 * @return Liefert <code>true</code> wenn nur der Button "Abbrechen" aktiv
	 *         sein darf, sonst <code>false</code>.
	 */
	private boolean isNurAbbrechenMoeglich()
	{
		return _nurAbrechenMoeglich;
	}

	public void initGui(Shell parent)
	{
		parent.setText("Temporre Seitenstreifenfreigabe schalten");

		 //Fr die F1 Taste
	    PlatformUI.getWorkbench().getHelpSystem().setHelp( parent, Activator.PLUGIN_ID + ".tsfAbschnittSchalten" );
		
		getCompositeAbschnitt(parent);
		getCompositeInfotext(parent);

		// getCompTitel(parent);
		getCompContent(parent);
		getCompCtrl(parent);

		/*
		 * Layout
		 */
		parent.setLayout(new GridLayout(1, false));

		GridData gdLAbs = new GridData();
		gdLAbs.horizontalSpan = 1;
		gdLAbs.grabExcessHorizontalSpace = true;
		gdLAbs.horizontalAlignment = GridData.FILL;
		getCompositeAbschnitt(parent).setLayoutData(gdLAbs);

		GridData gdCompInfoText = new GridData();
		gdCompInfoText.grabExcessHorizontalSpace = true;
		gdCompInfoText.grabExcessVerticalSpace = true;
		gdCompInfoText.horizontalAlignment = GridData.FILL;
		gdCompInfoText.verticalAlignment = GridData.FILL;
		getCompositeInfotext(parent).setLayoutData(gdCompInfoText);

		// GridData gdCompTitel = new GridData();
		// gdCompTitel.grabExcessHorizontalSpace = true;
		// gdCompTitel.horizontalAlignment = GridData.FILL;
		// getCompTitel(parent).setLayoutData(gdCompTitel);

		GridData gdCompContent = new GridData();
		gdCompContent.grabExcessHorizontalSpace = true;
		gdCompContent.grabExcessVerticalSpace = true;
		gdCompContent.horizontalAlignment = GridData.FILL;
		gdCompContent.verticalAlignment = GridData.FILL;
		getCompContent(parent).setLayoutData(gdCompContent);

		GridData gdCompCtrl = new GridData();
		gdCompCtrl.grabExcessHorizontalSpace = true;
		gdCompCtrl.horizontalAlignment = GridData.CENTER;
		getCompCtrl(parent).setLayoutData(gdCompCtrl);
	}

	private Composite getCompositeInfotext(Composite parent)
	{
		if (_compInfoTxt == null)
		{
			_compInfoTxt = new Composite(parent, SWT.BORDER);

			Label lFrag1 = new Label(_compInfoTxt, SWT.None);
			// lFrag1.setImage(getAchtungImage());
			lFrag1.setImage(getImageLinks());

			_lInfoTxt = new Label(_compInfoTxt, SWT.CENTER);
//			Font fontInfoTxt = new Font(null, "Arial", 18, SWT.BOLD);
			_lInfoTxt.setFont(DefsFonts.FONT_ARIAL_18_PKT_BOLD);
			// _lInfoTxt.setText("Ist der Abschnitt frei und\nkann geschalten werden?");
			_lInfoTxt.setText("Temporre\nSeitenstreifenfreigabe\nschalten");

			Label lFrag2 = new Label(_compInfoTxt, SWT.None);
			// lFrag2.setImage(getAchtungImage());
			lFrag2.setImage(getImageRechts());

			_compInfoTxt.setLayout(new GridLayout(3, false));

			GridData gdLFrag1 = new GridData();
			gdLFrag1.grabExcessHorizontalSpace = true;
			gdLFrag1.grabExcessVerticalSpace = true;
			gdLFrag1.horizontalAlignment = GridData.CENTER;
			gdLFrag1.verticalAlignment = GridData.CENTER;
			lFrag1.setLayoutData(gdLFrag1);

			GridData gdLInfoTxt = new GridData();
			gdLInfoTxt.grabExcessHorizontalSpace = true;
			gdLInfoTxt.horizontalAlignment = GridData.CENTER;
			_lInfoTxt.setLayoutData(gdLInfoTxt);

			GridData gdLFrag2 = new GridData();
			gdLFrag2.grabExcessHorizontalSpace = true;
			gdLFrag2.grabExcessVerticalSpace = true;
			gdLFrag2.horizontalAlignment = GridData.CENTER;
			gdLFrag2.verticalAlignment = GridData.CENTER;
			lFrag2.setLayoutData(gdLFrag2);
		}

		return _compInfoTxt;
	}

	private Composite getCompositeAbschnitt(Composite parent)
	{
		if (_compAbschnitt == null)
		{
			_compAbschnitt = new Composite(parent, SWT.BORDER);

			_lAbschnitt = new Label(_compAbschnitt, SWT.NONE);
//			Font fontAbschnitt = new Font(null, "Arial", 12, SWT.BOLD);
			_lAbschnitt.setFont(DefsFonts.FONT_ARIAL_12_PKT_BOLD);
			_lAbschnitt.setText("Abschnitt: -");

			_compAbschnitt.setLayout(new GridLayout(1, false));

			GridData gdLAbschnitt = new GridData();
			gdLAbschnitt.grabExcessHorizontalSpace = true;
			gdLAbschnitt.horizontalAlignment = GridData.CENTER;
			_lAbschnitt.setLayoutData(gdLAbschnitt);

			if (getSoAbschnitt() != null)
			{
				_lAbschnitt.setText(getSoAbschnitt().getNameOrPidOrId());
			} else
			{
				_lAbschnitt.setText("Abschnitt: KEIN ABSCHNITT GESETZT!");
			}

		}

		return _compAbschnitt;
	}

	public SystemObject getSoAbschnitt()
	{
		return _soAbschnitt;
	}

	public Composite getCompTitel(Composite parent)
	{
		if (_compTitel == null)
		{
			_compTitel = new Composite(parent, SWT.BORDER);

			// Image Links
			Label l1 = new Label(_compTitel, SWT.NONE);
			l1.setImage(ImageGetter.getImage("icons/system-shutdown_64x64.png"));

			// Abschnitt
			Label lAbschnitt = new Label(_compTitel, SWT.NONE);
//			Font fLA = new Font(null, "Arial", 14, SWT.BOLD);
			lAbschnitt.setFont(DefsFonts.FONT_ARIAL_14_PKT_BOLD);
			if (getSoAbschnitt() != null)
			{
				lAbschnitt.setText("Abschnitt: " + getSoAbschnitt().getNameOrPidOrId());
			} else
			{
				lAbschnitt.setText("Abschnitt: KEIN ABSCHNITT GESETZT!");
			}

			// Image rechts
			Label l2 = new Label(_compTitel, SWT.NONE);
			l2.setImage(ImageGetter.getImage("icons/system-shutdown_64x64.png"));

			/*
			 * Layout
			 */
			_compTitel.setLayout(new GridLayout(3, false));

			GridData gdLAbschnitt = new GridData();
			gdLAbschnitt.grabExcessHorizontalSpace = true;
			gdLAbschnitt.horizontalAlignment = GridData.CENTER;
			lAbschnitt.setLayoutData(gdLAbschnitt);
		}

		return _compTitel;
	}

	public Composite getCompContent(Composite parent)
	{
		if (_compContent == null)
		{
			_compContent = new Composite(parent, SWT.BORDER);

			// Aktueller Zustand
			Label lAktZustand = new Label(_compContent, SWT.None);
			lAktZustand.setText("Aktueller Zustand: ");

			_tAktZustand = new Text(_compContent, SWT.BORDER);
			_tAktZustand.setEditable(false);
			_tAktZustand.setText("Nicht Verfgbar");

			// Mgliche Zustnde
			Label lMoegZustaende = new Label(_compContent, SWT.NONE);
			lMoegZustaende.setText("Gewnschter Zustand: ");

			_cvMoegZustaende = new ComboViewer(_compContent, SWT.NONE);
			_cvMoegZustaende
					.addSelectionChangedListener(new ISelectionChangedListener()
					{

						@Override
						public void selectionChanged(SelectionChangedEvent event)
						{
							validateButtonSchalten();
						}
					});
			_cvMoegZustaende.setContentProvider(new CP());

			E_TSFBefehl[] initialeBefehle = new E_TSFBefehl[] { 
			    E_TSFBefehl.Freigabe,
					E_TSFBefehl.FreigabeArgoscan, 
					E_TSFBefehl.Sperrung,
					E_TSFBefehl.Schnellsperrung };
			_cvMoegZustaende.setInput(initialeBefehle);

			/*
			 * Layout
			 */
			_compContent.setLayout(new GridLayout(2, false));

			GridData gdTAktZustand = new GridData();
			gdTAktZustand.grabExcessHorizontalSpace = true;
			gdTAktZustand.horizontalAlignment = GridData.FILL;
			_tAktZustand.setLayoutData(gdTAktZustand);

			GridData gdComboMoegZustaende = new GridData();
			gdComboMoegZustaende.grabExcessHorizontalSpace = true;
			gdComboMoegZustaende.horizontalAlignment = GridData.FILL;
			_cvMoegZustaende.getCombo().setLayoutData(gdComboMoegZustaende);

		}

		return _compContent;
	}

	public Composite getCompCtrl(Composite parent)
	{
		if (_compCtrl == null)
		{
			_compCtrl = new Composite(parent, SWT.NONE);

			_bSchalten = new Button(_compCtrl, SWT.PUSH);
			_bSchalten.setText("Schalten");
			_bSchalten.addSelectionListener(new SelectionAdapter()
			{
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					aktionSchalten();
				}
			});

			_bAbbrechen = new Button(_compCtrl, SWT.PUSH);
			_bAbbrechen.setText("TSF Abbrechen");

			/*
			 * MO 05.09.11 Abbrechen muss immer mglich sein !!!
			 */
			// _bAbbrechen.setEnabled(false);
			_bAbbrechen.addSelectionListener(new SelectionAdapter()
			{
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					aktionAbbrechen();
				}
			});

			Button bSchliessen = new Button(_compCtrl, SWT.PUSH);
			bSchliessen.setText("Schlieen");
			bSchliessen.addSelectionListener(new SelectionAdapter()
			{

				@Override
				public void widgetSelected(SelectionEvent e)
				{
					if (_shell != null)
					{
						_shell.dispose();
					}
				}

			});

			/*
			 * Layout
			 */
			_compCtrl.setLayout(new RowLayout());
		}

		return _compCtrl;
	}

	public void dispose()
	{
		if (_dialogSchaltenGuiManager != null)
		{
			if (getSoAbschnitt() != null)
			{
				_dialogSchaltenGuiManager.removeDialogSchalten(getSoAbschnitt()
						.getPid());
			}

			DaVKomTSFAbschnittBefehl.getInstance().getModel().removeListener(this);
		}
	}

	public void setZustand(final M_TSFAbschnittBefehl absBefehl)
	{
		Runnable run = new Runnable()
		{

			@Override
			public void run()
			{
				try
				{
					if (absBefehl != null)
					{
					  ZustandSammlung zustandSteuerungSammlung = DaVKomTSFSteuerung.getInstance().getModelZustand().getZustandSteuerungSammlung(_soAbschnitt.getPid());
					  E_TSFBefehl zielProgramm = zustandSteuerungSammlung.getZielProgramm();
					  if (zielProgramm.equals(E_TSFBefehl.Sperrung) || zielProgramm.equals(E_TSFBefehl.Schnellsperrung))
					    _bAbbrechen.setEnabled(false);
					  else
					    _bAbbrechen.setEnabled(true);
						/*
						 * MO 05.09.11
						 */
						if (Defs.pruefeObSteuerungFuerAbschnittInBearbeitung(_soAbschnitt
								.getPid()))
						{
							_tAktZustand.setText("in Bearbeitung");

							// E_TSFBefehl[] aBefeheleAuswehlbar = new E_TSFBefehl[0];
							E_TSFBefehl[] aBefeheleAuswehlbar = new E_TSFBefehl[] { E_TSFBefehl.Schnellsperrung };
							_cvMoegZustaende.setInput(aBefeheleAuswehlbar);

							_nurAbrechenMoeglich = true;
							validateButtonSchalten();

							return;
						}

						_tAktZustand.setText(absBefehl.getAktion().toString());

						/*
						 * MO 05.09.11
						 */
						Color rot = new Color(_tAktZustand.getDisplay(), 255, 0, 0);
						Color gruen = new Color(_tAktZustand.getDisplay(), 0, 128, 0);

						LinkedList<E_TSFBefehl> liste = new LinkedList<E_TSFBefehl>();

						if ((absBefehl.getAktion() == E_TSFBefehl.Freigabe)
								|| (absBefehl.getAktion() == E_TSFBefehl.FreigabeArgoscan)
								)
						{
							_tAktZustand.setForeground(gruen);

							liste.add(E_TSFBefehl.Sperrung);
							liste.add(E_TSFBefehl.Schnellsperrung);
						} else if ((absBefehl.getAktion() == E_TSFBefehl.Sperrung)
								|| (absBefehl.getAktion() == E_TSFBefehl.Schnellfreigabe))
						{
							_tAktZustand.setForeground(rot);

							liste.add(E_TSFBefehl.Freigabe);
//							liste.add(E_TSFBefehl.FreigabeArgoscan);
						} else
						{
							liste.add(E_TSFBefehl.Freigabe);
//							liste.add(E_TSFBefehl.FreigabeArgoscan);
							liste.add(E_TSFBefehl.Sperrung);
							liste.add(E_TSFBefehl.Schnellsperrung);
						}

						E_TSFBefehl[] aBefeheleAuswehlbar = liste
								.toArray(new E_TSFBefehl[liste.size()]);
						_cvMoegZustaende.setInput(aBefeheleAuswehlbar);

						/*
						 * MO 05.09.11
						 */
						if (aBefeheleAuswehlbar.length > 0)
							_cvMoegZustaende.setSelection(new StructuredSelection(
									aBefeheleAuswehlbar[0]));

						/*
						 * Bestimmen, ob der Button TSF Abbrechen aktiv sein darf oder
						 * nicht. Ist gerade kein Schaltvorgang aktiv, muss dieser INAKTIV
						 * sein!
						 */

						/*
						 * MO 05.09.11
						 */
						if (Defs.pruefeObSteuerungFuerAbschnittInBearbeitung(_soAbschnitt
								.getPid()))
						{
							liste.clear();

							/*
							 * Schnellsperrung ist nur dann mglich, wenn nicht gerade
							 * Schnellsperrung im Gange ist.
							 */
							if ((absBefehl != null) && (absBefehl.getAktion() != null)
									&& (absBefehl.getAktion() != E_TSFBefehl.Schnellsperrung))
							{
								liste.add(E_TSFBefehl.Schnellsperrung);
							}

							aBefeheleAuswehlbar = liste
									.toArray(new E_TSFBefehl[liste.size()]);
							_cvMoegZustaende.setInput(aBefeheleAuswehlbar);

							_nurAbrechenMoeglich = true;
							validateButtonSchalten();
						} else
						{
							_nurAbrechenMoeglich = false;
							validateButtonSchalten();
						}

					} else
					{
						_tAktZustand.setText("-");
					}
				} catch (SWTException swte)
				{
					swte.printStackTrace();
				}
			}
		};

		PlatformUI.getWorkbench().getDisplay().asyncExec(run);

	}

	@Override
	public void propertyChange(PropertyChangeEvent evt)
	{
		if (evt.getPropertyName().equals(GUIModelTSFAbschnittBefehl.PROP_INIT))
		{
			if ((evt.getNewValue() != null)
					&& (evt.getNewValue() instanceof WrapperListenerBenachrichtigung[]))
			{
				WrapperListenerBenachrichtigung[] array = (WrapperListenerBenachrichtigung[]) evt
						.getNewValue();
				for (WrapperListenerBenachrichtigung wrapperListenerBenachrichtigung : array)
				{
					if (wrapperListenerBenachrichtigung.getAbschnittBefehl().getSo() == getSoAbschnitt())
					{
						if (wrapperListenerBenachrichtigung.getPidAsp().equals(
								Defs.ASP_TSF_ZUSTAND))
						{
							setZustand(wrapperListenerBenachrichtigung.getAbschnittBefehl());
						}
					}
				}
			}
		} else if (evt.getPropertyName().equals(
				GUIModelTSFAbschnittBefehl.PROP_ADDED))
		{
			if ((evt.getNewValue() != null)
					&& (evt.getNewValue() instanceof WrapperListenerBenachrichtigung))
			{
				WrapperListenerBenachrichtigung wrapperListenerBenachrichtigung = (WrapperListenerBenachrichtigung) evt
						.getNewValue();
				if (wrapperListenerBenachrichtigung.getAbschnittBefehl().getSo() == getSoAbschnitt())
				{
					if (wrapperListenerBenachrichtigung.getAbschnittBefehl() != null)
					{
						if (wrapperListenerBenachrichtigung.getPidAsp().equals(
								Defs.ASP_TSF_ZUSTAND))
						{
							setZustand(wrapperListenerBenachrichtigung.getAbschnittBefehl());
						}
					}
				}
			}
		} else if (evt.getPropertyName().equals(
				GUIModelTSFAbschnittBefehl.PROP_REMOVED))
		{
			if ((evt.getOldValue() != null)
					&& (evt.getOldValue() instanceof WrapperListenerBenachrichtigung))
			{
				WrapperListenerBenachrichtigung wrapperListenerBenachrichtigungEntfernt = (WrapperListenerBenachrichtigung) evt
						.getOldValue();
				if (wrapperListenerBenachrichtigungEntfernt.getAbschnittBefehl()
						.getSo() == getSoAbschnitt())
				{
					if (wrapperListenerBenachrichtigungEntfernt.getAbschnittBefehl() != null)
					{
						if (wrapperListenerBenachrichtigungEntfernt.getPidAsp().equals(
								Defs.ASP_TSF_ZUSTAND))
						{
							setZustand(null);
						}
					}
				}
			}
		}
	}

	public class CP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof E_TSFBefehl[])
			{
				re = (E_TSFBefehl[]) inputElement;
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}
	}

	private void validate()
	{
		validateButtonSchalten();
	}

	private void validateButtonSchalten()
	{
		if (_bSchalten != null)
		{
			ISelection selection = _cvMoegZustaende.getSelection();
			if (selection instanceof StructuredSelection)
			{
				StructuredSelection ss = (StructuredSelection) selection;
				if (!ss.isEmpty())
				{
					_bSchalten.setEnabled(true);
				} else
				{
					_bSchalten.setEnabled(false);
				}
			}
		}

		/*
		 * Selektion muss deaktiviert sein, Schalten nicht mehr mglich. Nur Noch
		 * der Button "Abbrechen" darf aktiv sein.
		 */
		if (isNurAbbrechenMoeglich())
		{
			// _cvMoegZustaende.getCombo().setEnabled(false);
			// _bSchalten.setEnabled(false);
		} else
		{
			// _cvMoegZustaende.getCombo().setEnabled(true);
		}
	}

	private void aktionSchalten()
	{
		if (getSoAbschnitt() != null)
		{
			if ((_cvMoegZustaende != null)
					&& (!_cvMoegZustaende.getSelection().isEmpty()))
			{
				if (_cvMoegZustaende.getSelection() instanceof StructuredSelection)
				{
					StructuredSelection ss = (StructuredSelection) _cvMoegZustaende
							.getSelection();
					Object firstElement = ss.getFirstElement();
					if ((firstElement != null) && (firstElement instanceof E_TSFBefehl))
					{
						E_TSFBefehl aktion = (E_TSFBefehl) firstElement;

						M_TSFAbschnittBefehl befehl = new M_TSFAbschnittBefehl(Defs
								.getClientId(), getSoAbschnitt(), aktion,
								E_TSFProgrammZustand.Geschaltet);

						WrapperULDInfo wrapperULDInfoFehlerInfo = DaVKomTSFAbschnittBefehl
								.getInstance().sendeAbschnittBefehl(befehl);

						String fehlerInfo = wrapperULDInfoFehlerInfo.getFehlerInfo();
						if ((fehlerInfo == null) && (_shell != null)
								&& (!_shell.isDisposed()))
						{
							_shell.dispose();
						} else
						{
							System.err.println("=> FEHLERINFO SCHALTEN: " + fehlerInfo);
						}

						// Immer schlieen!
						if (!_shell.isDisposed())
						{
							_shell.dispose();
						}
					}
				}
			}
		}
	}

	private void aktionAbbrechen()
	{
		if (getSoAbschnitt() != null)
		{
			M_TSFAbschnittBefehl befehl = new M_TSFAbschnittBefehl(
					Defs.getClientId(), getSoAbschnitt(), E_TSFBefehl.Abbrechen,
					E_TSFProgrammZustand.Geschaltet);

			WrapperULDInfo wrapperULDInfo = DaVKomTSFAbschnittBefehl.getInstance()
					.sendeAbschnittBefehl(befehl);

			String fehlerInfo = wrapperULDInfo.getFehlerInfo();
			if ((fehlerInfo == null) && (_shell != null) && (!_shell.isDisposed()))
			{
				_shell.dispose();
			} else
			{
				System.err.println("=> FEHLERINFO ABBRECHEN: " + fehlerInfo);
			}
		}
	}

	private static Image getImageLinks()
	{
		if (_kameraImageLinks == null)
		{
			_kameraImageLinks = Activator.getImageDescriptor("icons/tsf_links.png")
					.createImage();
		}

		return _kameraImageLinks;
	}

	private static Image getImageRechts()
	{
		if (_kameraImageRechts == null)
		{
			_kameraImageRechts = Activator.getImageDescriptor("icons/tsf_rechts.png")
					.createImage();
		}

		return _kameraImageRechts;
	}

}
