/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFSteuerung;
import de.sts.buv.plugin.workflow.tsf.dav.Defs;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFProgrammZustand;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung;
import de.sts.buv.plugin.workflow.tsf.dav.IListenerZustandsSammlungReset;
import de.sts.buv.plugin.workflow.tsf.dav.I_TSFZustand;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFZustandArgoscan;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFZustandSteuerung;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung.ZustandSammlung;

/**
 * Generiert ein Composite zur Darstellung des Zustandes der Steuerung (
 * <code>atg.tsfZustandSteuerung</code>) dar.
 * 
 * @author HofmannJ
 * 
 */
public class GeneratorCompositeZustand implements PropertyChangeListener,
		IListenerZustandsSammlungReset
{
	private static final int IDX_COL_ZEITSTEMPEL = 0;
	private static final int IDX_COL_PRG = 1;
	private static final int IDX_COL_STUFE = 2;
	private static final int IDX_COL_ZUSTAND = 3;
	private static final int IDX_COL_MAKRO_EREIGNIS = 4;
	private static final String[] COL_NAMES = new String[] { "Zeitstempel",
			"Programm", "Stufe", "Zustand", "Makro/Ereignis" };

	/**
	 * SystemObjekt vom Typ <code>typ.tfsAbschnitt</code>.
	 */
	private SystemObject _so = null;
	private Composite _composite;
	private Text _tAbschnitt;
	private Text _tZielPrg;
	private Text _tPrgZust;
	private TableViewer _tv;

	private ZustandSammlung _data = null;

	/**
	 * TabItem, auf dem dieses Element angeordnet ist.
	 */
	private CTabItem _tabItem = null;

	/**
	 * 
	 * @param so
	 *          PID eines {@link SystemObject}s vom Typ
	 *          <code>typ.tfsAbschnitt</code>.
	 */
	public GeneratorCompositeZustand(String pid)
	{
		if (pid != null)
		{
			ClientDavInterface con = DaVVerbindung.getInstanz().getDaVVerbindung();
			if (con != null)
			{
				DataModel dataModel = con.getDataModel();
				SystemObject so = dataModel.getObject(pid);
				if (so != null)
				{
					setSO(so);
				}
			}
		}
	}

	/**
	 * 
	 * @param so
	 *          Ein {@link SystemObject} vom Typ <code>typ.tfsAbschnitt</code>.
	 */
	public GeneratorCompositeZustand(SystemObject so)
	{
		setSO(so);
	}

	private void setSO(SystemObject so)
	{
		this._so = so;
	}

	public Composite getComp(Composite parent)
	{
		if (_composite == null)
		{
			_composite = new Composite(parent, SWT.BORDER);

			// Abschnitt
			Label lAbschnitt = new Label(_composite, SWT.NONE);
			lAbschnitt.setText("Abschnitt: ");

			_tAbschnitt = new Text(_composite, SWT.BORDER);
			_tAbschnitt.setEditable(false);

			// ZielPrg
			Label lZielPrg = new Label(_composite, SWT.NONE);
			lZielPrg.setText("Ziel-Programm: ");

			_tZielPrg = new Text(_composite, SWT.BORDER);
			_tZielPrg.setEditable(false);

			// Prg Zustand
			Label lPrgZust = new Label(_composite, SWT.NONE);
			lPrgZust.setText("Programm Zustand: ");

			_tPrgZust = new Text(_composite, SWT.BORDER);
			_tPrgZust.setEditable(false);

			createTable(_composite);

			/*
			 * Layout
			 */
			_composite.setLayout(new GridLayout(2, false));

			GridData gdTAbschnitt = new GridData();
			gdTAbschnitt.grabExcessHorizontalSpace = true;
			gdTAbschnitt.horizontalAlignment = GridData.FILL;
			_tAbschnitt.setLayoutData(gdTAbschnitt);

			GridData gdTZielPrg = new GridData();
			gdTZielPrg.grabExcessHorizontalSpace = true;
			gdTZielPrg.horizontalAlignment = GridData.FILL;
			_tZielPrg.setLayoutData(gdTZielPrg);

			GridData gdPrgZust = new GridData();
			gdPrgZust.grabExcessHorizontalSpace = true;
			gdPrgZust.horizontalAlignment = GridData.FILL;
			_tPrgZust.setLayoutData(gdPrgZust);

			GridData gdTable = new GridData();
			gdTable.grabExcessHorizontalSpace = true;
			gdTable.grabExcessVerticalSpace = true;
			gdTable.horizontalAlignment = GridData.FILL;
			gdTable.verticalAlignment = GridData.FILL;
			gdTable.horizontalSpan = 2;
			createTable(_composite).setLayoutData(gdTable);

			listenerAnmelden();
		}

		return _composite;
	}

	public SystemObject getSO()
	{
		return _so;
	}

	private void listenerAnmelden()
	{
		GUIModelTSFSteuerung modelVorgabe = DaVKomTSFSteuerung.getInstance()
				.getModelVorgabe();

		modelVorgabe.addListener(getSO(), this);
		modelVorgabe.addListenerZustandsSammlungReset(this);

	}

	private void listenerAbmelden()
	{
		GUIModelTSFSteuerung modelVorgabe = DaVKomTSFSteuerung.getInstance()
				.getModelVorgabe();
		modelVorgabe.removeListener(getSO(), this);
		modelVorgabe.removeListenerZustandsSammlungReset(this);
	}

	private Table createTable(Composite parent)
	{
		if (_tv == null)
		{
			Table table = new Table(parent, SWT.FULL_SELECTION | SWT.SINGLE
					| SWT.H_SCROLL | SWT.V_SCROLL);
			table.setHeaderVisible(true);
			table.setLinesVisible(true);

			_tv = new TableViewer(table);

			TableColumn tcZeitstempel = new TableColumn(_tv.getTable(), SWT.LEFT);
			tcZeitstempel.setText(COL_NAMES[IDX_COL_ZEITSTEMPEL]);
			tcZeitstempel.setWidth(160);

			TableColumn tcPrg = new TableColumn(_tv.getTable(), SWT.LEFT);
			tcPrg.setText(COL_NAMES[IDX_COL_PRG]);
			tcPrg.setWidth(160);

			TableColumn tcStufe = new TableColumn(_tv.getTable(), SWT.LEFT);
			tcStufe.setText(COL_NAMES[IDX_COL_STUFE]);
			tcStufe.setWidth(50);

			TableColumn tcZustand = new TableColumn(_tv.getTable(), SWT.LEFT);
			tcZustand.setText(COL_NAMES[IDX_COL_ZUSTAND]);
			tcZustand.setWidth(350);

			TableColumn tcMakroEreignis = new TableColumn(_tv.getTable(), SWT.LEFT);
			tcMakroEreignis.setText(COL_NAMES[IDX_COL_MAKRO_EREIGNIS]);
			tcMakroEreignis.setWidth(100);

			_tv.setContentProvider(new CP());
			_tv.setLabelProvider(new LP());
		}

		return _tv.getTable();
	}

	public void setData(final ZustandSammlung data)
	{
		this._data = data;

		Runnable run = new Runnable()
		{

			@Override
			public void run()
			{
				if (_data != null)
				{
					/*
					 * Abschnitt
					 */
					SystemObject soAbs = data.getSoTfsAbschnitt();
					if (soAbs != null)
					{
						_tAbschnitt.setText(soAbs.getNameOrPidOrId());
					} else
					{
						_tAbschnitt.setText("-");
					}

					/*
					 * Ziel-Programm
					 */
					E_TSFBefehl zielProgramm = data.getZielProgramm();
					if (zielProgramm != null)
					{
						_tZielPrg.setText(zielProgramm.toString());
					} else
					{
						_tZielPrg.setText("-");
					}

					/*
					 * Programm-Zustand
					 */
					E_TSFProgrammZustand prgZust = data.getProgrammZustand();
					if (prgZust != null)
					{
						_tPrgZust.setText(prgZust.toString());
					} else
					{
						_tPrgZust.setText("-");
					}

					/*
					 * Tabelle fllen
					 */
					_tv.setInput(data);

					Table table = _tv.getTable();
					if (table.getItemCount() > 0)
					{
						table.select(table.getItemCount() - 1);
						table.showSelection();
					}

				} else
				{
					_tAbschnitt.setText("");
					_tZielPrg.setText("");
					_tPrgZust.setText("");
					_tv.setInput(null);
				}
			}
		};
		PlatformUI.getWorkbench().getDisplay().asyncExec(run);

		// if (_data != null)
		// {
		// /*
		// * Abschnitt
		// */
		// SystemObject soAbs = data.getSo();
		// if (soAbs != null)
		// {
		// _tAbschnitt.setText(soAbs.getNameOrPidOrId());
		// } else
		// {
		// _tAbschnitt.setText("-");
		// }
		//
		// /*
		// * Ziel-Programm
		// */
		// E_TSFBefehl zielProgramm = data.getZielProgramm();
		// if (zielProgramm != null)
		// {
		// _tZielPrg.setText(zielProgramm.toString());
		// } else
		// {
		// _tZielPrg.setText("-");
		// }
		//
		// /*
		// * Programm-Zustand
		// */
		// E_TSFProgrammZustand prgZust = data.getProgrammZustand();
		// if (prgZust != null)
		// {
		// _tPrgZust.setText(prgZust.toString());
		// } else
		// {
		// _tPrgZust.setText("-");
		// }
		//
		// /*
		// * Tabelle fllen
		// */
		// _tv.setInput(data);
		//
		// } else
		// {
		// _tAbschnitt.setText("");
		// _tZielPrg.setText("");
		// _tPrgZust.setText("");
		// _tv.setInput(null);
		// }

	}

	public void dispose()
	{
		listenerAbmelden();
	}

	private class CP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof ZustandSammlung)
			{
				ZustandSammlung sammlung = (ZustandSammlung) inputElement;
				I_TSFZustand[] aZustaendeSteuerung = sammlung.getZustaendeSteuerung();
				re = aZustaendeSteuerung;
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private class LP implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;

			if (element != null)
			{
				if (element instanceof I_TSFZustand)
				{
					I_TSFZustand zustand = (I_TSFZustand) element;

					switch (columnIndex)
					{
					case IDX_COL_ZEITSTEMPEL:
						if (zustand.getTimestamp() != null)
						{
							re = Defs.SDF.format(zustand.getTimestamp());
						} else
						{
							re = "-";
						}
						break;

					default:
						break;
					}
				}

				if (element instanceof M_TSFZustandSteuerung)
				{
					M_TSFZustandSteuerung zustandStrg = (M_TSFZustandSteuerung) element;

					switch (columnIndex)
					{
					case IDX_COL_PRG:
						if (zustandStrg.getProgramm() != null)
						{
							re = zustandStrg.getProgramm().toString();
						}
						break;
					case IDX_COL_STUFE:
						re = "" + zustandStrg.getStufe();
						break;
					case IDX_COL_ZUSTAND:
						if (zustandStrg.getZustand() != null)
						{
							re = zustandStrg.getZustand().toString();
						} else
						{
							re = "-";
						}
						break;

					default:
						break;
					}
				}

				if (element instanceof M_TSFZustandArgoscan)
				{
					M_TSFZustandArgoscan zustandArgoscan = (M_TSFZustandArgoscan) element;

					switch (columnIndex)
					{
					case IDX_COL_PRG:
						re = "Argoscan";
						break;
					case IDX_COL_STUFE:
						re = "";
						break;
					case IDX_COL_ZUSTAND:
						if (zustandArgoscan.getZustand() != null)
						{
							re = zustandArgoscan.getZustand().toString();
						} else
						{
							re = "-";
						}
						break;
					case IDX_COL_MAKRO_EREIGNIS:
						re = zustandArgoscan.getMakroEreignis().toString();
						break;
					default:
						break;
					}
				}
			} else
			{
				// TODO implemetieren...
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	@Override
	public void propertyChange(PropertyChangeEvent evt)
	{

		if (evt != null)
		{
			if ((evt.getPropertyName().equals(GUIModelTSFSteuerung.PROP_INIT))
					&& (evt.getNewValue() instanceof ZustandSammlung[]))
			{
				ZustandSammlung aSammlung = (ZustandSammlung) evt.getNewValue();

				setData(aSammlung);

			} else if ((evt.getPropertyName().equals(GUIModelTSFSteuerung.PROP_ADD))
					&& (evt.getNewValue() instanceof ZustandSammlung))
			{
				ZustandSammlung aSammlung = (ZustandSammlung) evt.getNewValue();

				setData(aSammlung);
			}
		}

	}

	public CTabItem getTabItem()
	{
		return _tabItem;
	}

	public void setTabItem(CTabItem tabItem)
	{
		_tabItem = tabItem;
	}

	@Override
	public void resetListe(SystemObject soTfsAbschnitt)
	{
		if ((soTfsAbschnitt != null) && (soTfsAbschnitt == getSO()))
		{
//			Runnable run = new Runnable()
//			{
//				
//				@Override
//				public void run()
//				{
//					getTabItem().getParent().setSelection(getTabItem());					
//				}
//			};
//			
//			PlatformUI.getWorkbench().getDisplay().asyncExec(run);
			
		}
	}

}
