/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Date;
import java.util.LinkedList;
import java.util.Vector;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

//import de.bsvrz.buv.rw.basislib.login.UrlasserInfo;
//import de.bsvrz.buv.rw.basislib.login.UrlasserObjekt;
//import de.bsvrz.buv.rw.basislib.login.UrlasserDialog.UrlasserDialogInfo;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserInfo;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserObjekt;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog.UrlasserDialogInfo;
import de.sts.buv.plugin.workflow.tsf.Activator;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFAbschnittBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFSteuerung;
import de.sts.buv.plugin.workflow.tsf.dav.Defs;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFProgrammZustand;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelArgoscanRuhezustand;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFAbschnittBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFAbschnitte;
import de.sts.buv.plugin.workflow.tsf.dav.WrapperULDInfo;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung.ZustandSammlung;
import de.sts.buv.plugin.workflow.tsf.dlg.InfoWeiterschaltenAbgebrochen.AbgebrochenesWeiterschaltenAbschnitt;

/**
 * Schaltet die Abschnitte eines Bereichs in der vom System/Workflow angedachten
 * Reihenfolge in einem Rutsch durch. <br>
 * <br>
 * Intention: Ein Thread der die zu schaltenden Elemente abarbeitet und ein
 * Thread, der als Watchdog dient. Wird eine Abbrechen gesendet, mssen beide
 * Threads gestoppt und {@link E_TSFBefehl#Abbrechen} an die Steuerung fr alle
 * Abschnitte eines Bereichs gesendet werden.<br>
 * <br>
 * <b>Hinweis</b>: Die dispose()-Funktion ist aufzurufen, wenn dieses Objekt
 * nicht mehr bentigt wird. Dies ist z.B. dann der Fall wenn ein Schaltdialog
 * fr diesen Bereich aufgerufen wurde, ohne eine Schaltung auszulsen.
 * Ansonsten wird in {@link TSFBereichSchaltenVerwaltung} ein Platz unntig
 * belegt.<br>
 * <br>
 * Des weiteren ist ber {@link TSFBereichSchaltenVerwaltung} zur prfen, ob
 * schon eine Instanz dieser Klasse fr die gleiche TSF existiert. Dies muss
 * z.B. von verwendenden Dialogen (wie {@link DialogTSFBereichSchalten} vor
 * dessen Instanziierung) erfolgen.<br>
 * <br>
 * Diese Klasse implementiert einen {@link PropertyChangeListener}, der bei
 * {@link GUIModelTSFSteuerung} angemeldet wird. Ist keiner der Abschnitte des
 * hier zu bearbeitenden Bereichs mehr von der Steuerung abgedeckt oder einer
 * der Abschnitte wurde entfernt (d.h. die Prozesse von Rolf Asal leben nicht
 * mehr), dann wird ein Dispose auf diese Bereichsschaltung durchgefhrt. Die
 * Detektion erfolgt ber den Event
 * {@link GUIModelTSFSteuerung#PROP_SAMMLUNG_REMOVED}.
 * 
 * @author HofmannJ
 * 
 */
public class TSFBereichSchalten implements PropertyChangeListener
{
	/**
	 * Legt fest ob die Abarbeitung der Merker fr die Abschnitte der Bereiche
	 * Sqeuentiell oder Parallel erfolgen soll.
	 */
	private static final boolean SEQUENTIELL_ABARBEITEN = true;

	private M_TSFAbschnitte _tsfBereichAbschnitte = null;

	/**
	 * Abschnitte die in vorgegebener Reihenfolge geschaltet werden sollen vom Typ
	 * <code>typ.tfsAbschnitt</code>.
	 */
	private LinkedList<SystemObject> _lSOAbs = new LinkedList<SystemObject>();

	/**
	 * Liste der abgearbeiteten Merker.
	 */
	private LinkedList<AbarbeitungsMerker> _lAbgearbeitet = new LinkedList<AbarbeitungsMerker>();

	/**
	 * Liste der noch abzuarbeitenden Merker. Das erste Element befindet sich
	 * aktuell in der Bearbeitung.
	 */
	private LinkedList<AbarbeitungsMerker> _lNochAbzuarbeiten = new LinkedList<AbarbeitungsMerker>();

	/**
	 * Wird vom Ersten abzuarbeitenden Element bezogen um die Daten fr die
	 * Abarbeitung der weiteren Elemente zur Verfgung zu haben.
	 */
	private UrlasserDialogInfo _urlasserDialogInfo = null;

	/**
	 * Speichert den Thread, der fr die Abarbeitung zustndig ist.
	 */
	private ThreadAbarbeitungMO _thAbarbeitung;

	private JobAbarbeitung _jobAbarbeitung;

	/**
	 * Der Befehl, der geschaltet werden soll.
	 */
	private E_TSFBefehl _befehl = null;

	/**
	 * Zeigt an ob diese Klasse schon disposed wurde oder nicht.
	 */
	private boolean _disposed = false;

	public TSFBereichSchalten(M_TSFAbschnitte tsfAbschnitte)
	{
		if (tsfAbschnitte != null)
		{
			init(tsfAbschnitte);
		}
	}

	/**
	 * Fhrt die Initialisierung durch, bestimmt die zu schaltenden Abschnitte,
	 * meldet sich bei {@link TSFBereichSchaltenVerwaltung} an.
	 * 
	 * @param tsfAbschnitte
	 */
	private void init(M_TSFAbschnitte tsfAbschnitte)
	{
		this._tsfBereichAbschnitte = tsfAbschnitte;

		SystemObject[] aAbs = _tsfBereichAbschnitte.getAbschnitte();
		synchronized (_tsfBereichAbschnitte)
		{
			for (SystemObject systemObject : aAbs)
			{
				if (systemObject != null)
				{
					_lSOAbs.add(systemObject);
				}
			}
		}

		TSFBereichSchaltenVerwaltung.getInstance().putTSFBereichSchalten(this);
	}

	/**
	 * Startet das Schalten eines entsprechenden Programms
	 * 
	 * @param befehl
	 */
	public void doStartProgramm(E_TSFBefehl befehl)
	{
		if (befehl != null)
		{
			this._befehl = befehl;

			if (befehl == E_TSFBefehl.Abbrechen)
			{
				// TODO Thread beenden
				// TODO Abbrechen raushauen.
			} else
			{
				/*
				 * Entsprechenden Befehl raushauen. Gltige Programme ber
				 * bestimmeSchaltbareProgramme bestimmbar.
				 */
				SystemObject[] aAbs = getAbschnitte();

				// Liste der abzuarbeitenden Merker erstellen.
				synchronized (_lNochAbzuarbeiten)
				{
					for (SystemObject so : aAbs)
					{
						if (so != null)
						{
							/*
							 * MO 02.09.11 Nur die Abschnitte schalten, die sich nicht schon
							 * in dem Zielprogramm befinden
							 */
							if (!Defs.istBestimmtesProgrammFuerAbschnittErreicht(so.getPid(),
									befehl))
							{
								AbarbeitungsMerker merker = new AbarbeitungsMerker(so, befehl);
								_lNochAbzuarbeiten.add(merker);
							}
						}
					}
				}

				/*
				 * Listener bei GUIModelTSFSteuerung anmelden
				 */
				DaVKomTSFSteuerung.getInstance().getModelVorgabe()
						.addListenerAllgemein(this);

				/*
				 * MO 02.09.11 Ablauf der TSF fr einen ganzen Abschnitt erfolgt
				 * wahlweise als Thread oder als Job (aktivierbar ber den Activator).
				 */
				if (!Activator._starteAlsJob)
				{
					if (befehl != E_TSFBefehl.Schnellsperrung)
					{
						_thAbarbeitung = new ThreadAbarbeitungMO();
					} else
					{
						alleMerkerParallelAbarbeiten();
					}
				} else
				{
					_jobAbarbeitung = new JobAbarbeitung(
							"Abarbeitung TFSSchalten - Bereich "
									+ _tsfBereichAbschnitte.getSo().getPidOrNameOrId()
									+ new Date());
				}
			}
		}
	}

	/**
	 * Bestimmt das Schaltbare Programm an Hand des ersten zu schaltenden
	 * Abschnitts.
	 * 
	 * @return Array der schaltbaren Programme (als {@link E_TSFBefehl}). Das
	 *         Array ist leer, wenn keine Programme geschaltet werden knnen.
	 */
	public E_TSFBefehl[] bestimmeSchaltbareProgramme()
	{
		LinkedList<E_TSFBefehl> lSchaltbareProgramme = new LinkedList<E_TSFBefehl>();
		SystemObject[] aAbs = getAbschnitte();

		if ((aAbs != null) && (aAbs.length > 0) && (aAbs[aAbs.length - 1] != null))
		{
			/*
			 * Da gegen die Fahrtrichtung geschaltet werden soll und die Abschnitte in
			 * Fahrtrichtung im DaV hinterlegt sind, wird der Zustand des letzten
			 * Abschnitts als Referenz verwendet.
			 */
			SystemObject pruefAbs = aAbs[aAbs.length - 1];

			GUIModelTSFSteuerung modelZustand = DaVKomTSFSteuerung.getInstance()
					.getModelZustand();
			ZustandSammlung zustandSteuerungSammlung = modelZustand
					.getZustandSteuerungSammlung(pruefAbs);

			if (zustandSteuerungSammlung != null)
			{
				E_TSFBefehl zielProgramm = zustandSteuerungSammlung.getZielProgramm();
				E_TSFProgrammZustand programmZustand = zustandSteuerungSammlung
						.getProgrammZustand();

				if (programmZustand == E_TSFProgrammZustand.Geschaltet)
				{
					if ((zielProgramm == E_TSFBefehl.Freigabe)
							|| (zielProgramm == E_TSFBefehl.FreigabeArgoscan))
					{
						lSchaltbareProgramme.add(E_TSFBefehl.Sperrung);
						lSchaltbareProgramme.add(E_TSFBefehl.Schnellsperrung);
						lSchaltbareProgramme.add(E_TSFBefehl.Abbrechen);
					} else if ((zielProgramm == E_TSFBefehl.Sperrung)
							|| (zielProgramm == E_TSFBefehl.Schnellsperrung))
					{
						lSchaltbareProgramme.add(E_TSFBefehl.Freigabe);
						lSchaltbareProgramme.add(E_TSFBefehl.Abbrechen);
					} else
					{
						logge("Zum aktuellen Programm \"" + zielProgramm
								+ "\" knnen keine schaltbaren Programme bestimmt werden.",
								Status.WARNING);
					}
				} else if (programmZustand == E_TSFProgrammZustand.Undefiniert)
				{
					lSchaltbareProgramme.add(E_TSFBefehl.Freigabe);
					lSchaltbareProgramme.add(E_TSFBefehl.Sperrung);
					lSchaltbareProgramme.add(E_TSFBefehl.Schnellsperrung);
					lSchaltbareProgramme.add(E_TSFBefehl.Abbrechen);
				} else if (programmZustand == null)
				{
					System.out.println("PRG ZUST NULL !!!!!!!!");
					lSchaltbareProgramme.add(E_TSFBefehl.Freigabe);
					lSchaltbareProgramme.add(E_TSFBefehl.Sperrung);
					lSchaltbareProgramme.add(E_TSFBefehl.Schnellsperrung);
				} else
				{
					/*
					 * Kein Schalten mglich, da in Bearbeitung. Nur Abbrechen soll
					 * mglich sein.
					 */
					lSchaltbareProgramme.add(E_TSFBefehl.Abbrechen);
				}

			} else
			{
				lSchaltbareProgramme.add(E_TSFBefehl.Freigabe);
				lSchaltbareProgramme.add(E_TSFBefehl.Sperrung);
				lSchaltbareProgramme.add(E_TSFBefehl.Schnellsperrung);
			}
		}

		return lSchaltbareProgramme.toArray(new E_TSFBefehl[lSchaltbareProgramme
				.size()]);
	}

	/**
	 * MO 02.09.11 Methode bestimmt die schaltbaren Programme auf Basis der
	 * derzeit geschalteten Programme des Bereichs.
	 * 
	 * @param aktProgrammLetzerAbschnitt
	 *          String mit den Namen der derzeit geschalteten Programme des
	 *          Bereichs
	 * @return Feld mit mglichen Befehlen fr die TSF
	 */
	public E_TSFBefehl[] bestimmeSchaltbareProgramme(
			String aktProgrammLetzerAbschnitt)
	{
		Vector<E_TSFBefehl> v = new Vector<E_TSFBefehl>();

		if (aktProgrammLetzerAbschnitt.equals(""))
		{
			v.add(E_TSFBefehl.Freigabe);
			v.add(E_TSFBefehl.FreigabeArgoscan);
			v.add(E_TSFBefehl.Sperrung);
			v.add(E_TSFBefehl.Schnellsperrung);
		} else
		{
			if ((aktProgrammLetzerAbschnitt.contains(E_TSFBefehl.Freigabe.name()))
					|| (aktProgrammLetzerAbschnitt.contains(E_TSFBefehl.FreigabeArgoscan
							.name())))
			{
				v.add(E_TSFBefehl.Sperrung);
				v.add(E_TSFBefehl.Schnellsperrung);
			}

			if (aktProgrammLetzerAbschnitt.contains(E_TSFBefehl.Sperrung.name()))
			{
				v.add(E_TSFBefehl.Freigabe);
				v.add(E_TSFBefehl.FreigabeArgoscan);
			}
		}

		return v.toArray(new E_TSFBefehl[v.size()]);
	}

	private void logge(String msg, int serv)
	{
		ILog log = Activator.getDefault().getLog();
		log.log(new Status(serv, Activator.PLUGIN_ID, msg));
	}

	/**
	 * Liefert den Bereich der Abschnitte fr den die Bearbeitung erfolgt. Kann
	 * genutzt werden um den gesamten Vorgang abzubrechen.
	 * 
	 * @return
	 */
	public M_TSFAbschnitte getBereichAbschnitte()
	{
		return _tsfBereichAbschnitte;
	}

	/**
	 * Liefert die zu schaltenden Abschnitte vom Typ <code>typ.tfsAbschnitt</code>
	 * in Fahrtrichtung.
	 * 
	 * @return Array oder leeres Array, niemals <code>null</code>.
	 */
	public SystemObject[] getAbschnitte()
	{
		SystemObject[] re = null;

		synchronized (_lSOAbs)
		{
			re = _lSOAbs.toArray(new SystemObject[_lSOAbs.size()]);
		}

		return re;
	}

	/**
	 * Bestimmt das aktuelle Programm des letzten Abschnitts. Achtung: Dieses
	 * Programm kann sich gerade in einem "Flu" befinden, wenn der Zustand
	 * "Geschaltet" noch nicht erreicht ist.
	 * 
	 * @return
	 */
	public E_TSFBefehl bestimmeAktProgrammLetzerAbschnitt()
	{
		E_TSFBefehl re = null;
		GUIModelTSFSteuerung modelZustand = DaVKomTSFSteuerung.getInstance()
				.getModelZustand();

		SystemObject[] abschnitte = getAbschnitte();
		if ((abschnitte != null) && (abschnitte.length > 0))
		{
			SystemObject soLetzerAbs = abschnitte[abschnitte.length - 1];

			ZustandSammlung zustandSteuerungSammlung = modelZustand
					.getZustandSteuerungSammlung(soLetzerAbs);
			if (zustandSteuerungSammlung != null)
			{
				re = zustandSteuerungSammlung.getZielProgramm();
			}
		}

		return re;
	}

	/**
	 * Bestimmt den Zustand des gesamten Bereichs. Sollten die Zustnde der
	 * einzelnen Abschnitte unterschiedlich sein, so werden alle Zustnde durch
	 * "/" getrennt aufgefhrt. Als Zustnde wird nur Freigen bzw. gesperrt
	 * ausgegeben.
	 * 
	 * @return String mit den Zustnden
	 */
	public String bestimmeAktProgrammBereich()
	{
		/*
		 * MO 02.09.11 Das Bestimmen des aktuellen Zustandes erfolgt nicht auf Basis
		 * des letzten Abschnitts sonder auf Basis aller Abschnitte. Nur dann ist es
		 * mglich im Falle dass ein Abschnitt Frei der andere aber gesperrt ist,
		 * beide auf Frei bzw. Gesperrt zu setzen
		 */

		String s = "";

		GUIModelTSFSteuerung modelZustand = DaVKomTSFSteuerung.getInstance()
				.getModelZustand();

		SystemObject[] abschnitte = getAbschnitte();
		if ((abschnitte != null) && (abschnitte.length > 0))
		{
			for (SystemObject so : abschnitte)
			{
				ZustandSammlung zustandSteuerungSammlung = modelZustand
						.getZustandSteuerungSammlung(so);
				if (zustandSteuerungSammlung != null)
				{
					if (!s.equals(""))
						s += " / ";

					if ((zustandSteuerungSammlung.getZielProgramm() == E_TSFBefehl.Freigabe)
							|| (zustandSteuerungSammlung.getZielProgramm() == E_TSFBefehl.FreigabeArgoscan)
							|| (zustandSteuerungSammlung.getZielProgramm() == E_TSFBefehl.Schnellfreigabe))
						s += E_TSFBefehl.Freigabe.name();

					if ((zustandSteuerungSammlung.getZielProgramm() == E_TSFBefehl.Sperrung)
							|| (zustandSteuerungSammlung.getZielProgramm() == E_TSFBefehl.Schnellsperrung))
						s += E_TSFBefehl.Sperrung.name();
				}
			}
		}

		boolean freigabe = false;
		boolean sperrung = false;

		if ((s.contains(E_TSFBefehl.Freigabe.name()))
				|| s.contains(E_TSFBefehl.FreigabeArgoscan.name()))
			freigabe = true;

		if (s.contains(E_TSFBefehl.Sperrung.name()))
			sperrung = true;

		/*
		 * Texte "frei / frei" bzw. "gesperrt / gesperrt" in "frei" bzw. "gesperrt"
		 * umwandeln
		 */
		if (freigabe && !sperrung)
			s = E_TSFBefehl.Freigabe.name();
		else if (!freigabe && sperrung)
			s = E_TSFBefehl.Sperrung.name();

		return s;
	}

	public class AbarbeitungsMerker implements PropertyChangeListener
	{
		/**
		 * Abschnitt, fr den auf die Abarbeitung gewartet wird. Ist vom Typ
		 * <code>typ.tfsAbschnitt</code>.
		 */
		private SystemObject _soAbschnitt = null;

		private Date _abarbeitungsStart = null;
		private Date _abarbeitungsEnde = null;

		/**
		 * Zielprogramm welches erreicht sein muss, damit die Abarbeitung als
		 * Erfolgreich markiert werden kann.
		 */
		private E_TSFBefehl _zielProgramm = null;

		/**
		 * 
		 * 
		 * @param soAbschnitt
		 *          SystemObjekt des zu Bearbeitenden Abschnitts vom Typ
		 *          <code>typ.tfsAbschnitt</code>.
		 * @param zielProgramm
		 *          Das zu erreichende Zielprogramm, damit die Abarbeitung
		 *          determiniert werden kann.
		 */
		public AbarbeitungsMerker(SystemObject soAbschnitt, E_TSFBefehl zielProgramm)
		{
			if (soAbschnitt != null)
			{
				this._soAbschnitt = soAbschnitt;
				this._zielProgramm = zielProgramm;
			}
		}

		/**
		 * Prft ob das Zielprogramm bereits erreicht wurde oder sich noch in
		 * Bearbeitung befindet.
		 * 
		 * @return
		 */
		protected boolean pruefeObArbarbeitungNochGestartetWerdenMuss()
		{
			boolean mussNochGeschaltetWerden = true;

			/*
			 * Prfen ob das Zielprogramm nicht schon in Bearbeitung ist
			 */
			boolean zielPrgInBearbeitung = Defs
					.istZielProgrammFuerAbschnittInBearbeitung(
							getSoAbaschnitt().getPid(), getZielProgramm());
			boolean zielPrgBereitsErreicht = Defs
					.istBestimmtesProgrammFuerAbschnittErreicht(getSoAbaschnitt()
							.getPid(), getZielProgramm());

			if (zielPrgInBearbeitung || zielPrgBereitsErreicht)
			{
				mussNochGeschaltetWerden = false;
			}

			return mussNochGeschaltetWerden;
		}

		/**
		 * Startet die Abarbeitung in dem beim Modell ein Listener angemeldet wird
		 * und der Startzeitpunkt gesetzt wird.
		 */
		protected boolean starteAbarbeitung()
		{
			boolean bearbeitungGestartet = false;

			if (getZielProgramm() == E_TSFBefehl.FreigabeArgoscan)
			{
				/*
				 * Wenn sich Argoscan NICHT im Ruhezustand befindet mssen wir solange
				 * warten, bis sich Argoscan im Ruhezustand befindet.
				 */
				GUIModelArgoscanRuhezustand modelArgoscanRuhezustand = GUIModelArgoscanRuhezustand
						.getInstance();
				if (!modelArgoscanRuhezustand.isArgoscanInRuhezustand())
				{
//					System.err
//							.println("->ARGOSCAN _N I C H T_ IM RUHEZUSTAND. Pausiere die Bearbeitung, bis Ruhezustand erreicht.");
					synchronized (modelArgoscanRuhezustand)
					{
						try
						{
							modelArgoscanRuhezustand.wait();
						} catch (InterruptedException e)
						{
							e.printStackTrace();
						}
					}
				}
//				else
//				{
//					System.err
//							.println("->Argoscan befindet sich im Ruhezustand, setze die Bearbeitung fort.");
//				}
			}



			/*
			 * 1. Startzeitpkt. setzen und Listener anmelden.
			 */

			DaVKomTSFSteuerung.getInstance().getModelZustand().addListener(
					_soAbschnitt, this);

			InfoWeiterschaltenAbgebrochen.getInstance().addListener(this);

			/*
			 * 2. Zu setzendes Zielprogramm raushauen.
			 */

			M_TSFAbschnittBefehl befehl = new M_TSFAbschnittBefehl(
					Defs.getClientId(), getSoAbaschnitt(), getZielProgramm(),
					E_TSFProgrammZustand.Undefiniert);

			if (_urlasserDialogInfo == null)
			{
				/*
				 * Versand ber Urlasserdialog, falls noch nicht ein Urlasserdialog
				 * hierfr offen war.
				 */
				WrapperULDInfo sendeAbschnittBefehl = DaVKomTSFAbschnittBefehl
						.getInstance().sendeAbschnittBefehl(befehl);
				if (sendeAbschnittBefehl.getUldInfo() != null)
				{
					_urlasserDialogInfo = sendeAbschnittBefehl.getUldInfo();
				}

				if ((sendeAbschnittBefehl != null)
						&& (sendeAbschnittBefehl.getFehlerInfo() == null)
						&& (sendeAbschnittBefehl.getUldInfo() != null)
						&& (sendeAbschnittBefehl.getUldInfo().getEingabeOk()))
				{
					_abarbeitungsStart = new Date();
					bearbeitungGestartet = true;
				} else
				{
					System.err
							.println("ber den Urlasserdialog wurde Abbgebrochen oder es ist ein Fehler beim Versand aufgetreten (Versandfehler="
									+ sendeAbschnittBefehl.getFehlerInfo() + ")");
					this.dispose();
				}
			} else
			{
				/*
				 * Versenden mit den alten Verbindungsdaten.
				 */
				UrlasserInfo uli = new UrlasserInfo(_urlasserDialogInfo.getBenutzeer(),
						_urlasserDialogInfo.getPasswort(),
						_urlasserDialogInfo.getUrsache(), _urlasserDialogInfo
								.getVeranlasser());
//				uli.autokorrekturUrsacheVeranlasser();

				ClientDavInterface con = DaVVerbindung.getInstanz().getDaVVerbindung();
				Data data = befehl.toData(con);

				DataModel dataModel = con.getDataModel();
				AttributeGroup atg = dataModel
						.getAttributeGroup(Defs.ATG_TSF_ABSCHNITT_BEFEHL);
				Aspect asp = dataModel.getAspect(Defs.ASP_TSF_VORGABE);
				DataDescription desc = new DataDescription(atg, asp);
				ResultData rd = new ResultData(getSoAbaschnitt(), desc, System
						.currentTimeMillis(), data);

				LinkedList<ResultData> lRD = new LinkedList<ResultData>();
				lRD.add(rd);
				UrlasserObjekt ulo = new UrlasserObjekt(uli);

				if (ulo.baueDaVVerbindungAuf())
				{
					LinkedList<ResultData> lNichtVersendbareDaten = ulo.sendeDaten(lRD);
					if (lNichtVersendbareDaten.size() == 0)
					{
						_abarbeitungsStart = new Date();
						bearbeitungGestartet = true;
					} else
					{
						System.err
								.println("Automatischer Versand ber alte Urlasserdaten wurde abgebrochen. ");
						this.dispose();
					}
				} else
				{
					System.err
							.println("Es konnte kein Automatischer Versand ber alte Urlasserdaten veranlasst werden, da das UrlasserObjekt keine DaV-Verbindung aufbauen konnte.");
				}
			}

			return bearbeitungGestartet;
		}

		/**
		 * Liefert das {@link SystemObject} des Abschnitts (Typ
		 * <code>typ.tfsAbschnitt</code>), fr welches dieser Merker zustndig ist.
		 * 
		 * @return {@link SystemObject} vom Typ <code>typ.tfsAbschnitt</code>.
		 */
		protected SystemObject getSoAbaschnitt()
		{
			return _soAbschnitt;
		}

		protected E_TSFBefehl getZielProgramm()
		{
			return _zielProgramm;
		}

		/**
		 * Liefert den Abarbeitungs Startzeitpunkt.
		 * 
		 * @return
		 */
		protected Date getAbarbeitungsStart()
		{
			return _abarbeitungsStart;
		}

		/**
		 * Liefert den Abarbeitungs Endzeitpunkt.
		 * 
		 * @return
		 */
		protected Date getAbarbeitungsEnde()
		{
			return _abarbeitungsEnde;
		}

		/**
		 * Prft ob die Abarbeitung erfolgt ist.
		 * 
		 * @return Liefert <code>true</code> wenn die Abarbeitung erfolgt ist, sonst
		 *         <code>false</code>.
		 */
		public boolean isAbgearbeitet()
		{
			boolean abgearbeitet = false;

			if ((getAbarbeitungsStart() != null) && (getAbarbeitungsEnde() != null))
			{
				abgearbeitet = true;
			}

			return abgearbeitet;
		}

		@Override
		public void propertyChange(PropertyChangeEvent evt)
		{
			/*
			 * Wird bei GUIModelTSFSteuerung angemeldet, um aktuelle Informationen zu
			 * erhalten.
			 */

			if (evt != null)
			{

				if (evt.getSource() instanceof GUIModelTSFSteuerung)
				{
					if (evt.getNewValue() != null)
					{
						if (((evt.getPropertyName().equals(GUIModelTSFSteuerung.PROP_INIT)) || (evt
								.getPropertyName().equals(GUIModelTSFSteuerung.PROP_ADD)))
								&& (evt.getNewValue() instanceof ZustandSammlung))
						{
							/*
							 * Bei der gettigten Listener-Anmeldung werden KEINE Arrays als
							 * NewValue geliefert.
							 */

							/*
							 * Prfen ob wir uns im Zustand "Geschaltet" befinden und das
							 * Zielprogramm bereinstimmt.
							 */
							ZustandSammlung zustand = (ZustandSammlung) evt.getNewValue();
							// System.out.println("\t->" + zustand.toString());
							// System.out.println("\t->ZielPrg. Gesucht= " +
							// getZielProgramm());

							if ((zustand.getProgrammZustand() == E_TSFProgrammZustand.Geschaltet)
									&& (zustand.getZielProgramm() == getZielProgramm()))
							{
								_abarbeitungsEnde = new Date();
								dispose();

								synchronized (AbarbeitungsMerker.this)
								{
									AbarbeitungsMerker.this.notify(); // Notify auf den Merker
								}
							}

						} else
						{
							System.out
									.println(this.getClass().getSimpleName()
											+ "::propertyChange => Unbekannte Property/Value-Kombination:");
							StringBuffer sb = new StringBuffer();
							sb.append(this.getClass().getSimpleName());
							sb
									.append("::propertyChange => Unbekannte Property/Value-Kombination:\n");

							sb.append("\tPropertyName=");
							sb.append(evt.getPropertyName());
							sb.append("\n");

							sb.append("\tPropertyValue InstanceOf=");
							sb.append(evt.getNewValue().getClass().getName());
							sb.append("\n");

							System.err.println(sb.toString());
						}
					} else if (evt.getOldValue() != null)
					{
						// Hier ist eigentlich nichts zu tun.
					}
				} else if (evt.getSource() instanceof InfoWeiterschaltenAbgebrochen)
				{

					if ((evt.getPropertyName() == InfoWeiterschaltenAbgebrochen.PROP_SCHALTEN_ABSCHNITT_ABGEBROCHEN)
							&& (evt.getNewValue() != null)
							&& (evt.getNewValue() instanceof AbgebrochenesWeiterschaltenAbschnitt))
					{
						AbgebrochenesWeiterschaltenAbschnitt aAbs = (AbgebrochenesWeiterschaltenAbschnitt) evt
								.getNewValue();

						if (aAbs.getPidAbschnitt() != null)
						{
							/*
							 * Bereich bestimmen, zu dem der Abschnitt gehrt. Gehrt dieser
							 * zu dem hier behandelten Abschnitt, dann ist die Bearbeitung
							 * abzubrechen.
							 */
							if ((getSoAbaschnitt() != null)
									&& (getSoAbaschnitt().getPid().equals(aAbs.getPidAbschnitt())))
							{
								_abarbeitungsEnde = null; // Damit ist isAbgerabeitet false!
								dispose();

								synchronized (AbarbeitungsMerker.this)
								{
									AbarbeitungsMerker.this.notify(); // Notify auf den Merker
								}
							}
						}
					}
				}
			}

		}

		private void dispose()
		{
			/*
			 * Listener Abmelden
			 */
			DaVKomTSFSteuerung.getInstance().getModelZustand().removeListener(
					getSoAbaschnitt(), this);
			InfoWeiterschaltenAbgebrochen.getInstance().removeListener(this);
		}

		@Override
		public String toString()
		{
			StringBuffer sb = new StringBuffer();

			sb.append("Abarbeitungsmerker Abschnitt: ");
			if (getSoAbaschnitt() != null)
			{
				sb.append(getSoAbaschnitt().getNameOrPidOrId());
			} else
			{
				sb.append("-");
			}
			sb.append("\n");

			sb.append("Zielprogramm: ");
			if (getZielProgramm() != null)
			{
				sb.append(getZielProgramm());
			} else
			{
				sb.append("-");
			}
			sb.append("\n");

			sb.append("Start: ");
			if (getAbarbeitungsStart() != null)
			{
				sb.append(Defs.SDF.format(getAbarbeitungsStart()));
			} else
			{
				sb.append("-");
			}
			sb.append(", ");

			sb.append("Ende: ");
			if (getAbarbeitungsEnde() != null)
			{
				sb.append(Defs.SDF.format(getAbarbeitungsEnde()));
			} else
			{
				sb.append("-");
			}
			sb.append("\n");

			sb.append("Abarbeitung erfolgreich beendet: ");
			sb.append(isAbgearbeitet());
			sb.append("\n");

			return sb.toString();
		}

	}

	/**
	 * Prft ob alle {@link AbarbeitungsMerker} in der Liste der noch
	 * abzuarbeitenden Merker bereits komplett abgearbeitet wurden.
	 * 
	 * @return Liefert <code>true</code> wenn die komplette Abarbeitung bereits
	 *         erfolgt ist, sonst <code>false</code>.
	 */
	public boolean istKompletteAbarbeitungErfolgt()
	{
		boolean re = false;

		synchronized (_lNochAbzuarbeiten)
		{
			if (_lNochAbzuarbeiten.size() <= 0)
			{
				re = true;
			}
		}

		return re;
	}

	/**
	 * Prft ob der Job zur Abarbeitung gerade aktiv ist oder nicht.
	 * 
	 * @return Liefert <code>true</code> wenn der Job aktiv ist, sonst
	 *         <code>false</code>.
	 */
	public boolean istJobAktiv()
	{
		boolean re = false;

		if (_jobAbarbeitung != null)
		{
			re = true;
		}

		return re;
	}

	/**
	 * Meldet die {@link PropertyChangeListener} aller {@link AbarbeitungsMerker}
	 * vom {@link GUIModelTSFSteuerung} ab. Des weiteren wird sich von
	 * {@link TSFBereichSchaltenVerwaltung} abgemeldet.
	 */
	public void dispose()
	{
		if (!_disposed)
		{
			AbarbeitungsMerker[] abgearbeiteteMerker = getAbgearbeiteteMerker();
			for (AbarbeitungsMerker abarbeitungsMerker : abgearbeiteteMerker)
			{
				abarbeitungsMerker.dispose();
			}

			AbarbeitungsMerker[] nochAbzuarbeitetendeMerker = getNochAbzuarbeitetendeMerker();
			for (AbarbeitungsMerker abarbeitungsMerker : nochAbzuarbeitetendeMerker)
			{
				abarbeitungsMerker.dispose();
			}

			/*
			 * Diesen abgelaufenen Prozess von der Verwaltung entfernen.
			 */
			TSFBereichSchaltenVerwaltung.getInstance().removeTSFBereichSchalten(this);

			_disposed = true;

			if (_jobAbarbeitung != null)
			{
				_jobAbarbeitung.cancel();
			}

			if (_thAbarbeitung != null)
			{
				_thAbarbeitung.stopThread();
			}

			/*
			 * Abmelden von der Verwaltung.
			 */
			TSFBereichSchaltenVerwaltung.getInstance().removeTSFBereichSchalten(this);

			/*
			 * Listener bei GUIModelTSFSteuerung abmelden
			 */
			DaVKomTSFSteuerung.getInstance().getModelVorgabe()
					.removeListenerAllgemein(this);
		}
	}

	/**
	 * Liefert ein Array der abzuarbeitenden Merker
	 * 
	 * @return
	 */
	private AbarbeitungsMerker[] getAbgearbeiteteMerker()
	{
		AbarbeitungsMerker[] re = null;

		synchronized (_lAbgearbeitet)
		{
			re = _lAbgearbeitet
					.toArray(new AbarbeitungsMerker[_lAbgearbeitet.size()]);
		}

		return re;
	}

	/**
	 * Liefert ein Array der abgearbeitenden Merker.
	 * 
	 * @return
	 */
	private AbarbeitungsMerker[] getNochAbzuarbeitetendeMerker()
	{
		AbarbeitungsMerker[] re = null;

		synchronized (_lNochAbzuarbeiten)
		{
			re = _lNochAbzuarbeiten.toArray(new AbarbeitungsMerker[_lNochAbzuarbeiten
					.size()]);
		}

		return re;
	}

	private class ThreadAbarbeitung extends Thread
	{
		private boolean _laufe = false;

		/**
		 * Aktuell zu bearbeitender Merker.
		 */
		private AbarbeitungsMerker _merker = null;

		public ThreadAbarbeitung()
		{
			startThread();
		}

		private void startThread()
		{
			if (!_laufe)
			{
				this.setName("Abarbeitung TFSSchalten " + new Date());
				_laufe = true;
				this.start();
			}
		}

		protected void stopThread()
		{
			_laufe = false;
			if (_merker != null)
			{
				_merker.notify();
			}

			if (this.isAlive())
			{
				this.interrupt();
			}

			/*
			 * Thread lschen
			 */
			_thAbarbeitung = null;

			/*
			 * Aufrumen...
			 */
			TSFBereichSchalten.this.dispose();
		}

		@Override
		public void run()
		{
			/*
			 * Diesen abgelaufenen Prozess zur Verwaltung hinzufgen.
			 */
			TSFBereichSchaltenVerwaltung.getInstance().putTSFBereichSchalten(
					TSFBereichSchalten.this);

			while (_laufe && (!interrupted()))
			{
				_merker = getNaechstenMerker();
				if (_merker != null)
				{
					boolean bearbeitungGestartet = _merker.starteAbarbeitung();

					if (bearbeitungGestartet)
					{
						synchronized (_merker)
						{
							try
							{
								_merker.wait(); // WAITSYNC auf den Merker
							} catch (InterruptedException e)
							{
								e.printStackTrace();
							}
						}

						/*
						 * Es wurde nicht hart gestoppt und der Merker soll weiter
						 * verarbeitet werden
						 */
						if (_laufe && (!interrupted()))
						{
							if (_merker.isAbgearbeitet())
							{
								abgearbeitetenMerkerVerschieben(_merker);
								// ...weiter mit der dem nchsten abzuarbeitenden Merker.
							} else
							{
								System.err
										.println("Merker wurde NICHT korrekt abgearbeitet! Breche ab...");
								_laufe = false;
							}
						}
					} else
					{
						System.err
								.println("ThreadAbarbeitung: Die Bearbeitung des nachfolgenden Merkers wurde MANUELL abgebrochen:");
						System.err.println(getInBearbeitungBefindlichenMerker());
						_laufe = false;
					}
				} else
				{
					System.out
							.println("Kein weiterer Merker vorhanden, Breche bearbeitung ab.");
					_laufe = false;
				}
			}

			TSFBereichSchalten.this.dispose();
		}

		private void abgearbeitetenMerkerVerschieben(AbarbeitungsMerker merker)
		{
			if (merker != null)
			{
				synchronized (_lNochAbzuarbeiten)
				{
					_lNochAbzuarbeiten.remove(merker);
				}

				synchronized (_lAbgearbeitet)
				{
					_lAbgearbeitet.add(merker);
				}

				System.out.println("Abgearbeiteter Abschnitt: "
						+ merker.getSoAbaschnitt());
			}
		}

		protected AbarbeitungsMerker getInBearbeitungBefindlichenMerker()
		{
			return _merker;
		}
	}

	private class JobAbarbeitung extends Job
	{
		private boolean _laufe = false;

		/**
		 * Aktuell zu bearbeitender Merker.
		 */
		private AbarbeitungsMerker _merker = null;

		public JobAbarbeitung(String name)
		{
			super(name);
			this.setUser(true);
			this.setPriority(Job.INTERACTIVE);
			startThread();
		}

		private void startThread()
		{
			if (!_laufe)
			{
				_laufe = true;
				this.schedule();
			}
		}

		protected void stopThread()
		{
			this.cancel();
		}

		@Override
		protected void canceling()
		{
			super.canceling();
			System.out.println("JOB wird beendet...");

			_laufe = false;
			if (_merker != null)
			{
				synchronized (_merker)
				{
					_merker.notifyAll();
				}
			}

			/*
			 * Thread lschen
			 */
			_jobAbarbeitung = null;

			/*
			 * Aufrumen...
			 */
			TSFBereichSchalten.this.dispose();
		}

		@Override
		protected IStatus run(IProgressMonitor monitor)
		{
			String msg = "TSFBereichSchalten: Schaltsequenz durchfhren fr Bereich "
					+ _tsfBereichAbschnitte.getSo().getNameOrPidOrId();
			monitor.beginTask(msg, getAbschnitte().length);

			logge(msg, Status.INFO);

			/*
			 * Diesen abgelaufenen Prozess zur Verwaltung hinzufgen.
			 */
			TSFBereichSchaltenVerwaltung.getInstance().putTSFBereichSchalten(
					TSFBereichSchalten.this);

			while (_laufe && (!monitor.isCanceled()))
			{
				_merker = getNaechstenMerker();
				if (_merker != null)
				{
					String msgStart = "TSFBereichSchalten: Starte Bearbeitung fr Abschnitt "
							+ _merker.getSoAbaschnitt().getPidOrNameOrId();
					monitor.subTask(msgStart);
					logge(msgStart, Status.INFO);

					boolean bearbeitungGestertet = _merker.starteAbarbeitung();

					if (bearbeitungGestertet)
					{
						synchronized (_merker)
						{
							try
							{
								_merker.wait(); // WAITSYNC auf den Merker
							} catch (InterruptedException e)
							{
								e.printStackTrace();
							}
						}

						String msgMerkerAbgearbeitet = "TSFBereichSchalten: Abarbeitung fr nachfolgenden Abschnitt beendet: "
								+ _merker.getSoAbaschnitt().getPidOrNameOrId();
						logge(msgMerkerAbgearbeitet, Status.INFO);

						/*
						 * Es wurde nicht hart gestoppt und der Merker soll weiter
						 * verarbeitet werden
						 */
						if (_laufe && (!monitor.isCanceled()))
						{
							if (_merker.isAbgearbeitet())
							{
								abgearbeitetenMerkerVerschieben(_merker);
								// ...weiter mit der dem nchsten abzuarbeitenden Merker.
							} else
							{
								String msgMerkerAbbruch = "TSFBereichSchalten: Nachfolgender Abschnitt wurde NICHT korrekt abgearbeitet! Breche ab: "
										+ _merker.getSoAbaschnitt().getNameOrPidOrId();
								System.err.println(msgMerkerAbbruch);

								logge(msgMerkerAbgearbeitet, Status.WARNING);

								_laufe = false;
							}
						} else if ((!_laufe) || (monitor.isCanceled()))
						{
							String msgAbbruchJob = "TSFBereichSchalten: Der JOB zur Bearbeitung der sequentiellen Schaltungen der Abschnitte des Bereichs wurde manuell vom Benutzer abgebrochen.";
							if ((getBereichAbschnitte() != null)
									&& (getBereichAbschnitte().getSo() != null))
							{
								msgAbbruchJob = msgAbbruchJob + "Betroffener Bereich: "
										+ getBereichAbschnitte().getSo().getNameOrPidOrId();
								System.out.println(msgAbbruchJob);
								logge(msgAbbruchJob, Status.WARNING);
								// TODO: Hier wre der richte Pkt., um ein Abbrechen der
								// Schaltung rauszufeuern.
							}
						}
					} else
					{
						String msgManuellWeiterschaltenAbgebrochen = "TSFBereichSchalten: Die Bearbeitung des nachfolgenden Abschnitts wurde MANUELL ber den Weiterschalten-Dialog abgebrochen:\n"
								+ getInBearbeitungBefindlichenMerker();
						System.err.println(msgManuellWeiterschaltenAbgebrochen);
						logge(msgManuellWeiterschaltenAbgebrochen, Status.WARNING);

						_laufe = false;
					}

					monitor.worked(1);
				} else
				{
					String msgMerkerBearbeitungAbbgebrochen = "TSFBereichSchalten: Kein weiterer Bereich mit zu schaltenden Abschnitten vorhanden, die Bearbeitung der Schaltsequenz wurde ordnungsgem beendet.";
					System.out.println(msgMerkerBearbeitungAbbgebrochen);
					logge(msgMerkerBearbeitungAbbgebrochen, Status.OK);
					_laufe = false;
				}
			}

			monitor.done();

			TSFBereichSchalten.this.dispose();

			return Status.OK_STATUS;
		}

		private void abgearbeitetenMerkerVerschieben(AbarbeitungsMerker merker)
		{
			if (merker != null)
			{
				synchronized (_lNochAbzuarbeiten)
				{
					_lNochAbzuarbeiten.remove(merker);
				}

				synchronized (_lAbgearbeitet)
				{
					_lAbgearbeitet.add(merker);
				}

				System.out.println("Abgearbeiteter Abschnitt: "
						+ merker.getSoAbaschnitt());
			}
		}

		protected AbarbeitungsMerker getInBearbeitungBefindlichenMerker()
		{
			return _merker;
		}
	}

	private class ThreadAbarbeitungMO extends Thread
	{
		private boolean _laufe = false;

		/**
		 * Aktuell zu bearbeitender Merker.
		 */
		private AbarbeitungsMerker _merker = null;

		public ThreadAbarbeitungMO()
		{
			super();
			startThread();
		}

		private void startThread()
		{
			if (!_laufe)
			{
				this.setName("Abarbeitung TFSSchalten " + new Date());
				_laufe = true;
				this.start();
			}
		}

		protected void stopThread()
		{
			_laufe = false;
			if (_merker != null)
			{
				synchronized (_merker)
				{
					_merker.notify();
				}
			}

			if (this.isAlive())
			{
				this.interrupt();
			}

			/*
			 * Thread lschen
			 */
			_thAbarbeitung = null;

			/*
			 * Aufrumen...
			 */

			TSFBereichSchalten.this.dispose();
		}

		@Override
		public void run()
		{
			// String msg =
			// "TSFBereichSchalten: Schaltsequenz durchfhren fr Bereich "
			// + _tsfBereichAbschnitte.getSo().getNameOrPidOrId();
			// logge(msg, Status.INFO);

			/*
			 * Diesen abgelaufenen Prozess zur Verwaltung hinzufgen.
			 */
			TSFBereichSchaltenVerwaltung.getInstance().putTSFBereichSchalten(
					TSFBereichSchalten.this);

			while (_laufe)
			{
				_merker = getNaechstenMerker();

				if (_merker != null)
				{
					// String msgStart =
					// "TSFBereichSchalten: Starte Bearbeitung fr Abschnitt "
					// + _merker.getSoAbaschnitt().getPidOrNameOrId();
					// logge(msgStart, Status.INFO);

					/*
					 * Prfen ob der Abschnitt noch geschalten werden muss. Dies ist nicht
					 * der Fall, wenn zwischenzeitlich der entsprechende Zustand
					 * anderweitig schon erreicht wurde.
					 */
					if (_merker.pruefeObArbarbeitungNochGestartetWerdenMuss())
					{
						boolean bearbeitungGestertet = _merker.starteAbarbeitung();

						if (bearbeitungGestertet)
						{
							synchronized (_merker)
							{
								try
								{
									_merker.wait(); // WAITSYNC auf den Merker
								} catch (InterruptedException e)
								{
									e.printStackTrace();
								}
							}

							// String msgMerkerAbgearbeitet =
							// "TSFBereichSchalten: Abarbeitung fr nachfolgenden Abschnitt beendet: "
							// + _merker.getSoAbaschnitt().getPidOrNameOrId();
							// logge(msgMerkerAbgearbeitet, Status.INFO);

							/*
							 * Es wurde nicht hart gestoppt und der Merker soll weiter
							 * verarbeitet werden
							 */
							if (_laufe)
							{
								if (_merker.isAbgearbeitet())
								{
									abgearbeitetenMerkerVerschieben(_merker);
									// ...weiter mit der dem nchsten abzuarbeitenden Merker.
								} else
								{
									// String msgMerkerAbbruch =
									// "TSFBereichSchalten: Nachfolgender Abschnitt wurde NICHT korrekt abgearbeitet! Breche ab: "
									// + _merker.getSoAbaschnitt().getNameOrPidOrId();
									// logge(msgMerkerAbgearbeitet, Status.WARNING);

									_laufe = false;
								}
							} else if (!_laufe)
							{
								String msgAbbruchJob = "TSFBereichSchalten: Der JOB zur Bearbeitung der sequentiellen Schaltungen der Abschnitte des Bereichs wurde manuell vom Benutzer abgebrochen.";
								if ((getBereichAbschnitte() != null)
										&& (getBereichAbschnitte().getSo() != null))
								{
									// msgAbbruchJob = msgAbbruchJob + "Betroffener Bereich: "
									// + getBereichAbschnitte().getSo().getNameOrPidOrId();
									// logge(msgAbbruchJob, Status.WARNING);
									// TODO: Hier wre der richte Pkt., um ein Abbrechen der
									// Schaltung rauszufeuern.
								}
							}
						} else
						{
							// String msgManuellWeiterschaltenAbgebrochen =
							// "TSFBereichSchalten: Die Bearbeitung des nachfolgenden Abschnitts wurde MANUELL ber den Weiterschalten-Dialog abgebrochen:\n"
							// + getInBearbeitungBefindlichenMerker();
							// logge(msgManuellWeiterschaltenAbgebrochen, Status.WARNING);
							_laufe = false;
						}
					} else
					{
						System.out.println("Merker muss nicht mehr bearbeitet werden: Abs="
								+ _merker.getSoAbaschnitt().getPid() + " ZielPrg="
								+ _merker.getZielProgramm());
						/*
						 * Merker muss nicht mehr verarbeitet werden. Dieser muss wird in
						 * die Liste der abgearbeiteten Merker verschoben und dessen
						 * dispose-Methode aufgerufen.
						 */
						abgearbeitetenMerkerVerschieben(_merker);
						_merker.dispose();
					}
				} else
				{
					// String msgMerkerBearbeitungAbbgebrochen =
					// "TSFBereichSchalten: Kein weiterer Bereich mit zu schaltenden Abschnitten vorhanden, die Bearbeitung der Schaltsequenz wurde ordnungsgem beendet.";
					// logge(msgMerkerBearbeitungAbbgebrochen, Status.OK);
					_laufe = false;
				}
			}

			TSFBereichSchalten.this.dispose();
		}

		protected AbarbeitungsMerker getInBearbeitungBefindlichenMerker()
		{
			return _merker;
		}
	}

	protected void alleMerkerParallelAbarbeiten()
	{
		AbarbeitungsMerker merker = getNaechstenMerker();

		while (merker != null)
		{
			if (merker != null)
			{
				merker.starteAbarbeitung();
				abgearbeitetenMerkerVerschieben(merker);
			}
			merker = getNaechstenMerker();
		}

		/*
		 * Bereich aus der Struktur nehmen in der hinterlegt ist, dass der aktuelle
		 * Bereich gerade abgearbeitet wird.
		 */
		dispose();
	}

	/**
	 * Verschiebt einen abgearbeiteten Merker von der Liste der noch
	 * abzuarbeitenden Merker in die Liste der abgearbeiteten Merker.
	 * 
	 * @param merker
	 *          Der zu verschiebende Merker.
	 */
	private void abgearbeitetenMerkerVerschieben(AbarbeitungsMerker merker)
	{
		if (merker != null)
		{
			synchronized (_lNochAbzuarbeiten)
			{
				_lNochAbzuarbeiten.remove(merker);
			}

			synchronized (_lAbgearbeitet)
			{
				_lAbgearbeitet.add(merker);
			}
		}
	}

	/**
	 * Liefert den nchsten {@link AbarbeitungsMerker}, der bearbeitet werden
	 * soll. Da wir gegen die Fahrtrichtung schalten mssen, wird jeweils das
	 * letzte Element von der Liste der noch nicht abgearbeiteten Merker
	 * geliefert.<br>
	 * <br>
	 * Beim Schalten (Freigabe, Schnellfreigabe) wird gegen die Fahrtrichtung
	 * gearbeitet, beim Sperren (Sperren, Schnellsperren) in Fahrtrichtung.
	 * 
	 * @return Nchster {@link AbarbeitungsMerker} oder <code>null</code>, wenn
	 *         die List der noch abzuarbeitenden {@link AbarbeitungsMerker} leer
	 *         ist.
	 */
	private AbarbeitungsMerker getNaechstenMerker()
	{
		AbarbeitungsMerker next = null;

		boolean richtungVorwaerts = true;

		if (_befehl != null)
		{
			if ((_befehl == E_TSFBefehl.Freigabe)
					|| (_befehl == E_TSFBefehl.FreigabeArgoscan)
					|| (_befehl == E_TSFBefehl.Schnellfreigabe))
			{
				richtungVorwaerts = false;
			}
		}

		synchronized (_lNochAbzuarbeiten)
		{
			if (_lNochAbzuarbeiten.size() > 0)
			{
				if (richtungVorwaerts)
				{
					next = _lNochAbzuarbeiten.getFirst();
				} else
				{
					next = _lNochAbzuarbeiten.getLast();
				}
			}
		}

		return next;
	}

	/**
	 * Prft ob in den zum Bereich zugehrigen Abschnitten ein Abschnitt mit einer
	 * bestimmten PID vorhanden ist.
	 * 
	 * @param pid
	 *          PID des Abschnitts.
	 * @return Liefert <code>true</code> wenn die PID vorhanden ist, sonst
	 *         <code>false</code>.
	 */
	public boolean istAbschnittVorhanden(String pid)
	{
		boolean re = false;

		SystemObject[] abschnitte = getAbschnitte();
		for (SystemObject soAbs : abschnitte)
		{
			if (soAbs.getPid().equals(pid))
			{
				re = true;
				break;
			}
		}

		return re;
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt)
	{
		if ((evt != null) && (evt.getSource() != null)
				&& (evt.getSource() instanceof GUIModelTSFSteuerung))
		{
			/*
			 * Detektion von entfernten Sammlungen in GUIModelTSFSteuerung.
			 */
			if ((evt.getPropertyName() == GUIModelTSFSteuerung.PROP_SAMMLUNG_REMOVED)
					&& (evt.getOldValue() != null)
					&& (evt.getOldValue() instanceof ZustandSammlung))
			{
				ZustandSammlung removedZustandSteuerungSammlung = (ZustandSammlung) evt
						.getOldValue();

				/*
				 * Vergleich mit den hiesigen Abschnitten.
				 */
				SystemObject entfernterAbschnittGefunden = null;

				SystemObject removedSoAbs = removedZustandSteuerungSammlung
						.getSoTfsAbschnitt();
				SystemObject[] abschnitte = getAbschnitte();
				for (SystemObject soAbs : abschnitte)
				{
					if (soAbs == removedSoAbs)
					{
						entfernterAbschnittGefunden = soAbs;
						break;
					}
				}

				/*
				 * Entfernter TSF-Abschnitt der zu diesem Bereich gehrt wurde gefunden.
				 * Der Thread ist zu stoppen und die Bearbeitung zu beenden. Die
				 * Dispose-Methode erledigt diese Aufgaben.
				 */
				if (entfernterAbschnittGefunden != null)
				{
					// System.err
					// .println("=> Abschnitt "
					// + entfernterAbschnittGefunden
					// +
					// " wurde entfernt. Breche Thread zum Schalten eines Bereiches ab.");
					dispose();
				}
			}
		}
	}
}
