/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.LinkedList;
import java.util.Timer;
import java.util.TimerTask;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IPerspectiveListener;
import org.eclipse.ui.ISaveablePart2;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.ViewPane;
import org.eclipse.ui.internal.Workbench;
import org.eclipse.ui.internal.WorkbenchPage;
import org.eclipse.ui.internal.WorkbenchPartReference;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.Activator;
import de.sts.buv.plugin.workflow.tsf.DefsFonts;
import de.sts.buv.plugin.workflow.tsf.ImageGetter;
import de.sts.buv.plugin.workflow.tsf.PreferenceInit;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFAbschnittBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFSteuerung;
import de.sts.buv.plugin.workflow.tsf.dav.Defs;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFProgrammZustand;
import de.sts.buv.plugin.workflow.tsf.dav.GUIManager_WorkflowDialog;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFAbschnittBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFWorkflowAbschnitt;
import de.sts.buv.plugin.workflow.tsf.dav.WiedervorlageTSFWorkflowAbschnitt;
import de.sts.buv.plugin.workflow.tsf.dav.WrapperULDInfo;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung.ZustandSammlung;
import de.sts.buv.plugin.workflow.tsf.snd.AePlayWave;

/**
 * Vereinfachter Workflow zum Bearbeiten eines {@link M_TSFWorkflowAbschnitt}es. Je nach dem wieviele Abschnitte ein
 * Workflow hat, kann dieser Mehrmals auftauchen.
 * 
 * @author STS
 * 
 */
public class ViewWFEinfach extends ViewPart implements ISaveablePart2, IPerspectiveListener
{
  public static final String ID = "de.sts.buv.plugin.workflow.tsf.dlg.ViewWFEinfach";

  private static final String HELP_ID = Activator.PLUGIN_ID + ".dialogworkflowabfrage";

  private static final int BREITE = 650 + 25;

  private static final int HOEHE = 400;

  private Timer _sndTimer;

  private Label _lAbschnitt;

  private Text _tZeitstempel;

  private Text _tAktion;

  private Text _tUrsache;

  /**
   * Daten des Workflow-Abschnittes
   */
  private M_TSFWorkflowAbschnitt _tsfWorkflowAbschnitt = null;

  private Button _buttonTonAusschalten;

  private Button _bSchalten;

  // private Button _bSchaltenArgoscan;

  // private static final boolean BUTTON_ARGOSCAN_PASSIVIERT = true;

  private GUIManager_WorkflowDialog _guiManagerWorkflowDialog = null;

  private Button _buttonAblehnen;

  /**
   * Basis (TOP) Composite
   */
  private Composite _baseComp;

  /**
   * Flag das Anzeigt, dass diese View geschlossen werden darf.
   */
  private boolean _schliessenErlaubt = false;

  private Label labelProgrammVorschlag;

  private Shell shell;

  @Override
  public void createPartControl(Composite parent)
  {
    _baseComp = new Composite(parent, SWT.NONE);

    shell = this.getViewSite().getShell();

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_baseComp, HELP_ID);

    _baseComp.setLayout(new FillLayout());

    setPartName("Empfehlung");
    createCompositeImage(_baseComp);
    startSoundTimer();

    PlatformUI.getWorkbench().getActiveWorkbenchWindow().addPerspectiveListener(this);

    closeDetachedShells();

  }

  @Override
  public void setFocus()
  {
    if (_baseComp != null)
      _baseComp.setFocus();
  }

  public void setData(M_TSFWorkflowAbschnitt tsfWorkflowAbschnitt)
  {
    this._tsfWorkflowAbschnitt = tsfWorkflowAbschnitt;

    // Abschnitt
    if (_tsfWorkflowAbschnitt != null)
    {
      if (_tsfWorkflowAbschnitt.getAbschnitt() != null)
      {
        String title = _tsfWorkflowAbschnitt.getAbschnitt().getNameOrPidOrId();
        _lAbschnitt.setText(title);
        _lAbschnitt.getParent().layout(true); // TODO Prfen ob aktiv
        setPartName(title);

        String v = null;

        if (_tsfWorkflowAbschnitt.getAktion().equals(E_TSFBefehl.Schnellsperrung))
        {
          v = "\n" + "Aufgrund einer Strung der Auenanlage\n" + "werden vom Steuermodell\n"
              + "folgende Schaltungen empfohlen:\n\n";

        }
        else
        {
          v = "\n" + "Aufgrund genderter Indikatoren\n" + "werden vom Steuermodell\n"
              + "folgende Schaltungen empfohlen:\n\n";
        }
        labelProgrammVorschlag.setText(v);

        // Schalten-Taste abhngig vom Befehl mit unterschiedlichen Texten beschriften

        if (_tsfWorkflowAbschnitt.getAktion().equals(E_TSFBefehl.Freigabe))
        {
          _bSchalten.setText("&Scan starten");
        }
        else
        {
          _bSchalten.setText("&Schalten");
        }
      }
    }

    // Zeitstempel der Daten
    if (_tsfWorkflowAbschnitt.getTimeStamp() != null)
    {
      _tZeitstempel.setText(Defs.SDF.format(_tsfWorkflowAbschnitt.getTimeStamp()));
    }
    else
    {
      _tZeitstempel.setText("-");
    }

    // Aktion
    if (_tsfWorkflowAbschnitt.getAktion() != null)
    {
      E_TSFBefehl aktion = _tsfWorkflowAbschnitt.getAktion();
      _tAktion.setText(aktion.toString());

      if ((aktion == E_TSFBefehl.Freigabe) || (aktion == E_TSFBefehl.FreigabeArgoscan))
      {
        // if(!BUTTON_ARGOSCAN_PASSIVIERT){
        // _bSchaltenArgoscan.setEnabled(true);
        // }
      }
      else
      {
        // _bSchaltenArgoscan.setEnabled(false);
      }

    }
    else
    {
      _tAktion.setText("-");
    }

    // Ursache
    if (_tsfWorkflowAbschnitt.getUrsache() != null)
    {
      _tUrsache.setText(_tsfWorkflowAbschnitt.getUrsache().toString());
    }
    else
    {
      _tUrsache.setText("-");
    }

    // // Listener auf die Steuerung
    // if (_listenerSteuerung == null)
    // {
    // _listenerSteuerung = new PropertyChangeListenerSteuerung(_bSchalten,
    // _tsfWorkflowAbschnitt);
    // }
  }

  public M_TSFWorkflowAbschnitt getData()
  {
    return _tsfWorkflowAbschnitt;
  }

  /**
   * This method initializes compositeImage
   * 
   */
  private void createCompositeImage(Composite parent)
  {
    Composite top = new Composite(parent, SWT.NONE);
    top.setLayout(new GridLayout(1, true));

    // Zeile 0 - Fahrrichtung darstellen
    GridData gdZeile0 = new GridData();
    gdZeile0.grabExcessHorizontalSpace = true;
    gdZeile0.horizontalAlignment = GridData.FILL;
    Composite zeile0 = new Composite(top, SWT.BORDER);
    zeile0.setLayout(new GridLayout(1, false));
    zeile0.setLayoutData(gdZeile0);

    // Zeile 1

    GridData gdZeile1 = new GridData();
    gdZeile1.horizontalAlignment = GridData.FILL;
    gdZeile1.verticalAlignment = GridData.FILL;
    gdZeile1.grabExcessHorizontalSpace = true;
    gdZeile1.grabExcessVerticalSpace = true;

    Composite zeile1 = new Composite(top, SWT.BORDER);
    zeile1.setLayout(new GridLayout(1, false));
    zeile1.setLayoutData(gdZeile1);

    // Zeile 2

    GridData gdZeile2 = new GridData();
    gdZeile2.horizontalAlignment = GridData.FILL;
    gdZeile2.grabExcessHorizontalSpace = true;

    Composite zeile2 = new Composite(top, SWT.BORDER);
    zeile2.setLayout(new GridLayout(6, false));
    zeile2.setLayoutData(gdZeile2);

    addFahrtrichtung(zeile0);
    addProgrammVorschlagHeadline(zeile1);
    addProgrammVorschlagInhalt(zeile1);
    addProgrammButtons(zeile2);
  }

  private void addFahrtrichtung(Composite parent)
  {

    _lAbschnitt = new Label(parent, SWT.None);
    // Font f1 = new Font(null, "Arial", 12, SWT.BOLD);
    _lAbschnitt.setFont(DefsFonts.FONT_ARIAL_12_PKT_BOLD);
    _lAbschnitt.setText("Abschnitt: __________________________________________________");

    GridData gdLAbschnitt = new GridData();
    gdLAbschnitt.grabExcessHorizontalSpace = true;
    gdLAbschnitt.horizontalAlignment = GridData.CENTER;
    _lAbschnitt.setLayoutData(gdLAbschnitt);
  }

  /**
   * Methode realisiert das Composite mit dem Programmvorschlag
   * 
   * @param parent
   */
  private void addProgrammVorschlagHeadline(Composite parent)
  {
    Composite neu = new Composite(parent, SWT.NONE);
    neu.setLayout(new GridLayout(3, false));
    GridData gdInfoText = new GridData();
    gdInfoText.horizontalAlignment = GridData.FILL;
    gdInfoText.grabExcessHorizontalSpace = true;
    neu.setLayoutData(gdInfoText);

    // Achtung Symbol links

    Label achtungLinks = new Label(neu, SWT.NONE);
    achtungLinks.setImage(ImageGetter.getImage("icons/achtung.png"));

    GridData gridDataAchtungLinks = new GridData();
    gridDataAchtungLinks.horizontalAlignment = GridData.BEGINNING;
    achtungLinks.setLayoutData(gridDataAchtungLinks);

    // Textfeld
    GridData gridDataText = new GridData();
    gridDataText.grabExcessHorizontalSpace = true;
    gridDataText.horizontalAlignment = GridData.CENTER;
    gridDataText.verticalAlignment = GridData.BEGINNING;

    labelProgrammVorschlag = new Label(neu, SWT.NONE);
    labelProgrammVorschlag.setLayoutData(gridDataText);

    // Font f1 = new Font(null, "Arial", 16, SWT.BOLD);
    labelProgrammVorschlag.setFont(DefsFonts.FONT_ARIAL_16_PKT_BOLD);

    String v = "\n" + "Aufgrund genderter Indikatoren\n" + "werden vom Steuermodell\n"
        + "folgende Schaltungen empfohlen:\n\n";

    labelProgrammVorschlag.setText(v);
    labelProgrammVorschlag.setAlignment(SWT.CENTER);

    // Achtung Symbol rechts

    Label achtungRechts = new Label(neu, SWT.NONE);
    achtungRechts.setText("Achtung");
    achtungRechts.setImage(ImageGetter.getImage("icons/achtung.png"));

    GridData gridDataAchtungRechts = new GridData();
    gridDataAchtungRechts.horizontalAlignment = GridData.BEGINNING;
    achtungRechts.setLayoutData(gridDataAchtungLinks);
  }

  private void addProgrammVorschlagInhalt(Composite parent)
  {
    Composite neu = new Composite(parent, SWT.BORDER);

    Label lZeitstempel = new Label(neu, SWT.NONE);
    lZeitstempel.setText("Zeitstempel: ");

    _tZeitstempel = new Text(neu, SWT.BORDER);
    _tZeitstempel.setEditable(false);

    Label lAktion = new Label(neu, SWT.NONE);
    lAktion.setText("Aktion: ");

    _tAktion = new Text(neu, SWT.BORDER);
    _tAktion.setEditable(false);

    Label lUrsache = new Label(neu, SWT.NONE);
    lUrsache.setText("Ursache: ");

    _tUrsache = new Text(neu, SWT.BORDER);
    _tUrsache.setEditable(false);

    /*
     * Layout
     */
    neu.setLayout(new GridLayout(2, false));

    GridData gdCompNeu = new GridData();
    gdCompNeu.grabExcessHorizontalSpace = true;
    gdCompNeu.grabExcessVerticalSpace = true;
    gdCompNeu.horizontalAlignment = GridData.FILL;
    gdCompNeu.verticalAlignment = GridData.FILL;
    neu.setLayoutData(gdCompNeu);

    GridData gdTZeitstempel = new GridData();
    gdTZeitstempel.grabExcessHorizontalSpace = true;
    gdTZeitstempel.horizontalAlignment = GridData.FILL;
    _tZeitstempel.setLayoutData(gdTZeitstempel);

    GridData gdTAktion = new GridData();
    gdTAktion.grabExcessHorizontalSpace = true;
    gdTAktion.horizontalAlignment = GridData.FILL;
    _tAktion.setLayoutData(gdTAktion);

    GridData gdTUrsasche = new GridData();
    gdTUrsasche.grabExcessHorizontalSpace = true;
    gdTUrsasche.horizontalAlignment = GridData.FILL;
    _tUrsache.setLayoutData(gdTUrsasche);
  }

  private void addProgrammButtons(Composite parent)
  {
    GridData gridDataButton = new GridData();
    gridDataButton.horizontalAlignment = GridData.CENTER;
    gridDataButton.grabExcessHorizontalSpace = true;
    gridDataButton.grabExcessVerticalSpace = true;
    gridDataButton.verticalAlignment = GridData.CENTER;
    gridDataButton.minimumWidth = 100;

    _bSchalten = new Button(parent, SWT.PUSH);
    _bSchalten.setText("&Scan starten");
    _bSchalten.setLayoutData(gridDataButton);
    _bSchalten.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionSchalten();
      }
    });

    // _bSchaltenArgoscan = new Button(parent, SWT.PUSH);
    // _bSchaltenArgoscan.setText("Schalten &Argoscan");
    // _bSchaltenArgoscan.setLayoutData(gridDataButton);
    // _bSchaltenArgoscan.addSelectionListener(new SelectionAdapter()
    // {
    // @Override
    // public void widgetSelected(SelectionEvent e)
    // {
    // aktionFreigabeSchaltenArgoscan();
    // }
    // });
    // _bSchaltenArgoscan.setEnabled(false);

    // Wiedervorlage
    Button buttonWiedervorlage = new Button(parent, SWT.PUSH);
    buttonWiedervorlage.setText("&Wiedervorlage");
    buttonWiedervorlage.setLayoutData(gridDataButton);
    buttonWiedervorlage.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionWiedervorlage(null);
      }
    });

    // Schalten

    _buttonAblehnen = new Button(parent, SWT.CENTER);
    _buttonAblehnen.setText("&Ablehnen");
    _buttonAblehnen.setLayoutData(gridDataButton);

    _buttonAblehnen.addSelectionListener(new SelectionListener() {
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      public void widgetSelected(SelectionEvent e)
      {
        aktionAblehnen();
      }
    });

    // Hilfe
    Button bHilfe = new Button(parent, SWT.PUSH);
    bHilfe.setText("&Hilfe");
    bHilfe.setLayoutData(gridDataButton);
    bHilfe.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionHilfe();
      }

    });

    // Ton Ausschalten
    _buttonTonAusschalten = new Button(parent, SWT.CENTER);
    _buttonTonAusschalten.setText("&Ton ausschalten");
    _buttonTonAusschalten.setLayoutData(gridDataButton);
    // buttonTonAusschalten.setEnabled(false);
    _buttonTonAusschalten.addSelectionListener(new SelectionListener() {
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      public void widgetSelected(SelectionEvent e)
      {
        if (_sndTimer != null)
        {
          schalteTonAus();
          _buttonTonAusschalten.setText("&Ton einschalten");
        }
        else
        {
          startSoundTimer();
          if (_sndTimer != null)
          {
            _buttonTonAusschalten.setText("&Ton ausschalten");
          }
        }
      }
    });
    _buttonTonAusschalten.setEnabled(PreferenceInit.isSndWfAktiv());

    parent.pack();
  }

  /**
   * Ordnet einen Butten mit der Pid eines Abschnitts einen {@link PropertyChangeListener} zu.
   * 
   * @author HofmannJ
   * 
   */
  private class PropertyChangeListenerSteuerung implements PropertyChangeListener
  {
    private Button _button = null;

    private M_TSFWorkflowAbschnitt _tsfWorkflowAbschnitt;

    private PropertyChangeListenerSteuerung(Button button, M_TSFWorkflowAbschnitt tsfWorkflowAbschnitt)
    {
      this._button = button;
      this._tsfWorkflowAbschnitt = tsfWorkflowAbschnitt;
      listenerAnmelden();
    }

    public SystemObject getSoTfsAbschnitt()
    {
      SystemObject soTfsAbschnitt = null;

      if (getTsfWorkflowAbschnitt() != null)
      {
        soTfsAbschnitt = getTsfWorkflowAbschnitt().getAbschnitt();
      }

      return soTfsAbschnitt;
    }

    public M_TSFWorkflowAbschnitt getTsfWorkflowAbschnitt()
    {
      return _tsfWorkflowAbschnitt;
    }

    public Button getButton()
    {
      return _button;
    }

    /**
     * Liefert das lt. Workflow geforderte Zielprogramm
     * 
     * @return
     */
    public E_TSFBefehl getZielProgrammWf()
    {
      E_TSFBefehl zielProgrammWf = null;

      if (getTsfWorkflowAbschnitt() != null)
      {
        M_TSFWorkflowAbschnitt tsfWorkflowAbschnitt = getTsfWorkflowAbschnitt();
        zielProgrammWf = tsfWorkflowAbschnitt.getAktion();

      }

      return zielProgrammWf;
    }

    private void listenerAnmelden()
    {
      if (getSoTfsAbschnitt() != null)
      {
        DaVKomTSFSteuerung.getInstance().getModelVorgabe().addListener(getSoTfsAbschnitt(), this);
      }
    }

    private void listenerAbmelden()
    {
      if (getSoTfsAbschnitt() != null)
      {
        DaVKomTSFSteuerung.getInstance().getModelVorgabe().removeListener(getSoTfsAbschnitt(), this);
      }
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt)
    {
      if ((evt != null) && (evt.getNewValue() instanceof ZustandSammlung))
      {
        E_TSFBefehl zielProgrammWf = getZielProgrammWf();

        if (zielProgrammWf != null)
        {
          ZustandSammlung sammlung = (ZustandSammlung)evt.getNewValue();

          if (sammlung.getSoTfsAbschnitt() == getSoTfsAbschnitt())
          {
            /*
             * ZielProgramm lt. Steuerung muss dem geforderten Zielprogramm des Workflows entsprechen.
             */
            boolean istZielPrgGeschaltet = false;
            boolean istSchaltvorgangImGange = false;
            if ((sammlung.getZielProgramm() != null) && (sammlung.getZielProgramm() == zielProgrammWf))
            {
              if ((sammlung.getProgrammZustand() != null)
                  && (sammlung.getProgrammZustand() == E_TSFProgrammZustand.Geschaltet))
              {
                istZielPrgGeschaltet = true;
              }

              if (sammlung.getProgrammZustand() == E_TSFProgrammZustand.In_Bearbeitung)
              {
                istSchaltvorgangImGange = true;
              }
            }

            boolean buttonAktivieren = ((!istZielPrgGeschaltet) && (!istSchaltvorgangImGange));

            if (getButton() != null)
            {
              final boolean fButtonAktivieren = buttonAktivieren;

              Runnable run = new Runnable() {
                @Override
                public void run()
                {
                  /*
                   * Button aktivieren/deaktivieren nur wenn ntig.
                   */
                  if (getButton().isEnabled() != fButtonAktivieren)
                  {
                    getButton().setEnabled(fButtonAktivieren);
                    System.out.println(">>> Setze Button auf " + fButtonAktivieren + " fr Abs " + getSoTfsAbschnitt());
                  }

                }
              };
              PlatformUI.getWorkbench().getDisplay().asyncExec(run);

            }
          }
        }
      }
    }

    public void dispose()
    {
      listenerAbmelden();
    }
  }

  private void aktionSchalten()
  {
    if (getData() != null)
    {
      M_TSFWorkflowAbschnitt workflowAbschnitt = getData();
      E_TSFBefehl aktion = workflowAbschnitt.getAktion();
      M_TSFAbschnittBefehl befehl = new M_TSFAbschnittBefehl(Defs.getClientId(), workflowAbschnitt.getAbschnitt(),
          aktion, E_TSFProgrammZustand.Undefiniert);
      WrapperULDInfo sendeAbschnittBefehl = DaVKomTSFAbschnittBefehl.getInstance().sendeAbschnittBefehl(befehl);
      // if (sendeAbschnittBefehl != null)
      // {
      // /*
      // * Nur Schlieen wenn erfolgreich versandt
      // */
      // if ((sendeAbschnittBefehl.getFehlerInfo() == null)
      // && (sendeAbschnittBefehl.getUldInfo().getEingabeOk()))
      // {
      // _bSchalten.setEnabled(false);
      // dispose();
      // }
      // }

      if ((sendeAbschnittBefehl != null) && (sendeAbschnittBefehl.getUldInfo() != null)
          && (sendeAbschnittBefehl.getUldInfo().getEingabeOk()))
      {
        _bSchalten.setEnabled(false);
        // _bSchaltenArgoscan.setEnabled(false);
        // dispose();
        schliesseViewMitErlaubnis();
      }
    }
  }

  /**
   * Freigabe mit Argoscan schalten.
   */
  private void aktionFreigabeSchaltenArgoscan()
  {
    if (getData() != null)
    {
      M_TSFWorkflowAbschnitt workflowAbschnitt = getData();
      E_TSFBefehl aktion = E_TSFBefehl.FreigabeArgoscan;
      M_TSFAbschnittBefehl befehl = new M_TSFAbschnittBefehl(Defs.getClientId(), workflowAbschnitt.getAbschnitt(),
          aktion, E_TSFProgrammZustand.Undefiniert);
      WrapperULDInfo sendeAbschnittBefehl = DaVKomTSFAbschnittBefehl.getInstance().sendeAbschnittBefehl(befehl);

      if ((sendeAbschnittBefehl != null) && (sendeAbschnittBefehl.getUldInfo() != null)
          && (sendeAbschnittBefehl.getUldInfo().getEingabeOk()))
      {
        _bSchalten.setEnabled(false);
        // _bSchaltenArgoscan.setEnabled(false);
        // dispose();
        schliesseViewMitErlaubnis();
      }
    }
  }

  /**
   * Als Aktion zum Ablehnen soll nach Absprache mit Rolf Asal die Schnellfreigabe gesendet werden.
   */
  private void aktionAblehnen()
  {
    if (getData() != null)
    {
      M_TSFWorkflowAbschnitt workflowAbschnitt = getData();
      M_TSFAbschnittBefehl befehl = new M_TSFAbschnittBefehl(Defs.getClientId(), workflowAbschnitt.getAbschnitt(),
          E_TSFBefehl.Schnellfreigabe, E_TSFProgrammZustand.Undefiniert);
      WrapperULDInfo sendeAbschnittBefehl = DaVKomTSFAbschnittBefehl.getInstance().sendeAbschnittBefehl(befehl);

      System.out.println("Ablehnen: " + befehl.toString());

      // /*
      // * Nur Schlieen wenn erfolgreich versandt.
      // */
      // if (sendeAbschnittBefehl != null)
      // {
      // if ((sendeAbschnittBefehl.getFehlerInfo() == null)
      // && (sendeAbschnittBefehl.getUldInfo().getEingabeOk()))
      // {
      // _bSchalten.setEnabled(false);
      // _buttonAblehnen.setEnabled(false);
      // // dispose();
      // }
      // }

      if ((sendeAbschnittBefehl != null) && (sendeAbschnittBefehl.getUldInfo() != null)
          && (sendeAbschnittBefehl.getUldInfo().getEingabeOk()))
      {
        _bSchalten.setEnabled(false);
        _buttonAblehnen.setEnabled(false);
        setSchliessenErlaubt(true);
        // dispose();
        schliesseViewMitErlaubnis();
      }
    }
  }

  /**
   * 
   * @param alternateDelayWiedervorlage
   *          Alternative Wiedervorlage in Sekunden. Wenn nicht gesetzt wird die aus den Prferenzen verwendet.
   */
  public void aktionWiedervorlage(Long alternateDelayWiedervorlage)
  {
    if (getData() != null)
    {
      new WiedervorlageTSFWorkflowAbschnitt(getData(), alternateDelayWiedervorlage);
      // dispose();
      schliesseViewMitErlaubnis();
    }
  }

  /**
   * Schliet die View, das Flag SchlieenErlaubt wird auf <code>true</code> gesetzt, damit die View geschlossen wird.
   */
  public void schliesseViewMitErlaubnis()
  {
    String secondaryId = this.getViewSite().getSecondaryId();
    if (secondaryId != null)
    {
      setSchliessenErlaubt(true);
      ViewWFEinfach.schliesseViewWFEinfach(secondaryId);
    }
  }

  public void dispose()
  {
    super.dispose();
    // // /*
    // // * Listener auf die Steuerung abmelden.
    // // */
    // // if (_listenerSteuerung != null)
    // // {
    // // _listenerSteuerung.dispose();
    // // _listenerSteuerung = null;
    // // }

    /*
     * Sound aus
     */
    schalteTonAus();

    PlatformUI.getWorkbench().getActiveWorkbenchWindow().removePerspectiveListener(this);
  }

  private Timer startSoundTimer()
  {
    if (PreferenceInit.isSndWfAktiv())
    {
      if (_sndTimer == null)
      {
        int delaySndWf = PreferenceInit.getDelaySndWf();
        final String pathSndWf = PreferenceInit.getPathSndWf();

        TimerTask sndTask = new TimerTask() {
          @Override
          public void run()
          {
            new AePlayWave(pathSndWf).start();
          }
        };

        _sndTimer = new Timer("WF Timer");
        _sndTimer.schedule(sndTask, 0l, delaySndWf * 1000l);
      }
    }

    return _sndTimer;
  }

  private void schalteTonAus()
  {
    if (_sndTimer != null)
    {
      _sndTimer.cancel();
      _sndTimer = null;
    }
  }

  private void aktionHilfe()
  {
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(HELP_ID);
  }

  /**
   * Zeigt eine View fr einen WF an.
   * 
   * @param pidAbschnitt
   *          PID des jeweiligen TSF-Abschnitts.
   * @return Liefert die entsprechende {@link ViewWFEinfach} oder <code>null</code>, falls diese nicht erzeugt werden
   *         konnte.
   */
  public static ViewWFEinfach zeigeViewWFEinfach(String pidAbschnitt)
  {
    ViewWFEinfach re = null;

    if (pidAbschnitt != null)
    {
      IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
      IWorkbenchPage page = dw.getActivePage();

//      System.out.println("ViewWFEinfach.zeigeViewWFEinfach() PAGE not null? " + (page != null));

      IViewReference ref = page.findViewReference(ViewWFEinfach.ID, pidAbschnitt);

      if (ref == null)
      {
        /*
         * View erstellen und anzeigen.
         */
        try
        {
          // page.showView(ViewWFEinfach.ID, pidAbschnitt,
          // IWorkbenchPage.VIEW_CREATE);
//          System.out.println("PAGE=>" + page + " VIEWID: " + ViewWFEinfach.ID + " PID Abs.: " + pidAbschnitt);
          page.showView(ViewWFEinfach.ID, pidAbschnitt, IWorkbenchPage.VIEW_ACTIVATE);
          ref = page.findViewReference(ViewWFEinfach.ID, pidAbschnitt);
        }
        catch (PartInitException e)
        {
          e.printStackTrace();
        }

        if (ref != null)
        {
          ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(ref);
          IWorkbenchPart wbPart = ref.getPart(false);

          IViewPart viewPart = ref.getView(false);
          if (viewPart instanceof ViewWFEinfach)
          {
            ViewWFEinfach viewWFEinfach = (ViewWFEinfach)viewPart;
            re = viewWFEinfach;
          }
//          else
//          {
//            System.out.println("ViewWFEinfach.zeigeViewWFEinfach(): instance of1:  " + viewPart);
//          }

          Shell shell = wbPart.getSite().getShell();
          shell.setSize(BREITE, HOEHE);

          /*
           * Statische Anordnung des Dialoges auf dem Bildschirm
           */
          if (pidAbschnitt != null)
          {
            Point location = StaticDialogLocation.getInstance().getDlgLoc(ViewWFEinfach.class.getName(), pidAbschnitt);
            if (location != null)
            {
              shell.setLocation(location);
            }
          }

        }
      }
      else
      {
//        System.out.println("ViewWFEinfach.zeigeViewWFEinfach(): ref ist nicht null:  " + ref);
        /*
         * Existierender View den Focus geben.
         */
        try
        {
          page.showView(ViewWFEinfach.ID, pidAbschnitt, IWorkbenchPage.VIEW_ACTIVATE);
          IViewPart viewPart = ref.getView(true);

          // IViewPart viewPart = page.showView(ViewWFEinfach.ID, pidAbschnitt, IWorkbenchPage.VIEW_ACTIVATE);
          // IViewReference ref2 = viewPart.getViewSite().getPage().findViewReference(ViewWFEinfach.ID, pidAbschnitt);
          //          
          // System.out.println("\t=>Ref1 == Ref2? " + (ref == ref2));
          //                    
          // ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(viewPart.getViewSite().getPage().findViewReference(ViewWFEinfach.ID,
          // pidAbschnitt));

          if (viewPart instanceof ViewWFEinfach)
          {
            ViewWFEinfach viewWFEinfach = (ViewWFEinfach)viewPart;
            re = viewWFEinfach;
          }
//          else
//          {
//            System.out.println("ViewWFEinfach.zeigeViewWFEinfach(): instance of2:  " + viewPart);
//          }
        }
        catch (PartInitException e)
        {
          e.printStackTrace();
        }
      }

    }

    return re;
  }

  /**
   * Prft ob eine View zur Darstellung der Schaltempehlung des Workflows bereits existiert.
   * 
   * @param pidAbschnitt
   *          PID des TSF-Abschnitts, fr den geprft werden soll.
   * @return Liefert <code>true</code> wenn die View bereits existiert, sonst <code>false</code>.
   */
  public static boolean existiertViewWFEinfach(String pidAbschnitt)
  {
    boolean re = false;

    if (pidAbschnitt != null)
    {
      IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
      IWorkbenchPage page = dw.getActivePage();
      IViewReference ref = page.findViewReference(ViewWFEinfach.ID, pidAbschnitt);
      if (ref != null)
      {
        re = true;
      }
    }

    return re;
  }

  /**
   * Schliet die View zu einem WF an Hand der PID des jeweiligen TSF-Abschnitts.
   * 
   * @param pidAbschnitt
   *          PID des jeweiligen TSF-Abschnitts.
   * @return Liefert <code>true</code> wenn die View erfolgreich geschlossen werden konnte, sonst <code>false</code>.
   */
  public static boolean schliesseViewWFEinfach(String pidAbschnitt)
  {
    boolean viewKonnteGeschlossenWerden = true;

    if (pidAbschnitt != null)
    {
      if (existiertViewWFEinfach(pidAbschnitt))
      {
        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        IWorkbenchPage page = dw.getActivePage();
        IViewReference ref = page.findViewReference(ViewWFEinfach.ID, pidAbschnitt);
        IViewPart view = ref.getView(false);
        if (view instanceof ViewWFEinfach)
        {
          ViewWFEinfach viewWfEinfach = (ViewWFEinfach)view;
          viewWfEinfach.setSchliessenErlaubt(true);
          // ##tp
          viewWfEinfach.getSite().getShell().setVisible(false);
        }

        if ((view != null) && (view.getViewSite() != null) && (view.getViewSite().getPage() != null))
        {
          view.getViewSite().getPage().hideView(view);
        }

      }
      else
      {
        /*
         * View existiert nicht und kann nicht geschlossen werden.
         */
        // String msg = "ViewWFEinfach fr Abschnitt(-PID) " + pidAbschnitt
        // + " existiert nicht und kann nicht geschlossen werden!";
        // ViewTools.log(msg, IStatus.WARNING, null);
      }
    }

    return viewKonnteGeschlossenWerden;
  }

  public void setSchliessenErlaubt(boolean schliessenErlaubt)
  {
    this._schliessenErlaubt = schliessenErlaubt;
    firePropertyChange(IWorkbenchPartConstants.PROP_DIRTY);
  }

  public boolean isSchliessenErlaubt()
  {
    return _schliessenErlaubt;
  }

  @Override
  public int promptToSaveOnClose()
  {
    return ISaveablePart2.CANCEL;
  }

  @Override
  public void doSave(IProgressMonitor monitor)
  {
  }

  @Override
  public void doSaveAs()
  {
  }

  @Override
  public boolean isDirty()
  {
    return !isSchliessenErlaubt();
  }

  @Override
  public boolean isSaveAsAllowed()
  {
    return false;
  }

  @Override
  public boolean isSaveOnCloseNeeded()
  {
    return true;
  }

  @Override
  public void perspectiveActivated(IWorkbenchPage page, IPerspectiveDescriptor perspective)
  {
//    System.out.println("ViewWFEinfach.perspectiveActivated()");
    /*
     * Nach dem Umschalten der Perspektive die Wiedervorlage anstarten.
     */

    if (getData() != null)
    {
      /*
       * Wiedervorlage auf eine Sekunde setzen.
       */
      new WiedervorlageTSFWorkflowAbschnitt(getData(), 1l);
      schliesseViewMitErlaubnis();
    }

    closeDetachedShells();

    // this.dispose();
  }

  private void closeDetachedShells()
  {
    final Shell[] shells = PlatformUI.getWorkbench().getDisplay().getShells();

    TimerTask task = new TimerTask() {

      @Override
      public void run()
      {
        Display display = PlatformUI.getWorkbench().getDisplay();
        Runnable innerRun = new Runnable() {
          public void run()
          {
            for (Shell shell : shells)
            {
              if (shell.equals(ViewWFEinfach.this.shell))
                continue;

              if (shell != null && shell.getText().length() == 0)
              {

                boolean closeOk = false;
                // shell.setVisible(false);
//                System.out.println("SHELL CHILD COUNT => " + shell.getChildren().length);

                LinkedList<Control> lChildren = getChildrenRek(shell);
//                System.out.println("\t->" + lChildren);
                if (lChildren.size() >= 2)
                {
                  Control chk = lChildren.get(1);
                  if (chk instanceof CTabFolder)
                  {
                    CTabFolder tabFolder = (CTabFolder)chk;
                    Control[] childTabs = tabFolder.getChildren();
                    if (childTabs.length == 4)
                    {
                      closeOk = true;
                    }
                  }
//                  else
//                  {
//                    System.out.println("\t  -> no tabfolder");
//                    // closeOk = true;
//                  }
                }
//                else if ((lChildren.size() >= 0) && (lChildren.size() < 2))
//                {
//                  // closeOk = true;
//                }

                if (closeOk)
                {
                  shell.setVisible(false);
                  // Dispose?
                }
              }
            }
          }
        };
        display.asyncExec(innerRun);

      }
    };
    Timer t = new Timer("Timer Close Detached Empty Views");
    t.schedule(task, 100l);

//    System.out.println("\t ####>> DONE");
  }

  public LinkedList<Control> getChildrenRek(Control ctrl)
  {
    LinkedList<Control> list = new LinkedList<Control>();

    list.add(ctrl);

    if (ctrl instanceof Composite)
    {
      Composite comp = (Composite)ctrl;
      Control[] children = comp.getChildren();
      if ((children != null) && (children.length > 0))
      {
        for (Control childCtrl : children)
        {
          if (childCtrl instanceof Composite)
          {
            Composite childComp = (Composite)childCtrl;
            list.addAll(getChildrenRek(childComp));
          }
        }
      }
    }

    return list;
  }

  @Override
  public void perspectiveChanged(IWorkbenchPage page, IPerspectiveDescriptor perspective, String changeId)
  {
    // TODO Auto-generated method stub

  }

}
