/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;

import de.sts.buv.plugin.workflow.tsf.Activator;
import de.sts.buv.plugin.workflow.tsf.PreferenceInit;

/**
 * Preference-Page fr Parameter des Workflows. Versorgte Properties siehe
 * unten.
 * 
 * @author HofmannJ
 * 
 */
public class WorkflowPreferencePage extends FieldEditorPreferencePage implements
		IWorkbenchPreferencePage
{
	public static final String ID = "de.sts.buv.plugin.workflow.tsf.dlg.WorkflowPreferencePage";

	/**
	 * Initialisiert den zu verwendenden PreferenceStore.
	 */
	private void initStore()
	{
		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
		setPreferenceStore(store);
	}

	@Override
	public void init(IWorkbench workbench)
	{
		initStore();
	}

	@Override
	protected void createFieldEditors()
	{
		/*
		 * Verzgerung bis zur Wiedervorlage des Weiterschalten-Dialogs des
		 * Workflows.
		 */
		IntegerFieldEditor editorWFWeiterschaltenDelay = new IntegerFieldEditor(
				PreferenceInit.PROP_DELAY_WIEDERVORLAGE_WF_WEITERSCHALTEN,
				"Verzgerung Wiedervorlage WF Weiterschalten (Sekunden)",
				getFieldEditorParent());
		addField(editorWFWeiterschaltenDelay);

		/*
		 * Verzgerung bis zur Wiedervorlage der Schalt-Empfehlung.
		 */
		IntegerFieldEditor editorSchaltempfehlungDelay = new IntegerFieldEditor(
				PreferenceInit.PROP_DELAY_WIEDERVORLAGE_SCHALTEMPFEHLUNG,
				"Verzgerung Wiedervorlage Schaltempfehlung (Sekunden)",
				getFieldEditorParent());
		addField(editorSchaltempfehlungDelay);

		/*
		 * WF-Snd aktiv
		 */
		BooleanFieldEditor bWfSndaktiv = new BooleanFieldEditor(
				PreferenceInit.PROP_SND_WF_AKTIV, "Workflow-Klang aktiv",
				getFieldEditorParent());
		addField(bWfSndaktiv);

		/*
		 * Pfad WF Snd
		 */
		FileFieldEditor pathWfSnd = new FileFieldEditor(
				PreferenceInit.PROP_PATH_SND_WF, "Workflow-Klang: ",
				getFieldEditorParent());
		addField(pathWfSnd);

		/*
		 * Delay Wf Snd
		 */
		IntegerFieldEditor editorDelayWfSnd = new IntegerFieldEditor(
				PreferenceInit.PROP_DELAY_SND_WF,
				"Wiederholungszeit Workflow Klang (in Sekunden)",
				getFieldEditorParent());
		addField(editorDelayWfSnd);

		/*
		 * Weiterschalten snd aktiv
		 */
		BooleanFieldEditor bWeiterschaltenSndaktiv = new BooleanFieldEditor(
				PreferenceInit.PROP_SND_WEITERSCHALTEN_AKTIV,
				"Weiterschalten-Klang aktiv", getFieldEditorParent());
		addField(bWeiterschaltenSndaktiv);

		/*
		 * Pfad Weiterschalten Snd
		 */
		FileFieldEditor pathWeiterschaltenSnd = new FileFieldEditor(
				PreferenceInit.PROP_PATH_SND_WEITERSCHALTEN, "Weiterschalten-Klang: ",
				getFieldEditorParent());
		addField(pathWeiterschaltenSnd);

		/*
		 * Delay Weiterschalten Snd
		 */
		IntegerFieldEditor editorDelayWeiterschaltenSnd = new IntegerFieldEditor(
				PreferenceInit.PROP_DELAY_SND_WEITERSCHALTEN,
				"Wiederholungszeit Weiterschalten Klang (in Sekunden)",
				getFieldEditorParent());
		addField(editorDelayWeiterschaltenSnd);
		
		/*
     * Weiterschalten snd aktiv
     */
    BooleanFieldEditor bSchaltEmpfehlungInaktiv = new BooleanFieldEditor(
        PreferenceInit.PROP_SCHALTEMPFEHLUNG_INAKTIV,
        "Schaltempfehlung inaktiv", getFieldEditorParent());
    addField(bSchaltEmpfehlungInaktiv);
	}
	
	@Override
	protected Control createContents(Composite parent) {
		
    	//Fr die F1 Taste
		PlatformUI.getWorkbench().getHelpSystem().setHelp( parent, Activator.PLUGIN_ID + ".workflowPreference" );
		
		return super.createContents(parent);
	}
	
	@Override
	public void performHelp()
	{
		String helpContextId = Activator.PLUGIN_ID + ".workflowPreference";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}
}
