/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.snd;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.Status;
import org.osgi.framework.Bundle;

import de.sts.buv.plugin.workflow.tsf.Activator;

public class AePlayWave extends Thread
{

	private String filename;

	private Position curPosition;

	private final int EXTERNAL_BUFFER_SIZE = 524288; // 128Kb

	enum Position
	{
		LEFT, RIGHT, NORMAL
	};

	public AePlayWave(String wavfile)
	{
		filename = wavfile;
		curPosition = Position.NORMAL;
	}

	public AePlayWave(String wavfile, Position p)
	{
		filename = wavfile;
		curPosition = p;
	}

	public void run()
	{
		File soundFile = null;

		if ((filename != null) && (filename.length() > 0))
		{
			soundFile = new File(filename);

			if (!soundFile.exists())
			{
				logge("Sound Workflow - Kann Datei nicht finden: \"" + filename + "\"",
						Status.WARNING);
				soundFile = null;
			}
		} else
		{
			filename = null;
		}

		AudioInputStream audioInputStream = null;
		try
		{
			if (soundFile != null)
			{
				audioInputStream = AudioSystem.getAudioInputStream(soundFile);
			} else
			{
				Bundle bundle = Activator.getDefault().getBundle();
				if (bundle != null)
				{
					URL fileURL = bundle.getEntry("snd/alarm.wav");
					try
					{
					  InputStream audioSrc = fileURL.openStream();
					  //add buffer for mark/reset support
					  InputStream bufferedIn = new BufferedInputStream(audioSrc);
						audioInputStream = AudioSystem.getAudioInputStream(bufferedIn);
//						audioInputStream = AudioSystem.getAudioInputStream(fileURL
//								.openStream());
					}
					catch (IOException e1)
					{
						logge(
								"Workflow Sound (IO-Exception): Standard-Soundfile snd/alarm.wav kann nicht bezogen werden.",
								Status.WARNING, e1);
						e1.printStackTrace();
					}
				} else
				{
					logge("Workflow Sound: Bundle kann nicht bestimmt werden.",
							Status.WARNING);
				}

			}
		} catch (UnsupportedAudioFileException e1)
		{
			e1.printStackTrace();
			return;
		} catch (IOException e1)
		{
			e1.printStackTrace();
			return;
		}

		if (audioInputStream != null)
		{
			AudioFormat format = audioInputStream.getFormat();
			SourceDataLine auline = null;
			DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);

			try
			{
				auline = (SourceDataLine) AudioSystem.getLine(info);
				auline.open(format);
			} catch (LineUnavailableException e)
			{
				e.printStackTrace();
				return;
			} catch (Exception e)
			{
				e.printStackTrace();
				return;
			}

			if (auline.isControlSupported(FloatControl.Type.PAN))
			{
				FloatControl pan = (FloatControl) auline
						.getControl(FloatControl.Type.PAN);
				if (curPosition == Position.RIGHT)
					pan.setValue(1.0f);
				else if (curPosition == Position.LEFT)
					pan.setValue(-1.0f);
			}

			auline.start();
			int nBytesRead = 0;
			byte[] abData = new byte[EXTERNAL_BUFFER_SIZE];

			try
			{
				while (nBytesRead != -1)
				{
					nBytesRead = audioInputStream.read(abData, 0, abData.length);
					if (nBytesRead >= 0)
						auline.write(abData, 0, nBytesRead);
				}
			} catch (IOException e)
			{
				e.printStackTrace();
				return;
			} finally
			{
				auline.drain();
				auline.close();
			}
		}

		if (audioInputStream != null)
		{
			try
			{
				audioInputStream.close();
			} catch (IOException e)
			{
				e.printStackTrace();
			}
		}
	}

	private void logge(String msg, int srv)
	{
		ILog log = Activator.getDefault().getLog();
		log.log(new Status(srv, Activator.PLUGIN_ID, msg));
	}

	private void logge(String msg, int srv, Throwable t)
	{
		ILog log = Activator.getDefault().getLog();
		log.log(new Status(srv, Activator.PLUGIN_ID, msg, t));
	}
}
